/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ExecuteSetupDebug.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/05/16
 *
 */

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

import exec.*;
import server.*;

public class ExecuteSetupDebug extends DebugObject {

    public boolean setupDebug(int level) throws Exception {
        boolean         flag;
        ExecuteServer   execute_server;

        execute_server = new ExecuteServer();

        try {
            flag = execute_server.setupDebug( level );
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return flag;
    }

    public static void main(String[] args) {
        int                  level;
        String               message;
        String               debug_string;
        boolean              flag;
        MessageResource      message_resource;
        ExecuteSetupDebug    execute_setup_debug;

        message_resource = new MessageResource();
        debug_string = System.getProperty(MessageText.EXEC_REGION_LANG);
        try {
            if (debug_string == null) {
                message_resource.setLanguage(Locale.JAPAN.getISO3Language());
            } else {
                message_resource.setLanguage(debug_string);
            }
        } catch (Exception exception_object) {
            message = message_resource.getFormattedMessage(
                MessageText.DEBUG_EXCEPTION_KEY,
                message_resource.getMessage(
                    MessageText.DEBUG_EXCEPTION_KEY),
                SPACE,
                exception_object.getMessage() );
            System.err.println(message);
            return; 
        }

        if (args.length != 1) {
            message = message_resource.getFormattedMessage(
                MessageText.ARGUMENT_PARAMETER_EXCEPTION,
                message_resource.getMessage(
                    MessageText.ARGUMENT_PARAMETER_EXCEPTION));
            System.err.println(message);
            return;
        }

        try {
            level = Integer.valueOf(args[0]).intValue();
        } catch (Exception exception_object) {
            message = message_resource.getFormattedMessage(
                MessageText.DEBUG_EXCEPTION_KEY,
                message_resource.getMessage(
                    MessageText.DEBUG_EXCEPTION_KEY),
                SPACE,
                exception_object.getMessage() );
            System.err.println(message);
            return; 
        }
        execute_setup_debug = new ExecuteSetupDebug();
        try {
            flag = execute_setup_debug.setupDebug( level );
        } catch (Exception exception_object) {
            message = message_resource.getFormattedMessage(
                MessageText.DEBUG_EXCEPTION_KEY,
                message_resource.getMessage(
                    MessageText.DEBUG_EXCEPTION_KEY),
                SPACE,
                exception_object.getMessage() );
            System.err.println(message);
            return; 
        }
        message = message_resource.getFormattedMessage(
            MessageText.DEBUG_VALUE_KEY,
            message_resource.getMessage(
                    MessageText.DEBUG_VALUE_KEY),
            SPACE,
            String.valueOf(flag) );
        System.err.println(message);
    }
}

