/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ArticleEntry.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/08/17
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ArticleEntry extends BaseEntry {

    private boolean child = false;
    private int own = -1;
    private int depth = 0;
    private int board_index = -1;

    private ArticleEntry parent = null;
    private int parent_number = -1;

    private String tag = null;

    private static String COMMA = ",";
    private static String TAB_SPACE = "    ";

    public static String type_name = "Article";

    public static String PARAMETER_ERROR = "Parameter error";

    public String getName() {
        return type_name;
    }

    public String getTag() {
        return tag;
    }

    public void setTag(String param_tag) {
        tag = param_tag;
    }

    public void setParent(int number) {
        parent_number = number;
    }
    public int getParent() {
        return parent_number;
    }

    public void setChild(boolean flag) {
        child = flag;
    }

    public boolean getChild() {
        return child;
    }

    public boolean compareTag(BookEntry target) {
        String  source_tag;
        String  target_tag;

        if (target == null) {
            return false;
        }

        target_tag = target.getTag();
        source_tag = getTag();

        if (target_tag == null || source_tag == null ||
            target_tag.compareTo(source_tag) != 0) {
            return false;
        }
        return true;
    }

    public void setOwn(int number) {
        own = number;
    }

    public int getOwn() {
        return own;
    }

    public void setDepth(int number) {
        depth = number;
    }

    public int getDepth() {
        return depth;
    }

    public void setBoardNumber(int index) {
        board_index = index;
    }

    public int getBoardNumber() {
        return board_index;
    }

    public String[] getUnspecifiedTags(BookEntry master_entry, ArticleEntry[] article_entry, IndexCard list) throws Exception {
        int             i;
        String[]        tags;
        String[]        bk;
        ArticleEntry    parent_article;
        BookEntry       slave_entry;

        if (master_entry == null || article_entry == null || list == null) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }

        tags = new String[0];

        for (slave_entry = master_entry; 
             slave_entry != null && (slave_entry.getNumber() != 0 &&
             slave_entry.getParent() != -1);
             slave_entry = (BookEntry)list.get(slave_entry.getParent())) {

            for (i = 0; i < article_entry.length; i++) {
                if (slave_entry.getNumber() == article_entry[i].getOwn()) {
                    break;
                }
            }
            if (i == article_entry.length) {
                bk = new String[tags.length+1];
                for (int j = 0; j < tags.length; j++) {
                    bk[j] = tags[j];
                }
                tags = bk;
                tags[tags.length-1] = slave_entry.getTag();
            }
        }
        return tags;
    }

    public BookEntry getTopBookEntry(int child_no, IndexCard list) throws  Exception {
        String       message;
        Exception    exception_object;
        BaseEntry      base_entry;
        BookEntry      book_entry;

        if (list == null || list.size() <= child_no) {
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        base_entry = (BaseEntry)list.get(child_no); 
        if (base_entry.getName().compareTo(BookEntry.type_name) != 0) {
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        book_entry = (BookEntry)base_entry;
        for ( ;
            book_entry != null && (book_entry.getParent() != 0 && 
            book_entry.getParent() != -1) ;
            book_entry = (BookEntry)list.get(book_entry.getParent())) {
        }
        if (book_entry.getParent() == 0) {
            return book_entry;
        }
        message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
        exception_object = new Exception( message );
        throw exception_object;
    }

    public void printUnspecifiedTags(int offset, ArticleEntry[] article_entry, IndexCard list) {
        if (article_entry == null || list == null) {
            return;
        }
        for (int i = 0; i < article_entry.length; i++) {
            for (int j = offset; j < offset+i; j++) {
                System.out.print(TAB_SPACE);
            }
            System.out.println(article_entry[i].getOwn());
        }
    }


    public void printArticleStartTag(ArticleEntry[] article_entry, IndexCard list, StoreHouse store_house) {
        int             index;
        int             count;
        TagString       tag_string;
        BookEntry[]  hierarchy;
        BookEntry       child;
        String[]        unspecified_tags;

        if (article_entry == null || list == null || store_house == null) {
            return;
        }
        count = article_entry.length;
        hierarchy = new BookEntry[count];
        index = 1;
        child = null;
        for (int i = 0; i < hierarchy.length; i++) {
            hierarchy[i] 
                = (BookEntry)list.get(article_entry[i].getOwn());
            if (child != null) {
                try {
                    if (checkParent(hierarchy[i], child, list) == true) {
                        child = hierarchy[i];
                    }
                } catch (Exception exception_object) {
                    return;
                }
            } else {
                child = hierarchy[i];
            }
        }

        if (child == null) {
            return;
        }
        for (int i = 0; i < hierarchy.length; i++) {
            for (int j = 0; j < i; j++) {
                System.out.print(TAB_SPACE);
            }
            System.out.println(hierarchy[i].getTag());
        }
    }

    public void printArticleEndTag(ArticleEntry[] article_entry, IndexCard list, StoreHouse store_house) {
        int             index;
        int             count;
        TagString       tag_string;
        BookEntry[]  hierarchy;
        BookEntry       child;
        String[]        unspecified_tags;

        if (article_entry == null || list == null || store_house == null) {
            return;
        }
        count = article_entry.length;
        hierarchy = new BookEntry[count];
        index = 1;
        child = null;
        for (int i = 0; i < hierarchy.length; i++) {
            hierarchy[i] 
                = (BookEntry)list.get(article_entry[i].getOwn());
            if (child != null) {
                if (checkChild(child, hierarchy[count-index], list) == true) {
                    child = hierarchy[i];
                }
            } else {
                child = hierarchy[i];
            }
        }
        if (child == null) {
            return;
        }

        for (int i = hierarchy.length-1; i >= 0; i--) {
            for (int j = 0; j < i; j++) {
                System.out.print(TAB_SPACE);
            }
            try {
                tag_string = new TagString(hierarchy[i].getTag());
            } catch (Exception exception_object) {
                continue;
            }
            System.out.println(tag_string.getEndTag());
        }
    }

    public void printArticleContents(ArticleEntry[] article_entry, IndexCard list, StoreHouse store_house) {
        int             index;
        int             count;
        TagString       tag_string;
        BookEntry[]  hierarchy;
        BookEntry       child;
        String[]        unspecified_tags;

        if (article_entry == null || list == null || store_house == null) {
            return;
        }
        count = article_entry.length;
        hierarchy = new BookEntry[count];
        index = 1;
        child = null;
        for (int i = 0; i < hierarchy.length; i++) {
            hierarchy[i] 
                = (BookEntry)list.get(article_entry[i].getOwn());
            if (child != null) {
                try {
                    if (checkParent(hierarchy[i], child, list) == true) {
                        child = hierarchy[i];
                    }
                } catch (Exception exception_object) {
                    return;
                }
            } else {
                child = hierarchy[i];
            }
        }
        if (child == null) {
            return;
        }
        try {
            unspecified_tags = getUnspecifiedTags(child, article_entry, list);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            return;
        }
        if (unspecified_tags != null) {
            for (int i = 0; i < unspecified_tags.length; i++) {
                if (unspecified_tags[i] != null) {
                    for (int j = 0; j < hierarchy.length+i; j++) {
                        System.out.print(TAB_SPACE);
                    }
                    System.out.println(unspecified_tags[i]);
                }
            }
        }

        for (int i = 0; i < article_entry.length; i++) {
             System.out.print(TAB_SPACE);
        }
        try {
            store_house.printBookContent(0, child, list);
            System.out.println();
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
        }
        if (unspecified_tags != null) {
            for (int i = unspecified_tags.length-1; i >= 0; i--) {
                if (unspecified_tags[i] == null) {
                    continue;
                }
                for (int j = 0; j < hierarchy.length+i; j++) {
                    System.out.print(TAB_SPACE);
                }
                try {
                    tag_string = new TagString(unspecified_tags[i]);
                } catch (Exception exception_object) {
                    continue;
                }
                System.out.println(tag_string.getEndTag());
            }
        }
    }

    public void printArticle(ArticleEntry[]  article_entry, IndexCard list, StoreHouse store_house) {

        printArticleStartTag(article_entry, list, store_house);
       
        printArticleContents(article_entry, list, store_house);

        printArticleEndTag(article_entry, list, store_house);
    }

    public String[] getKeys(String keys, IndexCard list) throws Exception {
        int               el;
        int               index;
        String[]          tag_array; 
        StringTokenizer   string_tokenizer;
        String[][]        entry;
        String[]          element;
        String[]          options;
        int[]             count;
        int[]             repeat;
        BaseEntry         base_entry;
        BookEntry         book_entry;

        if (keys == null || list == null) {
            String       message;
            Exception    exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        
        tag_array = TagString.parseTags(keys) ;

        entry = new String[tag_array.length][];
        for (int i = 0; i < entry.length; i++) {
            entry[i] = new String[0];
        }
        for (int i = 0; i < tag_array.length; i++) {
            for (int j = 0; j < list.size(); j++) {
                base_entry = (BaseEntry)list.get(j);
                if (base_entry.getName().compareTo(BookEntry.type_name) == 0) {
                    book_entry = (BookEntry)base_entry;
                    if (TagString.compareKey(
                        book_entry.getTag(), tag_array[i]) == true) {
                        String[]   bk;

                        element = entry[i];
                        int  k;
                        for (k = 0; k < element.length; k++) {
                            if (TagString.compareTag(book_entry.getTag(),
                                element[k]) == true) {
                                break;
                            }
                        }
                        if (k < element.length) {
                            break;
                        }

                        bk = new String[element.length+1];
                        for (k = 0; k < element.length; k++) {
                            bk[k] = element[k];
                        }
                        bk[element.length] = book_entry.getTag();
                        entry[i] = bk;
                    }
                }
            }
        }

        count = new int[tag_array.length];
        repeat = new int[tag_array.length];
        el = 1;
        for (int i = 0; i < entry.length; i++) {
            if (entry[i].length == 0) {
                return null;
            }
            el = el * entry[i].length;
        }
        for (int i = 0; i < entry.length; i++) {
            count[i] = 1;
        }

        for (int i = 0; i < entry.length; i++) {
            for (int j = 1; j <= i; j++) {
                count[i] = count[i] * entry[j-1].length;
            }
        }

        for (int i = 0; i < entry.length; i++) {
            repeat[i] = 1;
        }
        for (int i = 0; i < entry.length; i++) {
            for (int j = 0; j < entry.length-i-1; j++) {
                repeat[i] = repeat[i] * entry[entry.length-1-j].length;
            }
        }
       
        options = new String[el];
        for (int i = 0; i < options.length; i++) {
            options[i] = new String();
        }
        for (int i = 0; i < entry.length; i++) {
            element = entry[i];
            for (int k = 0; k < repeat[i]; k++) {
                for (int j = 0; j < element.length; j++) {
                    for (int l = 0; l < count[i]; l++) {
                        options[l+j*count[i]+k*element.length*count[i]] = options[l+j*count[i]+k*element.length*count[i]].concat(element[j]);
                    }
                }
            }
        }
        return options;
    }

    private static boolean checkChild(
                BookEntry master, BookEntry slave, IndexCard  list) {
        int         index;
        BaseEntry   base_entry;
        BookEntry   child;
        int  master_no;
        int  slave_no;
        master_no = master.getNumber();
        slave_no = slave.getNumber();

        for (int i = 0; i < master.getEntrySize(); i++) {
            index = master.getEntryNumber(i);
            if (index == slave_no) { return true; }
            base_entry = (BaseEntry)list.get(index);
            if (base_entry.getName().compareTo(BookEntry.type_name) == 0) {
                if (checkChild((BookEntry)base_entry, slave, list) == true) {
                    return true;
                } 
            } 
        }
        return false;
    }

    public  boolean checkParent(
                BookEntry master, BookEntry slave, IndexCard  list) throws Exception {
        int         parent;
        int         slave_no;
        BookEntry   child;

        if (master == null || slave == null || list == null) {
            String       message;
            Exception    exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        slave_no = slave.getNumber();

        for (parent = master.getParent(); parent != -1; parent = child.getParent()) {
            if (slave_no == parent) {
                return true;
            }
            child = (BookEntry)list.get(parent);
        }
        return false;
    }

    private boolean checkRelation(ArticleEntry master, BookEntry slave, IndexCard list) throws Exception {
        BookEntry  master_entry;

        master_entry = (BookEntry)list.get(master.getOwn());
        try {
            if (checkParent(master_entry, slave, list) == true) {
                return true;
            }
            if (checkParent(slave, master_entry, list) == true) {
                return true;
            }
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return false;
    }
}


