/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ArticlePost.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/08/27
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;

public class ArticlePost extends IndexCard {

    private static String PARAMETER_ERROR = "Parameter error";
    private static String NOT_FOUND = "Not found.";

    private static ArticlePost[] getArticleHead(int depth, BookEntry[][] box, IndexCard list,
            MessageResource    message_resource,
            int                debug,
            LogPrintStream     log_print_stream) throws Exception {
        boolean         flag;
        IndexCard       index_card;
        BookEntry       book_entry;
        BookEntry       master_entry;
        ArticleEntry    article_entry;
        ArticleEntry    child_article;
        ArticlePost[]   article_post;
        ArticlePost[]   child_article_post;
        ArticlePost[]   bk;
        ArticleEntry    operate_entry;

        if (depth == box.length-1) {
            article_post = new ArticlePost[box[depth].length];
            for (int i = 0; i < box[depth].length; i++) {
                article_post[i] = new ArticlePost();
                article_entry = new ArticleEntry();
                article_entry.setTag(box[depth][i].getTag());
                article_entry.setOwn(box[depth][i].getNumber());
                article_post[i].add(article_entry);
            }
            return article_post;
        }
        article_post = new ArticlePost[0];
        try {
            child_article_post = getArticleHead(depth+1, box, list,
                message_resource, debug, log_print_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        operate_entry = new ArticleEntry();

        operate_entry.setMessageResource( message_resource );
        operate_entry.setDebug( debug );
        operate_entry.setLogPrintStream( log_print_stream );

        for (int j = 0; j < child_article_post.length; j++) {
            child_article = (ArticleEntry)child_article_post[j].get(
                child_article_post[j].size()-1
            );
            master_entry = (BookEntry)list.get(child_article.getOwn());
            for (int i = 0; i < box[depth].length; i++) {
                book_entry = box[depth][i];

                flag = false;
                try {
                    if (operate_entry.checkParent(master_entry, book_entry, list) == true) {
                        flag = true;
                    } else if ( operate_entry.checkParent(book_entry, master_entry, list) == true ) {
                        flag = true;
                    }
                } catch (Exception exception_object) {
                    throw exception_object;
                }

                if (flag == true) {
                    bk = new ArticlePost[article_post.length+1];
                    for (int k = 0; k < article_post.length; k++) {
                        bk[k] = article_post[k];
                    }
                    
                    ArticlePost post_entry;
                    ArticleEntry article_1;
                    ArticleEntry article_2;
                    post_entry = new ArticlePost();
                    post_entry.setMessageResource( message_resource );
                    post_entry.setDebug( debug );
                    post_entry.setLogPrintStream( log_print_stream );
                    for (int k = 0; k < child_article_post[j].size(); k++) {
                        article_1 = (ArticleEntry)child_article_post[j].get(k);
                        article_2 = new ArticleEntry();

                        article_2.setMessageResource( message_resource );
                        article_2.setDebug( debug );
                        article_2.setLogPrintStream( log_print_stream );

                        article_2.setTag(article_1.getTag());
                        article_2.setOwn(article_1.getOwn());
                        post_entry.add((Object)article_2);
                    }
                    bk[article_post.length] = post_entry;

                    article_post = bk;
                    article_entry = new ArticleEntry();

                    article_entry.setMessageResource( message_resource );
                    article_entry.setDebug( debug );
                    article_entry.setLogPrintStream( log_print_stream );

                    article_entry.setTag(book_entry.getTag());
                    article_entry.setOwn(book_entry.getNumber());
                    article_post[article_post.length-1].add(article_entry);
                }
            }
        }
        return article_post;
    }

    public static ArticlePost valueOf(String tags, IndexCard list, ArticlePost articles,
        MessageResource  param_message_resource,
        int              debug,
        LogPrintStream   param_log_print_stream) throws Exception {
        int               i, j;
        BaseEntry         base_entry;
        BookEntry         book_entry;
        BookEntry         master_entry;
        ArticleEntry      article_entry;
        String[]          tag_array; 
        BookEntry[][]     book_array;
        BookEntry[]       bk;
        ArticlePost[]     article_post_array;
        String            message;
        MessageResource   message_resource;
        LogPrintStream    log_print_stream;
        
        message_resource = param_message_resource;
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        log_print_stream = param_log_print_stream;
        if (log_print_stream == null) {
            log_print_stream = new LogPrintStream(System.err);
        }

        if (tags == null) {
            Exception   exception_object;
            message = message_resource.getFormattedMessage(
                          ARGUMENT_PARAMETER_EXCEPTION,
                          message_resource.getMessage(
                              ARGUMENT_PARAMETER_EXCEPTION));
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (list == null) {
            Exception   exception_object;
            message = message_resource.getFormattedMessage(
                          ARGUMENT_PARAMETER_EXCEPTION,
                          message_resource.getMessage(
                              ARGUMENT_PARAMETER_EXCEPTION));
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (articles == null) {
            Exception   exception_object;
            message = message_resource.getFormattedMessage(
                          ARGUMENT_PARAMETER_EXCEPTION,
                          message_resource.getMessage(
                              ARGUMENT_PARAMETER_EXCEPTION));
            exception_object = new Exception( message );
            throw exception_object;
        }
        tag_array = TagString.parseTags(tags) ;
        book_array = new BookEntry[tag_array.length][];
        for (i = 0; i < tag_array.length; i++) {
            book_array[i] = new BookEntry[0];
        }
        for (i = 0; i < tag_array.length; i++) {
            for (j = 1; j < list.size(); j++) {
                base_entry = (BaseEntry)list.get(j);
                if (base_entry.getName().compareTo(BookEntry.type_name) != 0) {
                    continue;
                }
                book_entry = (BookEntry)base_entry;

                if (TagString.compareTag(book_entry.getTag(),
                        tag_array[i]) == true) {
                    bk = new BookEntry[book_array[i].length+1];
                    for (int k = 0; k < book_array[i].length; k++) {
                        bk[k] = book_array[i][k];
                    }
                    bk[bk.length-1] = book_entry;
                    book_array[i] = bk;
                }
            }
        }

        try {
            article_post_array = getArticleHead(0, book_array, list,
                message_resource,  debug, log_print_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        ArticleEntry  article_1;
        ArticleEntry  article_2;
        int           parent;
        int           number;
        parent = 0;

        for (i = 0; i < article_post_array.length; i++) {
            for (j = article_post_array[i].size()-1; j >= 0; j--) {
                article_1 = (ArticleEntry)article_post_array[i].get(j);
                article_2 = new ArticleEntry();

                article_2.setMessageResource( message_resource );
                article_2.setDebug( debug );
                article_2.setLogPrintStream( log_print_stream );

                article_2.setOwn(article_1.getOwn());

                if (j == 0) {
                    article_2.setChild(true);
                }
                if (j != article_post_array[i].size()-1) {
                    article_2.setParent(parent);
                }
                article_2.setBoardNumber(list.getBoardNumber());
                article_2.setTag(article_1.getTag());
                number = articles.size();
                articles.add(article_2);
                article_1 = (ArticleEntry)articles.get(number);
                parent = article_1.getNumber();
            }
        }
        return articles;
    }

    public ArticleEntry[]  getArticleArray(ArticleEntry article_entry) 
                                                     throws Exception {
        int             parent_no;
        ArticleEntry    article_parent;
        ArticleEntry[]  article_array;
        ArticleEntry[]  bk;

        article_array = new ArticleEntry[0];
        for (article_parent = article_entry ; article_parent != null; ) {
            bk = new ArticleEntry[article_array.length+1];
            for (int j = 0; j < article_array.length; j++) {
                bk[j+1] = article_array[j];
            }
            article_array = bk;
            article_array[0] = article_parent;

            parent_no = article_parent.getParent();
            if (parent_no == -1) { break; }
            try {
                article_parent = (ArticleEntry)get( parent_no );
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return article_array;
    }

    private static int DEBUG_DEBUG = 1;

    public static void main(String[]  args) {
        int              no;
        int              board_number;
        String[]         tags;
        BookCenter       book_center;
        ArticlePost      article_post;
        IndexCard        book_card;
        StoreHouse       store_house;
        ArticleEntry     article_entry;
        ArticleEntry[]   article_array;

        ArticleEntry     operate_article;

        article_post = new ArticlePost();

        try {
            book_center = new BookCenter();
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }
        for (int i = 1; i < args.length; i++) {
            try {
                book_center.addBook(args[i]);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
        }
        operate_article = new ArticleEntry();
        operate_article.setMessageResource( new MessageResource() );
        operate_article.setDebug( DEBUG_DEBUG );
        operate_article.setLogPrintStream( new LogPrintStream( System.err ) );
     
        for (int i = 0; i < args.length-1; i++) {
            try {
                book_card = book_center.getIndexCard(i);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
            try {
                tags = operate_article.getKeys(args[0], book_card);
            } catch (Exception  exception_object) {
                exception_object.printStackTrace();
                return;
            }
            if (tags == null) {
                break;
            }

            for (int j = 0; j < tags.length; j++) {
                try {
                    article_post = ArticlePost.valueOf(tags[j], book_card, article_post,
                        operate_article.getMessageResource(),
                        operate_article.getDebug(),
                        operate_article.getLogPrintStream());
                } catch (Exception exception_object) {
                    continue;
                }
            }
        }

        for (int i = 0; i < article_post.size(); i++) {
            try {
                article_entry = (ArticleEntry)article_post.get(i);
            } catch (Exception exception_object) {
                continue;
            }

            if (article_entry.getChild() != true) {
                continue;
            }

            try {
                article_array = article_post.getArticleArray(article_entry) ;
            } catch (Exception exception_object) {
                continue;
            }

            board_number = article_entry.getBoardNumber();
            try {
                book_card = book_center.getIndexCard(board_number);
                store_house = book_center.getStoreHouse(board_number);
            } catch (Exception exception_object) {
                continue;
            }
            article_entry.printArticle(article_array, book_card, store_house);
        }
    }
}
