/**
 * Copyright (C) All rights reserved.
 *
 * @Module     BookBoard.java
 * @Author     Takahiko Tanaka.
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class BookBoard extends DebugObject {
    private static int BUFFER_SIZE = 4096;
    private static String COMMA = ",";
    private static String SLASH = "/";
    private int          board_number = -1;
    private IndexCard    list_object = new IndexCard();
    private BookEntry[]  book_entries = new BookEntry[0];

    private static String ADD_BOOK  = "addBook";

    private static String PARAMETER_ERROR = "Parameter error";

    public BookBoard() {
    }

    public BookBoard(int number) {
        board_number = number;
        list_object.setBoardNumber(number);
    }

    public int getBoardNumber() {
        return board_number;
    }

    protected int addBook(BookEntry entry) {
        BookEntry[]  bk;
        String       message;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage( ENTER_METHOD, ENTER_METHOD,
                SPACE, ADD_BOOK);
            getLogPrintStream().printLogln(message);
        }
        bk = new BookEntry[book_entries.length+1];
        for (int i = 0; i < book_entries.length; i++) {
            bk[i] = book_entries[i];
        }
        book_entries = bk;
        book_entries[book_entries.length-1] = entry;
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage( END_METHOD, END_METHOD,
                SPACE, ADD_BOOK);
            getLogPrintStream().printLogln(message);
        }
        return book_entries.length-1;
    }

    protected BookEntry getBook(int  index) throws Exception {
        BookEntry   entry;
        String      message;

        if (index < 0 || index >= book_entries.length) {
            Exception exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        entry = book_entries[index];
        return entry;
    }

    protected void replaceBook(int index, BookEntry entry) throws Exception {
        book_entries[index] = entry;
    }

    public int getBookSize() {
        return book_entries.length;
    }

    protected void deleteBook(int index) throws Exception {
        BookEntry[] bk;
        int       destination;
        String    message;

        if (index < 0 || index >= book_entries.length) {
            Exception exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        destination = 0;
        bk = new BookEntry[book_entries.length-1];
        for (int i = 0; i < bk.length; i++) {
            if (index == i) { continue; }
            bk[destination] = book_entries[i];
            destination++;
        }
        book_entries = bk;
    }

    public BookEntry makeTopEntry() throws Exception {
        int           entry_no;
        BookEntry     top_book;

        try {
            top_book = BookEntry.valueOf(getMessageResource(),
                getDebug(), 
                getLogPrintStream());
        } catch (Exception exception_object) {
            throw exception_object;
        }
        top_book.setTag(TagString.EMPTY_TAG);
        top_book.setDepth(getBookSize());
        entry_no = addBook(top_book);
        return top_book;
    }

    public void makeBookBoard(StoreHouse store_house) throws Exception {
        int           no;
        int           pno;
        BaseEntry     entry;
        BookEntry     book_entry;
        AreaRange     end_tag_area;
        String        message;

        no = 0;
        if (store_house == null) {
            Exception  exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        book_entry = makeTopEntry();
        no = list_object.size();

        list_object.add(book_entry);
        while (true) {
            try {
                entry = store_house.readEntry();
            } catch (Exception exception_object) {
                break;
            }
            if (entry == null) {
                if (getBookSize() <= 1) {
                    break;
                }
                book_entry = getBook(getBookSize()-1);

                end_tag_area = store_house.getEndTag();
                book_entry.setEndTag(end_tag_area);
                list_object.replace(book_entry.getNumber(), book_entry);
                deleteBook(getBookSize()-1);
                continue;
            }
            if ((entry instanceof BookEntry) == true) {
                book_entry = getBook(getBookSize()-1);
                pno = book_entry.getNumber();
                no = list_object.size();
                ((BookEntry)entry).setParent(pno);
                ((BookEntry)entry).setDepth(getBookSize());
                list_object.add(entry);

                book_entry.addEntry(no);
                replaceBook(getBookSize()-1, book_entry);
                addBook((BookEntry)entry);
                continue;
            }
            no = list_object.size();
            list_object.add(entry);
            book_entry = getBook(getBookSize()-1);
            book_entry.addEntry(no);
            replaceBook(getBookSize()-1, book_entry);
        }
    }

    public IndexCard getIndexCard() {
        return list_object;
    }

    public void setIndexCard(IndexCard book_card) {
        list_object = book_card;
    }

    public static void main(String[] args) {
        StoreHouse   store_house;
        BookBoard    book_board;
        IndexCard    book_card;

        try {
            store_house = StoreHouse.valueOf(args[0]);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        } 
        book_board = new BookBoard();
        try {
            book_board.makeBookBoard(store_house);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            store_house.printBook(0, 0, book_board.getIndexCard());
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
