/**
 * Copyright (C) All rights reserved.
 *
 * @Module     BookBoardTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2013/04/03
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;

public class BookBoardTest extends BookBoard {
    private static String TEST1 = "Test1";
    private static String TEST2 = "Test2";
    private static String TEST3 = "Test3";

    public void Test1(String[] args) {
        int        value;
        String     message;
        BookEntry  book_entry;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, TEST1);
            getLogPrintStream().printLogln( message );

            book_entry = BookEntry.valueOf( getMessageResource(),
            getDebug(),
            getLogPrintStream() );

            message = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY, SPACE, TEST1);
            getLogPrintStream().printLogln( message );

            addBook( book_entry );

            if (getDebug() > 0 && getLogPrintStream() != null) {
                message = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY, 
                SPACE, String.valueOf(getBookSize()) );
                getLogPrintStream().printLogln( message );
            }

            message = getMessage(END_METHOD, END_METHOD, SPACE, TEST1);
            getLogPrintStream().printLogln( message );
        }
    }

    private static String FILE_NAME = "Report.20110101";

    public void Test2(String[] args) {
        String      message;
        StoreHouse  store_house;
        IndexCard   index_card;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, TEST2);
            getLogPrintStream().printLogln( message );
        }

        try {
            store_house = StoreHouse.valueOf(FILE_NAME);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        } 
        try {
            makeBookBoard(store_house);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        index_card = getIndexCard();

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY, 
            SPACE, String.valueOf(index_card.size()) );
            getLogPrintStream().printLogln( message );
        }
     

        try {
            store_house.printBook(0, 0, index_card);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(END_METHOD, END_METHOD, SPACE, TEST2);
            getLogPrintStream().printLogln( message );
        }
    }

    public void Test3(String[] args) {
        int              rt;
        int              off;
        int              length;
        byte[]           file_area;
        String           message;
        StoreHouse       store_house;
        IndexCard        index_card;
        File             fd;
        FileInputStream  file_input_stream;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, TEST3);
            getLogPrintStream().printLogln( message );
        }
/**
        if (args.length != 1) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message = getMessage(ARGUMENT_PARAMETER_ERROR_EXCEPTION, 
                    ARGUMENT_PARAMETER_ERROR_EXCEPTION, SPACE, TEST3);
                getLogPrintStream().printLogln( message );

                message = getMessage(END_METHOD, END_METHOD, SPACE, TEST3);
                getLogPrintStream().printLogln( message );
            }
            return;
        }
**/
        try {
            fd = new File(FILE_NAME);
            file_input_stream = new FileInputStream(fd);
            length = (int)fd.length();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        off = 0;
        file_area = new byte[length]; 
        while (true) {
            try {
                rt = file_input_stream.read(file_area, off, file_area.length-off);
            } catch (Exception exception_object) {
                break;
            }
            if (rt == -1) {
                break;
            }
            if (rt + off >= file_area.length) {
                break;
            }
            off = off + rt;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY, 
            SPACE, String.valueOf(length) );
            getLogPrintStream().printLogln( message );
        }

        try {
            store_house = StoreHouse.valueOf(file_area,
                getMessageResource(),
                getDebug(),
                getLogPrintStream()  );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        } 

        try {
            makeBookBoard(store_house);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        index_card = getIndexCard();

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY, 
            SPACE, String.valueOf(index_card.size()) );
            getLogPrintStream().printLogln( message );
        }
     

        try {
            store_house.printBook(0, 0, index_card);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(END_METHOD, END_METHOD, SPACE, TEST3);
            getLogPrintStream().printLogln( message );
        }
    }
}
