/**
 * Copyright (C) All rights reserved.
 *
 * @Module     BookCenter.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/08/24
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;

public class BookCenter extends DebugObject {
    private  String[]     source_files = new String[0];
    private  StoreHouse[] center = new StoreHouse[0];
    private  IndexCard[]  area = new IndexCard[0];

    private static String ADD_BOOK  = "addBook";
    private static String ADD_ENTRY = "addEntry";

    public int addBook(String file_name) throws Exception{
        int          no;
        String       message;
        StoreHouse   store_house;
        BookBoard    book_board;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage( ENTER_METHOD, ENTER_METHOD,
                SPACE, ADD_BOOK);
            getLogPrintStream().printLogln(message);
        }
        if (file_name == null) {
            Exception    exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }

        try {
            store_house = StoreHouse.valueOf(file_name,
                getMessageResource(),
                getDebug(),
                getLogPrintStream() );
        } catch (Exception exception_object) {
            throw exception_object;
        } 
        no = area.length;
        book_board = new BookBoard(no);
        book_board.setDebug( getDebug() );
        book_board.setMessageResource( getMessageResource() );
        book_board.setLogPrintStream( getLogPrintStream() );
        try {
            book_board.makeBookBoard(store_house);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        IndexCard[] area_bk;
        area_bk = new IndexCard[area.length+1];
        for (int i = 0; i < area.length; i++) {
            area_bk[i] = area[i];
        }
        area = area_bk;
        area[area.length-1] = book_board.getIndexCard();

        String[]     source_files_bk;
        source_files_bk = new String[source_files.length+1];
        for (int i = 0; i < source_files.length; i++) {
            source_files_bk[i] = source_files[i];
        }
        source_files = source_files_bk;
        source_files[source_files.length-1] = new String(file_name);

        StoreHouse[]  center_bk;
        center_bk = new StoreHouse[center.length+1];
        for (int i = 0; i < center.length; i++) {
            center_bk[i] = center[i];
        }
        center = center_bk;
        center[center.length-1] = store_house;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage( END_METHOD, END_METHOD,
                SPACE, ADD_BOOK);
            getLogPrintStream().printLogln(message);
        }

        return no;
    }

    public IndexCard addEntry(String name, StoreHouse store_house, IndexCard index_card) throws Exception {
        int          no;
        String       message;
        BookBoard    book_board;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage( ENTER_METHOD, ENTER_METHOD,
                SPACE, ADD_ENTRY);
            getLogPrintStream().printLogln(message);
        }

        no = area.length;
        IndexCard[] area_bk;
        area_bk = new IndexCard[area.length+1];
        for (int i = 0; i < area.length; i++) {
            area_bk[i] = area[i];
        }
        area = area_bk;
        area[area.length-1] = index_card;
        index_card.setBoardNumber(no);

        String[]     source_files_bk;
        source_files_bk = new String[source_files.length+1];
        for (int i = 0; i < source_files.length; i++) {
            source_files_bk[i] = source_files[i];
        }
        source_files = source_files_bk;
        source_files[source_files.length-1] = new String(name);

        StoreHouse[]  center_bk;
        center_bk = new StoreHouse[center.length+1];
        for (int i = 0; i < center.length; i++) {
            center_bk[i] = center[i];
        }
        center = center_bk;
        center[center.length-1] = store_house;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage( END_METHOD, END_METHOD,
                SPACE, ADD_ENTRY);
            getLogPrintStream().printLogln(message);
        }

        return index_card;
    }

    public int getSize() {
        return area.length;
    }

    public StoreHouse getStoreHouse(int index) throws Exception {
        if (center.length <= index) {
            String       message;
            Exception    exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        return center[index];
    }

    public String getFileName(int index) throws Exception {
        if (source_files.length <= index) {
            String       message;
            Exception    exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        return source_files[index];
    }

    public IndexCard getIndexCard(int index) throws Exception {
        if (area.length <= index) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        return area[index];
    }

    public static void main(String[]  args) {
        String       file_name;
        BookCenter   book_center;
        String       debug_string;
        int          debug;

        debug = 0;
        debug_string = System.getProperty(EXEC_REGION_DEBUG);
        if (debug_string == null || debug_string.length() == 0) {
            debug_string = System.getProperty(DEBUG_LEVEL_KEY);
        }
        if (debug_string != null && debug_string.length() > 0) {
            try {
                debug = Integer.valueOf(debug_string).intValue();
            } catch (Exception exception_object) {
                debug = 0;
            }
        }
        book_center = new BookCenter();
        book_center.setDebug( debug );
        book_center.setMessageResource( 
            new MessageResource() );
        book_center.setLogPrintStream( new LogPrintStream( System.err ) );

        for (int i = 0; i < args.length; i++) {
            try {
                book_center.addBook(args[i]);    
            } catch (Exception  exception_object) {
                exception_object.printStackTrace();
                return;
            }
        }
        for (int i = 0; i < book_center.getSize(); i++) {
            try {
                file_name = book_center.getFileName(i);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
        }
    }
}

