/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    CondenseRegion.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version:   1.0 2011/08/01
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class CondenseRegion extends RegionObject implements MakeReport {

    private static String CONDENSE_REGION = "CondenseRegion";
    public static String CONDENSE_ERROR = "Couldn't condense";
    public static String PARAMETER_ERROR = "Parameter error";
    private static String EXECUTE = "execute";
    private static String MAKEREPORT = "makeReport";
    private static String CONDENSEMESSAGETABLE = "condenseMessageTable";

    public static int REGION_UNIT = 10;

    public static String FILE_OPEN_ERROR = "File error : ";

    public static String USAGE_MESSAGE = "Condense <TAG> Files...";

    /**
     *
    public static String USAGE_MESSAGE = "Condense -t <TAG> -d <Delete> Files...";
        -t "<WORK>,<CUSTOMER>:<WORK NAME=INTERNAL>,<CUSTOMER>,<MASTER>" 
        -d "<仕事 NAME=INTERNAL>,<CUSTOMER>" Files ....
    **/

    public String getConcatTagString(
        KeyList key_list, int start,  int end) {

        String tag_string;
        String key_list_string;

        if (key_list == null || end >= key_list.length()) {
            return null;
        }
        if (start < 0 || end > key_list.length()) {
            return null;
        }
        tag_string = new String();
        for ( int i = start; i <= end; i++ ) {
            if (i != start) {
                tag_string = tag_string.concat(COMMA);
            }
            try {
                key_list_string = key_list.getKeyEntry(i);
            } catch (Exception real_exception_object) {
                continue;
            }
            tag_string = tag_string.concat(key_list_string);
        }
        return tag_string;
    }

    private boolean  start_flag = false;
    private void setStart() {
        start_flag = true;
    }
    private void setEnd() {
        start_flag = false;
    }
    private boolean getStart() {
        return start_flag;
    }

    /**
     * @param  start is index number in tag list array.
     */
    public void printTags(PrintStream print_stream, 
        KeyList key_list, int start, int end) {
        int  inc;
        int  count;
        String key_list_string;

        if (getStart() == true) {
           inc = 1;
        } else {
           inc = -1;
        }
        if (print_stream == null || key_list == null || 
                    start < 0 || key_list.length() < end) {
            return;
        }
        
        count = start;
        for (int i = start; i != end + inc; i = i + inc) {
            for (int j = 0; j < count; j++) {
                print_stream.print(TAB_SPACE);
            }
            try {
                key_list_string = key_list.getKeyEntry(i);
            } catch (Exception real_exception_object) {
                continue;
            }
            if (getStart() == true) {
                print_stream.println(key_list_string);
            } else {
                print_stream.println(convertFromStartToEnd(key_list_string));
            }
            count = count + inc;
        }
        return;
    }

    public void condenseMessageTable(PrintStream print_stream,
            SortRegion sort_region_object) throws Exception {
        int             master_index;
        int             slave_index;
        int             message_number;
        int             list_index;
        String          message;
        String          message_log;
        String          master_head;
        String          slave_head;
        String          master_tag_string;
        String          slave_tag_string;
        String[]        master_tag_list;
        String[]        slave_tag_list;
        StringTokenizer stk;
        KeyList         master_key_list;
        String          master_key_list_string;
        KeyList         slave_key_list;
        String          slave_key_list_string;
        boolean         match_flag;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, 
                CONDENSEMESSAGETABLE);
            getLogPrintStream().printLogln( message );
        }
        if (print_stream == null || sort_region_object == null) {
            Exception exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION, 
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception(message);
            throw exception_object;
        }
        match_flag = false;
        slave_index = 1;
        message_number = sort_region_object.getMessageNumber();
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY, 
                SPACE, String.valueOf(message_number) );
            getLogPrintStream().printLogln( message_log );
        }
        for (master_index = 0; master_index < message_number; ) {
            try {
                master_tag_string = 
                    sort_region_object.getTagStringInOrder(master_index);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY, 
                    SPACE, String.valueOf(master_index) );
                getLogPrintStream().printLogln( message_log );
            }
            list_index = 0;
            try {
                master_key_list = KeyListFactory.createKeyList(
                            master_tag_string, COMMA,
                            getMessageResource(),
                            getDebug(),
                            getLogPrintStream());
            } catch (Exception real_exception_object) {
                throw real_exception_object;
            }

            setStart();
            printTags(print_stream, master_key_list, 0, master_key_list.length()-1);
            try {
                message = sort_region_object.getMessageInOrder(master_index);
                print_stream.println(message);
            } catch (Exception exception_object) {
                master_index = master_index + 1;
                slave_index = slave_index + 1;
                throw exception_object;
            }

            for (int j = 0; j < master_key_list.length(); j++) {
                master_head = getConcatTagString(master_key_list, 0, 
                            master_key_list.length() - 1 - j);

                for ( ; slave_index < message_number; ) {
                    try {
                        slave_tag_string = 
                            sort_region_object.getTagStringInOrder(slave_index);
                    } catch (Exception exception_object) {
                        break;
                    }
                    try {
                        slave_key_list = KeyListFactory.createKeyList(slave_tag_string, COMMA,
                            getMessageResource(),
                            getDebug(),
                            getLogPrintStream());
                    } catch (Exception real_exception_object) {
                        throw real_exception_object;
                    }
                    if (slave_key_list.length() <= master_key_list.length()-1-j) {
                        break;
                    }
                    slave_head = getConcatTagString(slave_key_list, 0,
                            master_key_list.length() - 1 - j);

                    if (master_head.toUpperCase().compareTo(
                                slave_head.toUpperCase()) != 0) {
                        if (match_flag == false) {
                            setEnd();
                            printTags(print_stream, master_key_list, 
                                master_key_list.length()-1-j,
                                master_key_list.length()-1-j);
                        }
                        break;
                    }
                    match_flag = true;
                    try {
                        message = sort_region_object.getMessageInOrder( 
                            slave_index);
                    } catch (Exception exception_object) {
                        master_index = master_index + 1;
                        slave_index = slave_index + 1;
                        throw exception_object;
                    }

                    setStart();
                    printTags(print_stream, slave_key_list, 
                        master_key_list.length()-j, slave_key_list.length()-1);
                    print_stream.println(message);
                    setEnd();
                    printTags(print_stream, slave_key_list, 
                       master_key_list.length()-j, slave_key_list.length()-1);
                    slave_index = slave_index + 1;
                }
                if (match_flag == true) {
                    match_flag = false;
                    setEnd();
                    printTags(print_stream, master_key_list, 
                        0, master_key_list.length()-j-1);
                    break;
                }
            }
            master_index = slave_index;
            slave_index = slave_index + 1;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE,
                CONDENSEMESSAGETABLE);
            getLogPrintStream().printLogln( message );
        }
    }


    public void makeReport(Object[] argument, PrintStream print_stream) throws Exception {
        int                      region_index = 0;
        String                   tag;
        String                   name;
        String                   keys;
        String                   tag_keys;
        String                   options;
        String                   message;
        ReportRegion[]           report_region;
        InputStream              system_input;
        PrintStream              system_output;
        String[]                 tag_list1;
        String[]                 tag_list2;
        String[]                 spec_list;
        String                   upper_option1;
        String                   upper_option2;
        boolean                  start_flag;
        byte[]                   buffer;
        SortRegion               sort_region_object;
        String                   lang;
        boolean                  check;

        check = false;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, MAKEREPORT);
            if (getLogPrintStream() == null) {
                setLogPrintStream( new LogPrintStream(System.err) );
            }
            getLogPrintStream().printLogln( message );
        }
        if (argument.length <= 1) {
            Exception exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION, 
                ARGUMENT_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                BLANK_IN_BLANK), CONDENSE_REGION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (print_stream == null) {
            Exception exception_object;

            message = getMessage( STREAM_PARAMETER_EXCEPTION, 
                STREAM_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                BLANK_IN_BLANK), CONDENSE_REGION);
            exception_object = new Exception(message);
            throw exception_object;
        }

        for (int i = 1; i < argument.length; i++) {
            if ((argument[i] instanceof InputStream) == false) {
                continue;
            }
            check = true;
        }
        if (argument == null || argument.length <= 1 || check == false) {
            Exception exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        start_flag = false;

        report_region = new ReportRegion[0];
        system_input = System.in;
        tag_keys = (String)argument[0];

        sort_region_object = new SortRegion();
        if (getDebug() > 0) {
            sort_region_object.setDebug( getDebug() );
        }
        if (getLogPrintStream() != null) {
            sort_region_object.setLogPrintStream( getLogPrintStream() );
        }
        sort_region_object.setMessageResource( getMessageResource() );
             
        try {
            sort_region_object.makeMessageTable(argument);
        } catch (Exception real_exception_object) {
            String    exception_message;
            Exception exception_object;

            exception_message = real_exception_object.getMessage();
            message = getMessage( INTERNAL_ERROR_EXCEPTION, 
                INTERNAL_ERROR_EXCEPTION,
                exception_message );

            exception_object = new Exception(message);
            throw exception_object;
        }

        try {
            condenseMessageTable(print_stream, sort_region_object ) ;
        } catch (Exception real_exception_object) {
            String    exception_message;
            Exception exception_object;

            exception_message = real_exception_object.getMessage();
            message = getMessage( INTERNAL_ERROR_EXCEPTION, 
                INTERNAL_ERROR_EXCEPTION,
                exception_message );

            exception_object = new Exception(message);
            throw exception_object;
        }

        System.setIn(system_input);
        return;
    }

    public void execute(String[] org_args) throws Exception {
        String           message;
        PrintStream      print_stream;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, EXECUTE);
            getLogPrintStream().printLogln( message );
        }
        print_stream = System.out;
        try {
            super.execute(org_args, print_stream, (MakeReport)this);
        } catch (Exception real_exception_object) {
            throw real_exception_object; 
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, EXECUTE);
            getLogPrintStream().printLogln( message );
        }
    }

    public static void Usage() {
        System.err.println(USAGE_MESSAGE);
    }

    public static void main(String[] org_args) {
        String            message;
        String[]          args;
        FileOutputStream  file_output_stream;
        PrintStream       print_stream;
        CondenseRegion    condense_region;
        String            debug_string;

        condense_region = new CondenseRegion();
        debug_string = System.getProperty(EXEC_REGION_DEBUG);
        if (debug_string == null || debug_string.length() == 0) {
            debug_string = System.getProperty(DEBUG_LEVEL_KEY);
        }
        if (debug_string != null && debug_string.length() > 0) {
            condense_region.setDebug(debug_string);
        }
        condense_region.setLogPrintStream(new LogPrintStream(System.err));

        if (org_args.length < 1) {
            message = condense_region.getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION,
                condense_region.getMessageResource().getMessage(
                    BLANK_IN_BLANK),
                CONDENSE_REGION);
            System.out.println(message);
            return;
        }

        try {
            print_stream = System.out;
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = condense_region.getMessage( IO_ERROR_EXCEPTION, 
                    IO_ERROR_EXCEPTION,
                    condense_region.getMessageResource().getMessage(
                        BLANK_IN_BLANK), CONDENSE_REGION);
            }
            System.out.println(message);
            System.exit(1);
            return;
        }

        try {
            condense_region.execute(org_args);
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = condense_region.getMessage( INTERNAL_ERROR_EXCEPTION, 
                    INTERNAL_ERROR_EXCEPTION,
                    condense_region.getMessageResource().getMessage(
                        BLANK_IN_BLANK), CONDENSE_REGION);
            }
            System.err.println(message);
            System.exit(1);
        }
    }
}

