/**
 * Copyright (C)  All rights reserved.
 *
 * @Module     DebugObject.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2011/09/05
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class DebugObject extends Object implements MessageText {

/****************
    public static String DEBUG_CALL_KEY = "Call";
    public static String DEBUG_EXCEPTION_KEY = "Exception";
    public static String DEBUG_DEBUG_KEY = "Debug";
    public static String DEBUG_DEBUG_CONDITION_KEY = "Debug condition";
    public static String DEBUG_ENABLE_KEY = "Enable";
    public static String DEBUG_DISABLE_KEY = "disable";
    public static String DEBUG_TAG_KEY = "Tag";
    public static String DEBUG_VALUE_KEY = "Value";
    public static String DEBUG_COUNT_KEY = "Count";

    public static String DEBUG_CODE_001 = "\001";
    public static String DEBUG_CODE_002 = "\002";
    public static String DEBUG_CODE_003 = "\003";
    public static String DEBUG_CODE_004 = "\004";

    public static String TAB_SPACE = "    ";
    public static String SPACE = " ";
    public static String COMMA = ",";
    public static String NAME_KEY = "name";
    private static String TMPDIR = "tmp";

    public static String PARENT = "..";
    public static String EMPTY = "";
    public static String DOT = ".";
    public static String EQUAL = "=";
    public static String CLOSE_TAG = ">";
    public static String WILDCARD_TAG = "<*>";
    public static char   NUMERIC_ZERO = '0';
    public static char   NUMERIC_NINE = '9';
    public static char   CODE_NEWLINE = '\n';
    public static char   CODE_RETURN = '\r';
    public static char   UPPER_A = 'A';
    public static char   UPPER_Z = 'Z';
    public static char   DATE_SEPARATOR_CHAR = '/';
    public static char   TERM_SEPARATOR_CHAR = '-';
    public static String TAG_START_STRING = "<";
    public static String TAG_END_STRING = ">";
    public static String TAG_END_MARK_STRING = "/";
    public static char   TAG_END = '>';
    public static String NEWLINE_STRING = "\n";
    public static String START_BRACKET_STRING = "[";
    public static String END_BRACKET_STRING = "]";
    public static char START_BRACKET_CHAR = '[';
    public static char END_BRACKET_CHAR = ']';
    public static char DOS_DIRECTORY_SEPARATOR_CHAR = '\\';

    public static int TRACE_DEBUG_LEVEL = 3;
    public static int FULL_DEBUG_LEVEL = 5;

    public static String JAPANESE_PREFIX = "ja_JP";
    public final static String JAPANESE_CHARSET = "UTF-8";
    public static String LANG_KEY = "LANG";
    public static String EXEC_REGION_LANG = "exec.region.lang";

    public static String DEBUG_VALUE_EXCEPTION_KEY = "Illegal debug value";

****************/

/**
    public static String CHECKTAGS_METHOD = "checkTags";
    public static String EXECUTE_METHOD = "Execute";
    public static String ENTER_METHOD = "Enter into  method";
    public static String END_METHOD = "End method";
    public static String ENTER_EXECUTE = "Enter into execute() method.";
    public static String END_EXECUTE = "End execute() method.";
    public static String PARAMETER_KEY = "Parameter";

    public static String DEBUG_CALL_KEY = "Call";
    public static String DEBUG_EXCEPTION_KEY = "Exception";
    public static String DEBUG_DEBUG_KEY = "Debug";
    public static String DEBUG_DEBUG_CONDITION_KEY = "Debug condition";
    public static String DEBUG_ENABLE_KEY = "Enable";
    public static String DEBUG_DISABLE_KEY = "disable";
    public static String DEBUG_TAG_KEY = "Tag";
    public static String DEBUG_VALUE_KEY = "Value";
    public static String DEBUG_COUNT_KEY = "Count";

    public static String DEBUG_CODE_001 = "\001";
    public static String DEBUG_CODE_002 = "\002";
    public static String DEBUG_CODE_003 = "\003";
    public static String DEBUG_CODE_004 = "\004";
    public static String CODE_101 = "\101";



    public String DEBUG_MESSAGE = "Message ";
    public final static String SYSTEM_IO_ERROR = "System I/O error";
    public static String SPACE_STRING = " ";

    public static String ARGUMENT_PARAMETER_EXCEPTION = "Argument Parameter Exception";
    public static String OVERFLOW_INDEX_EXCEPTION = "Overflow Index Exception";
    public static String TAGKEYS_PARAMETER_ERROR
        = "Tagkeys parameter error";

    public static String EXEC_REGION_DEBUG = "exec.debug.level";

**/
    protected static MessageResource message_resource = null;
    public int debug_value = 0;
    private static LogPrintStream  log_print_stream = null;

    protected static String[] language_list = {
        Locale.JAPAN.getISO3Language(),
        Locale.US.getISO3Language(),
        Locale.UK.getISO3Language(),
        Locale.ENGLISH.getISO3Language(),
        Locale.FRENCH.getISO3Language(),
        Locale.GERMAN.getISO3Language(),
        Locale.ITALIAN.getISO3Language(),
        Locale.JAPANESE.getISO3Language(),
        Locale.KOREAN.getISO3Language(),
        Locale.CHINESE.getISO3Language(),
        Locale.SIMPLIFIED_CHINESE.getISO3Language(),
        Locale.TRADITIONAL_CHINESE.getISO3Language(),
        Locale.FRANCE.getISO3Language(),
        Locale.GERMANY.getISO3Language(),
        Locale.ITALY.getISO3Language(),
        Locale.KOREA.getISO3Language(),
        Locale.CHINA.getISO3Language(),
        Locale.CANADA.getISO3Language(),
        Locale.CANADA_FRENCH.getISO3Language(),
        Locale.PRC.getISO3Language(),
        Locale.TAIWAN.getISO3Language(),
    };

    protected void initialMessageResource() {
        String   lang;
        String   message_lang;

        if (checkMessageResource() == false) {
            message_resource = new MessageResource();
            lang = System.getProperty(EXEC_REGION_LANG);
            if (lang == null) {
                lang = System.getenv(LANG_KEY);
                if (lang != null && lang.indexOf(JAPANESE_PREFIX) >= 0) {
                    message_lang = Locale.JAPAN.getISO3Language();
                } else {
                    message_lang = Locale.US.getISO3Language();
                }
            } else {
                message_lang = null;
                for (int i = 0; i < language_list.length; i++) {
                    if (lang.compareTo(language_list[i]) == 0) {
                        message_lang = lang;
                        break;
                    }
                }
            }
            if (message_lang == null) {
                message_resource.setDefaultLanguage();
                return;
            }
            try {
                message_resource.setLanguage(message_lang);
            } catch (Exception exception_object) {
                message_resource.setDefaultLanguage();
            }
        }
    }

    protected String getMessage(String format, String key, String... args) {
        String   lang;
        String   message;
        String[] message_list;
        String  debug_string;

        initialMessageResource();

        message_list = new String[args.length+1];
        message_list[0] = new String();
        if (key != null) {
            message_list[0] = message_resource.getMessage(key);
        }
        for (int i = 0; i < args.length; i++) {
            message_list[i+1] = args[i];
        }
        message = message_resource.getFormattedMessage(format, message_list);
        return message ;
    }

    public boolean setMessageResource(MessageResource param_message_resource) {
        if (param_message_resource != null) {
            message_resource = param_message_resource;
            return true;
        }
        return false;
    }

    public boolean checkMessageResource() {
        boolean  check;

        check = false;
        if (message_resource != null) {
            check = true;
        }
        return check;
    }

    protected boolean clearMessageResource() {
        boolean check;

        check = checkMessageResource();
        if (check == true) {
            message_resource = null;
        }
        return check;
    }

    public MessageResource getMessageResource() {
        String lang;

        initialMessageResource();
        return message_resource ;
    }

    public String getLanguage() {
        String  lang;

        lang = message_resource.getLanguage();
        return lang;
    }

    public int getDebug() {
        return debug_value;
    }

    public void setDebug(String debug_string) {
        String  warnning_message;

        if (debug_string != null) {
            try {
                debug_value = Integer.parseInt(debug_string);
            } catch (Exception exception_object) {
                warnning_message = getMessage(
                        DEBUG_VALUE_EXCEPTION_KEY,
                        DEBUG_VALUE_EXCEPTION_KEY);
                System.err.println(warnning_message);
            }
        }
    }

    public void setDebug(int value) {
        debug_value = value;
    }

    public LogPrintStream getLogPrintStream() {
        return log_print_stream;
    }

    public void setLogPrintStream(LogPrintStream param_log_print_stream) {
        log_print_stream = param_log_print_stream;
    }

    private static int DEBUG_DEBUG = 1;

    public static void main(String[]  args) {
        int              debug;
        String           message;
        DebugObject      debug_object;
        MessageResource  message_resource;

        debug_object = new DebugObject();
        debug = DEBUG_DEBUG;

        message_resource = new MessageResource();

        debug_object.setDebug( debug );
        debug_object.setLogPrintStream( new LogPrintStream( System.err ) );
        debug_object.setMessageResource( message_resource );

        System.out.println( debug_object.toString() );
    }
}
