/**
 * Copyright (C) All rights reserved.
 *
 * @Module     DebugTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2013/04/02
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.lang.reflect.*;

public class DebugTest extends DebugObject {

    private static int DEBUG_DEBUG = 1;
    private static String DEBUG_TEST_TEST1 = "Debug Test - Test Case1";
    private static String ALL_CASE = "ALL";

    private static String SETUP_DEBUG = "setupDebug";

    public void Test1(String[] args) {
        System.out.println(DEBUG_TEST_TEST1 
                + SPACE + String.valueOf(args.length));
    }

    public static DebugObject setupDebug(DebugObject basic_object,
        MessageResource param_message_resource,
        int             debug,
        LogPrintStream  log_print_stream) {
        String             debug_string;
        String             message;
        MessageResource    message_resource;

        message_resource = param_message_resource;
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        if (debug > 0 && log_print_stream != null) {
            message = message_resource.getFormattedMessage(ENTER_METHOD, 
                    message_resource.getMessage( ENTER_METHOD ), 
                    SPACE, SETUP_DEBUG);
            log_print_stream.printLogln( message );
        }
        basic_object.setDebug( debug );
        basic_object.setMessageResource( message_resource );
        basic_object.setLogPrintStream( log_print_stream );
        if (basic_object.getDebug() > 0 && basic_object.getLogPrintStream() != null) {
            message = basic_object.getMessage(END_METHOD, END_METHOD,
                SPACE, SETUP_DEBUG);
            basic_object.getLogPrintStream().printLogln( message );
        }
        return basic_object;
    }

    private static String INVOKE_MESSAGE = "Invoke";

    public static void invokeTest(DebugObject debug_test, String[] args) throws Exception {
        Class     debug_class;
        Method[]  method_list;
        String[]  new_args;
        Object[]  test_args;
        String    test_case;
        String    message;

        if (debug_test == null) {
            MessageResource  message_resource_object;
            message_resource_object = new MessageResource();
            message = message_resource_object.getFormattedMessage( 
                INVOKE_ERROR_EXCEPTION, 
                message_resource_object.getMessage(
                    INVOKE_ERROR_EXCEPTION));
            System.err.println( message );
            return;
        }
        if (args == null || args.length < 1) {
            MessageResource  message_resource_object;
            message_resource_object = new MessageResource();
            message = message_resource_object.getFormattedMessage( 
                INVOKE_ERROR_EXCEPTION, 
                message_resource_object.getMessage(
                    INVOKE_ERROR_EXCEPTION));
/**
            message = getMessage( INVOKE_ERROR_EXCEPTION, 
                INVOKE_ERROR_EXCEPTION);
*/
            System.err.println( message );
            return;
        }
        test_case = args[0];
        if (debug_test.getDebug() > 0 && debug_test.getLogPrintStream() != null) {
            message = debug_test.getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                SPACE, test_case);
            debug_test.getLogPrintStream().printLogln( message );
        }

        test_args = new Object[1];
        new_args = new String[args.length-1];
        for (int i = 0; i < args.length-1; i++) {
            new_args[i] = args[i+1];
            if (debug_test.getDebug() > 0 && debug_test.getLogPrintStream() != null) {
                message = debug_test.getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, new_args[i]);
                debug_test.getLogPrintStream().printLogln( message );
            }
        }
        test_args[0] = (Object)new_args;
        debug_class = debug_test.getClass();
        method_list = debug_class.getMethods();

        for (int i = 0; i < method_list.length; i++) {
            if (ALL_CASE.compareTo(test_case) == 0) {
                System.out.println(METHOD_KEY + SPACE +
                    method_list[i].getName());
                continue;
            }
            if (method_list[i].getName().compareTo(test_case) == 0) {

                if (debug_test.getDebug() > 0 && debug_test.getLogPrintStream() != null) {
                    message = debug_test.getMessage(
                        DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, method_list[i].getName(),
                        debug_test.getMessageResource().getMessage(
                        DEBUG_CODE_001)); 
  
                    debug_test.getLogPrintStream().printLogln( message );
                }
                method_list[i].invoke(debug_test, test_args);
                return;
            }
        }
    }

    private static String DEBUG_PARAMETER_ERROR = "Debug parameter";

    private static String DEBUG_TITLE = "Debug Test";
    private static String TEST_KEY = "Test";
    private static String PACKAGE = "exec";
    private static String FILE_PATH = "file:/";

    private static String CLASS_URL_HEAD = "URL";
    private static String CLASS_HEAD = "CLASS";

    public static void main(String[] args) {
        String             message;
        int                debug;
        String             debug_string;
        MessageResource    message_resource;
        LogPrintStream     log_print_stream;
        URL[]              urls;
        URL                url;
        URLClassLoader     uloader;
        Class              cls;
        Object             object_instance;
        DebugObject        debug_object;
        File               fd;
        String             path;
        String             class_string;
        String[]           new_args;
        
        message_resource = new MessageResource();
        debug = DEBUG_DEBUG;
        log_print_stream = new LogPrintStream( System.err );

        debug_string = System.getProperty(EXEC_REGION_LANG);
        try {
            if (debug_string == null) {
                message_resource.setLanguage(Locale.JAPAN.getISO3Language());
            } else {
                message_resource.setLanguage(debug_string);
            }
        } catch (Exception exception_object) {
            System.err.println( exception_object.getMessage() );
            return ;
        }
        if (args.length < 2) {
            message = message_resource.getFormattedMessage(
                DEBUG_PARAMETER_ERROR,
                message_resource.getMessage(
                    DEBUG_PARAMETER_ERROR));
            System.err.println( message );
            return;
        }

        fd = new File(PARENT+File.separator);
        path = fd.getAbsolutePath();
        path = FILE_PATH.concat(path);

        if (debug > 0 && log_print_stream != null) {
            message = message_resource.getFormattedMessage(
                DEBUG_START_KEY,
                message_resource.getMessage(
                    DEBUG_START_KEY));
            log_print_stream.printLogln(message);
            message = message_resource.getFormattedMessage(
                DEBUG_VALUE_KEY,
                message_resource.getMessage(
                    DEBUG_VALUE_KEY),
                SPACE, path);
            log_print_stream.printLogln(message);
        }

        try {
            url = new URL(path);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            return;
        }
       
        urls = new URL[1];
        urls[0] = url;
        uloader = new URLClassLoader(urls);
        new_args = new String[args.length-1];
        for (int i = 0; i < new_args.length; i++) {
            new_args[i] = args[i+1];
        }
        class_string = PACKAGE + DOT + args[0] + TEST_KEY;
        if (debug > 0 && log_print_stream != null) {
            message = message_resource.getFormattedMessage(
                DEBUG_VALUE_KEY,
                message_resource.getMessage(
                    DEBUG_VALUE_KEY),
                SPACE, class_string);
            log_print_stream.printLogln(message);
        }
        try {
            cls = Class.forName(class_string, true, (ClassLoader)uloader);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            return;
        }
        try {
            object_instance = cls.newInstance();
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            return;
        }
        debug_object = (DebugObject)object_instance;
        debug_object = DebugTest.setupDebug( debug_object,
            message_resource, debug, log_print_stream );

        debug_object.getMessage(DEBUG_DEBUG_KEY, DEBUG_DEBUG_KEY,
            SPACE, debug_object.toString());
        debug_object.getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
            SPACE, String.valueOf( debug_object.getDebug() ));

        try {
            DebugTest.invokeTest( debug_object, new_args );
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
        }
    }
}
