/**
 * Copyright (C) All rights reserved.
 *
 * @Module     DisplayHierarchy.java
 * @Author     Takahiko Tanaka.
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2013/02/01
 *
 */

package exec;

import java.lang.*;
import java.io.*;

public class DisplayHierarchy extends RegionObject implements MakeReport {

    public static String WILDCARD = "<*>";
    public static String NAME_KEY = "name";
    private static String TAB_STRING = "    ";
    private String current_tab = TAB_STRING;
    private static byte[] japanese_space_byte 
        = { (byte)0xe3, (byte)0x80, (byte)0x80 };
    private static String UTF8STRING = "UTF-8";
    private boolean japanese_flag = false;
    private static String JAPANESE_SPACE = "  ";

    private static String INDENT_OPTION = "-I";

/**
    private static String OVERFLOW_EXCEPTION = "Overflow exception.";
**/
    private static String MAKEREPORT_METHOD = "makeReport";

    private String[] file_name_list = null;

    private boolean indent_option = false;

    public void setIndentOption() {
        indent_option = true;
    }

    public boolean getIndentOption() {
        return indent_option;
    }

    public void setFileNames(String[] param) {
        file_name_list = param;
    }

    public String getFileName(int i) throws Exception {
        String    message;

        if (file_name_list.length < i) {
            Exception  exception_object;
            message = getMessage( OVERFLOW_INDEX_EXCEPTION,
                OVERFLOW_INDEX_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        return file_name_list[i];
    }

    public void makeReport(Object[] argument, PrintStream print_stream) throws Exception {
        int             start;
        int             end;
        byte[][][]      head;
        String    word;
        String    message;
        String    message_log;

        head = new byte[1][][];
        start = 0;
        end = 0;

        if (argument == null || argument.length == 0 || print_stream == null) {
            Exception  exception_object;
            message = getMessage(
                DISPLAY_HIERARCHY_PARAMETER_EXCEPTION,
                DISPLAY_HIERARCHY_PARAMETER_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (getDebug() > 0) {
            message_log = getMessage(ENTER_METHOD, ENTER_METHOD,
                SPACE, MAKEREPORT_METHOD);
            if (getLogPrintStream() == null) {
                setLogPrintStream( new LogPrintStream(System.err) );
            }
            getLogPrintStream().printLogln(message_log);
        }
        word = (String)argument[0];
        for (int i = 1; i < argument.length; i++) {
            if ((argument[i] instanceof InputStream) == false) {
                Exception  exception_object;
                message_log = getMessage(FILE_WARNING_MESSAGE,
                    FILE_WARNING_MESSAGE, SPACE,
                    getFileName(i-1),
                    getMessageResource().getMessage(CODE_101)); 
                if (getDebug() > 0) {
                    getLogPrintStream().printLogln(message_log);
                }
                continue;
            }
            message = getMessage(DISPLAY_HIERARCHY_TITLE_MESSAGE,
                DISPLAY_HIERARCHY_TITLE_MESSAGE,
                SPACE,
                getFileName(i-1));
            System.out.println(message);
            System.setIn((InputStream)argument[i]);
            try {
                getRegion(head, start, end, word, print_stream);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
    }

    public String getIndentMessage(byte[] message_byte, int position) throws Exception {
        String  message;
        String  sentence;
        String  sentence_sub;
        String  tab;
        String  local_tab;
        int     start;
        int     end;
        int     previous;
        int     length;
        int     count;
        int     contents;
        ByteArrayOutputStream  byte_array_output_stream;
        PrintStream            print_stream;

        if (message_byte == null || message_byte.length == 0 || position < 0) {
            Exception  exception_object;
            exception_object = new Exception(
                getMessage(ARGUMENT_PARAMETER_EXCEPTION, 
                           ARGUMENT_PARAMETER_EXCEPTION) );
            throw exception_object;
        } 
        byte_array_output_stream = new ByteArrayOutputStream();
        print_stream = new PrintStream(byte_array_output_stream);

        JAPANESE_SPACE = new String(japanese_space_byte, UTF8STRING);

        tab = new String();
        for (int i = 0; i < position; i++) {
            tab = tab.concat(current_tab);
        }

        start = 0;
        message = new String();
        previous = tab.length();
        count = 0;
        contents = 0;
        for (int i = 0; i < message_byte.length; i++) {
            if (message_byte[i] == (byte)0x20) {
                if (start == i) {
                    start = start + 1;
                    continue;
                }
            }
            if ( i + 2 < message_byte.length  &&
                message_byte[i] == (byte)0xe3 &&
                         message_byte[i+1] == (byte)0x80 &&
                         message_byte[i+2] == (byte)0x80 ) {
                if (start == i) {
                    count = count + 1;
                    if (japanese_flag == false) {
                        current_tab = new String(JAPANESE_SPACE + JAPANESE_SPACE);
                        tab = new String();
                        for (int j = 0; j < position; j++) {
                            tab = tab.concat(current_tab);
                        }
                        japanese_flag = true;
                    } 
                    start = i + 3;
                }
                i = i + 2;
                continue;
            }
            if (message_byte[i] == '\n' || message_byte[i] == '\r') {
                if (contents == 0) {
                    if (message_byte[i+1] == '\r' || message_byte[i+1] == '\n') {
                        i = i + 1;
                        start = start + 1;
                    }
                    start = start + 1;
                    continue;
                }
                end = i-1;
                if (message_byte[i+1] == '\r' || message_byte[i+1] == '\n') {
                    i = i + 1;
                }
                if (start <= end) {
                    sentence = new String(message_byte, start, end-start+1 );
                    sentence_sub = sentence.trim();
                    print_stream.println(tab+sentence_sub);
                    count = 0;
                    start = i + 1;
                }
            }
            contents = contents + 1;
        }
        if (start < message_byte.length) {
       
            sentence = new String(message_byte, start, message_byte.length-start );
            sentence_sub = sentence.trim();
            if (sentence_sub.length() != 0) {
                print_stream.println(tab+sentence_sub);
            }
        }
        message = new String(byte_array_output_stream.toByteArray());
        return message;
    }

    /**
     * This class does with following cases.
     *
      < >
        Case 1
        < >
        </ >
        Case 2
        < >
        </ >
        Case 3
      </ >
      < >
        Case 4
      </ >
    **/

    private void getRegion(byte[][][] head, int start, int end, 
        String word, PrintStream  print_stream) throws Exception {
        int      next_start;
        int      next_end;
        int      child_start;
        int      child_end;

        int      area_start;
        int      area_end;
        String   tag;
        String   error;
        Region   operate_region;
        Region   previous;
        Region   current_region;
        Region[] region_box;
        Region[] bk_region;
        byte[]   message_byte;

        boolean[]  flags;
        boolean[]  bk_flags;

        if (head == null || word == null || print_stream == null) {
            Exception  exception_object;
            exception_object = new Exception(
                getMessage(ARGUMENT_PARAMETER_EXCEPTION, 
                           ARGUMENT_PARAMETER_EXCEPTION) );
            throw exception_object;
        }
      
        next_start = start;
        next_end = end;
        flags      = new boolean[0];
        region_box = new Region[0];
        current_region = null;
        previous = null;
        error = null;

        operate_region = new Region();
/**
        operate_region.setDebug ( getDebug() );
        operate_region.setMessageResource( getMessageResource() );
        operate_region.setLogPrintStream( getLogPrintStream() );
**/

        while (true) {
            while (true) {
                try {
                    current_region = operate_region.findRegion( 
                        head, next_start, next_end, word, NAME_KEY);

                    bk_region = new Region[region_box.length+1];
                    for (int i = 0; i < region_box.length; i++) {
                        bk_region[i] = region_box[i];
                    }
                    region_box = bk_region;
                    region_box[region_box.length-1] = current_region;

                    bk_flags = new boolean[flags.length+1];
                    for (int i = 0; i < flags.length; i++) {
                        bk_flags[i] = flags[i];
                    }
                    flags = bk_flags;
                    flags[flags.length-1] = false;
                } catch (Exception exception_object) {
                    if (region_box.length == 0) {
                        if (exception_object.getMessage() != null &&
                            exception_object.getMessage().compareTo(
                                Region.END_OF_FILE_EXCEPTION) == 0) {
                            error = exception_object.getMessage();
                            break;
                        }
                        if (exception_object.getMessage() != null &&
                            exception_object.getMessage().compareTo(
                                Region.END_OF_REGION_EXCEPTION) == 0) {
                            next_start = 0;
                            next_end = 0;
                            break;
                        }
                        break;
                    }
                    current_region = region_box[region_box.length-1];
                    if (getIndentOption() == true) {
                        if (flags[flags.length-1] == false) {
                           /** Case 4 **/
                            area_start = current_region.getStartPosition();
                            area_end   = current_region.getEndPosition();
                        } else {
                           /** Case 3 **/
                            area_start = previous.getEndRegion()+1;
                            area_end   = current_region.getEndPosition();
                        }
                        if (area_start < area_end) {
                            message_byte = current_region.extractRegion(
                                head[0], 
                                area_start,
                                area_end);
                            print_stream.print( 
                                getIndentMessage(
                                    message_byte, region_box.length) );
                        }
                        previous = current_region;
                    }
                    for (int i = 0; i < region_box.length-1; i++) {
                        print_stream.print(current_tab);
                    }
                    print_stream.println(ReportRegion.convertFromStartToEnd(
                        region_box[region_box.length-1].getTagString()));

                    bk_region = new Region[region_box.length-1];
                    for (int i = 0; i < bk_region.length; i++) {
                        bk_region[i] = region_box[i];
                    }
                    region_box = bk_region;

                    bk_flags = new boolean[flags.length-1];
                    for (int i = 0; i < bk_flags.length; i++) {
                        bk_flags[i] = flags[i];
                    }
                    flags = bk_flags;

                    next_start = current_region.getEndRegion()+1;
                    if (region_box.length != 0) {
                        current_region = region_box[region_box.length-1]; 
                        /*
                        next_end = current_region.getEndRegion();
                        */
                        next_end = current_region.getEndPosition();
                        if (next_start -1 == next_end) {
                            next_start = next_start -1;
                        }
                    }else {
                        next_start = 0;
                        next_end = 0;
                    }

                    if (exception_object.getMessage() != null &&
                        exception_object.getMessage().compareTo(
                            Region.END_OF_FILE_EXCEPTION) == 0) {
                        error = exception_object.getMessage();
                        break;
                    }
                    if (exception_object.getMessage() != null &&
                        exception_object.getMessage().compareTo(
                            Region.END_OF_REGION_EXCEPTION) == 0) {
                        break;
                    }
                    break;
                }

                if (getIndentOption() == true) {
                    if (previous != null) {
                        if (flags.length > 1 && flags[flags.length-2] == false) {
                        /** Case 1 **/
                            area_start = previous.getStartPosition();
                            area_end   = current_region.getStartRegion()-1;
                            flags[flags.length-2] = true;
                        } else {
                        /** Case 2 **/
                            area_start = previous.getEndRegion()+1;
                            area_end   = current_region.getStartRegion()-1;
                        }
                        if (area_start <= area_end) {
                            message_byte = current_region.extractRegion(
                                head[0], area_start, area_end);
                            print_stream.print( 
                                getIndentMessage(
                                    message_byte, region_box.length));
                        }
                    }
                }
                for (int i = 0; i < region_box.length-1; i++) {
                    print_stream.print(current_tab);
                }
                tag = ReportRegion.getTagStringWithOption(current_region);
                print_stream.println(tag);

                next_start = current_region.getStartPosition();
                next_end = current_region.getEndPosition();
                previous = current_region;
            }            
            if (error != null && error.compareTo(Region.END_OF_FILE_EXCEPTION) == 0) {
                break;
            }
        }
    }

    public static void main(String[] args) {
        DisplayHierarchy  display_hierarchy;
        String[]          target_args;
        String[]          file_args;
        int               indent_flag;
        String            message;

        display_hierarchy = new DisplayHierarchy();
        if (args.length < 0) {
            message = display_hierarchy.getMessage(
                DISPLAY_HIERARCHY_PARAMETER_EXCEPTION,
                DISPLAY_HIERARCHY_PARAMETER_EXCEPTION);

            System.err.println( message );
            System.exit(0);
            return;
        }

        indent_flag = -1;
        for (int i = 0; i < args.length; i++) {
            if (args[i] != null && args[i].compareTo(INDENT_OPTION) == 0) {
                display_hierarchy.setIndentOption();
                indent_flag = i;
                break;
            }
        }
        if (indent_flag != -1) {
            target_args = new String[args.length];
            file_args = new String[args.length-1];
        } else {
            target_args = new String[args.length+1];
            file_args = new String[args.length];
        }
        target_args[0] = WILDCARD;
        int j = 0;
        for (int i = 0 ; i < args.length; i++) {
            if (indent_flag == i) {
                j = -1;
                continue;
            }
            target_args[i+j+1] = args[i];
            file_args[i+j] = args[i];
        }
        display_hierarchy.setFileNames(file_args);

        try {
            display_hierarchy.execute(target_args, System.out, 
                (MakeReport)display_hierarchy);
        } catch (Exception exception_object) {
            System.err.println( exception_object.getMessage() );
            return;
        }
    }
}
