/**
 * Copyright (C) All rights reserved.
 *
 * @Module     IndexCard.java
 * @Author     Takahiko Tanaka.
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/08/10
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;

public class IndexCard extends DebugObject {
    private int board_index = -1;

    private ArrayList  array_list = new ArrayList();
    private static String DEBUG_SIZE_TITLE = "Size ";
    private static String DEBUG_ENTRY_TITLE = "ENTRY ";
    private static String DEBUG_BOOK_TITLE = "Book Tag ";
    private static String DEBUG_CHILD_TITLE = "     child ";
    private static String DEBUG_SPACE = " ";

    public boolean add(Object obj) {
        return add((BaseEntry)obj);
    }

    public boolean add(BaseEntry obj) {
        boolean  rt;
        int index_number;

        index_number = array_list.size();
        if (obj == null) {
            return false;
        }
        obj.setNumber(index_number);
        obj.enable();

        rt = array_list.add((Object)obj);
        return rt;
    }

    public boolean disable(int index) {
        BaseEntry obj;

        if (index < 0 || index >= array_list.size()) {
            return false;
        }
        obj = (BaseEntry)array_list.get(index);
        if (obj == null) {
            return false;
        }
        obj.disable();
        array_list.set(index, (Object)obj);
        return true;
    }

    public void setBoardNumber(int number) {
        board_index = number;
    }

    public int getBoardNumber() {
        return board_index;
    }

    public boolean replace(int no, BaseEntry obj) {
        boolean    rt;
        int        index_number;

        if (obj == null || no < 0 || no >= array_list.size()) {
            return false;
        }
        obj.enable();
        array_list.set(no, (Object)obj);

        return true;
    }

    public int size() {
        return array_list.size();
    }

    public Object get(int i) {
        return array_list.get(i);
    }

    public Object set(int i, Object obj) {
        return array_list.set(i, obj);
    }

    public void dump() {
        int  no;
        int  size;
        int  entry_size;
        BaseEntry entry;
        BookEntry book_entry;

        size = array_list.size();

        System.out.println(DEBUG_SIZE_TITLE + String.valueOf(size));
        for (int i = 0; i < size; i++) {
            entry = (BaseEntry)array_list.get(i);
            System.out.println(DEBUG_ENTRY_TITLE + entry.getName());
            if (entry.getName().compareTo(BookEntry.type_name) == 0) {
                book_entry = (BookEntry)entry;
                System.out.println(DEBUG_BOOK_TITLE + book_entry.getTag());
                entry_size = book_entry.getEntrySize();
                if (entry_size > 0) {
                    System.out.println(DEBUG_CHILD_TITLE);
                }
                for (int j = 0; j < entry_size; j++) {
                    no = book_entry.getEntryNumber(j);
                    System.out.print(DEBUG_SPACE + String.valueOf(no) + DEBUG_SPACE);
                    if (((j + 1) % 8) == 0) {
                        System.out.println();
                    }
                }
            }
        }
    }
}
