/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    InsertRegion.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version:   1.0 2011/06/01
 *                 2011/07/15 Complete
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * InsertRegion "<DIVE>,[PERSON name=Takahiko],<CUSTOMER>" files ...
 * [] is a pair of indicators for inserting tag.
 * [] are replaced with < and > respectively.
 * Insert tag is allowed only one time.
 */
public class InsertRegion extends RegionObject implements MakeReport {

    private static String INSERT_REGION = "InsertRegion";
    private static String MAKEREPORT_METHOD = "makeReport";
    private static String EXECUTE = "execute";

    public static String INSERT_ERROR = "Couldn't insert";
    public static String PARAMETER_ERROR = "Parameter error";


    public void makeReport(Object[] argument, PrintStream print_stream) throws Exception {
        InputStream             system_input;
        ReportRegion            report_region;
        String                  insert_tag;
        String                  insert_end_tag;
        String                  tag_content;
        String                  tag_keys;
        int                     tag_index;
        int                     list_count;
        String                  message;
        String                  message_log;
        byte[]                  return_byte;
        Region[]                region_object;
        ReportRegion            report_region_object;
        String                  value_keys;
        String[]                value_list;
        int                     list_index;
        int                     insert_position;
        int                     tab_count;
        String                  value_string;
        StringTokenizer         stk;
        Exception               exception_object;
        boolean                 duplication_flag;
        String                  lang;
        KeyList                 key_list;
        String                  key_list_string;
        boolean                 check;

        if (getDebug() > 0) {
            message_log = getMessage(ENTER_METHOD, ENTER_METHOD,
                SPACE, MAKEREPORT_METHOD);
            if (getLogPrintStream() == null) {
                setLogPrintStream( new LogPrintStream(System.err) );
            }
            getLogPrintStream().printLogln(message_log);
        }
        if (argument.length <= 1) {
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION, 
                ARGUMENT_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                BLANK_IN_BLANK), INSERT_REGION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (print_stream == null) {
            message = getMessage( STREAM_PARAMETER_EXCEPTION, 
                STREAM_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                BLANK_IN_BLANK), INSERT_REGION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        tag_keys = (String)argument[0];
        if (tag_keys == null || tag_keys.indexOf(START_BRACKET_STRING) < 0 ||
            tag_keys.indexOf(END_BRACKET_STRING) < 0) {
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION, 
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception(message);
            throw exception_object; 
        }

        insert_tag = null;
        insert_position = 0;
        String tag_key_string;
        try {
            key_list = KeyListFactory.createKeyList(tag_keys, COMMA,
                            getMessageResource(),
                            getDebug(),
                            getLogPrintStream());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        boolean  tag_key_flag = false;
        tag_key_string = new String();

        for (int i = 0; i < key_list.length(); i++) {
            try {
                key_list_string = key_list.getKeyEntry(i);
            } catch (Exception real_exception_object) {
                throw real_exception_object;
            }
            if (checkInsertTag(key_list_string) == true) {
                insert_position = i;
                insert_tag = key_list_string;
                insert_tag = insert_tag.replace(
                    START_BRACKET_STRING, TAG_START_STRING);
                insert_tag = insert_tag.replace( 
                    END_BRACKET_STRING, TAG_END_STRING);
                continue;
            }
            if (tag_key_flag == true) {
                tag_key_string = tag_key_string.concat(COMMA);
            }
            tag_key_flag = true;
            tag_key_string = tag_key_string.concat(key_list_string);
        }
        tag_keys = tag_key_string;

        for (int i = 1; i < argument.length; i++) {
            System.setIn((InputStream)argument[i]);       
            report_region_object = new ReportRegion(); 
            if (getDebug() > 0) {
                report_region_object.setDebug(getDebug());
                report_region_object.setMessageResource( 
                    getMessageResource() );
                report_region_object.setLogPrintStream( 
                    getLogPrintStream() );
            }
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                    SPACE, ReportRegion.CLEARREGIONOBJECT_METHOD,
                getMessageResource().getMessage(DEBUG_CODE_001)); 
                getLogPrintStream().printLogln(message_log);
            }
            try {
                report_region_object.clearRegionObject();
            } catch (Exception real_exception_object) {
                System.in.close();
                throw real_exception_object;
            }
            while (true) {
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, ReportRegion.READREGIONLIST_METHOD,
                    getMessageResource().getMessage(DEBUG_CODE_001)); 
                    getLogPrintStream().printLogln(message_log);
                }
                message = new String();
                try {
                    report_region_object.readRegionList(tag_keys, tag_keys);
                    message = report_region_object.getMessageFromRegionList();
                } catch (Exception real_exception_object) {
                    break;
                }
                value_keys = report_region_object.getKeys();

                tab_count = 0;
                list_index = 0;
                try {
                    key_list = KeyListFactory.createKeyList(value_keys, COMMA,
                        getMessageResource(),
                        getDebug(),
                        getLogPrintStream());
                } catch (Exception real_exception_object) {
                    throw real_exception_object;
                }
                for (int j = 0; j < key_list.length(); j++) {
                    if (j == insert_position) {
                        tab_count = ReportRegion.printStartTag(print_stream, 
                            tab_count, insert_tag);
                    }
                    try {
                        key_list_string = key_list.getKeyEntry(j);
                    } catch (Exception real_exception_object) {
                        continue;
                    }
                    tab_count = ReportRegion.printStartTag(print_stream, 
                        tab_count, key_list_string);
                }
                if (key_list.length() == insert_position) {
                    tab_count = ReportRegion.printStartTag(print_stream, 
                            tab_count, insert_tag);
                }
                print_stream.println(message);
                if (key_list.length() == insert_position) {
                    tab_count = ReportRegion.printEndTag(print_stream, 
                            tab_count, insert_tag);
                }
                for (int j = key_list.length()-1; j >= 0; j--) {
                    try {
                        key_list_string = key_list.getKeyEntry(j);
                    } catch (Exception real_exception_object) {
                        continue;
                    }
                    tab_count = ReportRegion.printEndTag(print_stream, tab_count, 
                        key_list_string);
                    if (j == insert_position) {
                        tab_count = ReportRegion.printEndTag(print_stream, 
                            tab_count, insert_tag);
                    }
                }
            }
            System.in.close();
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(END_METHOD, END_METHOD, 
                SPACE, MAKEREPORT_METHOD);
            getLogPrintStream().printLogln( message );
        }
    }

    public void execute(String[] org_args) throws Exception {
        String           message;
        PrintStream      print_stream;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, EXECUTE);
            getLogPrintStream().printLogln( message );
        }
        print_stream = System.out;
        try {
            super.enableInsertTag() ;
            super.execute(org_args, print_stream, (MakeReport)this);
        } catch (Exception real_exception_object) {
            throw real_exception_object; 
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(END_METHOD, END_METHOD, SPACE, EXECUTE);
            getLogPrintStream().printLogln( message );
        }
    }

    public static void main(String[] org_args) {
        String             message;
        FileOutputStream   file_output_stream;
        PrintStream        print_stream;
        InsertRegion       insert_region;
        String[]           args;
        String             debug_string;

        insert_region = new InsertRegion();
        debug_string = System.getProperty(EXEC_REGION_DEBUG);
        if (debug_string == null || debug_string.length() == 0) {
            debug_string = System.getProperty(DEBUG_LEVEL_KEY);
        }
        if (debug_string != null && debug_string.length() > 0) {
            insert_region.setDebug(debug_string);
        }
        insert_region.setLogPrintStream( new LogPrintStream( System.err ) );

        if (org_args.length < 1) {
            message = insert_region.getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION,
                insert_region.getMessageResource().getMessage(
                    BLANK_IN_BLANK),
                INSERT_REGION);
            System.out.println(message);
            return;
        }
        try {
            print_stream = System.out;
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = insert_region.getMessage( IO_ERROR_EXCEPTION, 
                    IO_ERROR_EXCEPTION,
                    insert_region.getMessageResource().getMessage(
                        BLANK_IN_BLANK), INSERT_REGION);
            }
            System.exit(1);
            return;
        }
        args = new String[org_args.length-1];
        for (int i = 1; i < org_args.length; i++) {
            args[i-1] = org_args[i];
        }
        
        try {
            insert_region.enableInsertTag() ;
            insert_region.execute(args);
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = insert_region.getMessage( INTERNAL_ERROR_EXCEPTION, 
                    INTERNAL_ERROR_EXCEPTION,
                    insert_region.getMessageResource().getMessage(
                        BLANK_IN_BLANK), INSERT_REGION);
            }
            System.err.println(message);
            System.exit(1);
        }
    }
}
