/**
 * Copyright (C) All rights reserved.
 *
 * @Module     KeyList.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2011/09/02 
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class KeyList extends DebugObject {
    private String[]   list;

    private KeyList() {
        list = new String[0];
    }

    private void expand(int count) throws Exception {
        String   message;

        if (count <= 0) {
            Exception    exception_object;
            message = getMessageResource().getFormattedMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                ARGUMENT_PARAMETER_EXCEPTION));

            exception_object = new Exception( message );
            throw exception_object;
        }
        list = new String[count];
    }

    public KeyList(int count) throws Exception {
        Exception  exception_object;

        if (count <= 0) {
            exception_object = new Exception( ARGUMENT_PARAMETER_EXCEPTION );
            throw exception_object;
        }
        list = new String[count];
    }

    public int length() {
        return list.length;
    }

    public String getKeyEntry(int index) throws Exception {
        String   message;
        Exception  exception_object;

        if (index >= list.length) {
            message = getMessageResource().getFormattedMessage(
                OVERFLOW_INDEX_EXCEPTION,
                getMessageResource().getMessage(
                OVERFLOW_INDEX_EXCEPTION));
            exception_object = new Exception( message );
            throw exception_object;
        }
        return list[index];
    }

    public void setKeyEntry(int index, String entry) throws Exception {
        String   message;
        Exception  exception_object;

        if (index >= list.length) {
            message = getMessageResource().getFormattedMessage(
                OVERFLOW_INDEX_EXCEPTION,
                getMessageResource().getMessage(
                OVERFLOW_INDEX_EXCEPTION));
            exception_object = new Exception( message );
            throw exception_object;
        }
        list[index] = entry;
    }

    public static KeyList valueOf(int count,
        MessageResource message_resource,
        int             debug,
        LogPrintStream  log_print_stream) throws Exception {
        KeyList   key_list;

        key_list = new KeyList();
        key_list.setMessageResource( message_resource );
        key_list.setDebug( debug );
        key_list.setLogPrintStream( log_print_stream );

        try {
            key_list.expand(count);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return key_list;
    }
}
