/**
 * Copyright (C) All rights reserved.
 *
 * @Module     LogPrintStream.java from OServ
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/01/26 
 */

package exec;

import java.lang.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class LogPrintStream implements Serializable {
    public static String MESSAGE_SEPARATOR = " ";
    public static String EMPTY = "";
    public static PrintStream ps = null;
    public static String MESSAGE = "Message";
    
    public LogPrintStream(String file) throws Exception{
        FileOutputStream fos;

        try {
            fos = new FileOutputStream(file);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        ps = new PrintStream(fos);
    }

    public LogPrintStream(OutputStream os) {
        ps = new PrintStream(os);
    }

    public LogPrintStream(PrintStream lps) {
        ps = lps;
    }

    public synchronized void printLogln(String key,java.lang.String[] args) {
        if (ps == null) { return ; }
        Date dt = new Date();
        ps.print(dt.toLocaleString());
        ps.print(MESSAGE_SEPARATOR);
        ps.print(Thread.currentThread().getName());
        if (key != null) {
            ps.print(MESSAGE_SEPARATOR + key);
        }
        if (args != null) {
            for (int i = 0; i < args.length; i++) {
                ps.print(MESSAGE_SEPARATOR + (String)args[i]);
            }
        }
        ps.println(EMPTY);
    }

    public synchronized void printLogln(java.lang.String... args) {
        if (ps == null || args == null) { return ; }
        Date dt = new Date();
        ps.print(dt.toLocaleString());
        ps.print(MESSAGE_SEPARATOR);
        ps.print(Thread.currentThread().getName());
        for (int i = 0; i < args.length; i++) {
            ps.print(MESSAGE_SEPARATOR + (String)args[i]);
        }
        ps.println(EMPTY);
    }

    public void close() {
        if (ps == null) { return; }
        ps.close();
    }

    public void flush() {
        if (ps == null) { return; }
        ps.flush();
    }

    public PrintStream getPrintStream() {
        if (ps == null) { return null; }
        return ps;
    }

    public boolean equals(PrintStream pps) {
        
        if (ps == null)  { return false; }
        if (pps == null) { return false; }

        return ps.equals(pps);
    }

    public static void main(String[] args) {
        LogPrintStream  log_print_stream;

        log_print_stream = new LogPrintStream(System.out);

        log_print_stream.printLogln(MESSAGE);
    }
}
