/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    MessageResource.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version:   2.1 2011/11/23
 *
 */

package exec;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;
import java.util.jar.*;
import java.util.zip.*;
import java.net.*;

public class MessageResource implements Serializable {
/**
    private static StringObject string_object = new StringObject();
    private static StringObject format_object = new StringObject();
**/
    private StringObject string_object = null;
    private StringObject format_object = null;

    public final static String EXEC_REPORT_LANG = "exec.report.lang";

    private final static String ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE
        = "エントリが見つかりませんでした。";
    private final static String PARAMETER_ERROR_EXCEPTION_JAPANESE
        = "パラメータ異常が発生しました。";
    private final static String INITIALIZATION_ERROR_EXCEPTION_JAPANESE
        = "初期化例外が発生しました。";
    private final static String ILLEGAL_LANGUAGE_EXCEPTION_JAPANESE
        = "不正な言語が指定されました。";
    private final static String PARAMETER_ERROR_EXCEPTION 
        = "Parameter error exception";
    private final static String INITIALIZATION_ERROR_EXCEPTION
        = "Initialization error exception";
    private final static String ILLEGAL_LANGUAGE_EXCEPTION
        = "Illegal language exception";

    private String force_language = null;

    public final static int STRING_PAIR = 2;
    public final static int BUFFER_SIZE = 4096;

    public final static char COMMENT_CHAR = '#';
    public final static char SPACE_CHAR = ' ';

    public final static String EMPTY_MESSAGE = "";
    public final static String LIBRARY_SEPARATOR = ":";
    public final static String DIRECTORY_SEPARATOR = "/";
    public final static String VARIABLE_SEPARATOR = "_";
    public final static String OBJECT_DOT_SEPARATOR = ".";
    public final static String OBJECT_SEPARATOR = "/";
    public final static String SUFFIX_SEPARATOR = ".";
    public final static String MESSAGE_SEPARATOR = " ";
    public final static String MESSAGE_LIST_SEPARATOR = "=";
    public final static String PARAMETER_SEPARATOR = ",";

    public final static String JAVA_SUFFIX = "java";
    public final static String JAR_SUFFIX = "jar";

    public final static String PACKAGE_NAME = "exec";
    public final static String JAVA_CLASS_PATH = "java.class.path";

    public final static String FILE_URL_PROTOCOL = "file://";
    public final static String JAR_URL_PROTOCOL = "jar:";
    public final static String JAR_URL_HEAD = "jar:file:";
    public final static String JAR_URL_END  = "!/";

    private static String   package_name = PACKAGE_NAME;
    private final static String[] pathlist = { JAVA_CLASS_PATH };
    private final static String format_field_name = "format";

    public final static String[] lang_list
        = { "aar", "abk", "ave", "afr", "aka",
            "amh", "arg", "ara", "asm", "ava",
            "aym", "aze", "bak", "bel", "bul",
            "bih", "bis", "bam", "ben", "bod", 
            "bre", "bos", "cat", "che", "cha",
            "cos", "cre", "ces", "chu", "chv",
            "cym", "dan", 
/**
            "deu", 
**/
            Locale.GERMANY.getISO3Language(),
            "div", "dzo",
            "ewe", "ell", 
            Locale.US.getISO3Language(),
/**
            "eng", 
**/
            "epo", "spa",
            "est", "eus", "fas", "ful", "fin",
            "fij", "fao", 
            Locale.FRANCE.getISO3Language(),
/**
            "fra", 
**/
            "fry", "gle",
            "gla", "glg", "grn", "guj", "glv",
            "hau", "heb", "hin", "hmo", "hrv",
            "hat", "hun", "hye", "her", "ina",
            "ind", "ile", "ibo", "iii", "ipk",
            "ind", "ido", "isl", 
            Locale.ITALIAN.getISO3Language(),
/**
            "ita", 
**/
            "iku",
            "heb", 
            Locale.JAPAN.getISO3Language(),
/**
            "jpn", 
**/
            "yid", "jav", "kat",
            "kon", "kik", "kua", "kaz", "kal",
            "khm", "kan", 
            Locale.KOREAN.getISO3Language(),
/**
            "kor", 
**/
            "kau", "kas",
            "kur", "kom", "cor", "kir", "lat",
            "ltz", "lug", "lim", "lin", "lao",
            "lit", "lub", "lav", "mlg", "mah",
            "mri", "mkd", "mal", "mon", "mol",
            "mar", "msa", "mlt", "mya", "nau",
            "nob", "nde", "nep", "ndo", "nld",
            "nno", "nor", "nbl", "nav", "nya",
            "oci", "oji", "orm", "ori", "oss",
            "pan", "pli", "pol", "pus", "por",
            "que", "roh", "run", "ron", "rus",
            "kin", "san", "srd", "snd", "sme",
            "sag", "sin", "slk", "slv", "smo",
            "sna", "som", "sqi", "srp", "ssw",
            "sot", "sun", "swe", "swa", "tam",
            "tel", "tgk", "tha", "tir", "tuk",
            "tgl", "tsn", "ton", "tur", "tso",
            "tat", "twi", "tah", "uig", "ukr",
            "urd", "uzb", "ven", "vie", "vol",
            "wln", "wol", "xho", "yid", "yor",
            "zha", 
            Locale.CHINA.getISO3Language(),
/**
            "zho", 
**/
            "zul", }; 

    /**
     * message format is equal to a row of number.
     * The order means message's oder.
     * Each order is separated by ','.
     */
    private static String default_format = "0";

    public MessageResource() {
        /**
        loadLocaleMessage();
        **/
    }

    public boolean checkMessage() {
        if (string_object == null || string_object.size() <= 0) {
            return false;
        }
        return true;
    }

    public void setStringObject(StringObject    string_box,
        StringObject    format_box) {

        string_object = string_box;  
        format_object = format_box;
    }

    public StringObject getStringObject() {
        return string_object;
    }

    public StringObject getFormatObject() {
        return format_object;
    }
   
    public void setDefaultLanguage() {
        Locale   lc;

        lc = Locale.getDefault();

        force_language = lc.getISO3Language();
        string_object = new StringObject();
        format_object = new StringObject();
        loadLocaleMessage();
    }

    public void setLanguage(String language) throws Exception {
        String   message;

        if (language == null) {
            force_language = language;
            string_object = new StringObject();
            format_object = new StringObject();
            loadLocaleMessage();
        }
        for (int i = 0; i < lang_list.length; i++) {
            if (language.compareTo(lang_list[i]) == 0) {
                if (force_language == null ||
                    force_language.compareTo(language) != 0) {
                    force_language = language;
                    string_object = new StringObject();
                    format_object = new StringObject();
                    loadLocaleMessage();
                    return;
                }
            }
        }

        Exception exception_object;
        message = ILLEGAL_LANGUAGE_EXCEPTION;
        if (isJapanese() == true) {
            message = ILLEGAL_LANGUAGE_EXCEPTION_JAPANESE;
        }
        exception_object = new Exception( message );
        throw exception_object;
    }

    public String getLanguage() {
        return force_language;
    }

    private boolean loadMessage(String path) throws Exception {
        int      idx;
        int      rcount;
        byte[]   buffer;
        String   line;
        Locale   lc;
        Object   inst;
        String   file_entry;
        String   language;
        String[] element;
        String   filepath;
        String   message_path;
        String   message_class;
        String   field_name;
        Field[]  field_object;
        DataInputStream        data_input_stream;
        FileInputStream        file_input_stream;
        JarInputStream         jar_input_stream;
        ByteArrayInputStream   byte_array_input_stream;
        ByteArrayOutputStream  byte_array_output_stream;

        if (path == null || string_object == null) {
            return false;
        }
        
        language = getLanguage();
        if (language == null) {
            lc = Locale.getDefault();
            /**
             * jpn - Japan
             * eng - English
             *
             * java/util/LocaleISOData.java
             */
            language = lc.getISO3Language();
        } else {
        }

        file_entry = package_name + DIRECTORY_SEPARATOR +
                     package_name + VARIABLE_SEPARATOR + language;

        message_class = package_name + OBJECT_DOT_SEPARATOR +
               package_name + VARIABLE_SEPARATOR + language ;

        field_name = language;

        idx = path.lastIndexOf(SUFFIX_SEPARATOR+JAR_SUFFIX);
        if (idx > 0) {
            filepath = new String(JAR_URL_HEAD + path + JAR_URL_END);
        } else {
            filepath = new String(FILE_URL_PROTOCOL + path);
        }
                
        URL url;
        Class sp;
        try {
            url = new URL(filepath);
        } catch (Exception exception_object) {
            return false ;
        }
        URL[] urls = new URL[1];
        urls[0] = url;
        URLClassLoader uloader = new URLClassLoader(urls);      
        try {
            sp = Class.forName(
                message_class, true, (ClassLoader)uloader);
        } catch (Exception exception_object) {
            return false;
        }
        try {
            inst = sp.newInstance();
        } catch (Exception exception_object) {
            return false;
        }

        String[] message_list;
        element = new String[2];
        field_object = sp.getFields();
        for (int i = 0; i < field_object.length; i++) {
            if (field_object[i].getName().compareTo(field_name) == 0) {
                message_list = (String[])field_object[i].get(inst);
                for (int j = 0; j < message_list.length; j++) {
                    element = message_list[j].split(MESSAGE_LIST_SEPARATOR);
                    if (element.length == 2) {
                        string_object.add(element[0], element[1]);
                    }
                }
                continue;
            }
            if (field_object[i].getName().compareTo(format_field_name) == 0) {
                message_list = (String[])field_object[i].get(inst);
                for (int j = 0; j < message_list.length; j++) {
                    element = message_list[j].split(MESSAGE_LIST_SEPARATOR);
                    if (element.length == 2) {
                        format_object.add(element[0], element[1]);  
                    }
                }
            }
        }

        return false;
    }

    private boolean isJapanese() {
        String language;
        Locale lc;

        language = getLanguage();
        if (language == null) {
            lc = Locale.getDefault();
            language = lc.getISO3Language();
        }
        if (language.compareTo(Locale.JAPAN.getISO3Language()) == 0) {
            return true;
        }
        return false;
    }

    private String searchMessage(String key) throws Exception {
        int    index_number;
        String target_message;
        Exception    real_exception_object;

        if (key == null || key.length() == 0) {
            if (isJapanese() == true) {
                real_exception_object = new Exception(
                    PARAMETER_ERROR_EXCEPTION_JAPANESE);
            } else {
                real_exception_object = new Exception(
                    PARAMETER_ERROR_EXCEPTION    );
            }
            throw real_exception_object;
        }

        if (string_object == null || string_object.size() == 0) {
            if (isJapanese() == true) {
                real_exception_object = new Exception(
                    INITIALIZATION_ERROR_EXCEPTION_JAPANESE    );
            } else {
                real_exception_object = new Exception(
                    INITIALIZATION_ERROR_EXCEPTION    );
            }
            throw real_exception_object;
/*
            Exception exception_object;

            exception_object = new Exception(INITIALIZATION_ERROR_EXCEPTION);
            throw exception_object;
*/
        }
        index_number = 0;
        try {
            target_message = string_object.getMessage(key);
        } catch (Exception exception_object) {
            if (isJapanese() == true) {
                real_exception_object = new Exception(
                    ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE
                );
                throw real_exception_object;
            }
            throw exception_object;
        }

        return target_message;
    }

    private String getMessagePathList() {
        String library_path;

        library_path = System.getProperty(pathlist[0]);
     
        return library_path;
    }

    public String getMessage(String message_key) {
        String  message_string;
        String  message_path;
        String  message_path_list;
        StringTokenizer string_tokenizer_object;

        /**
        loadLocaleMessage();
        **/
        try {
            message_string = searchMessage(message_key);
        } catch (Exception exception_object) {
            message_string = new String();
            if (message_key != null) {
                message_string = message_key;
            }
        }
        return message_string;
    }

    private void loadLocaleMessage() {
        String  message_string;
        String  message_path;
        String  message_path_list;
        StringTokenizer string_tokenizer_object;

        if (string_object != null && checkMessage() != true) {
            message_path_list = getMessagePathList();
            string_tokenizer_object = new StringTokenizer(message_path_list,
                LIBRARY_SEPARATOR);
            while (string_tokenizer_object.hasMoreTokens()) {
                message_path = string_tokenizer_object.nextToken();
                try {
                    loadMessage(message_path);
                } catch (Exception exception_object) {
                }
                if (checkMessage() == true) { 
                    break;
                }
            }
        }
    }

    private String searchFormat(String key) throws Exception {
        String    target_message;
        int       index_number;
        Exception    real_exception_object;

        if (format_object == null) {
            return EMPTY_MESSAGE;
        }
        index_number = 0;
        try {
            target_message   = format_object.getMessage(key);
        } catch (Exception exception_object) {
            if (isJapanese() == true) {
                real_exception_object = new Exception(
                    ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE
                );
                throw real_exception_object;
            }
            throw exception_object;
        }
 
        return target_message;
    }

    public String getMessageFormat(String message_key) throws Exception {
        String format_string;
        loadLocaleMessage();
        try {
            format_string = searchFormat(message_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return format_string;
    }

    public String getFormattedMessage(
        String message_key, String... message_list ) {
        int                index_number;
        StringTokenizer    stk;
        String             format_string;
        String             return_message;

        return_message = new String();
        try {
            format_string = getMessageFormat(message_key);
            if (format_string == null) {
                for (int i = 0; i < message_list.length; i++) {
                    return_message
                        = return_message.concat(message_list[i]);
                }
                return return_message;
            }
        } catch (Exception exception_object) {
            for (int i = 0; i < message_list.length; i++) {
                return_message
                    = return_message.concat(message_list[i]);
            }
            return return_message;
        }
        stk = new StringTokenizer(format_string, PARAMETER_SEPARATOR);
        if (stk.countTokens() == 0) {
            for (int i = 0; i < message_list.length; i++) {
                return_message
                    = return_message.concat(message_list[i]);
            }
            return return_message;
        }

        while (stk.hasMoreTokens()) {
            try {
                index_number = Integer.valueOf(stk.nextToken().trim());
            } catch (Exception exception_object) {
                continue;
            }
            if (index_number < message_list.length) {
                if (message_list[index_number].length() >= 2 && message_list[index_number].charAt(0) == '\\') {
                    if (message_list[index_number].charAt(1) != '\\') {
                        continue;
                    }
                    return_message = return_message.concat(
                        message_list[index_number].substring(1));
                }
                
                return_message 
                    = return_message.concat(message_list[index_number]);
            }
        }
        return return_message;
    }

    private static String ARGUMENT_PARAMETER_EXCEPTION = "Argument Parameter Exception";

    public static void main(String[] args) {
        String           message;
        MessageResource  message_resource;

        message_resource = new MessageResource();

        if (args.length == 1) {
            try {
                message_resource.setLanguage(args[0]);
                message = message_resource.getMessage(
                    ARGUMENT_PARAMETER_EXCEPTION);
                System.out.println( message );
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
        }
    }
}
