/**
 * Copyright (C) All rights reserved.
 *
 * @Module     OperateReport.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/08/31
 *
 */

package exec;

import java.lang.*;
import java.io.*;

public class OperateReport extends DebugObject {
    private BookCenter  book_center;

    private static String EMPTY_STRING = "";
    private static String SPACE_STRING = " ";
    private static String FOUR_SPACE_STRING = "    ";
    private static String SLASH_STRING = "/";

    private static String REPORT_ERROR = "Report error";
    private static String PARAMETER_ERROR = "Parameter error";

    public OperateReport() {
        book_center = new BookCenter();
    }

    public OperateReport(String[]  files) throws Exception {
        String   message;
        book_center = new BookCenter();

        if (files == null) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        for (int i = 0; i < files.length; i++) {
            try {
                book_center.addBook(files[i]);    
            } catch (Exception  exception_object) {
                throw exception_object;
            }
        }
    }

    public static OperateReport valueOf(String[] files,
        MessageResource message_resource,
        int             debug,
        LogPrintStream  log_print_stream) throws Exception {

        OperateReport  operate_report;
        if (files == null) {
            operate_report = null;
            return operate_report;
        }
        try {
            operate_report = new OperateReport();
            operate_report.setDebug( debug );
            operate_report.setMessageResource( message_resource );
            operate_report.setLogPrintStream( log_print_stream );
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return operate_report;
    }

    public static OperateReport valueOf(String[] files) {
        OperateReport  operate_report;

        if (files == null) {
            operate_report = null;
            return operate_report;
        }
        try {
            operate_report = new OperateReport(files);
        } catch (Exception exception_object) {
            operate_report = null;
        }
        return operate_report;
    }

    public ArticlePost getArticles(String keys) throws Exception {
        int              no;
        String[]         tags;
        IndexCard        book_card;
        ArticleEntry     article_entry;
        ArticlePost      local_article_post;
        ArticleEntry     operate_entry;

        if (keys == null) {
            Exception exception_object;
            String   message;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        operate_entry = new ArticleEntry();
        operate_entry.setMessageResource( getMessageResource() );
        operate_entry.setDebug( getDebug() );
        operate_entry.setLogPrintStream( getLogPrintStream() );

        local_article_post = new ArticlePost();
        for (int i = 0; i < book_center.getSize(); i++) {
            try {
                book_card = book_center.getIndexCard(i);
            } catch (Exception exception_object) {
                continue;
            }

            try {
                tags = operate_entry.getKeys(keys, book_card);
            } catch (Exception  exception_object) {
                continue;
            }

            if (tags == null) {
                break;
            }
            for (int j = 0; j < tags.length; j++) {
                try {
                    local_article_post = ArticlePost.valueOf(tags[j], book_card, local_article_post,
                        getMessageResource(), getDebug(), getLogPrintStream());
                } catch (Exception exception_object) {
                    continue;
                }
            }
        }
        return local_article_post;
    }


    public ArticlePost addArticles(String keys, ArticlePost article_post) throws Exception {
        IndexCard        book_card;
        ArticlePost      local_article_post;

        if (keys == null || article_post == null) {
            Exception exception_object;
            String   message;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        for (int i = 0; i < book_center.getSize(); i++) {
            try {
                book_card = book_center.getIndexCard(i);
            } catch (Exception exception_object) {
                throw exception_object;
            }

            try {
                article_post = ArticlePost.valueOf(keys, book_card, article_post, getMessageResource(), getDebug(), getLogPrintStream());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return article_post;
    }

    public static String[] getTags(String tags) throws Exception {
        String[]    tag_array = new String[0];
        String[]    bk;
        int         count;
        int         index;
        String      target;

        if (tags == null) {
            Exception exception_object;
            String   message;
/**
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
**/
            message = ARGUMENT_PARAMETER_EXCEPTION;
            exception_object = new Exception( message );
            throw exception_object;
        }
        count = 0;
        target = tags;
        for (int i = tags.length(); i > 0; ) {
            index = target.indexOf(TagString.TAG_END_STRING);
            if (index < 0) {
                break;
            }
            bk = new String[tag_array.length+1];
            for (int j = 0; j < tag_array.length;j++) {
                bk[j] = tag_array[j];
            }
            tag_array = bk;
            tag_array[count] = target.substring(0, index+1);
            target = target.substring(index+1);
            i = i - index - 1;
            count = count + 1;
        }

        return tag_array;
    }

    public void printReportStartTag(String tags) throws Exception {
        Exception exception_object;
        boolean  flag;
        String   message;

        if (tags == null) {
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        flag = true;
        printReportTag(tags, flag);
    }

    public void printReportEndTag(String tags) throws Exception {
        Exception exception_object;
        boolean   flag;
        String    message;

        if (tags == null) {
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        flag = false;
        printReportTag(tags, flag);
    }

    private void printReportTag(String tags, boolean flag) throws Exception {
        int         index;
        int         count;
        int         length;
        String      target;
        String[]    hierarchy = new String[0];
        String[]    bk;
        TagString   tag_string;
        String      message;
        Exception   exception_object;

        if (tags == null) {
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        count = 0;

        length = tags.length();
        target = tags;
        for (int i = length; i > 0; ) {
            index = target.indexOf(TagString.TAG_END_STRING);
            if (index < 0) {
                break;
            }
            bk = new String[hierarchy.length+1];
            for (int j = 0; j < hierarchy.length;j++) {
                bk[j] = hierarchy[j];
            }
            hierarchy = bk;
            hierarchy[count] = target.substring(0, index+1);
            target = target.substring(index+1);
            i = i - index - 1;
            count++;
        }

        if (flag == true) {
            for (int i = 0; i < hierarchy.length; i++) {
                for (int j = 0; j < i; j++) {
                    System.out.print(FOUR_SPACE_STRING);
                }
                tag_string = new TagString(hierarchy[i]);
                System.out.println(tag_string.getStartTag());
            }
        } else {
            for (int i = 0; i < hierarchy.length; i++) {
                for (int j = hierarchy.length-1; j > i; j--) {
                    System.out.print(FOUR_SPACE_STRING);
                }
                tag_string = new TagString(hierarchy[hierarchy.length-i-1]);
                System.out.println(tag_string.getEndTag());
            }
        }
    }


    public IndexCard getReport(ArticlePost article_post) throws Exception {
        int             tag_count = 0;
        int             target_index;
        byte[]          report;
        String          tag_string;
        String          target_tag;
        ReportEntry     report_entry;
        ArticleEntry    article_entry;
        ArticleEntry    target_article;
        ArticleEntry[]  article_array;
        ArticleEntry[]  target_array;
        boolean[]       invalid;
        TagString       local_tag = null;
        TagString       sub_tag = null;
        IndexCard       report_card;
        IndexCard       book_card;
        IndexCard       target_card;

        Object          object_tmp;

        if (article_post == null) {
            String     message;
            Exception  exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION, ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        report_card = new IndexCard();
        invalid = new boolean[article_post.size()];
        for (int i = 0; i < invalid.length; i++) {
            invalid[i] = true;
        }
        for (int i = 0; i < article_post.size(); i++) {
            if (invalid[i] == false) { continue; }
            invalid[i] = false;
            try {
                object_tmp = article_post.get(i);

                article_entry = (ArticleEntry)article_post.get(i);
            } catch (Exception exception_object) {
/***
                exception_object.printStackTrace();
***/
                continue;
            }
            if (article_entry.getChild() != true) {
                continue;
            }
            try {
                book_card = book_center.getIndexCard(article_entry.getBoardNumber());
            } catch (Exception exception_object) {
/***
                exception_object.printStackTrace();
***/
                continue;
            }

            try {
                article_array = article_post.getArticleArray(article_entry) ;
            } catch (Exception exception_object) {
/***
                exception_object.printStackTrace();
***/
                continue;
            }

            tag_string = new String();
            for (int j = 0; j < article_array.length; j++ ) {
                tag_string = tag_string.concat( 
                ((BookEntry)book_card.get(article_array[j].getOwn())).getTag());
            }
            report_entry = new ReportEntry();

            report_entry.setTagString(tag_string);
            try {
                report_entry.appendArticle(i);
            } catch (Exception exception_object) {
/**
                exception_object.printStackTrace();
**/
                continue;
            }
            for (int j = i+1; j < article_post.size(); j++) {
                if (invalid[j] == false) { continue; }
                try {
                    target_article = (ArticleEntry)article_post.get(j);
                } catch (Exception exception_object) {
/**
                    exception_object.printStackTrace();
**/
                    continue;
                }
                if (target_article.getChild() != true) {
                    continue;
                }
                try {
                    target_card = book_center.getIndexCard(target_article.getBoardNumber());
                } catch (Exception exception_object) {
/**
                    exception_object.printStackTrace();
**/
                    continue;
                }
                try {
                    target_array = article_post.getArticleArray(target_article) ;
                } catch (Exception exception_object) {
/**
                    exception_object.printStackTrace();
**/
                    continue;
                }

                for (target_index = 0; target_index < article_array.length;
                                 target_index++) {
                    if (TagString.compareTag(
                        ((BookEntry)book_card.get(article_array[target_index].getOwn())).getTag(),
                        ((BookEntry)target_card.get(target_array[target_index].getOwn())).getTag()) != true) {
                        break;
                    }
                }
                if (target_index == article_array.length) {
                    try {
                        report_entry.appendArticle(j);
                    } catch (Exception exception_object) {
/**
                        exception_object.printStackTrace();
**/
                        continue;
                    }
                    invalid[j] = false; 
                }
            }
            report_card.add(report_entry);
        }
        return report_card;
    }

    public void printReport(ReportEntry report_entry, ArticlePost article_post) throws Exception {
        int             index;
        String          tag_string;

        if (report_entry == null || article_post == null) {
            String   message;
            Exception  exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        tag_string = report_entry.getTagString();
        try {
            printReportStartTag(tag_string);
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        for (int j = 0; j < report_entry.size(); j++) {
            try {
                index = report_entry.getArticleIndex(j);
            } catch (Exception real_exception_object) {
                continue;
            }
            try {
                printArticlePost(index, article_post);
            } catch (Exception real_exception_object) {
                continue;
            }
        }
        try {
            printReportEndTag(tag_string);
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
    }

    public void printReport(IndexCard report_card, ArticlePost article_post) throws Exception {
        int             index;
        ReportEntry     report_entry;
        ArticleEntry    article_entry;
        StoreHouse      store_house;
        IndexCard       book_card;
        Exception       exception_object;
        String          tag_string;
        ArticleEntry[]  article_array;
        String          message;

        if (report_card == null || article_post == null) {
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }

        for (int i = 0; i < report_card.size(); i++) {
            report_entry = (ReportEntry)report_card.get(i);

            try {
                printReport(report_entry, article_post);
            } catch (Exception real_exception_object) {
                throw real_exception_object;
            }
        } 
    }

    public void printArticlePost(int number, ArticlePost article_post) throws Exception {
        StoreHouse      store_house;
        IndexCard       book_card;
        ArticleEntry[]  article_array;
        ArticleEntry    article_entry;

        if (article_post == null) {
            String     message;
            Exception  exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            article_entry = (ArticleEntry)article_post.get(number);
            book_card = book_center.getIndexCard(article_entry.getBoardNumber());
            store_house = book_center.getStoreHouse(article_entry.getBoardNumber());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        if (article_entry.getChild() != true) {
            return;
        }
        try {
            article_array = article_post.getArticleArray(article_entry) ;
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        article_entry.printArticleContents(article_array, book_card, store_house);
    }

    
    public ArticlePost addArticle(byte[] area, ArticlePost article_post) 
        throws Exception {
        StoreHouse    store_house;
        IndexCard     index_card;
        BookEntry     book_entry;
        ArticleEntry  article_entry;

        if (area == null) {
            String     message;
            Exception  exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (article_post == null) {
            String     message;
            Exception  exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            store_house = getStoreHouse(area);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        index_card = new IndexCard();
        try {
            index_card = getBookIndexCard(index_card, store_house);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            index_card = setBookCenter(store_house, index_card);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            book_entry = (BookEntry)index_card.get(0);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        article_entry = new ArticleEntry();
        article_entry.setBoardNumber(index_card.getBoardNumber());
        article_entry.setOwn(book_entry.getNumber());
        article_entry.setTag(EMPTY_STRING);
        article_entry.setChild(true);
        
        article_post.add(article_entry);
        return article_post;
    }

    public IndexCard setBookCenter(StoreHouse store_house, 
        IndexCard  index_card) throws Exception {
        IndexCard new_index_card;

        if (index_card == null || store_house == null) {
            String     message;
            Exception  exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            new_index_card = book_center.addEntry(StoreHouse.MEMORY_STORAGE,
                store_house, index_card);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return new_index_card;
    }

    public StoreHouse getStoreHouse(byte[] area) throws Exception {
        String       message;
        StoreHouse   store_house;

        if (area == null) {
            Exception  exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            store_house = StoreHouse.valueOf(area);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return store_house;
    }

    public IndexCard getBookIndexCard(IndexCard index_card, 
        StoreHouse  store_house) throws Exception {
        int         no;
        BookEntry   book_entry;
        PageEntry   page_entry;
        AreaRange   start_tag;
        AreaRange   end_tag;
        String      message;

        if (index_card == null || store_house == null) {
            Exception  exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        no = -1;
        try {
            book_entry = BookEntry.valueOf();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        start_tag = new AreaRange();
        start_tag.setStartPosition(store_house.getPageStart()-1);
        start_tag.setEndPosition(store_house.getPageStart()-1);

        end_tag = new AreaRange();
        end_tag.setStartPosition(store_house.getPageEnd()+1);
        end_tag.setEndPosition(store_house.getPageEnd()+1);

        book_entry.setStartTag(start_tag);
        book_entry.setStartTag(end_tag);

        book_entry.setParent(no);
        index_card.add(book_entry);

        page_entry = new PageEntry();
        page_entry.setStart(store_house.getPageStart());
        page_entry.setEnd(store_house.getPageEnd());
        index_card.add(page_entry);
        book_entry.addEntry(page_entry.getNumber());
        index_card.replace(0, book_entry);

        return index_card;
    }

    public IndexCard addReport(
        IndexCard report_card, int article_index) throws Exception {
        String        message;
        ReportEntry   report_entry;

        if (report_card == null || article_index < 0) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        report_entry = new ReportEntry();
        report_entry.setTagString(EMPTY_STRING);

        try {
            report_entry.appendArticle(article_index);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            throw exception_object;
        }
        report_card.add(report_entry);

        return report_card;
    }

    public IndexCard insertArticle(
        IndexCard report_card,
        int index,
        int article_offset,
        int article_index) throws Exception {
        String       message;
        ReportEntry  report_entry;

        if (report_card == null || index < 0 || index >= report_card.size()) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        report_entry = (ReportEntry)report_card.get(index);
        if (article_offset < 0 || article_offset >= report_entry.size()) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            report_entry.insertArticle(article_offset, article_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        report_card.set(index, report_entry);
        return report_card;
    }

    public IndexCard insertTag(IndexCard report_card, int index, 
        String position, String tag) throws Exception {
        String      tag_string;
        String[]    tag_array;
        ReportEntry report_entry;

        if (report_card == null || tag == null ||position == null) {
            String      message;
            Exception   exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
       
        report_entry = (ReportEntry)report_card.get(index);
        tag_string = report_entry.getTagString();

        tag_array = getTags(tag_string);
        for (int j = 0; j < tag_array.length; j++) {
            if (tag_array[j] != null 
                && TagString.compareKey(tag_array[j],position) == true) {
                tag_string = new String();
                for (int k = 0; k < tag_array.length; k++) {
                    tag_string = tag_string.concat(tag_array[k]);
                    if (k == j) { 
                        tag_string = tag_string.concat(tag);
                    }
                }
                report_entry.setTagString(tag_string);
                report_card.replace(index, report_entry);
                break;
            }
        }
        return report_card;
    }

    public boolean checkChildren(IndexCard report_card, int index, 
        String tag_string, IndexCard article_card) throws Exception {
        int            no;
        ReportEntry    report_entry;
        ArticleEntry   article_entry;
        String         message;

        if (report_card == null || tag_string == null ||
            article_card == null || report_card.size() >= index) {
            Exception   exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );

            throw exception_object;
        }
        report_entry = (ReportEntry)report_card.get(index);
        if (report_entry.size() <= 0) {
            return false;
        }
        try {
            no = report_entry.getArticleIndex(index);
        } catch (Exception exception_object) {
            throw exception_object;
        } 
        try {
            article_entry = (ArticleEntry)article_card.get(no);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (tag_string.compareTo(article_entry.getTag()) != 0) {
            return false;
        }
        return true;
    }

    public int[] getParent(IndexCard report_card, int index,
        IndexCard article_card) throws Exception {
        int            no;
        int[]          parent_no;
        ArticleEntry   article_entry;
        ReportEntry    report_entry;
        String         message;

        if (report_card == null || report_card.size() >= index ||
            article_card == null) {
            Exception   exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        report_entry = (ReportEntry)report_card.get(index);

        parent_no = new int[report_entry.size()];
        for (int i = 0; i < report_entry.size(); i++) {
            try {
                no = report_entry.getArticleIndex(i);
            } catch (Exception exception_object) {
                throw exception_object;
            } 
            try {
                article_entry = (ArticleEntry)article_card.get(no);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            parent_no[i] = article_entry.getParent();
        } 
        return parent_no;
    }

    public int[] getChildren(IndexCard report_card, int index)
        throws Exception {
        int[]          children_no;
        String         tag_string;
        String[]       tag_array;
        ReportEntry    report_entry;
        String         message;

        if (report_card == null || report_card.size() >= index) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        report_entry = (ReportEntry)report_card.get(index);

        children_no = new int[report_entry.size()];
        for (int i = 0; i < report_entry.size(); i++) {
            try {
                children_no[i] = report_entry.getArticleIndex(i);
            } catch (Exception exception_object) {
                throw exception_object;
            } 
        } 
        return children_no;
    }

    public IndexCard deleteArticle(IndexCard report_card, int index, 
        int article_index) throws Exception {
        String      message;
        ReportEntry report_entry;

        if (report_card == null || index < 0 || index >= report_card.size()) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        report_entry = (ReportEntry)report_card.get(index);
        if (article_index < 0 || article_index >= report_entry.size()) {
            Exception   exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            report_entry.removeArticle(article_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        report_card.set(index, report_entry);
        return report_card;
    }

    public IndexCard deleteTag(IndexCard report_card, int index, int[] parent_no, String tag) throws Exception {
        int            child_no;
        int            child_index;
        int            parent_index;
        String         tag_string;
        String[]       tag_array;
        ArticleEntry   child_article;
        ArticleEntry   parent_article;
        ReportEntry    report_entry;

        if (report_card == null || tag == null) {
            String      message;
            Exception   exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        report_entry = (ReportEntry)report_card.get(index);
        tag_string = report_entry.getTagString();

        tag_array = getTags(tag_string);
        for (int j = 0; j < tag_array.length; j++) {
            if (tag_array[j] != null 
                && TagString.compareKey(tag_array[j],tag) == true) {
                tag_string = new String();
                for (int k = 0; k < tag_array.length; k++) {
                    if (k == j) { continue; }
                    tag_string = tag_string.concat(tag_array[k]);
                }
                report_entry.setTagString(tag_string);
                if (parent_no != null) {
                    if (parent_no.length == report_entry.size()) {
                        for (int k = 0; k < parent_no.length; k++) {
                            if (parent_no[k] != -1) {
                                report_entry.replaceArticle(k, parent_no[k]);
                            }
                        }
                    }
                }
                report_card.replace(index, report_entry);
                break;
            }
        }
        return report_card;
    } 

    private static int DEBUG_DEBUG = 1;

    public static void main(String[] args) {
        String[]       files;
        OperateReport  operate_report;
        ArticlePost    articles;
        byte[]         byte_report;
        IndexCard      report_card;
        String          message;
        MessageResource message_resource;

        files = new String[args.length-1];
        for (int i = 0; i < files.length; i++) {
            files[i] = args[i+1];
        }
        try {
            operate_report = OperateReport.valueOf(files);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        message_resource = new MessageResource();
        try {
            operate_report.setMessageResource(message_resource);
            operate_report.setDebug( DEBUG_DEBUG );
            operate_report.setLogPrintStream( 
                new LogPrintStream(System.err) );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        if (operate_report == null) {
            message = message_resource.getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                message_resource.getMessage(
                    INTERNAL_ERROR_EXCEPTION));
            System.err.println( message );
            return;
        }

        try {
            articles = operate_report.getArticles(args[0]);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            report_card = operate_report.getReport(articles);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            operate_report.printReport(report_card, articles);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
