/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Region.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2011/05/17
 *
 * This class treads with plain text for extracting specific region.
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class Region extends Object {

    private String tag_string;
    private String key_string = null;

    private int    start_position;
    private int    end_position;

    private int    start_region;
    private int    end_region;

    private int    option_space_counter;

    private  int    local_key_start = 0;
    private  int    local_key_end = 0;

    private int    tag_start;
    private int    tag_end;

    public static String TAB_STRING   = "    ";
    public static String COMMA        = ",";
    public static String SLASH_KEY    = "/";
    public static String TAG_START    = "<";
    public static String TAG_END      = ">";
    public static char   TAG_START_CHAR = '<';
    public static char   TAG_END_CHAR = '>';

    public static char SPACE_CHAR = ' ';
    public static char EQUAL_CHAR = '=';
    public static char SINGLE_QUOTE = '\'';
    public static char DOUBLE_QUOTE = '\"';
    public static char NEWLINE_CHAR = '\n';
    public static char RETURN_CHAR = 0x0d;

    public static String EQUAL_STRING = "=";
    public static String SPACE_STRING = " ";

    public static String WILD_CARD_STRING = "<*";

    public static String KEY_ERROR = "Key error";
    public static String PARAMETER_ERROR = "Parameter error";

    public static String END_OF_FILE_EXCEPTION = "EOF Exception";
    public static String END_OF_REGION_EXCEPTION = "EOR Exception";

    public static String NAME_KEY = "name";
    public static int BUFFER_SIZE = 4096;
    private static String tag_option_string = null;

    public static String LANG_KEY = "LANG";
    public static String JAPANESE_PREFIX = "ja_JP";
    public final static String JAPANESE_CHARSET = "UTF-8";

    public static char DOS_DIRECTORY_SEPARATOR_CHAR = '\\';

    public static int UNIT_SIZE = 1024;

    public static String BUFFER_REQUEST = "Buffer Request";

    public static String FINDREGION_METHOD = "findRegion";
    public static String EXTRACTREGION_METHOD = "extractRegion";


    private static int debug = 0;
    private static PrintStream print_stream = null;

    public static void setDebug(int level) {
        debug = level;
    }

    public static void setDebug(int level, PrintStream output_stream) {
        debug = level;
        print_stream = output_stream;
    }

    public Region() {
    }

    public Region(Region object) {
        tag_string = object.getTagString();
	key_string = object.getKeyString();
        start_position = object.getStartPosition();
        end_position = object.getEndPosition();
        start_region = object.getStartRegion();
        end_region = object.getEndRegion();
        option_space_counter = object.getOptionSpaceCount();
        local_key_start = object.getKeyStartPosition();
        local_key_end = object.getKeyEndPosition();
        tag_start = object.getTagOptionStart();
        tag_end = object.getTagOptionEnd();
    }

    public void setTagOptionKey(String key) {
        tag_option_string = key;
    }

    public String getTagOptionKey() {
        return tag_option_string;
    }

    public void setTagString(String tag) {
        tag_string = tag;
    }

    public String getTagString() {
        return tag_string;
    }

    public void setStartRegion(int position) {
        start_region = position;
    }

    public int getStartRegion() {
        return start_region;
    }

    public void setEndRegion(int position) {
        end_region = position;
    }

    public int getEndRegion() {
        return end_region;
    }

    public void setStartPosition(int position) {
        start_position = position;
    }

    public int getStartPosition() {
        return start_position;
    }

    public void setEndPosition(int position) {
        end_position = position;
    }

    public int getEndPosition() {
        return end_position;
    }

    public void setKeyStartPosition(int start) {
        local_key_start = start;
    }

    public int getKeyStartPosition() {
        return local_key_start;
    }

    public void setKeyEndPosition(int end) {
        local_key_end = end;
    }

    public int getKeyEndPosition() {
        return local_key_end;
    }

    public void resetOptionSpaceCount() {
        option_space_counter = -1;
        return;
    }

    public int getOptionSpaceCount() {
        return option_space_counter;
    }

    public static boolean checkTag(String tag) {
        if (tag.charAt(0) != TAG_START_CHAR || 
            tag.charAt(tag.length()-1) != TAG_END_CHAR) {
            return false;
        }
        return true;
    }

    public int getTagOptionStart() {
        return tag_start;
    }

    public void setTagOptionStart(int start) {
        tag_start = start;
    }

    public int getTagOptionEnd() {
        return tag_end;
    }

    public void setTagOptionEnd(int end) {
        tag_end = end;
    }

    public void setKeyString(byte[][] head, int start, int end) {
        byte[]  buffer;
        String  buffer_string;

        key_string = null;
        if (start < end) {
            try {
                buffer = extractRegion(head, start, end);
                key_string = new String(buffer);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
            }
        }
    }

    public String getKeyString() {
        byte[]  buffer;
        String  buffer_string;

        if (key_string == null) {
            return null;
        }
        return key_string;
    }

    public static boolean checkOmitChar(byte ch, boolean value_flag, 
                boolean equal_flag) { 
        if (ch == SPACE_CHAR) {
            if (value_flag == true) {
                if (equal_flag == false) {
                    return true;
                } else {
                    value_flag = false;
                    equal_flag = false;
                }
            } else {
                if (equal_flag == true) {
                    return true;
                }
            }
        } else if (ch == EQUAL_CHAR) {
            value_flag = false;
            equal_flag = true;
        } else {
            value_flag = true;    
        }
        return false;
    }

    private static boolean isSingleQuote(byte s) {
        if (s == SINGLE_QUOTE) { return true; }
        return false;
    }

    private static boolean isDoubleQuote(byte s) {
        if (s == DOUBLE_QUOTE) { return true; }
        return false;
    }

    public static byte[] extractRegion(byte[][] area,
        int start_position, int end_position) throws Exception {

        int    start;
        int    end;
        int    offset;
        long   block_size;
        byte[] target_area;
        int    start_block;
        int    start_offset;
        int    end_block;
        int    end_offset;
        byte[] array;

        if (area == null) {
            Exception exception_object;
            exception_object = new Exception(PARAMETER_ERROR);
            throw exception_object;
        }
        array = area[0];
        block_size = array.length;

        if (start_position >= end_position ||
            start_position < 0 || end_position <= 0 ||
            block_size <= 0 || area == null) {
            Exception exception_object;
            exception_object = new Exception(PARAMETER_ERROR);
            throw exception_object;
        }

        start_block  = start_position / (int)block_size;
        start_offset = start_position % (int)block_size;
        end_block    = end_position / (int)block_size;
        end_offset   = end_position % (int)block_size;

        if (area.length <= end_block) {
            Exception exception_object;
            exception_object = new Exception(PARAMETER_ERROR);
            throw exception_object;
        }

        target_area = new byte[
            end_offset + end_block * (int)block_size -
            (start_offset + start_block * (int)block_size) +1];

        int k = 0;
        for (int blk = start_block; blk <= end_block; blk++) {
            array = (byte[])area[blk];
            if (blk == start_block) {
                start = start_offset;
            } else {
                start = 0;
            }
            if (blk == end_block) {
                end = end_offset;
            } else {
                end = (int)block_size-1;
            }
            for (offset = start; offset <= end; offset++) {
                target_area[k++] = array[offset];
            }
        }
        return target_area;
    }


    public static int setRepeatCount(byte[] tag_start, int[] repeat_area) 
    throws Exception {
        int repeat_start_local;

        if (tag_start == null || repeat_area == null || 
                        tag_start.length != repeat_area.length) {
            Exception exception_object;
            exception_object = new Exception(PARAMETER_ERROR);
            throw exception_object;
        }

        repeat_start_local = -1;
        for (int i = 1; i < tag_start.length; i++) {
            if (tag_start[0] == tag_start[i]) {
                for (repeat_area[i] = 1; 
                     i+repeat_area[i] < tag_start.length; 
                     repeat_area[i]++) {
                    if(tag_start[repeat_area[i]]
                        != tag_start[i+repeat_area[i]]){
                        break;
                    }
                }
                if (repeat_area[i] > 1) {
                    repeat_start_local = i;
                } else {
                    repeat_area[i] = 0;
                }
            } else {
                repeat_area[i] = 0;
            }
        }
        return repeat_start_local;
    }

    public static String encode(byte[] byte_array, int limit) {
        String  encode;
        encode = new String();
        for (int j = 0; j < limit; j++) {
            if (j != 0) {
                encode = encode.concat(SPACE_STRING);
            }
            encode = encode.concat(Byte.toString(byte_array[j]));
        }
        return encode;
    }


    public static Region findRegion(byte[][][] head_area, 
        int start_position, int end_position, String tag, String key,
        int debug_level, PrintStream output
        )
        throws Exception {
        setDebug(debug_level, output);
        return findRegion(head_area, start_position, end_position,
               tag, key);
    }

    /**
     * Region region = new Region();
     * head = region.setRegion();
     */
    public static Region findRegion(byte[][][] head_area, 
        int start_position, int end_position, String tag, String key)
        throws Exception {

        Region target_region;
        int    current;
        int    block_size;
        int    start_count;
        int    end_count;
        int    start_block;
        int    start_offset;
        int    end_block;
        int    end_offset;

        int    tstart_block;
        int    tend_block;
        int    tstart_offset;
        int    tend_offset;
        int    tstart;
        int    tend;
        int    tblk;

        int    remain_start;
        int    remain_end;
        int    start;
        int    end;
        int    offset;
        int    option_space_counter_local;

        int    idx;
        int    record;

        int    key_value_index;
        String key_value;
        byte[] key_value_upper;
        byte[] key_value_lower;
        int    key_value_start;
        int    key_value_end;
        byte[] key_upper;
        byte[] key_lower;
        int    key_remain_start;
        int    key_start_state;

        int    tag_option_end_local;
        int    tag_option_start_local;
        int    tag_option_key_end;
        int    repeat_start;
        int    repeat_end;
        int[]  repeat_start_count;
        int[]  repeat_end_count;

        byte[] start_upper;
        byte[] start_lower;
        byte[] end_upper;
        byte[] end_lower;
        String end_tag;

        byte[] array;
        byte[] tarray;
        byte[] tag_array;
        byte[] return_area;
        byte[][] area;
        boolean remain_flag;
        int    key_start_position = 0;
        int    key_end_position = 0;

        int number_match;

        area = head_area[0];
        if (area == null) {
            area = new byte[1][];
            head_area[0] = area;
            area[0] = new byte[BUFFER_SIZE];
        }
        key_start_position = 0;
        key_end_position = 0;

        remain_flag = true;
        if (start_position == 0 && end_position == 0) {
            block_size = BUFFER_SIZE;
            start_block = 0;
            start_offset = 0;
            end_block = area.length - 1;
            end_offset = BUFFER_SIZE - 1;
        } else {
            array = area[0];
            block_size = array.length;
            start_block  = start_position / (int)block_size;
            start_offset = start_position % (int)block_size;
            end_block    = end_position / (int)block_size;
            end_offset   = end_position % (int)block_size;
        }
        tag_option_key_end = -1;
        tag_option_end_local = -1;
        tag_option_start_local = -1;
        
        option_space_counter_local = 0;
       
        start_count = -1;
        end_count = -1;

        key_value_index = 0; 
        key_value = null;
        key_value_upper = new byte[0];
        key_value_lower = new byte[0];
        boolean wild_card_string;
        boolean reset_flag ;
        reset_flag = false;
        wild_card_string = false;

        if (tag.indexOf(WILD_CARD_STRING) == 0) {
            wild_card_string = true;
            key_value = null;
            if (tag.charAt(WILD_CARD_STRING.length()) == SPACE_CHAR) {
                key_value = tag.substring(WILD_CARD_STRING.length()+1);
                idx = key_value.toUpperCase().indexOf(key.toUpperCase());
                if (idx >= 0) {

                    key_value = key_value.substring(idx + key.length()).trim();
                    idx = key_value.indexOf(EQUAL_STRING);
                    if (idx >= 0) {
                        key_value = key_value.substring(idx + 1).trim();
                    }

                    idx = key_value.indexOf(SPACE_STRING);
                    if (idx >= 0) {
                        key_value = key_value.substring(0, idx);
                    }
                    idx = key_value.indexOf(TAG_END);
                    if (idx >= 0) {
                        key_value = key_value.substring(0, idx);
                    }

                    key_value_upper = key_value.toUpperCase().getBytes();
                    key_value_lower = key_value.toLowerCase().getBytes();
                } else {
                    Exception exception_object;
                    exception_object = new Exception(KEY_ERROR);
                    throw exception_object;
                }
            }
        }
        if (wild_card_string == false) {
            if (tag.charAt(tag.length()-1) == TAG_END_CHAR) {
                start_upper = tag.substring(0, tag.length()-1).toUpperCase().getBytes();
                start_lower = tag.substring(0,tag.length()-1).toLowerCase().getBytes();
                end_tag = tag.substring(0,1) + SLASH_KEY + tag.substring(1);

                String start_upper_string;
                key_value = null;
                start_upper_string = new String(start_upper);
                idx = start_upper_string.indexOf(SPACE_STRING);
                if (idx > 0){
                    end_tag = tag.substring(0,1) + SLASH_KEY +
                            start_upper_string.substring(1, idx) + TAG_END;

                    String start_tag_string = start_upper_string.substring(0, idx) ;
                    start_upper = start_tag_string.toUpperCase().getBytes();
                    start_lower = start_tag_string.toLowerCase().getBytes();

                    key_value = start_upper_string.substring(idx+1);
                
                    idx = key_value.toUpperCase().indexOf(key.toUpperCase());
                    if (idx >= 0) {
                        key_value = key_value.substring(idx + key.length());
                        idx = key_value.indexOf(EQUAL_STRING);
                        if (idx >= 0) {
                            key_value = key_value.substring(idx + 1).trim();
                        }
                        idx = key_value.indexOf(SPACE_STRING);
                        if (idx >= 0) {
                            key_value = key_value.substring(0, idx);
                        }
                        idx = key_value.indexOf(TAG_END);
                        if (idx >= 0) {
                            key_value = key_value.substring(0, idx);
                        }
                        key_value_upper = key_value.toUpperCase().getBytes();
                        key_value_lower = key_value.toLowerCase().getBytes();
                    } else {
                        Exception exception_object;
                        exception_object = new Exception(KEY_ERROR);
                        throw exception_object;
                    }
                }
            } else {
                start_upper = tag.toUpperCase().getBytes();
                start_lower = tag.toLowerCase().getBytes();
                end_tag = tag.substring(0,1) + SLASH_KEY + tag.substring(1) + TAG_END;
            }
            end_upper = end_tag.toUpperCase().getBytes();
            end_lower = end_tag.toLowerCase().getBytes();
        } else {
            start_upper = new byte[0];
            start_lower = new byte[0];
            end_upper = new byte[0];
            end_lower = new byte[0];
        }

        if (key == null) {
            key_start_state = -1;
            key_upper = null;
            key_lower = null;
            key_remain_start = 0;
        } else {
            key_start_state = 0;
            key_upper = key.toUpperCase().getBytes();
            key_lower = key.toLowerCase().getBytes();
            key_remain_start = key_upper.length;
        }

        target_region = new Region();
        target_region.setTagString(tag);
        target_region.setTagOptionKey(key);

        remain_start = start_upper.length;
        remain_end = end_upper.length;

        repeat_start = -1;
        repeat_start_count = new int[start_upper.length];
        try {
            repeat_start = setRepeatCount(start_upper, repeat_start_count);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        repeat_end = -1;
        repeat_end_count = new int[end_upper.length];
        try {
            repeat_end = setRepeatCount(end_upper, repeat_end_count);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        int start_index;
        int  end_index;

        boolean key_valid_flag = false;
        start_index = repeat_start;
        boolean value_flag = false;
        boolean equal_flag = false;
        boolean over_remain_start = false;
        boolean over_remain_end = false;

        for (int blk = start_block; remain_flag == true ; blk++) {
            if (start_position == end_position && end_position == 0) {
                if (area.length == blk) {
                    byte[][] bk;
                    bk = area;
                    area = new byte[bk.length+1][];
                    for (int i = 0; i< bk.length; i++) {
                        area[i] = bk[i];
                    }
                }
                area[blk] = new byte[BUFFER_SIZE];
            } else {
                if (blk > end_block) {
                    Exception exception_object = new Exception(END_OF_REGION_EXCEPTION);
                    throw exception_object;
                }
            }
            array = (byte[])area[blk];
            if (blk == start_block) {
                start = start_offset;
            } else {
                start = 0;
            }
            if (blk == end_block) {
                end = end_offset;
            } else {
                end = (int)block_size - 1;
            }
            for (offset = start; offset <= end; offset++ ) {
                if (start_position == end_position && end_position == 0) {
                    try {
                        record = System.in.read(array, offset, 1);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                    if (record == -1) {
                        Exception exception_object = new Exception(END_OF_FILE_EXCEPTION);
                        throw exception_object;
                    }
                }
                if (wild_card_string == true) {
                    if (start_count == -1) {
                        if ( array[offset] == TAG_START_CHAR) {
                            start_count = offset+
                                blk*(int)block_size;
                        }
                        continue;
                    } else {
                        if ( tag_option_start_local == -1) {
                            if (array[offset] == TAG_END_CHAR) {
                                tag_option_start_local = offset+
                                        blk*(int)block_size;

                                tag_option_end_local = offset+
                                        blk*(int)block_size;
                                tag_option_key_end = offset+
                                        blk*(int)block_size-1;

                            } else if (array[offset] == SPACE_CHAR) {
                                key_valid_flag = true;
                                tag_option_start_local = offset+
                                        blk*(int)block_size;
                                tag_option_key_end = offset+
                                        blk*(int)block_size-1;
                            } else if (key_valid_flag == true) {
                                tag_option_start_local = offset+
                                        blk*(int)block_size;
                            }
                        }
                        if (array[offset] == SPACE_CHAR || array[offset] == TAG_END_CHAR) {
                            key_start_state = 0;
                            tstart_block  = (start_count+1) / (int)block_size;
                            tstart_offset = (start_count+1) % (int)block_size;
                            tend_block    = (tag_option_key_end) / (int)block_size;
                            tend_offset   = (tag_option_key_end) % (int)block_size;
                            tag_array     = new byte[tag_option_key_end-start_count];
                            int k = 0;
                            for (tblk = tstart_block; tblk <= tend_block; tblk++ ) {
                                tarray = (byte[])area[tblk];
                                if (tblk == tstart_block) {
                                    tstart = tstart_offset;
                                } else {
                                    tstart = 0;
                                }
                                if (tblk == tend_block) {
                                    tend = tend_offset;
                                } else {
                                    tend = (int)block_size - 1;
                                }
                                for (int i = tstart; i <= tend; i++) {
                                    tag_array[k] = tarray[i];
                                    k++;
                                }
                            }

                            String work_string = new String(tag_array);
                            target_region.setTagString(new String(TAG_START + work_string + TAG_END));
                            end_tag = new String(TAG_START + SLASH_KEY + work_string + TAG_END);
                            end_upper = end_tag.toUpperCase().getBytes();
                            end_lower = end_tag.toLowerCase().getBytes();
                            remain_end = end_upper.length;

                            start_upper = (new String(TAG_START+work_string)).toUpperCase().getBytes();
                            start_lower = (new String(TAG_START+work_string)).toLowerCase().getBytes();
                            repeat_end_count = new int[end_upper.length];
                            try {
                                repeat_end = setRepeatCount(end_upper, repeat_end_count);
                            } catch (Exception exception_object) {
                                throw exception_object;
                            }
                            wild_card_string = false;
                        }
                        continue;
                    }
                }

                if (start_count != -1 && tag_option_end_local == -1) {
                    if (checkOmitChar(array[offset], 
                        value_flag, equal_flag) == true) {
                        option_space_counter_local++;
                    }
                }

                if (start_count == -1 && 
                   ((remain_start+offset > array.length && 
                     over_remain_start == true) || 
                     remain_start > array.length)) {
                    if (array[offset]!=
                        start_upper[start_upper.length-remain_start]
                      && array[offset]!=
                        start_lower[start_lower.length-remain_start]){

                        number_match = start_upper.length-remain_start;
                        if (number_match <= 1) {
                            remain_start = start_upper.length;
                            continue;
                        }

                        int  i;
                        for (i = 0; i < number_match; i++) {
                            if (repeat_start_count[i] >= number_match-i) {
                                break;
                            }
                        }
                        if (i == repeat_start_count.length) {
                            remain_start = start_upper.length;
                            continue;
                        }
                        if (array[offset] == start_upper[repeat_start_count[i]]) {
                            remain_start = start_upper.length
                                - repeat_start_count[i]-1;
                        } else {
                            remain_start = start_upper.length;
                        }
                        over_remain_start = false;
                    } else {
                        remain_start--;
                        over_remain_start = true;
                    }
                    continue;
                } else {
                    over_remain_start = false;
                }
                if (start_count != -1 && tag_option_end_local != -1 
                    && end_count == -1 && ((remain_end+offset > array.length 
                    && over_remain_end == true) || remain_end > array.length)) {
                    if (array[offset]!=
                        end_upper[end_upper.length-remain_end]
                      && array[offset]!=
                        end_lower[end_lower.length-remain_end]){

                        number_match = end_upper.length-remain_end;
                        if (number_match <= 1) {
                            remain_end = end_upper.length;
                            continue;
                        }

                        int  i;
                        for (i = 0; i < number_match; i++) {
                            if (repeat_end_count[i] >= number_match-i) {
                                break;
                            }
                        }
                        if (i == repeat_end_count.length) {
                            remain_end = end_upper.length;
                            continue;
                        }
                        if (array[offset] == end_upper[repeat_end_count[i]]) {
                            remain_end = end_upper.length
                                - repeat_end_count[i]-1;
                        } else {
                            remain_end = end_upper.length;
                        }
                        over_remain_end = false;
                    } else {
                        remain_end--;
                        over_remain_end = true;
                    }
                    continue;
                } else {
                    over_remain_end = false;
                }

                if (start_count != -1 && end_count == -1 
                  && tag_option_end_local != -1 && remain_end > array.length) {
                    if (array[offset]!=
                        end_upper[end_upper.length-remain_end]
                      && array[offset]!=
                        end_lower[end_lower.length-remain_end]){
                        number_match = end_upper.length-remain_end;
                        if (number_match <= 1) {
                            remain_end = end_upper.length;
                            continue;
                        }
                        int  i;
                        for (i = 0; i < number_match; i++) {
                            if (repeat_end_count[i] >= number_match-i) {
                                break;
                            }
                        }
                        if (i == repeat_start_count.length) {
                            remain_end = end_upper.length;
                            continue;
                        }
                        if (array[offset] == end_upper[repeat_end_count[i]]) {
                            remain_end = end_upper.length
                                - repeat_end_count[i]-1;
                        } else {
                            remain_end = end_upper.length;
                        }
                    } else {
                        remain_end--;
                    }
                    continue;
                }

                if (start_count == -1 &&  (remain_start - 1 == offset || 
                     start_upper.length <= offset)) {
                    int  i;
                    for (i = 0; i < remain_start; i++) {
                        if (array[offset-i]!=start_upper[start_upper.length-i-1]
                      && array[offset-i]!=start_lower[start_lower.length-i-1]){
                            break;
                        }
                    }
                    if (i == remain_start) {
                        start_count = offset+blk*(int)block_size-start_upper.length+1;

                        tag_option_start_local = offset+blk*(int)block_size;
                        remain_start = start_upper.length;
                        start_index = repeat_start;
                        remain_end = end_upper.length;
                        continue;
                    } else {
                        if (repeat_start != -1) {
                            if (remain_start < start_upper.length) {
                                int j;

                                for (j = repeat_start; 
                                    j < start_upper.length; j++) {
                                    if (repeat_start_count[j] != 0
                                    && start_upper.length-
                                    repeat_start_count[j]<=remain_start+j
                                    && remain_start+j <= start_upper.length) {
                                        remain_start=remain_start+j;
                                        break;
                                    }
                                }
                                if (j == start_upper.length) {
                                    remain_start = start_upper.length;
                                }
                            }
                        } else {
                            remain_start = start_upper.length;
                        }
                    }
                }
                if (start_count != -1 && end_count == -1
                    &&  tag_option_end_local == -1 && array[offset] == TAG_END_CHAR) {
                    tag_option_end_local = offset + blk * (int)block_size;
                    if (key != null) {
                        if (key_start_state == 3) {
                            key_start_state = 0;
                            key_end_position = (int)(offset+blk*block_size-1);
                        } else if (key_start_state == -1) {
                            key_end_position = (int)(offset+blk*block_size-1);
                        }
                    }
                }
                if (start_count != -1 && tag_option_end_local == -1) {
                    if (key != null) {
                        if (key_start_state == 0 && offset >= key_remain_start - 1) {
                            int  i;
                            for (i = 0; i < key_remain_start; i++) {
                                if (array[offset-i]!=key_upper[key_upper.length-i-1]
                                    && array[offset-i]!=key_lower[key_lower.length-i-1]){
                                    break;
                                }
                            }
                            if (i == key_remain_start) {
                                key_start_state = 1;
                            }
                        } else if (key_start_state == 1 && array[offset] == EQUAL_CHAR) {
                            key_start_state = 2;
                        } else if (key_start_state == 2 && array[offset] != SPACE_CHAR 
&& array[offset] != DOUBLE_QUOTE && array[offset] != SINGLE_QUOTE) {
                            key_start_position = (int)(offset + blk * block_size);
                            key_start_state = 3;
                            if (key_value != null && key_value_index < key_value_upper.length) {
                                if (array[offset] != key_value_upper[key_value_index]
                                && array[offset] != key_value_lower[key_value_index]) {
                                    key_start_state = 0;
                                    start_count = -1;
                                    key_start_position = 0;
                                    key_value_index = 0;
                                    tag_option_start_local = -1;
                                    tag_option_end_local = -1;
                                    remain_start = start_upper.length;
                                    continue;
                                } else {
                                    key_value_index++;
                                }        
                            }
                        } else if (key_start_state == 3) {
                            if (array[offset] == SPACE_CHAR || array[offset] == DOUBLE_QUOTE || array[offset] == SINGLE_QUOTE) {
                                key_start_state = 0;
                                key_end_position = (int)(offset+blk*block_size-1);
                            }
                            if (key_value != null && key_value_index < key_value_upper.length) {
                                if (array[offset] != key_value_upper[key_value_index]

                                && array[offset] != key_value_lower[key_value_index]) {
                                    key_start_state = 0;
                                    start_count = -1;
                                    key_start_position = 0;
                                    key_value_index = 0;
                                    tag_option_start_local = -1;
                                    tag_option_end_local = -1;
                                    remain_start = start_upper.length;
                                } else {
                                    key_value_index++;
                                }        
                            }
                        }
                    }
                }
                if (start_count != -1 &&  
                    (remain_end-1 == offset || end_upper.length <= offset)) {
                    int  i;
                    for (i = 0; i < remain_end; i++) {
                        if (array[offset-i]!=end_upper[end_upper.length-i-1]
                          && array[offset-i]!=end_lower[end_lower.length-i-1]){

                            break;
                        }
                    }
                    if (i == remain_end) {
                        end_count = offset+blk*(int)block_size;
                        remain_end = end_upper.length;
                        end_index = repeat_end;
                        break;
                    } else {
                        if (repeat_end != -1) {
                            if (remain_end < end_upper.length) {
                                int j;
                                for (j = repeat_end; 
                                    j < end_upper.length; j++) {
                                    if (repeat_end_count[j] != 0
                                      && end_upper.length-repeat_end_count[j]
                                                  <=remain_end+j
                                      && remain_end+j <= end_upper.length) {
                                        remain_end=remain_end+j;
                                        break;
                                    }
                                }
                                if (j == end_upper.length) {
                                    remain_end = end_upper.length;
                                }
                            }
                        } else {
                            remain_end = end_upper.length;
                        }
                    }
                }
                if (offset == array.length - 1) {
                    if (start_count == -1) {
                        int i;
                        int j;
                        i = 0;
                        for (j = 1; j < start_upper.length; j++) {
                            for (i = 0; i < start_upper.length - j; i++) {
                                if (array[offset-i] != 
                                    start_upper[start_upper.length-j-i-1]
                                && array[offset-i] != 
                                    start_lower[start_lower.length-j-i-1]) {
                                    break;
                                }
                            }
                            if (i == start_upper.length-j) {
                                break;
                            }
                        }
                        if (j < start_upper.length) {
                            remain_start = j;
                        } else {
                            break;
                        }
                    } else {
                        int i;
                        int j;
                        for (j = 1; j < end_upper.length; j++) {
                            for (i = 0; i < end_upper.length - j; i++) {
                                if (array[offset-i] != 
                                    end_upper[end_upper.length-j-i-1]
                                && array[offset-i] != 
                                    end_lower[end_lower.length-j-i-1]) {
                                    break;
                                }
                            }
                            if (i == end_upper.length - j) {
                                break;
                            }
                        }
                        if (j < end_upper.length) {
                            remain_end = j;
                        }
                        if (tag_option_end_local == -1) {
                            if (key != null) {
                                for (j = 1; j < key_upper.length; j++) {
                                    for (i = 0; i < key_upper.length - j; i++) {
                                        if (array[offset-i] != 
                                            key_upper[key_upper.length-j-i-1]
                                            && array[offset-i] != 
                                            key_lower[key_lower.length-j-i-1]) {
                                            break;
                                        }
                                    }
                                    if (i == key_upper.length-j) {
                                        break;
                                    }
                                }
                                if (j < key_upper.length) {
                                    key_remain_start = j;
                                } else {
                                    break;
                                }
                            }
                        }
                    }
                    continue;
                }
            }
            if (end_count != -1) { 
                break; 
            }
        }
        target_region.setStartRegion(start_count);
        target_region.setEndRegion(end_count);

        byte[] ta;
        int array_index_front;
        int array_index_rear;
        {
            array_index_front = tag_option_end_local + 1;
            array_index_rear = end_count-end_upper.length;

            int check_size;
            int check_start;
            
            for (int i = array_index_front/BUFFER_SIZE; 
                                    i <= array_index_rear/BUFFER_SIZE; i++) {
                array = (byte[])area[i];
                if (i == array_index_front/BUFFER_SIZE) {
                    check_start = array_index_front%BUFFER_SIZE;
                    if (i == array_index_rear/BUFFER_SIZE) {
                        check_size = array_index_rear%BUFFER_SIZE;
                    } else {
                        check_size = BUFFER_SIZE-1;
                    }
                } else {
                    check_start = 0;
                    if (i == array_index_rear/BUFFER_SIZE) {
                        check_size = array_index_rear % BUFFER_SIZE;
                    } else {
                        check_size = BUFFER_SIZE-1;
                    }
                }
                int j;
                for (j = check_start; j <= check_size; j++) {
                    if (array[j] != NEWLINE_CHAR && array[j] != RETURN_CHAR) {
                        break;
                    }
                    array_index_front = array_index_front+1;
                }
                if (j <= check_size) {
                    break;
                }
            }
            for (int i = array_index_rear/BUFFER_SIZE; 
                                    i >= array_index_front/BUFFER_SIZE; i--) {
                array = (byte[])area[i];
                if (i == array_index_front/BUFFER_SIZE) {
                    check_start = array_index_front%BUFFER_SIZE;
                    if (i == array_index_rear/BUFFER_SIZE) {
                        check_size = array_index_rear % BUFFER_SIZE;
                    } else {
                        check_size = BUFFER_SIZE - 1;
                    }
                } else {
                    check_start = 0;
                    if (i == array_index_rear/BUFFER_SIZE) {
                        check_size =  array_index_rear % BUFFER_SIZE;
                    } else {
                        check_size = BUFFER_SIZE-1;
                    }
                }
                int j;
                for (j = check_size; j > check_start; j--) {
                    if (array[j] != NEWLINE_CHAR && array[j] != RETURN_CHAR) {
                        break;
                    }
                    array_index_rear = array_index_rear-1;
                }
                if (j <= check_size) {
                    break;
                }
            }
        }

        target_region.setStartPosition(array_index_front);
        target_region.setEndPosition(array_index_rear);

        if (key != null) {
            if (key_start_position >= key_end_position) {
                String start_upper_string = new String(start_upper);
                if (start_upper_string.indexOf(key.toUpperCase()) > 0) {
                    idx = start_upper_string.indexOf(EQUAL_STRING);
                    if (idx > 0) {
                        key_start_position = start_count + idx +1;
                        key_end_position = tag_option_end_local-1;
                    }
                }
            }
            start_block  = key_start_position / (int)block_size;
            start_offset = key_start_position % (int)block_size;
            end_block    = key_end_position / (int)block_size;
            end_offset   = key_end_position % (int)block_size;
            if (
            (isSingleQuote(area[start_block][start_offset]) &&
	     isSingleQuote(area[end_block][end_offset]) )
          ||
            (isDoubleQuote(area[start_block][start_offset]) &&
             isDoubleQuote(area[end_block][end_offset]) )
	    ) {
                if (key_start_position < key_end_position) {
                    key_start_position++;
                    key_end_position--;
                }
            }
            target_region.setKeyStartPosition( key_start_position );
            target_region.setKeyEndPosition( key_end_position );
            target_region.setKeyString(area, key_start_position, key_end_position);
        }
/**
 *        V-- tag_option_start_local
 *    <XXX>
 *        ^-- tag_option_end_local
 */
        target_region.setTagOptionStart(tag_option_start_local);
        target_region.setTagOptionEnd(tag_option_end_local);

        return target_region;
    }

    public static String TAB_SPACE = "    ";
    public static String ILLEGAL_ARGUMENT = "Illegal Argument";
    public static String BAD_INPUT_FILE = "Bad Input File";
/**
 * TAG
 */
    public static int TERM_UNIT = 1024;

/*
 * java -classpath ./ Region <tag> input_file 
 *
 */
/*
 * java -classpath ./ Region "<DIVE>,<CUSTOMER>" Takahiko.20110101 
 */

    public static void main(String[] ag) throws Exception {
        byte[]        byte_area;
        InputStream   input_stream;
        InputStream   system_input;
        Region  region_object = new Region();
        String           lang;

        ByteArrayOutputStream   baos;
        PrintStream             pstr;

        lang = System.getenv(LANG_KEY);
        system_input = System.in;
        baos = new ByteArrayOutputStream();
        if ( lang != null && lang.indexOf(JAPANESE_PREFIX) >= 0) {
            try {
                pstr = new PrintStream(baos, true, JAPANESE_CHARSET);
            } catch (Exception exception_object) {
                System.err.println(exception_object.getMessage());
                return;
            }
        } else {
            pstr = new PrintStream(baos);
        }

        byte[] byte_array;
        String encode;
        for (int i = 0; i < ag.length; i++) {
            byte_array = ag[i].getBytes();
            encode = encode(byte_array, byte_array.length);
            pstr.println(encode);
            /**
             * for EncodeStream
            print_stream.println(org_args[i]);
             **/
        }
        byte_area = baos.toByteArray();
        input_stream = (InputStream)new ByteArrayInputStream(byte_area);
        System.setIn(input_stream);

        try {
            region_object.execute();
        } catch (Exception exception_object) {
            if (exception_object.getMessage().compareTo(END_OF_FILE_EXCEPTION)
                != 0) {
                System.setIn(system_input);
                System.out.println(exception_object.getMessage());
                System.exit (1);
            }
        }
        System.setIn(system_input);
    }

    public static void execute() throws Exception {
        File             fdes;
        int              record;
        FileInputStream  fis;
        DataInputStream  data_input_stream;
        byte[]           area;
        byte[][]         head = new byte[0][];
        byte[][][]       head_area = new byte[1][][];
        String[]         tag_list;
        int              tag_count;
        int              term_index;
        String           tag;
        StringTokenizer  string_token;
        int              tag_index;
        int              find_start;
        int              find_end;
        boolean[]        block_end;
        String[]         ag = new String[0];
        String[]            bk;
        String           line;
        String           lang;

        DataInputStream dis;

        lang = System.getenv(LANG_KEY);
        data_input_stream = new DataInputStream(System.in);

        while (true) {
            try {
                line = data_input_stream.readLine();
                if (line == null) { break; }
                bk = new String[ag.length+1];
                for (int i = 0; i < ag.length; i++) {
                    bk[i] = ag[i];
                }
                bk[bk.length-1] = line;
                ag = bk;
            } catch (Exception exception_object) {
                break;
            } 
        }
        if (ag.length != 2) {
            Exception exception_object;
            exception_object = new Exception(ILLEGAL_ARGUMENT);
            throw exception_object;    
        }

        Region  region_object;
        Region  region_object_body;
        Region  region_object1;
        Region  region_object1_body;

        Region[] region_block;

        term_index = 0;
        try {
            fdes = new File(ag[1]);
            fdes = fdes.getAbsoluteFile();
            if (fdes.exists() == false) {
                System.out.println(BAD_INPUT_FILE);
                return;
            }
            fis = new FileInputStream(fdes);
        } catch (Exception exception_object) {
            System.out.println(BAD_INPUT_FILE);
            return;
        }
        InputStream save_in;
        save_in = System.in;
        System.setIn(fis);

        tag = ag[0];
        string_token = new StringTokenizer(tag, COMMA);
        tag_count = 0;
        tag_list = new String[string_token.countTokens()];
        while (string_token.hasMoreTokens()) {
            tag_list[tag_count] = string_token.nextToken().trim();
            tag_count = tag_count + 1;
        }
        region_block = new Region[tag_list.length];
        

        record = -1;
        head = new byte[((int)fdes.length() + BUFFER_SIZE - 1) / BUFFER_SIZE][];
        head_area[0] = head;

        byte[] buffer;
        String buffer_string;
        int first_start;
        int first_end;

        while (true) {
            find_start = 0;
            find_end = 0;
            for (tag_index = 0; tag_index < tag_list.length; tag_index++) {
                try {
                    region_block[tag_index]=Region.findRegion(
                            head_area, find_start, find_end, 
                            tag_list[tag_index], NAME_KEY);
                } catch (Exception exception_object) {
                    if (tag_index == 0) {
                        return;
                    }
                    if (tag_index == 1) {
                        break;
                    }
                    find_start = region_block[tag_index-1].getEndPosition();
                    find_end = region_block[tag_index-2].getEndPosition(); 
                    tag_index = tag_index - 2;
           
                    continue;
                }
                if (region_block[tag_index] == null) {
                    break;
                }
                if (region_block[tag_index].getStartPosition() <
                    region_block[tag_index].getEndPosition() ) {
                    if (tag_list.length -1 == tag_index) {
                        try {
                            buffer = extractRegion(head, 
                                region_block[tag_index].getStartPosition(),
                                region_block[tag_index].getEndPosition()
                            );
                            buffer_string = new String(buffer);
                            System.out.println(buffer_string);
                        } catch (Exception exception_object) {
                            throw exception_object;
                        }
                        if(tag_list.length > 1) {
                            find_start=region_block[tag_index].getEndPosition();
                            find_end=region_block[tag_index-1].getEndPosition(); 
                            tag_index = tag_index -1;
                        }
                    } else {
                        find_start = region_block[tag_index].getStartPosition();
                        find_end = region_block[tag_index].getEndPosition(); 
                    }
                }
            }
        }
    }
}
