/**
 * Copyright (C)  All rights reserved.
 *
 * @Module     RegionObject.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2011/09/05
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class RegionObject extends DebugObject {

    private static String FILE_PROCEDURE = "File procedure";

    /** Search Region **/

    /**
     **  Error definition
     **/

    public RegionObject() {
    }

    public static String encode(byte[] byte_array, int limit) {
        String  encode;

        encode = new String();
        if (byte_array == null || limit < 0 || limit >= byte_array.length) {
            return encode; 
        }

        for (int j = 0; j < limit; j++) {
            if (j != 0) {
                encode = encode.concat(SPACE_STRING);
            }
            encode = encode.concat(Byte.toString(byte_array[j]));
        }
        return encode;
    }

    public static String deleteKey(String tag) {
        int  space_index;
        String tag_label;

        if (tag == null) {
            return tag;
        }
        space_index = tag.indexOf(SPACE);
        if (space_index >= 0) {
            tag_label = tag.substring(0, space_index) + TAG_END_STRING;
        } else {
            tag_label = tag;
        }
        return tag_label;
    }

    public static String convertFromStartToEnd(String tag_string)
    {
        int end_index;
        String work_string;

        if (tag_string == null) {
            return tag_string;
        }
        work_string = deleteKey(tag_string);

        end_index = work_string.indexOf(TAG_START_STRING);
        if (end_index >= 0) {
            work_string = new String(
                TAG_START_STRING + TAG_END_MARK_STRING +
                work_string.substring(end_index+1));
        }
        return work_string;
    }

    public static String deleteCarriageReturnEnd(String line) {
        String message;

        if (line == null || line.length() == 0) {
            return line;
        }
        message = line;
        if (line.length() >= 1 && line.charAt(0) == CODE_NEWLINE) {
            if (line.length() >= 2 && line.charAt(1) == CODE_RETURN) {
                message = line.substring(2);
            } else {
                message = line.substring(1);
            }
        } else if (line.length() >= 1 && line.charAt(0) == CODE_RETURN) {
            if (line.length() >= 2 && line.charAt(1) == CODE_NEWLINE) {
                message = line.substring(2);
            } else {
                message = line.substring(1);
            }
        }
        return message;
    }

    public static boolean isCarriageReturnEnd(String line) {
        if (line == null || line.length() == 0) {
            return false;
        }
        if (line.charAt(line.length() -1) != CODE_NEWLINE &&
            line.charAt(line.length() -1) != CODE_RETURN) {
            return false;
        }
        return true;
    }

    private static boolean insert_tag_option = false;
    public static void enableInsertTag() {
        insert_tag_option = true;
    }

    public static void disableInsertTag() {
        insert_tag_option = false;
    }

    public static boolean validateInsertTag() {
        return insert_tag_option;
    }

    public boolean checkInsertTag(String tag) throws Exception {
        if (tag == null) {
            String      message;
            Exception   exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);

            exception_object = new Exception( message );
            throw exception_object;
        }
        if (validateInsertTag() == false) {
            return false;
        }
        if (tag.charAt(0) != START_BRACKET_CHAR ||
            tag.charAt(tag.length()-1) != END_BRACKET_CHAR) {
            return false;
        }
        return true;
    }

    public  boolean checkTags(String tags) throws Exception {
        int             tag_index;
        boolean         flag;
        String[]        tag_list;
        StringTokenizer stk;
        KeyList         key_list;
        String          key_list_string;
        String          message;

        if (tags == null) {
            Exception   exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);

            exception_object = new Exception( message );
            throw exception_object;
        }
        flag = true;

        try {
            key_list = KeyListFactory.createKeyList(tags, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            throw exception_object;
        }
        for (int i = 0; i < key_list.length(); i++) {
            try {
                key_list_string = key_list.getKeyEntry(i);
                if (Region.checkTag(key_list_string) == false) {
                    if (checkInsertTag(key_list_string) == false) {
                        return false;
                    }
                }
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return flag ;
    }

    public String getCommonPath(String file_name) {
        File    fd;
        String  path;

        fd = new File(EMPTY);
        path = fd.getAbsolutePath();
        path = new String(path + File.separator + file_name);
        return path;
    }

    public void execute(String[] org_args, PrintStream print_stream, MakeReport instance) throws Exception {
        Object[]  object_array;
        Object[]   bk;
        File       fd;
        boolean    check_tag;
        String     message;
        String     message_log;
        String     path;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                EXECUTE_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (org_args.length < 2) {
            Exception  exception_object;
            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        object_array = new Object[1];
        object_array[0] = org_args[0];

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                SPACE, 
                CHECKTAGS_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        check_tag = false;
        try {
            check_tag = checkTags(org_args[0]);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (check_tag == false) {
            Exception  exception_object;

            message = getMessage( TAGKEYS_PARAMETER_ERROR,
                TAGKEYS_PARAMETER_ERROR);
            exception_object = new Exception( message );
            throw exception_object; 
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(DEBUG_TRACE_KEY, DEBUG_TRACE_KEY,
                SPACE, 
                FILE_PROCEDURE);
            getLogPrintStream().printLogln(message_log);
        }

        for (int j = 1; j < org_args.length; j++) {
            try {
                bk = new Object[object_array.length+1];
                for (int i = 0; i < object_array.length; i++) {
                    bk[i] = object_array[i];
                }
                path = org_args[j];
                fd = new File(path);
                if (fd.exists() == false) {
                    Exception  exception_object;
                    message = getMessage(FILE_NOT_FOUND_EXCEPTION, 
                        FILE_NOT_FOUND_EXCEPTION,
                        SPACE, 
                        org_args[j]);
                    exception_object = new Exception( message );
                    throw exception_object;
                }
                bk[bk.length-1] = (Object)new FileInputStream(fd);
                object_array = bk;
            } catch (Exception exception_object) {
                throw exception_object;
            } 
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                SPACE, 
                MakeReport.MAKEREPORT_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        try {
            instance.makeReport(object_array, print_stream);
        } catch (Exception exception_object) {
            System.err.println(exception_object.toString());
            return;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(END_METHOD, END_METHOD, SPACE,
                EXECUTE_METHOD);
            getLogPrintStream().printLogln( message_log );
        }
    }

    public static void main(String[]  args) {
        String         message;
        RegionObject   region_object;

        region_object = new RegionObject();
        region_object.setDebug(System.getProperty(EXEC_REGION_DEBUG));
        if (region_object.getDebug() > 0) {
            message = region_object.getMessage(
                DEBUG_DEBUG_CONDITION_KEY, DEBUG_DEBUG_CONDITION_KEY,
                SPACE,
                region_object.getMessageResource().getMessage(
                    DEBUG_ENABLE_KEY),
                region_object.getMessageResource().getMessage(
                    DEBUG_CODE_003),
                region_object.getMessageResource().getMessage(
                    DEBUG_CODE_004));
            System.err.println(message);
        }
    }
}
