/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    ReportRegion.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version:   1.0 2011/04/05 
 *
 * This module creates new report combining blocks in files.
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * class represents text region blocks.
 */
public class ReportRegion extends RegionObject implements MakeReport {

    /**
     * definition for increment size
     */
    public   static   int      TAG_TABLE_UNIT = 16;
    private  static   int      REGION_UNIT = 128;

    private  static   int  debug_flag = 0;

    private static int BUFFER_SIZE = 4096;
    public static String BUFFER_REQUEST = "Buffer Request";
    public static String DEBUG_STRING = "Debug";
    public static String NULL_MESSAGE = "Null";

    private InputStream[]   istream = null;
    private ReportRegion[]  merge_object = null;

    private String VIOLATE_UNUSED_REGION_RANGE = "violate unused region range";

    /**
     * initialized variable for regions with unused tag
     */
    private  Region[][] unused_tag_region_list = new Region[0][];
    private  int[]      unused_tag_region_index = new int[0];
    private  int[][]    unused_tag_region_reference = new int[0][];
    private  int[][]    unused_tag_region_start = new int[0][];
    private  String[][] unused_tag_region_string = new String[0][];
    private  int        unused_tag_region_list_index = 0;

    private  Region[]   bk_current_tag_region_list = new Region[0];
    private  int        bk_current_tag_region_index;
    private  int[]      bk_current_tag_region_reference = new int[0];
    private  int[]      bk_current_tag_region_start = new int[0];
    private  String[]   bk_current_tag_region_string = new String[0];

    private  Region[]   tag_region = new Region[0];
    private  int        tag_region_index;
    private  int[]      tag_region_reference = new int[0];
    private  int[]      tag_region_start = new int[0];
    private  String[]   tag_region_string = new String[0];

    private  int        copy_tag_region_index;

    private  Region[]   read_region_box = new Region[0];
    private  int        read_region_index = 0;

    /**
     * current region object
     */
    private  Region[]   current_region_object = new Region[0];
    private  Region[]   bkup_region_object = null;

    private  Region[][] unknown_region = new Region[0][];
    private  byte[][][] unknown_head = new byte[0][][];
    private  int        unknown_region_index = 0;
    private  int        current_region_list_index = 0;

    private  String   format_keys = null;

    public static String USAGE_STRING    
        = "Usage : ReportRegion tag [files | term prefix]";

    public static String NOT_FOUND_ENTRY_EXCEPTION = "Not Found Entry Exception";
    /**
     * definition for error
     */
    public static String ERROR_SEPARATOR = ":";

    public static String INTERNAL_ERROR  = "Internal error";
    public static String FORMAT_ERROR    = "Format error";
    public static String PARAMETER_ERROR = "Parameter error";
    public static String FILE_ERROR      = "File error";
    public static String SYSTEM_IO_ERROR = "System I/O error";
    
    public static String PARAMETER_ERROR_MESSAGE1 
                                         = "Parameter not found.";
    public static String PARAMETER_ERROR_MESSAGE2
                                         = "Tag parameter is illegal.";
    public static String FILE_ERROR_MESSAGE1
                                         = "File not found";

    public static String INPUT_FILE_ERROR= "Input file error";

    public static String FILE_STREAM = "File stream";

    public static String EMPTY_INPUT_ERROR_MESSAGE
        = "Input file list is empty.";
    public static String INVALID_FILE_ERROR_MESSAGE2
        = " don't exists().";
    public static String INVALID_FILE_ERROR_MESSAGE 
        = " does not exists().";
    public static String INPUT_FILE_ERROR_MESSAGE
        = "Input file is abnormal.";
    public static String PROMPT_REPLACE_INPUT_MESSAGE
        = "Please specify input files again.";
    public static String PROMPT_INPUT_FILE_MESSAGE
        = "Please specify input files.";
    public static String TAGKEYS_PARAMETER_ERROR_MESSAGE2
        = "Tagkeys is null in first parameter.";

    public static String EMPTY_PARAMETER_ERROR_MESSAGE1
        = "Parameter list is empty. Please specify parameter.";
    public static String TAG_PARAMETER_ERROR_MESSAGE1
        = " is illegal format.";

    public static String GET_FORMATTED_TAGS = "getFormattedTags()";
    public static String GET_FORMATTED_TAGS_FIRST_PARAMETER_IS_NULL 
        = "Key parameter is null in getFormattedTags().";
    public static String GET_FORMATTED_TAGS_SECOND_PARAMETER_IS_NULL 
        = "Format parameter is null in getFormattedTags().";
    public static String GET_FORMATTED_TAGS_UNMATCHED_FORMAT
        = "Unmatched format in getFormattedTags().";
    public static String LIST_FORMAT_IS_ILLEGAL
        = "Tag format is illegal.";

    private static String DEBUG_TAG = "DEBUG Tag";

    public final static String CLEARREGIONOBJECT_METHOD = "clearRegionObject";
    private static String REPORT_METHOD = "report";
    private static String EXECUTE_METHOD = "execute";
    private static String CHECKREGIONLIST_METHOD = "CheckRegionList";
    public final static String READREGIONLIST_METHOD = "readRegionList";
    private static String GETREGIONLIST_METHOD = "getRegionList";
    private static String ADDREGIONLIST_METHOD = "addRegionList";
    private static String NEXTREGIONLIST_METHOD = "nextRegionList";
    private static String NEXTCONTENT_METHOD = "nextContent";
    private static String GETMESSAGEFROMREGIONLIST_METHOD="getMessageFromRegionList";
    private static String PRINTSTARTTAGS_METHOD = "printStartTags";
    private static String PRINTENDTAGS_METHOD = "printEndTags";
    private static String SAVEUNUSEDREGIONLIST_METHOD = "saveUnusedRegionList";
    private static String RESTOREUNUSEDREGIONLIST_METHOD = "restoreUnusedRegionList";
    private static String DELETEUNUSEDREGIONLIST_METHOD = "deleteUnusedRegionList";
    private static String MOREUNKNOWNREGIONLISTS_METHOD = "moreUnknownRegionLists";
    private static String GETKEYNUMBER_METHOD = "getKeyNumber";

    public static String BAD_FORMAT_ERROR_TAIL = " is bad format.";

    public static String ASTARISC = "*";
    public static String WILDCARD = "<*>";
    public static String NAME_KEY = "name";
    public static String PERSON_TAG = "Person";
    public static String FILE_TAG = "File";
    public static String DATE_TAG = "Date";

    private String lang = null;
/*
    public static String LANG_KEY = "LANG";
    public static String JAPANESE_PREFIX = "ja_JP";
    public final static String JAPANESE_CHARSET = "UTF-8";
*/
    private static String[] test = {"<DIVE>,<CUSTOMER>", "out.20110101"};

    public static String READ_REGION_LIST_ERROR = "readRegionList error";
    public static String REGION_BUFFER_ERROR = "Region buffer error";

    public static String NOT_FOUND_CURRENT_REGION_OBJECT = "Not found current region object";

    public static String REPORT_REGION_PARAMETER_EXCEPTION
        = "Parameter exception in ReportRegion.";
    public static String REPORT_REGION_ILLEGAL_DEBUG_VALUE
        = "Illegal debug value in ReportRegion.";

    public static long   DATE_TIME = 86400;
    public static long   MONTH_YEAR = 12;
    public static String UNMATCH = "Unmatch tags";

    private  byte[][][]  head ;
    private  byte[][]    bkup_head;

    public static String REPLACEUNUSEDREGION_METHOD = "replaceUnusedRegion";
    public static String DELETEUNUSEDREGION_METHOD = "deleteUnusedRegion";
    public static String CHECKUNUSEDREGION_METHOD = "checkUnusedRegion";
    public static String ADDUNUSEDREGION_METHOD = "addUnusedRegion";
    public static String GETUNUSEDREGION_METHOD = "getUnusedRegion";
    public static String SEARCHUNUSEDREGIONLIST_METHOD = "searchUnusedRegionList";
    public ReportRegion() {
        super();
    }


    /**
     * A region list is held when option name doesn't match with 
     * that of region that is read from first file.
     */

    /**
     * This method deletes current unknown region list from
     * the unknown region list table.
     *
     */
    private void deleteRegionList() 
    {
        Region[] local_region_list;
        int i;
        int j;
        Region   local_region;
        String   local_region_tag;
        String   current_region_tag;
        
        if (checkCurrentRegionObject() == false 
                    || getCurrentRegionObjectLength() == 0) {
            return;
        }
        for (i = 0; i < unknown_region_index; i++) {
            local_region_list = unknown_region[i];
            if (getCurrentRegionObjectLength() != local_region_list.length) {
                continue;
            }
            for (j = 0; j < getCurrentRegionObjectLength(); j++) {
                local_region = getCurrentRegionObjectEntry(j);
                current_region_tag = getTagStringWithOption(local_region);
                local_region_tag = getTagStringWithOption(local_region_list[j]);
                if (current_region_tag.compareTo(
			local_region_tag ) != 0) {
                    break;
                }
	    }
            if (j < getCurrentRegionObjectLength()) {
                continue;
            }
            for (j = i; j < unknown_region_index-1; j++) {
                unknown_region[j] = unknown_region[j+1];
                unknown_head[j] = unknown_head[j+1];
            }
            break;
        }
        unknown_region_index--;
        clearCurrentRegionObject();
        return;
    }

    private void addRegionList() 
    {
        Region[] region_new_list;

        if (checkCurrentRegionObject() == false || 
                getCurrentRegionObjectLength() == 0) {
            return;
        }
        if (unknown_region_index <= unknown_region.length) {
            Region[][] bk;
            byte[][][] h_bk;
            bk = new Region[unknown_region.length+REGION_UNIT][];
            h_bk = new byte[unknown_region.length+REGION_UNIT][][];
            for (int i = 0; i < unknown_region.length; i++) {
                bk[i] = unknown_region[i];
                h_bk[i] = unknown_head[i];
            }
            unknown_region = bk;
            unknown_head = h_bk;
        }
        region_new_list = new Region[getCurrentRegionObjectLength()];
        for (int i = 0; i < getCurrentRegionObjectLength(); i++) {
            region_new_list[i] = new Region(current_region_object[i]);
        }
        unknown_region[unknown_region_index] = region_new_list;
        unknown_head[unknown_region_index] = head[0];

        unknown_region_index++;
    }

    private void deleteUnusedRegionList(String tag_string, int index) 
    {
        int    descriptor;
        if (tag_string == null) {
            return;
        }
        descriptor = searchUnusedRegionList(tag_string, index);
        if (descriptor >= unused_tag_region_list_index || descriptor < 0) {
            return;
        }

        for (int i = descriptor; i < unused_tag_region_list_index -1; i++) {
            unused_tag_region_list[i] = unused_tag_region_list[i+1];
            unused_tag_region_string[i] = unused_tag_region_string[i+1];
            unused_tag_region_reference[i] = unused_tag_region_reference[i+1];
            unused_tag_region_start[i] = unused_tag_region_start[i+1];
            unused_tag_region_index[i] = unused_tag_region_index[i+1];
        }
        --unused_tag_region_list_index;
    }

    private boolean saveCurrentUnusedRegionList() 
    {
        bk_current_tag_region_list = tag_region;
        bk_current_tag_region_index = tag_region_index;
        bk_current_tag_region_reference = tag_region_reference;
        bk_current_tag_region_start = tag_region_start;
        bk_current_tag_region_string = tag_region_string;
        return true;
    }

    private boolean restoreCurrentUnusedRegionList()
    {
        tag_region = bk_current_tag_region_list;
        tag_region_index = bk_current_tag_region_index;
        tag_region_reference = bk_current_tag_region_reference;
        tag_region_start = bk_current_tag_region_start;
        tag_region_string = bk_current_tag_region_string;
        return true;
    } 

    private void saveUnusedRegionList(String tag_strings) 
    {
        Region[][]   bk_unused_tag_region_list;
        int[]        bk_unused_tag_region_index;
        int[][]      bk_unused_tag_region_reference;
        int[][]      bk_unused_tag_region_start;
        String[][]   bk_unused_tag_region_string;
        Region[]     new_unused_tag_region_list;
        int          new_unused_tag_region_index;
        int[]        new_unused_tag_region_reference;
        int[]        new_unused_tag_region_start;
        String[]     new_unused_tag_region_string;
        String       message_string;
        String       message_log;
        KeyList      key_list;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
            ENTER_METHOD, ENTER_METHOD,
            SPACE, 
            SAVEUNUSEDREGIONLIST_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (tag_strings == null) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(
                    DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, 
                    MessageText.DEBUG_NULL_KEY);
                getLogPrintStream().printLogln(message_log);
            }
            return;
        }
        try {
            key_list = KeyListFactory.createKeyList(tag_strings, COMMA,
                getMessageResource(),
                getDebug(),
                getLogPrintStream() );
        } catch (Exception exception_object) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String exception_message;
                exception_message = exception_object.getMessage();
                if (exception_message == null) {
                    exception_message = MessageText.DEBUG_NULL_KEY;
                }
                message_log = getMessage(
                    DEBUG_EXCEPTION_KEY, DEBUG_EXCEPTION_KEY,
                    SPACE, 
                    SAVEUNUSEDREGIONLIST_METHOD,
                    SPACE,
                    exception_message);
                getLogPrintStream().printLogln(message_log);
            }
            return;
        }

        if (unused_tag_region_list.length == unused_tag_region_list_index) {
            bk_unused_tag_region_list 
                = new Region[unused_tag_region_list.length+REGION_UNIT][];
            bk_unused_tag_region_index 
                = new int[unused_tag_region_list.length+REGION_UNIT];
            bk_unused_tag_region_reference 
                = new int[unused_tag_region_list.length+REGION_UNIT][];
            bk_unused_tag_region_start 
                = new int[unused_tag_region_list.length+REGION_UNIT][];
            bk_unused_tag_region_string 
                = new String[unused_tag_region_list.length+REGION_UNIT][];

            for (int i = 0; i < unused_tag_region_list.length; i++) {
                bk_unused_tag_region_list[i] = unused_tag_region_list[i];

                bk_unused_tag_region_index[i] = unused_tag_region_index[i];
                bk_unused_tag_region_reference[i] 
                    = unused_tag_region_reference[i];
                bk_unused_tag_region_start[i] = unused_tag_region_start[i];
                bk_unused_tag_region_string[i] = unused_tag_region_string[i];
            }
            unused_tag_region_list = bk_unused_tag_region_list;
            unused_tag_region_index = bk_unused_tag_region_index;
            unused_tag_region_reference = bk_unused_tag_region_reference;
            unused_tag_region_start = bk_unused_tag_region_start;
            unused_tag_region_string = bk_unused_tag_region_string;
        }

        new_unused_tag_region_list = new Region[tag_region.length];
        for (int i = 0; i < tag_region_index; i++) {
            new_unused_tag_region_list[i] = new Region(tag_region[i]);
        }
        new_unused_tag_region_reference = new int[tag_region_reference.length];
        for (int i = 0; i < tag_region_index; i++) {
            new_unused_tag_region_reference[i] = tag_region_reference[i];
        }
        new_unused_tag_region_start = new int[tag_region_start.length];
        for (int i = 0; i < tag_region_index; i++) {
            new_unused_tag_region_start[i] = tag_region_start[i];
        }
        new_unused_tag_region_string = new String[tag_region_string.length];
        for (int i = 0; i < tag_region_string.length; i++) {
            try {
                message_string = key_list.getKeyEntry(i);
            } catch (Exception exception_object) {
                continue;
            }
            new_unused_tag_region_string[i] = message_string;
        }

        unused_tag_region_list[unused_tag_region_list_index] 
            = new_unused_tag_region_list;
        unused_tag_region_index[unused_tag_region_list_index] 
            = tag_region_index;
        unused_tag_region_reference[unused_tag_region_list_index]
            = new_unused_tag_region_reference;
        unused_tag_region_start[unused_tag_region_list_index]
            = new_unused_tag_region_start;
        unused_tag_region_string[unused_tag_region_list_index]
            = new_unused_tag_region_string;

        unused_tag_region_list_index++;
    }

    public int getNumberUnusedTagString(String tag) 
    {
        int    number;

        number = getNumberOfUnusedRegion(tag); 
        return number;
    }

    public Region getReportRegion(int index) 
    {
        if (read_region_index <= index) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String message_log;
                message_log = getMessage(
                    DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, 
                    MessageText.DEBUG_NULL_KEY);
                getLogPrintStream().printLogln(message_log);
            }
            return null;
        }
        return read_region_box[index];
    }

    public String getUnusedTagStrings() {
        int      index;
        int      number;
        String   tag_string;
        Region   local_region_object;
        String   upper_tag_string;

        tag_string = new String();
        for (index = 0; index < tag_region_string.length; index++) {
            number = getNumberOfUnusedRegion(tag_region_string[index]); 
            for (int i = 0; i < number; i++) {
                local_region_object = getUnusedRegion(
                        tag_region_string[index], i) ;
                upper_tag_string = getTagStringWithOption(local_region_object);
                if (index != 0 || i != 0) {
                    tag_string = tag_string.concat(COMMA);
                }
                tag_string = tag_string.concat(upper_tag_string);
            } 
        }
        return tag_string;
    }

    public String getUnusedTagString(String tag, int specified_index) {
        int      number;
        Region   local_region_object;
        String   upper_tag_string;

        if (tag == null) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String message_log;
                message_log = getMessage(
                    DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, 
                    MessageText.DEBUG_NULL_KEY);
                getLogPrintStream().printLogln(message_log);
            }
            return null;
        }
        number = getNumberOfUnusedRegion(tag); 
        if (number <= specified_index) {
            return null;
        }
        local_region_object = getUnusedRegion(tag, specified_index) ;
        upper_tag_string = getTagStringWithOption(local_region_object);
        return upper_tag_string;
    }

    /**
     * before specified tag
     **/
    public Region getUnusedRegion(String tag, int specified_index) {
        int i;
        int index;
        int count;
        String  message_log;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                GETUNUSEDREGION_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (tag == null) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(
                    DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, 
                    MessageText.DEBUG_NULL_KEY);
                getLogPrintStream().printLogln(message_log);
            }
            return null;
        }
        for (index = 0; index < tag_region_string.length; index++) {
            if (tag_region_string[index].compareTo(tag) == 0) {
                break;
            }
        }
        for (count = 0, i = 0; i < tag_region_index; i++) {
            if (tag_region_reference[i] == index) {
                if (specified_index == count) {
                    return tag_region[i];
                }
                count++;
            }
        }
        return null;
    }

    public int getNumberOfUnusedRegionWithTransparent(String tag) {
        int   ret;

        ret = getNumberOfUnusedRegionBody(tag, true);
        return ret;
    }

    public int getNumberOfUnusedRegion(String tag) {
        int   ret;

        ret = getNumberOfUnusedRegionBody(tag, false);
        return ret;
    }

    public int getNumberOfUnusedRegionBody(String tag, boolean flag) {
        int i;
        int index;
        int count;

        if (tag == null) {
            return 0;
        }
        for (index = 0; index < tag_region_string.length; index++) {
            if (flag == false) {
                if (tag_region_string[index].compareTo(tag) == 0) {
                    break;
                }
            } else {
                if (matchTransparentTag(tag_region_string[index], null,
                    tag) == true) {
                    break;
                }
            }
        }
        if (tag_region_string.length == index) {
            return 0;
        }
        for (count = 0, i = 0; i < tag_region_index; i++) {
            if (tag_region_reference[i] == index) {
                count++;
            }
        }
        return count;
    }

    private boolean restoreUnusedRegionList(int index) {

        if (index >= unused_tag_region_list.length) {
            return false;
        }
        tag_region = unused_tag_region_list[index];

        tag_region_reference = unused_tag_region_reference[index];
        tag_region_start = unused_tag_region_start[index]; 
        tag_region_index = unused_tag_region_index[index]; 

        tag_region_string = new String[unused_tag_region_string[index].length];
        for (int i = 0; i < tag_region_string.length; i++) {
            tag_region_string[i] =
                unused_tag_region_string[index][i];
        }
        for (int i = 0; i < tag_region_string.length; i++) {
            tag_region_string[i] = deleteKey(tag_region_string[i]);
        }

        return true;
    }

    private boolean restoreUnusedRegionList(String tag_list, int index) {
        int descriptor;
        boolean flag;

        if (tag_list == null) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String message_log;
                message_log = getMessage(
                    DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, 
                    MessageText.DEBUG_NULL_KEY);
                getLogPrintStream().printLogln(message_log);
            }
            return false;
        }

        descriptor = searchUnusedRegionList(tag_list, index);
        if (descriptor != -1) {
            flag = restoreUnusedRegionList(descriptor);
            return flag;
        }
        return false;
    }

    private int searchUnusedRegionList(String tag_string, int index)
    {
        int               i, j;
        String[]          list;
        Region[]          local_region_object;
        int               list_index;
        StringTokenizer   stk;
        int               key_list_count;
        KeyList           key_list;
        String            key_list_string;
        String            message_log;
        int               index_count;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                SEARCHUNUSEDREGIONLIST_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (tag_string == null) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(
                    DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                    SPACE, 
                    MessageText.DEBUG_NULL_KEY);
                getLogPrintStream().printLogln(message_log);
            }
            return -1;
        }
        try {
            key_list = KeyListFactory.createKeyList(tag_string, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String exception_message;
                exception_message = exception_object.getMessage();
                if (exception_message == null) {
                    exception_message = MessageText.DEBUG_NULL_KEY;
                }
                message_log = getMessage(
                    DEBUG_EXCEPTION_KEY, DEBUG_EXCEPTION_KEY,
                    SPACE, 
                    SAVEUNUSEDREGIONLIST_METHOD,
                    SPACE,
                    exception_message);
                getLogPrintStream().printLogln(message_log);
            }
            return -1;
        }

        index_count = 0;
        for (i = 0; i < unused_tag_region_list_index; i++) {
            for (j = 0; j < unused_tag_region_string[i].length; j++) {
                try {
                    key_list_string = key_list.getKeyEntry(j);
                } catch (Exception exception_object) {
                    return -1;
                }
                if (unused_tag_region_string[i][j].compareTo(key_list_string) != 0) {
                    if (index_count != index) {
                        index_count = index_count + 1;
                        continue;
                    }
                    break;
                }
            }
            if (j == unused_tag_region_string[i].length) {
                break;
            }
        }
        if (i < unused_tag_region_list_index) {
            return (i);
        }
        return -1;
    }

    public void clearUnusedRegion() {
        tag_region = new Region[0];
        tag_region_reference = new int[0];
        tag_region_start = new int[0]; 
        tag_region_string = new String[0]; 
        tag_region_index = 0;
    }

    public void deleteUnusedRegion(String unused_tag_string, int index) {
        int       count;
        int       work;
        boolean   lpflag;
        int       loop;
        int       reference_count;
        int[]     bk_tag_region_reference;
        int[]     bk_tag_region_start;
        Region[]  bk_region;
        String    message_log;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                DELETEUNUSEDREGION_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (unused_tag_string == null) {
            return;
        }
        if (tag_region_index <= index) {
            return;
        }
        for (count = 0; count < tag_region_string.length; count++) {
            if (tag_region_string[count].compareTo(unused_tag_string) == 0) {
                break;
            }
        }

        reference_count = 0;
        for (work = 0; work < tag_region_index; work++) {
            if (tag_region_reference[work] == count) {
                if (reference_count == index) {
                    break;
                }
                reference_count++;
            }
        }
        if (tag_region_reference.length <= work) {
            return;
        }

        lpflag = false;
        for (loop = work+1; loop < tag_region_index ; loop++) {
            if (tag_region_reference[loop] == count) {
                lpflag = true;
            } else {
                if (lpflag == true) {
                    break;
                }
            }
        }

        bk_tag_region_reference = new int[tag_region_reference.length];
        bk_tag_region_start = new int[tag_region_start.length];
        bk_region = new Region[tag_region.length];
        for (int i = 0; i < work; i++) {
            bk_region[i] = tag_region[i];
        }
        for (int i = work; i < tag_region.length-1; i++) {
            bk_region[i] = tag_region[i+1];
        }

        if (loop == tag_region_reference.length) {
            String[] bk_string;
            bk_string = new String[tag_region_string.length-1]; 
            for (int i = 0; i < count; i++) {
                bk_string[i] = tag_region_string[i];
            }
            for (int i = count; i < bk_string.length-1; i++) {
                bk_string[i] = tag_region_string[i+1];
            }
            tag_region_string = bk_string;
            for (int i = 0; i < work; i++) {
                bk_tag_region_reference[i] = tag_region_reference[i];
                bk_tag_region_start[i] = tag_region_start[i];
            }

            for (int i = work; i < tag_region.length-1; i++) {
                if (loop <= i) {
                    bk_tag_region_reference[i] = tag_region_reference[i+1]-1;
                } else {
                    bk_tag_region_reference[i] = tag_region_reference[i+1];
                }
                bk_tag_region_start[i] = tag_region_start[i+1] - 1;
            }
        } else {
            for (int i = 0; i < work; i++) {
                bk_tag_region_reference[i] = tag_region_reference[i];
                bk_tag_region_start[i] = tag_region_start[i];
            }
            for (int i = work; i < tag_region.length-1; i++) {
                bk_tag_region_reference[i] = tag_region_reference[i+1];
                bk_tag_region_start[i] = tag_region_start[i+1] - 1;
            }
        }

        if (index == 0 && lpflag == false) {
            String[] bk_string;
            bk_string = new String[tag_region_string.length-1]; 
            for (int i = 0; i < bk_string.length; i++) {
                bk_string[i] = tag_region_string[i];
            }
            tag_region_string = bk_string;
        }

        tag_region_index = tag_region_index - 1;
    }

    public boolean checkUnusedRegion(String tag) {
        int         count;
        boolean     return_flag;
        String      message_log;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                CHECKUNUSEDREGION_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        return_flag = false;
        if (tag == null) {
            return return_flag;
        }

        for (count = 0; count < tag_region_string.length; count++) {
            if (tag_region_string[count] == tag) { break; }
        }

        for (int i = 0; i < tag_region_reference.length; i++) {
            if (tag_region_reference[i] == count) {
                return true;
            }
        }
        return return_flag;
    }

    public void replaceUnusedRegion
       (String unused_tag_string, String tag_string) {
        int         count;
        String      message_log;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                REPLACEUNUSEDREGION_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (unused_tag_string == null || tag_string == null) {
            return;
        }
        for (count = 0; count < tag_region_string.length; count++) {
            if (tag_region_string[count].compareTo(unused_tag_string) == 0)  { 
                break; 
            }
        }
        if (count >= tag_region_string.length) {
            return;
        }
        tag_region_string[count] = tag_string;
    }

    public void addUnusedRegion(
        Region unused_region, int unused_reference, String unused_tag_string) {
        String   message_log;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                ADDUNUSEDREGION_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        if (unused_region == null || unused_tag_string == null) {
            return;
        }
        if (tag_region.length <= tag_region_index) {
            Region[]     bk;
            int[]        bk_ref;
            int[]        bk_start;

            bk = new Region[tag_region.length+REGION_UNIT];
            bk_ref = new int[tag_region_reference.length+REGION_UNIT];
            bk_start = new int[tag_region_start.length+REGION_UNIT];
            for (int i = 0; i < tag_region.length; i++) {
                bk[i] = tag_region[i];
                bk_ref[i] = tag_region_reference[i];
                bk_start[i] = tag_region_start[i];
            }
            tag_region = bk;
            tag_region_reference = bk_ref;
            tag_region_start = bk_start; 
        }
        if (tag_region_string.length <= unused_reference) {
            String[]   bk_string;
          
            bk_string = new String[unused_reference+1];
            for (int i = 0; i < tag_region_string.length; i++) {
                bk_string[i] = tag_region_string[i];
            }

            for (int i = tag_region_string.length; i < bk_string.length; i++) {
                bk_string[i] = new String();
            }
            tag_region_string = bk_string;
            tag_region_string[unused_reference] = unused_tag_string;
        }
/**
        tag_region[tag_region_index] = unused_region;
**/
        tag_region[tag_region_index] = new Region(unused_region);
        tag_region_reference[tag_region_index] = unused_reference;
        for (int i = 0; i < tag_region_reference.length; i++) {
            if (unused_reference == tag_region_reference[i]) {
                tag_region_start[tag_region_index] = i;
                break;
            }
        }

        tag_region_index = tag_region_index + 1;

        return;
    }

    private boolean saveRegionBuffer() {
        if (head == null) {
            return false;
        }
        if (head.length == 0) {
            return false;
        }
        bkup_head = head[0];
        bkup_region_object = current_region_object;
        return true;
    }

    private boolean restoreRegionBuffer() {
        if (bkup_head == null) {
            return false;
        }
        if (head == null) {
            return false;
        }
        if (head.length == 0) {
            return false;
        }
        head[0] = bkup_head;
        current_region_object = bkup_region_object;
        bkup_head = null;
        bkup_region_object = null;
        return true;
    }

    private boolean setRegionBuffer(byte[][] buffer) {
        if (buffer == null) {
            return false;
        }
        if (head == null) {
            return false;
        }
        if (head.length == 0) {
            return false;
        }
        head[0] = buffer;
        return true;
    }

    public boolean clearRegionBuffer() {
        if (head == null || head.length == 0) {
            head = new byte[1][][];
        }
        head[0] = null;
        return true;
    }

    public boolean checkRegionBuffer() {
        if (head == null || head.length == 0 || head[0] == null) {
            return false;
        }
        return true;
    }

    public boolean checkRegionList(String tags) {
        String[]    list;
        String      target_tag;
        Region      local_region;
        String      message_log;

        if (tags == null) {
            return false;
        }
        if (checkCurrentRegionObject() == false || 
                getCurrentRegionObjectLength() == 0) {
            return false;
        }
        int               list_index;
        StringTokenizer   stk;
        KeyList           key_list;
        String            key_list_string;

        try {
            key_list = KeyListFactory.createKeyList(tags, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String exception_message;
                exception_message = real_exception_object.getMessage();
                if (exception_message == null) {
                    exception_message = MessageText.DEBUG_NULL_KEY;
                }
                message_log = getMessage(
                    DEBUG_EXCEPTION_KEY, DEBUG_EXCEPTION_KEY,
                    SPACE, 
                    CHECKREGIONLIST_METHOD,
                    SPACE,
                    exception_message);
                getLogPrintStream().printLogln(message_log);
            }
            return false;
        } 

        int j;
        for (int i = 0; i < getCurrentRegionObjectLength(); i++) {
            local_region = getCurrentRegionObjectEntry(i);
            target_tag = getTagStringWithOption(local_region);
            for (j = 0; j < key_list.length(); j++) {
                try {
                    key_list_string = key_list.getKeyEntry(j);
                } catch (Exception real_exception_object) {
                    continue;
                }

                if (compareTags(key_list_string, target_tag,NAME_KEY)==true) {
                    break;
                }
            }
            if (j == key_list.length()) { return false; }
        }
        return true;
    }

    /**
     * @module     getRegionListKey
     *
     * @param      Region[]
     *             Region hierarchy
     *
     * @return     String
     *             string combined with COMMA to tag.
     */
    public String getRegionListKey(Region[] region_list) {
        String[]    list;
        String      target_tag;

        if (region_list == null || region_list.length == 0) {
            return null;
        }
        target_tag = null;
        for (int i = 0; i < region_list.length; i++) {
            if (i == 0) {
                target_tag = getTagStringWithOption(region_list[i]);
            } else {
                target_tag = target_tag.concat(
                    COMMA + getTagStringWithOption(region_list[i]));
            }
        }
        return target_tag;
    }

    public boolean moreUnknownRegionLists(String tags) {
        Region[]          region_object;
        KeyList           key_list;
        String            key_list_string;
        String            target_tag;
        String            message_log;

        if (tags == null) {
            return false;
        }
        try {
            key_list = KeyListFactory.createKeyList(tags, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String exception_message;
                exception_message = real_exception_object.getMessage();
                if (exception_message == null) {
                    exception_message = MessageText.DEBUG_NULL_KEY;
                }
                message_log = getMessage(
                    DEBUG_EXCEPTION_KEY, DEBUG_EXCEPTION_KEY,
                    SPACE, 
                    MOREUNKNOWNREGIONLISTS_METHOD,
                    SPACE,
                    exception_message);
                getLogPrintStream().printLogln(message_log);
            }
            return false;
        } 

        int j;
        for (int i = 0; i < unknown_region_index; i++) {
            region_object = unknown_region[i];
           
            for (j = 0; j < region_object.length; j++) {
                target_tag = getTagStringWithOption(region_object[j]);
                try {
                    key_list_string = key_list.getKeyEntry(j);
                } catch (Exception real_exception_object) {
                    continue;
                }
                if (compareTags(key_list_string,target_tag,NAME_KEY)!=true) {
                    break;
                }
            }
            if (j == region_object.length) {
                return true;
            }
        }
        return false;
    }

    public boolean moreUnknownRegionLists() {
        if (unknown_region_index > 0) {
            return true;
        }
        return false;
    }

    public void debugRegionList() {
        Region[]   region_object;

        for (int i = 0; i < unknown_region_index; i++) {
            region_object = unknown_region[i];
            for (int j = 0; j < region_object.length; j++) {
                System.err.println(DEBUG_STRING
                    + getTagStringWithOption(region_object[j]));
            }
        }
    }

    public int nextRegionList(int index) {
        String message;
        byte[][] local_head;

        if (unknown_region_index <= 0) {
            return -1;
        }
        if (unknown_region == null) {
            return -1;
        }

        current_region_object = unknown_region[index];
        local_head = unknown_head[index];
        Region[][] bk;
        byte[][][] h_bk;

        bk = new Region[unknown_region.length-1][];
        h_bk = new byte[unknown_head.length-1][][];
        for (int i = 0; i < unknown_region_index-1; i++) {
            bk[i] = unknown_region[i+1];
            h_bk[i] = unknown_head[i+1];
        }
        unknown_region = bk;
        unknown_head = h_bk;
        unknown_region_index--;
        head[0] = local_head;

        return getCurrentRegionObjectLength();
    }


    public int getRegionList(String tags, int index ) {
        int      j;
        int      index_count;
        Region[] region_list = null;
        String[] list;
        String   tag_string;
        byte[][] local_head;
        String   work_entry;

        if (tags == null) {
            return 0;
        }
        local_head = new byte[0][];

        int               list_index;
        StringTokenizer   stk;
        int               key_list_count;
        KeyList           key_list;

        if (unknown_region_index == 0) {
            return 0;
        }

        try {
            key_list = KeyListFactory.createKeyList(tags, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            return 0;
        }

        region_list = current_region_object;
        index_count = 0;
        for (int i = 0; i < unknown_region_index; i++) {
            current_region_object = unknown_region[i];
            local_head = unknown_head[i];

            for (j = 0; j < getCurrentRegionObjectLength(); j++) {
                tag_string = getTagStringWithOption(current_region_object[j]);
                try {
                    work_entry = key_list.getKeyEntry(j);
                } catch (Exception exception_object) {
                    current_region_object = region_list;
                    return 0;
                }
                if (compareTags(tag_string, work_entry, NAME_KEY) == false) {
                    break;
                }
            }
            if (j == getCurrentRegionObjectLength()) {
                if (index_count != index) {
                    index_count = index_count + 1;
                    continue;
                }
                break;
            }
            clearCurrentRegionObject();
        }
        if (checkCurrentRegionObject() == true) {
            head[0] = local_head;
        } else {
            current_region_object = region_list;
            return 0;
        }

        return getCurrentRegionObjectLength();
    }

    public String getMessageFromRegionList() {
        byte[] buffer;
        String buffer_string;
        byte[][] local_head;


        if (current_region_object[getCurrentRegionObjectLength()-1].getStartPosition() <
            current_region_object[getCurrentRegionObjectLength()-1].getEndPosition() ) {
            try {
                local_head = head[0];

                buffer = current_region_object[getCurrentRegionObjectLength()-1].extractRegion(
                    local_head,
	  	    current_region_object[getCurrentRegionObjectLength()-1].getStartPosition(),
		    current_region_object[getCurrentRegionObjectLength()-1].getEndPosition());
                buffer_string = new String(buffer);
                return buffer_string;
            } catch (Exception real_exception_object) {
                return new String(EMPTY);
            }
        }
        return new String(EMPTY);
    }

    /**
     * This method clears the current region list.
     */
    public void clearCurrentRegionObject() {
        current_region_object = null;
    }

    public boolean checkCurrentRegionObject() {
        if (current_region_object != null) {
            return true;
        }
        return false;
    }

    /**
     * @module    insertKey
     *
     * @param     String   tag is a word between '<' and '>'
     * @param     String   key is a option string in tag
     *
     * @return    String   new tag string with the option
     */
    public static String insertKey(String tag, String key) {
        String tag_head;
        String tag_tail;
        String key_string;

        if (tag == null || key == null) {
            return null;
        }
        if (tag.charAt(tag.length()-1) == TAG_END) {
            tag_head = tag.substring(0, tag.length()-1);
            tag_tail = tag.substring(tag.length()-1);
            key_string = tag_head + SPACE + key + tag_tail;
            return key_string;
        }
        return null;
    }

    public String getFormattedTags(String keys, String format) 
        throws Exception {
        String[]  list;
        String[]  new_list;
        String[]  format_list;
        Exception exception_object;
        String    key_string;
        String    message;

        if (keys == null || format == null) {
            message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        int               list_index;
        StringTokenizer   stk;
        int               key_list_count;
        KeyList           key_list;
        KeyList           new_key_list;
        KeyList           format_key_list;
        String            key_list_string;
        String            format_key_list_string;

        try {
            key_list = KeyListFactory.createKeyList(keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION, keys, format);
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (key_list.length() <= 0) {
            message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION, keys, format);
            exception_object = new Exception(message);
            throw exception_object;
        }

        try {
            new_key_list = new KeyList(key_list.length());
        } catch (Exception real_exception_object) {
            message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION, keys, format);
            exception_object = new Exception(message);
            throw exception_object;
        }
        try {
            format_key_list = KeyListFactory.createKeyList(format, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }

        int j;
        for (int i = 0; i < format_key_list.length(); i++) {
            for (j = 0; j < key_list.length(); j++) {
                try {
                    key_list_string = key_list.getKeyEntry(j);
                    format_key_list_string = format_key_list.getKeyEntry(i);
                } catch (Exception real_exception_object) {
                    throw real_exception_object;
                }
                if (compareTags(format_key_list_string, 
                        key_list_string, NAME_KEY)  == true) {
                    try {
                        new_key_list.setKeyEntry(i, key_list_string);
                    } catch (Exception real_exception_object) {
                        throw real_exception_object;
                    }
                    break;
                }
            }
            if (j == key_list.length()) {
                message = getExceptionMessage(
                    ARGUMENT_PARAMETER_EXCEPTION, keys, format);
                exception_object = new Exception( message );
                throw exception_object;
            }
        }
        key_string = null;
        for (int i = 0; i < new_key_list.length(); i++) {
            try {
                key_list_string = new_key_list.getKeyEntry(i);
            } catch (Exception real_exception_object) {
                throw real_exception_object;
            }
            if (i == 0) {
                key_string = new String(key_list_string);
            } else {
                key_string = key_string.concat(COMMA+
                    key_list_string);
            }
        }
        return key_string;
    }

    public static String getTagStringWithOption(Region region_object_parameter) {
        String  tag;
        String  key;
        String  name;
        String  key_string;

        if (region_object_parameter == null) {
            return null;
        }
        tag = region_object_parameter.getTagString(); 
        name = region_object_parameter.getTagOptionKey();
        key = region_object_parameter.getKeyString();
        if (key == null || name == null) {
            key_string = tag;
        } else {
            key_string = insertKey(tag, name + EQUAL + key);
        }
        return key_string;
    }


    public String getStartTag(String tag, String option) {
        String work_string;

        if (tag == null || option == null) {
            return null;
        }
        work_string = new String(TAG_START_STRING 
            + tag + SPACE + NAME_KEY + EQUAL + option
            + TAG_END_STRING);

        return work_string;
    }

    public String getStartTag(String tag) {
        String work_string;

        if (tag == null) {
            return null;
        }
        work_string = new String(TAG_START_STRING 
            + tag
            + TAG_END_STRING);

        return work_string;
    }

    public String getKeys() {
        String  tag;
        String  key;
        String  keys;
        String  name;
        String  key_string;
        
        if (checkCurrentRegionObject() == false) { return null; }
        keys = null;

        for (int i = 0; i < getCurrentRegionObjectLength(); i++) {
            if (current_region_object[i] == null) {
                continue;
            }
            tag = current_region_object[i].getTagString(); 
            name = current_region_object[i].getTagOptionKey();
            key = current_region_object[i].getKeyString();

            if (key == null || name == null) {
                key_string = tag;
            } else {
                key_string = insertKey(tag, name + EQUAL + key);
            }
            if (key_string != null) {
                if (keys == null) {
                    keys = key_string;
                } else {
                    keys = keys.concat(COMMA + key_string);
                }
            } else {
                keys = keys.concat(tag);
            }
        }
        return keys;
    }

    public static String getOptionValue(String key, String name_key) {
        String options;
        String key_upper;
        String name_upper;
        String[] list;
        String value;
        int option_index;
        String after_equal;

        if (key == null) {
            return null;
        }
        option_index = key.indexOf(SPACE);
        if (option_index < 0) {
            return null;
        }
        if (name_key == null) {
            return null;
        }
        key_upper = key.toUpperCase();
        name_upper = name_key.toUpperCase();
        option_index = key_upper.indexOf(name_upper);
        if (option_index < 0) {
            return null;
        }
        after_equal = key_upper.substring(option_index+name_upper.length());
        option_index = after_equal.indexOf(EQUAL);
        if (option_index < 0) {
            return null;
        }
        value = after_equal.substring(option_index+EQUAL.length()).trim();

        value = deleteKey(value);
        option_index = value.indexOf(TAG_END_STRING);
        if (option_index >= 0) {
            value = value.substring(0, option_index);
        }
        return value;
    }

    /**
     * key1 -> specified key2 -> detect 
     */
    public static boolean compareTags(String key1, String key2, String name_key) {
        String key1_key;
        String key2_key;
        String key1_option;
        String key2_option;
        int    space_index;

        if (key1 == null || key2 == null) {
            return false;
        }
        space_index = key1.indexOf(SPACE);
        if (space_index >= 0) {
            key1_key = key1.substring(0, space_index);
        } else {
            key1_key = key1.substring(0, key1.length()-1);
        }
        space_index = key2.indexOf(SPACE);
        if (space_index >= 0) {
            key2_key = key2.substring(0, space_index);
        } else {
            key2_key = key2.substring(0, key2.length()-1);
        }
        if (key1_key.compareTo(key2_key) != 0) {
            return false;
        }
        /** 
         * key1 -> specified key2 -> detect 
         */
        space_index = key1.indexOf(SPACE);
        if (space_index < 0) {
            return true;
        }

/** 2013/2/7 **/
        space_index = key2.indexOf(SPACE);
        if (space_index < 0) {
            return false;
        }
        if (name_key == null) {
            return true;
        }
       
        key1_option = getOptionValue(key1, name_key);
        key2_option = getOptionValue(key2, name_key);
        if (key1_option.compareTo(key2_option) == 0) {
            return true;
        }
        return false;
    }
    
    public static int printStartTag(PrintStream print_stream,
        int tabcount, String tag) {

        if (print_stream == null) {
            return 0;
        }
        
        for (int i = 0; i < tabcount; i++) {
            print_stream.print(ReportRegion.TAB_SPACE);
        }
        if (tag != null) {
            print_stream.println(tag);
        } else {
            print_stream.println(EMPTY);
        }
        tabcount = tabcount + 1;
        return tabcount;
    }

    public static int printEndTag(PrintStream print_stream,
        int tabcount, String tag) {

        if (print_stream == null) {
            return 0;
        }
        tabcount = tabcount - 1;
        for (int i = 0; i < tabcount; i++) {
            print_stream.print(ReportRegion.TAB_SPACE);
        }
        if (tag != null) {
            print_stream.println(ReportRegion.convertFromStartToEnd(tag));
        } else {
            print_stream.println(EMPTY);
        }
        return tabcount;
    }

    public int getKeyNumber(String key_string) throws Exception {
        KeyList   key_list;

        if (key_string == null) {
            String     message;
            Exception  exception_object;
            message = getExceptionMessage( TAGKEYS_PARAMETER_ERROR );
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            key_list = KeyListFactory.createKeyList(key_string, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                String message_log;
                String exception_message;
                exception_message = exception_object.getMessage();
                if (exception_message == null) {
                    exception_message = MessageText.DEBUG_NULL_KEY;
                }
                message_log = getMessage(
                    DEBUG_EXCEPTION_KEY, DEBUG_EXCEPTION_KEY,
                    SPACE, 
                    GETKEYNUMBER_METHOD,
                    SPACE,
                    exception_message);
                getLogPrintStream().printLogln(message_log);
            }
            throw exception_object;
        }
        return key_list.length();
    }

    private boolean isTransparent(String tag) {
        StringTokenizer stk;
        String key;

        if (tag == null) {
            return false;
        }
        stk = new StringTokenizer(tag, TAG_START_STRING);
        if (stk.countTokens() > 1) {
            while (stk.hasMoreTokens()) {
                key = stk.nextToken();
                if (key.charAt(key.length()-1) != TAG_END) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    private boolean matchTransparentTag(String transparent_tag, 
        String target, String reverse ) {
        StringTokenizer stk;
        String key;

        if (transparent_tag == null || (target == null && reverse == null)) {
            return false;
        }
        stk = new StringTokenizer(transparent_tag, TAG_START_STRING);
        while (stk.hasMoreTokens()) {
            key = TAG_START_STRING + stk.nextToken();

            if (target != null) {
                if (compareTags(key, target, NAME_KEY) == true) {
                    return true;
                }
            } else {
                if (compareTags(reverse, key, NAME_KEY) == true) {
                    return true;
                }
            }
        }
        return false;
    }

    private boolean checkNameKey(String tag_keys) {
        int     space_index;
        int     name_index;
        String  name_string;

        if (tag_keys == null) {
            return false;
        }
        space_index = tag_keys.indexOf(SPACE);
        if (space_index >= 0) {
            name_string = tag_keys.substring(space_index+1) ;
            if (name_string != null) {
                name_index = name_string.indexOf(NAME_KEY+EQUAL);
                if (name_index >= 0) {
                    return true;
                }
            }
        }
        return false;
    }

/*
    private String getTagHeader() throws Exception {
*/
    private String getTagHeader(String key_string) throws Exception {
/*
        String       key_string;
*/
        String       unknown_key_string;
        String       key_string_length;
        String       unknown_key_string_length;
        String       tag_header;

        int               list_index;
        KeyList           key_list;
        String            key_list_string;
        String            work_string;
        int               loop;

/**
        key_string = getKeys();
**/
        try {
            key_list = KeyListFactory.createKeyList(key_string, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            throw exception_object;
        }
        unknown_key_string = new String();
        for (int k = 0; k < key_list.length(); k++) {
            try {
                key_list_string = key_list.getKeyEntry(k);
            } catch (Exception exception_object_key) {
                continue;
            }
            loop = getNumberUnusedTagString(deleteKey(key_list_string));
            for (int m = 0; m < loop; m++) {
                work_string = getUnusedTagString(deleteKey(key_list_string), m);
                if (work_string == null) {
                    continue;
                }
                if (unknown_key_string.length() > 0) {
                    unknown_key_string = unknown_key_string.concat(COMMA);
                }
                unknown_key_string = unknown_key_string.concat(work_string);
            }
        }

        key_string_length = String.valueOf(key_string.length());
        unknown_key_string_length = String.valueOf(unknown_key_string.length());

        tag_header = key_string_length + SPACE + key_string + SPACE
                   + unknown_key_string_length + SPACE
                   + unknown_key_string;
        return tag_header;
    }

    private String formatTagHeader(String key_string, 
        String unknown_key_string) {

        String key_string_length;
        String unknown_key_string_length;
        String tag_header;

        key_string_length = String.valueOf(key_string.length());
        unknown_key_string_length = String.valueOf(unknown_key_string.length());

        tag_header = key_string_length + SPACE + key_string + SPACE
                   + unknown_key_string_length + SPACE
                   + unknown_key_string;
        return tag_header;
    }

    /**
     * <DIVISION>,<CUSTONER>
     * 
     * TAG = NON    NAME = NON   - <DIVISION><CUSTOMER>, <CUSTOMER><DIVISION>
     *                             getContents - <>
     * TAG = VALID  NAME = NON   - <DIVISION><CUSTOMER>
     * TAG = VALID  NAME = VALID - <DIVISION NAME=DEVELOP><CUSTOMER NAME=HONDA>
     * TAG = NON    NAME = VALID - <DIVISION NAME=DEVELOP><CUSTOMER NAME=HONDA>,
     *                             <CUSTOMER NAME=HONDA><DIVISION NAME=DEVELOP>
     */
    public String nextContent(String tag_keys, String org_tags) throws Exception {
        Region[]  regionlist_object;
        String    transparent;
        String    format_string;
        String    org_format_string;
        String[]  list;
        int       index;
        boolean   name_flag;
        String    formatted_tags = null;
        String    message;
        String    message_log;
        Exception exception_object;
        Region[]  local_region_object;
        int       list_count;
        KeyList   key_list;
        String    key_list_string;
        KeyList   org_key_list;
        String    org_key_list_string;

        String    key_string;
        String    tag_length_string;
        String    length_plus_tag;
        String    length_tag_message;
        String    target_tags;

        if (getDebug() > 0) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                NEXTCONTENT_METHOD);
            if (getLogPrintStream() == null) {
                setLogPrintStream(new LogPrintStream(System.err));
            }
            getLogPrintStream().printLogln(message_log);
        }
        if (tag_keys == null) {
            message = getExceptionMessage( TAGKEYS_PARAMETER_ERROR );
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (org_tags == null) {
            message = getExceptionMessage( ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }

        int               list_index;
        StringTokenizer   stk;

        try {
            org_key_list = KeyListFactory.createKeyList(org_tags, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        } 
        try {
            key_list = KeyListFactory.createKeyList(tag_keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        } 

        name_flag = checkNameKey(tag_keys);

        format_string = getFormat();
        if (format_string == null) {
            transparent = new String();
            org_format_string = new String();
            for (int i = 0; i < key_list.length(); i++) {
                if (i > 0) {
                    transparent = transparent.concat(COMMA);
                    org_format_string = org_format_string.concat(COMMA);
                }
                for (int j = 0; j < key_list.length(); j++) {
                    if (name_flag == true) {
                        transparent = transparent.concat(deleteKey(
                            key_list.getKeyEntry(j)));
                    } else {
                        transparent = transparent.concat(
                            key_list.getKeyEntry(j));
                    }
                    org_format_string = org_format_string.concat(
                            org_key_list.getKeyEntry(j));
                }
            }
            format_string = transparent;
            clearRegionObject();
        } else {
            if (saveRegionBuffer() == false) {
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(END_METHOD, END_METHOD,
                        SPACE, 
                        NEXTCONTENT_METHOD);
                    getLogPrintStream().printLogln(message_log);
                }
                message = getExceptionMessage(
                    REGION_BUFFER_ERROR, (String[])null);
                exception_object = new Exception(message);
                throw exception_object;                
            }
            saveCurrentUnusedRegionList();
            transparent = null;

            if (name_flag != true) {
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, 
                        NEXTREGIONLIST_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                    getLogPrintStream().printLogln(message_log);
                }
                index = 0;

                list_count = nextRegionList(index);
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                        SPACE, 
                        getMessageResource().getMessage(DEBUG_COUNT_KEY) +
                        SPACE +
                        String.valueOf(list_count) );
                    getLogPrintStream().printLogln(message_log);
                }
                if (list_count > 0) {
                    formatted_tags = getFormattedTags(getKeys(), format_string);
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, 
                        RESTOREUNUSEDREGIONLIST_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001));
                        message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                        SPACE,formatted_tags+SPACE+String.valueOf(index));
                        getLogPrintStream().printLogln(message_log);
                    }
                    if (restoreUnusedRegionList(formatted_tags, index)==true){
                        deleteUnusedRegionList(formatted_tags, index);
                    }
                    message = getMessageFromRegionList();

                    try {
                        key_string = getTagHeader(getKeys());
                    } catch (Exception exception_object_key) {
                        throw exception_object_key;
                    }

                    restoreCurrentUnusedRegionList();
                    restoreRegionBuffer();
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(END_METHOD, END_METHOD,
                            SPACE, NEXTCONTENT_METHOD);
                        getLogPrintStream().printLogln(message_log);
                    }
                    message = key_string.concat(message);

                    return message;
                }
            } else {
                formatted_tags = getFormattedTags(tag_keys, format_string);
                if (formatted_tags == null) {
                    restoreCurrentUnusedRegionList();
                    restoreRegionBuffer();
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(END_METHOD, END_METHOD,
                            SPACE, NEXTCONTENT_METHOD);
                        getLogPrintStream().printLogln(message_log);
                    }
                    message = getExceptionMessage(
                        FORMAT_ERROR_EXCEPTION, NULL_MESSAGE);
                    exception_object = new Exception( message );
                    throw exception_object;
                }
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, GETREGIONLIST_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                    getLogPrintStream().printLogln(message_log);
                }
                index = 0;
                list_count = getRegionList(formatted_tags, index);
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                        SPACE, 
                        getMessageResource().getMessage(DEBUG_COUNT_KEY),
                        SPACE,
                        String.valueOf(list_count) );
                    getLogPrintStream().printLogln(message_log);
                }
                if (list_count > 0) {
                    if (restoreUnusedRegionList(formatted_tags, index) == true) {
                        deleteUnusedRegionList(formatted_tags, index);
                    }
                    message = getMessageFromRegionList();

                    try {
                        key_string = getTagHeader(getKeys());
                    } catch (Exception exception_object_key) {
                        throw exception_object_key;
                    }
                    deleteRegionList();

                    restoreCurrentUnusedRegionList();
                    restoreRegionBuffer();
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(END_METHOD, END_METHOD,
                            SPACE, NEXTCONTENT_METHOD);
                        getLogPrintStream().printLogln(message_log);
                    }
                    message = key_string.concat(message);

                    return message;
                }
            }
            restoreCurrentUnusedRegionList();
            restoreRegionBuffer();
            org_format_string = getFormattedTags(org_tags, format_string);
        }

        while (true) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                    SPACE, READREGIONLIST_METHOD,
                    getMessageResource().getMessage(DEBUG_CODE_001)); 
                getLogPrintStream().printLogln(message_log);
            }
            try {
                readRegionList(format_string, org_format_string);
            } catch (Exception standard_exception) {
                if (name_flag == true) {
                    message = standard_exception.getMessage();
                    if (message != null && message.compareTo(
                        Region.END_OF_REGION_EXCEPTION) == 0) {
                        continue;
                    }
                    if (moreUnknownRegionLists() == true) {
                        if (getDebug() > 0 && getLogPrintStream() != null) {
                            message_log = getMessage(END_METHOD, END_METHOD,
                                SPACE, NEXTCONTENT_METHOD);
                            getLogPrintStream().printLogln(message_log);
                        }

                        exception_object 
                            = new Exception( NOT_FOUND_ENTRY_EXCEPTION );
                        throw exception_object;
                    }
                }
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(END_METHOD, END_METHOD,
                        SPACE, NEXTCONTENT_METHOD);
                    getLogPrintStream().printLogln(message_log);
                }
                throw standard_exception;
            }
            if (name_flag == true) {
                if (transparent != null) {
                    format_string = getFormat();
                    formatted_tags = getFormattedTags(tag_keys, format_string);
                }
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, CHECKREGIONLIST_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                    getLogPrintStream().printLogln(message_log);
                }
                if (checkRegionList(formatted_tags)==true){
                    message = getMessageFromRegionList();

                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(END_METHOD, END_METHOD,
                            SPACE, NEXTCONTENT_METHOD);
                        getLogPrintStream().printLogln(message_log);
                    }

                    try {
                        key_string = getTagHeader(getKeys());
                    } catch (Exception exception_object_key) {
                        throw exception_object_key;
                    }

                    message = key_string.concat(message);

                    return message;
                } else {            
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                            SPACE, ADDREGIONLIST_METHOD,
                            getMessageResource().getMessage(DEBUG_CODE_001)); 
                        getLogPrintStream().printLogln(message_log);
                    }
                    target_tags = getKeys();
                    addRegionList();
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                            SPACE, SAVEUNUSEDREGIONLIST_METHOD,
                            getMessageResource().getMessage(DEBUG_CODE_001)); 
                        getLogPrintStream().printLogln(message_log);
                    }
                    saveUnusedRegionList(target_tags);
                }
            } else {
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, GETMESSAGEFROMREGIONLIST_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                    getLogPrintStream().printLogln(message_log);
                }
                message = getMessageFromRegionList();
                copy_tag_region_index = tag_region_index;

                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(END_METHOD, END_METHOD,
                        SPACE, NEXTCONTENT_METHOD);
                    getLogPrintStream().printLogln(message_log);
                }
                try {
                    key_string = getTagHeader(getKeys());
                } catch (Exception exception_object_key) {
                    throw exception_object_key;
                }
                message = key_string.concat(message);

                return message;
            }
        }
    }


    public int nextStartRegion(int tag_index) {
        int start;

        if (checkCurrentRegionObject() == false) {
            return 0;

        }
        if (getCurrentRegionObjectLength() <= tag_index) {
            return 0;
        }
        start = current_region_object[tag_index].getEndRegion()+1;
        return start;
    }

    public int nextEndRegion(int tag_index) {
        int end;
        Region    work_region;

        if (tag_index < 0 || checkCurrentRegionObject() == false) {
            return 0;
        }
        try {
            work_region = getCurrentRegionObjectEntry(tag_index);
        } catch (Exception exception_object) {
            return 0;
        }

        end = work_region.getEndRegion();
        return end;
    }

    public int getCurrentRegionObjectLength() {
        if (checkCurrentRegionObject() == false) {
            return -1;
        }
        return current_region_object.length;
    }

    public Region getCurrentRegionObjectEntry(int index) {
        if (checkCurrentRegionObject() == false 
                || getCurrentRegionObjectLength() <= index
                || index < 0) {
            return null;
        }
        return current_region_object[index];
    }

    public void setCurrentRegionObjectEntry(int index, Region param_region_object) 
    {
        Exception exception_object;

        if (checkCurrentRegionObject() == false
            || getCurrentRegionObjectLength() <= 0) {
            return ;
        }
        if (index < 0 || index >= current_region_object.length) {
            return;
        }
        current_region_object[index] = param_region_object;        
    }

    public void allocateCurrentRegionObject(int length) {
        if (length < 0) {
            return;
        }
        current_region_object =  new Region[length];
        for (int i = 0; i < getCurrentRegionObjectLength();i++) {
           current_region_object[i] = null;
        }
        return;
    }

    public Region[] getCurrentRegionObject(int length) {
        if (length < 0) {
            return null;
        }
        if (checkCurrentRegionObject() == true) {
            return current_region_object;
        }
        allocateCurrentRegionObject(length);
        return current_region_object;
    }

    public void clearRegionObject() {
        clearRegionBuffer();
        clearCurrentRegionObject();
/** **/
        clearUnusedRegion();
    }

    private String getReadRegionListExceptionMessage(String title) {
        String message;
 
        if (title == null) {
            return null;
        }
        message = new String();
        if (title.compareTo(PARAMETER_ERROR) >= 0) {
            return title;
        }
        if (title.compareTo(FORMAT_ERROR) >= 0) {
            message = LIST_FORMAT_IS_ILLEGAL;
        }

        return message;
    }

    private String extractTag(String tag_list, String tag) {
        KeyList key_list;
        String  key_string;
        
        if (tag_list == null || tag == null) {
            return tag;
        }
        try {
            key_list = KeyListFactory.createKeyList(tag_list, TAG_START_STRING,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            return null;
        }
        for (int i = 0; i < key_list.length(); i++) {
            try {
                key_string = key_list.getKeyEntry(i);
            } catch (Exception exception_obejct) {
                continue;
            }
            key_string = TAG_START_STRING.concat(key_string);
            if (compareTags(deleteKey(tag), key_string, NAME_KEY) == true) {
                return key_string;
            }
        }
        return null;
    }

    private String deleteTag(String tag_list, String tag) {
        KeyList           key_list;
        String            key_string;
        String            return_list;

        if (tag_list == null || tag == null) {
            return tag_list;
        }
        try {
            key_list = KeyListFactory.createKeyList(tag_list, TAG_START_STRING,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            return null;
        }
        return_list = new String();
        for (int i = 0; i < key_list.length(); i++) {
            try {
                key_string = key_list.getKeyEntry(i);
            } catch (Exception exception_obejct) {
                continue;
            }
            key_string = TAG_START_STRING.concat(key_string);
            if (deleteKey(key_string).compareTo(deleteKey(tag)) == 0 ) {
                continue;
            }
            return_list = return_list.concat(key_string);
        }
        return return_list;
    }

    public int readRegionList(String tag_keys, String org_tags) throws Exception {
        int      start;
        int      end;
        String   word;
        String   tag;
        String   tag_option;
        String   tag_org;
        int      tag_index;
        String[] list;
        Exception exception_object;
        int      list_count;
        String   message;
        String   message_log;
        boolean  unused_flag;
        boolean  transparent_flag = false;
        Region   operate_region;

        int number;
        Region local_region;

        if (getDebug() > 0) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                READREGIONLIST_METHOD);
            if (getLogPrintStream() == null) {
                setLogPrintStream(new LogPrintStream(System.err));
            }
            getLogPrintStream().printLogln(message_log);
        }
        if (tag_keys == null) {
            message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (org_tags == null) {
            message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                SPACE, tag_keys, SPACE, org_tags);
            getLogPrintStream().printLogln(message_log);
        }

        int               list_index;
        StringTokenizer   stk;
        KeyList           key_list;
        String            key_list_string;
        KeyList           org_key_list;
        String            org_key_list_string;

        try {
            org_key_list = KeyListFactory.createKeyList(org_tags, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        try {
            key_list = KeyListFactory.createKeyList(tag_keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }

        if (isTransparent(key_list.getKeyEntry(0)) == true) {
            transparent_flag = true;
        }

        if (checkRegionBuffer() == false) {
            clearRegionObject();
            start = 0;
            end = 0;
            unused_flag = true;
            tag_index = 0;
        } else {
            if (transparent_flag == true) {
                number = getNumberOfUnusedRegionWithTransparent(
                    key_list.getKeyEntry(key_list.length()-1));
            } else {
                number = getNumberOfUnusedRegion( 
                    key_list.getKeyEntry(key_list.length()-1));
            }
            if (number == 0) {
                if (key_list.length() == 1) {
                    clearRegionObject();
                    start = 0;
                    end = 0;
                    unused_flag = true;
                    tag_index = 0;
                } else {
                    start = nextStartRegion(key_list.length()-1);
                    end = nextEndRegion(key_list.length()-2);
                    unused_flag = false;
                    tag_index = key_list.length()-1;
                    tag_option = getTagStringWithOption(
                       getCurrentRegionObjectEntry(key_list.length()-2));

/****
                    replaceUnusedRegion(key_list.getKeyEntry(key_list.length()-2), tag_option);
****/
                }
            } else {
                tag_index = key_list.length()-1;
                local_region = getUnusedRegion(
                    key_list.getKeyEntry(key_list.length()-1), number-1);
                start = nextStartRegion(key_list.length()-1);
                end = local_region.getEndRegion();
                unused_flag = true;
            }
        }

        operate_region = new Region();
        operate_region.setDebug( getDebug() );
/*
        operate_region.setMessageResource( getMessageResource() );
        operate_region.setDebug( getDebug() );
        operate_region.setLogPrintStream( getLogPrintStream() );
*/

        if (checkCurrentRegionObject() == false) {
            allocateCurrentRegionObject(key_list.length());
        }
        for (; tag_index < key_list.length(); ) {
            if (isTransparent(key_list.getKeyEntry(tag_index)) == false) {
                word = key_list.getKeyEntry(tag_index);
                if (key_list.getKeyEntry(tag_index).compareTo(WILDCARD) != 0) {
                    word = WILDCARD;
                }
            } else {
                word = WILDCARD;
            }
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, 
                        Region.FINDREGION_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                getLogPrintStream().printLogln(message_log);
            }
            try {
                current_region_object[tag_index] = operate_region.findRegion(
                    head, start, end, word, NAME_KEY);
                unused_flag = false;
            } catch (Exception exception_object_work) {
                if (unused_flag == true) {
                    try {
                        key_list_string = key_list.getKeyEntry(tag_index);
                    } catch (Exception real_exception_object) {
                        throw real_exception_object;
                    }
                    if (transparent_flag == true) {
                        number = getNumberOfUnusedRegionWithTransparent(
                            key_list_string ); 
                    } else {
                        number = getNumberOfUnusedRegion( key_list_string ); 
                    }
                    if (number == 1) {
                        deleteUnusedRegion(key_list_string, number-1);
                        if (tag_index == 0) {
                            start = 0;
                            end = 0;
                            clearRegionObject();
                            allocateCurrentRegionObject(key_list.length());
                            continue;
                        }
                        start = end + 1;
                        end = nextEndRegion(tag_index-1);
                        unused_flag = false;
                        continue;
                    } 
                    if (number > 1) {
                        local_region = getUnusedRegion(
                            key_list_string, number-1);
                        start = local_region.getEndRegion()+1;
                        local_region = getUnusedRegion(
                            key_list_string, number-2);
                        end = local_region.getEndRegion();
                        deleteUnusedRegion(
                            key_list_string, number-1);
                        continue;
                    }
                    throw exception_object_work;
                }
                tag_option = getTagStringWithOption(current_region_object[tag_index-1]);
                number = getNumberOfUnusedRegion(
                    current_region_object[tag_index-1].getTagString() );
                if (number >= 1) {
                    current_region_object[tag_index] = null;

                    tag_index--;

                    local_region = getUnusedRegion(
                        current_region_object[tag_index-1].getTagString(),
                        number-1);

                    try {
                        key_list_string = key_list.getKeyEntry(tag_index);
                    } catch (Exception real_exception_object) {
                        throw real_exception_object;
                    }
                    start = nextStartRegion(tag_index);
                    end = local_region.getEndPosition();
                    unused_flag = true;

                    if (transparent_flag == true) {
                        if (tag_index+1 < key_list.length()) {
                            String next_key_list_string;
                            try {
                                tag_org = key_list.getKeyEntry(tag_index);
                                next_key_list_string = key_list.getKeyEntry(
                                tag_index+1);
                                tag_org = tag_org + next_key_list_string;
                                for (int i = tag_index; i < key_list.length(); i++) {
                                    key_list.setKeyEntry(i, tag_org);
                                }
                                tag_org = org_key_list.getKeyEntry(tag_index);
                                next_key_list_string = org_key_list.getKeyEntry(
                                    tag_index+1);
                                tag_org = tag_org + next_key_list_string;
                                for (int i = tag_index; i < org_key_list.length(); i++) {
                                    org_key_list.setKeyEntry(i, tag_org);
                                }
                            } catch (Exception real_exception_object) {
                                throw real_exception_object;
                            }
                        }
                    }

                    continue;
                }
                if (tag_index > 0) {
                    current_region_object[tag_index] = null;
                    tag_index--;

                    if (transparent_flag == true) {
                        if (tag_index+1 < key_list.length()) {
                            String next_key_list_string;
                            try {
                                tag_org = key_list.getKeyEntry(tag_index);
                                next_key_list_string = key_list.getKeyEntry(
                                tag_index+1);
                                tag_org = tag_org + next_key_list_string;
                                for (int i = tag_index; i < key_list.length(); i++) {
                                    key_list.setKeyEntry(i, tag_org);
                                }
                                tag_org = org_key_list.getKeyEntry(tag_index);
                                next_key_list_string = org_key_list.getKeyEntry(
                                tag_index+1);
                                tag_org = tag_org + next_key_list_string;
                                for (int i = tag_index; i < org_key_list.length(); i++) {
                                    org_key_list.setKeyEntry(i, tag_org);
                                }
    
                            } catch (Exception real_exception_object) {
                                throw real_exception_object;
                            }
                        }
                    }
                    if (tag_index == 0) {
                        start = 0;
                        end = 0;
                        clearRegionObject();
                        allocateCurrentRegionObject(key_list.length());
                        unused_flag = true;
                        continue;
                    }

                    start = nextStartRegion(tag_index);
                    end = nextEndRegion(tag_index-1);

                    continue;
                }
                throw exception_object_work;
            }

            start = current_region_object[tag_index].getStartPosition();
            end = current_region_object[tag_index].getEndPosition();
            try {
                key_list_string = key_list.getKeyEntry(tag_index);
            } catch (Exception real_exception_object) {
                throw real_exception_object;
            }

            if (isTransparent(key_list_string) == false) {
                if (word.compareTo(WILDCARD) == 0) {
                    tag = current_region_object[tag_index].getTagString();
                    tag_option = getTagStringWithOption(current_region_object[tag_index]) ;
                    if (getDebug() > 0 && getLogPrintStream() != null) {
                        message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                            SPACE, 
                            tag,
                            SPACE,
                            tag_option);
                        getLogPrintStream().printLogln(message_log);
                    }
                    try {
                        key_list_string = key_list.getKeyEntry(tag_index);
                        org_key_list_string = org_key_list.getKeyEntry(tag_index);
                    } catch (Exception real_exception_object) {
                        throw real_exception_object;
                    }

                    if (compareTags(key_list_string, tag, NAME_KEY) == false
                    && compareTags(org_key_list_string, tag_option, NAME_KEY) == false) {
                        number = getNumberOfUnusedRegion(tag); 
                        if (number == 0) {
                            start = current_region_object[tag_index].getTagOptionEnd();
                            end = current_region_object[tag_index].getEndRegion();
                        } else {
                            local_region = getUnusedRegion(tag, number-1);
                            start = current_region_object[tag_index].getEndRegion()+1;
                            end = local_region.getEndRegion();
                        }
                        addUnusedRegion(current_region_object[tag_index], tag_index, 
                            key_list_string);
                        unused_flag = true;
                        continue;
                    } else {
                        unused_flag = false;
                    }
                    tag_index++;
                }
            } else {
                tag = current_region_object[tag_index].getTagString();
                tag_option = getTagStringWithOption(current_region_object[tag_index]) ;
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                        SPACE, 
                        tag,
                        SPACE,
                        tag_option);
                    getLogPrintStream().printLogln(message_log);
                }
                try {
                    key_list_string = key_list.getKeyEntry(tag_index);
                    org_key_list_string = org_key_list.getKeyEntry(tag_index);
                } catch (Exception real_exception_object) {
                    throw real_exception_object;
                }

                 if (matchTransparentTag(key_list_string, null, tag) == true
                 && matchTransparentTag(org_key_list_string, tag_option, null) == true) {

                    String   org_tag;
                    org_tag = key_list_string;
                    String   extra_tag;
                    String   org_extra_tag;

                    try {
                        extra_tag = extractTag(key_list_string, tag);
                        key_list.setKeyEntry(tag_index, extra_tag);
                        org_extra_tag = extractTag(org_key_list_string, tag_option);
                        org_key_list.setKeyEntry(tag_index, org_extra_tag);
                    } catch (Exception real_exception_object) {
                        throw real_exception_object;
                    }
                    for (int i = tag_index+1; i < key_list.length(); i++) {
                        tag_org = key_list.getKeyEntry(i);
                        tag_org = deleteTag(tag_org, tag_option);
                        key_list.setKeyEntry(i, tag_org);
                        tag_org = org_key_list.getKeyEntry(i);
                        tag_org = deleteTag(tag_org, tag_option);
                        org_key_list.setKeyEntry(i, tag_org);
                    }
                    tag_index++;
                    unused_flag = false;
                } else {
                    number = getNumberOfUnusedRegion(tag); 

                    if (number == 0) {
                        start = current_region_object[tag_index].getTagOptionEnd();
                        end = current_region_object[tag_index].getEndRegion();
                    } else {
                        local_region = getUnusedRegion(tag, number-1);
                        start = current_region_object[tag_index].getEndRegion()+1;
                        end = local_region.getEndRegion();
                    }
                    addUnusedRegion(current_region_object[tag_index], tag_index, 
                            key_list_string);
                    unused_flag = true;
                    continue;
                }
            }
        }
        read_region_box = current_region_object;
        read_region_index = getCurrentRegionObjectLength();

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(END_METHOD, END_METHOD,
                SPACE, READREGIONLIST_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        return read_region_index;
    }

    public String getContents(String tag_keys) throws Exception {
        String    tag;
        String    tag_org;
        String    word;
        String[]  list;
        String[]  order;
        int tag_index; 
        int start;
        int end;
        int loop;
        String buffer_string;
        boolean exception_flag;
        byte[]   buffer;
        String   message;
        String   message_log;
        Region   operate_region;


        if (tag_keys == null) {
             message = getExceptionMessage(
                ARGUMENT_PARAMETER_EXCEPTION );
            Exception exception_object = new Exception(message);
            throw exception_object;
        }
        operate_region = new Region();

        int               list_index;
        StringTokenizer   stk;
        KeyList           key_list;
        String            key_list_string;

        try {
            key_list = KeyListFactory.createKeyList(tag_keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            throw exception_object;
        }
        list_index = 0;
        stk = new StringTokenizer(tag_keys, COMMA);
        key_list_string = null;
        try {
            key_list_string = key_list.getKeyEntry(0);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        word = key_list_string;
        loop = 0;
        exception_flag = false;

        if (current_region_object == null) {
            head = new byte[1][][];
            head[0] = null;
            current_region_object = new Region[key_list.length()];
            start = 0;
            end = 0;
            tag_index = 0;
            exception_flag = true;
        } else {
            if (getCurrentRegionObjectLength() > 1) {
                start = current_region_object[getCurrentRegionObjectLength()-1].getEndRegion()+1;
                end = current_region_object[getCurrentRegionObjectLength()-2].getEndRegion();
            } else {
                start = 0;
                end = 0;
            }
            tag_index = getCurrentRegionObjectLength()-1;
        }

        for (; tag_index < key_list.length(); ) {
            try {
                key_list_string = key_list.getKeyEntry(tag_index);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (isTransparent(key_list_string) == false) {
                word = key_list_string;
            } else {
                word = WILDCARD;
            }
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, 
                        Region.FINDREGION_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                getLogPrintStream().printLogln(message_log);
            }
            try {
                current_region_object[tag_index] = operate_region.findRegion(
                    head, start, end, word, NAME_KEY);
            } catch (Exception exception_object) {
                if (tag_index > 0) {
                    setCurrentRegionObjectEntry(tag_index, null);
                    tag_index--;
                    if (tag_index == 0) {
                        start = 0;
                        end = 0;
                    } else {
                        start = current_region_object[tag_index].getEndRegion()+1;
                        end = current_region_object[tag_index-1].getEndRegion();
                    }
                    continue;
                } 
                if (exception_flag == true) {
                    clearCurrentRegionObject();
                }
                throw exception_object;
            }
            start = current_region_object[tag_index].getStartPosition();
            end = current_region_object[tag_index].getEndPosition();
            if (isTransparent(key_list_string) == false) {
                tag_index++;
            } else {
                tag = current_region_object[tag_index].getTagString();
                if (matchTransparentTag(key_list_string, tag, null) == true) {
                    try {
                        key_list.setKeyEntry(tag_index, tag);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                    for (int i = tag_index+1; i < key_list.length(); i++) {
                        try {
                            key_list_string = key_list.getKeyEntry(tag_index);
                        } catch (Exception exception_object) {
                            throw exception_object;
                        }
                        tag_org = key_list_string;
                        tag_org = tag_org.replace(tag, EMPTY);
                        try {
                            key_list.setKeyEntry(i, tag_org);
                        } catch (Exception exception_object) {
                            throw exception_object;
                        }
                    }
                    tag_index++;
                } else {
                    loop = loop + 1;
                }
            }
        }

        if (current_region_object == null || current_region_object[key_list.length()-1] == null) {
            message = getMessage(REGION_ERROR_EXCEPTION,
                REGION_ERROR_EXCEPTION);
            Exception exception_object = new Exception(message);
            throw exception_object;
        }
        if (current_region_object[key_list.length()-1].getStartPosition() <
            current_region_object[key_list.length()-1].getEndPosition() ) {
            try {
                buffer = current_region_object[key_list.length()-1].extractRegion(head[0], 
                    current_region_object[key_list.length()-1].getStartPosition(),
                    current_region_object[key_list.length()-1].getEndPosition()
                );
                buffer_string = new String(buffer);
                return buffer_string;
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return new String(EMPTY);
    }

    public void setFormat(String format_key_string) {
        format_keys = format_key_string;
    }

    public String getFormat() throws Exception {
        String     word;
        String     keys;
        String     return_key;
        int        space_index;

        if (format_keys != null) {
            return format_keys;
        }
        keys = getKeys();
        if (keys == null) { return keys; }
        int               list_index;
        StringTokenizer   stk;
        KeyList           key_list;
        String            key_list_string;

        try {
            key_list = KeyListFactory.createKeyList(keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            throw exception_object;
        }

        for (int i = 0; i < key_list.length(); i++) {
            try {
                word = key_list.getKeyEntry(i);
                key_list.setKeyEntry(i, deleteKey(word));
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return_key = new String();
        for (int i = 0; i < key_list.length(); i++) {
            try {
                key_list_string = key_list.getKeyEntry(i);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (i != 0) {
                return_key = return_key.concat(COMMA+key_list_string);
            } else {
                return_key = return_key.concat(key_list_string);
            }
        }
        setFormat(return_key);
        return return_key;
    }

    public String getFileTag(String value) {
        String  tag_string;

        if (value == null || value.length() == 0) {
            tag_string = new String(
                TAG_START_STRING + FILE_TAG + TAG_END_STRING
            );
        
            return tag_string;
        }

        tag_string = new String(
            TAG_START_STRING +
            FILE_TAG + SPACE + NAME_KEY + EQUAL + value +
            TAG_END_STRING
        );

        return tag_string;
    }

    public final static String TERM_SEPARATOR = "-";


    public static boolean checkDate(String term) {
        if (term == null) {
            return false;
        }
        for (int i = 0; i < term.length(); i++) {
            if ((term.charAt(i) < NUMERIC_ZERO || term.charAt(i) > NUMERIC_NINE)
                && term.charAt(i) != DATE_SEPARATOR_CHAR 
                && term.charAt(i) != TERM_SEPARATOR_CHAR) {
                return false;
            }
        }
        return true;
    }

    public static Date getStartDate(String term) {
        Date   start_date;
        int    term_index;
        String start_string;

        if (term == null) {
            start_date = new Date();
            return start_date;
        }
        term_index = term.indexOf(TERM_SEPARATOR);
        if (term_index >= 0) {
            start_string = term.substring(0, term_index);
        } else {
            start_string = term;
        }
        start_date = new Date(start_string);
        return start_date;
    }

    public static Date getEndDate(String term) {
        Date   end_date;
        int    term_index;
        String end_string;

        if (term == null) {
            end_date = new Date();
            return end_date;
        }
        term_index = term.indexOf(TERM_SEPARATOR);
        if (term_index >= 0) {
            end_string = term.substring(term_index+1);
        } else {
            end_string = term;
        }
        end_date = new Date(end_string);
        return end_date;
    }

    public static long getTime(Date one_day) {
        long  time_value;
        if (one_day == null) {
            time_value = ((new Date()).getTime() / DATE_TIME) * DATE_TIME;
        } else {
            time_value = (one_day.getTime() / DATE_TIME) * DATE_TIME;
        }
        return time_value;
    }

    private String date_string = null;

    private void setDateString(String param_string) {
        date_string = param_string;
    }

    private String getDateString() {
        return date_string;
    }

    public static String getDateSuffix(Date date) {
        int  year_value;
        int  month_value;
        int  date_value;
        String year_string;
        String month_string;
        String date_string;
        String date_suffix;

        if (date == null) {
            date = new Date();
        }
        year_value = date.getYear();
        month_value = date.getMonth();
        date_value = date.getDate();

        year_string = String.valueOf(year_value+1900);
        month_string = String.valueOf((month_value+1) / 10) +
                       String.valueOf((month_value+1) % 10);
        date_string = String.valueOf(date_value / 10) +
                       String.valueOf(date_value % 10);
        date_suffix = new String(year_string + month_string + date_string);
        return date_suffix;
    }

    public static long getMonth(Date one_day) {
        long  month_value;
        
        if (one_day == null) {
            month_value = (new Date()).getYear() * MONTH_YEAR + one_day.getMonth()+1;
        } else {
            month_value = one_day.getYear() * MONTH_YEAR + one_day.getMonth()+1;
        }
        return month_value;
    }

    public static Date getDateFromMonth(long current_month) {
        long  year_value;
        long  month_value;
        long  date_value;
        Date  date_object;

        year_value = current_month / MONTH_YEAR;
        month_value = (current_month - 1)%MONTH_YEAR;
        date_value = 1;

        date_object = new Date();
        date_object.setYear((int)year_value);
        date_object.setMonth((int)month_value);
        date_object.setDate((int)date_value);

        return date_object;
    }

    public static String getUsage(String[] parameter) {
        String usage_string;

        usage_string = USAGE_STRING;
        return usage_string;
    }

    public String printFilePromptMessage(String[] ag, String work_message) {
        String message;
        String[]  message_list;

        if (ag == null || work_message == null) {
            return null;
        }
        message_list = new String[ag.length+1];
        message_list[0] = work_message;
        for (int i = 0; i < ag.length; i++) {
            message_list[i+1] = ag[i];
        }
        message = getMessage( PROMPT_INPUT_FILE, PROMPT_INPUT_FILE );
        return message;
    }


    public int printStartTags(int tab_count, String value_keys, PrintStream print_stream) 
    {
        String            work_string;
        int               list_index;
        StringTokenizer   stk;
        String[]          value_list;
        KeyList           key_list;
        String            key_list_string;

        if (value_keys == null || print_stream == null) {
            return tab_count;
        }

        try {
            key_list = KeyListFactory.createKeyList(value_keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            return -1;
        }

        for (int k = 0; k < key_list.length(); k++) {
            try {
                key_list_string = key_list.getKeyEntry(k);
            } catch (Exception exception_object) {
                return -1;
            }
            tab_count = printStartTag( print_stream, tab_count, key_list_string);
        }
        return tab_count;
    }

    public int printEndTags(int tab_count, String value_keys, PrintStream print_stream) 
    {
        String            work_string;
        int               list_index;
        StringTokenizer   stk;
        String[]          value_list;
        KeyList           key_list;
        String            key_list_string;

        if (value_keys == null || print_stream == null) {
            return tab_count;
        }

        try {
            key_list = KeyListFactory.createKeyList(value_keys, COMMA,
                    getMessageResource(),
                    getDebug(),
                    getLogPrintStream());
        } catch (Exception exception_object) {
            return -1;
        }

        for (int k = 0; k < key_list.length(); k++) {
            try {
                key_list_string = key_list.getKeyEntry(key_list.length()-k-1);
            } catch (Exception exception_object) {
                return -1;
            }
            tab_count = printEndTag( print_stream, tab_count, key_list_string);
        }
        return tab_count;
    }

    private String getExceptionMessage(String message, String... org_args) {
        String return_message;
        String[] args;

        args = new String[org_args.length+1];
       
        args[0] = getMessageResource().getMessage( message );
        for (int i = 0; i < org_args.length; i++) {
            args[i+1] = org_args[i];
        }

        return_message = getMessageResource().getFormattedMessage(
            message, 
            args);
        return return_message;
    }

    private InputStream current_input_stream = null;
    public InputStream getCurrentInputStream() {
        return current_input_stream;
    }

    public void setCurrentInputStream(InputStream ip) {
        current_input_stream = ip;
    }


    public void getRegions(Object[] obj_args, PrintStream print_stream) 
        throws Exception {
        String          message;
        String          tags;
        int             tab_count;
        int             new_count;
        ReportRegion[]  merge_object;
        String   body;
        int      length;
        int      message_index;
        boolean  start_flag;
        String   value_keys;
        String   unknown_tags; 
        KeyList  unknown_key_list;
        String   unknown_key_list_string;
        String   formatted_tags;

        int      length_index;
        int      length_int;
        int      unknown_index;
        int      unknown_int;
        String   length_string;
        String   unknown_string;
        String   remain;

        if (obj_args == null || obj_args.length < 2) {
            Exception  exception_object;
            message = getMessage(
                REPORT_REGION_PARAMETER_EXCEPTION,
                REPORT_REGION_PARAMETER_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (print_stream == null) {
            Exception  exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (obj_args[0] == null || obj_args[1] == null) {
            Exception  exception_object;
            message = getMessage(
                REPORT_REGION_PARAMETER_EXCEPTION,
                REPORT_REGION_PARAMETER_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        tags = (String)obj_args[0];

        merge_object = new ReportRegion[obj_args.length-1];

        for (int i = 1; i < obj_args.length; i++) {
            merge_object[i-1] = new ReportRegion();
            if (getDebug() > 0) {
                merge_object[i-1].setDebug( getDebug() );
                merge_object[i-1].setMessageResource( getMessageResource() );
                merge_object[i-1].setLogPrintStream( getLogPrintStream() );
            }
            if ((obj_args[i] instanceof InputStream) == false) {
                merge_object[i-1].setCurrentInputStream(null);
                continue;
            }
            merge_object[i-1].setCurrentInputStream(
                (InputStream)obj_args[i]);
        }

        tab_count = 0;
        new_count = 0;

        while (true) {
            try {
                message = nextRegion(tags, merge_object);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (message == null) {
                break;
            }
            body = new String();
            start_flag = false;
            value_keys  = null;
            unknown_tags = null;

            message_index = message.indexOf(SPACE);
            if (message_index <= 0) {
                break;
            }
            length_index = message.indexOf(SPACE);
            if (length_index >= 0) {
                length_string = message.substring(0, length_index);
                length_int = Integer.valueOf(length_string).intValue();
                if (value_keys == null) {
                    value_keys = message.substring(length_index+1, 
                        length_int+length_index+1);
                    formatted_tags = getFormattedTags(value_keys, tags);
                    value_keys = formatted_tags;
                }
                remain = message.substring(length_int+1+length_index+1);
                unknown_index = remain.indexOf(SPACE);
                if (unknown_index >= 0) {
                   unknown_string = remain.substring(0, unknown_index);
                   unknown_int = Integer.valueOf(unknown_string).intValue(); 
                   unknown_tags = remain.substring(unknown_index+1,
                       unknown_int+unknown_index+1);
                   message = remain.substring(unknown_int+unknown_index+1);
                }
            }
            if (start_flag == false) {
                new_count = printStartTags(tab_count, value_keys, print_stream);
                if (new_count != -1) {
                    tab_count = new_count;
                }
                start_flag = true;
            }
            print_stream.println(message);
            if (start_flag == true) {
                tab_count = printEndTags(tab_count, value_keys, print_stream);
            }
        }
    }


    public String nextRegion(
        String specified_tag, ReportRegion[] target) throws Exception {
        int           out_file;
        int           last_file;
        String        tags;
        String        keys;
        String        value_keys;
        String        tag_keys;
        String        key_header;
        String        message;
        String        message_log;
        int           tab_count;
        int           new_count;
        String        formatted_tags;

        String        unknown_tags;
        KeyList       unknown_key_list;
        String        unknown_key_list_string;
        DisplayHierarchy   display_hierarchy_object;

        InputStream  input_stream;
        out_file = -1;
        last_file = target.length -1;

        if (specified_tag == null || target == null) {
            Exception    exception_object;
            message = getMessage(
                REPORT_REGION_PARAMETER_EXCEPTION,
                REPORT_REGION_PARAMETER_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }

        display_hierarchy_object = new DisplayHierarchy();
        display_hierarchy_object.setMessageResource( getMessageResource() );
        display_hierarchy_object.setDebug( getDebug() );
        display_hierarchy_object.setLogPrintStream( getLogPrintStream() );

        for (int i = 0; i < target.length; i++) {
            if (target[i] != null) {
                out_file = i; 
                break;
            }
        }
        if (out_file == -1) {
            return null;
        }
        for (int i = target.length - 1; i>= 0; i--) {
            if (target[i] != null) {
                last_file = i;
                break;
            }
        }

        ByteArrayOutputStream   byte_array_output_stream;
        PrintStream             print_stream_object;
        String   unknown_message;
        String   output_message;
        String   message_length_string;
        output_message = new String();

        tags = specified_tag;
        keys = null;
        tag_keys = tags;

        tab_count = getKeyNumber(tags);

        value_keys = null;
        unknown_tags = new String();

        for (int i = 0; i < target.length; i++) {
            if (target[i] == null) {
                if (out_file == i) {
                    for (int j = out_file+1; j < target.length; j++) {
                        if (target[j] != null) {
                            out_file = j;
                        }
                    }
                    if (out_file == i) {
                       out_file = target.length;
                       break;
                    }
                }
                if (i == last_file) {
                   for (int j = target.length-1; j >= out_file; j--) {
                       if (target[j] != null) {
                           last_file = j;
                       }
                   }
                   if (last_file == i) {
                       out_file = target.length;
                       break;
                   }
                }
                continue;
            }
            if ((input_stream = target[i].getCurrentInputStream())!=null){
                System.setIn(input_stream);
            } else {
                continue;
            }

            unknown_tags = new String();
            try {
                message = target[i].nextContent(tag_keys, tags);
                if (message == null) {
                    AssertionError assertion_error_object;
                    assertion_error_object 
                            = new AssertionError(message == null);
                    throw assertion_error_object;
                }
                unknown_message = message;

                int      length_index;
                int      length_int;
                int      unknown_index;
                int      unknown_int;
                String   length_string;
                String   unknown_string;
                String   remain;

                length_index = message.indexOf(SPACE);
                if (length_index >= 0) {
                    length_string = message.substring(0, length_index);
                    length_int = Integer.valueOf(length_string).intValue();

                    if (value_keys == null) {
                        value_keys = message.substring(length_index+1, 
                        length_int+length_index+1);
                        formatted_tags = getFormattedTags(value_keys, tags);
                        value_keys = formatted_tags;
                        tag_keys = value_keys;
                    }
                    remain = message.substring(length_int+1+length_index+1);
                    unknown_index = remain.indexOf(SPACE);
                    if (unknown_index >= 0) {
                        unknown_string = remain.substring(0, unknown_index);
                        unknown_int = Integer.valueOf(unknown_string).intValue(); 
                        unknown_tags = 
                            remain.substring(unknown_index+1,
                            unknown_int+unknown_index+1);
                        message = remain.substring(unknown_int+unknown_index+1);
                    }
                }
                unknown_message = display_hierarchy_object.getIndentMessage(
                    message.getBytes(), tab_count);
            } catch (Exception real_exception_object) {
                if (last_file == i && value_keys != null) {
                    int               list_index;
                    StringTokenizer   stk;
                    KeyList           key_list;
                    String            key_list_string;
                }
                message = real_exception_object.getMessage();
                if (message == null || NOT_FOUND_ENTRY_EXCEPTION.compareTo(
                    message) != 0) {
                    target[i] = null;
                    if (last_file == i) {
                        for (int j = target.length-1; j >= out_file; j--) {
                            if (target[j] != null) {
                                last_file = j;
                            }
                        }
                        if (last_file == i) {
                            out_file = target.length;
                            break;
                        }
                    }
                    if (out_file == i) {
                        for (int j = out_file+1; j < target.length; j++) {
                            if (target[j] != null) {
                                out_file = j;
                            }
                        }
                        if (out_file == i) {
                           out_file = target.length;
                           break;
                        }
                    }
                }
                continue;
            }
            output_message = output_message.concat(unknown_message);
        }
        if (output_message == null || value_keys == null) {
            return null;
        }
        try {
            key_header = formatTagHeader(value_keys, unknown_tags);
        } catch (Exception exception_object_key) {
            throw exception_object_key;
        }
        key_header = key_header.concat(SPACE+output_message);
        output_message = key_header;
        return output_message;
    }

    public void makeReport(Object[] obj_args, PrintStream print_stream) throws Exception {
        makeReportBody(obj_args, print_stream);
    }

    public String makeReportBody(Object[] obj_args, PrintStream print_stream) throws Exception {
        String        keys;
        String        tags;
/**
        InputStream[] istream;
**/
        InputStream   backup;
/**
        ReportRegion[]  merge_object;
**/
        Region[]      bk_region_object;
        String        tag_keys;
        int           out_file;
        int           last_file;
        boolean       start_flag;
        String        message;
        String        message_log;
        String        value_keys;
        String[]      value_list;
        String        work_string;
        String        date_tag;
        int           space_index;
        int           dot_index;
        int           tab_count;
        int           new_count;
        Date          start_date;
        Date          end_date;
        Date          current;
        long          start_time;
        long          end_time;
        long          current_time;
        long          start_month;
        long          end_month;
        long          current_month;
        long          value;
        String        file_name;
        String[]      args;
        int           arg_index;
        int           list_count;
        boolean       individual_flag;
        boolean       check_flag;
        Exception     exception_object;
        String        format_string;
        String        formatted_tags;
        boolean       retry_flag;

        String        unknown_tags;
        KeyList       unknown_key_list;
        String        unknown_key_list_string;

        String        print_message;
        String        print_header;

        unknown_tags = null;

        if (getDebug() > 0) {
            message_log = getMessage(ENTER_METHOD, ENTER_METHOD,
                SPACE, REPORT_METHOD);
            if (getLogPrintStream() == null) {
                setLogPrintStream( new LogPrintStream(System.err) );
            }
            getLogPrintStream().printLogln(message_log);
        }
        if (obj_args == null || obj_args.length == 0 || 
            obj_args[0] == null || (obj_args[0] instanceof String) == false 
            || ((String)obj_args[0]).length() == 0) {
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(END_METHOD, END_METHOD,
                    SPACE, REPORT_METHOD);
                getLogPrintStream().printLogln(message_log);
            }
            message = getMessage(
                REPORT_REGION_PARAMETER_EXCEPTION,
                REPORT_REGION_PARAMETER_EXCEPTION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        date_tag = DATE_TAG;
        tags = (String)obj_args[0];

        print_message = new String();
        check_flag = false;
        if (merge_object == null) {
            merge_object = new ReportRegion[obj_args.length-1];
            for (int i = 1; i < obj_args.length; i++) {
                merge_object[i-1] = new ReportRegion();
                if (getDebug() > 0) {
                    merge_object[i-1].setDebug( getDebug() );
                    merge_object[i-1].setMessageResource( getMessageResource() );
                    merge_object[i-1].setLogPrintStream( getLogPrintStream() );
                }
            }
        }
        if (istream == null) {
            istream = new InputStream[obj_args.length-1];

            for (int i = 1; i < obj_args.length; i++) {
                if ((obj_args[i] instanceof InputStream) == false) {
                    istream[i-1] = null;
                    continue;
                }
                istream[i-1] = (InputStream)obj_args[i];
                check_flag = true;
            }
        }
        tab_count = 0;

        individual_flag = false;
/**
        if (checkDate(getDateString()) == true) {
            check_flag = true;
        } else {
            individual_flag = true;
        }
        if (check_flag == false) {
            String work_message = new String();

            if (getDebug() > 0 && getLogPrintStream() != null) {
                message_log = getMessage(END_METHOD, END_METHOD,
                    SPACE, REPORT_METHOD);
                getLogPrintStream().printLogln(message_log);
            }
            message = getMessage (INVALID_FILE_ERROR_EXCEPTION,
                INVALID_FILE_ERROR_EXCEPTION );
            exception_object = new Exception( message );
            throw exception_object;
        }
**/

        backup = System.in;
        out_file = 0;
        last_file = merge_object.length -1;
        tab_count = 0;
        start_flag = false;
        retry_flag = false;
        while (true) {
            keys = null;
            tag_keys = tags;

            value_keys = null;

            for (int i = 0; i < merge_object.length; i++) {
                if (istream[i] == null) { 
                } else {
                    System.setIn(istream[i]);
                }
                if (merge_object[i] == null) {
                    if (out_file == i) {
                        for (int j = out_file+1; j < merge_object.length; j++) {
                            if (merge_object[j] != null) {
                                out_file = j;
                            }
                        }
                        if (out_file == i) {
                           out_file = merge_object.length;
                           break;
                        }
                    }
                    if (i == last_file) {
                       for (int j = merge_object.length-1; j >= out_file; j--) {
                           if (merge_object[j] != null) {
                               last_file = j;
                           }
                       }
                       if (last_file == i) {
                           out_file = merge_object.length;
                           break;
                       }
                    }
                    continue;
                }
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                        SPACE, NEXTCONTENT_METHOD,
                        getMessageResource().getMessage(DEBUG_CODE_001)); 
                 
                    getLogPrintStream().printLogln(message_log);
                    message_log = getMessage(DEBUG_TAG_KEY, DEBUG_TAG_KEY,
                        SPACE, tag_keys);
                    getLogPrintStream().printLogln(message_log);
                    message_log = getMessage(DEBUG_TAG_KEY, DEBUG_TAG_KEY,
                        SPACE, tags);
                    getLogPrintStream().printLogln(message_log);
                }
                try {
                    message = merge_object[i].nextContent(tag_keys, tags);

                    int      length_index;
                    int      length_int;
                    int      unknown_index;
                    int      unknown_int;
                    String   length_string;
                    String   unknown_string;
                    String   remain;

                    length_index = message.indexOf(SPACE);
                    if (length_index >= 0) {
                        length_string = message.substring(0, length_index);
                        length_int = Integer.valueOf(length_string).intValue();

                        if (value_keys == null) {
                            value_keys = message.substring(length_index+1, 
                            length_int+length_index+1);
                            tag_keys = value_keys;
                            formatted_tags = getFormattedTags(value_keys, tags);
                            value_keys = formatted_tags;
                        }
                        remain = message.substring(length_int+1+length_index+1);
                        unknown_index = remain.indexOf(SPACE);
                        if (unknown_index >= 0) {
                            unknown_string = remain.substring(0, unknown_index);
                            unknown_int = Integer.valueOf(unknown_string).intValue(); 
                            unknown_tags = remain.substring(unknown_index+1,
                                unknown_int+unknown_index+1);
                            message = remain.substring(unknown_int+unknown_index+1);
                        }
                    }
                } catch (Exception real_exception_object) {
                    if (last_file == i && value_keys != null) {

                        if (getDebug() > 0 && getLogPrintStream() != null) {
                            message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                            SPACE, 
                            PRINTENDTAGS_METHOD,
                            getMessageResource().getMessage(DEBUG_CODE_001)); 
                            getLogPrintStream().printLogln(message_log);
                            message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                            SPACE,
                            value_keys );
                            getLogPrintStream().printLogln(message_log);
                        }

                        if (print_stream == null) {
                            print_header = getTagHeader(value_keys);
                            print_header = print_header.concat(print_message);
                            return print_header;
                        } 
                        tab_count = printEndTags(tab_count, value_keys, print_stream);
                        retry_flag = false;
                    }
                    message = real_exception_object.getMessage();
                    if (message == null || NOT_FOUND_ENTRY_EXCEPTION.compareTo(
                        message) != 0) {
                        merge_object[i] = null;
                        if (last_file == i) {
                            for (int j = merge_object.length-1; j >= out_file; j--) {
                                if (merge_object[j] != null) {
                                    last_file = j;
                                }
                            }
                            if (last_file == i) {
                                out_file = merge_object.length;
                                break;
                            }
                        }
                        if (out_file == i) {
                            for (int j = out_file+1; j < merge_object.length; j++) {
                                if (merge_object[j] != null) {
                                    out_file = j;
                                }
                            }
                            if (out_file == i) {
                               out_file = merge_object.length;
                               break;
                            }
                        }
                    }
                    if (message == null || NOT_FOUND_ENTRY_EXCEPTION.compareTo(message) != 0) {
                        i--;
                    }
                    continue;
                }

                int               list_index;
                StringTokenizer   stk;
                KeyList           key_list;
                String            key_list_string;

                try {
                    key_list = KeyListFactory.createKeyList(value_keys, COMMA,
                        getMessageResource(),
                        getDebug(),
                        getLogPrintStream());
                } catch (Exception exception_object_key) {
                    throw exception_object_key;
                }
                if (retry_flag != true) {
                    if (i == out_file) {
                        try {
                            key_list_string = key_list.getKeyEntry(0);
                        } catch (Exception exception_object_key) {
                            throw exception_object_key;
                        }
                        if (date_tag == null) {
                            if (key_list_string.charAt(2) >= UPPER_A &&
                                key_list_string.charAt(2) <= UPPER_Z) {
                                date_tag = DATE_TAG.toUpperCase();
                            } else {
                                date_tag = DATE_TAG;
                            }
                        }
                        if (getDebug() > 0 && getLogPrintStream() != null) {
                            message_log = getMessage(DEBUG_CALL_KEY, DEBUG_CALL_KEY,
                            SPACE, 
                            PRINTSTARTTAGS_METHOD,
                            getMessageResource().getMessage(DEBUG_CODE_001)); 
                            getLogPrintStream().printLogln(message_log);
                            message_log = getMessage(DEBUG_VALUE_KEY, DEBUG_VALUE_KEY,
                            SPACE,
                            value_keys );
                            getLogPrintStream().printLogln(message_log);
                        }
                        if (print_stream != null) {
                            new_count = printStartTags(tab_count, value_keys, print_stream);
                            if (new_count != -1) {
                                tab_count = new_count;
                            }
                        }
                        retry_flag = true;
                    }
                }
                if (unknown_tags != null && unknown_tags.length() > 0) {
                    try {
                        unknown_key_list = KeyListFactory.createKeyList(
                            unknown_tags, COMMA,
                            getMessageResource(),
                            getDebug(),
                            getLogPrintStream());
                    } catch (Exception exception_object_key) {
                        throw exception_object_key;
                    }
                    for (int k = 0; k < unknown_key_list.length(); k++) {
                        try {
                            unknown_key_list_string 
                                = unknown_key_list.getKeyEntry(k);
                        } catch (Exception exception_object_key) {
                            throw exception_object_key;
                        }
                        if (print_stream == null) {
                            new_count = tab_count + 1;
                        } else {
                            new_count = printStartTag(print_stream, 
                                tab_count, unknown_key_list_string);
                        }
                        if (new_count != -1) {
                            tab_count = new_count;
                        }
                    }
                }
                if (getDateString() != null) {
                    work_string = getDateString();
                    dot_index = work_string.indexOf(DOT);
                    if (dot_index > 0) {
                        work_string = work_string.substring(dot_index+1);
                    }
                } else {
                    work_string = getDateString();
                }
/**
                if (individual_flag == false) {
	            if (checkDate(work_string) == true) {
                        if (print_stream != null) {
                            new_count = printStartTag(print_stream,
                               tab_count, getStartTag(date_tag, work_string));
                            if (new_count != -1) {
                                tab_count = new_count;
                            }
                        }
                    }
                }
**/
                if (print_stream == null) {
                    print_message = print_message.concat(message);
                } else {
                    print_stream.println(message);
                }
/**
                if (individual_flag == false) {
	            if (checkDate(work_string) == true) {
                        if (print_stream != null) {
                            new_count = printEndTag(print_stream,
                                tab_count, getStartTag(date_tag));
                            if (new_count != -1) {
                                tab_count = new_count;
                            }
                        }
                    }
                }
**/
                if (unknown_tags != null && unknown_tags.length() > 0) {
                    try {
                        unknown_key_list = KeyListFactory.createKeyList(
                            unknown_tags, COMMA,
                            getMessageResource(),
                            getDebug(),
                            getLogPrintStream());
                    } catch (Exception exception_object_key) {
                        throw exception_object_key;
                    }
                    for (int k = unknown_key_list.length()-1; k >= 0; k--) {
                        try {
                            unknown_key_list_string 
                                = unknown_key_list.getKeyEntry(k);
                        } catch (Exception exception_object_key) {
                            throw exception_object_key;
                        }
                        if (print_stream == null) {
                            new_count = tab_count - 1;
                        } else {
                            new_count = printEndTag(print_stream, 
                                tab_count, unknown_key_list_string);
                        }
                        if (new_count != -1) {
                            tab_count = new_count;
                        }
                    }
                }
                i--;
            }
            start_flag = true;
            if (merge_object.length == out_file) {
                break;
            }
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(END_METHOD, END_METHOD,
                SPACE, REPORT_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        return null;
    }

    public void execute(String[] org_args) throws Exception {
        PrintStream      print_stream;

        print_stream = System.out;
        try {
            super.execute(org_args, print_stream, (MakeReport)this);
        } catch (Exception real_exception_object) {
            throw real_exception_object; 
        }
    }

    public static String TEST_FILE = "Takahiko.20110101";

    public static void main(String[] org_args) {
        String            lang;
        String            message;
        String[]          arg_string;
        PrintStream       save_output;
        PrintStream       stdout_stream;
        ReportRegion      report_region;
        FileOutputStream  file_output_stream;
        PrintStream       print_stream;
        String            debug_string;
        int               debug_value;

        report_region = new ReportRegion(); 

        debug_string = System.getProperty(EXEC_REGION_DEBUG);
        if (debug_string == null || debug_string.length() == 0) {
            debug_string = System.getProperty(DEBUG_LEVEL_KEY);
        }
        if (debug_string != null && debug_string.length() > 0) {
            report_region.setDebug(debug_string);
        }
        report_region.setLogPrintStream(new LogPrintStream(System.err));

        save_output = System.out;
        try {
            report_region.execute(org_args);
        } catch (Exception real_exception_object) {
            System.err.println(real_exception_object.getMessage());
            System.exit(1);
        }
    }
}
