/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ReportSort.java
 * @Author     Takahiko Tanaka.
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/10/02
 *
 */

package exec;

import java.lang.*;
import java.io.*;

import dict.*;

public class ReportSort extends ReportRegion {
    private static String dictionary_file = "dictionary";
    private static dict.JapaneseDictionary dictionary_object = null;

    private static String ISORDER_METHOD = "isOrder";

    public  boolean isOrder(String s1, String s2) throws Exception {
        int         rt;
        String[]    s1_tags;
        String[]    s2_tags;
        TagString   s1_tag_string;
        TagString   s2_tag_string;
        String      s1_key;
        String      s2_key;
        String      s1_option;
        String      s2_option;
        String      message_log;
        String      message;

        if (s1 == null || s2 == null) {
            Exception  exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_log = getMessage(
                ENTER_METHOD, ENTER_METHOD,
                SPACE, 
                ISORDER_METHOD);
            getLogPrintStream().printLogln(message_log);
        }
        try {
            s1_tags = (String[])OperateReport.getTags(s1);
        } catch (Exception exception_object) {
            return true;
        }

        try {
            s2_tags = (String[])OperateReport.getTags(s2);
        } catch (Exception exception_object) {
            Exception   real_exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            real_exception_object = new Exception(message);
            throw real_exception_object;
/**
            return false;
**/
        }

        if (dictionary_object == null) {
            try {
                dictionary_object = JapaneseDictionary.valueOf(dictionary_file, true);
            } catch (Exception exception_object) {
                throw exception_object;
/**
                return false;
**/
            }
        }

        for (int i = 0; i < s1_tags.length; i++) {
            try {
                s1_tag_string = new TagString(s1_tags[i]);
                s1_key = s1_tag_string.getKey();
                s1_option = s1_tag_string.getOptionValue();
                s2_tag_string = new TagString(s2_tags[i]);
                s2_key = s2_tag_string.getKey();
                s2_option = s2_tag_string.getOptionValue();
            } catch (Exception exception_object) {
                Exception  real_exception_object;

                message = getMessage(
                    ARGUMENT_PARAMETER_EXCEPTION,
                    ARGUMENT_PARAMETER_EXCEPTION);
 
                real_exception_object = new Exception(message);
                throw real_exception_object;
/**
                return true;
**/
            } 
            try {
                rt = dictionary_object.compareKanji(s1_key, s2_key);
                if (rt < 0) {
                    return true;
                } else if (rt > 0) {
                    return false;
                }
                rt = dictionary_object.compareKanji(s1_option, s2_option);
                if (rt < 0) {
                    return true;
                } else if (rt > 0) {
                    return false;
                }
            } catch (Exception exception_object) {
                throw exception_object;
/**
                return false;
**/
            }
        }
        return false; 
    }

    public int[] right(int[] index, int p, IndexCard report_card) 
        throws Exception {

        int   j;
        boolean flag = false;
        int[]  new_index = null;
        String    message;

        if (index == null) {
            Exception  exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (report_card == null) {
            Exception  exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            exception_object = new Exception(message);
            throw exception_object;
        }

        for (int i = p; i < index.length - 1; i++) {
            try {
                if (isOrder(
                    ((ReportEntry)report_card.get(index[i])).getTagString(),
                  ((ReportEntry)report_card.get(index[i+1])).getTagString()) 
                        == false) {
                    j = index[i];
                    index[i] = index[i+1];
                    index[i+1] = j;
                    new_index = left(index, i, report_card);
                    if (new_index != null) {
                        index = new_index;
                    }
                    flag = true;
                }
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        if (flag == true) {
            return index;
        }
        return null;
    }

    public int[] left(int[] index, int p, IndexCard report_card) 
        throws Exception {

        int   j;
        boolean flag = false;
        int[]  new_index = null;
        String message;

        if (index == null) {
            Exception  exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (report_card == null) {
            Exception  exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            exception_object = new Exception(message);
            throw exception_object;
        }

        for (int i = p; i > 0; i--) {
            try {
                if (isOrder(
                ((ReportEntry)report_card.get(index[i])).getTagString(),
                  ((ReportEntry)report_card.get(index[i-1])).getTagString()) 
                 == true) {
                    j = index[i];
                    index[i] = index[i-1];
                    index[i-1] = j;
                    new_index = right(index, i, report_card);
                    if (new_index != null) {
                        index = new_index;
                    }
                }
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        if (flag == true) {
            return index;
        }
        return null;
    }

    public IndexCard doSort(IndexCard report_card) throws Exception {
        int[]  index_array;
        int[]  new_index_array;
        IndexCard   new_report_card;
        String      message;

        if (report_card == null) {
            Exception  exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
 
            exception_object = new Exception(message);
            throw exception_object;
        }

        index_array = new int[report_card.size()];
        for (int i = 0; i < index_array.length; i++) {
            index_array[i] = i;
        }
        try {
            new_index_array = left(index_array, report_card.size()/2, report_card);
            if (new_index_array == null) {
                new_index_array = right(index_array, report_card.size()/2, report_card);
                if (new_index_array != null) {
                    index_array = new_index_array;
                }
            }
        } catch (Exception exception_object) {
            throw exception_object;
        }

        new_report_card = new IndexCard();
        for (int i = 0; i < report_card.size(); i++) {
            new_report_card.add(report_card.get(index_array[i]));
        }

        return new_report_card;
    }

}
