/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    SearchRegion.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.nejp
 * @Version:   1.0 2011/09/13
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class SearchRegion extends RegionObject implements MakeReport {

    private static String SEARCH_REGION = "SearchRegion";
    private static String MAKEREPORT = "makeReport";
    private static String EXECUTE = "execute";

    private String word_string = null;

    public void setWord(String word) {

        word_string = word;
    }

    public String getWord() {
        return word_string;
    }

    public void makeReport(Object[] args, PrintStream print_stream) throws Exception {
        String    word;
        String    tags;
        String    keys;
        String                   lang;
        String                   keys1;
        String                   keys2;
        String                   message;
        String                   msg;
        KeyList                  word_list;
        KeyList                  key_list;
        String                   key_entry;
        StringTokenizer          stk;
        int                      region_index;
        SearchString             search_string_object;
        ByteArrayOutputStream    byte_array_output_stream;
        ReportRegion             report_region;
        byte[]                   report_byte;
        boolean                  check;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, MAKEREPORT);
            getLogPrintStream().printLogln( message );
        }
        if (args == null || print_stream == null) {
            Exception exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION, 
                ARGUMENT_PARAMETER_EXCEPTION );
            exception_object = new Exception(message);
            throw exception_object;
        }
        lang = System.getenv(LANG_KEY);
        check = false;
        for (int i = 1; i < args.length; i++) {
           if ((args[i] instanceof InputStream) == false) {
               continue;
           }
           check = true;
        }
        if (args.length <= 1 || check == false) {
            Exception exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION, 
                ARGUMENT_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                BLANK_IN_BLANK), SEARCH_REGION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (print_stream == null) {
            Exception exception_object;

            message = getMessage( STREAM_PARAMETER_EXCEPTION, 
                STREAM_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                BLANK_IN_BLANK), SEARCH_REGION);
            exception_object = new Exception(message);
            throw exception_object;
        }
        tags = (String)args[0];
        word = (String)getWord();

        if (tags == null) {
            Exception exception_object;
            message = getMessage( TAG_PARAMETER_EXCEPTION, 
                TAG_PARAMETER_EXCEPTION );
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (word == null) {
            Exception exception_object;
            message = getMessage( WORD_PARAMETER_EXCEPTION, 
                WORD_PARAMETER_EXCEPTION );
            exception_object = new Exception(message);
            throw exception_object;
        }
        report_byte = null;
        try {
            word_list = KeyListFactory.createKeyList(word, SPACE,
                getMessageResource(),
                getDebug(),
                getLogPrintStream() );
        } catch (Exception exception_object) {
            throw exception_object;
        }

        search_string_object = new SearchString();
        for (int i = 1; i < args.length; i++) {
            if (args[i] == null ||
               (args[i] instanceof InputStream) == false) {
                continue;    
            }
            try {
                System.setIn((InputStream)args[i]);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            report_region = new ReportRegion();
            report_region.clearRegionObject();
            while (true) {
                try {
                    message = report_region.nextContent(tags, tags);
                } catch (Exception exception_object) {
                    if (exception_object.getMessage() != null && 
                        exception_object.getMessage().compareTo(
                        Region.END_OF_FILE_EXCEPTION) == 0) {
                        break;
                    }
                    report_region.clearRegionObject();
                    throw exception_object;
                }
                if (message == null) {
                    AssertionError assertion_error_object;
                    assertion_error_object 
                            = new AssertionError(message == null);
                    throw assertion_error_object;
                }
                String unknown_message = message;

                int      length_index;
                int      length_int;
                int      unknown_index;
                int      unknown_int;
                String   length_string;
                String   unknown_string;
                String   remain;

                keys1 = EMPTY;
                keys2 = null;
                length_index = message.indexOf(SPACE);
                if (length_index >= 0) {
                    length_string = message.substring(0, length_index);
                    length_int = Integer.valueOf(length_string).intValue();

                    keys1 = message.substring(length_index+1, 
                        length_int+length_index+1);
                    remain = message.substring(length_int+1+length_index+1);
                    unknown_index = remain.indexOf(SPACE);
                    if (unknown_index >= 0) {
                        unknown_string = remain.substring(0, unknown_index);
                        unknown_int = Integer.valueOf(unknown_string).intValue(); 
                        keys2 = remain.substring(unknown_index+1,
                            unknown_int+unknown_index+1);
                        message = remain.substring(unknown_int+unknown_index+1);
                    }
                }
/*
                unknown_message = display_hierarchy_object.getIndentMessage(
                    message.getBytes(), tab_count);
*/
                search_string_object.setMessage(message);
                search_string_object.setKey(word);
                search_string_object.setLimit(word_list.length());
                msg = search_string_object.searchKey();;
                if (msg == null) {
                     continue;
                }
/**
                keys1 = report_region.getKeys() ;
                keys2 = report_region.getUnusedTagStrings();
**/
                if (keys2 != null && keys2.length() > 0) {
                    keys = keys1 + COMMA + keys2;
                } else {
                    keys = keys1;
                }
                try {
                    key_list = KeyListFactory.createKeyList(keys, COMMA,
                        getMessageResource(),
                        getDebug(),
                        getLogPrintStream() );
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                for (int j = 0; j < key_list.length(); j++) {
                    try {
                        key_entry = key_list.getKeyEntry(j);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                    for (int k = 0; k < j; k++) {
                        print_stream.print(TAB_SPACE);
                    }
                    print_stream.println(key_entry);
                }
                print_stream.println(msg);
                for (int j = key_list.length() - 1; j >= 0; j--) {
                    try {
                        key_entry = key_list.getKeyEntry(j);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                    for (int k = 0; k < j; k++) {
                        print_stream.print(TAB_SPACE);
                    }
                    print_stream.println(convertFromStartToEnd(key_entry));
                }
            }
        }
    }


    public void execute(String[] org_args) throws Exception {
        String           word;
        String           message;
        String[]         args;
        PrintStream      print_stream;

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(ENTER_METHOD, ENTER_METHOD, SPACE, EXECUTE);
            getLogPrintStream().printLogln( message );
        }

        args = new String[org_args.length-1];
        word = org_args[0];
        setWord(word);
        for (int i = 1; i < org_args.length; i++) {
            args[i-1] = org_args[i];
        }
 
        print_stream = System.out;
        try {
            super.execute(args, print_stream, (MakeReport)this);
        } catch (Exception real_exception_object) {
            throw real_exception_object; 
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = getMessage(END_METHOD, END_METHOD, SPACE, EXECUTE);
            getLogPrintStream().printLogln( message );
        }
    }

    public static void main(String[] args) {
        String              message;
        String              word;
        String[]            argument;
        SearchRegion        search_region;
        PrintStream         print_stream;
        FileOutputStream    file_output_stream;
        String              debug_string;

        search_region = new SearchRegion();
        debug_string = System.getProperty(EXEC_REGION_DEBUG);
        if (debug_string == null || debug_string.length() == 0) {
            debug_string = System.getProperty(DEBUG_LEVEL_KEY);
        }
        if (debug_string != null && debug_string.length() > 0) {
            search_region.setDebug(debug_string);
        }
        search_region.setLogPrintStream( new LogPrintStream( System.err ) );

        if (args.length <= 1) {
            message = search_region.getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION,
                search_region.getMessageResource().getMessage(
                    BLANK_IN_BLANK), SEARCH_REGION);
            System.out.println(message);
            return;
        }
        word = args[0];
        search_region.setWord(word);
        argument = new String[args.length-1];
        for (int i = 1; i < args.length; i++) {
            argument[i-1] = args[i];
        }
        try {
            print_stream = System.out;
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = search_region.getMessage( IO_ERROR_EXCEPTION, 
                    IO_ERROR_EXCEPTION,
                    search_region.getMessageResource().getMessage(
                    BLANK_IN_BLANK), SEARCH_REGION);
            }
            System.exit(1);
            return;
        }
        try {
            search_region.execute(argument);
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = search_region.getMessage( INTERNAL_ERROR_EXCEPTION, 
                    INTERNAL_ERROR_EXCEPTION,
                    search_region.getMessageResource().getMessage(
                    BLANK_IN_BLANK), SEARCH_REGION);
            }
            System.err.println(message);
            System.exit(1);
        }
    }
}
