/**
 * Copyright (C) All rights reserved.
 *
 * @Author     Takahiko Tanaka
 * @Module     SearchString.java
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2011/09/20
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;

public class SearchString {

    private String    message = null;
    private String    key = null;
    private String[]  key_words = null;
    private int[][]   key_relation;
    private int[][]   key_reset;

    private int[][]   key_start_position;

    private int       limit;
    private int       hit_number;
    private String    KEY_START = "<B>";
    private String    KEY_END = "</B>";

    public static String SPACE = " ";
    public static String COMMA = ",";

    public void setReset(int master, int slave) {
        setSlaveArray(master, slave, key_reset);
    }

    public void setRelation(int master, int slave) {
        setSlaveArray(master, slave, key_relation);
    }

    public void setSlaveArray(int master, int slave, int[][] slave_array) {
        int[] key_work1;
        int[] key_work2;

        key_work1 = slave_array[master];
        if (key_work1 != null || key_work1.length > 0) {
            key_work2 = new int[key_work1.length+1];
            for (int k = 0; k < slave_array[master].length; k++) {
                key_work2[k] = key_work1[k];
            }
        } else {
            key_work1 = new int[0];
            key_work2 = new int[key_work1.length+1];
        }
        key_work2[key_work2.length-1] = slave;
        slave_array[master] = key_work2;
    }

    public void setMessage(String message_parameter) {
        message = message_parameter;
    }

    private void setHit(int number) {
        hit_number = number;
    }

    public int getHit() {
        return hit_number;
    }

    public int getPositionCount() {
        int    count;
        int[]  key_work;

        count = 0;
        for (int i = 0; i < key_start_position.length; i++) {
            key_work = key_start_position[i];
            count = count + key_work.length;
        }
        return count;
    }

    public int getLastPosition() {
        int        position;
        int[]      key_work;

        position = 0;
        for (int i = 0; i < key_start_position.length; i++) {
            key_work = key_start_position[i];
            if (key_work == null || key_work.length == 0) {
                continue;
            }
            if (key_work[key_work.length-1] + key_words[i].length()
                 > position) {
                position = key_work[key_work.length-1] 
                         + key_words[i].length();
            }
        }
        return position;
    }

    public void setKey(String key_parameter) {
        StringTokenizer string_tokenizer;
        String[] new_key_words;
        int key_word_index;
        int count;
        String work_string;
        int master;
        int slave;

        key = key_parameter;
        string_tokenizer = new StringTokenizer(key, SPACE);
        limit = string_tokenizer.countTokens();
        key_words = new String[limit];
        for (int i = 0; i < key_words.length; i++) {
            key_words[i] = string_tokenizer.nextToken();
        } 
        count = 0;
        for (int i = 0; i < key_words.length; i++) {
            if (key_words[i] == null) {
                continue;
            }
            for (int j = i + 1; j < key_words.length; j++) {
                if (key_words[j] == null) {
                    continue;
                }
                if (key_words[i].compareTo(key_words[j]) == 0) {
                    key_words[j] = null;
                    count++;
                    continue;
                }
            }
        }
        new_key_words = new String[key_words.length-count];
        key_word_index = 0;
        for (int i = 0; i < key_words.length; i++) {
            if (key_words[i] != null) {
                new_key_words[key_word_index] = key_words[i];
                key_word_index++;
            }
        }
        key_words = new_key_words;

        for (int i = 0; i < key_words.length; i++) {
            for (int j = i+1; j < key_words.length; j++) {
                if (key_words[i].length() > key_words[j].length()) {
                    work_string = key_words[i];
                    key_words[i] = key_words[j];
                    key_words[j] = work_string;
                }
            }
        } 

        key_relation = new int[key_words.length][];
        key_reset = new int[key_words.length][];
        key_start_position = new int[key_words.length][];
        for (int i = 0; i < key_relation.length; i++) {
            key_reset[i]          = new int[0];
            key_relation[i]       = new int[0];
            key_start_position[i] = new int[0];
        }

        int k;
        for (int i = 0; i < key_words.length-1; i++) {
            for (int j = i+1; j < key_words.length; j++) {
                if (key_words[i].length() > key_words[j].length()) {
                    master = i;
                    slave = j;
                } else {
                    master = j;
                    slave = i;
                }
                if (key_words[master].indexOf(key_words[slave]) >= 0) {
                    setRelation(master, slave);
                } else {
                    for (k = 1; k < key_words[slave].length(); k++) {
                        work_string = key_words[slave].substring(k);
                        if (key_words[slave].indexOf(work_string) == 0) {
                            setRelation(master, slave);
                            break;
                        }
                    }
                    if (k >= key_words[slave].length()) {
                        for (k = key_words[slave].length()-1; k > 0; k--) {
                            work_string = key_words[master].substring(
					 key_words[master].length()-k);
                            if (key_words[slave].indexOf(work_string) == 0) {
                                setReset(master, slave);
                                break;
                            }
                        }
	            }
                }
            }
        }
    }

    public void setLimit(int limit_parameter) {
        limit = limit_parameter;
    }

    public String searchKey() {
        int[]   key_index;
        int[]   key_start;
        int[]   key_work;
        int[]   key_work2;
        int     work_count;
        int     work_index;
        String  return_message;
        String  work_string;
        int     count;
        int     length;
        int i;
        int j;
        int k;
        int l;

        if (message == null || key == null || key_words == null) {
            if (message == null) {
                return null;
            }
            if (key == null) {
                return null;
            }
            return null;
        }

        length = message.length();
        return_message = new String();
        key_index = new int[key_words.length];
        key_start = new int[key_words.length];
        for (i = 0; i < key_index.length; i++) {
            key_index[i] = 0;
            key_start[i] = 0;
        }


        for (i = 0; i < length; i++) {
            for (j = 0; j < key_words.length; j++) {
                if (key_words[j].charAt(key_index[j]) == message.charAt(i)) {
                    if (key_index[j] == 0) {
                        key_start[j] = i;
                    }
                    key_index[j] = key_index[j] + 1;
                    if (key_index[j] == key_words[j].length()) {
                        key_work = key_relation[j];
                        for (k = 0; k < key_work.length; k++) {
                            key_work2 = key_start_position[key_work[k]];
                            work_count = 0;
                            for (l = key_work2.length - 1; l >= 0; l--) {
                                if (key_work2[l] + 
                                    key_words[key_work[k]].length() 
                                    >= key_start[j]) {
                                    work_count++;
                                }
                            }
                            if (work_count == 0) {
                               continue;
                            }
                            int[]  bk_work;
                            bk_work = new int[key_work2.length-work_count];
                            for (l = 0; l < bk_work.length; l++) {
                                bk_work[l] = key_work2[l];
                            }
                            key_start_position[key_work[k]] = bk_work;
                            work_count = getPositionCount()*(
                                KEY_START.length()+KEY_END.length())
                                +getLastPosition();
                            return_message 
                                = return_message.substring(0, work_count);
                        }
                        work_index = getLastPosition();

                        work_string = message.substring(work_index,
                                i+1-key_words[j].length());

                        return_message = return_message.concat(
                                work_string);

                        work_string = message.substring(
                                i+1-key_words[j].length(), i+1);

                        return_message = return_message.concat(
                                KEY_START + work_string + KEY_END);

                        key_work = key_start_position[j];
                        if (key_work == null || key_work.length == 0) {
                            key_work = new int[1];
                        } else {
                            int[] bk;
                            bk = new int[key_work.length+1];
                            for (l = 0; l < key_work.length; l++) {
                                bk[l] = key_work[l];
                            }
                            key_work = bk;
                        }
                        key_work[key_work.length-1]=key_start[j];
                        key_start_position[j] = key_work;

                        key_index[j] = 0;
                        key_start[j] = i+1;
                        key_work = key_reset[j];
                        for (k = 0; k < key_work.length; k++) {
                            key_index[key_work[k]] = 0;
                            key_start[key_work[k]] = i+1;
                        }
                    }
                } else {
                    for (k = key_index[j]-1; k >= 0; k--) {
                        if (key_words[j].charAt(k) 
                                    == message.charAt(i)) {
                            for (l = k-1; l >= 0; l--) {
                                if (key_words[j].charAt(key_index[l]) 
                                    != message.charAt(i-k+l)) {
                                    break;
                                }
                            }
                            if (l >= 0) {
                                continue;
                            }
                            break;
                        }
                    }
                    key_index[j] = k+1;
                    key_start[j] = i-(k+1);
                }
            }
        }
        count = 0;
        for (i = 0; i < key_index.length; i++) {
            key_work = key_start_position[i];
            if (key_work != null && key_work.length > 0) {
                count++;
            }
        }
        work_index = getLastPosition();
        if (work_index < message.length() -1) {
            return_message = return_message.concat(message.substring(work_index));
        }
        setHit(count);
        if (count < limit) {
            return null;
        }
        return return_message;
    }
}
