/**
 * Copyright (C) All rights reserved.
 *
 * @Module     SortReportTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/10/03
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;

public class SortReportTest extends DebugObject {
    private OperateReport  operate_report = null;
    public static String ARTICLE_FAILS = "Article fails.";
    public static String REPORT_FAILS = "Report fails.";

    public SortReportTest(String[] files,
        MessageResource  message_resource,
        int              debug,
        LogPrintStream   log_print_stream) throws Exception {

        try {
            operate_report = OperateReport.valueOf(files,
                message_resource,
                debug,
                log_print_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public SortReportTest(String[] files) throws Exception {
        try {
            operate_report = OperateReport.valueOf(files);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void execute(String[] args) throws Exception {
        String       message = "This is a test message.";
        int          no;
        byte[]       area;
        ArticlePost  article_post;
        ArticleEntry article_entry;
        IndexCard    report_card;
        int          article_index;

        try {
            article_post = operate_report.getArticles(args[0]);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            report_card = operate_report.getReport(article_post);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        ReportSort report_sort;
        try {
            report_sort = new ReportSort();
            report_card = report_sort.doSort(report_card);
        } catch (Exception exception_object) {
            return;
        }
        try {
            operate_report.printReport(report_card, article_post);
        } catch (Exception exception_object) {
            return;
        }
    }

    private static int DEBUG_DEBUG = 1;

    public static void main(String[] args) {
        int                debug;
        String[]           files;
        String             message;
        MessageResource    message_resource;
        LogPrintStream     log_print_stream;
        SortReportTest     sort_report_test;

        debug = DEBUG_DEBUG;

        message_resource = new MessageResource();
        try {
            message_resource.setLanguage(Locale.JAPAN.getISO3Language());
        } catch (Exception exception_object) {
            System.err.println( exception_object.getMessage() );
            return;
        }

        if (args.length <= 1) {
            message = message_resource.getFormattedMessage(
                    ARGUMENT_PARAMETER_EXCEPTION,
                    message_resource.getMessage(
                        ARGUMENT_PARAMETER_EXCEPTION));
            System.err.println( message );
            return;
        }
        files = new String[args.length-1];
        for (int i = 0; i < files.length; i++) {
            files[i] = args[i+1];
        }

        log_print_stream = new LogPrintStream( System.err );
        try {
            sort_report_test = new SortReportTest(files,
                message_resource,
                debug,
                log_print_stream);
        } catch (Exception exception_object) {
            return;
        }

        try {
            sort_report_test.execute(args);
        } catch (Exception exception_object) {
            return;
        }
    }
}
