/**
 * Copyright (C) All rights reserved.
 *
 * @Module     StoreHouse.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 */

package exec;

import java.lang.*;
import java.io.*;
import java.util.*;

public class StoreHouse extends DebugObject {

    private static String PRINTCONTENTS = "printContents";
    public  final static String MEMORY_STORAGE = "Memory";
    public  final static String FILE_STORAGE = "File";
    private  String  storage_type = FILE_STORAGE;

    private  byte[]  memory_area = new byte[0];
    private  int     memory_seek_pointer = -1;

    private  String  memory_name = "memory";
    private  String  file_name = null;
    private  File    fd = null;
    private  static  int     length;
    private  static  int     BLOCK_SIZE = 8192;
    private  static  boolean book_flag = false;
    private  static  boolean page_flag = false;
    private  static  boolean active_flag = false;
    private  FileInputStream   file_input_stream = null;

    private  static  byte previous_byte = 0;
    private  static  boolean end_flag = true;

/**
    private static String PARAMETER_ERROR = "Parameter error";
    private static String FORMAT_ERROR = "Format error";
    private static String ASSERTION_ERROR = "Assertion error";
    private static String ILLEGAL_METHOD_ERROR = "Illegal method error";
**/

    private static char START_CHAR = '<';
    private static char END_CHAR = '>';
    private static char SLASH_CHAR = '/';
    private static char BACK_SLASH_CHAR = '\\';

    private static byte START_BYTE = '<';
    private static byte END_BYTE = '>';
    private static byte SLASH_BYTE = '/';
    private static byte BACK_SLASH_BYTE = '\\';

    private static byte NEW_LINE_CHAR = '\n';
    private static byte RETURN_CHAR = '\r';
    private static byte TAB_CHAR = '\t';
    private static byte SPACE_CHAR = ' ';


    protected StoreHouse() {
    }

    private StoreHouse(byte[] a) throws Exception {
        storage_type = MEMORY_STORAGE;
        file_name = memory_name;
        memory_area = a;
        memory_seek_pointer = 0;
        setPageStart(0);
        setPageEnd(a.length-1);
    }

    private StoreHouse(String  filename) throws Exception {
        storage_type = FILE_STORAGE;
        file_name = filename;
        try {
            fd = new File(filename);
            if (fd.exists() == false) {
                String       message;
                Exception    exception_object;
                message = getMessage(
                    ARGUMENT_PARAMETER_EXCEPTION,
                    ARGUMENT_PARAMETER_EXCEPTION);
                exception_object = new Exception( message );
                throw exception_object;
            }
            active_flag = true;
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (file_input_stream == null) {
            try {
                file_input_stream = new FileInputStream(fd);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
    }

    public String getFilename() {
        return file_name;
    }

    public static StoreHouse valueOf(byte[] a,
        MessageResource param_message_resource,
        int             debug,
        LogPrintStream  log_print_stream) throws Exception {
        StoreHouse   store_house;
        MessageResource  message_resource;

        message_resource = param_message_resource;
        if (message_resource == null) {
            message_resource = new MessageResource();
        }

        if (a == null) {
            String       message;
            Exception    exception_object;
            message = message_resource.getFormattedMessage(
                ARGUMENT_PARAMETER_EXCEPTION, 
                    message_resource.getMessage(
                    ARGUMENT_PARAMETER_EXCEPTION));
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            store_house = new StoreHouse(a);
            store_house.setMessageResource( message_resource );
            store_house.setDebug( debug );
            store_house.setLogPrintStream( log_print_stream );
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return store_house;
    }

    public static StoreHouse valueOf(byte[] a) throws Exception {
        StoreHouse   store_house;

        if (a == null) {
            String       message;
            Exception    exception_object;

            message = ARGUMENT_PARAMETER_EXCEPTION;
            exception_object = new Exception( message );
            throw exception_object;
        }
        try {
            store_house = new StoreHouse(a);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return store_house;
    }

    public static StoreHouse valueOf(String filename,
        MessageResource message_resource,
        int             debug,
        LogPrintStream  log_print_stream) throws Exception {
        StoreHouse   store_house;

        try {
            store_house = new StoreHouse(filename);
            store_house.setMessageResource( message_resource );
            store_house.setDebug( debug );
            store_house.setLogPrintStream( log_print_stream );
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return store_house;
    }

    public static StoreHouse valueOf(String filename) throws Exception {
        StoreHouse   store_house;

        try {
            store_house = new StoreHouse(filename);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return store_house;
    }

    private static byte previous_data = 0;

    public static byte getPrevious() {
        return previous_data;
    }

    public static boolean getEndFlag() {
        return end_flag;
    }

    public void disableEndFlag() {
        end_flag = false;
    }

    public void enableEndFlag() {
        end_flag = true;
    }

    public int length() throws Exception {
        if (fd.exists() == false) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ASSERTION_EXCEPTION, ASSERTION_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        return (int)fd.length();
    }

    public static boolean moreEntry() {
        return active_flag;
    } 

    public static boolean isBook() {
        return book_flag;
    }

    public static boolean isPage() {
        return page_flag;
    }

    private static int page_number = 0;

    public static void clearPage() {
        page_number = 0;
    }

    public static void incPage() {
        page_number = page_number + 1;
    }

    public static int getPage() {
        return page_number;
    }

    public static int getPosition(int offset) {
        int position;
        position = getPage() * getPageSize() + offset;
        return position;
    }
  
    private static int page_size = BLOCK_SIZE;

    public static int getPageSize() {
        return page_size;
    }
    public static void setPageSize(int  buffer_page_size) {
        page_size = buffer_page_size;
    }

    private int start_tag_start = 0;
    private int start_tag_end = 0;
    private int end_tag_start = 0;
    private int end_tag_end = 0;

    public void setStartTagStart(int position) {
        start_tag_start = position;
    }
    public int getStartTagStart() {
        return start_tag_start;
    }
    public void setStartTagEnd(int position) {
        start_tag_end = position;
    }
    public int getStartTagEnd() {
        return start_tag_end;
    }
    public void setEndTagStart(int position) {
        end_tag_start = position;
    }
    public int getEndTagStart() {
        return end_tag_start;
    }
    public void setEndTagEnd(int position) {
        end_tag_end = position;
    }
    public int getEndTagEnd() {
        return end_tag_end;
    }

    public AreaRange getStartTag() {
        AreaRange area_range;
        area_range = new AreaRange();
        area_range.setStartPosition(start_tag_start);
        area_range.setEndPosition(start_tag_end);

        return area_range;
    }

    public AreaRange getEndTag() {
        AreaRange area_range;
        area_range = new AreaRange();
        area_range.setStartPosition(end_tag_start);
        area_range.setEndPosition(end_tag_end);

        return area_range;
    }

    private  int page_start = 0;
    private  int page_end = 0;

    public  void setPageStart(int position) {
        page_start = position;
    }
    public int getPageStart() {
        return page_start;
    }
    public void setPageEnd(int position) {
        page_end = position;
    }
    public int getPageEnd() {
        return page_end;
    }

    private static boolean valid_page_flag = false;

    private static boolean checkPageFlag() {
        return valid_page_flag;
    }

    private static void enablePageFlag() {
        valid_page_flag = true;
    }

    private static void disablePageFlag() {
        valid_page_flag = false;
    }

    private static boolean tag_start_flag = false;
    private static boolean checkTagFlag() {
        return tag_start_flag ;
    }
    private static void enableTagFlag() {
        tag_start_flag = true;
    }
    private static void disableTagFlag() {
        tag_start_flag = false;
    }

    /**
      rid of return and blank
      in front of <
    */
    private int     offset = 0;
    public static byte[] buffer = new byte[BLOCK_SIZE];
    private    int       position = 0;
    private byte[] last = { 0 };

    private void change_page() {
        last[1] = buffer[offset];
        offset = 0;
    }

    public byte[] getLast() {
        return last;
    }

    private byte[] hold_tag_area = new byte[0];

    private void holdTag(byte[] tag, int offset, int tag_length) {
        byte[]  local_tag_area;

        if (tag.length < offset + tag_length) {
            return;
        }
        local_tag_area = new byte[hold_tag_area.length+tag_length];
        for (int i = 0; i < hold_tag_area.length;i++) {
            local_tag_area[i] = hold_tag_area[i];
        }
        for (int i = 0; i < tag_length;i++) {
            local_tag_area[hold_tag_area.length+i] = tag[offset+i];
        }
        hold_tag_area = local_tag_area;
    }

    private void clearTag() {
        hold_tag_area = new byte[0];
    }

    private byte[] getTagArea() {
        return hold_tag_area;
    }

    private int getPageFromPosition(int param_position) {
        int   page_no;

        page_no = param_position / BLOCK_SIZE;
        return page_no;
    }

    private int getOffset(int  position) {
        int   page_offset;

        page_offset = position % BLOCK_SIZE;
        return page_offset;
    }

    private boolean skipByte(byte byte_code) {
        
        if (byte_code != NEW_LINE_CHAR && byte_code != RETURN_CHAR && 
                        byte_code != SPACE_CHAR && byte_code != TAB_CHAR) {
            return false;
        }
        return true;
    }

    private int seekPointer(byte[] check_buffer, int offset, int limit) { 
        int  position;

        if (offset < 0 || offset >= check_buffer.length || offset > limit) {
            return 0;
        }
        for (position = offset; position <= limit; position++) {
            if (skipByte(check_buffer[position]) == false) {
                break;
            }
        }
        return position;
    }

    public BaseEntry readEntry() throws Exception {
        int       rt;
        int       length;
        boolean   start_flag;
        boolean   escape_flag;
        PageEntry page_entry;
        BookEntry book_entry;
        AreaRange start_tag;
        AreaRange end_tag;
        byte[]    tag_buffer;
        String    tag_string;

/**
        if (storage_type.compareTo(FILE_STORAGE) != 0) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ILLEGAL_METHOD_EXCEPTION, ILLEGAL_METHOD_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
**/
        escape_flag = false;
        length = 1;
        rt = 0;
        while (true) {
            for ( ;offset < buffer.length ; offset = offset + rt) {
                try {
                    rt = readBuffer(buffer, offset, length);
                } catch (Exception exception_object) {
                    if ((exception_object instanceof EOFException) == true) {
                        if (getPosition(offset) == 0) {
                            return null;
                        }
                        String       message;
                        Exception    real_exception_object;
                        message = getMessage(
                            FORMAT_ERROR_EXCEPTION, FORMAT_ERROR_EXCEPTION);
                        real_exception_object = new Exception( message );
                        throw real_exception_object;
                    }
                    throw exception_object;
                }
                if (rt == -1) {
                    return null;
                }
                if (escape_flag == true) {
                    escape_flag = false;
                    continue;
                }

                if (offset == 0) {
                    byte[]  last_buffer;
                    last_buffer = getLast();
                    if (last_buffer[0] == START_BYTE) {
                        if (buffer[offset] == SLASH_BYTE) {
                            enableEndFlag(); 
                        } else {
                            disableEndFlag(); 
                        }
                        setStartTagStart(getPosition(offset)-1);
                    } else if (last_buffer[0] == END_BYTE) {
                        position = position +1;
                    } else if (skipByte(buffer[offset]) == true) {
                        if (checkTagFlag() != false) {
                            position = position + 1;
                        }
                    }
                } else {
                    if (buffer[offset-1] == START_BYTE) {
                        if (buffer[offset] == SLASH_BYTE) {
                            enableEndFlag(); 
                        } else {
                            disableEndFlag(); 
                        }
                        setStartTagStart(getPosition(offset)-1);
                    } else if (buffer[offset-1] == END_BYTE) {
                        position = position +1;
                    } else if (skipByte(buffer[offset]) == true) {
                        if (checkTagFlag() != false) {
                            position = position + 1;
                        }
                    }
                }
                switch (buffer[offset]) {
                    case '<':
                        if (checkTagFlag() != false) {
                            break;
                        }
                        enableTagFlag();
                        if (position >= getPosition(0)) {
                            if (position <= getPosition(offset)) {
                                int   seek_byte;
                                int   page_offset;

                                page_offset = getOffset(position);
                                seek_byte = seekPointer(buffer, page_offset, offset);
                                if (seek_byte != 0) {
                                    position = getPosition(seek_byte);
                                }
                            }
                        } 
                        

                        if (getPosition(offset) != 0 && getPosition(offset) -1 >= position) {
                            page_entry = PageEntry.valueOf();
                            page_entry.setStart(position);
                            page_entry.setEnd(getPosition(offset)-1);
                            offset = offset + rt;
                            return page_entry;
                        }
                        break;
                    case '>':
                        if (checkTagFlag() != true) {
                            break;
                        }
                        disableTagFlag();
                        offset = offset + rt;
                        if (getEndFlag() == false) {
                            setStartTagEnd(getPosition(offset));
                            position = getPosition(offset);

                            book_entry = BookEntry.valueOf();

                            start_tag = new AreaRange();
                            start_tag.setStartPosition(getStartTagStart());
                            start_tag.setEndPosition(getStartTagEnd());

                            book_entry.setStartTag(start_tag);

                            if (getPageFromPosition(getStartTagStart()) == getPage()) {
                                holdTag(buffer, getOffset(getStartTagStart()), 
                                    offset-getOffset(getStartTagStart())+1-rt);
                            } else {
                                holdTag(buffer, 0, offset+1-rt);
                            }
                            tag_buffer = getTagArea();
                            tag_string = new String(tag_buffer, 0, tag_buffer.length);
                            book_entry.setTag(tag_string);
                            clearTag();
                            return book_entry;
                        } else {
                            setEndTagEnd(getPosition(offset));
                            position = getPosition(offset);
                            return null;
                        }
                    case '/':
                        if (checkTagFlag() != true) {
                            break;
                        }
                        setEndTagStart(getPosition(offset)-1);
                        break;
                    case '\\':
                        escape_flag = true;
                        break;
                    default:
                        break;
                }
            }
            if (rt == -1) {
                break;
            }
            if (checkTagFlag() == true) {
                if (getPageFromPosition(getStartTagStart()) == getPage()) {
                    holdTag(buffer, getOffset(getStartTagStart()),
                        buffer.length-getOffset(getStartTagStart()));
                } else {
                    holdTag(buffer, 0, buffer.length);
                }
            }
            incPage();
            change_page();
        }
        return null;
    }

    public int readBuffer(byte[] buffer, int offset, int length) 
        throws Exception {
        int    rt;
        int    rsize;

        if (storage_type.compareTo(MEMORY_STORAGE) == 0) {
            if (memory_area.length == memory_seek_pointer) {
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    String message;
                    message = getMessage(
                        REACH_END_OF_AREA, REACH_END_OF_AREA );
                    message = getMessage( 
                        DEBUG_TRACE_KEY, DEBUG_TRACE_KEY,
                        SPACE,  message);
                    getLogPrintStream().printLogln(message);
                }
                memory_seek_pointer = -1;
            }
            if (memory_seek_pointer == -1) {
                String       message;
                EOFException    eof_exception_object;
                message = getMessage(EOF_EXCEPTION, EOF_EXCEPTION);
                eof_exception_object = new EOFException( message );
                throw eof_exception_object;
            }
            rsize = length;
            if (memory_area.length - memory_seek_pointer <  length) {
               rsize = memory_area.length - memory_seek_pointer;
            }
            rt = 0;
            for ( ; rt < rsize; rt++) {
                buffer[offset+rt] = memory_area[memory_seek_pointer+rt]; 
            }
            memory_seek_pointer = memory_seek_pointer + rt;
            return rt;
        }
        if (storage_type.compareTo(FILE_STORAGE) != 0) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ILLEGAL_METHOD_EXCEPTION, ILLEGAL_METHOD_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (buffer == null || buffer.length <= offset || buffer.length <=
            offset + length) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION, ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (fd == null) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ASSERTION_EXCEPTION, ASSERTION_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        
        if (file_input_stream == null) {
            try {
                file_input_stream = new FileInputStream(fd);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            rt = file_input_stream.read(buffer, offset, length);
        } catch (Exception  exception_object) {
            throw exception_object;
        }
        return rt;
    }

    public int[] searchKey(String key, int book_index, IndexCard list) throws Exception {
        int         index;
        int         start;
        int         end;
        int         local_book_index;
        int[]       key_entry;
        int[]       bk_key_entry;
        int[]       book_key_entry;
        byte[]      key_buffer;
        PageEntry   page_entry;
        BaseEntry   base_entry;
        BookEntry   book_entry;

        if (key == null || list == null) {
            String       message;
            Exception    exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION, ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        key_entry = new int[0];
        key_buffer = key.getBytes();
        base_entry = (BaseEntry)list.get(book_index);
        book_entry = (BookEntry)base_entry;
        for (int i = 0; i < book_entry.getEntrySize(); i++) {
            index = book_entry.getEntryNumber(i);
            if (index == 0) {
                continue;
            }
            base_entry = (BaseEntry)list.get(index);
            if (base_entry.getName().compareTo(PageEntry.type_name) == 0) {
                page_entry = (PageEntry)base_entry;
                start = page_entry.getStart();
                end   = page_entry.getEnd();
                try {
                    OperateBuffer   operate_buffer;
                    operate_buffer = new OperateBuffer();

                    operate_buffer.setDebug( getDebug() );
                    operate_buffer.setMessageResource(
                        getMessageResource() );
                    operate_buffer.setLogPrintStream (
                        getLogPrintStream() );

                    operate_buffer.searchWord(memory_area, start, end,
                        key_buffer);
                } catch (Exception exception_object) {
                    continue;
                }
                bk_key_entry = new int[key_entry.length+1];
                for (int j = 0; j < key_entry.length; j++) {
                    bk_key_entry[j] = key_entry[j];
                }
                bk_key_entry[key_entry.length] = page_entry.getNumber();
                key_entry = bk_key_entry;
            } else {
                book_entry = (BookEntry)base_entry;
                local_book_index = book_entry.getNumber();
                try {
                    book_key_entry = searchKey(key, local_book_index, list);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                bk_key_entry = new int[key_entry.length+book_key_entry.length];
                for (int j = 0; j < key_entry.length; j++) {
                    bk_key_entry[j] = key_entry[j];
                }
                for (int j = 0; j < book_key_entry.length; j++) {
                    bk_key_entry[j+key_entry.length] = book_key_entry[j];
                }
                key_entry = bk_key_entry;
            }
        }
        return key_entry;
    }

    private void printSpace(int tab) {
        for (int i = 0; i < tab; i++) {
            System.out.print(TAB_SPACE);
        }
    }

    public void printBookContent(int hierarchy, BookEntry book_entry, IndexCard list) throws Exception {
        int        index;
        BaseEntry  base_entry;

        if (book_entry == null || list == null) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        for (int i = 0; i < book_entry.getEntrySize(); i++) {
            index = book_entry.getEntryNumber(i);
            if (index == 0) {
                continue;
            }
            base_entry = (BaseEntry)list.get(index);
            if (base_entry.getName().compareTo(BookEntry.type_name) == 0) {
                printBook(hierarchy, index, list);
            } else {
                printPage(hierarchy, index, list);
            }
        }
    }

    public void printBook(int hierarchy, int number, IndexCard list) throws Exception {
        int        index;
        BookEntry  book_entry;
        BaseEntry  base_entry;
        AreaRange  start_tag;
        AreaRange  end_tag;

        if (list == null) {
            String       message;
            Exception    exception_object;
            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        book_entry = (BookEntry)list.get(number);        
        if (number != 0) {
            start_tag = book_entry.getStartTag();
            printContents(hierarchy, start_tag.getStartPosition(),
                start_tag.getEndPosition());
            System.out.println();
        }
        printBookContent(hierarchy + 1, book_entry, list);
        if (number != 0) {
            end_tag = book_entry.getEndTag();
            printContents(hierarchy, end_tag.getStartPosition(), 
                end_tag.getEndPosition());
            System.out.println();
        }
    }

    public void printPage(int hierarchy, int number, IndexCard list) throws Exception {
        int        start;
        int        end;
        PageEntry  page_entry;

        if (list == null) {
            String       message;
            Exception    exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        page_entry = (PageEntry)list.get(number);
        start = page_entry.getStart();
        end   = page_entry.getEnd();
        printContents(hierarchy, start, end);
    }

    public byte[] getContents(int start, int end) throws Exception {
        byte[]  buffer;

        if (end - start <= 0 || start < 0) {
            String       message;
            Exception    exception_object;

            message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION,
                ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        }
        buffer = new byte[end - start+1];

        if (storage_type.compareTo(MEMORY_STORAGE) == 0) {
            if (end >= memory_area.length) {
                String       message;
                Exception    exception_object;

                message = getMessage(
                    ARGUMENT_PARAMETER_EXCEPTION,
                    ARGUMENT_PARAMETER_EXCEPTION);
                exception_object = new Exception( message );
                throw exception_object;
            }
            for (int i = start; i <= end; i++) {
                buffer[i-start] = memory_area[i];
            }
            return buffer;
        }
     
        if (fd != null) {
            file_input_stream = new FileInputStream(fd);

            try {
                file_input_stream.skip(start);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            try {
                file_input_stream.read(buffer, 0, end - start+1);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            try {
                file_input_stream.close();
            } catch (Exception exception_object) {
                throw exception_object;
            }
            return buffer;
        }
        String       message;
        Exception    exception_object;

        message = getMessage(
                ARGUMENT_PARAMETER_EXCEPTION, ARGUMENT_PARAMETER_EXCEPTION);
        exception_object = new Exception( message );
        throw exception_object;
    }

    public void printContents(int hierarchy, int start, int end) throws Exception {
        String  line;
        byte[] buffer;
        int    check;
        int    length;
        String message;
        ByteArrayInputStream  byte_array_input_stream;
        DataInputStream       data_input_stream;

        if (fd != null || storage_type.compareTo(MEMORY_STORAGE) == 0) {
/**
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message = getMessage( ENTER_METHOD, ENTER_METHOD,
                    SPACE, PRINTCONTENTS);
                getLogPrintStream().printLogln(message);
            }
**/
            try {
                buffer = getContents(start, end);
            } catch (Exception exception_object) {
                throw exception_object;
            }
/************
            byte_array_input_stream = new ByteArrayInputStream(buffer);
            data_input_stream = new DataInputStream(byte_array_input_stream);
            while (true) {
                try {
                    line = data_input_stream.readLine();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                printSpace(hierarchy);
                System.out.print(line.trim());
            }
*********/

/*
*/
            System.out.print(new String(buffer, 0, buffer.length));
/*
*/
/**
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message = getMessage( END_METHOD, END_METHOD,
                    SPACE, PRINTCONTENTS);
                getLogPrintStream().printLogln(message);
            }
**/
        } 
    }

    public static void main(String[]  args) {
        int           no;
        StoreHouse    store_house;
        BaseEntry     entry;
        BookEntry[]   hierarchy;
        BookEntry     book_entry;
        AreaRange     end_tag_area;
        IndexCard     list_object;

        if (args.length == 1) {
            list_object = new IndexCard();
            no = 0;
            try {
                store_house = StoreHouse.valueOf(args[0]);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }

            System.out.println(store_house.toString());

            hierarchy = new BookEntry[1];
            entry = (BaseEntry)BookEntry.valueOf();
            list_object.add(entry);
            hierarchy[0] = (BookEntry)entry;
            while (true) {
                try {
                    entry = store_house.readEntry();
                } catch (Exception exception_object) {
                    break;
                }
                if (entry == null) {
                    if (hierarchy.length <= 1) {
                        break;
                    }
                    entry = (BaseEntry)hierarchy[hierarchy.length-1];
                    end_tag_area = store_house.getEndTag();
                    try {
                        ((BookEntry)entry).setEndTag(end_tag_area);
                    } catch (Exception exception_object) {
                         exception_object.printStackTrace();
                         return;
/*
                        throw exception_object;
*/
                    }
                    BookEntry[] bk;
                    bk = new BookEntry[hierarchy.length-1];
                    for (int i =0; i < bk.length; i++ ) {
                        bk[i] = hierarchy[i];
                    }
                    hierarchy = bk;
                    continue;
                }
                if ((entry instanceof BookEntry) == true) {
                    no = list_object.size();
                    list_object.add(entry);
                    if (hierarchy.length != 0) {
                        hierarchy[(int)hierarchy.length-1].addEntry(no);
                    }
                    BookEntry[] bk;
                    bk = new BookEntry[hierarchy.length+1];
                    for (int i =0; i < hierarchy.length; i++ ) {
                        bk[i] = hierarchy[i];
                    }
                    bk[bk.length-1] = (BookEntry)entry;
                    hierarchy = bk;
                    continue;
                }
                no = list_object.size();
                list_object.add(entry);
                if (hierarchy.length != 0) {
                    book_entry = hierarchy[(int)hierarchy.length-1];

                    hierarchy[(int)hierarchy.length-1].addEntry(no);
                }
            }
            try {
                store_house.printBook(0, 0, list_object);
            } catch (Exception  exception_object) {
                exception_object.printStackTrace();
            }
        }
    }
}
