/**
 * Copyright (C) All rights reserved.
 *
 * @Module     TagString.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/08/20
 *
 */

package exec;

import java.lang.*;
import java.io.*;
import java.net.*;

public class TagString extends DebugObject {
    private  String  tag = null;

    private  String  key;
    private  String  option = null;

    private String NAME_KEY = "NAME";
    private String SPACE_STRING = " ";
    private final static char TAG_START = '<';
    private final static char TAG_END   = '>';
    private final static char SPACE_CHAR = ' ';
    public final static String TAG_START_STRING = "<";
    public final static String TAG_END_STRING   = ">";
    private final static String END_TAG_STRING = "/";
    private final static String OPTION_SEPARATOR = "=";
    public final static String EMPTY_TAG = TAG_START_STRING + TAG_END_STRING;

    private String PARAMETER_ERROR = "Parameter error";

    private TagString() throws Exception {
    }

    public TagString(String param_tag) throws Exception {
        TagStringBody(param_tag);
    }

    public static TagString valueOf(String param_tag, MessageResource message_resource, int debug, LogPrintStream  log_print_stream) throws Exception {
        TagString  tag_string;

        tag_string = new TagString();
        tag_string.setMessageResource (message_resource);
        tag_string.setDebug(debug);
        tag_string.setLogPrintStream(log_print_stream);

        try {
            tag_string.TagStringBody(param_tag);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return tag_string;
    }

    private void TagStringBody(String param_tag) throws Exception {
        int    index;
        String  upp_opt;
        String  low_opt;

        if (param_tag == null || param_tag.length() == 0 || param_tag.charAt(0) != TAG_START 
            || param_tag.charAt(param_tag.length()-1) != TAG_END) {
            String       message;
            Exception    exception_object;

            message = getMessage( ARGUMENT_PARAMETER_EXCEPTION,
                    ARGUMENT_PARAMETER_EXCEPTION);
            exception_object = new Exception( message );
            throw exception_object;
        } 
        tag = param_tag;
        index = tag.indexOf(SPACE_STRING);
        if (index > 0) {
            key = tag.substring(1, index).trim();
            option = tag.substring(index+1, tag.length()-1).trim(); 
            index = option.indexOf(OPTION_SEPARATOR);
            low_opt = option.substring(index+1).trim();
            upp_opt = option.substring(0, index);
            for (index = upp_opt.length()-1; index >= 0; index--) {
                if (upp_opt.charAt(index) != SPACE_CHAR) {
                    break;
                }
            }
            if (upp_opt.length()-1 > index) {
                upp_opt = upp_opt.substring(0, index+1);
            }
            option = upp_opt + OPTION_SEPARATOR + low_opt;
        } else {
            key = tag.substring(1, tag.length()-1).trim();
            option = null;
        }
    }

    public String getTag() {
        return tag;
    }

    public String getKey() {
        return key;
    }

    public String getOption() {
        return option;
    }

    public String getOptionValue() {
        int     index;
        String  value;

        if (option == null) { return null; }
        index = option.indexOf(OPTION_SEPARATOR);
        if (index > 0) {
            value = option.substring(index+1);
            return value;
        }
        return option;
    }

    public void setOption(String param_option) {
        option = param_option;
    }

    public String getStartTagWithoutOption() {
        String  tag_string;

        tag_string = new String(
            TAG_START_STRING + key + TAG_END_STRING );
        return tag_string;
    }

    public String getStartTag() {
        String  tag_string;

        if (option == null) {
            tag_string = new String(
            TAG_START_STRING + key + TAG_END_STRING );
        } else {
            tag_string = new String(
            TAG_START_STRING + key + SPACE_STRING + option + TAG_END_STRING );
        }
        return tag_string;
    }

    public String getEndTag() {
        String  tag_string;

        tag_string = new String(
            TAG_START_STRING + END_TAG_STRING + key + TAG_END_STRING );
        return tag_string;
    }

    public static boolean compareKey(String s1, String s2) {
        TagString  s1_tag;
        TagString  s2_tag;
        String     s1_key;
        String     s2_key;

        if (s1 == null || s2 == null) {
            return false;
        }
        try {
            s1_tag = new TagString(s1);
        } catch (Exception exception_object) {
            return false;
        }
        try {
            s2_tag = new TagString(s2);
        } catch (Exception exception_object) {
            return false;
        }

        s1_key = s1_tag.getKey();
        s2_key = s2_tag.getKey();

        if (s1_key == null || s2_key == null) {
            return false;
        }
        if (s1_key.compareTo(s2_key) == 0) {
            return true;
        }
        return false;
    }

    public static boolean isOrder(String s1, String s2) {
        TagString  s1_tag;
        TagString  s2_tag;
        String     s1_key;
        String     s2_key;
        String     s1_option;
        String     s2_option;

        if (s1 == null) {
            return false;
        }
        if (s2 == null) {
            return true;
        }
        try {
            s1_tag = new TagString(s1);
        } catch (Exception exception_object) {
            return false;
        }
        try {
            s2_tag = new TagString(s2);
        } catch (Exception exception_object) {
            return true;
        }

        s1_key = s1_tag.getKey();
        s2_key = s2_tag.getKey();
        s1_option = s1_tag.getOption();
        s2_option = s2_tag.getOption();

        if (s1_key == null) {
            return false;
        }
        if (s2_key == null) {
            return true;
        }
        if (s1_key.compareTo(s2_key) > 0) {
            return true;
        }
        if (s1_key.compareTo(s2_key) == 0) {
            if (s1_option == null) {
                return false;
            }
            if (s1_option.compareTo(s2_option) > 0) {
                return true;
            }
        }
        return false;
    }

    public static boolean compareTag(String s1, String s2) {
        TagString  s1_tag;
        TagString  s2_tag;
        String     s1_key;
        String     s2_key;
        String     s1_option;
        String     s2_option;

        if (s1 == null || s2 == null) {
            return false;
        }
        try {
            s1_tag = new TagString(s1);
        } catch (Exception exception_object) {
            return false;
        }
        try {
            s2_tag = new TagString(s2);
        } catch (Exception exception_object) {
            return false;
        }

        s1_key = s1_tag.getKey();
        s2_key = s2_tag.getKey();
        s1_option = s1_tag.getOption();
        s2_option = s2_tag.getOption();

        if (s1_key == null || s2_key == null) {
            return false;
        }
        if (s1_key.compareTo(s2_key) == 0) {
            if (s1_option == null && s2_option == null) {
                return true;
            }
            if (s1_option == null || s2_option == null) {
                return false;
            }
            if (s1_option.compareTo(s2_option) == 0) {
                return true;
            }
        }
        return false;
    }

    public static String[] parseTags(String tags) {
        int         index;
        String      remain;
        String      entry;
        String[]    array;
        String[]    bk;

        array = new String[0];
        remain = tags;
        while ( true ) {
            index = remain.indexOf(TAG_END_STRING);
            if (index < 0) {
                break;
            }
            entry = remain.substring(0, index+1);
            remain = remain.substring(index+1);
            bk = new String[array.length+1];
            for (int i = 0; i < array.length; i++) {
                bk[i] = array[i];
            }
            array = bk;
            array[array.length-1] = entry;
        }
        return array;
    }

    public static void main(String[] args) {
    }
}
