/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Delete.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/18
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class Delete extends DirectoryBase {
    public static final String COMMAND_NAME = "odir.Delete";
    public static final String DELETE_ERROR = "Delete error.";
    public static final String DIRECTORY_PARAMETER_ERROR 
        = "You must specify a directory path except root.";
    public final static String DELETE_USAGE_MESSAGE = "odir.Delete odir://host:port/directory|directory owner password";

    public final static char separator_char = '/';
    public final static String SEPARATOR_STRING = "/";

    public Delete() {
        super();
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String[]   auth;
        String[]   param;
        String     directory_entry;
        DataInputStream  din;
        DirectoryServerException   directory_server_exception;

        if (command_name == null || command_name.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ILLEGAL_COMMAND_NAME,
                    getMessageResource().getMessage(
                        DirectoryServerException.ILLEGAL_COMMAND_NAME)));
            throw directory_server_exception;
        }
        try {
            param = Directory.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return param;
    }

    public static boolean doDelete(URI key, String owner, String password) 
        throws Exception {
        boolean flag;
        URI uri_key;
        DirectoryServer dserv;
        String session_key;
        SealedObject    sealed_password;
        DirectoryServerException    directory_server_exception;

        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
            flag = dserv.delete(key, session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return flag;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            Delete.getMessageResource().getFormattedMessage(Delete.DELETE_USAGE_MESSAGE,
                Delete.getMessageResource().getMessage(Delete.DELETE_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        Delete  delete_object;
        URI     uri_string;
        String[] param;

        if (ag.length != 3) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    delete_object = new Delete();
                    param = Delete.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0) {
                        System.err.println( delete_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            delete_object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println( delete_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            delete_object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[2] == null || param[2].length() == 0 ) {
                        System.err.println( delete_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                                delete_object.getMessageResource().getMessage(
                                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    uri_string = delete_object.getKey(param[0]);
                    if (uri_string == null) {
                        System.err.println( delete_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                                delete_object.getMessageResource().getMessage(
                                    DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (uri_string.getPath().compareTo(File.separator) == 0) {
                        System.err.println( delete_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.DIRECTORY_PARAMETER_ERROR,
                            delete_object.getMessageResource().getMessage(
                                DirectoryServerException.DIRECTORY_PARAMETER_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    
                    if (delete_object.doDelete(uri_string, param[1], param[2])
                        == false) {
                        System.err.println( delete_object.getMessageResource().getFormattedMessage(
                            Delete.DELETE_ERROR,
                            delete_object.getMessageResource().getMessage( Delete.DELETE_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            Delete.printUsage(ag);
            return ;
        }
        delete_object = new Delete();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println(
                delete_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    delete_object.getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println(
                delete_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    delete_object.getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[2] == null || ag[2].length() == 0) {
            System.err.println(
                delete_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    delete_object.getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            uri_string = delete_object.getKey(ag[0]);
            if (uri_string == null) {
                System.err.println(
                    delete_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                        delete_object.getMessageResource().getMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                System.exit(Register.ERROR_EXIT_CODE);
            }
            if (uri_string.getPath().compareTo(File.separator) == 0) {
                System.err.println( delete_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.DIRECTORY_PARAMETER_ERROR,
                    delete_object.getMessageResource().getMessage(
                        DirectoryServerException.DIRECTORY_PARAMETER_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
            }
            if (delete_object.doDelete(uri_string, ag[1], ag[2]) == false) {
                System.err.println( delete_object.getMessageResource().getFormattedMessage(
                    Delete.DELETE_ERROR,
                    delete_object.getMessageResource().getMessage( Delete.DELETE_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
            }
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }
}
