/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Directory.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class Directory extends DirectoryBase {
    public final static String DIRECTORY_USAGE_MESSAGE = "odir.Directory odir://host:port/directory|directory owner password";
    public final static String SCHEME_STRING = "odir";
    public final static String DIRECTORY_ERROR = "Directory Error";
    public final static int ARGUMENT_NUMBER = 3;
    public final static String DIRECTORY_MESSAGE = "Directory :";
    public final static String URI_FORMAT_ERROR = "URI Format Error";
    public final static String COMMAND_NAME = "odir.Directory";

    public final static char separator_char = '/';

    public Directory() {
        super();
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String[]   auth;
        String[]   param;
        DataInputStream  din;
        DirectoryServerException   directory_server_exception;

        if (command_name == null || command_name.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    Register.ILLEGAL_COMMAND_NAME,
                    getMessageResource().getMessage(
                        Register.ILLEGAL_COMMAND_NAME)));
            throw directory_server_exception;
        }
        param = new String[ARGUMENT_NUMBER];
        System.out.print( getMessageResource().getFormattedMessage(
            Directory.DIRECTORY_MESSAGE,
            getMessageResource().getMessage(Directory.DIRECTORY_MESSAGE)));
        din = new DataInputStream(System.in);
      
        try {
            param[0] = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        param[1] = auth[0];
        param[2] = auth[1];
        return param;
    }

    public boolean doDirectory(URI key, String owner, String password) throws
        Exception {
        DirectoryServer dserv;
        boolean  flag;
        String   session_key;
        SealedObject   sealed_password;
        DirectoryServerException directory_server_exception;

        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
            flag = dserv.directory(key, session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return flag;
    }

    public static void printUsage(String[] ag) {
        System.out.println( 
            Directory.getMessageResource().getFormattedMessage(
                Directory.DIRECTORY_USAGE_MESSAGE,
                Directory.getMessageResource().getMessage(
                    Directory.DIRECTORY_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        int   idx;
        Directory directory_object;
        URI uri_string;
        String[] param;

        if (ag.length != 3) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    param = Directory.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println( Directory.getMessageResource().getFormattedMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                            Directory.getMessageResource().getMessage(
                                DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0) {
                        System.err.println( Directory.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            Directory.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[2] == null || param[2].length() == 0) {
                        System.err.println( Directory.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                            Directory.getMessageResource().getMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    directory_object = new Directory();
                    if ((uri_string = directory_object.getKey(param[0])) == null) {
                        System.err.println( Directory.getMessageResource().getFormattedMessage(
                            URI_FORMAT_ERROR,
                            Directory.getMessageResource().getMessage(
                                URI_FORMAT_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    directory_object.doDirectory(uri_string, 
                        param[1], param[2]);
                } catch (Exception exception_object) {
                    if ((exception_object instanceof URISyntaxException) == true) {
                        System.err.println( Directory.getMessageResource().getFormattedMessage(
                            URI_FORMAT_ERROR,
                            Directory.getMessageResource().getMessage(
                                URI_FORMAT_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    return;
                }
                return;
            }
            Directory.printUsage(ag);
            return;
        }
        if (ag[0] == null || ag[0].length() == 0) {
            System.err.println( Directory.getMessageResource().getFormattedMessage(
                DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                Directory.getMessageResource().getMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0) {
            System.err.println( Directory.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                Directory.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[2] == null || ag[2].length() == 0) {
            System.err.println( Directory.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                Directory.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            directory_object = new Directory();
            if ((uri_string = directory_object.getKey(ag[0])) == null) {
                System.err.println( Directory.getMessageResource().getFormattedMessage(
                    URI_FORMAT_ERROR,
                    Directory.getMessageResource().getMessage(
                        URI_FORMAT_ERROR)));
                return;
            }
            directory_object.doDirectory(uri_string, ag[1], ag[2]);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }
}
