/**
 * Copyright (C) All rights reserved.
 *
 * @Module     DirectoryBase.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class DirectoryBase extends Command {
    public final static String SCHEME_SEPARATOR = ":";
    public final static String LOCAL_HOST_STRING = "localhost";
    public final static String SCHEME_STRING = "odir";
    public final static int ERROR_EXIT_CODE = -1;
    public final static String DEFAULT_PORT = "2001";
    public final static int default_port = 2001;

    public final static String yes_string = "yes";
    public final static String ye_string  = "ye";
    public final static String y_string   = "y";
    public final static String no_string  = "no";
    public final static String n_string   = "n";

    public static int RETRY_LIMIT = 5;

    public DirectoryBase() {
        super();
    }

    public URI getKey(String path) {
        URI   key;
        int   idx;

        idx = path.indexOf(SCHEME_SEPARATOR);
        try {
            if (idx > 0) {
                key = new URI(path);
            } else {
                if (path.charAt(0) != File.separatorChar) {
                    key = new URI(SCHEME_STRING, LOCAL_HOST_STRING, File.separator + path, null);
                } else {
                    key = new URI(SCHEME_STRING, LOCAL_HOST_STRING, path, null);
                }
            }
        } catch (Exception exception_object) {
            return null;
        }
        return key;
    }

    public URI getKey(String path, String hostname) {
        URI   key;

        try {
            key = new URI(SCHEME_STRING, (String)null, hostname, 
                default_port, path, (String)null, (String)null);
        } catch (Exception exception_object) {
            return null;
        }
        return key;
    }

    public String getFileEntry(String parameter_file) {
        int    index;
        String file_name;

        if (parameter_file == null) {
            return null;
        }
        index = parameter_file.lastIndexOf(File.separator);
        if (index < 0) {
            return parameter_file;
        } 
        file_name = parameter_file.substring(index+1);
        return file_name;
    }

    public URI expandKey(URI key, String parameter_file) {
        URI       key_object;
        String    file;

        if (key == null || parameter_file == null) {
            return null;
        }
        file = getFileEntry(parameter_file);
        try {
            key_object = new URI(key.getScheme(), key.getUserInfo(), key.getHost(),
            key.getPort(), key.getPath() + File.separator + file,
            key.getQuery(), key.getFragment());
        } catch (Exception exception_object) {
            return null;
        }
        return key_object;
    }

    public boolean getYesNo(String message) throws Exception {
        DataInputStream   din;
        int retry_count;
        String reply;
        DirectoryServerException    directory_server_exception;

        System.out.print(message);
        din = new DataInputStream(System.in);
        retry_count = 0;
        while (true) {
            try {
                reply = din.readLine();
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
                throw directory_server_exception;
            }
            if (reply == null) {
                if (retry_count > RETRY_LIMIT) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.INPUT_RETRY_LIMIT,
                            getMessageResource().getMessage(
                                DirectoryServerException.INPUT_RETRY_LIMIT)));
                    throw (Exception)directory_server_exception;
                }
            }
            if ((reply.length() == 3 && 
                reply.toLowerCase().compareTo(yes_string) == 0) 
                 || (reply.length() == 2 &&
                reply.toLowerCase().compareTo(ye_string) == 0) 
                 || (reply.length() == 1 &&
                reply.toLowerCase().compareTo(y_string) == 0) 
            ) {
                return true;
            }
            if ((reply.length() == 2 && 
                reply.toLowerCase().compareTo(no_string) == 0) 
                || (reply.length() == 1 &&
                reply.toLowerCase().compareTo(n_string) == 0) 
            ) {
                return false;
            }
            retry_count = retry_count + 1;
        }
    }

}
