/**
 * Copyright (C) All rights reserved.
 *
 * @Module     DirectoryServer.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

import odir.*;

public class DirectoryServer extends ClassLoader implements DirectoryService {
    public final static String USER_DATABASE = "user_database";
    private String des_key = "DES";
    private String password_key = "?u:->X#!";
    public final static String SPACE = " ";
    public final static String DOT = ".";
    public final static String TEMPORARY_SUFFIX = ".";
    public final static char DOS_SEPARATOR = '\\';
    public final static String LOCK_DIRECTORY = ".lock_dir";
    public final static String LOCK_PREFIX = ".lock.";
    public final static String RETURN_STRING = "\n";
    public final static String TEXT_SUFFIX = "txt";
    public final static String PLAIN_TEXT = "plain text :";
    public final static String JAVA_SUFFIX = "java";
    public final static String JAVA_SOURCE = "java source text :";
    public final static String CLASS_SUFFIX = "class";
    public final static String CLASS_OBJECT = "java class object :";
    public final static String JAR_SUFFIX = "jar";
    public final static String JAR_ARCHIVE = "jar archive file :";
    public final static String GENERAL_FILE = "general file :";
    public final static String SIZE_TEXT  = "size";

    public final static String STRING_PARAMETER_TYPE = "java.lang.String";

    public static String base_directory = "./";
    public static String DIRECTORY_SEPARATOR = "/";

    public static int BUFFER_SIZE = 4096;
    public static int SPLIT_LIMIT = 1000; 

    public static int INTERVAL = 100;
    public static int LOOP_LIMIT = 100;
    public static String KEY_DIRECTORY = ".SESSION_KEY";
    public static String SESSION_TIMEOUT = "Session Timeout";
    public static String COLON = ":";
    public static String SESSION_KEY_SEPARATOR = COLON;

    public static long LIMIT_MILLI = 86400000;

/*
    public static String CLASS_SUFFIX = "class";
    public static String JAR_SUFFIX = "jar";
*/
    public static String EMPTY_PATH = "";

    public static String FILE_URL_HEAD = "file://";
    public static String JAR_URL_HEAD = "jar:file:";
    public static String JAR_URL_END  = "!/";

    public static String VOID_TYPE_KEY = "void";
    public static String INT_NATIVE_TYPE_KEY = "int";
    public static String LONG_NATIVE_TYPE_KEY = "long";
    public static String FLOAT_NATIVE_TYPE_KEY = "float";
    public static String DOUBLE_NATIVE_TYPE_KEY = "double";
    public static String BOOLEAN_NATIVE_TYPE_KEY = "boolean";
    public static String BYTE_NATIVE_TYPE_KEY = "byte";
    public static String CHAR_NATIVE_TYPE_KEY = "char";
    public static String SHORT_NATIVE_TYPE_KEY = "short";

    public static String STRING_OBJECT_TYPE_KEY = "java.lang.String";
    public static String INTEGER_OBJECT_TYPE_KEY = "java.lang.Integer";
    public static String LONG_OBJECT_TYPE_KEY = "java.lang.Long";
    public static String FLOAT_OBJECT_TYPE_KEY = "java.lang.Float";
    public static String DOUBLE_OBJECT_TYPE_KEY = "java.lang.Double";
    public static String SHORT_OBJECT_TYPE_KEY = "java.lang.Short";
    public static String CHARACTER_OBJECT_TYPE_KEY = "java.lang.Character";
    public static String BOOLEAN_OBJECT_TYPE_KEY = "java.lang.Boolean";
    public static String BYTE_OBJECT_TYPE_KEY = "java.lang.Byte";

    public final static char ARRAY_INT = 'I';
    public final static char ARRAY_BYTE = 'B';
    public final static char ARRAY_LONG = 'J';
    public final static char ARRAY_SHORT = 'S';
    public final static char ARRAY_CHAR = 'C';
    public final static char ARRAY_DOUBLE = 'D';
    public final static char ARRAY_FLOAT = 'F';
    public final static char ARRAY_BOOLEAN = 'Z';
    public final static char ARRAY_REFERENCE = 'L';

    public final static String ARRAY_KEY = "[";
    public final static int FILE_SIZE_LIMIT = 4096000;

    public final static String LOCK_ERROR_MESSAGE = "Lock directory was illegal.";
    public final static String FORMAT_ERROR_EXCEPTION
        = "format error exception";

    public static int write_mode = 0;
    public final static int NORMAL_WRITE = 1;
    public final static int APPEND_WRITE = 2;

    public static long read_position = 0;
    public final static int FILE_SIZE_MODE = -1;

    public final static String DIRECTORY_SERVER_MESSAGE_HEADER
        = "Logging Message in DirectoryServer";

    private static MessageResource  message_resource = null;

    public static MessageResource  getMessageResource() {
        Locale   locale_object;

        if (message_resource == null) {
            message_resource = new MessageResource();
            locale_object = Locale.getDefault();
            message_resource.setLanguage(locale_object.getISO3Language()); 
        }
        return message_resource;
    }

    private void printServerLog(String... messages) {
        String  long_message;
        long_message = new String();
        for (int  i = 0; i < messages.length; i++) {
            long_message = long_message.concat(messages[i]);
        }
        System.err.println(
            getMessageResource().getFormattedMessage(
                DIRECTORY_SERVER_MESSAGE_HEADER,
                getMessageResource().getMessage(
                    DIRECTORY_SERVER_MESSAGE_HEADER),
                    SPACE, long_message));
    }

    public void OServ() {
        String  owner;
        String  path;
        String  read_entry;
        String[] entries;
        FileInputStream fis;
        DataInputStream din;

        File fdes = new File(USER_DATABASE);
        String user_database = fdes.getAbsolutePath();
        fdes = new File(user_database);
        try {
            fis = new FileInputStream(fdes);
        } catch (Exception exception_object) {
            printServerLog( getMessageResource().getFormattedMessage(
                DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                getMessageResource().getMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            return;
        }
        try {
            din = new DataInputStream(fis);
        } catch (Exception exception_object) {
            printServerLog( getMessageResource().getFormattedMessage(
                DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                getMessageResource().getMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            return;
        }

        while (true) {
            try {
                read_entry = din.readLine();
            } catch (Exception exception_object) {
                read_entry = null;
                break;
            }
            if (read_entry == null) {
                break;
            }
            owner = getOwnerFromEntry(read_entry) ;
            try {
                path = getDirectory(owner) + File.separator + LOCK_DIRECTORY;
            } catch (Exception exception_object) {
                break;
            }
            clearLock(path);
        }
    }

    public boolean clearLock(String path) {
        File[] list;
        File lock_fdes;

        lock_fdes = new File(path);
        if (lock_fdes.isDirectory() == true) {
            list = lock_fdes.listFiles();
            for (int i = 0; i < list.length; i++) {
                if (list[i].isDirectory() == true) {
                    try {
                        removeDirectory(list[i]);
                    } catch (Exception exception_object) {
                    }
                } else {
                    list[i].delete();
                }
            }
            return true;
        }
        return false;
    }


    public String getOwnerFromEntry(String entry) {
        String owner;
        int idx;

        idx = entry.indexOf(SPACE);
        if (idx > 0) {
            owner = entry.substring(0, idx);
        } else {
            owner = entry;
        }
        return owner;
    }

    public String getDirectory(String owner) throws Exception {
        File   fdes;
        String path;
        String home_directory;
        DirectoryServerException directory_server_exception;

        fdes = new File(owner);
        home_directory = fdes.getAbsolutePath();
        fdes = new File(home_directory);
        if (fdes.exists() == false) {
            directory_server_exception = new DirectoryServerException (
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.HOME_DIRECTORY_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.HOME_DIRECTORY_NOT_FOUND)));
            throw directory_server_exception;
        }
        return home_directory;
    }

    public boolean createDirectory(String owner) 
    throws Exception {
        File   fdes;
        String path;
        String home_directory;
        DirectoryServerException directory_server_exception;

        fdes = new File(owner);
        home_directory = fdes.getAbsolutePath();
        fdes = new File(home_directory);
        if (fdes.exists() == true) {
            directory_server_exception = new DirectoryServerException (
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ENTRY_ALREADY_EXIST,
                    getMessageResource().getMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST)));
            throw directory_server_exception;
        }
        fdes.mkdir();
        path = getDirectory(owner) + File.separator + LOCK_DIRECTORY;
        fdes = new File(path);
        if (fdes.exists() != true) {
            fdes.mkdir();
        }

        return true;
    }

    public boolean removeDirectory(File fdes) 
    throws Exception {
  
        boolean flag;
        File[]  list;

        if (fdes.isDirectory() == true) {
            list = fdes.listFiles();
            for (int i = 0; i < list.length; i++) {
                if (list[i].isDirectory() == true) {
                    removeDirectory(list[i]);
                } else {
                    list[i].delete();
                }
            }
            flag = fdes.delete();
            return flag;
        } else {
            fdes.delete();
        }
        return true;
    }

    public boolean deleteDirectory(String owner) 
    throws Exception {
        File fdes;
        String  home_directory;

        fdes = new File(owner);
        home_directory = fdes.getAbsolutePath();
        fdes = new File(home_directory);
        if (removeDirectory(fdes) == false) {
            return false;
        }
        return true;
    }

    public boolean executeOn(URI key, String session_key) throws Exception
    {
        String  path;
        String  owner;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + LOCK_DIRECTORY 
            + File.separator + key.getPath();
        
        return executeLock(path);
    }


    public boolean executeLock(String path) throws Exception {
        boolean flag;
        File fdes;
        int  idx;

        try {
            fdes = new File(path);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        if (fdes.exists() == true) {
            return false;
        }

        String parent_entry;
        String parent_dir;
        parent_dir = fdes.getParent();
        idx = parent_dir.lastIndexOf(LOCK_DIRECTORY);
        if (idx > 0) {
            parent_entry = parent_dir.substring(idx);

            if (parent_entry.compareTo(LOCK_DIRECTORY) != 0) {
                File parent_des;
                parent_des = new File(parent_dir);
                parent_des.mkdirs();
            }
        } else {
            return false;
        }

        try {
             flag = fdes.createNewFile();
        } catch (Exception exception_object) {
            return false;
        }
        return flag;
    }

    public boolean executeOff(URI key, String session_key) throws Exception
    {
        String  path;
        String  owner;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + LOCK_DIRECTORY
            + File.separator + key.getPath();
        
        return executeUnlock(path);
    }

    public boolean executeUnlock(String path)  throws Exception {
        int    idx;
        File   fdes;
        String dpath;
        String entry;
        String[] dir_entries;

        try {
            fdes = new File(path);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        if (fdes.exists() == false) {
            return false;
        }
       
        if (fdes.isDirectory() == true) {
            DirectoryServerException exception_object = 
                new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST,
                        getMessageResource().getMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST)));
            throw exception_object;
        }
        fdes.delete();
        idx = path.lastIndexOf(DOT);
        if (idx <= 0) {
            return true;
        }
        dpath = path.substring(0, idx);
        while (dpath != null) {
            if (dpath.length() > 0) {
                break;
            }
            idx = dpath.lastIndexOf( LOCK_DIRECTORY );
            if (idx > 0) {
                entry = dpath.substring(idx);
                if (entry.compareTo(LOCK_DIRECTORY) == 0) {
                    return true;
                }
            }
            fdes = new File(dpath);
            dir_entries = fdes.list();
            if (File.separatorChar != DOS_SEPARATOR) {
                if (dir_entries.length != 2) {
                    break;
                }
            } else {
                if (dir_entries.length > 0) {
                    break;
                }
            }
            fdes.delete();
            dpath = fdes.getParent();
        }

        return true;
    }

    public boolean checkValidSession(String session_key) {
        String key_path;
        String absolute_path;
        File fd;
        boolean return_flag;
        long    milli;
        long    current_milli_time;

        if (session_key == null) {
            return false;
        }
        current_milli_time = System.currentTimeMillis();
        try {
            milli = getMilliTime(session_key);
        } catch (Exception exception_object) {
            return false;
        }
        key_path = KEY_DIRECTORY + File.separator + session_key;
        fd = new File(key_path);
        absolute_path = fd.getAbsolutePath();
        fd = new File(absolute_path);
        if (fd.exists() == true) {
            return true;
        }
        return false;
    }

    public boolean removeSessionKey(String session_key) {
        String  key_path;
        boolean return_flag;
        File    fd;

        key_path = KEY_DIRECTORY + File.separator + session_key;

        fd = new File(key_path);
        fd = fd.getAbsoluteFile();
        if (fd.exists() == true) {
            return_flag = fd.delete();
            return return_flag;
        }
        return false;
    }

    public String getPasswordfromSessionFile(String session_key) 
    throws Exception {
        File            fd;
        String          owner;
        String          owner_string;
        String          key_path;
        String          password;
        FileInputStream file_input_stream;
        ObjectInputStream object_input_stream;
        SealedObject      sealed_password;
        DirectoryServerException    directory_server_exception;

        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        key_path = KEY_DIRECTORY + File.separator + session_key;
        fd = new File(key_path);
        fd = fd.getAbsoluteFile();
        if (fd.exists() == false) {
            return null;
        }
        try {
            file_input_stream = new FileInputStream(fd);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            throw directory_server_exception;
        }
        try {
            object_input_stream = new ObjectInputStream(file_input_stream);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM)));
            throw directory_server_exception;
        }
        try {
            owner_string = (String)object_input_stream.readObject();
            sealed_password = (SealedObject)object_input_stream.readObject();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT)));
            throw directory_server_exception;
        }
        password = getPasswordFromSealedObject(sealed_password);
        return password;
    }


    public String getOwnerfromSessionKey(String session_key) 
        throws Exception
    {
        String          local_host;
        String          milli;
        String          local_owner;
        Exception       exception_object;
        DirectoryServerException   directory_server_exception;

        StringTokenizer string_tokenizer;
        string_tokenizer = new StringTokenizer(session_key, 
                  SESSION_KEY_SEPARATOR);

        if (string_tokenizer.hasMoreTokens() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        local_host = string_tokenizer.nextToken();
        if (string_tokenizer.hasMoreTokens() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        milli = string_tokenizer.nextToken();
        if (string_tokenizer.hasMoreTokens() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        local_owner = string_tokenizer.nextToken();
        return local_owner;
    }

    public long getMilliTime(String session_key) 
        throws Exception
    {
        String          local_host;
        String          milli;
        String          local_owner;
        long            return_value;
        Exception       exception_object;
        DirectoryServerException   directory_server_exception;
        StringTokenizer string_tokenizer;

        string_tokenizer = new StringTokenizer(session_key, 
                  SESSION_KEY_SEPARATOR);

        if (string_tokenizer.hasMoreTokens() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        local_host = string_tokenizer.nextToken();
        if (string_tokenizer.hasMoreTokens() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        milli = string_tokenizer.nextToken();
        if (string_tokenizer.hasMoreTokens() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_FORMAT_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        local_owner = string_tokenizer.nextToken();
        return_value = Long.valueOf(milli).longValue();
        return return_value;
    }

    public String mkKeyDirectory() throws Exception {
        File         fd;
        String       key_path;
        key_path = KEY_DIRECTORY;
        fd = new File(key_path);
        fd = fd.getAbsoluteFile();
        if (fd.exists() == false) {
            fd.mkdir();
        }
        return null;
    }

    public String generateSessionKey(String owner, SealedObject sealed_password) 
        throws Exception
    {
        String                key;
        String                key_path;
        String                password;
        InetAddress           inet_address;
        String                hostname;
        long                  current_milli_time;
        File                  fd;
        FileOutputStream      file_output_stream;
        ObjectOutputStream    object_output_stream;
        DirectoryServerException    directory_server_exception;

        try {
            inet_address = InetAddress.getLocalHost();
            hostname = inet_address.getHostName();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.NETWORK_ADDRESS_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.NETWORK_ADDRESS_ERROR_EXCEPTION)));
            throw directory_server_exception;
        }
        mkKeyDirectory();
        for (int i = 0; i < LOOP_LIMIT; i++) {
            current_milli_time = System.currentTimeMillis();
            key = hostname + SESSION_KEY_SEPARATOR
                  + String.valueOf(current_milli_time) + SESSION_KEY_SEPARATOR
                  + owner ;

            key_path = KEY_DIRECTORY + File.separator + key;
            fd = new File(key_path);
            fd = fd.getAbsoluteFile();
            if (fd.createNewFile() == true) {
                try {
                    file_output_stream = new FileOutputStream(fd);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                    throw directory_server_exception;
                }
                try {
                    object_output_stream = new ObjectOutputStream(file_output_stream);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM)));
                    throw directory_server_exception;
                }
                try {
                    object_output_stream.writeObject(owner);
                    object_output_stream.writeObject(sealed_password);
                    object_output_stream.close();
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT)));
                    throw directory_server_exception;
                }
                return key;
            }
            try {
                Thread.sleep(INTERVAL);
            } catch (Exception exception_object) {
            }
        }
        directory_server_exception = new DirectoryServerException(
            getMessageResource().getFormattedMessage(
                DirectoryServerException.SESSION_TIMEOUT,
                getMessageResource().getMessage(
                    DirectoryServerException.SESSION_TIMEOUT)));
        throw directory_server_exception;
    }

    public String login(String owner, SealedObject sealed_password) 
        throws Exception
    {
        String key_string;
        String password;
        DirectoryServerException directory_server_exception;

        if (owner == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (sealed_password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        password = getPasswordFromSealedObject(sealed_password);
        try {
            if (check(owner, password) == false) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION)));
                throw directory_server_exception;
            }
        } catch (Exception exception_object) {
            if (exception_object.getMessage().compareTo(
                DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION) == 0) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION)));
                throw directory_server_exception;
            }
            if (exception_object.getMessage().compareTo(
                DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION) == 0) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION)));
                throw directory_server_exception;
            }
            throw exception_object;
        }
        try {
            key_string = generateSessionKey(owner, sealed_password) ;
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return key_string;
    }

    public boolean logout(String session_key) 
        throws Exception
    {
        boolean flag;

        flag = false;
        flag = removeSessionKey(session_key);
        return flag;
    }

    private String getPasswordFromSealedObject(SealedObject sealed_password)
        throws Exception 
    {
        String              password_string;
        Cipher              ci;
        SecretKeySpec       secret_key_spec;
        DESKeySpec          des_key_spec;
        KeyFactory          key_factory;
        PublicKey           public_key;
        PrivateKey          private_key;
        SealedObject        sealed_object;
        FileInputStream     file_input_stream;
        ObjectInputStream   object_input_stream;
        DirectoryServerException   directory_server_exception;

        try {
            ci = Cipher.getInstance(des_key);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ENCRYPT_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.ENCRYPT_EXCEPTION)));
            throw directory_server_exception;
        }
        secret_key_spec = new SecretKeySpec(password_key.getBytes(), des_key);
        try {
            ci.init(Cipher.DECRYPT_MODE, secret_key_spec);
            password_string = (String)sealed_password.getObject(ci);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ENCRYPT_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.ENCRYPT_EXCEPTION)));
            throw directory_server_exception;
        }
        return password_string;
    }

    public boolean register(String owner, SealedObject sealed_password)
        throws Exception
    {
        DirectoryServerException directory_server_exception;
        FileOutputStream         file_output_stream;
        ObjectOutputStream       object_output_stream;
        FileInputStream          file_input_stream;
        ObjectInputStream        object_input_stream;
        String                   password;
        String                   read_owner;
        String                   read_password;
        SealedObject             read_sealed_password;
        byte[]                   buffer;
        ByteArrayOutputStream    byte_array_output_stream;

        if (owner == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (sealed_password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        password = getPasswordFromSealedObject(sealed_password);
        try {
            check(owner, password);
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ENTRY_ALREADY_EXIST,
                    getMessageResource().getMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST)));
            throw directory_server_exception;
        } catch (Exception exception_object) {
            if (exception_object.getMessage().compareTo(
                DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION) != 0
                && exception_object.getMessage().compareTo(
                DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION) != 0) {
                throw exception_object;
            }
        }

        File fdes = new File(USER_DATABASE);
        String user_database = fdes.getAbsolutePath();
        fdes = new File(user_database);
        boolean flag;

        if (fdes.exists() == false) {
            try {
                flag = fdes.createNewFile();
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.BUSY_WAIT,
                        getMessageResource().getMessage(
                            DirectoryServerException.BUSY_WAIT)));
                throw directory_server_exception;
            }
            if (flag == false) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.BUSY_WAIT,
                        getMessageResource().getMessage(
                            DirectoryServerException.BUSY_WAIT)));
                throw directory_server_exception;
            }
            try {
                file_output_stream = new FileOutputStream(fdes);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                throw directory_server_exception;
            }
            try {
                object_output_stream = new ObjectOutputStream(file_output_stream);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM)));
                throw directory_server_exception;
            }
            try {
                object_output_stream.writeObject(owner);
                object_output_stream.writeObject(sealed_password); 
                object_output_stream.close();
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT)));
                throw directory_server_exception;
            }
            createDirectory(owner);
            return true;
        }
        try {
            file_input_stream = new FileInputStream(fdes);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            throw directory_server_exception;
        }
        try {
            object_input_stream = new ObjectInputStream(file_input_stream);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM)));
            throw directory_server_exception;
        }
        byte_array_output_stream = new ByteArrayOutputStream();
        try {
            object_output_stream = new ObjectOutputStream(byte_array_output_stream);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM)));
            throw directory_server_exception;
        }
        while(true) {
            try {
                read_owner = (String)object_input_stream.readObject();
                read_sealed_password = (SealedObject)object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT)));
                throw directory_server_exception;
            }
            try {
                object_output_stream.writeObject(read_owner);
                object_output_stream.writeObject(read_sealed_password);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT)));
                throw directory_server_exception;
            }
        }
        try {
            object_output_stream.writeObject(owner);
            object_output_stream.writeObject(sealed_password); 
            object_output_stream.close();
            object_input_stream.close();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT)));
            throw directory_server_exception;
        }
        buffer = byte_array_output_stream.toByteArray();
        try {
            file_output_stream = new FileOutputStream(fdes);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
            throw directory_server_exception;
        }
        try {
            file_output_stream.write(buffer);
            file_output_stream.close();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE)));
            throw directory_server_exception;
        }
        createDirectory(owner);
        return true;
    }

    public boolean erase(String session_key)
        throws Exception
    {
        String        entry;
        String   read_entry;
        String        owner;
        String     password;
        String   read_owner;
        SealedObject  read_sealed_password;
        String   read_password;
        DirectoryServerException directory_server_exception;
        FileInputStream  fis;
        FileOutputStream fos;
        DataInputStream din;
        ByteArrayOutputStream bos;
        PrintStream pst;
        byte[]   buffer;
        ObjectOutputStream   object_output_stream;
        ObjectInputStream    object_input_stream;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            password = getPasswordfromSessionFile(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        File fdes = new File(USER_DATABASE);
        String user_database = fdes.getAbsolutePath();
        fdes = new File(user_database);
        if (fdes.exists() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            fis = new FileInputStream(fdes);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            throw directory_server_exception;
        }
        try {
            object_input_stream = new ObjectInputStream(fis);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM)));
            throw directory_server_exception;
        }
        bos = new ByteArrayOutputStream();
        try {
            object_output_stream = new ObjectOutputStream(bos);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM)));
            throw directory_server_exception;
        }
        while (true) {
            try {
                read_owner = (String)object_input_stream.readObject();
                read_sealed_password = (SealedObject)object_input_stream.readObject();
                read_password = getPasswordFromSealedObject(read_sealed_password);
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT)));
                throw directory_server_exception;
            }
            if (read_owner.compareTo(owner) == 0 
                && read_password.compareTo(password) == 0) {
                while(true) {
                    try {
                        read_owner = (String)object_input_stream.readObject();
                        read_sealed_password = (SealedObject)object_input_stream.readObject();
                    } catch (Exception exception_object) {
                        if ((exception_object instanceof EOFException) == true) {
                            break;
                        }
                        directory_server_exception = new DirectoryServerException(
                            getMessageResource().getFormattedMessage(
                                DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT,
                                getMessageResource().getMessage(
                                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT)));
                        throw directory_server_exception;
                    }
                    try {
                        object_output_stream.writeObject(read_owner);
                        object_output_stream.writeObject(read_sealed_password);
                    } catch (Exception exception_object) {
                        directory_server_exception = new DirectoryServerException(
                            getMessageResource().getFormattedMessage(
                                DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT,
                                getMessageResource().getMessage(
                                    DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT)));
                        throw directory_server_exception;
                    }
                }
                try {
                    object_input_stream.close();
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_CLOSE,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_CLOSE)));
                    throw directory_server_exception;
                }
                buffer = bos.toByteArray();
                try {
                    fos = new FileOutputStream(fdes);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                    throw directory_server_exception;
                }
                try {
                    fos.write(buffer);
                    fos.close();
                    deleteDirectory(owner);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE)));
                    throw directory_server_exception;
                }
                return true;
            }
            try {
                object_output_stream.writeObject(read_owner);
                object_output_stream.writeObject(read_sealed_password);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT)));
                throw directory_server_exception;
            }
        }
        
        directory_server_exception = new DirectoryServerException(
            getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_EXCEPTION,
                getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_EXCEPTION)));
        throw directory_server_exception;
    }

    public String user(String session_key)
        throws Exception
    {
        int       idx;
        int       count;
        String[]  entry_list;
        String    read_entry;
        String    read_owner;
        SealedObject    read_sealed_password;
        String    read_password;
        FileInputStream fis;
        DataInputStream din;
        ObjectInputStream  object_input_stream;
        String   entry_list_string;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }

        File fdes = new File(USER_DATABASE);
        String user_database = fdes.getAbsolutePath();
        fdes = new File(user_database);
        if (fdes.exists() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            fis = new FileInputStream(fdes);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            throw directory_server_exception;
        }

        try {
            object_input_stream = new ObjectInputStream(fis);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM)));
            throw directory_server_exception;
        }
        count = 0;
        java.util.ArrayList  entry = new java.util.ArrayList();
        while (true) {
            try {
                read_owner = (String)object_input_stream.readObject();
                if (read_owner == null) {
                    break;
                }
                read_sealed_password = (SealedObject)object_input_stream.readObject();
                read_password = getPasswordFromSealedObject(read_sealed_password);
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT)));
                throw directory_server_exception;
            }
            entry.add((Object)read_owner);
        }
        Object[] object_list;
        object_list = (Object[])entry.toArray();

        entry_list_string = new String();
        for (int i = 0; i < object_list.length; i++) {
            if (i != 0) {
                entry_list_string = entry_list_string.concat(RETURN_STRING);
            }
            entry_list_string = entry_list_string.concat((String)object_list[i]);
        }
        return entry_list_string;
    }

    public boolean check(String owner, String password) 
        throws Exception {

        DirectoryServerException directory_server_exception;
        FileOutputStream fos;
        PrintStream      pst;
        DataInputStream  din;
        FileInputStream  fis;
        ObjectInputStream  object_input_stream;
        String         entry;
        String    read_entry;
        String        read_owner;
        String        read_password;
        SealedObject  read_sealed_password;

        if (owner == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        File fdes = new File(USER_DATABASE);
        String user_database = fdes.getAbsolutePath();
        fdes = new File(user_database);
        if (fdes.exists() == false) {
/*
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION)));
*/
            directory_server_exception = new DirectoryServerException(
                    DirectoryServerException.DATABASE_NOT_FOUND_EXCEPTION);
            throw directory_server_exception;
        }

        try {
            fis = new FileInputStream(fdes);
        } catch (Exception exception_object) {

            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM)));

            throw directory_server_exception;
        }
        try {
            object_input_stream = new ObjectInputStream(fis);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_OBJECT_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM)));

            throw directory_server_exception;
        }
        while (true) {
            try {
                read_owner = (String)object_input_stream.readObject();
                if (read_owner == null) { break; }
                read_sealed_password = (SealedObject)object_input_stream.readObject();
                read_password = getPasswordFromSealedObject(read_sealed_password);
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_OBJECT_INPUT_STREAM_READ_OBJECT)));
                throw directory_server_exception;
            }
            if (read_owner == null || read_password == null) {
                continue;
            }
            if (read_owner.compareTo(owner) == 0) {
                if (read_password.compareTo(password) == 0) {
                    return true;
                }
                return false;
            }
        }
/*
        directory_server_exception = new DirectoryServerException(
            getMessageResource().getFormattedMessage(
                DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION)));
*/
        directory_server_exception = new DirectoryServerException(
                DirectoryServerException.USER_ENTRY_NOT_FOUND_EXCEPTION);
        throw directory_server_exception;
    }

    public void setAppend() 
    {
        write_mode = APPEND_WRITE;
    }

    public void setNormal()
    {
        write_mode = NORMAL_WRITE;
    }

    public boolean isAppend() 
    {
        if (write_mode == APPEND_WRITE) {
            return true;
        }
        return false;
    }
    /**
     *
     * @param   URI  odir://hostname:port/target
     */
    public boolean put(URI key, byte[] buffer, String session_key)
        throws Exception
    {
        boolean flag;

        setNormal();
        flag = writeData(key, buffer, session_key);
        return flag;
    }

    public boolean writeData(URI key, byte[] buffer, String session_key)
        throws Exception
    {
        File    pfd;
        File    fdes;
        String  path;
        String  owner;
        String  home_path;
        FileOutputStream fos;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        
        fdes = new File(path);

        boolean mode_flag;
        mode_flag = isAppend();

        if (mode_flag == false) {
            if (fdes.exists() == true) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.BUSY_WAIT,
                        getMessageResource().getMessage(
                            DirectoryServerException.BUSY_WAIT)));
                throw directory_server_exception;
            }
        }

        int  count;
        count = 0;
        pfd = fdes;
        while (true) {
            pfd = pfd.getParentFile();
            if (pfd.exists() == false) { 
                count = count + 1;
                continue;
            }
            break;
        }
        if (count > 0) {
            fdes.getParentFile().mkdirs();
        }

        boolean flag;
        if (mode_flag == false || fdes.exists() == false) {
            try {
                flag = fdes.createNewFile();
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.BUSY_WAIT,
                        getMessageResource().getMessage(
                            DirectoryServerException.BUSY_WAIT)));
                throw directory_server_exception;
            }
            if (flag == false) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.BUSY_WAIT,
                        getMessageResource().getMessage(
                            DirectoryServerException.BUSY_WAIT)));
                throw directory_server_exception;
            }
        }
        if (mode_flag == false) {
            try {
                fos = new FileOutputStream(fdes);
            } catch (Exception   exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                throw directory_server_exception;
            }
        } else {
            try {
                fos = new FileOutputStream(fdes, true);
            } catch (Exception   exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                throw directory_server_exception;
            }
        }
        try {
            fos.write(buffer);
            fos.close();
        } catch (Exception exception_object) {
        }
        return true;
    }

    public byte[] cut(URI key, String session_key, long offset, int size)
        throws Exception
    {
        setPosition(offset);
        return readData(key, session_key, size);
    }

    public void setPosition(long offset) {
        read_position = offset;
    }

    public void resetPosition() {
        read_position = 0;
    }

    public long getPosition() {
        return read_position;
    }


    public byte[] get(URI key, String session_key)
        throws Exception
    {
        resetPosition();
        return readData(key, session_key, FILE_SIZE_MODE);
    }

    public byte[] readData(URI key, String session_key, int psize)
        throws Exception
    {
        int     size;
        byte[]  buffer;
        String  path;
        int     rcnt;
        long    offset;
        int     rcount;
        String  owner;
        FileInputStream fis;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
           throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        File fdes = new File(path);
        if (fdes.exists() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (psize == FILE_SIZE_MODE) {
            size = (int)fdes.length();
        } else {
            size = psize;
        }
        buffer = new byte[size];
        fis = new FileInputStream(fdes);
        offset = getPosition();
        fis.skip(offset);
        
        rcnt = 0;
        while (true) {
            try {
                rcount = fis.read(buffer, (int)rcnt, (int)size - rcnt);
                if (rcount == -1) {
                    break;
                }
                if (rcnt + rcount >= (int)size) {
                    fis.close();
                    break;
                }
                rcnt = rcnt + rcount;
              
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return buffer;
    }

    /**
     *
     * @param   URI  odir://hostname:port/target
     */
    public boolean append(URI key, byte[] buffer, String session_key)
        throws Exception
    {
        boolean flag;

        setAppend();
        flag = writeData(key, buffer, session_key);
        return flag;
    }


    public byte[] execute(URI key, byte[] input, String session_key)
        throws Exception
    {
        int    idx;
        int    count;
        File   fd;
        File   fd2;
        PrintStream save_out;
        PrintStream save_err;
        String path;
        String suffix;
        String jar_path;
        String jar_string;
        String class_name;
        String method_name;
        String base_path;
        String count_string;
        Integer count_integer;
        Method[] method;
        Object[] param;
        Object[] param_string;
        Class   class_object;
        Class[] ptypes;
        Object  object_instance;
        byte[] output_buffer = null;
        byte[] buffer;
        URL[] urls;
        URL jar_url;
        File fdes;
        String owner;
        DirectoryServerException directory_server_exception;
        FileInputStream fin;
        Class  ReturnType;
        ByteArrayOutputStream baos1;
        PrintStream sysout;
        PrintStream syserr;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        path = getDirectory(owner) + File.separator + key.getPath();
        fdes = new File(path);
        if (fdes.exists() == false || fdes.isDirectory() == true) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }

        idx = key.getPath().lastIndexOf(DOT);
        if (idx < 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        suffix = key.getPath().substring(idx + 1);
        if (suffix.compareTo(CLASS_SUFFIX) != 0 &&
            suffix.compareTo(JAR_SUFFIX) != 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }

        ByteArrayInputStream bis = new ByteArrayInputStream(input);
        DataInputStream dis = new DataInputStream(bis);

        try {
            class_name = dis.readLine();
            method_name = dis.readLine();
            count_string = dis.readLine();
            count_integer = new Integer(count_string);
            count = count_integer.intValue();
            param = new Object[count]; 
            param_string = new Object[count]; 
            for (int i = 0; i < count; i++) {
                param_string[i] = (Object)dis.readLine();
                if (param_string[i] == null) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.INPUT_PARAMETER_EXCEPTION,
                            getMessageResource().getMessage(
                                DirectoryServerException.INPUT_PARAMETER_EXCEPTION)));
                    throw directory_server_exception;
                }
            }
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.INPUT_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.INPUT_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        fd = new File(owner);

        base_path = fd.getAbsolutePath();
        if (suffix.compareTo(CLASS_SUFFIX) == 0) {
            jar_path = base_path + File.separator + key.getPath();
            fd2 = new File(jar_path);
            buffer = new byte[(int)fd2.length()];
     
            try {
                fin = new FileInputStream(fd2);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
                throw directory_server_exception;
            }
            try {
                fin.read(buffer, 0, buffer.length);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM_READ,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_INPUT_STREAM_READ)));
                throw directory_server_exception;
            }
            try {
                class_object = defineClass(
                    class_name, buffer, 0, buffer.length);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION)));
                throw directory_server_exception;
            }
        } else if (suffix.compareTo(JAR_SUFFIX) == 0) {

            jar_path = base_path + File.separator + key.getPath();
            jar_string = new String(JAR_URL_HEAD + jar_path + JAR_URL_END);
            try {
                jar_url = new URL(jar_string);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_URL_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_URL_EXCEPTION)));
                throw directory_server_exception;
            }
            urls = new URL[1];
            urls[0] = jar_url;
            URLClassLoader uloader = new URLClassLoader(urls);

            try {
                class_object = Class.forName(class_name, true, uloader);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_CLASS_FOR_NAME,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_CLASS_FOR_NAME)));
                throw directory_server_exception;
            }
        } else {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }

        try {
            count = 0;
            method = class_object.getMethods();
            for ( ; count < method.length; count++) {
                if (method[count].getName().compareTo(method_name) == 0) {
                    ptypes = method[count].getParameterTypes();
                    if (param_string.length != ptypes.length) {
                        continue;
                    }
                    int  i;
                    for (i = 0; i < ptypes.length; i++) {
                        if (ptypes[i].getName().indexOf(ARRAY_KEY) >= 0) {
                            directory_server_exception = new DirectoryServerException(
                                getMessageResource().getFormattedMessage(
                                    DirectoryServerException.NOT_SUPPORT_PARAMETER_TYPE_EXCEPTION,
                                    getMessageResource().getMessage(
                                        DirectoryServerException.NOT_SUPPORT_PARAMETER_TYPE_EXCEPTION)));
                            throw directory_server_exception;
                        }
                        if (ptypes[i].getName().compareTo(STRING_PARAMETER_TYPE)!=0){
                            try {
                                if (ptypes[i].getName().compareTo(INT_NATIVE_TYPE_KEY) == 0) {
                                    int  value;
                                    value = Integer.valueOf((String)param_string[i]).intValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(LONG_NATIVE_TYPE_KEY) == 0) {
                                    long  value;
                                    value = Long.valueOf((String)param_string[i]).longValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(FLOAT_NATIVE_TYPE_KEY) == 0) {
                                    float  value;
                                    value = Float.valueOf((String)param_string[i]).floatValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(DOUBLE_NATIVE_TYPE_KEY) == 0) {
                                    double  value;
                                    value = Double.valueOf((String)param_string[i]).doubleValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(BOOLEAN_NATIVE_TYPE_KEY) == 0) {
                                    boolean  value;
                                    value = Boolean.valueOf((String)param_string[i]).booleanValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(BYTE_NATIVE_TYPE_KEY) == 0) {
                                    byte  value;
                                    value = Byte.valueOf((String)param_string[i]).byteValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(CHAR_NATIVE_TYPE_KEY) == 0) {
                                    char  value;
                                    value = (char)Integer.valueOf((String)param_string[i]).intValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(SHORT_NATIVE_TYPE_KEY) == 0) {
                                    short  value;
                                    value = Short.valueOf((String)param_string[i]).shortValue();
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(INTEGER_OBJECT_TYPE_KEY) == 0) {
                                    Integer  value;
                                    value = Integer.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(LONG_OBJECT_TYPE_KEY) == 0) {
                                    Long  value;
                                    value = Long.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(FLOAT_OBJECT_TYPE_KEY) == 0) {
                                    Float  value;
                                    value = Float.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(DOUBLE_OBJECT_TYPE_KEY) == 0) {
                                    Double  value;
                                    value = Double.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(SHORT_OBJECT_TYPE_KEY) == 0) {
                                    Short  value;
                                    value = Short.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(CHARACTER_OBJECT_TYPE_KEY) == 0) {
                                    Character  value;
                                    value = Character.valueOf((char)
                                        Integer.valueOf((String)param_string[i]).intValue() );
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(BOOLEAN_OBJECT_TYPE_KEY) == 0) {
                                    Boolean  value;
                                    value = Boolean.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
                                } else if (ptypes[i].getName().compareTo(BYTE_OBJECT_TYPE_KEY) == 0) {
                                    Byte  value;
                                    value = Byte.valueOf((String)param_string[i]);
                                    param[i] = (Object)value;
				} else {
                                    directory_server_exception = new DirectoryServerException(
                                        getMessageResource().getFormattedMessage(
                                            DirectoryServerException.NOT_SUPPORT_PARAMETER_TYPE_EXCEPTION,
                                            getMessageResource().getMessage(
                                                DirectoryServerException.NOT_SUPPORT_PARAMETER_TYPE_EXCEPTION)));
                                    throw directory_server_exception;
                                }
                            } catch (Exception exception_object) {
                                directory_server_exception = new DirectoryServerException(
                                        getMessageResource().getFormattedMessage(
                                            DirectoryServerException.NOT_SUPPORT_PARAMETER_TYPE_EXCEPTION,
                                            getMessageResource().getMessage(
                                                DirectoryServerException.NOT_SUPPORT_PARAMETER_TYPE_EXCEPTION)));
                                throw directory_server_exception;
                            }

                        } else {
                            param[i] = param_string[i];
                        }
                    }
                    if (i != ptypes.length) {
                        continue;
                    }
                    break;
                }
             
            }
            if (count == method.length) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.METHOD_PARAMETER_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.METHOD_PARAMETER_EXCEPTION)));
                throw directory_server_exception;
            }

            object_instance = class_object.newInstance();

            save_out = System.out;
            save_err = System.err;

            baos1 = new ByteArrayOutputStream();

            sysout = new PrintStream(baos1);
            syserr = sysout;
            System.setOut(sysout);
            System.setErr(syserr);

            ReturnType = method[count].getReturnType();
            if (ReturnType.getName().compareTo(VOID_TYPE_KEY) == 0) {
                method[count].invoke(object_instance, (Object[])param);
            } else if (ReturnType.getName().compareTo(INT_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);
                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(LONG_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);
                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(FLOAT_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);
                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(DOUBLE_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);
                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(BOOLEAN_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);
                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(BYTE_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(CHAR_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(SHORT_NATIVE_TYPE_KEY) == 0) {
                Object return_value;
                return_value = method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(STRING_OBJECT_TYPE_KEY) == 0) {
                String return_value;
                return_value = (String)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value);
            } else if (ReturnType.getName().compareTo(INTEGER_OBJECT_TYPE_KEY) == 0) {
                Integer return_value;
                return_value = (Integer)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(LONG_OBJECT_TYPE_KEY) == 0) {
                Long return_value;
                return_value = (Long)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(FLOAT_OBJECT_TYPE_KEY) == 0) {
                Float return_value;
                return_value = (Float)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(DOUBLE_OBJECT_TYPE_KEY) == 0) {
                Double return_value;
                return_value = (Double)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(SHORT_OBJECT_TYPE_KEY) == 0) {
                Short return_value;
                return_value = (Short)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(CHARACTER_OBJECT_TYPE_KEY) == 0) {
                Character return_value;
                return_value = (Character)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(BOOLEAN_OBJECT_TYPE_KEY) == 0) {
                Boolean return_value;
                return_value = (Boolean)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else if (ReturnType.getName().compareTo(BYTE_OBJECT_TYPE_KEY) == 0) {
                Byte return_value;
                return_value = (Byte)method[count].invoke(object_instance, (Object[])param);

                sysout.println();
                sysout.println(return_value.toString());
            } else {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.NOT_SUPPORT_RETURN_TYPE_EXCEPTION,
                        getMessageResource().getMessage(
                            DirectoryServerException.NOT_SUPPORT_RETURN_TYPE_EXCEPTION)));
                throw directory_server_exception; 
            } 
            output_buffer = baos1.toByteArray();
            System.setOut(save_out);
            System.setErr(save_err);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.METHOD_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.METHOD_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }

        return output_buffer;
    }

    public boolean delete(URI key, String session_key)
        throws Exception
    {
        int     size;
        byte[]  buffer;
        String  path;
        int     rcnt;
        int     rcount;
        String  owner;
        boolean flag;
        FileInputStream fis;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (key.getPath() == null || key.getPath().length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
    
        String key_path;
        key_path = key.getPath().replace(DIRECTORY_SEPARATOR,File.separator);
        if (key_path.compareTo(File.separator + LOCK_DIRECTORY) == 0
            || key_path.compareTo(File.separator)  == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        File fdes = new File(path);
        if (fdes.exists() == false || fdes.isDirectory() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            flag = removeDirectory(fdes);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        return flag;
    }

    public boolean directory(URI key, String session_key)
        throws Exception
    {
        File          fdes;
        String        path;
        String        owner;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (key.getPath() == null || key.getPath().length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        String key_path;
        key_path = key.getPath().replace(DIRECTORY_SEPARATOR,File.separator);
        if (key_path.compareTo(File.separator)  == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        fdes = new File(path);

        if (fdes.exists() == true) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        fdes.mkdir();
        return true;
    }

    public boolean isDirectory(URI key, String session_key) throws Exception
    {
        File          fdes;
        String        path;
        String        owner;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (key.getPath() == null || key.getPath().length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        String key_path;
        key_path = key.getPath().replace(DIRECTORY_SEPARATOR,File.separator);
        if (key_path.compareTo(File.separator)  == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        fdes = new File(path);

        if (fdes.exists() == true) {
            return true;
        }
        return false;
    }

    public String list(URI key, String session_key)
        throws Exception
    {
        File          fdes;
        String        path;
        String        owner;
        String        list_string;
        String type_message;
        String return_string;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        fdes = new File(path);
        if (fdes.exists() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (fdes.isDirectory() == false) {
            int      idx;
            String[] slist;
            slist = new String[1];
            type_message = GENERAL_FILE;
            idx = key.getPath().lastIndexOf(DOT);
            if (idx > 0) {
                String suffix;

                suffix = key.getPath().substring(idx+1);
                if (suffix.compareTo(TEXT_SUFFIX) == 0) {
                    type_message = PLAIN_TEXT;
                } else if (suffix.compareTo(JAR_SUFFIX) == 0) {
                    type_message = JAR_ARCHIVE;
                } else if (suffix.compareTo(JAVA_SUFFIX) == 0) {
                    type_message = JAVA_SOURCE;
                }  else if (suffix.compareTo(CLASS_SUFFIX) == 0) {
                    type_message = CLASS_OBJECT;
                }
            }
            return_string = new String(key.getPath() + SPACE 
                + type_message + SIZE_TEXT 
                + SPACE + String.valueOf(fdes.length()));

            slist[0] = return_string;
            return_string = new String();
            for (int i = 0; i < slist.length; i++) {
                if (i != 0) {
                    return_string = return_string.concat(RETURN_STRING);
                }
                return_string = return_string.concat(slist[i]);
            }
            return return_string;
        }
        String[] slist = fdes.list();
        /**
         * for MS-DOS
         */
        String key_path;
        list_string = new String();
        String[]  return_array;
        key_path = key.getPath().replace(DIRECTORY_SEPARATOR,File.separator);
        if (key_path.compareTo(File.separator) == 0) {
            String[] nlist = new String[slist.length - 1];
            int i = 0;
            int j = 0;
            for ( ; i < slist.length - 1; i++) {
                if (slist[i].compareTo(LOCK_DIRECTORY) == 0) {
                    continue;
                }
                nlist[j]  = slist[i];
                j = j + 1;
            }
            if (j == i) {
                if (slist[i].compareTo(LOCK_DIRECTORY) == 0) {
                    return_array = nlist;
                } else {
                    return_array = slist;
                } 
            } else {
                nlist[j] = slist[i];
                return_array = nlist;
            }
        } else {
            return_array = slist;
        }
        return_string = new String();
        for (int i = 0; i < return_array.length; i++) {
            if (i != 0) {
                return_string = return_string.concat(RETURN_STRING);
            }
            return_string = return_string.concat(return_array[i]);
        }
        return return_string;
    }

    public boolean remove(URI key, String session_key)
        throws Exception
    {
        File    fdes;
        String  path;
        boolean flag;
        String  owner;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (key.getPath() == null || key.getPath().length() == 0
            || key.getPath().compareTo(File.separator)  == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();

        fdes = new File(path);
        if (fdes.exists() == false || fdes.isDirectory() == true) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        flag = fdes.delete();
        return flag;
    }

    public boolean merge(URI key, String session_key, int count)
        throws Exception
    {
        int    idx;
        File   fd;
        File   fdes;
        String path;
        int block_size ;
        int file_size;
        int r_count;
        int length;
        int offset;
        boolean flag;
        byte[] buffer;
        String head;
        String tail;
        String owner;
        FileInputStream fin;
        FileOutputStream fos;
        DirectoryServerException directory_server_exception;
        
        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }

        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        fdes = new File(path);
        if (fdes.exists() == true) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ENTRY_ALREADY_EXIST,
                    getMessageResource().getMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST)));
            throw directory_server_exception;
        }
        try {
            fos = new FileOutputStream(fdes);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
            throw directory_server_exception;
        }

        int  i;
        for (i = 0; i < count; i++) {
            idx = path.lastIndexOf(File.separator);
            head = path.substring(0, idx + 1);
            tail = path.substring(idx + 1);
            fd = new File(head + TEMPORARY_SUFFIX + tail 
                + DOT + String.valueOf(i));
            if (fd.exists() == false) { break; }
            if (fd.length() > BUFFER_SIZE) {
                block_size = BUFFER_SIZE;
            } else {
                block_size = (int)fd.length();
            }
            buffer = new byte[block_size];
            file_size = 0;
            offset = 0;
            try {
                fin = new FileInputStream(fd);
            } catch (Exception  exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
                throw directory_server_exception;
            }
            while (true) {
                try {
                    r_count = fin.read(buffer, offset, block_size);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_INPUT_STREAM_READ,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_INPUT_STREAM_READ)));
                    throw directory_server_exception;
                }
                if (r_count == -1) {
                    break;
                }
                try {
                    fos.write(buffer, 0, r_count);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE)));
                    throw directory_server_exception;
                }
                offset = offset + r_count;
                block_size = block_size - r_count;
                if (block_size > 0) {
                    continue;
                }
                file_size = file_size + offset;
                if (fd.length() - file_size <= 0) {
                    break;
                }
                offset = 0;
                if (fd.length() - file_size > BUFFER_SIZE) {
                    block_size = BUFFER_SIZE;
                } else {
                    block_size = (int)fd.length() - file_size;
                }
            }
        }
        if (i == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        flag = delete_split(key, session_key, count);
        return flag;
    }

    public boolean delete_split(URI key, String session_key, int number)
        throws Exception
    {
        int    idx;
        String path;
        String head;
        String tail;
        String owner;
        File fdes;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        for (int i = 0; i < number; i++) {
            idx = key.getPath().lastIndexOf(File.separator);
            head = key.getPath().substring(0, idx + 1);
            tail = key.getPath().substring(idx + 1);
           
            path = getDirectory(owner) + File.separator + 
                    head + TEMPORARY_SUFFIX + tail
                    + DOT + String.valueOf(i);
            fdes = new File(path);
            fdes.delete();
        }
        return true;
    }

    public boolean split(URI key, int split_number, String session_key)
        throws Exception
    {
        int    idx;
        File   fd;
        File   fdes;
        String path;
        int block_size ;
        int file_size;
        int file_limit;
        int r_count;
        byte[] buffer;
        String head;
        String tail;
        String owner;
        FileInputStream fin;
        FileOutputStream fos;
        DirectoryServerException directory_server_exception;

        if (checkValidSession(session_key) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_TIMEOUT,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_TIMEOUT)));
            throw directory_server_exception;
        }
        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        if (split_number <= 0 || split_number > SPLIT_LIMIT) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SPLIT_NUMBER_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.SPLIT_NUMBER_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        try {
            owner = getOwnerfromSessionKey(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        path = getDirectory(owner) + File.separator + key.getPath();
        fdes = new File(path);
        if (fdes.exists() == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_EXCEPTION)));
            throw directory_server_exception;
        }
        int  i;
        fdes = new File(path);
        file_limit = ((int)fdes.length() + split_number - 1) / split_number;
        fin = new FileInputStream(path);
        for (i = 0; i < split_number; i++) {
            idx = path.lastIndexOf(File.separator);
            head = path.substring(0, idx + 1);
            tail = path.substring(idx + 1);
            fd = new File(head + TEMPORARY_SUFFIX + tail 
                + DOT + String.valueOf(i));

            if (fd.exists() == true) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST,
                        getMessageResource().getMessage(
                            DirectoryServerException.ENTRY_ALREADY_EXIST)));
                throw directory_server_exception;
            }
            try {
                fos = new FileOutputStream(fd);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                throw directory_server_exception;
            }
            if (file_limit > BUFFER_SIZE) {
                block_size = BUFFER_SIZE;
            } else {
                block_size = file_limit;
            }
            buffer = new byte[block_size];
            file_size = 0;
            while (true) {
                try {
                    r_count = fin.read(buffer, 0, block_size);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_INPUT_STREAM_READ,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_INPUT_STREAM_READ)));
                    throw directory_server_exception;
                }
                if (r_count == -1) {
                    break;
                }
                try {
                    fos.write(buffer, 0, r_count);
                } catch (Exception exception_object) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE,
                            getMessageResource().getMessage(
                                DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE)));
                    throw directory_server_exception;
                }
                file_size = file_size + r_count;
                if (file_limit > file_size) {
                    if (file_limit - file_size < BUFFER_SIZE) {
                        block_size = file_limit - file_size;
                        buffer = new byte[block_size];
                    }
                    continue;
                }
                break;
            }
            try {
                fos.close();
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_CLOSE,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_CLOSE)));
                throw directory_server_exception;
            }
        }
        return true;
    }
}
