/**
 * Copyright (C) All rights reserved.
 *
 * @Module     DirectoryService.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public interface DirectoryService {

    public String login(String owner, SealedObject password)
        throws Exception;
    public boolean logout(String session_key)
        throws Exception;

    public boolean executeOn(URI key, String session_key) throws Exception;
    public boolean executeOff(URI key, String session_key) throws Exception;

    public boolean register(String owner, SealedObject password)
        throws Exception;
    public boolean erase(String session_key)
        throws Exception;
    public String user(String session_key)
        throws Exception;

    public boolean put(URI key, byte[] buffer, String session_key)
        throws Exception;
    public byte[] get(URI key, String session_key)
        throws Exception;

    public boolean append(URI key, byte[] buffer, String session_key)
        throws Exception;
    public byte[] cut(URI key, String session_key, long offset, int size)
        throws Exception;

    public byte[] execute(URI key, byte[] input, String session_key)
        throws Exception;

    /* file */
    public boolean delete(URI key, String session_key)
        throws Exception;

    public boolean directory(URI key, String session_key)
        throws Exception;

    public boolean isDirectory(URI key, String session_key)
        throws Exception;

    public String list(URI key, String session_key)
        throws Exception;

    /* remove */
    public boolean remove(URI key, String session_key)
        throws Exception;

    public boolean merge(URI key, String session_key, int chunk)
        throws Exception;

    public boolean split(URI key, int chunk, String session_key)
        throws Exception;

    public boolean delete_split(URI key, String session_key, int count)
        throws Exception;
}
