/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Erase.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/20
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class Erase extends Command {
    public final static String ERASE_ERROR = "Erase Error";
    public final static String FALSE_ERROR = "";
    public final static String ERASE_USAGE_MESSAGE = "odir.Erase user_name password";
    public final static String ERASE_CONFIRM_MESSAGE = "Do you want to delete ";
    public final static String QUESTION_MARK = "?";
    public final static String YES_STRING = "YES";
    public final static String YE_STRING = "YE";
    public final static String Y_STRING = "Y";
    public final static String COMMAND_NAME = "odir.Erase";
    public final static String CANCEL_EXCEPTION = "Request was Cancelled.";
    public final static char separator_char = '/';

    public Erase() {
        super();
    }

    public boolean doErase(String owner, String password) 
        throws Exception {
        boolean flag;
        DirectoryServer dserv;
        String          session_key;
        SealedObject    sealed_password;
        DirectoryServerException   directory_server_exception;

        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        dserv = new DirectoryServer();
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            session_key = dserv.login(owner, sealed_password);
            flag = dserv.erase(session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return flag;    
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String owner;
        String[] param;
        DirectoryServerException   directory_server_exception;

        if (command_name == null || command_name.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ILLEGAL_COMMAND_NAME,
                    getMessageResource().getMessage(
                        DirectoryServerException.ILLEGAL_COMMAND_NAME)));
            throw directory_server_exception;
        }
        param = Register.doInteraction(command_name);

        owner = param[0];

        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (doConfirm(owner) == false) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.CANCEL_EXCEPTION,
                    getMessageResource().getMessage(
                        DirectoryServerException.CANCEL_EXCEPTION)));
            throw directory_server_exception;
        }
        return param;
    }

    public static boolean doConfirm(String owner) throws Exception {
        DataInputStream  din;
        DirectoryServerException    directory_server_exception;

        System.out.print( getMessageResource().getFormattedMessage(
            ERASE_CONFIRM_MESSAGE, 
            getMessageResource().getMessage(ERASE_CONFIRM_MESSAGE)) +
                owner + QUESTION_MARK);
        din = new DataInputStream(System.in);
      
        String  answer;

        try {
            answer = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        if ((YES_STRING.compareTo(answer.toUpperCase()) == 0) 
            || (YE_STRING.compareTo(answer.toUpperCase()) == 0) 
            || (Y_STRING.compareTo(answer.toUpperCase()) == 0)) {
            return true;
        }
        return false;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(Erase.ERASE_USAGE_MESSAGE,
                getMessageResource().getMessage(Erase.ERASE_USAGE_MESSAGE)));
    } 

    public static void main(String[] ag) {
        String[]  param;
        Erase  erase_object;

        if (ag.length != 2) {
            erase_object = new Erase();
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    param = erase_object.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println( erase_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            erase_object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println( erase_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                            erase_object.getMessageResource().getMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (erase_object.doErase(param[0], param[1]) == false) {
                        System.err.println( erase_object.getMessageResource().getFormattedMessage(
                            erase_object.ERASE_ERROR,
                            erase_object.getMessageResource().getMessage( 
                                erase_object.ERASE_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                        return;
                    }
                } catch (Exception exception_object) {
                    if (exception_object.getMessage().indexOf(CANCEL_EXCEPTION) >= 0) {
                        return;
                    }
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            erase_object.printUsage(ag);
            return;
        }
        erase_object = new Erase();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println( erase_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                erase_object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println( erase_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                erase_object.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            if (erase_object.doConfirm(ag[0]) == false) {
                System.err.println( erase_object.getMessageResource().getFormattedMessage(
                    erase_object.ERASE_ERROR,
                    erase_object.getMessageResource().getMessage( 
                        erase_object.ERASE_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
                return;
            }
            erase_object.doErase(ag[0], ag[1]);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
        return;
    }
}
