/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Execute.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class Execute extends DirectoryBase {
    public final static String EXECUTE_USAGE_MESSAGE = "odir.Execute owner password odir://host[:port]/file|file class_name method_name";
    public final static int ERROR_EXIT_CODE = -1;
    public final static int FILE_SIZE_LIMIT = 4096000;
    public final static String DESTINATION_MESSAGE = "Destination File :";
    public final static String TARGET_MESSAGE = "SOURCE File :";
    public static final String EXECUTE_ERROR = "Execute error.";
    public static String DOT_SEPARATOR = ".";
    public static String SPACE = " ";
    public final static String SIZE_TEXT  = "size";
    public final static String COMMAND_NAME = "odir.Execute";
    public final static String COUNT_MESSAGE = "Count :";
    public final static char separator_char = '/';

    public final static int ARGUMENT_NUMBER = 4;

    public final static String ZERO = "0";

    public final static String ERROR_FILE_NOT_FOUND 
        = "File not found.";

    public Execute() {
        super();
    }

    /**
     * owner password  source  destination
     */
    public static String[] doInteraction(String command_name) throws Exception {
        String[]  auth;
        String[]  param;
        DataInputStream din;
        int    count;
        String count_string;
        Integer count_integer;
        String target;
        DirectoryServerException     directory_server_exception;
                                                 
        param = new String[ARGUMENT_NUMBER];
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }

        System.out.print(
            getMessageResource().getFormattedMessage(
                Execute.TARGET_MESSAGE,
                getMessageResource().getMessage(
                    Execute.TARGET_MESSAGE)));

        din = new DataInputStream(System.in);
        try {
            target = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }

        System.out.print(
            getMessageResource().getFormattedMessage(
                Execute.COUNT_MESSAGE,
                getMessageResource().getMessage(
                    Execute.COUNT_MESSAGE)));
        din = new DataInputStream(System.in);
        try {
            count_string = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        try {
            count_integer = new Integer(count_string);
            count = count_integer.intValue();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ILLEGAL_VALUE,
                    getMessageResource().getMessage(
                        DirectoryServerException.ILLEGAL_VALUE)));
            throw directory_server_exception;
        }

        param = new String[ARGUMENT_NUMBER + count];
        param[0] = auth[0];
        param[1] = auth[1];
        param[2] = target;
        param[3] = count_string;
        
        for (int i = 0; i < count; i++) {
            param[ARGUMENT_NUMBER + i] = din.readLine();
            if (param[ARGUMENT_NUMBER + i] != null) {
                break;
            }
            
        }

        return param;
    }

    public byte[] doExecute(URI key, byte[] input, String owner, String password)
        throws Exception {
        boolean flag;
        URI uri_string;
        byte[] output_buffer;
        int    idx;
        int    length;
        int    read_count;
        int    offset;
        int    count;
        int    isize;
        File   fd;
        String size_string;
        String[]  contents;
        String word;
        DirectoryServer dserv;
        FileOutputStream fos;
        String session_key;
        SealedObject   sealed_password;
        DirectoryServerException    directory_server_exception;

        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (input == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.INPUT_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.INPUT_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }

        try {
            sealed_password = getSealedPassword(password);
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
            output_buffer = dserv.execute(key, input, session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        return output_buffer;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(
                Execute.EXECUTE_USAGE_MESSAGE,
                getMessageResource().getMessage(
                    Execute.EXECUTE_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        URI       uri_string;
        Execute   object;
        String[]  param;
        byte[] output_buffer;
        ByteArrayOutputStream bos;
        PrintStream pnt;
        String  count_string;
        String  class_name;
        String  method_name;
        Integer count_integer;
        int     count;
        byte[] input;

        if (ag.length < 5) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                object = new Execute();
                try {
                    param = Execute.doInteraction(COMMAND_NAME);
                    uri_string = object.getKey(param[2]);
                    class_name  = param[3];
                    method_name = param[4];

                    count_string = param[5];
                    count_integer = new Integer(count_string);
                    count = count_integer.intValue();

                    bos = new ByteArrayOutputStream();
                    pnt = new PrintStream(bos);

                    pnt.println(class_name);
                    pnt.println(method_name);
                    pnt.println(count_string);
                    for (int j = 0; j < count; j++) {
                        pnt.println(param[6+j]);
                    }

                    input = bos.toByteArray();
                    if ((output_buffer = object.doExecute(
                        uri_string, input, param[0], param[1]))
                        == null) {
                        System.out.println(
                            object.getMessageResource().getFormattedMessage(
                                Execute.EXECUTE_ERROR,
                                object.getMessageResource().getMessage(
                                    Execute.EXECUTE_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    System.out.println(new String(output_buffer));
                } catch (Exception exception_object) {
                    System.out.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            Execute.printUsage(ag);
            return;
        }
        object = new Execute();
        try {
            uri_string = object.getKey(ag[2]);
            bos = new ByteArrayOutputStream();
            pnt = new PrintStream(bos);

/*** class ***/
            pnt.println(ag[3]);
/*** method ***/
            pnt.println(ag[4]);
            if (ag.length > 6) {
                pnt.println(String.valueOf(ag.length - 5));
                for (int i = 5; i < ag.length; i++) {
                    pnt.println(ag[i]);
                }
            } else {
               pnt.println(ZERO);
            }
            input = bos.toByteArray();
            if ((output_buffer = object.doExecute(uri_string, 
                input, ag[0], ag[1])) == null) {
                System.out.println(
                    object.getMessageResource().getFormattedMessage(
                        Execute.EXECUTE_ERROR,
                        object.getMessageResource().getMessage(
                            Execute.EXECUTE_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
            }

            System.out.println(new String(output_buffer));
        } catch (Exception exception_object) {
            System.out.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
        return;
    }
}
