/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Get.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/20
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

import sun.misc.*;

class GetSignalHandler implements SignalHandler {
    public final static int ERROR_EXIT_CODE = -1;
    URI key_param = null;
    String session_key_param;
    int    count_param;

    public void handle(Signal sig) {
        DirectoryServer dserv;

        if (key_param != null) {
            dserv = new DirectoryServer();
            try {
                dserv.executeOff(key_param, session_key_param);
                dserv.logout(session_key_param);
            } catch (Exception exception_object) {
            }
        }
        System.exit(ERROR_EXIT_CODE);
    }

    public void setParameter(URI key, String session_key) {
        key_param            = key;
        session_key_param    = session_key;
    }
}

public class Get extends DirectoryBase {
    public final static String GET_USAGE_MESSAGE = "odir.Get owner password odir://host[:port]/file|file file";
    public static final String GET_ERROR = "Get error.";

    public final static int ERROR_EXIT_CODE = -1;
    public final static String RETURN_STRING = "\n";
    public final static long FILE_SIZE_LIMIT = 4096000;
    public final static String DESTINATION_MESSAGE = "Destination File :";
    public final static String TARGET_MESSAGE = "SOURCE File :";
    public static String DOT_SEPARATOR = ".";
    public static String SPACE = " ";
    public final static String SIZE_TEXT  = "size";
    public final static String COMMAND_NAME = "odir.Get";
    public final static char separator_char = '/';
    public final static String EMPTY_STRING = "";
    public final static String TEMPORARY_SUFFIX = ".";

    public final static String FORCE_MODE_MESSAGE 
        = "System held the exclusive status. Do you want to execute Get operation (yes/no) ?";

    public final static int ARGUMENT_NUMBER = 4;
    public final static int RETRY_LIMIT = 3;

    public final static String ERROR_FILE_NOT_FOUND 
        = "File not found.";

    public final static String HUP_SIGNAL = "HUP";
    public final static String INT_SIGNAL = "INT";
    public final static String BUS_SIGNAL = "BUS";

    public Get() {
        super();
    }

    /**
     * owner password  source  destination
     */
    public static String[] doInteraction(String command_name) throws Exception {
        String[]  auth;
        String[]  param;
        DataInputStream din;
        DirectoryServerException   directory_server_exception;
    
        param = new String[ARGUMENT_NUMBER];
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        param[0] = auth[0];
        param[1] = auth[1];

        System.out.print( getMessageResource().getFormattedMessage(
            Get.TARGET_MESSAGE,
            getMessageResource().getMessage(Get.TARGET_MESSAGE)));
        din = new DataInputStream(System.in);
        try {
            param[2] = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }

        System.out.print( getMessageResource().getFormattedMessage(
            Get.DESTINATION_MESSAGE,
            getMessageResource().getMessage(Get.DESTINATION_MESSAGE)));
        din = new DataInputStream(System.in);
      
        try {
            param[3] = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }

        return param;
    }

    public boolean doGet(URI key, String owner, String password, String param_destination)
        throws Exception {
        boolean flag;
        URI uri_string;
        byte[] buffer;
        int    idx;
        long   lidx;
        int    length;
        int    read_count;
        long   offset;
        int    count;
        int    rcount;
        int    start;
        long   lsize;
        int    rsize;
        File   fd;
        File   pfd;
        File   sfdes;
        String destination;
        String head;
        String tail;
        String suffix;
        String target;
        String size_string;
        String[]  contents;
        String   contents_string;
        String[] entries;
        String word;
        String   file_entry;
        long   transfer_size;
        DirectoryServer dserv;
        FileOutputStream fos;
        FileInputStream sfis;
        String session_key;
        SealedObject  sealed_password;
        DirectoryServerException directory_server_exception;

        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        destination = param_destination;

        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            contents_string = dserv.list(key, session_key);
            StringTokenizer stk;
            stk = new StringTokenizer(contents_string, RETURN_STRING);
            int token_count;
            token_count = stk.countTokens();
            contents = new String[token_count];
            for (int i = 0; i < token_count; i++) {
                contents[i] = stk.nextToken();
            }
            if (contents == null) {
                try {
                    dserv.logout(session_key);
                } catch (Exception exception_object_work) {
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.ERROR_FILE_NOT_FOUND,
                        getMessageResource().getMessage(
                        DirectoryServerException.ERROR_FILE_NOT_FOUND)));
                throw directory_server_exception;
            }
 	    word = new String(SIZE_TEXT + SPACE);
            idx = contents[0].indexOf(word);
            if (idx < 0) {
                try {
                    dserv.logout(session_key);
                } catch (Exception exception_object_work) {
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.ERROR_FILE_NOT_FOUND,
                        getMessageResource().getMessage(
                        DirectoryServerException.ERROR_FILE_NOT_FOUND)));
                throw directory_server_exception;
            }
            size_string = contents[0].substring(idx + word.length());
            lsize = Long.valueOf(size_string).longValue();
        } catch (Exception exception_object) {
            dserv.logout(session_key);
            throw exception_object;
        }

        fd = new File(destination);
/*
        fd = new File(fd.getAbsolutePath());
*/
        if (fd.isDirectory() == true) {
            file_entry = getFileEntry(key.getPath());
            destination = destination + File.separator + file_entry;
            fd = new File(destination);
/**
            dserv.logout(session_key);
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.FILE_IS_DIRECTORY,
                    getMessageResource().getMessage(
                        DirectoryServerException.FILE_IS_DIRECTORY)));
            throw directory_server_exception;
**/
        }
        if (lsize > FILE_SIZE_LIMIT) {
            count = (int)((lsize + FILE_SIZE_LIMIT - 1)/ FILE_SIZE_LIMIT);
            GetSignalHandler get_signalhandler;
            flag = false;
            for (int i = 0; i < RETRY_LIMIT; i++) {
                try {
                    flag = dserv.executeOn(key, session_key);
                } catch (Exception exception_object) {
                    if ((exception_object instanceof DirectoryServerException)
                      == true) {
                        try {
                            dserv.logout(session_key);
                        } catch (Exception exception_object_work) {
                        }
                        throw exception_object;
                    }
                    flag = false;
                }
                if (flag == true) { break; }
            }
            if (flag == false) {
                boolean yes_no;
                yes_no = getYesNo( getMessageResource().getFormattedMessage(
                    FORCE_MODE_MESSAGE,
                    getMessageResource().getMessage(FORCE_MODE_MESSAGE)));

                if (yes_no == false) {
                    try {
                        dserv.logout(session_key);
                    } catch (Exception exception_object) {
                        return false;
                    }
                    return true;
                }
            }
            start = 0;
            entries = new String[0];
            if (fd.exists() == true) {
                lidx = fd.length() / FILE_SIZE_LIMIT;
             
                if (lidx * FILE_SIZE_LIMIT != fd.length()) {
                    start = 0;
                    fd.delete();
                } else {
                    start = (int)lidx;
                }
            } else {
                start = 0;
            }
            for (int i = start; i < count; i++) {
                offset = i * FILE_SIZE_LIMIT;
                if (lsize - offset > FILE_SIZE_LIMIT) {
                    transfer_size = FILE_SIZE_LIMIT;
                } else {
                    transfer_size = (int)lsize - offset;
                }

                try {
                    buffer = dserv.cut(key, session_key, offset, (int)transfer_size);
                } catch (Exception exception_object) {
                    try {
                        dserv.executeOff(key, session_key);
                    } catch (Exception exception_object2) {
                    }
                    try {
                        dserv.logout(session_key);
                    } catch (Exception exception_object2) {
                    }
                    throw exception_object;
                }
                try {
                    fos = new FileOutputStream(destination, true);
                } catch (Exception exception_object) {
                    try {
                        dserv.executeOff(key, session_key);
                    } catch (Exception exception_object2) {
                    }
                    try {
                        dserv.logout(session_key);
                    } catch (Exception exception_object2) {
                    }
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                            getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
                    throw directory_server_exception;
                }
                try {
                    fos.write(buffer, 0, buffer.length);
                    fos.close();
                } catch (Exception exception_object) {
                    try {
                        dserv.executeOff(key, session_key);
                    } catch (Exception exception_object2) {
                    }
                    try {
                        dserv.logout(session_key);
                    } catch (Exception exception_object2) {
                    }
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE,
                            getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_OUTPUT_STREAM_WRITE)));
                    throw directory_server_exception;
                }
            }
            try {
                dserv.executeOff(key, session_key);
            } catch (Exception exception_object) {
                return false;
            }
            try {
                dserv.logout(session_key);
            } catch (Exception exception_object) {
                return false;
            }
            return true;
        }
        if (fd.exists() == true) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ENTRY_ALREADY_EXIST,
                    getMessageResource().getMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST)));
            try {
                dserv.logout(session_key);
            } catch (Exception exception_object) {
            }
            throw directory_server_exception;
        }
        try {
            fos = new FileOutputStream(fd);
        } catch (Exception exception_object) {
            try {
                dserv.logout(session_key);
            } catch (Exception exception_object_work) {
            }
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                DirectoryServerException.JAVA_FILE_OUTPUT_STREAM,
                    getMessageResource().getMessage(
                    DirectoryServerException.JAVA_FILE_OUTPUT_STREAM)));
            throw directory_server_exception;
        }
        dserv = new DirectoryServer();
        try {
            buffer = dserv.get(key, session_key);
            fos.write(buffer, 0, buffer.length);
        } catch (Exception exception_object) {
            try {
                dserv.logout(session_key);
            } catch (Exception exception_object_work) {
            }
            throw exception_object;
        }
        try {
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            return false;
        }
        return true;
    }

    public static void printUsage(String[] ag) {
        System.out.println( getMessageResource().getFormattedMessage(
           GET_USAGE_MESSAGE,
           getMessageResource().getMessage(GET_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        URI       uri_string;
        Get       object;
        String[]  param;

        if (ag.length != 4) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    object = new Get();
                    param = Get.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                            object.getMessageResource().getMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[2] == null || param[2].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                            object.getMessageResource().getMessage(
                                DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[3] == null || param[3].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.DESTINATION_PARAMETER_NOT_FOUND,
                            object.getMessageResource().getMessage(
                                DirectoryServerException.DESTINATION_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }

                    uri_string = object.getKey(param[2]);
                    if (object.doGet(uri_string, param[0], param[1], param[3]) == false) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            Get.GET_ERROR,
                            object.getMessageResource().getMessage( Get.GET_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            Get.printUsage(ag);
            return;
        }

        object = new Get();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                object.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[2] == null || ag[2].length() == 0) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    object.getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[3] == null || ag[3].length() == 0) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.DESTINATION_PARAMETER_NOT_FOUND,
                object.getMessageResource().getMessage(
                    DirectoryServerException.DESTINATION_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            uri_string = object.getKey(ag[2]);
            if (object.doGet(uri_string, ag[0], ag[1], ag[3]) == false) {
                System.err.println( object.getMessageResource().getFormattedMessage(
                    Get.GET_ERROR,
                    object.getMessageResource().getMessage( Get.GET_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
            }
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
        return;
    }
}
