/**
 * Copyright (C) All rights reserved.
 *
 * @Module     List.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class List extends DirectoryBase {
    public final static String LIST_ERROR = "List Error";
    public final static String LIST_USAGE_MESSAGE = "odir.List path owner password";
    public final static String RETURN_STRING = "\n";
    public final static String ROOT_PATH = "/";
    public final static int ARGUMENT_NUMBER = 3;
    public final static char separator_char = '/';
    public static String SPACE = " ";

    public final static String DIRECTORY_MESSAGE = "Directory/File :";

    public final static String COMMAND_NAME = "odir.List";

    public List() {
        super();
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String[]   auth;
        String[]   param;
        String     directory_entry;
        DataInputStream  din;
        DirectoryServerException   directory_server_exception;

        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        System.out.print( getMessageResource().getFormattedMessage(
            List.DIRECTORY_MESSAGE,
            getMessageResource().getMessage(List.DIRECTORY_MESSAGE)));
        din = new DataInputStream(System.in);
      
        try {
            directory_entry = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        if (directory_entry == null || directory_entry.length() == 0) {
            param = new String[ARGUMENT_NUMBER - 1];
            param[0] = auth[0];
            param[1] = auth[1];
            return param;
        } 
        param = new String[ARGUMENT_NUMBER];
        param[0] = auth[0];
        param[1] = auth[1];
        param[2] = directory_entry;
        return param;
    }

    public boolean doList(URI key, String owner, String password) 
        throws Exception {
        String[]  entries;
        String    entry;
        int       count;
        String entries_string;
        DirectoryServer dserv;
        String session_key;
        SealedObject   sealed_password;
        DirectoryServerException   directory_server_exception;

        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        dserv = new DirectoryServer();
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            session_key = dserv.login(owner, sealed_password);
            entries_string = dserv.list(key, session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        StringTokenizer stk;
        stk = new StringTokenizer(entries_string, RETURN_STRING);
        count = stk.countTokens();
        for (int i = 0; i < count; i++) {
            entry = stk.nextToken();
            System.out.println(entry);
        }
        return true;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(List.LIST_USAGE_MESSAGE,
                getMessageResource().getMessage(List.LIST_USAGE_MESSAGE)));
    }

    public static int getSize(String entry)  throws Exception {
        int  idx;
        int  size;
        String size_string;
        DirectoryServerException dsp;

        idx = entry.lastIndexOf(SPACE);
        if (idx > 0) {
            size_string = entry.substring(idx + 1);
        } else {
            dsp = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ILLEGAL_VALUE,
                    getMessageResource().getMessage(
                        DirectoryServerException.ILLEGAL_VALUE)));
            throw (Exception)dsp;
        }
        try {
            size = Integer.valueOf(size_string).intValue();
        } catch (Exception exception_object) {
            dsp = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ILLEGAL_VALUE,
                    getMessageResource().getMessage(
                        DirectoryServerException.ILLEGAL_VALUE)));
            throw (Exception)dsp;
        }
        return size;
    }

    public static void main(String[] ag) {
        int   idx;
        String[]  entries;
        odir.List list_object;
        String[]  param;
        URI       uri_string;

        if (ag.length != 3) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    list_object = new odir.List();
                    param = odir.List.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println( list_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                            list_object.getMessageResource().getMessage(
                                DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println( list_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            list_object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[2] == null || param[2].length() == 0 ) {
                        System.err.println( list_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                            list_object.getMessageResource().getMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param.length == 2) {
                        if ((uri_string = list_object.getKey(ROOT_PATH))
                            == null) {
                            System.err.println( list_object.getMessageResource().getFormattedMessage(
                                List.LIST_ERROR,
                                list_object.getMessageResource().getMessage( List.LIST_ERROR)));
                            System.exit(Register.ERROR_EXIT_CODE);
                        }
                    } else {
                        if ((uri_string = list_object.getKey(param[2])) == null) {
                            System.err.println( list_object.getMessageResource().getFormattedMessage(
                                List.LIST_ERROR,
                                list_object.getMessageResource().getMessage( List.LIST_ERROR)));
                            System.exit(Register.ERROR_EXIT_CODE);
                        }
                    }
                    list_object.doList(uri_string, param[0], param[1]);
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            odir.List.printUsage(ag);
            return;
        }
        list_object = new odir.List();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println( getMessageResource().getFormattedMessage(
                DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                Remove.getMessageResource().getMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println( list_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                list_object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[2] == null || ag[2].length() == 0) {
            System.err.println( list_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                list_object.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            if ((uri_string = list_object.getKey(ag[0])) == null) {
                System.err.println( list_object.getMessageResource().getFormattedMessage(
                    List.LIST_ERROR,
                    list_object.getMessageResource().getMessage( List.LIST_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
            }
            list_object.doList(uri_string, ag[1], ag[2]);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }
}
