/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Login.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/20
 */

package odir;

import java.lang.*;
import java.io.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;


public class Login extends Command {

    private final static int ARGUMENT_NUMBER = 2;
    public final static String LOGIN_USAGE_MESSAGE = "odir.Login user_name password";
    public static String LOGIN_ERROR = "Login error";
    public static String SPACE = " ";
    public final static char separator_char = '/';
    public final static int ERROR_EXIT_CODE = -1;
    public static String SESSION_KEY_MESSAGE = "Session key : ";
    public final static String COMMAND_NAME = "odir.Login";

    public static String ILLEGAL_COMMAND_NAME
        = "Illegal command name.";

    public Login() {
        super();
    }

    public static String[] doInteraction(String command_name) 
        throws Exception 
    {
        String[]   auth;
        String[]   param;
        DataInputStream  din;
        DirectoryServerException   directory_server_exception;

        if (command_name == null || command_name.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.ILLEGAL_COMMAND_NAME,
                    getMessageResource().getMessage(
                        DirectoryServerException.ILLEGAL_COMMAND_NAME)));
            throw directory_server_exception;
        }
        param = new String[ARGUMENT_NUMBER];
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        param[0] = auth[0];
        param[1] = auth[1];
        return param;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(Login.LOGIN_USAGE_MESSAGE,
                getMessageResource().getMessage(Login.LOGIN_USAGE_MESSAGE)));
    }

    public static String doLogin(String owner, String password) throws Exception
    {
        DirectoryServer dserv;
        String session_key;
        SealedObject   sealed_password;
        DirectoryServerException     directory_server_exception;

        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        try {
            sealed_password = getSealedPassword(password);
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return session_key;
    }

    public static void main(String[] args) {
        String[]  param;
        Login     login_object;
        String    session_key;
        DirectoryServerException    directory_server_exception;

        if (args.length != 2) {
            if (args.length == 0 && File.separatorChar == separator_char) {
                login_object = new Login();
                try {
                    param = Login.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        directory_server_exception = new DirectoryServerException(
                            login_object.getMessageResource().getFormattedMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                                login_object.getMessageResource().getMessage(
                                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        directory_server_exception = new DirectoryServerException(
                            login_object.getMessageResource().getFormattedMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                                login_object.getMessageResource().getMessage(
                                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(ERROR_EXIT_CODE);
                    }
                    session_key = login_object.doLogin(param[0], param[1]);
                    System.out.println( 
                        login_object.getMessageResource().getFormattedMessage(
                            Login.SESSION_KEY_MESSAGE,
                            login_object.getMessageResource().getMessage(
                                Login.SESSION_KEY_MESSAGE),
                                Login.SPACE, session_key));
                } catch (Exception exception_object) {
                    System.err.println(
                        login_object.getMessageResource().getFormattedMessage(
                            Login.LOGIN_ERROR,
                            login_object.getMessageResource().getMessage(
                                Login.LOGIN_ERROR)));
                    System.exit(ERROR_EXIT_CODE);
                }
                return;
            }
            Login.printUsage(args);
            return;
        } 
        login_object = new Login();
        if (args[0] == null || args[0].length() == 0 ) {
            directory_server_exception = new DirectoryServerException(
                login_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    login_object.getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(ERROR_EXIT_CODE);
        }
        if (args[1] == null || args[1].length() == 0 ) {
            directory_server_exception = new DirectoryServerException(
                login_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    login_object.getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(ERROR_EXIT_CODE);
        }
        try {
            session_key = login_object.doLogin(args[0], args[1]);
            System.out.println( 
                login_object.getMessageResource().getFormattedMessage(
                    Login.SESSION_KEY_MESSAGE,
                    login_object.getMessageResource().getMessage(
                        Login.SESSION_KEY_MESSAGE),
                Login.SPACE, session_key));
        } catch (Exception exception_object) {
            System.err.println(
                login_object.getMessageResource().getFormattedMessage( Login.LOGIN_ERROR,
                    login_object.getMessageResource().getMessage( Login.LOGIN_ERROR)));
            System.exit(ERROR_EXIT_CODE);
        }
    }
}
