/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Logout.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2011/09/07
 *
 */

package odir;

import java.lang.*;
import java.io.*;

public class Logout extends Command {

    private final static int ARGUMENT_NUMBER = 1;
    public static final String PARAMETER_ERROR = "Invalid parameter";
    public static final String LOGOUT_ERROR = "Logout error";
    public static final String LOGOUT_USAGE_MESSAGE = "odir.Logout session";
    public final static String COMMAND_NAME = "odir.Login";
    public final static String LOGOUT_SESSION_KEY = "Session : ";
    public final static char separator_char = '/';
    public final static int ERROR_EXIT_CODE = -1;

    public Logout() {
        super();
    }

    public static String[] doInteraction(String command_name) 
        throws Exception 
    {
        String[]   auth;
        String[]   param;
        DataInputStream  din;
        Runtime    runtime;
        String     session_key;
        DirectoryServerException    directory_server_exception;

        if (command_name == null || command_name.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    Register.ILLEGAL_COMMAND_NAME,
                    getMessageResource().getMessage(
                        Register.ILLEGAL_COMMAND_NAME)));
            throw directory_server_exception;
        }
        try {
            runtime = Runtime.getRuntime();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_RUNTIME_GET_RUNTIME,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_RUNTIME_GET_RUNTIME)));
            throw directory_server_exception;
        }
        System.out.print( getMessageResource().getFormattedMessage(
            LOGOUT_SESSION_KEY,
            getMessageResource().getMessage(LOGOUT_SESSION_KEY)));
        din = new DataInputStream(System.in);
      
        try {
            session_key = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }

        param = new String[ARGUMENT_NUMBER];
        param[0] = session_key;
        return param;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            Logout.getMessageResource().getFormattedMessage(LOGOUT_USAGE_MESSAGE,
                Logout.getMessageResource().getMessage(LOGOUT_USAGE_MESSAGE)));
    }

    public static String doLogout(String session_key) throws Exception
    {
        DirectoryServer dserv;
        DirectoryServerException   directory_server_exception;

        if (session_key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.SESSION_KEY_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.SESSION_KEY_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        try {
            dserv = new DirectoryServer();
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return session_key;
    }

    public static void main(String[] args) {
        Logout  logout_object;
        String[] param;

        param = new String[1];
        logout_object = new Logout();
        if (args.length != 1) {
            if (args.length == 0 && File.separatorChar == separator_char) {
                try {
                    param = Logout.doInteraction(COMMAND_NAME);
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                try {
                    logout_object.doLogout(param[0]);
                } catch (Exception exception_object) {
                    System.err.println(
                        logout_object.getMessageResource().getFormattedMessage(
                            Logout.LOGOUT_ERROR,
                            logout_object.getMessageResource().getMessage(
                                Logout.LOGOUT_ERROR)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
            }
        }else {
            param[0] = args[0];
        }
        try {
            logout_object.doLogout(param[0]);
        } catch (Exception exception_object) {
            System.err.println( logout_object.getMessageResource().getFormattedMessage(
                    Logout.LOGOUT_ERROR,
                    logout_object.getMessageResource().getMessage(
                        Logout.LOGOUT_ERROR)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }
}
