/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    MessageResource.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version:   1.0 2011/11/23
 *
 */

package odir;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;
import java.util.jar.*;
import java.util.zip.*;
import java.net.*;

public class MessageResource extends Object {
    private static StringObject string_object = null;
    private static StringObject format_object = null;

    private final static String ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE
        = "エントリが見つかりませんでした。";
    private final static String PARAMETER_ERROR_EXCEPTION_JAPANESE
        = "パラメータ異常が発生しました。";
    private final static String INITIALIZATION_ERROR_EXCEPTION_JAPANESE
        = "初期化例外が発生しました。";
    private final static String PARAMETER_ERROR_EXCEPTION 
        = "Parameter error exception";
    private final static String INITIALIZATION_ERROR_EXCEPTION
        = "Initialization error exception";

    private static String force_language = null;

    public final static int STRING_PAIR = 2;
    public final static int BUFFER_SIZE = 4096;

    public final static char COMMENT_CHAR = '#';
    public final static char SPACE_CHAR = ' ';

    public final static String PACKAGE_NAME = "odir";
    public final static String JAVA_CLASS_PATH = "java.class.path";
    public final static char COMMENT_LINE = '#';
    public final static String OBJECT_SEPARATOR = "/";
    public final static String OBJECT_DOT_SEPARATOR = ".";
    public final static String ADDRESS_SEPARATOR = ".";
    public final static String QUERY_SEPARATOR = "&";
    public final static String OPTION_SEPARATOR = "?";
    public final static String QUERY_PARAMETER_SEPARATOR = "=";
    public final static String MESSAGE_LIST_SEPARATOR = "=";
    public final static String MESSAGE_SEPARATOR = " ";
    public final static String SUFFIX_SEPARATOR = ".";
    public final static String DIRECTORY_SEPARATOR = "/";
    public final static String LIBRARY_SEPARATOR = ":";
    public final static String LINE_NUMBER_SEPARATOR = ":";
    public final static String PARAMETER_SEPARATOR = ",";
    public final static String VARIABLE_SEPARATOR = "_";
    public final static String START_PARENTHESIS = "(";
    public final static String END_PARENTHESIS = ")";
    public final static String JAR_SUFFIX = "jar";
    public final static String CLASS_SUFFIX = "class";
    public final static String JAVA_SUFFIX = "java";
    public final static String VARIABLE_SUFFIX = "var";
    public final static String FILE_URL_PROTOCOL = "file://";
    public final static String JAR_URL_PROTOCOL = "jar:";
    public final static String JAR_URL_HEAD = "jar:file:";
    public final static String JAR_URL_END  = "!/";
    public final static String EMPTY_MESSAGE = "";

    private static String   package_name = PACKAGE_NAME;
    private final static String[] pathlist = { JAVA_CLASS_PATH };
    private final static String format_field_name = "format";

    /**
     * message format is equal to a row of number.
     * The order means message's oder.
     * Each order is separated by ','.
     */
    private static String default_format = "0";

    public MessageResource() {
        /**
        loadLocaleMessage();
        **/
    }

    public static boolean checkMessage() {
        if (string_object == null || string_object.size() <= 0) {
            return false;
        }
        return true;
    }

    public static void setStringObject(StringObject    string_box,
        StringObject    format_box) {

        string_object = string_box;  
        format_object = format_box;
    }

    public static StringObject getStringObject() {
        return string_object;
    }

    public static StringObject getFormatObject() {
        return format_object;
    }
   
    public static void setLanguage(String language) {
        force_language = language;
        string_object = new StringObject();
        format_object = new StringObject();
        loadLocaleMessage();
        return;
    }

    public static String getLanguage() {
        return force_language;
    }

    private static boolean loadMessage(String path) throws Exception {
        int      idx;
        int      rcount;
        byte[]   buffer;
        String   line;
        Locale   lc;
        Object   inst;
        String   file_entry;
        String   language;
        String[] element;
        String   filepath;
        String   message_path;
        String   message_class;
        String   field_name;
        Field[]  field_object;
        DataInputStream        data_input_stream;
        FileInputStream        file_input_stream;
        JarInputStream         jar_input_stream;
        ByteArrayInputStream   byte_array_input_stream;
        ByteArrayOutputStream  byte_array_output_stream;

        if (path == null || string_object == null) {
            return false;
        }
        
        language = getLanguage();
        if (language == null) {
            lc = Locale.getDefault();
            /**
             * jpn - Japan
             * eng - English
             *
             * java/util/LocaleISOData.java
             */
            language = lc.getISO3Language();
        }

        file_entry = package_name + DIRECTORY_SEPARATOR +
                     package_name + VARIABLE_SEPARATOR + language;

        message_class = package_name + OBJECT_DOT_SEPARATOR +
               package_name + VARIABLE_SEPARATOR + language ;

        field_name = language;

        idx = path.lastIndexOf(SUFFIX_SEPARATOR+JAR_SUFFIX);
        if (idx > 0) {
            filepath = new String(JAR_URL_HEAD + path + JAR_URL_END);
        } else {
            filepath = new String(FILE_URL_PROTOCOL + path);
        }
                
        URL url;
        Class sp;
        try {
            url = new URL(filepath);
        } catch (Exception exception_object) {
            return false ;
        }
        URL[] urls = new URL[1];
        urls[0] = url;
        URLClassLoader uloader = new URLClassLoader(urls);      
        try {
            sp = Class.forName(
                message_class, true, (ClassLoader)uloader);
        } catch (Exception exception_object) {
            return false;
        }
        try {
            inst = sp.newInstance();
        } catch (Exception exception_object) {
            return false;
        }

        String[] message_list;
        element = new String[2];
        field_object = sp.getFields();
        for (int i = 0; i < field_object.length; i++) {
            if (field_object[i].getName().compareTo(field_name) == 0) {
                message_list = (String[])field_object[i].get(inst);
                for (int j = 0; j < message_list.length; j++) {
                    element = message_list[j].split(MESSAGE_LIST_SEPARATOR);
                    if (element.length == 2) {
                        string_object.add(element[0], element[1]);
                    }
                }
                continue;
            }
            if (field_object[i].getName().compareTo(format_field_name) == 0) {
                message_list = (String[])field_object[i].get(inst);
                for (int j = 0; j < message_list.length; j++) {
                    element = message_list[j].split(MESSAGE_LIST_SEPARATOR);
                    if (element.length == 2) {
                        format_object.add(element[0], element[1]);  
                    }
                }
            }
        }

        return false;
    }

    private static boolean isJapanese() {
        String language;
        Locale lc;

        language = getLanguage();
        if (language == null) {
            lc = Locale.getDefault();
            language = lc.getISO3Language();
        }
        if (language.compareTo(Locale.JAPAN.getISO3Language()) == 0) {
            return true;
        }
        return false;
    }

    private static String searchMessage(String key) throws Exception {
        int    index_number;
        String target_message;
        DirectoryServerException    directory_server_exception;

        if (key == null || key.length() == 0) {
            if (isJapanese() == true) {
                directory_server_exception = new DirectoryServerException(
                    PARAMETER_ERROR_EXCEPTION_JAPANESE);
            } else {
                directory_server_exception = new DirectoryServerException(
                    PARAMETER_ERROR_EXCEPTION    );
            }
            throw directory_server_exception;
        }

        if (string_object == null || string_object.size() == 0) {
            if (isJapanese() == true) {
                directory_server_exception = new DirectoryServerException(
                    INITIALIZATION_ERROR_EXCEPTION_JAPANESE    );
            } else {
                directory_server_exception = new DirectoryServerException(
                    INITIALIZATION_ERROR_EXCEPTION    );
            }
            throw directory_server_exception;
        }
        index_number = 0;
        try {
            target_message = string_object.getMessage(key);
        } catch (Exception exception_object) {
            if (isJapanese() == true) {
                directory_server_exception = new DirectoryServerException(
                    ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE);
                throw directory_server_exception;
            }
            throw exception_object;
        }

        return target_message;
    }

    private static String getMessagePathList() {
        String library_path;

        library_path = System.getProperty(pathlist[0]);
     
        return library_path;
    }

    public static String getMessage(String message_key) {
        String  message_string;
        String  message_path;
        String  message_path_list;
        StringTokenizer string_tokenizer_object;

        /**
        loadLocaleMessage();
        **/
        try {
            message_string = searchMessage(message_key);
        } catch (Exception exception_object) {
            return message_key;
        }
        return message_string;
    }


    private static void loadLocaleMessage() {
        String  message_string;
        String  message_path;
        String  message_path_list;
        StringTokenizer string_tokenizer_object;

        if (string_object != null && checkMessage() != true) {
            message_path_list = getMessagePathList();
            string_tokenizer_object = new StringTokenizer(message_path_list,
                LIBRARY_SEPARATOR);
            while (string_tokenizer_object.hasMoreTokens()) {
                message_path = string_tokenizer_object.nextToken();
                try {
                    loadMessage(message_path);
                } catch (Exception exception_object) {
                }
                if (checkMessage() == true) { 
                    break;
                }
            }
        }
    }

    private static String searchFormat(String key) throws Exception {
        String    target_message;
        int       index_number;
        DirectoryServerException    directory_server_exception;

        if (format_object == null) {
            return EMPTY_MESSAGE;
        }
        index_number = 0;
        try {
            target_message   = format_object.getMessage(key);
        } catch (Exception exception_object) {
            if (isJapanese() == true) {
                directory_server_exception = new DirectoryServerException(
                    ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE);
                throw directory_server_exception;
            }
            throw exception_object;
        }
 
        return target_message;
    }

    public static String getMessageFormat(String message_key) throws Exception {
        String format_string;
        loadLocaleMessage();
        try {
            format_string = searchFormat(message_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return format_string;
    }

    public static String getFormattedMessage(
        String message_key, String... message_list ) {
        int                index_number;
        StringTokenizer    stk;
        String             format_string;
        String             return_message;

        return_message = new String();
        try {
            format_string = getMessageFormat(message_key);
            if (format_string == null) {
                for (int i = 0; i < message_list.length; i++) {
                    return_message
                        = return_message.concat(message_list[i]);
                }
                return return_message;
            }
        } catch (Exception exception_object) {
            for (int i = 0; i < message_list.length; i++) {
                return_message
                    = return_message.concat(message_list[i]);
            }
            return return_message;
        }
        stk = new StringTokenizer(format_string, PARAMETER_SEPARATOR);
        if (stk.countTokens() == 0) {
            for (int i = 0; i < message_list.length; i++) {
                return_message
                    = return_message.concat(message_list[i]);
            }
            return return_message;
        }

        while (stk.hasMoreTokens()) {
            try {
                index_number = Integer.valueOf(stk.nextToken().trim());
            } catch (Exception exception_object) {
                continue;
            }
            if (index_number < message_list.length) {
                if (message_list[index_number].length() >= 2 && message_list[index_number].charAt(0) == '\\') {
                    if (message_list[index_number].charAt(1) != '\\') {
                        continue;
                    }
                    return_message = return_message.concat(
                        message_list[index_number].substring(1));
                }
                
                return_message 
                    = return_message.concat(message_list[index_number]);
            }
        }
        return return_message;
    }
}
