/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Put.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

import sun.misc.*;

class PutSignalHandler implements SignalHandler {
    public final static int ERROR_EXIT_CODE = -1;
    URI key_param = null;
    String session_key_param;
    int    count_param;

    public void handle(Signal sig) {
        DirectoryServer dserv;

        if (key_param != null) {
            dserv = new DirectoryServer();
            try {
                dserv.executeOff(key_param, session_key_param);
                dserv.logout(session_key_param);
            } catch (Exception exception_object) {
            }
        }
        System.exit(ERROR_EXIT_CODE);
    }

    public void setParameter(URI key, String session_key) {
        key_param      = key;
        session_key_param = session_key;
    }
}

public class Put extends DirectoryBase {
    public final static String PUT_USAGE_MESSAGE = "odir.Put owner password file odir://host[:port]/path_of_file";
    public final static int ERROR_EXIT_CODE = -1;
    public final static long FILE_SIZE_LIMIT = 4096000;
    public final static String RETURN_STRING = "\n";

    public final static String TEMPORARY_SUFFIX = ".";
    public final static String EMPTY_STRING = "";

    public final static String DESTINATION_MESSAGE = "Destination File :";
    public final static String TARGET_MESSAGE = "SOURCE File :";
    public final static String DOT_SEPARATOR = ".";
    public final static String SPACE = " ";
    public final static String SIZE_TEXT  = "size";
    public final static String COMMAND_NAME = "odir.Put";
    public final static String  EXCEED_FILE_SIZE 
        = "File's size is too large.";

    public static final String PUT_ERROR = "Put error.";
    public final static char separator_char = '/';

    public final static int ARGUMENT_NUMBER = 4;
    public final static int RETRY_LIMIT = 3;

    public final static String FORCE_MODE_MESSAGE 
        = "System held the exclusive status. Do you want to execute Put operation (yes/no) ?";

    public final static String HUP_SIGNAL = "HUP";
    public final static String INT_SIGNAL = "INT";
    public final static String BUS_SIGNAL = "BUS";


    public Put() {
        super();
    }

    /**
     * owner password  source  destination
     */
    public static String[] doInteraction(String command_name) throws Exception {
        String[]  auth;
        String[]  param;
        DataInputStream din;
     
    
        param = new String[ARGUMENT_NUMBER];
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        param[0] = auth[0];
        param[1] = auth[1];

        System.out.print( getMessageResource().getFormattedMessage(
            Put.TARGET_MESSAGE,
            getMessageResource().getMessage(Put.TARGET_MESSAGE)));
        din = new DataInputStream(System.in);
        try {
            param[2] = din.readLine();
        } catch (Exception exception_object) {
            throw exception_object;
        }

        System.out.print( getMessageResource().getFormattedMessage(
            Put.DESTINATION_MESSAGE,
            getMessageResource().getMessage(Put.DESTINATION_MESSAGE)));

        din = new DataInputStream(System.in);
      
        try {
            param[3] = din.readLine();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return param;
    }

    public boolean doPut(URI key_org, String owner, String password, String source)
        throws Exception {
        boolean flag;
        URI uri_string;
        URI    key;
        byte[] buffer;
        int    length;
        int    read_count;
        int    offset;
        int    count;
        int    off;
        int    rz;
        int    start;
        int    idx;
        File   fd;
        String head;
        String tail;
        String suffix;
        DirectoryServer dserv;
        FileInputStream fis;
	String file_name;
        String session_key;
        SealedObject   sealed_password;
        DirectoryServerException    directory_server_exception;

        if (key_org == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                        getMessageResource().getMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        session_key = null;
        key = key_org;

        fd = new File(source);
        if (fd.isDirectory() == true) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.FILE_IS_DIRECTORY,
                    getMessageResource().getMessage(
                        DirectoryServerException.FILE_IS_DIRECTORY)));
            throw directory_server_exception;
        }
        start = 0;
        if (fd.length() > FILE_SIZE_LIMIT) {
     
            PutSignalHandler put_signalhandler;

            try {
                fis = new FileInputStream(fd);
            } catch (Exception exception_object) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                        getMessageResource().getMessage(
                            DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
                throw directory_server_exception;
            }
            try {
                sealed_password = getSealedPassword(password);
                dserv = new DirectoryServer();
                session_key = dserv.login(owner, sealed_password);
            } catch (Exception exception_object) {
                throw exception_object;
            }
/** */
            boolean  dflag;
            try {
                dflag = dserv.isDirectory(key, session_key);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (dflag == true) {
                key = expandKey(key_org, source);
            }
/*  **/
            count = (int)((fd.length() + FILE_SIZE_LIMIT - (long)1)/ FILE_SIZE_LIMIT);

            if (File.separatorChar == separator_char) {
                put_signalhandler = new PutSignalHandler();
                put_signalhandler.setParameter(key, session_key);
            }
            flag = false;
            for (int i = 0; i < RETRY_LIMIT; i++) {
                try {
                    flag = dserv.executeOn(key, session_key);
                } catch (Exception exception_object) {
                    if ((exception_object instanceof DirectoryServerException)
                      == true) {
                        dserv.logout(session_key);
                        throw exception_object;
                    }
                    flag = false;
                }
                if (flag == true) { break; }
            }
            if (flag == false) {
                boolean yes_no;
                yes_no = getYesNo(FORCE_MODE_MESSAGE);
                if (yes_no == false) {
                    dserv.logout(session_key);
                    return true;
                }
            }
                
            int size;
            String[] entries;
            String   entry_string;
            start = 0;
            try {
                entry_string = dserv.list(key, session_key);
                StringTokenizer stk;
                stk = new StringTokenizer(entry_string, RETURN_STRING);
                int token_count = stk.countTokens();
                entries = new String[token_count];
                for (int i = 0; i < token_count; i++) {
                    entries[i] = stk.nextToken();
                }
                if (entries == null) {
                    entries = new String[0];
                }
            } catch (Exception exception_object) {
                entries = new String[0];
            }

            if (entries.length > 1) {
                try {
                    flag = dserv.executeOff(key, session_key);
                } catch (Exception exception_object) {
                    dserv.logout(session_key);
                    throw exception_object;
                }
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.ENTRY_ALREADY_EXIST,
                        getMessageResource().getMessage(
                           DirectoryServerException.ENTRY_ALREADY_EXIST)));

                dserv.logout(session_key);
                throw directory_server_exception;
            }
            if (entries.length == 1) {
                size = List.getSize(entries[0]);
                if (fd.length() == size) {     
                    try {
                        flag = dserv.executeOff(key, session_key);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                    try {
                        dserv.logout(session_key);
                    } catch (Exception exception_object) {
                    }
                    return true;
                }
                int i = size / (int)FILE_SIZE_LIMIT;
                if (size != i * (int)FILE_SIZE_LIMIT) {
                    dserv.remove(key, session_key);
                    start = 0;
                } else {
                    start = i;
                }
            }
            offset = start * (int)FILE_SIZE_LIMIT;
            fis.skip(offset);
            for (int i = start; i < count; i++) {
                try {
                    if (i < count - 1) {
                        length = (int)FILE_SIZE_LIMIT;
                    } else {
                        length = (int)fd.length() - i * (int)FILE_SIZE_LIMIT;
                    }
                    offset = 0;
                    buffer = new byte[length];
                    while (true) {
                        read_count = fis.read(buffer, offset, length);
                        if (read_count == -1) {
                            break;
                        }
                        offset = offset + read_count;
                        length = length - read_count;
                        if (length == 0) { break; }
                    }
                    if (read_count == -1) {
                        break;
                    }
                    dserv.append(key, buffer, session_key);
                } catch (Exception exception_object) {
                    try {
                        dserv.executeOff(key, session_key);
                        dserv.logout(session_key);
                    } catch (Exception exception_object2) {
                    }
                    throw exception_object;
                }
                try {
                    Thread.sleep(100);
                } catch (Exception exception_object) {
                }
            }
            try {
                dserv.executeOff(key, session_key);
                dserv.logout(session_key);
            } catch (Exception exception_object2) {
            }
            return true;
        }
        try {
            sealed_password = getSealedPassword(password);
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
        } catch (Exception exception_object) {
            throw exception_object;
        } 
/** */
        boolean  dflag;
        try {
            dflag = dserv.isDirectory(key, session_key);
        } catch (Exception exception_object) {
            try {
                dserv.logout(session_key);
            } catch (Exception exception_object_work) {
            }
            throw exception_object;
        }
        if (dflag == true) {
            key = expandKey(key_org, source);
        }
/*  **/
        buffer = new byte[(int)fd.length()];
        try {
            fis = new FileInputStream(fd);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_FILE_INPUT_STREAM,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_FILE_INPUT_STREAM)));
            throw directory_server_exception;
        }

        off = 0;
        rz = buffer.length;
        while (true) {
            try {
                read_count = fis.read(buffer, off, rz);
                if (read_count == -1) {
                    break;
                }
                off = off + read_count;
                rz = rz - read_count;
                if (rz <= 0) {
                    break;
                }
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
/*
            session_key = dserv.login(owner, sealed_password);
*/
            dserv.put(key, buffer, session_key);
        } catch (Exception exception_object) {
            try {
                dserv.logout(session_key);
            } catch (Exception exception_object_work) {
            }
            throw exception_object;
        } 
        try {
            dserv.logout(session_key);
        } catch (Exception exception_object) {
        } 
        return true;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(Put.PUT_USAGE_MESSAGE,
                getMessageResource().getMessage(Put.PUT_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        URI       uri_string;
        Put       object;
        String[]  param;

        if (ag.length != 4) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                object = new Put();
                try {
                    param = Put.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println(object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                            object.getMessageResource().getMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[2] == null || param[2].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.SOURCE_PARAMETER_NOT_FOUND,
                                object.getMessageResource().getMessage(
                                    DirectoryServerException.SOURCE_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[3] == null || param[3].length() == 0 ) {
                        System.err.println( object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                                object.getMessageResource().getMessage(
                                    DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }

                    uri_string = object.getKey(param[3]);
                    if (object.doPut(uri_string, param[0], param[1], param[2])
                        == false) {
                        System.err.println(
                            object.getMessageResource().getFormattedMessage(Put.PUT_ERROR,
                                object.getMessageResource().getMessage(Put.PUT_ERROR)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                } catch (Exception exception_object) {
                    System.out.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            Put.printUsage(ag);
            return;
        }

        object = new Put();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println(object.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                object.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[2] == null || ag[2].length() == 0) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.SOURCE_PARAMETER_NOT_FOUND,
                    object.getMessageResource().getMessage(
                        DirectoryServerException.SOURCE_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[3] == null || ag[3].length() == 0) {
            System.err.println( object.getMessageResource().getFormattedMessage(
                DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    object.getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            uri_string = object.getKey(ag[3]);

            if (object.doPut(uri_string, ag[0], ag[1], ag[2]) == false) {
                System.err.println(
                    object.getMessageResource().getFormattedMessage(Put.PUT_ERROR,
                        object.getMessageResource().getMessage(Put.PUT_ERROR)));
                System.exit(Register.ERROR_EXIT_CODE);
            }
        } catch (Exception exception_object) {
            System.out.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
        return;
    }
}
