/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Register.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/18
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class Register extends Command {
    static String specific_owner;
    static String specific_password;
    public final static String REGISTER_USAGE_MESSAGE = "odir.Register user_name password";
    public final static String LOGIN_MESSAGE = "login :";
    public final static String PASSWORD_MESSAGE = "password :";
    public final static String TTY_COMMAND = "/usr/bin/tty";
    public final static char separator_char = '/';
    public final static String SPACE = " ";
    public final static String EMPTY = "";
    public final static String ECHO_OFF = "-echo -echoe";
    public final static String ECHO_ON = "echo echoe";
    public final static String STTY_COMMAND = "/bin/stty";
    public final static String STTY_COMMAND_F_OPTION = "-f";
    public final static String TTY_KEY = "tty";
    public final static String DEVICE = "/dev/";
    public final static String PROCESS_LIST_COMMAND = "ps -l";
    public static int NUMBER_OF_ARGUMENT_IN_AUTHENTICATION = 2;

    public final static String COMMAND_NAME = "odir.Register";

    public final static String REGISTRATION_ERROR = "Registratin Error";
    public final static String REGISTER_ERROR = "Register Error";
    public final static int ERROR_EXIT_CODE = -1;

    public static String TERMINAL_NAME_NOT_FOUND
        = "Terminal name is not found.";
    public static String OWNER_PARAMETER_NOT_FOUND
        = "Owner parameter is not found.";
    public static String PASSWORD_PARAMETER_NOT_FOUND
        = "Passowrd parameter is not found.";
    public static String URI_PARAMETER_NOT_FOUND
        = "URI parameter is not found.";
    public static String ILLEGAL_COMMAND_NAME
        = "Illegal command name.";
    public static String INPUT_PARAMETER_NOT_FOUND
        = "Input parameter is not found.";


    public Register() {
        super();
    }

    public boolean doRegistration() throws Exception {
        return false;    
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String owner;
        String password;
        DataInputStream  din;
        InputStream      pin;
        Process  proc;
        String   command;
        String   tty;
        String   line;
        Runtime  runtime;
        boolean  flag;
        String[] param;
        DirectoryServerException     directory_server_exception;

        try {
            runtime = Runtime.getRuntime();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_RUNTIME_GET_RUNTIME,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_RUNTIME_GET_RUNTIME)));
            throw directory_server_exception;
        }
        System.out.print(
            getMessageResource().getFormattedMessage(
                Register.LOGIN_MESSAGE,
                getMessageResource().getMessage(
                    Register.LOGIN_MESSAGE)));

        din = new DataInputStream(System.in);
      
        try {
            owner = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        command = PROCESS_LIST_COMMAND;
        try {
            proc = runtime.exec(command);
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_RUNTIME_EXEC,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        try {
            pin  = proc.getInputStream();
            din  = new DataInputStream(pin);
 
            while (true) {
                line = din.readLine();
             
                if (line == null) {
                    directory_server_exception = new DirectoryServerException(
                        getMessageResource().getFormattedMessage(
                            DirectoryServerException.TERMINAL_NAME_NOT_FOUND,
                            getMessageResource().getMessage(
                                DirectoryServerException.TERMINAL_NAME_NOT_FOUND)));
                    throw directory_server_exception;
                }
                if (line.indexOf(command_name) >= 0) {
                    break;
                }
            }
            if (line == null || line.indexOf(command_name) < 0) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.TERMINAL_NAME_NOT_FOUND,
                        getMessageResource().getMessage(
                            DirectoryServerException.TERMINAL_NAME_NOT_FOUND)));
                throw directory_server_exception;
            }

            int idx = line.indexOf(TTY_KEY);
            if (idx == 0) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.TERMINAL_NAME_NOT_FOUND,
                        getMessageResource().getMessage(
                            DirectoryServerException.TERMINAL_NAME_NOT_FOUND)));
                throw directory_server_exception;
            }
            tty = line.substring(idx);

            idx = tty.indexOf(SPACE);
            if (idx == 0) {
                directory_server_exception = new DirectoryServerException(
                    getMessageResource().getFormattedMessage(
                        DirectoryServerException.TERMINAL_NAME_NOT_FOUND,
                        getMessageResource().getMessage(
                            DirectoryServerException.TERMINAL_NAME_NOT_FOUND)));
                throw directory_server_exception;
            }
            tty = DEVICE + tty.substring(0, idx);

            din.close();
            command = STTY_COMMAND + SPACE + STTY_COMMAND_F_OPTION 
                + SPACE + tty + SPACE + ECHO_OFF;

        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        try {
            proc = runtime.exec(command);
            pin  = proc.getInputStream();
            pin.close();

        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_RUNTIME_EXEC,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }

        System.out.print(
            getMessageResource().getFormattedMessage(
                Register.PASSWORD_MESSAGE,
                getMessageResource().getMessage(
                    Register.PASSWORD_MESSAGE)));

        din = new DataInputStream(System.in);
        try {
            password = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        try {
            command = STTY_COMMAND + SPACE + STTY_COMMAND_F_OPTION 
                + SPACE + tty + SPACE + ECHO_ON;

            proc = runtime.exec(command);
            pin  = proc.getInputStream();
            pin.close();

        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_RUNTIME_EXEC,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        System.out.println(EMPTY);
        param = new String[NUMBER_OF_ARGUMENT_IN_AUTHENTICATION];
        param[0] = owner;
        param[1] = password;
        return param;
    }

    public boolean doRegistration(String owner, String password) 
        throws Exception {
        boolean flag;
        DirectoryServer dserv;
        SealedObject    sealed_password;
        DirectoryServerException   directory_server_exception;

        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        dserv = new DirectoryServer();
        sealed_password = getSealedPassword(password);
        try {
            flag = dserv.register(owner, sealed_password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return flag;    
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(Register.REGISTER_USAGE_MESSAGE,
                getMessageResource().getMessage(Register.REGISTER_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        String[]  param;
        Register  register_object;

        if (ag.length != 2) {
            register_object = new Register();
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    param = Register.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println( register_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                            register_object.getMessageResource().getMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println( register_object.getMessageResource().getFormattedMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                            register_object.getMessageResource().getMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    register_object.doRegistration(param[0], param[1]);
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            register_object.printUsage(ag);
            return;
        } 
        register_object = new Register();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println( register_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                register_object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println( register_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                register_object.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            register_object.doRegistration(ag[0], ag[1]);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }

}
