/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Remove.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class Remove extends DirectoryBase {
    public static final String REMOVE_ERROR = "Remove error.";
    public final static String RETURN_STRING = "\n";
    public static final String COMMAND_NAME = "odir.Remove";
    public static final String FILE_PARAMETER_ERROR 
        = "You must specify a file path except root.";
    public final static int ARGUMENT_NUMBER = 3;
    public final static String DOT_SEPARATOR = ".";
    public final static String WILDCARD = "*";
    public final static String FILE_MESSAGE = "Remove File :";
    public final static String USAGE_MESSAGE = "odir.Remove owner password odir://host:port/file|file ...";
    public final static String REMOVE_USAGE_MESSAGE = USAGE_MESSAGE;
    public final static char separator_char = '/';

    public Remove() {
        super();
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String[]   auth;
        String[]   param;
        DataInputStream  din;
        DirectoryServerException   directory_server_exception;

        param = new String[ARGUMENT_NUMBER];
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        System.out.print( getMessageResource().getFormattedMessage(
            Remove.FILE_MESSAGE,
            getMessageResource().getMessage(Remove.FILE_MESSAGE)));

        din = new DataInputStream(System.in);

        try {
            param[0] = din.readLine();
        } catch (Exception exception_object) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE,
                    getMessageResource().getMessage(
                        DirectoryServerException.JAVA_DATA_INPUT_STREAM_READ_LINE)));
            throw directory_server_exception;
        }
        param[1] = auth[0];
        param[2] = auth[1];
        return param;
    }

    public boolean doRemove(URI key, String owner, String password)
        throws Exception {
        boolean flag;
        URI uri_key;
        DirectoryServer dserv;
        String session_key;
        SealedObject   sealed_password;
        DirectoryServerException    directory_server_exception;

        if (key == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
            flag = dserv.remove(key, session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return flag;
    }

    public String getList(String wild_path, String owner, String password) {
        int idx;
        URI uri_string;
        int file_count;
        String prefile;
        String parent_path;
        String file_name;
        String session_key;
        SealedObject sealed_password;

        idx = wild_path.lastIndexOf(File.separator);
        if (idx > 0) {
            parent_path = wild_path.substring(0, idx);
            file_name = wild_path.substring(idx + 1);
        } else {
            parent_path = File.separator;
            file_name = wild_path;
        }
        idx = file_name.lastIndexOf(DOT_SEPARATOR);
        if (idx > 0) {
            prefile = file_name.substring(0, idx);
        } else {
            prefile = file_name;
        }
        uri_string = getKey(parent_path);

        DirectoryServer dserv = new DirectoryServer();
        String[] entries;
        String   entry_string = new String();

        session_key = null;
        sealed_password = null;
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            entry_string = new String();
        }
        if (sealed_password != null) {
            try {
                session_key = dserv.login(owner, sealed_password);
                entry_string = dserv.list(uri_string, session_key) ;
                if (entry_string == null) {
                    entry_string = new String();
                }
            } catch (Exception exception_object) {
                entry_string = new String();
            }
            try {
                if (session_key != null) {
                    dserv.logout(session_key);
                }
            } catch (Exception exception_object) {
            }
        }

        StringTokenizer stk;
        stk = new StringTokenizer(entry_string, RETURN_STRING);
        int count;
        count = stk.countTokens();
        entries = new String[count];
        for (int i = 0; i < count; i++) {
            entries[i] = stk.nextToken();
        }

        file_count = 0;
        int[] idx_array = new int[entries.length];
        for (int i = 0; i < entries.length; i++) {
            if (entries[i].indexOf(prefile) == 0) {
                idx_array[file_count] = i;
                file_count = file_count + 1;
            }    
        }
        entry_string = new String();
        for (int i = 0; i < file_count; i++) {
            if (i == 0) {
                entry_string = entry_string.concat(RETURN_STRING);
            }
            entry_string = entry_string.concat(entries[idx_array[i]]);
        }
        return entry_string;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(Remove.REMOVE_USAGE_MESSAGE,
                getMessageResource().getMessage(Remove.REMOVE_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        int       count;
        int       idx;
        Remove    remove_object;
        String[]  param;
        String[]  file_list;
        String    file_list_string;
        URI       uri_string;

        if (ag.length == 0 && File.separatorChar == separator_char) {
            try {
                remove_object = new Remove();
                param = Remove.doInteraction(COMMAND_NAME);
                if (param[0] == null || param[0].length() == 0 ) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                        remove_object.getMessageResource().getMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                if (param[1] == null || param[1].length() == 0 ) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                        remove_object.getMessageResource().getMessage(
                            DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                if (param[2] == null || param[2].length() == 0) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                        remove_object.getMessageResource().getMessage(
                            DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                uri_string = remove_object.getKey(param[0]);
                if (uri_string == null) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                        remove_object.getMessageResource().getMessage(
                            DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                if (uri_string.getPath().compareTo(File.separator) == 0) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.DIRECTORY_PARAMETER_ERROR,
                        remove_object.getMessageResource().getMessage(
                            DirectoryServerException.DIRECTORY_PARAMETER_ERROR)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                if (remove_object.doRemove(uri_string, param[1], param[2])
                    == false) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        Remove.REMOVE_ERROR,
                        remove_object.getMessageResource().getMessage( Remove.REMOVE_ERROR)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
            } catch (Exception exception_object) {
                System.err.println(exception_object.getMessage());
                System.exit(Register.ERROR_EXIT_CODE);
            }
            return;
        }
        remove_object = new Remove();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println( remove_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                remove_object.getMessageResource().getMessage(
                    DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println( remove_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                remove_object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[2] == null || ag[2].length() == 0) {
            System.err.println( remove_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                remove_object.getMessageResource().getMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            String suffix = null;

            suffix = ag[0].substring(ag.length - 1);

            if (suffix != null && suffix.compareTo(WILDCARD) == 0) {
                file_list_string = remove_object.getList(ag[0], ag[1], ag[2]);
                StringTokenizer stk;
                stk = new StringTokenizer(file_list_string, RETURN_STRING);
                file_list = new String[stk.countTokens()];
                for (int j = 0; j < file_list.length; j++) {
                    file_list[j] = stk.nextToken();
                }
                count = file_list.length;
            } else {
                file_list = ag;
                count = 1;
            }
            for (int i = 0; i < count; i++) { 
                uri_string = remove_object.getKey(file_list[i]);
                if (uri_string == null) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.URI_PARAMETER_NOT_FOUND,
                            remove_object.getMessageResource().getMessage(
                                DirectoryServerException.URI_PARAMETER_NOT_FOUND)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                if (uri_string.getPath().compareTo(File.separator) == 0) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        DirectoryServerException.DIRECTORY_PARAMETER_ERROR,
                        remove_object.getMessageResource().getMessage(
                            DirectoryServerException.DIRECTORY_PARAMETER_ERROR)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                if (remove_object.doRemove(uri_string, ag[1], ag[2]) == false) {
                    System.err.println( remove_object.getMessageResource().getFormattedMessage(
                        Remove.REMOVE_ERROR,
                        remove_object.getMessageResource().getMessage( Remove.REMOVE_ERROR)));
                    System.exit(Register.ERROR_EXIT_CODE);
                }
            }
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }
}
