/**
 * Copyright (C) All rights reserved.
 *
 * @Module     StringObject.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/11/25
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.util.*;

public class StringObject extends Object {
    private ArrayList keys = new ArrayList();
    private ArrayList messages = new ArrayList();

    private final static String ENTRY_NOT_FOUND_STRINGOBJECT 
        = "Entry Not found in StringObject.";
    private final static String PARAMETER_ERROR_STRINGOBJECT 
        = "Parameter Error in StringObject.";
    private final static String ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE
        = "エントリが見つかりませんでした。";
    private final static String PARAMETER_ERROR_STRINGOBJECT_JAPANESE
        = "StringObjectの中でパラメータ異常が発生しました。";

    public void add (String key, String message)  {
        if (key == null || message == null) {
            return;
        }
        keys.add((Object)key);
        messages.add((Object)message);
    }

    private String getDefaultLanguage() {
        String    language;
        Locale    locale_object;

        locale_object = Locale.getDefault();
        language = locale_object.getISO3Language();
        return language;
    }

    public String getMessage(String key) throws Exception {
        String key_string;
        String message_string;
        DirectoryServerException  directory_server_exception;

        if (key == null) {
            if (getDefaultLanguage().compareTo(Locale.JAPANESE.getISO3Language()) == 0) {
                directory_server_exception = new DirectoryServerException(
                    PARAMETER_ERROR_STRINGOBJECT_JAPANESE);
            } else {
                directory_server_exception = new DirectoryServerException(
                    PARAMETER_ERROR_STRINGOBJECT);
            }
            throw directory_server_exception;
        }

        for (int i = 0; i < keys.size(); i++) {
            key_string = (String)keys.get(i);
            if (key_string.compareTo(key) == 0) {
                message_string = (String)messages.get(i);
                return message_string;
            }
        }
        if (getDefaultLanguage().compareTo(Locale.JAPANESE.getISO3Language()) == 0) {
            directory_server_exception = new DirectoryServerException(
                ENTRY_NOT_FOUND_STRINGOBJECT_JAPANESE);
        } else {
            directory_server_exception = new DirectoryServerException(
                ENTRY_NOT_FOUND_STRINGOBJECT);
        }
        throw directory_server_exception;
    }

    public int size() {
        return keys.size();
    }
}
