/**
 * Copyright (C) All rights reserved.    
 *
 * @Module     User.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/07/23
 *
 */

package odir;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;

import javax.crypto.*;
import javax.crypto.spec.*;

public class User extends DirectoryBase {
    public final static String USER_USAGE_MESSAGE = "odir.User owner password";
    public final static String RETURN_STRING = "\n";
    public final static String COMMAND_NAME = "odir.User";
    public final static String SCHEME_STRING = "odir";
    public final static String USER_ERROR = "User Error";
    public final static String USER_ENTRY_MESSAGE = "Entry";
    public final static int ARGUMENT_NUMBER = 2;
    public final static char separator_char = '/';

    public User() {
        super();
    }

    public static String[] doInteraction(String command_name) throws Exception {
        String[]   auth;
        String[]   param;
        DataInputStream  din;
        DirectoryServerException   directory_server_exception;

        if (command_name == null || command_name.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                User.getMessageResource().getFormattedMessage(
                    Register.ILLEGAL_COMMAND_NAME,
                    User.getMessageResource().getMessage(
                        Register.ILLEGAL_COMMAND_NAME)));
            throw directory_server_exception;
        }
        param = new String[ARGUMENT_NUMBER];
        try {
            auth = Register.doInteraction(command_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        param[0] = auth[0];
        param[1] = auth[1];
        return param;
    }

    public String doUser(String owner, String password) throws
        Exception {
        DirectoryServer dserv;
        SealedObject sealed_password;
        String   entry_list_string;
        String   session_key;
        DirectoryServerException  directory_server_exception;

        if (owner == null || owner.length() == 0) {
            directory_server_exception = new DirectoryServerException(
                User.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                User.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        if (password == null) {
            directory_server_exception = new DirectoryServerException(
                User.getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    User.getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            throw directory_server_exception;
        }
        try {
            sealed_password = getSealedPassword(password);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            dserv = new DirectoryServer();
            session_key = dserv.login(owner, sealed_password);
            entry_list_string = dserv.user(session_key);
            dserv.logout(session_key);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return entry_list_string;
    }

    public static void printUsage(String[] ag) {
        System.out.println(
            getMessageResource().getFormattedMessage(User.USER_USAGE_MESSAGE,
                getMessageResource().getMessage(User.USER_USAGE_MESSAGE)));
    }

    public static void main(String[] ag) {
        int   idx;
        User user_object;
        int      count;
        String   entry;
        String[] param;
        String[] entry_list;
        String   entry_list_string;
        DirectoryServerException   directory_server_exception;

        if (ag.length != 2) {
            if (ag.length == 0 && File.separatorChar == separator_char) {
                try {
                    user_object = new User();
                    param = User.doInteraction(COMMAND_NAME);
                    if (param[0] == null || param[0].length() == 0 ) {
                        System.err.println(
                            user_object.getMessageResource().getFormattedMessage(
                                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                                user_object.getMessageResource().getMessage(
                                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    if (param[1] == null || param[1].length() == 0 ) {
                        System.err.println(
                            user_object.getMessageResource().getFormattedMessage(
                                DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                                user_object.getMessageResource().getMessage(
                                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
                        System.exit(Register.ERROR_EXIT_CODE);
                    }
                    entry_list_string=user_object.doUser( param[0], param[1] );
                    StringTokenizer stk;
                    stk = new StringTokenizer(entry_list_string, RETURN_STRING);
                    count = stk.countTokens();
                    System.out.println( User.getMessageResource().getFormattedMessage(
                        User.USER_ENTRY_MESSAGE,
                        user_object.getMessageResource().getMessage(
                            User.USER_ENTRY_MESSAGE)));
                    for (int i = 0; i < count; i++) {
                        entry = stk.nextToken();
                        System.out.println(entry);
                    }
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                    System.exit(Register.ERROR_EXIT_CODE);
                }
                return;
            }
            User.printUsage(ag);
            return;
        }
        user_object = new User();
        if (ag[0] == null || ag[0].length() == 0 ) {
            System.err.println(user_object.getMessageResource().getFormattedMessage(
                DirectoryServerException.OWNER_PARAMETER_NOT_FOUND,
                user_object.getMessageResource().getMessage(
                    DirectoryServerException.OWNER_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        if (ag[1] == null || ag[1].length() == 0 ) {
            System.err.println(
                user_object.getMessageResource().getFormattedMessage(
                    DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND,
                    user_object.getMessageResource().getMessage(
                        DirectoryServerException.PASSWORD_PARAMETER_NOT_FOUND)));
            System.exit(Register.ERROR_EXIT_CODE);
        }
        try {
            entry_list_string = user_object.doUser(ag[0], ag[1]);
            StringTokenizer stk;
            stk = new StringTokenizer(entry_list_string, RETURN_STRING);
            count = stk.countTokens();
            System.out.println( user_object.getMessageResource().getFormattedMessage(
                User.USER_ENTRY_MESSAGE,
                user_object.getMessageResource().getMessage(
                    User.USER_ENTRY_MESSAGE)));
            for (int i = 0; i < count; i++) {
                entry = stk.nextToken();
                System.out.println(entry);
            }
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            System.exit(Register.ERROR_EXIT_CODE);
        }
    }
}
