/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     BaseVariable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/07/05
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

public class BaseVariable {
    public static String PARAMETER_ERROR_EXCEPTION
        = "Parameter error exception.";

    public static int BUFFER_SIZE = 8192;
    public static String CLASS_SUFFIX = ".class";
    public static String DOT_STRING = ".";
    public final static String TEST_CLASS_NAME = "Test";

    public final static String START_BRACKET = "[";
    public final static String END_BRACKET = "]";
    public final static String JAVA_IO_SERIALIZABLE_TYPE_KEY = "java.io.Serializable";
    public final static String SEMICOLON = ";";
    public final static char ARRAY_INT = 'I';
    public final static char ARRAY_BYTE = 'B';
    public final static char ARRAY_LONG = 'J';
    public final static char ARRAY_SHORT = 'S';
    public final static char ARRAY_CHAR = 'C';
    public final static char ARRAY_DOUBLE = 'D';
    public final static char ARRAY_FLOAT = 'F';
    public final static char ARRAY_BOOLEAN = 'Z';
    public final static char ARRAY_REFERENCE = 'L';

    public boolean isSerializable(Field variable) {
        int       idx;
        String    key;
        String    type_name;
        Class     object_class;
        Class[]   object_interfaces;
        Field[]   variable_list;

        if (variable == null) {
            return false;
        }

        if (variable.getType().getName().compareTo(int.class.getName()) == 0
        || variable.getType().getName().compareTo(long.class.getName()) == 0
        || variable.getType().getName().compareTo(float.class.getName()) == 0
        || variable.getType().getName().compareTo(double.class.getName()) == 0
        || variable.getType().getName().compareTo(byte.class.getName()) == 0
        || variable.getType().getName().compareTo(boolean.class.getName()) == 0
        || variable.getType().getName().compareTo(short.class.getName()) == 0
        || variable.getType().getName().compareTo(char.class.getName()) == 0
        || variable.getType().getName().compareTo(String.class.getName()) == 0
        || variable.getType().getName().compareTo(Integer.class.getName()) == 0
        || variable.getType().getName().compareTo(Long.class.getName()) == 0
        || variable.getType().getName().compareTo(Float.class.getName()) == 0
        || variable.getType().getName().compareTo(Double.class.getName()) == 0
        || variable.getType().getName().compareTo(Boolean.class.getName()) == 0
        || variable.getType().getName().compareTo(Byte.class.getName()) == 0
        || variable.getType().getName().compareTo(Character.class.getName()) == 0
        || variable.getType().getName().compareTo(Short.class.getName()) == 0
        ) {
            return true;
        }
        type_name = variable.getType().getName();
        idx = type_name.lastIndexOf(START_BRACKET);
        if (idx >= 0) {
            key = type_name.substring(idx+1);
            if (key == null || key.length() == 0) {
                return false;
            }
            if (key.charAt(0) == ARRAY_INT
            || key.charAt(0) == ARRAY_LONG
            || key.charAt(0) == ARRAY_FLOAT
            || key.charAt(0) == ARRAY_DOUBLE
            || key.charAt(0) == ARRAY_BYTE
            || key.charAt(0) == ARRAY_SHORT
            || key.charAt(0) == ARRAY_CHAR
            || key.charAt(0) == ARRAY_BOOLEAN) {
                return true;
            }
            if (key.charAt(0) != ARRAY_REFERENCE) {
                return false;
            }
            idx = key.lastIndexOf(SEMICOLON);
            if (idx < 0) {
                return false;
            }
            key = key.substring(1, idx);
            try {
                object_class = Class.forName(key);
            } catch (Exception exception_object) {
                return false;
            }
            key = object_class.getName();
            if (key.compareTo(String.class.getName()) == 0
                || key.compareTo(Integer.class.getName()) == 0
                || key.compareTo(Long.class.getName()) == 0
                || key.compareTo(Float.class.getName()) == 0
                || key.compareTo(Double.class.getName()) == 0
                || key.compareTo(Boolean.class.getName()) == 0
                || key.compareTo(Byte.class.getName()) == 0
                || key.compareTo(Character.class.getName()) == 0) {
                return true;
            }
        } else {
/**
            try {
                object_class = Class.forName(type_name);
            } catch (Exception exception_object) {
                return false;
            }
**/
            object_class = variable.getType();
        }
        variable_list = object_class.getFields();
        for (int i = 0; i < variable_list.length; i++) {
            if (isSerializable(variable_list[i]) == false) {
                return false;
            }
        }
        object_interfaces = object_class.getInterfaces();
        for (int i = 0; i < object_interfaces.length; i++) {
            if (object_interfaces[i].getName().compareTo(JAVA_IO_SERIALIZABLE_TYPE_KEY) == 0) {
                return true;
            }
        }
        return false;
    }
}
