/**
 * Copyright (C) All rights reserved.
 *
 * @Module     CallMethod.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/06/13
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

public class CallMethod extends CallMethodTransfer  {

    private static String DEBUG_SET_METHOD_TRACE = "setMethod()";
    private static String DEBUG_GET_PARAMETER_TRACE = "getParameter()";
    private static String DEBUG_GET_PARAMETER_TYPE_TRACE = "getParameterType()";
    private static String DEBUG_GET_PARAMETER_COUNT_TRACE = 
        "getParameterCount()";
/**
    private static String DEBUG_SET_INT_PARAMETER_TRACE = "setintParameter()";
    private static String DEBUG_SET_LONG_PARAMETER_TRACE = "setlongParameter()";
    private static String DEBUG_SET_FLOAT_PARAMETER_TRACE = "setfloatParameter()";
    private static String DEBUG_SET_DOUBLE_PARAMETER_TRACE = "setdoubleParameter()";
    private static String DEBUG_SET_SHORT_PARAMETER_TRACE = "setshortParameter()";
    private static String DEBUG_SET_BYTE_PARAMETER_TRACE = "setbyteParameter()";
    private static String DEBUG_SET_CHAR_PARAMETER_TRACE = "setcharParameter()";
    private static String DEBUG_SET_BOOLEAN_PARAMETER_TRACE = "setbooleanParameter()";
    private static String DEBUG_SET_OBJECT_PARAMETER_TRACE = "setObjectParameter()";
**/
    private static String DEBUG_INVOKE_TRACE = "invoke()";
/**
    private static String DEBUG_GET_INT_VALUE_TRACE = "getIntValue()";
    private static String DEBUG_GET_LONG_VALUE_TRACE = "getLongValue()";
    private static String DEBUG_GET_FLOAT_VALUE_TRACE = "getFloatValue()";
    private static String DEBUG_GET_DOUBLE_VALUE_TRACE = "getDoubleValue()";
    private static String DEBUG_GET_BYTE_VALUE_TRACE = "getByteValue()";
    private static String DEBUG_GET_CHAR_VALUE_TRACE = "getCharValue()";
    private static String DEBUG_GET_SHORT_VALUE_TRACE = "getShortValue()";
    private static String DEBUG_GET_BOOLEAN_VALUE_TRACE = "getBooleanValue()";
*/
    private static String DEBUG_GET_OBJECT_VALUE_TRACE = "getValue()";

    private int debug = 0;

    public void setMethod(Method method) throws Exception {
        if (method == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            super.setMethod(method);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public Object getParameter(Integer integer_index) throws Exception {
        int  int_index;
        Object   object_rt;

        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        int_index = integer_index.intValue();
        try {
            object_rt = super.getParameter(int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return object_rt;
    }

    public String  getParameterType(Integer integer_index) throws Exception {
        int  int_index;
        String  string_name;

        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        int_index = integer_index.intValue();
        try {
            string_name = super.getParameterType(int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return string_name;
    }

    public Integer getCount() throws Exception {
        int   rt;
        Integer   integer_rt;

        try {
            rt = super.getParameterCount();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        integer_rt = Integer.valueOf(rt);
        return integer_rt;
    }

    public void setIntValue(Object obj, Integer arg, Integer integer_index) throws Exception {
        int  int_arg;
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        int_arg = arg.intValue();
        int_index = integer_index.intValue();

        try {
            super.setInt(obj, int_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setLongValue(Object obj, Long arg, Integer integer_index) throws Exception {
        long  long_arg;
        int   int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        long_arg = arg.longValue();
        int_index = integer_index.intValue();

        try {
            super.setLong(obj, long_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setFloatValue(Object obj, Float arg, Integer integer_index) throws Exception {
        float  float_arg;
        int    int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        float_arg = arg.floatValue();
        int_index = integer_index.intValue();

        try {
            super.setFloat(obj, float_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setDoubleValue(Object obj, Double arg, Integer integer_index) throws Exception {
        double  double_arg;
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        double_arg = arg.doubleValue();
        int_index = integer_index.intValue();

        try {
            super.setDouble(obj, double_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setShortValue(Object obj, Short arg, Integer integer_index) throws Exception {
        short  short_arg;
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        short_arg = arg.shortValue();
        int_index = integer_index.intValue();

        try {
            super.setShort(obj, short_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setByteValue(Object obj, Byte arg, Integer integer_index) throws Exception {
        byte  byte_arg;
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        byte_arg = arg.byteValue();
        int_index = integer_index.intValue();

        try {
            super.setByte(obj, byte_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setBooleanValue(Object obj, Boolean arg, Integer integer_index) throws Exception {
        boolean  boolean_arg;
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        boolean_arg = arg.booleanValue();
        int_index = integer_index.intValue();

        try {
            super.setBoolean(obj, boolean_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setCharValue(Object obj, Integer arg, Integer integer_index) throws Exception {
        char  char_arg;
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (arg == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        char_arg = (char)arg.intValue();
        int_index = integer_index.intValue();

        try {
            super.setChar(obj, char_arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setValue(Object obj, Object arg, Integer integer_index) throws Exception {
        int  int_index;

        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        if (integer_index == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }

        int_index = integer_index.intValue();

        if (arg != null && (arg.getClass().getName().compareTo(byte[].class.getName()))==0) {
            if (((byte[])arg).length != 0) {
                ObjectInputManager object_input_manager;
                object_input_manager = new ObjectInputManager();

                arg = object_input_manager.toObject((byte[])arg);
            }
        }

        try {
            super.set(obj, arg, int_index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void invoke(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException    null_pointer_exception;
            null_pointer_exception  = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            super.invoke(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public Integer getIntValue(Object obj) throws Exception {
        int  int_rt;
        Integer  integer_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            int_rt = super.getInt(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        integer_rt = Integer.valueOf(int_rt);
        return integer_rt;
    }

    public Long getLongValue(Object obj) throws Exception {
        long  long_rt;
        Long  long_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            long_rt = super.getLong(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        long_object_rt = Long.valueOf(long_rt);
        return long_object_rt;
    }

    public Float getFloatValue(Object obj) throws Exception {
        float  float_rt;
        Float  float_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            float_rt = super.getFloat(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        float_object_rt = Float.valueOf(float_rt);
        return float_object_rt;
    }

    public Double getDoubleValue(Object obj) throws Exception {
        double  double_rt;
        Double  double_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            double_rt = super.getDouble(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        double_object_rt = Double.valueOf(double_rt);
        return double_object_rt;
    }

    public Short getShortValue(Object obj) throws Exception {
        short  short_rt;
        Short  short_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            short_rt = super.getShort(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        short_object_rt = Short.valueOf(short_rt);
        return short_object_rt;
    }

    public Boolean getBooleanValue(Object obj) throws Exception {
        boolean  boolean_rt;
        Boolean  boolean_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            boolean_rt = super.getBoolean(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        boolean_object_rt = Boolean.valueOf(boolean_rt);
        return boolean_object_rt;
    }

    public Byte getByteValue(Object obj) throws Exception {
        byte  byte_rt;
        Byte  byte_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            byte_rt = super.getByte(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        byte_object_rt = Byte.valueOf(byte_rt);
        return byte_object_rt;
    }

    public Integer getCharValue(Object obj) throws Exception {
        char  char_rt;
        Integer  char_object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            char_rt = super.getChar(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        char_object_rt = Integer.valueOf(char_rt);
        return char_object_rt;
    }

    public Object getValue(Object obj) throws Exception {
        Object   object_rt;

        if (obj == null) {
            NullPointerException   null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        try {
            object_rt = super.get(obj);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        byte[]                     target;
        if (object_rt == null) {
            target = new byte[0];
            return (Object)target;
        }
        ObjectOutputManager        object_output_manager;
        object_output_manager = new ObjectOutputManager();
        target = object_output_manager.toByteArray(object_rt);
        return (Object)target;
    }
}
