/**
 * Copyright (C) All rights reserved.
 *
 * @Module     CallMethodFactory.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/01/10
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;

public class CallMethodFactory extends SerializableObject {

    public static CallMethod getCallMethod(Method method) throws Exception  {
        CallMethod call_method;

        if (method == null) {
            ObjectServerException illegal_argument_exception;
            illegal_argument_exception = new ObjectServerException(
                ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION);
            throw illegal_argument_exception;
        }
        try {
            call_method = new CallMethod();
            call_method.setMethod(method);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return call_method;
    }

    public static CallMethod getCallMethod(Method method,
        MessageResource    parameter_message_resource) throws Exception  {
        CallMethod call_method;
        ObjectServerException illegal_argument_exception;
        ObjectServerException internal_exception;

        if (method == null) {
            if (parameter_message_resource == null) {       
                illegal_argument_exception = new ObjectServerException(
                    ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION);
                throw illegal_argument_exception;
            }
            illegal_argument_exception = new ObjectServerException(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION,
                    parameter_message_resource.getMessage( 
                        ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        call_method = new CallMethod();
        try {
            call_method.setMethod(method);
        } catch (Exception exception_object) {
            if (parameter_message_resource == null) {       
                illegal_argument_exception = new ObjectServerException(
                    ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION);
                throw illegal_argument_exception;
            }
            illegal_argument_exception = new ObjectServerException(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION,
                    parameter_message_resource.getMessage( 
                        ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        if (call_method == null) {
            if (parameter_message_resource == null) {       
                internal_exception = new ObjectServerException(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION);
                throw internal_exception;
            }
            internal_exception = new ObjectServerException(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION,
                    parameter_message_resource.getMessage( 
                        ObjectServerException.ILLEGAL_ARGUMENT_EXCEPTION)));
            throw internal_exception;
        }
        return call_method;
    }
}
