/**
 * Copyright (C) All rights reserved.
 *
 * @Module     CallMethodInvoke.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/08/09
 *             2.1 2011/12/22 import from Exec sample.
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

public class CallMethodInvoke {
    protected Method     method_object = null;
    protected String TRACE_CLASSNAME  = "oserv.CallMethod";
    protected String TRACE_CALLMETHOD = "<init>";
    protected String TRACE_SETPARAMETER = "setParameter";
    protected String TRACE_INVOKE = "invoke";

    protected Class      return_type = null;
    protected Class[]    parameter_classes = null;
    protected boolean[]  parameter_validation = null;
    protected Object[]   parameter_box = null;
    protected boolean    call_flag = false;

    protected int      int_parameter_value;
    protected long     long_parameter_value;
    protected char     char_parameter_value;
    protected short    short_parameter_value;
    protected byte     byte_parameter_value;
    protected boolean  boolean_parameter_value;
    protected float    float_parameter_value;
    protected double   double_parameter_value;
    protected Object   object_parameter_value;

    protected int      int_return_value;
    protected char     char_return_value;
    protected short    short_return_value;
    protected long     long_return_value;
    protected float    float_return_value;
    protected double   double_return_value;
    protected boolean  boolean_return_value;
    protected byte     byte_return_value;
    protected Object   object_return_value;

    public static String NULL_STRING = "Object is Null.";

    public final static String VOID_TYPE = "void";

    public static String SPACE = " ";

    public static String USAGE = "Usage: CallMethod ClassName MethodName";
    public static String ILLEGAL_METHOD_NAME = "Illegal method name";
    public static String INSTANCE_CREATION_ERROR = "Instance creation error.";
    public static String CONSTRUCTION_ERROR = "Construction error.";

    private static String DEBUG_INVOKE_TRACE = "invoke()";

    public static String LIMIT_KEY = "Limit";
    public static String VALUE_KEY = "value";

    protected int debug = 0;

    protected void setMethod(Method method) throws Exception {
        if (method == null) {
            NullPointerException null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        method_object = method;
        parameter_classes = method_object.getParameterTypes();
        if (parameter_classes == null) {
            parameter_box = new Object[0];
            parameter_validation = new boolean[0];
        } else {
            parameter_box = new Object[parameter_classes.length];
            parameter_validation = new boolean[parameter_classes.length];
            for (int i = 0; i < parameter_classes.length; i++) {
                parameter_validation[i] = false;
            }
        }
        call_flag = false;
        return_type = method.getReturnType();
    }

    public void setDebug(int level) {
        debug = level;
    }

    public int getDebug() {
        return debug;
    }

    public Method getMethod() {
        return method_object;
    }

    public void invoke(Object object_instance) throws Exception {
        Object rt;
        InvocationTargetException invocation_real_exception;
        Exception real_exception;

        if (object_instance == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            if (getDebug() > 0) {
                System.err.println(DEBUG_INVOKE_TRACE);
                null_pointer_exception.printStackTrace(System.err);
            }
            throw null_pointer_exception;
        }
        for (int i = 0; i < parameter_validation.length; i++) {

            if (parameter_validation[i] == false) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
        }
        if (return_type.getName().compareTo(VOID_TYPE) == 0) {
            try {
                method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            } catch (Exception exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            }
        } else
        if (return_type.getName().compareTo(int.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            } catch (Exception exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            int_return_value = (int)Integer.valueOf(rt.toString()).intValue();
        } else 
        if (return_type.getName().compareTo(long.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            } catch (Exception exception_object) {
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    exception_object.printStackTrace(System.err);
                }
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            long_return_value = (long)Long.valueOf(rt.toString()).longValue();
        } else 
        if (return_type.getName().compareTo(float.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            float_return_value = (float)Float.valueOf(rt.toString()).floatValue();
        } else 
        if (return_type.getName().compareTo(double.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            double_return_value = (double)Double.valueOf(rt.toString()).doubleValue();
        } else 
        if (return_type.getName().compareTo(byte.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            byte_return_value = (byte)Byte.valueOf(rt.toString()).byteValue();
        } else 
        if (return_type.getName().compareTo(boolean.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            boolean_return_value = (boolean)Boolean.valueOf(rt.toString()).booleanValue();
        } else 
        if (return_type.getName().compareTo(char.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            if ((rt instanceof Character) == true) {
                char_return_value = ((Character)rt).charValue();
            } else {
                char_return_value = (char)Integer.valueOf(rt.toString()).intValue();
            }
        } else 
        if (return_type.getName().compareTo(short.class.getName()) == 0) {
            try {
                rt = method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
            short_return_value = (short)Short.valueOf(rt.toString()).shortValue();
        } else {
            try {
                object_return_value = (Object)method_object.invoke(object_instance, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                throw exception_object;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (object_return_value == null) {
                NullPointerException  null_pointer_exception;
                null_pointer_exception = new NullPointerException();
                if (getDebug() > 0) {
                    System.err.println(DEBUG_INVOKE_TRACE);
                    null_pointer_exception.printStackTrace(System.err);
                }
                throw null_pointer_exception;
            }
        }
        call_flag = true;
        for (int i = 0; i < parameter_validation.length; i++) {
            parameter_validation[i] = false;
        }
    }

    public Boolean isInputStreamParameter(Integer integer_index) throws Exception {
        int     count;
        int     index;
        Object  parameter_object;
        String  parameter_type;
        Boolean boolean_false = Boolean.valueOf(false);
        Boolean boolean_true = Boolean.valueOf(true);

        if (integer_index == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        index = integer_index.intValue();

        if (index >= parameter_classes.length) {
            return boolean_false;
        }
        try {
            parameter_type = parameter_classes[index].getName();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (parameter_type == null) { return boolean_false; }
        if (parameter_type.compareTo( int.class.getName() ) == 0
        || parameter_type.compareTo( long.class.getName() ) == 0
        || parameter_type.compareTo( float.class.getName() ) == 0
        || parameter_type.compareTo( double.class.getName() ) == 0
        || parameter_type.compareTo( short.class.getName() ) == 0
        || parameter_type.compareTo( char.class.getName() ) == 0
        || parameter_type.compareTo( byte.class.getName() ) == 0
        || parameter_type.compareTo( boolean.class.getName() ) == 0
        ) {
            return boolean_false;
        }
        if ((parameter_box[index] instanceof NamedInputStream) == true) {
            return boolean_true;
        }
        return boolean_false;
    }

    public static void main(String[] args) {
    }
}
