/**
 * Copyright (C) All rights reserved.
 *
 * @Module     CallMethodTransfer.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/06/12
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

public class CallMethodTransfer extends CallMethodInvoke {

    public static String SPACE = " ";

    public static String ILLEGAL_METHOD_NAME = "Illegal method name";
    public static String INSTANCE_CREATION_ERROR = "Instance creation error.";
    public static String CONSTRUCTION_ERROR = "Construction error.";

    public static String LIMIT_KEY = "Limit";
    public static String VALUE_KEY = "value";


    protected void setMethod(Method method) throws Exception {
        super.setMethod(method);
    }

    public int getParameterCount() throws Exception {
        if (parameter_box != null) {
            return parameter_box.length;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public String getParameterType(int index) throws Exception {
        if ( parameter_classes == null ) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        if (index < 0 || parameter_classes.length <= index) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        return parameter_classes[index].getName();
    }

    public Object getParameter(int index) throws Exception {
        if ( parameter_box == null ) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        if (index < 0 || parameter_box.length <= index) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        return parameter_box[index];
    }

    public void setParameter(Object obj, int index) throws Exception {
        call_flag = false;

        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();

            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        if (parameter_classes.length <= index) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (parameter_classes[index].getName().compareTo(int.class.getName()) == 0) {
            parameter_box[index] = (Object)int_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(long.class.getName()) == 0) {
            parameter_box[index] = (Object)long_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(float.class.getName()) == 0) {
            parameter_box[index] = (Object)float_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(double.class.getName()) == 0) {
            parameter_box[index] = (Object)double_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(short.class.getName()) == 0) {
            parameter_box[index] = (Object)short_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(boolean.class.getName()) == 0) {
            parameter_box[index] = (Object)boolean_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(byte.class.getName()) == 0) {
            parameter_box[index] = (Object)byte_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(char.class.getName()) == 0) {
            parameter_box[index] = (Object)char_parameter_value;
            parameter_validation[index] = true;
            return;
        }

        if (object_parameter_value != null && (object_parameter_value.getClass().getName().compareTo(byte[].class.getName()))==0) {
            if (((byte[])object_parameter_value).length == 0) {
                parameter_box[index] = null;
            } else {
                parameter_box[index] = object_parameter_value;
            }
        } else {
            parameter_box[index] = object_parameter_value;
        }
        parameter_validation[index] = true;
    }

    public void setInt(Object obj, int arg, int index) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        int_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setLong(Object obj, long arg, int index) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        long_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setFloat(Object obj, float arg, int index) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        float_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setDouble(Object obj, double arg, int index) throws 
        Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        double_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setShort(Object obj, short arg, int index) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        short_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setChar(Object obj, char arg, int index) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        char_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setByte(Object obj, byte arg, int index) 
                        throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        byte_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setBoolean(Object obj, boolean arg, int index) 
                        throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        boolean_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void set(Object obj, Object arg, int index) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (method_object == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        object_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public int getInt(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(int.class.getName()) == 0) {
            return int_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public long getLong(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(long.class.getName()) == 0) {
            return long_return_value;
        }

        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public float getFloat(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(float.class.getName()) == 0) {
            return float_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public double getDouble(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(double.class.getName()) == 0) {
            return double_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public short getShort(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(short.class.getName()) == 0) {
            return short_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public char getChar(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(char.class.getName()) == 0) {
            return char_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public boolean getBoolean(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(boolean.class.getName()) == 0) {
            return boolean_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public byte getByte(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(byte.class.getName()) == 0) {
            return byte_return_value;
        }
        NullPointerException  null_pointer_exception;
        null_pointer_exception = new NullPointerException();
        throw null_pointer_exception;
    }

    public Object get(Object obj) throws Exception {
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (call_flag == false) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        } 
        if (return_type.getName().compareTo(VOID_TYPE) == 0
         || return_type.getName().compareTo(byte.class.getName()) == 0
         || return_type.getName().compareTo(boolean.class.getName()) == 0
         || return_type.getName().compareTo(int.class.getName()) == 0
         || return_type.getName().compareTo(long.class.getName()) == 0
         || return_type.getName().compareTo(double.class.getName()) == 0
         || return_type.getName().compareTo(float.class.getName()) == 0
         || return_type.getName().compareTo(short.class.getName()) == 0
         || return_type.getName().compareTo(char.class.getName()) == 0
        ) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        return object_return_value;
    }

    /**
     * native Methods of reflect.Method class.
     */

    public static void main(String[] args) {
    }
}
