/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     CallMethodWrapper.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/06/13
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class CallMethodWrapper extends ObjectServerHeader {
    /** CallMethod Instance **/
    private Object object_instance = null;
    private Class  call_method_class = null;

    private static final String WRAPPER_OBJECT_FIELD_CLASS 
        = "call_method_object";

    private Method   invoke_method = null;
    private String   INVOKE_METHOD_NAME = "invoke";
    private Object[] invoke_args_object = new Object[1];
    private Class[]  invoke_args_class = { Object.class };
    private CallMethod  invoke_call_method = null;

    private Method   set_method_method = null;
    private String   SET_METHOD_METHOD_NAME = "setMethod";
    private Object[] set_method_args_object = new Object[1];
    private Class[]  set_method_args_class = { Method.class };
    private CallMethod  set_method_call_method = null;

    private Method   is_input_stream_parameter_method = null;
    private String   IS_INPUT_STREAM_PARAMETER_METHOD_NAME = "isInputStreamParameter";
    private Object[] is_input_stream_parameter_args_object = new Object[1];
    private Class[]  is_input_stream_parameter_args_class = { Integer.class };
    private CallMethod  is_input_stream_parameter_call_method = null;

    private Method   get_count_method = null;
    private String   GET_COUNT_METHOD_NAME = "getCount";
    private Object[] get_count_args_object = new Object[0];
    private Class[]  get_count_args_class = new Class[0];
    private CallMethod  get_count_call_method = null;

    private Method   get_parameter_type_method = null;
    private String   GET_PARAMETER_TYPE_METHOD_NAME = "getParameterType";
    private Object[] get_parameter_type_args_object = new Object[1];
    private Class[]  get_parameter_type_args_class = 
        {  Integer.class };
    private CallMethod  get_parameter_type_call_method = null;

    private Method   get_parameter_method = null;
    private String   GET_PARAMETER_METHOD_NAME = "getParameter";
    private Object[] get_parameter_args_object = new Object[1];
    private Class[]  get_parameter_args_class = 
        {  Integer.class };
    private CallMethod  get_parameter_call_method = null;

    private Method   set_int_parameter_method = null;
    private String   SET_INT_PARAMETER_METHOD_NAME = "setIntValue";
    private Object[] set_int_parameter_args_object = new Object[3];
    private Class[]  set_int_parameter_args_class = { 
        Object.class, Integer.class, Integer.class };
    private CallMethod  set_int_parameter_call_method = null;

    private Method   set_long_parameter_method = null;
    private String   SET_LONG_PARAMETER_METHOD_NAME = "setLongValue";
    private Object[] set_long_parameter_args_object = new Object[3];
    private Class[]  set_long_parameter_args_class = { 
        Object.class, Long.class, Integer.class };
    private CallMethod  set_long_parameter_call_method = null;

    private Method   set_float_parameter_method = null;
    private String   SET_FLOAT_PARAMETER_METHOD_NAME = "setFloatValue";
    private Object[] set_float_parameter_args_object = new Object[3];
    private Class[]  set_float_parameter_args_class = { 
        Object.class, Float.class, Integer.class };
    private CallMethod  set_float_parameter_call_method = null;

    private Method   set_double_parameter_method = null;
    private String   SET_DOUBLE_PARAMETER_METHOD_NAME = "setDoubleValue";
    private Object[] set_double_parameter_args_object = new Object[3];
    private Class[]  set_double_parameter_args_class = { 
        Object.class, Double.class, Integer.class };
    private CallMethod  set_double_parameter_call_method = null;

    private Method   set_byte_parameter_method = null;
    private String   SET_BYTE_PARAMETER_METHOD_NAME = "setByteValue";
    private Object[] set_byte_parameter_args_object = new Object[3];
    private Class[]  set_byte_parameter_args_class = { 
        Object.class, Byte.class, Integer.class };
    private CallMethod  set_byte_parameter_call_method = null;

    private Method   set_boolean_parameter_method = null;
    private String   SET_BOOLEAN_PARAMETER_METHOD_NAME = "setBooleanValue";
    private Object[] set_boolean_parameter_args_object = new Object[3];
    private Class[]  set_boolean_parameter_args_class = { 
        Object.class, Boolean.class, Integer.class };
    private CallMethod  set_boolean_parameter_call_method = null;

    private Method   set_short_parameter_method = null;
    private String   SET_SHORT_PARAMETER_METHOD_NAME = "setShortValue";
    private Object[] set_short_parameter_args_object = new Object[3];
    private Class[]  set_short_parameter_args_class = { 
        Object.class, Short.class, Integer.class };
    private CallMethod  set_short_parameter_call_method = null;

    private Method   set_char_parameter_method = null;
    private String   SET_CHAR_PARAMETER_METHOD_NAME = "setCharValue";
    private Object[] set_char_parameter_args_object = new Object[3];
    private Class[]  set_char_parameter_args_class = { 
        Object.class, Integer.class, Integer.class };
    private CallMethod  set_char_parameter_call_method = null;

    private Method   set_object_method = null;
    private String   SET_OBJECT_METHOD_NAME = "setValue";
    private Object[] set_object_args_object = new Object[3];
    private Class[]  set_object_args_class = { 
        Object.class, Object.class, Integer.class };
    private CallMethod  set_object_call_method = null;

    private Method   get_int_value_method = null;
    private String   GET_INT_VALUE_METHOD_NAME = "getIntValue";
    private Object[] get_int_value_args_object = new Object[1];
    private Class[]  get_int_value_args_class = { Object.class };
    private CallMethod  get_int_value_call_method = null;

    private Method   get_long_value_method = null;
    private String   GET_LONG_VALUE_METHOD_NAME = "getLongValue";
    private Object[] get_long_value_args_object = new Object[1];
    private Class[]  get_long_value_args_class = { Object.class };
    private CallMethod  get_long_value_call_method = null;

    private Method   get_float_value_method = null;
    private String   GET_FLOAT_VALUE_METHOD_NAME = "getFloatValue";
    private Object[] get_float_value_args_object = new Object[1];
    private Class[]  get_float_value_args_class = { Object.class };
    private CallMethod  get_float_value_call_method = null;

    private Method   get_double_value_method = null;
    private String   GET_DOUBLE_VALUE_METHOD_NAME = "getDoubleValue";
    private Object[] get_double_value_args_object = new Object[1];
    private Class[]  get_double_value_args_class = { Object.class };
    private CallMethod  get_double_value_call_method = null;

    private Method   get_short_value_method = null;
    private String   GET_SHORT_VALUE_METHOD_NAME = "getShortValue";
    private Object[] get_short_value_args_object = new Object[1];
    private Class[]  get_short_value_args_class = { Object.class };
    private CallMethod  get_short_value_call_method = null;

    private Method   get_byte_value_method = null;
    private String   GET_BYTE_VALUE_METHOD_NAME = "getByteValue";
    private Object[] get_byte_value_args_object = new Object[1];
    private Class[]  get_byte_value_args_class = { Object.class };
    private CallMethod  get_byte_value_call_method = null;

    private Method   get_boolean_value_method = null;
    private String   GET_BOOLEAN_VALUE_METHOD_NAME = "getBooleanValue";
    private Object[] get_boolean_value_args_object = new Object[1];
    private Class[]  get_boolean_value_args_class = { Object.class };
    private CallMethod  get_boolean_value_call_method = null;

    private Method   get_char_value_method = null;
    private String   GET_CHAR_VALUE_METHOD_NAME = "getCharValue";
    private Object[] get_char_value_args_object = new Object[1];
    private Class[]  get_char_value_args_class = { Object.class };
    private CallMethod  get_char_value_call_method = null;

    private Method   get_object_value_method = null;
    private String   GET_OBJECT_VALUE_METHOD_NAME = "getValue";
    private Object[] get_object_value_args_object = new Object[1];
    private Class[]  get_object_value_args_class = { Object.class };
    private CallMethod  get_object_value_call_method = null;


    public CallMethodWrapper(Class param_call_method_class) throws Exception {
        ObjectServerException object_server_exception;

        if (param_call_method_class == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_CONSTRUCTOR_WRAPPER,
                    getMessageResource().getMessage(
                      ObjectServerError.PARAMETER_ERROR_CONSTRUCTOR_WRAPPER)));
            throw object_server_exception;
        }

        call_method_class = param_call_method_class;

        try {
            object_instance = call_method_class.newInstance();
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_CLASS_NEW_INSTANCE_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerError.JAVA_CLASS_NEW_INSTANCE_EXCEPTION)));
            throw object_server_exception;
        }

        try {
            get_parameter_type_method = call_method_class.getMethod(
                GET_PARAMETER_TYPE_METHOD_NAME, 
                get_parameter_type_args_class);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_TYPE_METHOD_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_TYPE_METHOD_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_parameter_type_call_method = CallMethodFactory.getCallMethod(
                get_parameter_type_method, getMessageResource());
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            get_parameter_method = call_method_class.getMethod(
                GET_PARAMETER_METHOD_NAME, 
                get_parameter_args_class);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_METHOD_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_METHOD_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_parameter_call_method = CallMethodFactory.getCallMethod(
                get_parameter_method, getMessageResource());
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void invoke(Object obj) throws Exception {
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_INVOKE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_INVOKE_WRAPPER)));
            throw object_server_exception;
        }
        if (invoke_method == null) {
            try {
                invoke_method = call_method_class.getMethod(
                    INVOKE_METHOD_NAME, invoke_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.INVOKE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.INVOKE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                invoke_call_method = CallMethodFactory.getCallMethod(
                    invoke_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            invoke_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.INVOKE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                         ObjectServerError.INVOKE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            invoke_call_method.invoke(object_instance);
        } catch (java.lang.reflect.InvocationTargetException exception_object) {
            Throwable   cause;
            cause = exception_object.getCause();
            if (cause != null && cause.getClass().getName().compareTo(
            ObjectServerHeader.JAVA_INVOCATION_TARGET_EXCEPTION_KEY) == 0) {
                throw (Exception)cause;
            }
            throw exception_object;
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.INVOKE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                         ObjectServerError.INVOKE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
    }

    public void setMethod(Method method_object) throws Exception {
        ObjectServerException object_server_exception;

        if (method_object == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SET_METHOD_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SET_METHOD_WRAPPER)));
            throw object_server_exception;
        }
        if (set_method_method == null) {
            try {
                set_method_method = call_method_class.getMethod(
                    SET_METHOD_METHOD_NAME, set_method_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_METHOD_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_METHOD_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_method_call_method = CallMethodFactory.getCallMethod(
                    set_method_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            set_method_call_method.set(object_instance, (Object)method_object, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_METHOD_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_METHOD_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_method_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_METHOD_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_METHOD_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
    }

    public int getParameterCount() throws Exception {
        int  int_rt;
        Object   object_rt;
        byte[]   byte_rt;
        Integer  integer_rt;
        ObjectServerException object_server_exception;

        if (get_count_method == null) {
            try {
                get_count_method = call_method_class.getMethod(
                    GET_COUNT_METHOD_NAME, 
                    get_count_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_COUNT_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_COUNT_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_count_call_method = CallMethodFactory.getCallMethod(
                    get_count_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }

        try {
            get_count_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_COUNT_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_COUNT_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
        
            integer_rt = (Integer)get_count_call_method.get(object_instance);

        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_COUNT_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_COUNT_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        int_rt = integer_rt.intValue();
        return int_rt;
    }

    public String getParameterType(int index) throws Exception {
        int  int_rt;
        String   string_rt;
        Object   object_rt;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        integer_index = Integer.valueOf(index);
        try {
            get_parameter_type_call_method.set(object_instance, (Object)integer_index, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_TYPE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_TYPE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_parameter_type_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_TYPE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_TYPE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            string_rt = (String)get_parameter_type_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_TYPE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_TYPE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return string_rt;
    }

    public Object getParameter(int index) throws Exception {
        int  int_rt;
        Object object_rt;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        integer_index = Integer.valueOf(index);
        try {
            get_parameter_call_method.set(object_instance, (Object)integer_index, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            object_rt = (Object)get_parameter_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_PARAMETER_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_PARAMETER_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return object_rt;
    }

    public void setInt(Object obj, int arg, int index) throws Exception {
        Integer  integer_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETINTPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETINTPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_int_parameter_method == null) {
            try {
                set_int_parameter_method = call_method_class.getMethod(
                    SET_INT_PARAMETER_METHOD_NAME, set_int_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_INT_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_INT_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_int_parameter_call_method = CallMethodFactory.getCallMethod(
                    set_int_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        integer_arg = Integer.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_int_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_int_parameter_call_method.set(object_instance, (Object)integer_arg, 1);
            set_int_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_INT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_INT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_int_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_INT_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_INT_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setLong(Object obj, long arg, int index) throws Exception {
        Long  long_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETLONGPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETLONGPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_long_parameter_method == null) {
            try {
                set_long_parameter_method = call_method_class.getMethod(
                    SET_LONG_PARAMETER_METHOD_NAME, set_long_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_LONG_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_LONG_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_long_parameter_call_method = CallMethodFactory.getCallMethod(
                    set_long_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        long_arg = Long.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_long_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_long_parameter_call_method.set(object_instance, (Object)long_arg, 1);
            set_long_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_LONG_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_LONG_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_long_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_LONG_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_LONG_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setFloat(Object obj, float arg, int index) throws Exception {
        Float  float_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETFLOATPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETFLOATPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_float_parameter_method == null) {
            try {
                set_float_parameter_method = call_method_class.getMethod(
                    SET_FLOAT_PARAMETER_METHOD_NAME, set_float_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_FLOAT_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_FLOAT_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_float_parameter_call_method = CallMethodFactory.getCallMethod(
                    set_float_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        float_arg = Float.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_float_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_float_parameter_call_method.set(object_instance, (Object)float_arg, 1);
            set_float_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_FLOAT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_FLOAT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_float_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_FLOAT_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_FLOAT_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setDouble(Object obj, double arg, int index) throws Exception {
        Double  double_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETDOUBLEPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETDOUBLEPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_double_parameter_method == null) {
            try {
                set_double_parameter_method = call_method_class.getMethod(
                    SET_DOUBLE_PARAMETER_METHOD_NAME, set_double_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_DOUBLE_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_DOUBLE_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_double_parameter_call_method = CallMethodFactory.getCallMethod(
                    set_double_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        double_arg = Double.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_double_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_double_parameter_call_method.set(object_instance, (Object)double_arg, 1);
            set_double_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_DOUBLE_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_DOUBLE_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_double_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_DOUBLE_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_DOUBLE_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setShort(Object obj, short arg, int index) throws Exception {
        Short  short_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETSHORTPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETSHORTPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_short_parameter_method == null) {
            try {
                set_short_parameter_method = call_method_class.getMethod(
                    SET_SHORT_PARAMETER_METHOD_NAME, 
                    set_short_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_SHORT_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_SHORT_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_short_parameter_call_method = CallMethodFactory.getCallMethod(
                    set_short_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        short_arg = Short.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_short_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_short_parameter_call_method.set(object_instance, (Object)short_arg, 1);
            set_short_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_SHORT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_SHORT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_short_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_SHORT_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_SHORT_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setByte(Object obj, byte arg, int index) throws Exception {
        Byte  byte_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETBYTEPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETBYTEPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_byte_parameter_method == null) {
            try {
                set_byte_parameter_method = call_method_class.getMethod(
                    SET_BYTE_PARAMETER_METHOD_NAME, set_byte_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_BYTE_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_BYTE_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_byte_parameter_call_method = CallMethodFactory.getCallMethod(
                    set_byte_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }

        byte_arg = Byte.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_byte_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_byte_parameter_call_method.set(object_instance, (Object)byte_arg, 1);
            set_byte_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_BYTE_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_BYTE_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_short_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_BYTE_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_BYTE_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setBoolean(Object obj, boolean arg, int index) throws Exception {
        Boolean  boolean_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETBOOLEANPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETBOOLEANPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_boolean_parameter_method == null) {
            try {
                set_boolean_parameter_method = call_method_class.getMethod(
                    SET_BOOLEAN_PARAMETER_METHOD_NAME, 
                    set_boolean_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_BOOLEAN_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_BOOLEAN_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_boolean_parameter_call_method 
                    = CallMethodFactory.getCallMethod(
                    set_boolean_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        boolean_arg = Boolean.valueOf(arg);
        integer_index = Integer.valueOf(index);
        try {
            set_boolean_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_boolean_parameter_call_method.set(object_instance, (Object)boolean_arg, 1);
            set_boolean_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_BOOLEAN_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_BOOLEAN_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_boolean_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_BOOLEAN_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_BOOLEAN_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void setChar(Object obj, char arg, int index) throws Exception {
        Integer  integer_arg;
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETCHARPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETCHARPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_char_parameter_method == null) {
            try {
                set_char_parameter_method = call_method_class.getMethod(
                    SET_CHAR_PARAMETER_METHOD_NAME, 
                    set_char_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_CHAR_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_CHAR_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_char_parameter_call_method 
                    = CallMethodFactory.getCallMethod(
                    set_char_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        integer_arg = Integer.valueOf((int)arg);
        integer_index = Integer.valueOf(index);
        try {
            set_char_parameter_call_method.set(object_instance, (Object)obj, 0);
            set_char_parameter_call_method.set(object_instance, (Object)integer_arg, 1);
            set_char_parameter_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_CHAR_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_CHAR_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_char_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_CHAR_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_CHAR_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public void set(Object obj, Object arg, int index) throws Exception {
        Integer  integer_index;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SETOBJECTPARAMETER_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SETOBJECTPARAMETER_WRAPPER)));
            throw object_server_exception;
        }
        if (set_object_method == null) {
            try {
                set_object_method = call_method_class.getMethod(
                    SET_OBJECT_METHOD_NAME, 
                    set_object_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.SET_OBJECT_PARAMETER_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.SET_OBJECT_PARAMETER_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                set_object_call_method 
                    = CallMethodFactory.getCallMethod(
                    set_object_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        integer_index = Integer.valueOf(index);
        try {
            set_object_call_method.set(object_instance, (Object)obj, 0);
            set_object_call_method.set(object_instance, (Object)arg, 1);
            set_object_call_method.set(object_instance, (Object)integer_index, 2);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_OBJECT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_OBJECT_PARAMETER_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            set_object_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.SET_OBJECT_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.SET_OBJECT_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return ;
    }

    public int getInt(Object obj) throws Exception {
        int  int_rt;
        Object   object_rt;
        Integer  integer_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETINTVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETINTVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_int_value_method == null) {
            try {
                get_int_value_method = call_method_class.getMethod(
                    GET_INT_VALUE_METHOD_NAME, get_int_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_INT_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_INT_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_int_value_call_method = CallMethodFactory.getCallMethod(
                    get_int_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_int_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_INT_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_INT_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_int_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_INT_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_INT_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            integer_rt = (Integer)get_int_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_INT_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_INT_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        int_rt = integer_rt.intValue();
        return int_rt;
    }

    public long getLong(Object obj) throws Exception {
        long  long_rt;
        Object   object_rt;
        Long  long_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETLONGVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETLONGVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_long_value_method == null) {
            try {
                get_long_value_method = call_method_class.getMethod(
                    GET_LONG_VALUE_METHOD_NAME, get_long_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_LONG_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_LONG_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_long_value_call_method = CallMethodFactory.getCallMethod(
                    get_long_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_long_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_LONG_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_LONG_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_long_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_LONG_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_LONG_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            long_object_rt = (Long)get_long_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_LONG_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_LONG_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        long_rt = long_object_rt.longValue();
        return long_rt;
    }

    public float getFloat(Object obj) throws Exception {
        float  float_rt;
        Float  float_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETFLOATVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETFLOATVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_float_value_method == null) {
            try {
                get_float_value_method = call_method_class.getMethod(
                GET_FLOAT_VALUE_METHOD_NAME, get_float_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_FLOAT_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_FLOAT_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_float_value_call_method = CallMethodFactory.getCallMethod(
                get_float_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_float_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_FLOAT_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_FLOAT_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_float_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_FLOAT_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_FLOAT_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            float_object_rt = (Float)get_float_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_FLOAT_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_FLOAT_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        float_rt = float_object_rt.floatValue();
        return float_rt;
    }

    public double getDouble(Object obj) throws Exception {
        double  double_rt;
        Double  double_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETDOUBLEVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETDOUBLEVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_double_value_method == null) {
            try {
                get_double_value_method = call_method_class.getMethod(
                GET_DOUBLE_VALUE_METHOD_NAME, get_double_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_DOUBLE_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_DOUBLE_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_double_value_call_method = CallMethodFactory.getCallMethod(
                get_double_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_double_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_DOUBLE_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_DOUBLE_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_double_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_DOUBLE_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_DOUBLE_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            double_object_rt = (Double)get_double_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_DOUBLE_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_DOUBLE_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        double_rt = double_object_rt.doubleValue();
        return double_rt;
    }

    public short getShort(Object obj) throws Exception {
        short  short_rt;
        Short  short_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETSHORTVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETSHORTVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_short_value_method == null) {
            try {
                get_short_value_method = call_method_class.getMethod(
                    GET_SHORT_VALUE_METHOD_NAME, get_short_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_SHORT_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_SHORT_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_short_value_call_method = CallMethodFactory.getCallMethod(
                    get_short_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_short_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_SHORT_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_SHORT_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_short_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_SHORT_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_SHORT_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            short_object_rt = (Short)get_short_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_SHORT_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_SHORT_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        short_rt = short_object_rt.shortValue();
        return short_rt;
    }

    public byte getByte(Object obj) throws Exception {
        byte  byte_rt;
        Byte  byte_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETBYTEVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETBYTEVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_byte_value_method == null) {
            try {
                get_byte_value_method = call_method_class.getMethod(
                GET_BYTE_VALUE_METHOD_NAME, get_byte_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_BYTE_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_BYTE_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_byte_value_call_method = CallMethodFactory.getCallMethod(
                get_byte_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_byte_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_BYTE_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_BYTE_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_byte_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_BYTE_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_BYTE_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            byte_object_rt = (Byte)get_byte_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_BYTE_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_BYTE_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        byte_rt = byte_object_rt.byteValue();
        return byte_rt;
    }

    public boolean getBoolean(Object obj) throws Exception {
        boolean  boolean_rt;
        Boolean  boolean_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETBOOLEANVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETBOOLEANVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_boolean_value_method == null) {
            try {
                get_boolean_value_method = call_method_class.getMethod(
                    GET_BOOLEAN_VALUE_METHOD_NAME, get_boolean_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_BOOLEAN_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_BOOLEAN_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_boolean_value_call_method = CallMethodFactory.getCallMethod(
                    get_boolean_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_boolean_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_BOOLEAN_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_BOOLEAN_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_boolean_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_BOOLEAN_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_BOOLEAN_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            boolean_object_rt = (Boolean)get_boolean_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_BOOLEAN_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_BOOLEAN_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        boolean_rt = boolean_object_rt.booleanValue();
        return boolean_rt;
    }

    public char getChar(Object obj) throws Exception {
        char  char_rt;
        Object   object_rt;
        Integer  integer_object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETCHARVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETCHARVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_char_value_method == null) {
            try {
                get_char_value_method = call_method_class.getMethod(
                    GET_CHAR_VALUE_METHOD_NAME, get_char_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_CHAR_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_CHAR_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_char_value_call_method = CallMethodFactory.getCallMethod(
                    get_char_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_char_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_CHAR_VALUE_SET_PARAMETER_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_CHAR_VALUE_SET_PARAMETER_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_char_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_CHAR_VALUE_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_CHAR_VALUE_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            integer_object_rt = (Integer)get_char_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_CHAR_VALUE_RETURN_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_CHAR_VALUE_RETURN_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        char_rt = (char)integer_object_rt.intValue();
        return char_rt;
    }

    public Object get(Object obj) throws Exception {
        Object  object_rt;
        ObjectServerException object_server_exception;

        if (obj == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GETOBJECTVALUE_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GETOBJECTVALUE_WRAPPER)));
            throw object_server_exception;
        }
        if (get_object_value_method == null) {
            try {
                get_object_value_method = call_method_class.getMethod(
                    GET_OBJECT_VALUE_METHOD_NAME, get_object_value_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                get_object_value_call_method = CallMethodFactory.getCallMethod(
                    get_object_value_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            get_object_value_call_method.set(object_instance, (Object)obj, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            get_object_value_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            object_rt = get_object_value_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.GET_OBJECT_VALUE_METHOD_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return object_rt;
    }

    public boolean isInputStreamParameter(int index) throws Exception {
        Boolean flag;
        Integer integer_index;
        ObjectServerException object_server_exception;

        if (is_input_stream_parameter_method == null) {
            try {
                is_input_stream_parameter_method = 
                    call_method_class.getMethod(
                        IS_INPUT_STREAM_PARAMETER_METHOD_NAME, 
                        is_input_stream_parameter_args_class);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.IS_INPUT_STREAM_PARAMETER_ERROR_WRAPPER,
                        getMessageResource().getMessage(
                            ObjectServerError.IS_INPUT_STREAM_PARAMETER_ERROR_WRAPPER)));
                throw object_server_exception;
            }
            try {
                is_input_stream_parameter_call_method = 
                    CallMethodFactory.getCallMethod(
                    is_input_stream_parameter_method, getMessageResource());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        integer_index = Integer.valueOf(index);
        try {
            is_input_stream_parameter_call_method.set(object_instance, (Object)integer_index, 0);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.IS_INPUT_STREAM_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.IS_INPUT_STREAM_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            is_input_stream_parameter_call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.IS_INPUT_STREAM_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.IS_INPUT_STREAM_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        try {
            flag = (Boolean)is_input_stream_parameter_call_method.get(object_instance);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.IS_INPUT_STREAM_PARAMETER_INVOKE_ERROR_WRAPPER,
                    getMessageResource().getMessage(
                        ObjectServerError.IS_INPUT_STREAM_PARAMETER_INVOKE_ERROR_WRAPPER)));
            throw object_server_exception;
        }
        return flag.booleanValue();
    }
}

