/**
 * Copyright (C) All rights are reserved. 
 *
 * @Module     ClassInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/03/01
 *
 */

package oserv;

import java.lang.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

public class ClassInputStream extends FileInputStream {
    private    String             target_file_name = null;
    private    String             target_class_name = null;
    private    String             target_type = null;
    private    boolean            class_flag = false;
    private    static String      JPN_LANGUAGE = "jpn";

    private    long               target_length;
    private int debug = 0;
    private LogPrintStream logStream = null;
    private MessageResource message_resource = new MessageResource();

    protected ClassInputStream(String file_name, String class_name, String type, long file_length) throws FileNotFoundException {
        super(file_name);
        File   file_object;
        target_file_name = file_name;
        target_class_name = class_name;
        target_type = type;
        target_length = file_length;
    }

    public void setMessageResource(MessageResource param_message_resource) {
        message_resource = param_message_resource;
    }

    public void setLogPrintStream(LogPrintStream  log_print_stream) {
        logStream = log_print_stream;
    }

    public LogPrintStream getLogPrintStream() {
        return logStream;
    }

    public void setDebug(int debug_level) {
        debug = debug_level;
    }

    public int getDebug() {
        return debug;
    }

    public static ClassInputStream valueOf(String class_name,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) 
        throws FileNotFoundException, ClassNotFoundException, Exception {
        int                index;
        long               length;
        File               file_object;
        String             type;
        String             file_name;
        String             base_file_name;
        String             user_home_string;
        String             message;
        ClassInputStream   class_input_stream;

        if (class_name == null) {
           Exception  exception_object;

           message = parameter_message_resource.getFormattedMessage(
               ObjectServerException.PARAMETER_ERROR_EXCEPTION,
               parameter_message_resource.getMessage(
                   ObjectServerException.PARAMETER_ERROR_EXCEPTION));
           exception_object = new Exception(message);
           throw exception_object;
        }
        user_home_string = System.getProperty(
              ObjectServerHeader.USER_HOME_KEY) ;
        if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
            && parameter_log_stream != null) {
            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_USER_HOME,
                parameter_message_resource.getMessage(
                    ObjectServerDebug.DEBUG_USER_HOME),
                ObjectServerHeader.BLANK_STRING,
                user_home_string);
            parameter_log_stream.printLogln(message);
        }

        index = class_name.lastIndexOf(ObjectServerHeader.OBJECT_DOT_SEPARATOR);
        if (index == 0 || index == class_name.length()) {
            Exception   exception_object;
            message = parameter_message_resource.getFormattedMessage(
                ObjectServerException.CLASS_NOT_FOUND_EXCEPTION,
                parameter_message_resource.getMessage(
                    ObjectServerException.CLASS_NOT_FOUND_EXCEPTION));
            exception_object = new Exception(message);
            throw exception_object;
        }
        if (index > 0) {
            base_file_name = class_name.substring(index+1);
        } else {
            base_file_name = class_name;    
        }
        if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
            && parameter_log_stream != null) {

            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_BASE_FILE_NAME,
                parameter_message_resource.getMessage(
                    ObjectServerDebug.DEBUG_BASE_FILE_NAME),
                ObjectServerHeader.BLANK_STRING,
                base_file_name);
            parameter_log_stream.printLogln(message);
        }

        file_name = base_file_name;
        file_name = file_name.concat(ObjectServerHeader.SUFFIX_SEPARATOR +
            ObjectServerHeader.JAR_SUFFIX);
        file_object = new File(file_name);

        if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
            && parameter_log_stream != null) {
            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_JAR_NAME,
                parameter_message_resource.getMessage(
                    ObjectServerDebug.DEBUG_JAR_NAME),
                ObjectServerHeader.BLANK_STRING,
                file_name);
            parameter_log_stream.printLogln(message);
        }

        if (file_object.exists() == false) {
            if (user_home_string != null) {
                file_name = user_home_string.concat(File.separator +
                    file_name);
                if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
                    && parameter_log_stream != null) {
                    message = parameter_message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_JAR_NAME,
                        parameter_message_resource.getMessage(
                            ObjectServerDebug.DEBUG_JAR_NAME),
                       ObjectServerHeader.BLANK_STRING,
                       file_name);
                   parameter_log_stream.printLogln(message);
                }
                file_object = new File(file_name);
            } 
            if (file_object.exists() == false) {
                file_name = base_file_name.concat(
                    ObjectServerHeader.SUFFIX_SEPARATOR +
                    ObjectServerHeader.CLASS_SUFFIX);
                if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
                    && parameter_log_stream != null) {
                    message = parameter_message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_CLASS_NAME,
                        parameter_message_resource.getMessage(
                            ObjectServerDebug.DEBUG_CLASS_NAME),
                       ObjectServerHeader.BLANK_STRING,
                       file_name);
                   parameter_log_stream.printLogln(message);
                }
                file_object = new File(file_name);
                if (file_object.exists() == false) {
                    if (user_home_string != null) {
                        file_name = user_home_string.concat(
                        File.separator +
                        file_name);
                    }
                    if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
                    && parameter_log_stream != null) {
                        message = parameter_message_resource.getFormattedMessage(
                            ObjectServerDebug.DEBUG_CLASS_NAME,
                            parameter_message_resource.getMessage(
                                ObjectServerDebug.DEBUG_CLASS_NAME),
                            ObjectServerHeader.BLANK_STRING,
                            file_name);
                        parameter_log_stream.printLogln(message);
                    }
                    file_object = new File(file_name);
                    if (file_object.exists() == false) {
                        Exception   exception_object;
                        message = parameter_message_resource.getFormattedMessage(
                            ObjectServerException.CLASS_NOT_FOUND_EXCEPTION,
                            parameter_message_resource.getMessage(
                                ObjectServerException.CLASS_NOT_FOUND_EXCEPTION));
                        exception_object = new Exception(message);
                        throw exception_object;
                    }
                }
                if (checkClassFile(file_name, class_name,
                    parameter_message_resource,
                    parameter_debug, parameter_log_stream) == false) {
                    Exception   exception_object;
                    message = parameter_message_resource.getFormattedMessage(
                        ObjectServerException.CLASS_NOT_FOUND_EXCEPTION,
                        parameter_message_resource.getMessage(
                            ObjectServerException.CLASS_NOT_FOUND_EXCEPTION));
                    exception_object = new Exception(message);
                    throw exception_object;
                }
                type = ObjectServerHeader.CLASS_SUFFIX;
                length = file_object.length();
            } else {
                if (checkJarFile(file_name, class_name,
                    parameter_message_resource,
                    parameter_debug, parameter_log_stream) == false) {
                    Exception   exception_object;
                    message = parameter_message_resource.getFormattedMessage(
                        ObjectServerException.CLASS_NOT_FOUND_EXCEPTION,
                        parameter_message_resource.getMessage(
                            ObjectServerException.CLASS_NOT_FOUND_EXCEPTION));
                    exception_object = new Exception(message);
                    throw exception_object;
                }
                type = ObjectServerHeader.JAR_SUFFIX;
                length = file_object.length();
            }
        } else {
            if (checkJarFile(file_name, class_name,
                    parameter_message_resource,
                    parameter_debug, parameter_log_stream) == false) {
                Exception   exception_object;
                message = parameter_message_resource.getFormattedMessage(
                    ObjectServerException.CLASS_NOT_FOUND_EXCEPTION,
                    parameter_message_resource.getMessage(
                        ObjectServerException.CLASS_NOT_FOUND_EXCEPTION));
                exception_object = new Exception(message);
                throw exception_object;
            }
            type = ObjectServerHeader.JAR_SUFFIX;
            length = file_object.length();
        }
        try {
            class_input_stream = new ClassInputStream(
                file_name, class_name, type, length);
        } catch (FileNotFoundException exception_object) {
            throw exception_object;
        } catch (Exception exception_object) {
            throw exception_object;
        }
        class_input_stream.setDebug(parameter_debug);
        class_input_stream.setMessageResource(parameter_message_resource);
        class_input_stream.setLogPrintStream(parameter_log_stream);
        return class_input_stream;
    }

    private static boolean checkJarFile(String file_name, String class_name,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) {
        String  url_path;
        Boolean flag;
        String  message;

        url_path = new String(
            ObjectServerHeader.JAR_URL_HEAD + file_name + 
            ObjectServerHeader.JAR_URL_END);

        if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
            && parameter_log_stream != null) {

            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_URL_PATH,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_URL_PATH),
                    ObjectServerHeader.BLANK_STRING,
                    url_path);
            parameter_log_stream.printLogln(message);
            message = parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_CLASS_NAME,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CLASS_NAME),
                    ObjectServerHeader.BLANK_STRING,
                    class_name);
            parameter_log_stream.printLogln(message);
        }

        flag = checkFileByURL(url_path, class_name,
            parameter_message_resource, parameter_debug, parameter_log_stream);
        return flag;
    }

    private static boolean checkClassFile(String file_name, String class_name,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) {
        String  url_path;
        Boolean flag;
        String  message;

        url_path = new String(
            ObjectServerHeader.JAR_URL_HEAD + file_name + 
            ObjectServerHeader.JAR_URL_END);

        if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
            && parameter_log_stream != null) {

            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_URL_PATH,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_URL_PATH),
                    ObjectServerHeader.BLANK_STRING,
                    url_path);
            parameter_log_stream.printLogln(message);
            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_CLASS_NAME,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CLASS_NAME),
                    ObjectServerHeader.BLANK_STRING,
                    class_name);
            parameter_log_stream.printLogln(message);
        }

        flag = checkFileByURL(url_path, class_name,
            parameter_message_resource, parameter_debug, parameter_log_stream);
        return flag;
    }

    private static boolean checkFileByURL(String url_string, String class_name,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) {
        URL   url;
        Class  sp;
        URL[] urls;
        URLClassLoader uloader;
        String         message;

        if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
            && parameter_log_stream != null) {
            message = parameter_message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_CLASS_NAME,
                parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CLASS_NAME),
                ObjectServerHeader.BLANK_STRING,
                class_name
            );
            parameter_log_stream.printLogln(message);
        }
        try {
            url = new URL(url_string);
        } catch (Exception exception_object) {
            if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
                && parameter_log_stream != null) {
                message = parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                    ObjectServerHeader.BLANK_STRING,
                    exception_object.getMessage());
                parameter_log_stream.printLogln(message);
            }
            return false;
        }
        urls = new URL[1];
        urls[0] = url;
        uloader = new URLClassLoader(urls);      
        try {
            sp = Class.forName(class_name, true, (ClassLoader)uloader);
        } catch (Exception exception_object) {
            if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE 
                && parameter_log_stream != null) {
                message = parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                    ObjectServerHeader.BLANK_STRING,
                    exception_object.getMessage());
                parameter_log_stream.printLogln(message);
            }
            return false;
        }
        return true;
    }

    public String getFilePath() {
        return target_file_name;
    }

    public String getClassPath() {
        return target_class_name;
    }

    public String getType() {
        return target_type;
    }

    public long length() {
        return target_length;
    }

    private static String DEBUG_PARAMETER_ERROR = "Parameter error";
    private static String DEBUG_LENGTH_STRING = "length";
    private static String DEBUG_CLASSPATH_STRING = "Class path";
    private static String DEBUG_FILEPATH_STRING = "FILE path";
    private static String DEBUG_TYPE_STRING = "Type";

    public static void main(String[] args) {
        int                length;
        String             type;
        String             class_string;
        String             file_string;
        String             error_message;
        ClassInputStream   class_input_stream;
        int                debug;
        LogPrintStream     log_print_stream;
        MessageResource    message_resource;

        if (args.length != 1) {
            error_message = DEBUG_PARAMETER_ERROR; 
            System.err.println(error_message);
        }

        debug = 9;
        log_print_stream = new LogPrintStream(System.out);
        message_resource = new MessageResource();

        message_resource.setLanguage(JPN_LANGUAGE);

        try {
            class_input_stream = ClassInputStream.valueOf(args[0],
                message_resource,
                debug,
                log_print_stream );
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage()); 
            return;
        }

        length = (int)class_input_stream.length();
        System.out.println(DEBUG_LENGTH_STRING + 
            ObjectServerHeader.BLANK_STRING +
            String.valueOf(length));
        type = class_input_stream.getType();
        System.out.println(DEBUG_TYPE_STRING +
            ObjectServerHeader.BLANK_STRING + type );
        class_string = class_input_stream.getClassPath();
        System.out.println(DEBUG_CLASSPATH_STRING +
            ObjectServerHeader.BLANK_STRING + class_string );
        file_string = class_input_stream.getFilePath();
        System.out.println(DEBUG_FILEPATH_STRING +
            ObjectServerHeader.BLANK_STRING + file_string );
    }
}
