/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ClientSideObjectTable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/11/10
 *             2.1 2011/10/07
 *
 */

package oserv;

import java.lang.*;
import java.net.*;
import java.io.*;

import oserv.*;

/**
 * This class is defined as ClientSideObjectTable.
 *
 * @see  oserv.ObjectServerHeader
 */
public class ClientSideObjectTable extends ObjectServerHeader {
    private static String[]            entry_table = null;
    private static String[]            type_table = null;
    private static String[]            forward_entry_table = null;

    public final static String EMPTY_STRING = "";
    public final static String SPACE_STRING = " ";
    public final int NAME_WIDTH = 20;
    /**
           1234567890123456789012345678901234567890
    **/
    public final static String BLANK_STRING 
        = "                                        ";
    public final static String LIST_HEADER  
        = "Entry                                   Service Type";
    public final static String FORWARD_LIST_HEADER = "Entry";
    public final static String EMPTY_LIST  = "Entry is empty.";
    public final static String JAR_SUFFIX = "jar";
    public final static String SUFFIX_SEPARATOR = ".";
    public final static String OBJECT_DOT_SEPARATOR = ".";
    public final static String DIRECTORY_SEPARATOR = "/";
    public final static String DEBUG_LIST_SIZE = "List size ";
    public final static String DEBUG_CLIENT_SIDE_OBJECT_TABLE 
        = "ClientSideObjectTable";
    public final static String DEBUG_LIST_CONTENTS = "List contents ";
    public static int debug = 0;

    private static String TRACE_CLASSNAME = "oserv.ClientSideObjectTable";
    private static String TRACE_CONSTRUCTOR = "<init>";

    public static String directory_separator_key = DIRECTORY_SEPARATOR;

    public ClientSideObjectTable(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        String  list_request = new String(EMPTY_STRING);
        int     int_list_size;

        Integer table_size = null;
        Integer list_size = null;
        try {
            list_size = (Integer)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONSTRUCTOR, 0, exception_object);
            throw exception_object;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            System.out.println(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_LIST_SIZE,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_LIST_SIZE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    list_size.toString()));
        }
        int_list_size = list_size.intValue();
        entry_table = new String[int_list_size];
        type_table = new String[int_list_size];
        for (int n = 0; n < int_list_size; n++) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                System.out.println(DEBUG_LIST_CONTENTS + String.valueOf(n));
            }
            try {
                entry_table[n] = 
                    (String)retry_object_input_stream.retryReadObject();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_CONSTRUCTOR, 0, exception_object);
                throw exception_object;
            }
            try {
                type_table[n] = 
                    (String)retry_object_input_stream.retryReadObject();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_CONSTRUCTOR, 0, exception_object);
                throw exception_object;
            }
        }

        try {
            table_size = (Integer)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONSTRUCTOR, 0, exception_object);
            throw exception_object;
        }
        try {
            list_size = (Integer)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONSTRUCTOR, 0, exception_object);
            throw exception_object;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            System.out.println(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_LIST_SIZE,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_LIST_SIZE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    list_size.toString()));
        }
        int_list_size = list_size.intValue();
        forward_entry_table = new String[int_list_size];
        for (int n = 0; n < int_list_size; n++) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                System.out.println(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_INDEX_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_INDEX_KEY),
                            String.valueOf(n)));
            }
            try {
                forward_entry_table[n] 
                    = (String)retry_object_input_stream.retryReadObject();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_CONSTRUCTOR, 0, exception_object);
                throw exception_object;
            }
        }
    }

    public URI getForwardServer(String service) {
        int    idx;
        URI    service_host;
        URI    forward_entry;
        String forward_service_host;
        String forward_entry_service;

        if (service == null) {
            return null;
        }
        if (forward_entry_table == null) {
            return null;
        }
        for (int n = 0; n < forward_entry_table.length; n++) {
            try {
                forward_entry = new URI(forward_entry_table[n]);
            } catch (Exception exception_object) {
                continue;
            }
            forward_entry_service = forward_entry.getPath();
            forward_service_host = forward_entry.toString();
            idx = forward_entry_service.indexOf(DIRECTORY_SEPARATOR);
            if (idx == 0) {
                forward_entry_service = forward_entry_service.substring(1);
            }
            idx = forward_service_host.lastIndexOf(forward_entry_service);
            if (idx >= 0) {
                forward_service_host = forward_service_host.substring(0, idx);
            }

            forward_entry_service = forward_entry_service.replace(
                                       DIRECTORY_SEPARATOR, 
                                       OBJECT_DOT_SEPARATOR
                                   );
            if (forward_entry_service.compareTo(service) == 0) {
                try {
                    service_host = new URI(forward_service_host);
                    return service_host; 
                } catch (Exception exception_object) {
                    continue;
                }
            }
        }
        return null;
    }

    public String[] getServiceList() {
        return entry_table;
    }

    public String[] getForwardServiceList() {
        return forward_entry_table;
    }

    private String getListEntry(String entry, String type) {
        String head_word;

        if (entry == null) {
            if (type != null) {
                head_word = BLANK_STRING + type;
            } else {
                head_word = BLANK_STRING;
            }
        } else {
            if (BLANK_STRING.length() > entry.length()) {
                if (type != null) {
                    head_word = entry + BLANK_STRING.substring(entry.length()) + type;
                } else {
                    head_word = entry + BLANK_STRING.substring(entry.length());
                }
            } else {
                if (type != null) {
                    head_word = entry + SPACE_STRING + type;
                } else {
                    head_word = entry + SPACE_STRING;
                }
            }
        }
        return head_word;
    }

    public void printList()  {
        int idx;

        if (entry_table == null || entry_table.length == 0) {
            System.out.println(
                getMessageResource().getFormattedMessage(
                    EMPTY_LIST,
                    getMessageResource().getMessage(EMPTY_LIST) ));
            return;
        }
        System.out.println(
            getMessageResource().getFormattedMessage(
                LIST_HEADER,
                getMessageResource().getMessage(LIST_HEADER) ));
        String entry_string;


        for (int n = 0; n < entry_table.length; n++) {
            entry_string = getListEntry(entry_table[n],
                getMessageResource().getMessage(type_table[n]) );
           
            /**
             * remove forward entry
             */
            idx = entry_string.indexOf(OSERV_SCHEME);
            if (idx >= 0) {
                continue;
            }
            System.out.println(entry_string);
        }
    }

    public void printForwardList()  {
        int idx;

        if (forward_entry_table == null || forward_entry_table.length == 0) {
            System.out.println(
                getMessageResource().getFormattedMessage(
                    EMPTY_LIST,
                    getMessageResource().getMessage(EMPTY_LIST) ));
            return;
        }
        System.out.println(
            getMessageResource().getFormattedMessage(
                FORWARD_LIST_HEADER,
                getMessageResource().getMessage(FORWARD_LIST_HEADER) ));
        String entry_string;

        for (int n = 0; n < forward_entry_table.length; n++) {
            entry_string = forward_entry_table[n];

            /**
             * remove forward entry
             */
            idx = entry_string.indexOf(OSERV_SCHEME);
            if (idx < 0) {
                continue;
            }
            System.out.println(entry_string);
        }
    }
}
