/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ConnectedSocket.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/01/19
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.lang.reflect.*;
import java.util.*;

public class ConnectedSocket extends SerializableObject {
    private Socket                 socket = null;
    private ObjectOutputStream     socket_object_output_stream;
    private RetryObjectInputStream socket_object_input_stream;

    public ConnectedSocket() {
    }

    public Socket connect(String host_address, int port_number) 
                throws IOException  {
        InetSocketAddress  inet_socket_address;

        if (host_address == null) {
            IOException exception_object;
            exception_object = new IOException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.HOSTNAME_PARAMETER_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.HOSTNAME_PARAMETER_ERROR_EXCEPTION)));
            throw exception_object;
        }
        socket = new Socket();
        inet_socket_address = new InetSocketAddress( 
                host_address, port_number );
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CALL_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        inet_socket_address.toString(),
                        getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_CODE_002)));
        }
        try {
            socket.connect(inet_socket_address);
        } catch (IOException exception_object) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        exception_object.toString()));
            }
            throw exception_object;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_END_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_END_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket.toString()));
        }

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && getLogStream() != null) {
            if (socket.getTcpNoDelay() == true) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket.toString(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_TCP_NO_DELAY_TRUE));
            } else {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket.toString(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_TCP_NO_DELAY_FALSE));
            }
        }
        try {
            socket.setTcpNoDelay(true);
        } catch (SocketException exception_object) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        exception_object.toString()));
            }
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
            if (socket.getTcpNoDelay() == true) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket.toString(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_TCP_NO_DELAY_TRUE));
            } else {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket.toString(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_TCP_NO_DELAY_FALSE));
            }
        }
        return socket;
    }

    public ObjectOutputStream getOutputStream() throws IOException, Exception {
        OutputStream  socket_output_stream;
        ObjectServerException   object_server_exception;

        try {
            socket_output_stream = socket.getOutputStream();
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CREATE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CREATE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket_output_stream.toString(),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        } catch (IOException exception_object) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        exception_object.toString()));
            }
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            socket_object_output_stream = new ObjectOutputStream(socket_output_stream);
        } catch (IOException exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                      ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CREATE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CREATE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket_object_output_stream.toString(),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
        }
        return socket_object_output_stream;
    }

    public RetryObjectInputStream getInputStream() throws Exception {
        InputStream  socket_input_stream;
        ObjectServerException object_server_exception;

        try {
            socket_input_stream = socket.getInputStream();
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CREATE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CREATE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket_input_stream.toString(),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
/*
            socket_object_input_stream = new RetryObjectInputStream(socket_input_stream);
*/
            socket_object_input_stream = RetryObjectInputStream.valueOf(
                socket_input_stream,
                getMessageResource(),
                getDebug(),
                getLogStream()
            );
        } catch (Exception exception_object) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        exception_object.toString()));
            }
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.IO_CONSTRUCTION_ERROR_LIBRARY,
                    getMessageResource().getMessage(
                        ObjectServerError.IO_CONSTRUCTION_ERROR_LIBRARY)));
            throw object_server_exception;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CREATE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CREATE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket_object_input_stream.toString(),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
        }
        return socket_object_input_stream;
    }

    public void shutdownSocket() throws IOException {
        IOException io_exception_object;

        if (socket == null) {
            return;
        }
        io_exception_object = null;
        if (socket.isConnected() == true) {
            try {
                socket.shutdownInput();
            } catch (IOException exception_object) {
                io_exception_object = exception_object;
            }
        }
        if (socket.isConnected() == true) {
            try {
                socket.shutdownOutput();
            } catch (IOException exception_object) {
                if (io_exception_object == null) {
                    io_exception_object = exception_object;
                }
            }
        }
        if (io_exception_object != null) {
            throw io_exception_object;
        }
    }

    public void shutdownInput() throws IOException, Exception {
        ObjectServerException   object_server_exception;

        try {
            socket.shutdownInput();
        } catch (IOException io_exception) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_SHUTDOWN_INPUT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_SHUTDOWN_INPUT_IO_EXCEPTION)));
            throw object_server_exception;
        }
    }

    public void shutdownOutput() throws IOException, Exception {
        ObjectServerException   object_server_exception;

        try {
            socket.shutdownOutput();
        } catch (IOException io_exception) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_SHUTDOWN_INPUT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_SHUTDOWN_INPUT_IO_EXCEPTION)));
            throw object_server_exception;
        }
    }

    public boolean isConnected() {
        return socket.isConnected();
    }

    public void close() throws IOException, Exception {
        ObjectServerException   object_server_exception;
        try {
            socket.close();
        } catch (IOException io_exception) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_CLOSE_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_CLOSE_IO_EXCEPTION)));
            throw object_server_exception;
        }
    }
}
