/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     DataClass.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/05/29
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

public class DataClass extends ObjectCommand {
    public static String CLASS_SUFFIX = ".class";
    public static String DATA_SOURCE = "Data.java";
    public static String DATA_KEY = "Data";
    public static String DOUBLE_QUOTE = "\"";

    public static String OPTION_KEY_STRING = "-";
    public static String PACKAGE_ARGS = "-p";
    public static String DATA_CLASS_ARGS = "p:";

    public static String SEMI_COLOMN = ";";
    public static String SPACE = " ";
    public static String BLANK_HEAD = "        ";

    public static String LINE_BANNER_GENERATION_MESSAGE =
        "Generation Date - ";
    public static String LINE_BANNER_HEADER_HEAD = " * ";
    public static String LINE_BANNER_HEADER1 = "/**";
    public static String LINE_BANNER_HEADER2 = " * Copyright (C) All rights reserved.";
    public static String LINE_BANNER_HEADER3 = " * This file is generated by DataClass.";
    public static String LINE_BANNER_HEADER4 = " */";
    public static String LINE_PACKAGE_KEYWORD_STRING = "package";
    public static String LINE_PACKAGE_STRING = "package oserv;";
    public static String LINE_IMPORT_STRING1 = "import java.lang.*;";
    public static String LINE_IMPORT_STRING2 = "import java.lang.reflect.*;";
    public static String LINE_IMPORT_STRING3 = "import java.io.*;";
    public static String LINE_IMPORT_STRING4 = "import java.net.*;";
    public static String LINE_IMPORT_STRING5 = "import java.util.*;";
    public static String LINE_IMPORT_STRING6 = "import java.util.zip.*;";
    public static String LINE_IMPORT_STRING7 = "import java.util.jar.*;";

    public static String LINE_PUBLIC_CLASS_HEAD = "public class ";
    public static String LINE_PUBLIC_CLASS_TAIL = "{";

    public static String LINE_CLASS_DATA_START = "    private static byte[] class_data = {";
    public static String LINE_CLASS_DATA_END = "    };";

    public static String LINE_GET_CLASS_DATA_1 = "    public static byte[] getClassData() {";
    public static String LINE_GET_CLASS_DATA_2 = "        return class_data;";
    public static String LINE_GET_CLASS_DATA_3 = "    }";
    public static String LINE_BLANK_1 = "";

    public static String LINE_MAIN_START = "    public static void main(String[] args) {";
    public static String LINE_MAIN_END = "    }";

    public static String LINE_PUBLIC_CLASS_END = "}";

    public static String[] line_pre_head = {
        LINE_BANNER_HEADER_HEAD,
        LINE_BANNER_HEADER3,
        LINE_BANNER_HEADER4,
        LINE_BLANK_1,
    };
    public static String[] line_head = {
        LINE_BLANK_1,
        LINE_IMPORT_STRING1,
        LINE_IMPORT_STRING2,
        LINE_IMPORT_STRING3,
        LINE_IMPORT_STRING4,
        LINE_IMPORT_STRING5,
        LINE_IMPORT_STRING6,
        LINE_IMPORT_STRING7,
        LINE_BLANK_1,
    };

    public static String[] line_mid = {
        LINE_PUBLIC_CLASS_TAIL,
        LINE_CLASS_DATA_START,
    };

    public static String[] line_tail = {
        LINE_CLASS_DATA_END,
        LINE_GET_CLASS_DATA_1,
        LINE_GET_CLASS_DATA_2,
        LINE_GET_CLASS_DATA_3,
        LINE_BLANK_1,
        LINE_MAIN_START,
        LINE_MAIN_END,
        LINE_PUBLIC_CLASS_END,
    };

    private static MessageResource message_resource = new MessageResource();
    private static int debug = 0;
    private static LogPrintStream logStream = null;

    private String package_path_string = null;

    public DataClass() {
        String value_string;

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }
    }

    public void setPackage(String package_string) {
        package_path_string = package_string;
    }

    public void writeDataClass(String base_name) throws Exception {
        int              rt;
        String           source_class;
        String           destination_java;
        byte[]           buffer;
        FileInputStream  file_input_stream;
        FileOutputStream file_output_stream;
        PrintStream      print_stream;
        File             source_des;
        File             destination_des;

        if (base_name == null) {
            Exception  exception_object;
            exception_object = new Exception(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_DATACLASS,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_DATACLASS)));
            throw exception_object;
        }
        source_class = base_name.concat(CLASS_SUFFIX);
        destination_java = base_name.concat(DATA_SOURCE);

        source_des = new File(source_class);

        destination_des = new File(destination_java);

        if (destination_des.exists() == true) {
            destination_des.delete();
        }
        try {
            file_output_stream = new FileOutputStream(destination_des);
            print_stream = new PrintStream(file_output_stream);
        } catch (Exception real_exception_object) {
            Exception  exception_object;
            exception_object = new Exception(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.IO_ERROR_DATACLASS,
                    getMessageResource().getMessage(
                        ObjectServerError.IO_ERROR_DATACLASS)));
            throw exception_object;
        }
        if (source_des.exists() == false) {
            buffer = new byte[0];
            rt = 0;
        } else {
            buffer = new byte[(int)source_des.length()];

            try {
                file_input_stream = new FileInputStream(source_des);
            } catch (Exception real_exception_object) {
                Exception  exception_object;
                exception_object = new Exception(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.IO_ERROR_DATACLASS,
                        getMessageResource().getMessage(
                            ObjectServerError.IO_ERROR_DATACLASS)));
                throw exception_object;
            }
            try {
                rt = file_input_stream.read(buffer, 0, buffer.length);
            } catch (Exception real_exception_object) {
                Exception  exception_object;
                exception_object = new Exception(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.IO_ERROR_DATACLASS,
                        getMessageResource().getMessage(
                            ObjectServerError.IO_ERROR_DATACLASS)));
                throw exception_object;
            }
        }
        print_stream.println( LINE_BANNER_HEADER1 );
        print_stream.println( LINE_BANNER_HEADER2 );
        print_stream.println( LINE_BANNER_HEADER_HEAD );
        Date dt;
        dt = new Date();

        print_stream.println( 
            LINE_BANNER_HEADER_HEAD + 
            LINE_BANNER_GENERATION_MESSAGE +
            dt.toLocaleString() );

        for (int i = 0; i < line_pre_head.length; i++) {
            print_stream.println(line_pre_head[i]);
        }
        if (package_path_string == null) {
            print_stream.println(LINE_PACKAGE_STRING);
        } else {
            print_stream.println(LINE_PACKAGE_KEYWORD_STRING
            + SPACE
            + package_path_string
            + SEMI_COLOMN);
        }
        for (int i = 0; i < line_head.length; i++) {
            print_stream.println(line_head[i]);
        }
        print_stream.print(LINE_PUBLIC_CLASS_HEAD + base_name + DATA_KEY);
        for (int i = 0; i < line_mid.length; i++) {
            print_stream.println(line_mid[i]);
        }
        for (int i = 0; i < buffer.length; i++) {
            if ((i%16) == 0) {
                print_stream.print(BLANK_HEAD);
            }
            print_stream.print( Byte.toString(buffer[i])
                + ObjectServerHeader.PARAMETER_SEPARATOR);

            if (((i+1) % 16) == 0) {
                print_stream.println();
            }
        }
        for (int i = 0; i < line_tail.length; i++) {
            print_stream.println(line_tail[i]);
        }
    }

    public static void main(String[] args) {
        ParseOptions    parse_options;
        String          arg_value;

        DataClass data_class_object;
        data_class_object = new DataClass();
        try {
            parse_options = ParseOptions.valueOf(
                args, DataClass.DATA_CLASS_ARGS,
                null, null,
                data_class_object.getMessageResource(),
                data_class_object.getDebug(),
                data_class_object.getLogStream() );
        } catch (Exception exception_object) {
            return;
        }
        if (parse_options.getRealArgumentCount() != 1) {
            return;
        }


        if (parse_options.checkOption(PACKAGE_ARGS) == true) {
            arg_value = parse_options.getOption(PACKAGE_ARGS);
            data_class_object.setPackage(arg_value);
        }
        
        try {
            data_class_object.writeDataClass(
                    parse_options.getRealArgument(0) );
        } catch (Exception exception_object) {
            return;
        }
    }
}

