/**
 * Copyright (C) All rights reserved.
 * 
 * @Module     JarArchiveObject.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/10/03 Add to Forward Feature for getting a file in jar.
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.util.jar.*;
import java.util.zip.*;

public class JarArchiveObject extends ObjectServerHeader {
    private String     filename;
    private byte[]     file_byte = new byte[0];
    private String     NOT_READ_JAR_ARCHIVE = "Not call readJarArchive().";
    private String     NOT_FOUND_ENTRY = "Not found entry.";
    private static int        BUFFER_SIZE = 4096;
    private int        buffer_size;

    private String DEBUG_GET_FILE_CONTENTS = "getFileContents()";
    private String DEBUG_READ_JAR_ARCHIVE_FILE = "readJarArchiveFile()";

    private String TRACE_CLASSNAME = "oserv.JarArchiveObject";
    private String TRACE_GETFILECONTENTS = "getFileContents";
    private String TRACE_READJARARCHIVEFILE = "readJarArchiveFile";

    public void setDebugLogStream(int level, LogPrintStream ps) {
        if (level > getDebug()) {
            setDebug(level);
        }
        if (ps != null) { 
            setLogStream(ps);
        }
    }

    private JarArchiveObject() {
    }

    public JarArchiveObject(String file) throws Exception {
        this.setJarArchive(file);
    }

    public  void setJarArchive(String file) throws Exception {
        String     vstring;
        int        buffer_size;

        filename = file;
        if (file == null) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerException.PARAMETER_ERROR_EXCEPTION)));
            throw object_server_exception;
        }
        vstring  = getPropertyValue(OSERV_BUFFER_MAXIMUM_KEY);
        if (vstring != null) {
            try {
                buffer_size = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                buffer_size = BUFFER_SIZE;
            }
        } else {
            buffer_size = BUFFER_SIZE;
        }
    }

    public static JarArchiveObject valueOf(
        String file, 
        MessageResource parameter_message_resource,
        int   parameter_debug,
        LogPrintStream parameter_log_print_stream) throws Exception {

        JarArchiveObject    jar_archive_object;
        jar_archive_object = new JarArchiveObject();
        
        jar_archive_object.setDebugLogStream(
            parameter_debug, parameter_log_print_stream);
        jar_archive_object.setMessageResource(parameter_message_resource);

        try {
            jar_archive_object.setJarArchive(file);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return jar_archive_object;
    }

    public void getFileContents(String file_entry,
            ObjectOutputStream  object_output_stream
            ) throws Exception {

        FileInputStream      file_input_stream;
        JarInputStream       jar_input_stream;
        Byte[]               byte_object_array;
        Byte                 byte_object;
        ZipEntry             zipentry;
        byte[]               buffer;
        int                  rcnt;
        long                 request_count;
        Long                 file_length;
        ObjectServerException    object_server_exception;
        VariableByte variable_byte;

        if (filename == null) {
            object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.NOT_FOUND_JAR_ARCHIVE_OBJECT,
                        getMessageResource().getMessage(
                            ObjectServerError.NOT_FOUND_JAR_ARCHIVE_OBJECT)));
            throw object_server_exception;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE) {
            if (getLogStream() != null && getMessageResource() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getMessage(
                            DEBUG_GET_FILE_CONTENTS),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }
        file_input_stream = new FileInputStream(filename);
        jar_input_stream = new JarInputStream(file_input_stream);
        
        try {
            zipentry = jar_input_stream.getNextEntry();
        } catch (Exception real_exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETFILECONTENTS, 0, real_exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_JAR_INPUT_STREAM_GET_NEXT_ENTRY_IO_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerError.JAVA_JAR_INPUT_STREAM_GET_NEXT_ENTRY_IO_EXCEPTION)));
            throw object_server_exception;
        }
        while (zipentry != null) {
            if (zipentry.getName().compareTo( file_entry ) == 0) {
                if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (getLogStream() != null && getMessageResource() != null) {
                        getLogStream().printLogln(
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_FILE_NAME,
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_FILE_NAME),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                file_entry),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_JAR_NAME,
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_JAR_NAME),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                zipentry.getName()));
                    }
                }
                buffer = new byte[BUFFER_SIZE];
                while (true) {
                 
                    try {
                        rcnt = jar_input_stream.read(buffer, 0, 
                            (int)buffer.length);
                    } catch (Exception real_exception_object) {
                        printExceptionMessage(TRACE_CLASSNAME, TRACE_GETFILECONTENTS, 0, real_exception_object);
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION)));
                        throw object_server_exception;
                    }
                    if (rcnt == -1) {
                        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                            if (getLogStream() != null && getMessageResource() != null) {
                                getLogStream().printLogln(
                                    getMessageResource().getFormattedMessage(
                                        ObjectServerDebug.DEBUG_END_KEY +
                                        ObjectServerDebug.DEBUG_READJARFILE,
                                        getMessageResource().getMessage(
                                            ObjectServerDebug.DEBUG_READJARFILE
                                        ),
                                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                                        getMessageResource().getMessage(
                                            ObjectServerDebug.DEBUG_END_KEY
                                        )
                                    ));
                            }
                        }
                        break;
                    }
                    try {
                        variable_byte = VariableByte.valueOf(buffer, rcnt,
                            getMessageResource());
                    } catch (Exception real_exception_object) {
                        printExceptionMessage(TRACE_CLASSNAME, TRACE_GETFILECONTENTS, 0, real_exception_object);
                        throw real_exception_object;
                    }
                    try {
                        object_output_stream.writeObject(variable_byte);
                    } catch (Exception real_exception_object) {
                        printExceptionMessage(TRACE_CLASSNAME, TRACE_GETFILECONTENTS, 0, real_exception_object);
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
                        throw object_server_exception;
                    }
                }
                return;
            }
            zipentry = jar_input_stream.getNextEntry();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerError.NOT_FOUND_JAR_ARCHIVE_OBJECT,
                getMessageResource().getMessage(
                    ObjectServerError.NOT_FOUND_JAR_ARCHIVE_OBJECT)));
        throw object_server_exception;
    }

    public void readJarArchiveFile(ObjectOutputStream object_output_stream) throws Exception {
        int        rcnt;
        long       request_count;
        long       size;
        Long       file_size;
        File       file_descriptor;
        Byte              byte_object;
        FileInputStream   file_input_stream;
        VariableByte      variable_byte;
        ObjectServerException  object_server_exception;

        file_descriptor = new File(filename);

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE) {
            if (getLogStream() != null && getMessageResource() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getMessage(
                            DEBUG_READ_JAR_ARCHIVE_FILE),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }

        size = file_descriptor.length();
        file_size = Long.valueOf(size);
        try {
            file_input_stream = new FileInputStream(file_descriptor);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_READJARARCHIVEFILE, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        request_count = size;
        while (true) {
            if (request_count > (long)BUFFER_SIZE) {
                file_byte = new byte[BUFFER_SIZE];
            } else {
                file_byte = new byte[(int)request_count];
            }
            try {
                rcnt = file_input_stream.read(file_byte, 0, 
                    (int)file_byte.length);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_READJARARCHIVEFILE, 0, exception_object);
                if ((exception_object instanceof java.io.EOFException) == true) {
                    break;
                }
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_FILE_INPUT_STREAM_READ_IO_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_FILE_INPUT_STREAM_READ_IO_EXCEPTION)));
                throw object_server_exception;
            }
            if (rcnt == -1) {
                break;
            }
            try {
                variable_byte = VariableByte.valueOf(file_byte, rcnt,
                            getMessageResource());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_READJARARCHIVEFILE, 0, exception_object);
                throw exception_object;
            }
            try {
                object_output_stream.writeObject(variable_byte);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_READJARARCHIVEFILE, 0, exception_object);
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
                throw object_server_exception;
            }
            if (request_count <= 0) {
                break;
            }
        }
    }
}
