/**
 * Copyright (C) All rights reserved.
 *
 * @Module     MakeDirectories.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/10/18
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;

public class MakeDirectories extends SerializableObject {

    private String path = null;
    private String exist_path = null;

    public static String PARAMETER_ERROR_EXCEPTION = "Parameter error";
    public static String EMPTY_STRING = "";
    public static String END_STRING = "End";
    public static String CREATE_STRING = "Create";

    public static String MAKE_DIRECTORIES_ARGS = "";

    private MakeDirectories() {
    }


    public MakeDirectories(String directory_path) throws Exception {
        setPath(directory_path);
    }

    private MakeDirectories setPath(String directory_path) throws Exception {
        int    idx;
        String dir_path;
        String target;
        StringTokenizer stk;
        File   target_descriptor;
        ObjectServerException    object_server_exception;

        if (directory_path == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_MAKEDIRECTORIES,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_MAKEDIRECTORIES)));

            throw object_server_exception;
        }
        path = directory_path;

        exist_path = new String(EMPTY_STRING);
        target = exist_path;
        stk = new StringTokenizer(path, File.separator);
        while (stk.hasMoreTokens()) {
            if (target.length() == 0) {
                target = stk.nextToken();
            } else {
                target = target + File.separator + stk.nextToken();
            }
            target_descriptor = new File(target);
            if (target_descriptor.exists() == false) {
                break;
            } 
            exist_path = target;
        }
        return this;
    }

    /**
     * make directory
     */
    public boolean mkdirs() {
        boolean   make_flag;
        File      file_descriptor;

        file_descriptor = new File(path);
        make_flag = file_descriptor.mkdirs();
        return make_flag;
    }

    public boolean rmdirs() {
        int        idx;
        boolean    rmdir_flag;
        File       target_descriptor;
        String     target;
        String     token_path;
        StringTokenizer stk;

        target = path;
        while (true) {
            if (target.compareTo(exist_path) == 0) {
                break;
            }
            target_descriptor = new File(target);
            if (target_descriptor.exists() == true) {
                target_descriptor.delete();
            }
            idx = target.lastIndexOf(File.separator);
            if (idx <= 0) {
                break;
            }
            target = target.substring(0, idx);
        }
        return true;
    }

    public static MakeDirectories valueOf(
        String path, 
        MessageResource parameter_message_resource,
        int parameter_debug,
        LogPrintStream parameter_log_print_stream  ) throws Exception {
        MakeDirectories   make_directories;

        make_directories = new MakeDirectories( );
        make_directories.setMessageResource(parameter_message_resource);
        make_directories.setDebugLogStream(parameter_debug,
                parameter_log_print_stream );
        try {
            make_directories.setPath(path);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return make_directories;
    }

    /**
     * This is used for Test.
     */
    public static void main(String[] args) {
        String            directory;
        ParseOptions      parse_options;
        MakeDirectories   directory_object;
        File              file_descriptor;

        try {
            parse_options = new ParseOptions(args, MAKE_DIRECTORIES_ARGS);
        } catch (Exception exception_object) {
            System.err.println(PARAMETER_ERROR_EXCEPTION);
            return;
        }
        for (int i = 0; i < parse_options.getRealArgumentCount(); i++) {
            try {
                directory = parse_options.getRealArgument(i); 
            } catch (Exception exception_object) {
                continue;
            }
            try {
                directory_object = new MakeDirectories(directory);
            } catch (Exception exception_object) {
                return;
            }

            directory_object.mkdirs();

            file_descriptor = new File(directory);
            if (file_descriptor.exists() == true) {
                System.out.println(CREATE_STRING + directory);
            }
            directory_object.rmdirs();
            System.out.println(END_STRING);
        }
    }
}
