/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedBufferedOutputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedBufferedOutputStream extends BufferedOutputStream implements NamedOutputStream {

    private static NamedFileOutputStream named_file_output_stream;
    private static boolean close_flag = false;
    private static String FILE_NAME = "File name";
    private static String SPACE = " ";

    public final static String LINE1_HEAD = "Line 1";
    public final static String LINE2_HEAD = "Line 2";
    public final static String LINE3_HEAD = "Line 3";
    public final static String LINE1 = "Message 1\n";
    public final static String LINE2 = "Message 2\n";
    public final static String LINE3 = "Message 3\n";

    public final static String PARAMETER_ERROR = "Parameter error";

    public NamedBufferedOutputStream(NamedFileOutputStream param_file_output_stream) throws IOException {
        super(param_file_output_stream);
        named_file_output_stream = param_file_output_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_output_stream != null) {
            filename = named_file_output_stream.getFileName();
        }
        return filename;
    }

    public void closeStream() throws Exception {
        try {
            super.flush();
            super.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void close() throws IOException {
        if (close_flag == true) {
            return;
        }
        try {
            super.flush();
            super.close();
        } catch (IOException io_exception) {
            throw io_exception;
        }
        close_flag = true;
    }

    public static void main(String[] args) {
        int                        rt;
        String                     line;
        byte[]                     buffer;
        DataInputStream            data_input_stream;
        NamedFileInputStream       named_file_input_stream;
        NamedFileOutputStream      named_file_output_stream;
        NamedBufferedOutputStream  named_buffered_output_stream;
        PrintStream                print_stream;

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }

        try {
            named_file_output_stream = new NamedFileOutputStream(args[0]); 
            named_buffered_output_stream = new NamedBufferedOutputStream(
                named_file_output_stream );
            System.out.println(FILE_NAME + SPACE +
                named_buffered_output_stream.getFileName());
            print_stream = new PrintStream(named_file_output_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            print_stream.print(LINE1);
            print_stream.print(LINE2);
            print_stream.print(LINE3);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            print_stream.close();
            named_file_input_stream = new NamedFileInputStream(args[0]);
            data_input_stream = new DataInputStream(named_file_input_stream);
            line = data_input_stream.readLine();
            System.out.println(LINE1_HEAD + SPACE + line);
            line = data_input_stream.readLine();
            System.out.println(LINE2_HEAD + SPACE + line);
            line = data_input_stream.readLine();
            System.out.println(LINE3_HEAD + SPACE + line);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
