/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedDataOutputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedDataOutputStream extends DataOutputStream implements NamedOutputStream {

    private static NamedFileOutputStream named_file_output_stream;
    private static boolean close_flag = false;

    public final static String PARAMETER_ERROR = "Parameter error";

    public final static String ILLEGAL_ARGUMENT = "illegal argument";

    public final static String FILE_NAME_HEAD = "File name";
    public final static String DATA_HEAD = "Boolean data";
    public final static String SPACE = " ";

    public NamedDataOutputStream(NamedFileOutputStream param_file_output_stream) throws IOException {
        super(param_file_output_stream);
        named_file_output_stream = param_file_output_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_output_stream != null) {
            filename = named_file_output_stream.getFileName();
        }
        return filename;
    }

    public void closeStream() throws Exception {
        try {
            super.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void close() throws IOException {
        if (close_flag == true) {
            return;
        }
        try {
            super.close();
        } catch (IOException io_exception) {
            throw io_exception;
        }
        close_flag = true;
    }

    public static void main(String[] args) {
        boolean                boolean_object;
        boolean                boolean_result;
        String                 file_name;
        DataInputStream        data_input_stream;
        FileInputStream        file_input_stream;
        NamedFileOutputStream  named_file_output_stream;
        NamedDataOutputStream  named_data_output_stream;

        boolean_object = true;

        if (args.length != 1) {
            System.out.println(ILLEGAL_ARGUMENT);
            return;
        }
        try {
            named_file_output_stream = new NamedFileOutputStream(args[0]);
            named_data_output_stream = new NamedDataOutputStream(named_file_output_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        file_name = named_data_output_stream.getFileName();
        System.out.println(FILE_NAME_HEAD + SPACE + file_name);

        try {
            named_data_output_stream.writeBoolean(boolean_object);
            named_data_output_stream.close();
            file_input_stream = new FileInputStream(args[0]);
            data_input_stream = new DataInputStream(file_input_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }


        try {
            boolean_result = data_input_stream.readBoolean();
            System.out.println(DATA_HEAD + SPACE + String.valueOf(boolean_result));
            data_input_stream.close();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
