/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedFilterInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedFilterInputStream extends FilterInputStream implements NamedInputStream {

    private static NamedFileInputStream named_file_input_stream;
    public final static String SPACE = " ";
    private static String FILE_NAME = "File name";

    public final static String LINE1_HEAD = "Line 1";
    public final static String LINE2_HEAD = "Line 2";
    public final static String LINE3_HEAD = "Line 3";
    public final static String LINE1 = "Message 1";
    public final static String LINE2 = "Message 2";
    public final static String LINE3 = "Message 3";

    public final static String PARAMETER_ERROR = "Parameter error";

    public NamedFilterInputStream(NamedFileInputStream param_file_input_stream) throws IOException {
        super(param_file_input_stream);
        named_file_input_stream = param_file_input_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_input_stream != null) {
            filename = named_file_input_stream.getFileName();
        }
        return filename;
    }

    public void close() throws IOException {
        try {
            super.close();
        } catch (IOException io_exception) {
            throw io_exception;
        }
    }

    public static void main(String[] args) {
        String                  line;
        byte[]                  buffer;
        NamedFileInputStream    named_file_input_stream;
        NamedFilterInputStream  named_filter_input_stream;
        File                    file_descriptor;
        FileOutputStream        file_output_stream;
        PrintStream             print_stream;

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }

        try {
            file_output_stream = new FileOutputStream(args[0]);
            print_stream = new PrintStream(file_output_stream);
            print_stream.print(LINE1);
            print_stream.print(LINE2);
            print_stream.print(LINE3);
            print_stream.close();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            named_file_input_stream = new NamedFileInputStream(args[0]); 
            named_filter_input_stream = new NamedFilterInputStream(
                named_file_input_stream );
            System.out.println(FILE_NAME + SPACE +
                named_filter_input_stream.getFileName());
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            buffer = new byte[LINE1.length()];
            named_filter_input_stream.read(buffer, 0, buffer.length);
            line = new String(buffer);
            System.out.println(LINE1_HEAD + line);

            buffer = new byte[LINE2.length()];
            named_filter_input_stream.read(buffer, 0, buffer.length);
            line = new String(buffer);
            System.out.println(LINE2_HEAD + line);

            buffer = new byte[LINE3.length()];
            named_filter_input_stream.read(buffer, 0, buffer.length);
            line = new String(buffer);
            System.out.println(LINE3_HEAD + line);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
