/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedLineNumberInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedLineNumberInputStream extends LineNumberInputStream implements NamedInputStream {

    private static NamedFileInputStream named_file_input_stream;

    public final static String PARAMETER_ERROR = "Parameter error";

    public final static String LINE1 = "Line 1 message\n";
    public final static String LINE2 = "Line 2 message\n";
    public final static String LINE3 = "Line 3 message\n";

    public final static String LINE_ERROR1 = "Message 1 error";
    public final static String LINE_ERROR2 = "Message 2 error";
    public final static String LINE_ERROR3 = "Message 3 error";

    public final static String LINE1_HEAD = "Line1 number";
    public final static String LINE2_HEAD = "Line2 number";
    public final static String LINE3_HEAD = "Line3 number";
    public final static String SPACE = " ";

    public NamedLineNumberInputStream(NamedFileInputStream param_file_input_stream) throws IOException {
        super(param_file_input_stream);
        named_file_input_stream = param_file_input_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_input_stream != null) {
            filename = named_file_input_stream.getFileName();
        }
        return filename;
    }

    public void close() throws IOException {
        try {
            super.close();
        } catch (IOException io_exception) {
            throw io_exception;
        }
    }

    public static void main(String[] args) {
        String                      line;
        byte[]                      buffer;
        int                         rt;
        File                        file_descriptor;
        FileOutputStream            file_output_stream;
        PrintStream                 print_stream;
        NamedFileInputStream        named_file_input_stream;
        NamedLineNumberInputStream  named_line_number_input_stream;

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }

        try {
            file_output_stream = new FileOutputStream(args[0]);
            print_stream = new PrintStream(file_output_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        print_stream.print(LINE1);
        print_stream.print(LINE2);
        print_stream.print(LINE3);
        try {
            print_stream.close();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            named_file_input_stream = new NamedFileInputStream(args[0]);
            named_line_number_input_stream
                = new NamedLineNumberInputStream(named_file_input_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            buffer = new byte[LINE1.length()];
            rt = named_line_number_input_stream.read(buffer, 0, 
                buffer.length);
            if (rt != LINE1.length()) {
                System.out.println(LINE_ERROR1);
                return;
            }
            System.out.println(LINE1_HEAD + SPACE + 
                String.valueOf(named_line_number_input_stream.getLineNumber())+
                SPACE +
                new String(buffer, 0, buffer.length-1) );
            buffer = new byte[LINE2.length()];
            rt = named_line_number_input_stream.read(buffer, 0, 
                buffer.length);
            if (rt != LINE2.length()) {
                System.out.println(LINE_ERROR2);
                return;
            }
            System.out.println(LINE2_HEAD + SPACE +
                String.valueOf(named_line_number_input_stream.getLineNumber())+
                SPACE +
                new String(buffer, 0, buffer.length-1) );
            buffer = new byte[LINE3.length()];
            rt = named_line_number_input_stream.read(buffer, 0, 
                buffer.length);
            if (rt != LINE3.length()) {
                System.out.println(LINE_ERROR3);
                return;
            }
            System.out.println(LINE3_HEAD + SPACE + 
                String.valueOf(named_line_number_input_stream.getLineNumber()) +
                SPACE +
                new String(buffer, 0, buffer.length-1) );

        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
