/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedObjectInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedObjectInputStream extends ObjectInputStream implements NamedInputStream {

    private static NamedFileInputStream  named_file_input_stream = null;

    public final static String LINE1_HEAD = "Line 1";
    public final static String LINE2_HEAD = "Line 2";
    public final static String LINE3_HEAD = "Line 3";

    public final static String LINE1 = "Message 1";
    public final static String LINE2 = "Message 2";
    public final static String LINE3 = "Message 3";

    public final static String LINE1_ERROR = "Line 1 Error";
    public final static String LINE2_ERROR = "Line 2 Error";
    public final static String LINE3_ERROR = "Line 3 Error";

    public final static String SPACE = " ";
    public final static String PARAMETER_ERROR = "Parameter error";

    public final static String FILE_NAME = "File Name";

    public NamedObjectInputStream(NamedFileInputStream param_file_input_stream) throws IOException {
        super((InputStream)param_file_input_stream);
        named_file_input_stream = param_file_input_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_input_stream != null) {
            filename = named_file_input_stream.getFileName();
        }
        return filename;
    }

    public void close() throws IOException {
        try {
            super.close();
        } catch (IOException io_exception) {
            throw io_exception;
        }
    }

    public static void main(String[] args) {
        String   line;
        FileOutputStream       file_output_stream;
        ObjectOutputStream     object_output_stream;
        NamedFileInputStream   named_file_input_stream;
        NamedObjectInputStream named_object_input_stream;

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }

        try {
            file_output_stream = new FileOutputStream(args[0]);
            object_output_stream = new ObjectOutputStream(file_output_stream);
            object_output_stream.writeObject(LINE1);
            object_output_stream.writeObject(LINE2);
            object_output_stream.writeObject(LINE3);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            object_output_stream.close();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            named_file_input_stream = new NamedFileInputStream(args[0]);
            named_object_input_stream = new NamedObjectInputStream(
                named_file_input_stream);
            System.out.println(FILE_NAME + SPACE +
                named_object_input_stream.getFileName());
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            line = (String)named_object_input_stream.readObject();
            if (line == null) {
                System.out.println(LINE1_ERROR);
                return;
            }
            System.out.println(LINE1_HEAD + SPACE + line);
            line = (String)named_object_input_stream.readObject();
            if (line == null) {
                System.out.println(LINE2_ERROR);
                return;
            }
            System.out.println(LINE2_HEAD + SPACE + line);
            line = (String)named_object_input_stream.readObject();
            if (line == null) {
                System.out.println(LINE3_ERROR);
                return;
            }
            System.out.println(LINE3_HEAD + SPACE + line);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
    }
}
