/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedObjectOutputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedObjectOutputStream extends ObjectOutputStream implements NamedOutputStream {

    private static NamedFileOutputStream named_file_output_stream;
    private static boolean close_flag = false;
    private static String FILE_NAME = "File name";

    public final static String SPACE = " ";

    public final static String LINE1_HEAD = "Line 1";
    public final static String LINE2_HEAD = "Line 2";
    public final static String LINE3_HEAD = "Line 3";
    public final static String LINE1 = "Message 1";
    public final static String LINE2 = "Message 2";
    public final static String LINE3 = "Message 3";

    public final static String PARAMETER_ERROR = "Parameter error";

    public NamedObjectOutputStream(NamedFileOutputStream param_file_output_stream) throws IOException {
        super(param_file_output_stream);
        named_file_output_stream = param_file_output_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_output_stream != null) {
            filename = named_file_output_stream.getFileName();
        }
        return filename;
    }

    public void closeStream() throws Exception {
        try {
            super.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void close() throws IOException {
        if (close_flag == true) {
            return;
        }
        try {
            if (named_file_output_stream != null) {
                named_file_output_stream.close();
            }
        } catch (IOException io_exception) {
            throw io_exception;
        }
        close_flag = true;
    }

    public static void main(String[] args) {
        String                      line;
        String                      message1;
        String                      message2;
        String                      message3;
        FileInputStream             file_input_stream;
        ObjectInputStream           object_input_stream;
        NamedFileOutputStream       named_file_output_stream;
        NamedObjectOutputStream     named_object_output_stream; 

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }
        try {
            named_file_output_stream = new NamedFileOutputStream(args[0]);
            named_object_output_stream = new NamedObjectOutputStream(
                named_file_output_stream);
            System.out.println(FILE_NAME + SPACE +
                named_object_output_stream.getFileName());
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            named_object_output_stream.writeObject((Object)LINE1);
            named_object_output_stream.writeObject((Object)LINE2);
            named_object_output_stream.writeObject((Object)LINE3);
            named_object_output_stream.close();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            file_input_stream = new FileInputStream(args[0]);
            object_input_stream = new ObjectInputStream(file_input_stream);

            message1 = (String)object_input_stream.readObject();
            System.out.println(LINE1_HEAD + SPACE + message1);
            message2 = (String)object_input_stream.readObject();
            System.out.println(LINE2_HEAD + SPACE + message2);
            message3 = (String)object_input_stream.readObject();
            System.out.println(LINE3_HEAD + SPACE + message3);

        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
     
    }
}
