/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedPrintStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/05/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class NamedPrintStream extends PrintStream implements NamedOutputStream {

    private static NamedFileOutputStream named_file_output_stream;
    private static boolean close_flag = false;

    public final static String PARAMETER_ERROR = "Parameter error";

    public final static String SPACE = " ";
    private final static String FILE_NAME = "File name";

    public final static String LINE1 = "Message 1";
    public final static String LINE2 = "Message 2";
    public final static String LINE3 = "Message 3";

    public final static String LINE1_HEADER = "Head 1";
    public final static String LINE2_HEADER = "Head 2";
    public final static String LINE3_HEADER = "Head 3";

    public NamedPrintStream(NamedFileOutputStream param_file_output_stream) throws IOException {
        super(param_file_output_stream);
        named_file_output_stream = param_file_output_stream;
    }

    public String getFileName() {
        String filename = null;
        if (named_file_output_stream != null) {
            filename = named_file_output_stream.getFileName();
        }
        return filename;
    }

    public void closeStream() {
        if (close_flag == true) {
            return;
        }
        try {
            if (named_file_output_stream != null) {
                named_file_output_stream.close();
            }
        } catch (IOException io_exception) {
        }
        close_flag = true;
    }

    public static void main(String[] args) {
        String                      line;
        FileInputStream             file_input_stream;
        DataInputStream             data_input_stream;
        NamedPrintStream            named_print_stream;
        NamedFileOutputStream       named_file_output_stream;

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }
        try {
            named_file_output_stream = new NamedFileOutputStream(args[0]);
            named_print_stream = new NamedPrintStream(
                named_file_output_stream);
            System.out.println(FILE_NAME + SPACE +
                named_print_stream.getFileName());
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            named_print_stream.println(LINE1);
            named_print_stream.println(LINE2);
            named_print_stream.println(LINE3);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            file_input_stream = new FileInputStream(args[0]);
            data_input_stream = new DataInputStream(file_input_stream);
            line = data_input_stream.readLine();
            System.out.println(LINE1_HEADER + line);
            line = data_input_stream.readLine();
            System.out.println(LINE2_HEADER + line);
            line = data_input_stream.readLine();
            System.out.println(LINE3_HEADER + line);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
