/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NamedSequenceInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/06/05
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class NamedSequenceInputStream extends SequenceInputStream implements NamedInputStream {

    private static NamedFileInputStream[] named_file_input_stream_array;
    private static NamedFileInputStream named_file_input_stream = null;
/*
    private static java.util.Enumeration named_file_input_stream_array;
*/

    public final static String PARAMETER_ERROR = "Parameter error";

    public NamedSequenceInputStream(
        InputStream param_file_input_stream1,
        InputStream param_file_input_stream2) throws IOException {
        super(param_file_input_stream1, param_file_input_stream2);

        if ((param_file_input_stream1 instanceof NamedFileInputStream) == true) {
            named_file_input_stream = (NamedFileInputStream)param_file_input_stream1;
        }
    }

    public NamedSequenceInputStream(
        Enumeration param_file_input_stream) throws IOException {
        super(param_file_input_stream);

        NamedFileInputStream[]   bk;

        while (param_file_input_stream.hasMoreElements()) {
            bk = new NamedFileInputStream[
                named_file_input_stream_array.length+1];
            bk[bk.length-1] = (NamedFileInputStream)param_file_input_stream.nextElement();
            named_file_input_stream_array = bk;
        }
    }

    public String getFileName() {
        String filename = null;
        if (named_file_input_stream != null) {
            filename = named_file_input_stream.getFileName();
        }
        return filename;
    }

    public String[] getFileNames() {
        String[] filenames;

        filenames = new String[named_file_input_stream_array.length];
        for (int i = 0; i < named_file_input_stream_array.length; i++) {
            filenames[i] = named_file_input_stream_array[i].getFileName();
        }
        return filenames;
    }

    public void close() throws IOException {
/*
        for (int i = 0; i < named_file_input_stream_array.length; i++) {
            try {
                named_file_input_stream_array[i].close();
            } catch (IOException io_exception) {
                throw io_exception;
            }
        }
*/
        try {
            super.close();
        } catch (IOException io_exception) {
            throw io_exception;
        }
    }

    private static int BUFFER_SIZE = 4096;

    public static void main(String[] args) {
        int                   rt;
        String                line;
        byte[]                buffer;

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }
        NamedFileInputStream           named_file_input_stream;
        NamedFileInputStream           named_file_input_stream1;
        NamedFileInputStream           named_file_input_stream2;
        NamedSequenceInputStream       named_sequence_input_stream;
        ByteArrayInputStream           byte_array_input_stream;
        DataInputStream                data_input_stream;

        try {
            named_file_input_stream = new NamedFileInputStream(args[0]);
            byte_array_input_stream = new ByteArrayInputStream(
                ObjectServerHeader.EMPTY_MESSAGE.getBytes()
            );
            named_sequence_input_stream = new NamedSequenceInputStream(
                named_file_input_stream, 
                byte_array_input_stream );
            data_input_stream = new DataInputStream(named_sequence_input_stream);
        } catch (Exception exception_object) {
exception_object.printStackTrace();
            return;
        }
        while (true) {
            try {
                line = data_input_stream.readLine();
            } catch (Exception exception_object) {
                break;
            }
            if (line == null) { break; }
            System.out.println(line);
        }
    }
}
