/**
 * Copyright (C) All rights reserved.
 *
 * @Module     NetworkObject.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/12/27
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.jar.*;
import java.util.zip.*;
import java.security.*;

public class NetworkObject extends SerializableObject {

    public Integer int_value;
    public static String INT_FIELD_NAME = "int_value";
    public Long    long_value;
    public static String LONG_FIELD_NAME = "long_value";
    public Float   float_value;
    public static String FLOAT_FIELD_NAME = "float_value";
    public Double  double_value;
    public static String DOUBLE_FIELD_NAME = "double_value";
    public Short   short_value;
    public static String SHORT_FIELD_NAME = "short_value";
    public Character    char_value;
    public static String CHAR_FIELD_NAME = "char_value";
    public Byte    byte_value;
    public static String BYTE_FIELD_NAME = "byte_value";
    public Boolean boolean_value;
    public static String BOOLEAN_FIELD_NAME = "boolean_value";
    private Exception exception_value;

    public Object  object_value = null;
    public static String OBJECT_FIELD_NAME = "object_value";
    public static String VOID_TYPE = "void";
    public String  value_type_string = new String(ObjectServerHeader.EMPTY_MESSAGE);

/*
    private static MessageResource message_resource = new MessageResource();

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }
*/

    public String getName() {
        return value_type_string;
    }

    public void setName(String name) {
        value_type_string = name;
    }

    public void setVoidType() {
        value_type_string = VOID_TYPE;
    }

    public void setInt(int value) {
        value_type_string = int.class.getName();
        int_value = Integer.valueOf(value);
    }

    public int getInt() throws Exception {
        ObjectServerException  object_server_exception;
        Integer  integer_value;

        if (value_type_string == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
            throw object_server_exception;
        }
        if (value_type_string.compareTo(int.class.getName()) != 0) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
            throw object_server_exception;
        }
        if (int_value == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
            throw object_server_exception;
        }
        return int_value.intValue();
    }

    public void setLong(long value) {
        value_type_string = long.class.getName();
        long_value = Long.valueOf(value);
    }

    public long getLong() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(long.class.getName()) == 0 && long_value != null) {
            return long_value.longValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void setFloat(float value) {
        value_type_string = float.class.getName();
        float_value = Float.valueOf(value);
    }

    public float getFloat() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(float.class.getName()) == 0 && float_value != null) {
            return float_value.floatValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void setDouble(double value) {
        value_type_string = double.class.getName();
        double_value = Double.valueOf(value);
    }

    public double getDouble() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(double.class.getName()) == 0 && double_value != null) {
            return double_value.doubleValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void setByte(byte value) {
        value_type_string = byte.class.getName();
        byte_value = Byte.valueOf(value);
    }

    public byte getByte() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(byte.class.getName()) == 0 && byte_value != null) {
            return byte_value.byteValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void setBoolean(boolean value) {
        value_type_string = boolean.class.getName();
        boolean_value = Boolean.valueOf(value);
    }

    public boolean getBoolean() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(boolean.class.getName()) == 0 && boolean_value != null) {
            return boolean_value.booleanValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void setShort(short value) {
        value_type_string = short.class.getName();
        short_value = Short.valueOf(value);
    }

    public short getShort() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(short.class.getName()) == 0 && short_value != null) {
            return short_value.shortValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void setChar(char value) {
        value_type_string = char.class.getName();
        char_value = Character.valueOf(value);
    }

    public char getChar() throws Exception {
        ObjectServerException  object_server_exception;

        if (value_type_string != null && value_type_string.compareTo(char.class.getName()) == 0 && char_value != null) {
            return char_value.charValue();
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }

    public void set(Object value) {
        if (value != null) {
            value_type_string = value.getClass().getName();
        }
        object_value = value;
    }

    public Object get() throws Exception {
        ObjectServerException  object_server_exception;

        if (object_value != null) {
            return object_value;
        }
        if (value_type_string != null &&
            value_type_string.compareTo(VOID_TYPE) == 0) {
            return null;
        }
        if (value_type_string != null &&
            value_type_string.compareTo(ObjectServerHeader.EMPTY_MESSAGE) != 0) {
            return null;
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TYPE_NOT_FOUND_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TYPE_NOT_FOUND_EXCEPTION)));
        throw object_server_exception;
    }
}

