/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectClient.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/07/23
 *             2.1 2010/09/21 Define Debug message.
 *             2.1 2010/10/08
 *             2.1 2010/11/04 add exit procedure for property error
 *             2.1 2012/05/02 add Transient feature.
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

import oserv.*;

/**
 * This class is stub for user program.
 *
 * @see       oserv.ObjectServerHeader
 */
public class ObjectClient extends Object {
    public final static String __ARRAY_BYTE_CLASS_NAME = "[B";
    public final static int __COMMAND_COUNT = 6;
    public final static String __OBJECT_OUTPUT_STREAM_BOUT = "bout";
    public final static String __JAVA_CLASS_PATH = "java.class.path";
    public final static String __DEBUG_LIBRARIES = "libraries";
    public final static String __PARAMETER_ERROR  = "Parameter error";
    public final static String __CONNECTION_ERROR = "Connection error";

    public final static int __ERROR_EXIT_CODE = -1;

    public final static String[] __pathlist = {
	__JAVA_CLASS_PATH };
    public static int __RETRY_NUMBER = 3;
    private int __retry_count = __RETRY_NUMBER;

    public final static String __DEBUG_DEBUG_LEVEL = "Debug level ";
    public final static String __DEBUG_ERROR = "Error ";
    public final static String __DEBUG_INTERNAL_ERROR = "Internal Error ";
    public final static String __DEBUG_BLANK = " ";
    public final static String __DEBUG_TOKEN_COUNT = "Token count ";
    public final static String __DEBUG_PARAMETER_HEADER = "Parameter Header ";
    public final static String __DEBUG_SOCKET_EXCEPTION = "Socket exception ";
    public final static String __DEBUG_INETSOCKETADDRESS_EXCEPTION
            = "InetSocketAddress exception ";
    public final static String __DEBUG_START_TRY_TO_CONNECT
            = "Start Try to connect ";
    public final static String __DEBUG_SOCKET = "Socket ";
    public final static String __DEBUG_CONNECTED_SOCKET
            = "Connected Socket ";
    public final static String __DEBUG_WRITE_OBJECT = "writeObject ";
    public final static String __DEBUG_NAME = "Name ";
    public final static String __DEBUG_END_WRITE_OBJECT_WRITE
            = "end writeObject with ";
    public final static String __DEBUG_SSOCK_CONNECT_EXCEPTION
            = "ssock connect exception ";
    public final static String __DEBUG_CALL_OBJECTOUTPUTSTEAM
            = "Call ObjectOutputStream ";
    public final static String __DEBUG_ARUMENTOUTPUTSTREAM
            = "ArumentOutputStream";
    public final static String __DEBUG_CALL_GETINPUTSTREAM
            = "Call getInputStream()";
    public final static String __DEBUG_CALL_READOBJECT
            = "Call readObject()";
    public final static String __DEBUG_READ_OBJECT
            = "Read Object ";
    public final static String __DEBUG_TRY_TO_CONNECT
            = "Try to connect ";
    public final static String __DEBUG_PARAMETER_COUNT
            = "Parameter count ";
    public final static String __DEBUG_TOKEN = "token ";
    public final static String __DEBUG_OBJECTCLIENT_CMD
            = "$$ObjectClient() Command ";
    public final static String __DEBUG_END_MAKE_PARAMETER
	    = "End make parameter[] ";
    public final static String __DEBUG_GETINPUTSTREAM
            = "getInputStream() ";
    public final static String __DEBUG_CALL_OBJECTINPUTSTREAM
            = "Call ObjectInputStream() ";
    public final static String __DEBUG_CREATE_OBJECTINPUTSTREAM
            = "create ObjectInputStream() ";
    public final static String __DEBUG_REQUEST_PORT
            = "Request Port";
    public final static String __DEBUG_HOSTNAME
            = "Hostname";
    public final static String __DEBUG_COMMAND
            = "Command";
    public final static String __DEBUG_FIELD
            = "Field";
    public final static String __DEBUG_OBJECT
            = "Object";
    public final static String __DEBUG_SETUP_PROPERTIES = "setup properties";
    public final static String __DEBUG_CREATE_PROPERTIES = "create properties";
    public final static String __DEBUG_LOAD_PROPERTIES = "load properties";
    public final static String __DEBUG_LOAD_PROPERTIES_FROM_JAR 
        = "__loadPropertiesFromJar().";
    public final static String __DEBUG_READ_PROPERTIES_FROM_JAR 
        = "__readPropertiesFromJar().";

    public final static String __TRACE_CLASSNAME
            = "oserv.$$ObjectClient";
    public final static String __TRACE_CONNECT
            = "__connect";
    public final static String __TRACE_CLOSESOCKET
            = "__closeSocket";
    public final static String __TRACE_INTCONNECT
            = "__intconnect";
    public final static String __TRACE_LONGCONNECT
            = "__longconnect";
    public final static String __TRACE_FLOATCONNECT
            = "__floatconnect";
    public final static String __TRACE_DOUBLECONNECT
            = "__doubleconnect";
    public final static String __TRACE_BYTECONNECT
            = "__byteconnect";
    public final static String __TRACE_BOOLEANCONNECT
            = "__booleanconnect";
    public final static String __TRACE_SHORTCONNECT
            = "__shortconnect";
    public final static String __TRACE_CHARCONNECT
            = "__charconnect";
    public final static String __TRACE_GETSERVICECOMMANDPARAMETER
            = "__getServiceCommandParameter";
    public final static String __TRACE_CHECKCOMMAND
            = "__checkCommand";
    public final static String __TRACE_GETPROPERTIES
            = "__getProperties";
    public final static String __TRACE_LOADPROPERTYFROMJAR 
            = "__loadPropertyFromJar";

    private static ObjectServerProperties __sproperties = null;
    private static MessageResource __message_resource = new MessageResource();
    private static int __debug = 0;
    private static LogPrintStream __logStream = null;
    private static int __timeout    = 3000;
    /**
     * This constructor is called for initializing property values.
     *
     */
    public ObjectClient() {
        String  value_string;
        __getProperties();

        value_string = __sproperties.getProperty(ObjectServerHeader.OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            __setMessageLanguage(value_string);
        } else {
            __setMessageLanguage(ObjectServerHeader.OSERV_MESSAGE_LANGUAGE_VALUE);
        }

        __initialize();
    }

    public int __getTimeout() {
        return __timeout;
    }

    public void __setTimeout(int time_out_value) {
        __timeout = time_out_value;
    }

    public int __getRetryCount() {
        return __retry_count;
    }

    public void __setRetryCount(int retry_count) {
        __retry_count = retry_count;
    }

    /**
     * This method sets debug level.
     *
     * @param      int - debug level  
     * @return     void
     */
    protected void __setDebug(int level) {
        __debug = level;
    }

    protected int __getDebug() {
        return __debug;
    }

    /**
     * This method sets Stream for switching.
     *
     * @param      LogPrintStream 
     */
    protected void __setLogStream(LogPrintStream ps) {
        __logStream = ps;
    }

    protected LogPrintStream __getLogStream() {
        return __logStream;
    }

    public MessageResource __getMessageResource() {
        return __message_resource;
    }

    public void __setMessageLanguage(String language) {
        if (language == null) {
            return;
        }
        MessageResource __message_resource;
        if (__getMessageResource() != null) {
            __message_resource = __getMessageResource();
            __message_resource.setLanguage(language);
        }
    }

    public void __printExceptionMessage(String classname, String function, int nest, Exception exception_object) {
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
            __writeExceptionTrace(__getLogStream(), __getMessageResource(), 
                exception_object, classname, function, nest);
        }
    }

    public void __printExceptionMessage(LogPrintStream param_log_print_stream,
        String classname, String function, int nest, Exception exception_object) {
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
            __writeExceptionTrace(param_log_print_stream, __getMessageResource(), 
                exception_object, classname, function, nest);
        }
    }

    public void __writeExceptionTrace(
        LogPrintStream param_log_print_stream, 
        MessageResource param_message_resource,
        Exception param_exception,
        String    classname,
        String    function,
        int       nest
    ) {

        if (param_log_print_stream == null || param_message_resource == null 
            || param_exception == null || function == null || classname == null) {
            return ;
         }

         param_log_print_stream.printLogln(
             param_message_resource.getFormattedMessage(
                 ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                 param_message_resource.getMessage(
                    ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                 ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                 TraceMessage.getCallerMethod(param_exception, classname, function, nest),
                 ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                 param_exception.toString() ));
    }

    /**
     * This function provides a feature for initializing property values.
     *
     * Following values are updated.
     *    OSERV_DEBUG_LEVEL_KEY
     *    OSERV_RETRY_COUNT_KEY
     *    OSERV_REQUEST_PORT_KEY
     *    OSERV_SERVICE_HOSTNAME_KEY
     */
    private void __initialize() {
        String variable_value_string;
        int retry_count = 0;
        int port_number = 0;
        int local_timeout = 0;
        int local_debug = 0;

        variable_value_string = __sproperties.getProperty(
            ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY);

        if (variable_value_string != null) {
            try {
                local_debug = Integer.valueOf(variable_value_string).intValue();
                __setDebug(local_debug);
            } catch (Exception exception_object) {
                if (__getMessageResource() != null) {
                    System.err.println( __getMessageResource().getFormattedMessage(
                        ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_CLIENT)));
                }
                System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
            }
        } else {
            if (__getMessageResource() != null) {
                System.err.println( __getMessageResource().getFormattedMessage(
                    ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_CLIENT)));
            }
            System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
        }
        if (__getDebug() != 0) {
            __setDebug(__getDebug());
        }

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null && __getMessageResource() != null) {
            __getLogStream().printLogln( __getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_DEBUG_VALUE,
                __getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_DEBUG_VALUE ),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(__getDebug())));
        }
        variable_value_string = __sproperties.getProperty( ObjectServerHeader.OSERV_RETRY_COUNT_KEY);
        if (variable_value_string != null) {
            try {
                retry_count = Integer.valueOf(variable_value_string).intValue();
            } catch (Exception exception_object) {
                if (__getMessageResource() != null) {
                    System.err.println(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerError.RETRY_COUNT_VALUE_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.RETRY_COUNT_VALUE_ERROR_CLIENT)));
                }
                System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
            }
        } else {
            if (__getMessageResource() != null) {
                System.err.println( __getMessageResource().getFormattedMessage(
                    ObjectServerError.RETRY_COUNT_VALUE_NOT_FOUND_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.RETRY_COUNT_VALUE_NOT_FOUND_CLIENT)));
            }
            System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
        }
        if (retry_count > __getRetryCount()) {
            __setRetryCount(retry_count);
        }
        variable_value_string = __sproperties.getProperty(ObjectServerHeader.OSERV_REQUEST_PORT_KEY);
        if (variable_value_string != null) {
            try {
                port_number = Integer.valueOf(variable_value_string).intValue();
            } catch (Exception exception_object) {
                if (__getMessageResource() != null) {
                    System.err.println(__getMessageResource().getFormattedMessage(
                        ObjectServerError.PORT_CONFIGURATION_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.PORT_CONFIGURATION_ERROR_CLIENT)));
                }
                System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
            }
        } else {
            if (__getMessageResource() != null) {
                System.err.println(__getMessageResource().getFormattedMessage(
                    ObjectServerError.PORT_CONFIGURATION_NOT_FOUND_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.PORT_CONFIGURATION_NOT_FOUND_CLIENT)));
            }
            System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
        }
        variable_value_string = __sproperties.getProperty(ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY);
        if (variable_value_string == null || variable_value_string.length() == 0) {
            if (__getMessageResource() != null) {
                System.err.println(__getMessageResource().getFormattedMessage(
                    ObjectServerError.HOST_CONFIGURATION_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.HOST_CONFIGURATION_ERROR_CLIENT)));
            }
            System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
        }

        variable_value_string = __sproperties.getProperty(ObjectServerHeader.OSERV_CONNECTION_TIMEOUT_KEY);
        if (variable_value_string != null && variable_value_string.length() != 0) {
            try {
                local_timeout = Integer.valueOf(variable_value_string).intValue();
                __setTimeout(local_timeout);
            } catch (Exception exception_object) {
                if (__getMessageResource() != null) {
                    System.err.println(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_CLIENT)));
                }
                System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
            }
        } else {
            if (__getMessageResource() != null) {
                System.err.println(
                __getMessageResource().getFormattedMessage(
                    ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_CLIENT)));
            }
            System.exit(ObjectServerHeader.ERROR_EXIT_CODE);
        }
        if (__getLogStream() == null) {
            __setLogStream(new LogPrintStream(System.out));
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_LOG_START,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_LOG_START)));
            }
        }
    }

    public int __intconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_INTCONNECT, 0, exception_object);
            throw exception_object;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln( __getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_RETURN_VALUE,
                __getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_RETURN_VALUE),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                object_variable.getClass().getName()));
        }

        if ((object_variable instanceof NetworkObject) == true) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(int.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getInt());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_RETURN_TYPE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        network_object.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        value_string));
            }
            return network_object.getInt();
        }
        ObjectServerException ose ;
        ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public long __longconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_LONGCONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(long.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getLong());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln( __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_RETURN_TYPE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        network_object.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        value_string));
            }
            return network_object.getLong();
        }
        ObjectServerException ose ;
        ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public float __floatconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_FLOATCONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(float.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getFloat());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln( __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_RETURN_TYPE,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    network_object.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    value_string));
            }
            return network_object.getFloat();
        }
        ObjectServerException ose;
        ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public double __doubleconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_DOUBLECONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(double.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getDouble());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln( __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_RETURN_TYPE,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    network_object.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    value_string));
            }
            return network_object.getDouble();
        }
        ObjectServerException ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public byte __byteconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_BYTECONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(byte.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getByte());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln( __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_RETURN_TYPE,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    network_object.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    value_string));
            }
            return network_object.getByte();
        }
        ObjectServerException ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public boolean __booleanconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_BOOLEANCONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(boolean.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getBoolean());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_RETURN_TYPE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        network_object.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        value_string));
            }
            return network_object.getBoolean();
        }
        ObjectServerException ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public short __shortconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_SHORTCONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(short.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getShort());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_RETURN_TYPE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        network_object.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        value_string));
            }
            return network_object.getShort();
        }
        ObjectServerException ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }

    public char __charconnect(String command, Field[] variable_field, Object source_object)  
        throws Exception, InvocationTargetException
    {
        String      variable_name;
        String      value_string;
        Object      object_variable;
        NetworkObject network_object;
        try {
            object_variable = __connect(command, variable_field, source_object);
        } catch (Exception exception_object) {

            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CHARCONNECT, 0, exception_object);
            throw exception_object;
        }
        if (object_variable instanceof NetworkObject) {
            network_object = (NetworkObject)object_variable;
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
                variable_name = network_object.getName();
                if (variable_name != null && 
                    variable_name.compareTo(char.class.getName()) == 0) {
                    value_string = String.valueOf(network_object.getChar());
                } else {
                    value_string = ObjectServerHeader.EMPTY_MESSAGE;
                }
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_RETURN_TYPE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_RETURN_TYPE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        network_object.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        value_string));
            }
            return network_object.getChar();
        }
        ObjectServerException ose;
        ose = new ObjectServerException(
            __getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                __getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
        throw (Exception)ose;
    }


    public void __closeSocket(Socket socket_object) {
        if (socket_object == null) {
            return;
        }
        if (socket_object.isConnected() == true) {
            try {
                socket_object.shutdownOutput();
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CLOSESOCKET, 0, exception_object);
            }
        }
        if (socket_object.isConnected() == true) {
            try {
                socket_object.shutdownInput();
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CLOSESOCKET, 0, exception_object);
            }
        }
        if (socket_object.isConnected() == false) {
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
                __getLogStream().printLogln( 
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CONNECTION_END_KEY,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CONNECTION_END_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        socket_object.toString()));
            }
        } 
    }

    private boolean __isInputStream(String parameter_type) {
        if (parameter_type != null &&
            (parameter_type.compareTo( 
            ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_PIPEDINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_BUFFEREDINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_FILTERINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_DATAINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_INPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_LINENUMBERINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_PUSHBACKINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_FILEINPUTSTREAM_TYPE_KEY) == 0)
        ) {
            return true;
        }
        return false;
    }

    private boolean __isOutputStream(String parameter_type) {
        if (parameter_type != null &&
            (parameter_type.compareTo( 
            ObjectServerHeader.JAVA_OBJECTOUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_PIPEDOUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_PRINTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_BYTEARRAYOUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_BUFFEREDOUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_FILTEROUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_DATAOUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_OUTPUTSTREAM_TYPE_KEY) == 0 ||
            parameter_type.compareTo(
            ObjectServerHeader.JAVA_FILEOUTPUTSTREAM_TYPE_KEY) == 0)
        ) {
            return true;
        }
        return false;
    }

    /**
     * This method provides a feature to call server method.
     *
     * @param    object list
     * @return   java.lang.Object is a opaque type of return.
     * @throws   java.lang.Exception is a opaque type of exception.
     */ 
    public Object __connect(String Command, Field[] variable_field, Object source_object) 
            throws Exception, InvocationTargetException {
        String hostname;
        String port_number_string;
        int    port_number;
        Socket output_socket;
        Field[] pfld;
        InetSocketAddress output_internet_socket_address;
        InputStream fin;
        OutputStream fot;
        ObjectInputStream oin = null;
        ObjectInputStream object_input_stream;
        RetryObjectInputStream retry_object_input_stream;
        ObjectServerException object_server_exception;
        ObjectOutputStream dot;
        Object obj;
        Object[] req;
        String path[];
        int    retry;
        int       i;
        int       k;
        int count;
        String vhead;
        String chead;
        String head;
        int      iop;
        long     lop;
        short    sop;
        float    fop;
        double   dop;
        Object    op;
        boolean  bop;
        byte    byop;
        char     cop;
        String  fldtype;
        int    cnt;
        int    idx;
        String param;
        int    tcount;
        ArgumentOutputStream aot;
        String jar_path;
        String jar_name;
        String transmit_jarname = null;
        File   transmit_jarname_file;
        String transient_home_string = null;

        if (Command == null) {
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.COMMAND_PARAMETER_NULL_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.COMMAND_PARAMETER_NULL_CLIENT)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.COMMAND_PARAMETER_NULL_CLIENT );
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(object_server_exception.toString());
            }
            throw object_server_exception;
        }

        if (variable_field == null) {
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.FIELD_PARAMETER_NULL_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.FIELD_PARAMETER_NULL_CLIENT)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.FIELD_PARAMETER_NULL_CLIENT );
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(object_server_exception.toString());
            }
            throw object_server_exception;
        }
        if (source_object == null) {
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.OBJECT_PARAMETER_NULL_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.OBJECT_PARAMETER_NULL_CLIENT)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.OBJECT_PARAMETER_NULL_CLIENT );
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(object_server_exception.toString());
            }
            throw object_server_exception;
        }

        hostname = __sproperties.getProperty(ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY);
        if (hostname == null) {
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.HOST_CONFIGURATION_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.HOST_CONFIGURATION_ERROR_CLIENT)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.HOST_CONFIGURATION_ERROR_CLIENT );
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(object_server_exception.toString());
            }
            throw object_server_exception;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(__getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_HOST_NAME,
                __getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_HOST_NAME),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                hostname));
        }
        port_number_string = __sproperties.getProperty(ObjectServerHeader.OSERV_REQUEST_PORT_KEY);
        if (port_number_string == null) {
            if (__getMessageResource() != null) {
                object_server_exception =
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.PORT_CONFIGURATION_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.PORT_CONFIGURATION_ERROR_CLIENT)));
            } else {
                object_server_exception =
                new ObjectServerException( 
                    ObjectServerError.PORT_CONFIGURATION_ERROR_CLIENT );
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(object_server_exception.toString());
            }
            throw object_server_exception;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_REQUEST_PORT,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_REQUEST_PORT),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    port_number_string));
        }
        port_number = Integer.valueOf(port_number_string).intValue();

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_COMMAND_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_COMMAND_KEY), 
                    Command));
        }

        StringTokenizer string_tokenizer = new StringTokenizer(Command, ObjectServerHeader.OBJECT_SEPARATOR);
        count = string_tokenizer.countTokens();
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_PARAMETER_NUMBER,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_PARAMETER_NUMBER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(count) ));
        }
        if (count > __COMMAND_COUNT) {
            if (__getMessageResource() != null) {
                object_server_exception 
                = new oserv.ObjectServerException(
                __getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_CLIENT)));
            } else {
                object_server_exception 
                = new oserv.ObjectServerException(
                    ObjectServerError.PARAMETER_ERROR_CLIENT );
            }

            throw (Exception)object_server_exception;
        }
        try {
            __checkCommand(Command);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            throw exception_object;
        }

        param = null;
        try {
            param = __getServiceCommandParameter(Command);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            throw exception_object;
        }
        count = 0;
        tcount = 0;
        if (param != null) {
            string_tokenizer = new StringTokenizer(param, 
                ObjectServerHeader.PARAMETER_SEPARATOR);
            tcount = string_tokenizer.countTokens();
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln( 
                    __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_PARAMETER_NUMBER,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_PARAMETER_NUMBER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(tcount)));
            }
        }

        head = __replace(Command, ObjectServerHeader.OBJECT_DOT_SEPARATOR, ObjectServerHeader.VARIABLE_SEPARATOR);
        head = __replace(head, ObjectServerHeader.OBJECT_SEPARATOR, ObjectServerHeader.VARIABLE_SEPARATOR);
        head = __replace(head, ObjectServerHeader.PARAMETER_SEPARATOR, ObjectServerHeader.VARIABLE_SEPARATOR);

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_COMMAND_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_COMMAND_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    head));
        }
        if (param != null && tcount > 0) {
            for (int j = 0; j < tcount; j++) {
                for (i = 0; i < variable_field.length; i++) {
                    vhead = head + ObjectServerHeader.PARAMETER_NAME_KEY + String.valueOf(j);
                    if (variable_field[i].getName().compareTo(vhead) == 0) {
                        count++;
                    }
                }
            }
        } else {
            count = 0;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_PARAMETER_NUMBER,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_PARAMETER_NUMBER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(count)));
        }

        output_socket = new Socket();
        output_internet_socket_address 
                     = new InetSocketAddress(hostname, port_number);

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_SOCKET_NAME,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_SOCKET_NAME),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    output_socket.toString()));
        }
        for ( retry = 0; retry < __retry_count; retry++) {
            try {
                output_socket.connect(output_internet_socket_address, __getTimeout());
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                continue;
            }
            if (output_socket.isConnected() == true) {
                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_CONNECTION_START_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_CONNECTION_START_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            output_socket.toString()));
                }
                break;
            }
        }
        if (output_socket.isConnected() == false) {
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_SOCKET_NAME,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_SOCKET_NAME),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        output_socket.toString()));
            }
            if (__getMessageResource() != null) {
                object_server_exception 
                    = new oserv.ObjectServerException(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerError.CONNECTION_ERROR_CLIENT,
                            __getMessageResource().getMessage(
                                ObjectServerError.CONNECTION_ERROR_CLIENT)));
            } else {
                object_server_exception 
                    = new oserv.ObjectServerException(
                    ObjectServerError.CONNECTION_ERROR_CLIENT );
            }
            __closeSocket(output_socket);
            throw (Exception)object_server_exception;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_SOCKET_NAME,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_SOCKET_NAME),
                    output_socket.toString()));
        }

        try {
            fot = output_socket.getOutputStream();
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    __getMessageResource().getMessage(
                        ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION);
            }
            __closeSocket(output_socket);
            throw exception_object;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln( 
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    fot.toString(),
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
        }
        try {
            dot = new ObjectOutputStream((OutputStream)fot);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    __getMessageResource().getMessage(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION);
            }
            __closeSocket(output_socket);
            throw exception_object;
        }    

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_COMMAND_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_COMMAND_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ObjectServerHeader.OSERV_COMMAND_TYPE_REQUEST_KEY));
        }
        jar_path = null;
        transmit_jarname = null;
        transient_home_string = __sproperties.getProperty(ObjectServerHeader.OSERV_TRANSIENT_HOME_KEY);
        if (transient_home_string != null) {
            jar_path = __getServiceCommandClass(Command);
            idx = jar_path.lastIndexOf(ObjectServerHeader.OBJECT_DOT_SEPARATOR);
            if (idx >= 0) {
                jar_name = jar_path.substring(idx+1);
            } else {
                jar_name = jar_path;
            }
            transmit_jarname = transient_home_string + File.separator + jar_name
                +ObjectServerHeader.SUFFIX_SEPARATOR+ObjectServerHeader.JAR_SUFFIX;
            transmit_jarname_file = new File(transmit_jarname);
            if (transmit_jarname_file.exists() == false) {
                idx = jar_path.lastIndexOf(ObjectServerHeader.OBJECT_DOT_SEPARATOR);
                if (idx >= 0) {
                    transmit_jarname = transient_home_string + File.separator + jar_path
                        +ObjectServerHeader.SUFFIX_SEPARATOR+ObjectServerHeader.JAR_SUFFIX;
                    transmit_jarname_file = new File(transmit_jarname);
                    if (transmit_jarname_file.exists() == false) {
                        Exception exception_object;
                        exception_object = new Exception( );
                        throw exception_object;
                    }
                }
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_FILE_PATH,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_FILE_PATH),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    transmit_jarname));
            }
        }
        if (transmit_jarname != null) {
            try {
                dot.writeObject(ObjectServerHeader.OSERV_COMMAND_TYPE_TRANSIENT_KEY);
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                if (__getMessageResource() != null) {
                    object_server_exception = 
                        new ObjectServerException( __getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        __getMessageResource().getMessage(
                            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
                } else {
                    object_server_exception = 
                    new ObjectServerException( 
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                }
                throw exception_object;
            }
            ClassInputStream          class_input_stream;
            ObjectTool                ofunc;

            ofunc = new ObjectTool();

            try {
                class_input_stream = ClassInputStream.valueOf(jar_path,
                __getMessageResource(),
                __getDebug(),
                __getLogStream());
                ofunc.requireRegistration(dot, class_input_stream);
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                throw exception_object;
            }
        } else {

            try {
                dot.writeObject(ObjectServerHeader.OSERV_COMMAND_TYPE_REQUEST_KEY);
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                if (__getMessageResource() != null) {
                    object_server_exception = 
                        new ObjectServerException( __getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        __getMessageResource().getMessage(
                            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
                } else {
                    object_server_exception = 
                    new ObjectServerException( 
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                }
                throw exception_object;
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_COMMAND_KEY,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_COMMAND_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        Command));
            }
        }

        try {
            dot.writeObject(Command);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            if (__getMessageResource() != null) {
                object_server_exception = 
                    new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    __getMessageResource().getMessage(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            }
            __closeSocket(output_socket);
            throw exception_object;
        }
        
        try {
            output_socket.setSoTimeout(__getTimeout());
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    output_socket.toString(),
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
        }
        try {
            fin = output_socket.getInputStream();
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    __getMessageResource().getMessage(
                        ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION);
            }
            __closeSocket(output_socket);
            throw exception_object;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_COMMAND_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_COMMAND_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    Command));
        }

        pfld = null;
        if (count != 0) {
            pfld = new Field[count];
            for (k = 0; k < count; k++) {
                for(i = 0; i < variable_field.length; i++) {
                    chead = head + ObjectServerHeader.PARAMETER_NAME_KEY + String.valueOf(k);
                    vhead = variable_field[i].getName();
                    if (chead.compareTo(vhead) == 0) {
                        pfld[k] = variable_field[i];

                        break;
                    }
                }
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln( 
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_FIELD_COUNT,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_FIELD_COUNT),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		        String.valueOf(count),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(pfld.length)));
                for (k = 0; k < count;k++) {
                    chead = head + ObjectServerHeader.PARAMETER_NAME_KEY + String.valueOf(k);
                    if (__getMessageResource() != null) {

                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_FIELD_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_FIELD_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pfld[k].getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            chead));
                    }
                }
            }
            try {
                aot = new ArgumentOutputStream(dot);
                if (__getMessageResource() != null) {
                    aot.setMessageResource(__getMessageResource());
                }
                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && __getLogStream() != null) {
                    if (__getLogStream() != null) {
                        aot.setDebugLogStream(__getDebug(), __getLogStream());
                    }
                }
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                __closeSocket(output_socket);
                throw exception_object;
            }    
            try {
                aot.writeArgument(pfld, source_object);
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                __closeSocket(output_socket);
                throw exception_object;
            }
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    fin.toString(),
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
        }
        try {
            retry_object_input_stream = RetryObjectInputStream.valueOf(fin,
                __getMessageResource(),
                __getDebug(),
                __getLogStream()
            );
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            if (__getMessageResource() != null) {
                object_server_exception = 
                new ObjectServerException( __getMessageResource().getFormattedMessage(
                    ObjectServerError.IO_CONSTRUCTION_ERROR_CLIENT,
                    __getMessageResource().getMessage(
                        ObjectServerError.IO_CONSTRUCTION_ERROR_CLIENT)));
            } else {
                object_server_exception = 
                new ObjectServerException( 
                    ObjectServerError.IO_CONSTRUCTION_ERROR_CLIENT );
            }
            throw object_server_exception;
        }
        try {
            retry_object_input_stream.setRetryCount(__getRetryCount());
            if (__getDebug() > 0) {
                if (__getLogStream() != null) {
                    retry_object_input_stream.setDebugLogStream(__getDebug(), __getLogStream());
                }
            }
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            __closeSocket(output_socket);
            throw exception_object;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    __DEBUG_CALL_READOBJECT));
        }

        obj = new Object();
        try {
            obj = retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
            obj = exception_object;
        }
        if ((obj instanceof SODObject) == true && pfld != null) {
            byte[]              byte_data;
            NetworkObject       network_object;
            Field               local_field;
            Object              contents_obj;
            OutputStream        local_output_stream;
            ObjectOutputStream  local_object_output_stream;
            FileOutputStream    local_file_output_stream;
            DataOutputStream    local_data_output_stream = null;
            Class               local_object_output_stream_class = null;

            for (i = 0; i < pfld.length; i++) {
                if (__isOutputStream(pfld[i].getType().getName()) == true) {

                    local_output_stream = (OutputStream)pfld[i].get(source_object);
                    if ((local_output_stream instanceof ObjectOutputStream) == true) {
                        local_object_output_stream_class = ObjectOutputStream.class;
                        local_field = local_object_output_stream_class.getDeclaredField( __OBJECT_OUTPUT_STREAM_BOUT);
                        try {
                            local_data_output_stream = (DataOutputStream)local_field.get(local_output_stream);
                        } catch (Exception exception_object) {
                            __closeSocket(output_socket);
                            object_server_exception
                                = new ObjectServerException(
                                __getMessageResource().getFormattedMessage(
                                    ObjectServerException.OBJECT_OUTPUT_STREAM_NOT_SUPPORT_EXCEPTION,
                                    __getMessageResource().getMessage(
                                        ObjectServerException.OBJECT_OUTPUT_STREAM_NOT_SUPPORT_EXCEPTION)));
                        
                            throw object_server_exception;
                        }
                    }
                    if (local_output_stream == null) {
                        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                            __getLogStream().printLogln(
                                __getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM,
                                    __getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    ObjectServerDebug.DEBUG_NULL_KEY));
                        }
                        continue;
                    }
                    if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                        __getLogStream().printLogln(
                            __getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM,
                                __getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                local_output_stream.toString()
                        ));
                    }
                    while (true) {
                        try {
                            contents_obj = retry_object_input_stream.retryReadObject();
                        } catch (Exception exception_object) {
                            if (exception_object != null && (exception_object instanceof EOFException) == true) {
                                break;
                            }
                            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                            __closeSocket(output_socket);
                            throw exception_object;
                        }
                        if ((contents_obj instanceof EODObject) == true) {
                            if (((SODObject)obj).isClose() == true) {
                                try {
                                    local_output_stream.close();
                                } catch (Exception exception_object) {
                                    __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                                    if (__getMessageResource() != null) {
                                        object_server_exception = 
                                            new ObjectServerException( __getMessageResource().getFormattedMessage(
ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_CLOSE_IO_EXCEPTION,
                                        
                                        __getMessageResource().getMessage(
ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_CLOSE_IO_EXCEPTION)));
                                    } else {
                                        object_server_exception = 
                                            new ObjectServerException( 
                                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_CLOSE_IO_EXCEPTION);
                                    }
                                    __closeSocket(output_socket);
                                    throw exception_object;
                                }
                            }
                            break;
                        }
                        if ((contents_obj instanceof NetworkObject) == true) {
                            network_object = (NetworkObject)contents_obj;
                            byte_data = (byte[])network_object.get();
                            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                                __getLogStream().printLogln(
                                    __getMessageResource().getFormattedMessage(
                                        ObjectServerDebug.DEBUG_SIZE_KEY,
                                        __getMessageResource().getMessage(
                                            ObjectServerDebug.DEBUG_SIZE_KEY),
                                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                        String.valueOf(byte_data.length)
                                ));
                            }
                            if ((local_output_stream instanceof ObjectOutputStream) != true) {
                                local_output_stream.write(byte_data, 0, byte_data.length);
                            } else {
                                try {

                                    local_data_output_stream.write(byte_data, 0, byte_data.length);
                                } catch (Exception exception_object) {
                                    __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                                    throw exception_object;
                                }
                            }
                        }
                    }
                    try {
                        obj = retry_object_input_stream.retryReadObject();
                        if ((obj instanceof SODObject) != true) {
                            break;
                        }
                    } catch (Exception exception_object) {
                        if (exception_object != null && (exception_object instanceof EOFException) == true) {
                            break;
                        }
                        __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                        __closeSocket(output_socket);
                        throw exception_object;
                    }
                }
            }
        }
        ByteArrayOutputStream   byte_array_output_stream;
        ByteArrayInputStream    byte_array_input_stream;
        StringBufferInputStream string_buffer_input_stream;

        if ((obj instanceof SODObject) == true) {
            byte[]   byte_data;
            NetworkObject network_object;
            String type_string;
            String file_name;
            file_name = ObjectServerHeader.SUFFIX_SEPARATOR + 
                    Thread.currentThread().getName();
            NamedFileInputStream named_file_input_stream = null;
            FileOutputStream     file_output_stream = null;
            byte_array_output_stream = null;
            type_string = __getServiceCommandReturn(Command);

            if (type_string.compareTo(
                  ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) != 0 &&
                type_string.compareTo(
                  ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) != 0) {
                try {
                    file_output_stream = new FileOutputStream(file_name);
                } catch (Exception exception_object) {
                    __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                    __closeSocket(output_socket);
                    throw exception_object;
                }
            } else {
                byte_array_output_stream = new ByteArrayOutputStream();
            }

            while (true) {
                try {
                    obj = retry_object_input_stream.retryReadObject();
                } catch (Exception exception_object) {
                    __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                    if (file_output_stream != null) {
                        try {
                            file_output_stream.close();
                        } catch (Exception second_exception_object) {
                            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, second_exception_object);
                            __closeSocket(output_socket);
                            throw exception_object;
                        }
                    }
                    if (type_string.compareTo(
                        ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) != 0 &&
                       type_string.compareTo(
                        ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) != 0) {
                        named_file_input_stream = new NamedFileInputStream(file_name);
                        try {
                            obj = (Object)__getReturnInputStream(type_string, named_file_input_stream);
                        } catch (Exception second_exception_object) {
                            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, second_exception_object);
                            __closeSocket(output_socket);
                            throw exception_object;
                        }
                    } else {
                       if (type_string.compareTo(
                           ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0) {
                            string_buffer_input_stream
                                = new StringBufferInputStream(
                                new String(byte_array_output_stream.toByteArray() ) );
                            obj = (Object)string_buffer_input_stream;
                        } else {
                            byte_array_input_stream
                                = new ByteArrayInputStream(
                                    byte_array_output_stream.toByteArray() );
                            obj = (Object)byte_array_input_stream;
                        }
                    }
                    break;
                }
                if ((obj instanceof EODObject) == true) {
                    try {
                        file_output_stream.close();
                    } catch (Exception second_exception_object) {
                        __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, second_exception_object);
                        __closeSocket(output_socket);
                        throw second_exception_object;
                    }
                    if (type_string.compareTo(
                        ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) != 0 &&
                       type_string.compareTo(
                        ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) != 0) {
                        named_file_input_stream = new NamedFileInputStream(file_name);
                        try {
                            obj = (Object)__getReturnInputStream(type_string, named_file_input_stream);
                        } catch (Exception second_exception_object) {
                            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, second_exception_object);
                            __closeSocket(output_socket);
                            throw second_exception_object;
                        }
                    } else {
                       if (type_string.compareTo(
                           ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0) {
                            string_buffer_input_stream
                                = new StringBufferInputStream(
                                new String(byte_array_output_stream.toByteArray() ));
                            obj = (Object)string_buffer_input_stream;
                        } else {
                            byte_array_input_stream
                                = new ByteArrayInputStream(
                                    byte_array_output_stream.toByteArray() );
                            obj = (Object)byte_array_input_stream;
                        }
                    }
/*
                    obj = named_file_input_stream;
*/
                    break;
                }
                if ((obj instanceof NetworkObject) == true) {
                    network_object = (NetworkObject)obj;
                    byte_data = (byte[])network_object.get();
                    if (type_string.compareTo(
                        ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) != 0) {
                        file_output_stream.write(byte_data, 0, byte_data.length);
                    } else {
                        byte_array_output_stream.write(byte_data, 0, byte_data.length);
                    }
                }
            }
        }

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_OBJECT_NAME,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_OBJECT_NAME),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    obj.toString()));
            if ((obj instanceof NetworkObject) == true) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_TYPE_KEY,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_TYPE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ((NetworkObject)obj).getName()));

                if (((NetworkObject)obj).getName().compareTo(
                        int.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getInt())
                        ));
                } else if (((NetworkObject)obj).getName().compareTo(
                        long.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getLong())
                        ));
                } else if (((NetworkObject)obj).getName().compareTo(
                        float.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getFloat())
                        ));
                } else if (((NetworkObject)obj).getName().compareTo(
                        double.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getDouble())
                        ));
                } else if (((NetworkObject)obj).getName().compareTo(
                        boolean.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getBoolean())
                        ));
                } else if (((NetworkObject)obj).getName().compareTo(
                        byte.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getByte())
                        ));
                } else if (((NetworkObject)obj).getName().compareTo(
                        char.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getChar())));
                } else if (((NetworkObject)obj).getName().compareTo(
                        short.class.getName()
                ) == 0)  {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(((NetworkObject)obj).getShort())));
                } else {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VALUE_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VALUE_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            obj.toString()));
                }
            }
        }

        __closeSocket(output_socket);

        if (obj instanceof oserv.ObjectServerException) {
            object_server_exception = (oserv.ObjectServerException)obj;
            if (object_server_exception.getMessage() != null &&
                object_server_exception.getMessage().indexOf(
                oserv.ObjectServerException.DEBUG_RETURN) == 0) {
                String msg = object_server_exception.getMessage();
                String key = msg.substring(
                    oserv.ObjectServerException.DEBUG_RETURN.length());
                idx = key.indexOf(
                    oserv.ObjectServerException.USER_DEBUG_SEPARATOR);
                if (idx <= 0) {
                    throw (Exception)object_server_exception;
                }
                int    isz;
                String ssz = key.substring(0, idx);
                try {
                    isz = Integer.valueOf(ssz);
                } catch (Exception exception_object) {
                    __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                    throw (Exception)object_server_exception;
                }
                String rvalue = key.substring(idx + 1, idx + 1 + isz); 
                String dbgmsg = key.substring(idx + 1 + isz); 
                byte[] buffer;
                Object return_value;
                buffer = rvalue.getBytes();
                try {
                    ByteArrayInputStream bis = new ByteArrayInputStream(buffer);
                    ObjectInputStream    ois = new ObjectInputStream(bis);
                    String rtype   = (String)ois.readObject();
                    return_value  = (Object)ois.readObject();
                } catch (Exception exception_object) {
                    __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CONNECT, 0, exception_object);
                    throw (Exception)object_server_exception;
                }
                System.out.print(dbgmsg);
                return return_value;
            }
            throw (Exception)object_server_exception;
        }
        if (obj instanceof java.lang.reflect.InvocationTargetException) {
            java.lang.reflect.InvocationTargetException inv;    
            inv = (java.lang.reflect.InvocationTargetException)obj;    
            throw inv;
        }
        if ((obj instanceof NetworkObject) == true) {
            if (
                ((NetworkObject)obj).getName().compareTo(
                        int.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        long.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        float.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        double.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        byte.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        boolean.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        char.class.getName()) == 0 ||
                ((NetworkObject)obj).getName().compareTo(
                        short.class.getName()) == 0 
            ) {
                return obj;
            }
            if ((obj instanceof NetworkObject) == true) {
                NetworkObject network_object;
                network_object = (NetworkObject)obj;
                obj = network_object.get();
                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && obj != null) {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_OBJECT_NAME,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_OBJECT_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            network_object.toString(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_OBJECT_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            obj.toString()));
                }
                if (obj != null && obj.getClass().getName().compareTo(__ARRAY_BYTE_CLASS_NAME) == 0) {
                    ByteArrayInputStream  sub_byte_array_input_stream;
                    ObjectInputStream     sub_object_input_stream;
                    sub_byte_array_input_stream = new ByteArrayInputStream((byte[])obj); 
                    try {
                        sub_object_input_stream = new ObjectInputStream(sub_byte_array_input_stream);
                        obj = sub_object_input_stream.readObject();
                    } catch (Exception exception_object) {
                        throw exception_object; 
                    }
                }
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && obj != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_OBJECT_NAME),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        obj.toString()));
            }
        }
        return obj;
    }

    public InputStream __getReturnInputStream(String type_string,
        NamedFileInputStream  named_file_input_stream) throws Exception {

        NamedBufferedInputStream      internal_named_buffered_input_stream;
        NamedSequenceInputStream      internal_named_sequence_input_stream;
        NamedDataInputStream          internal_named_data_input_stream;
        NamedFileInputStream          internal_named_file_input_stream;
        NamedFilterInputStream        internal_named_filter_input_stream;
        NamedLineNumberInputStream    internal_named_line_number_input_stream;
        NamedPushbackInputStream      internal_named_pushback_input_stream;
        NamedObjectInputStream        internal_named_object_input_stream;
        ByteArrayInputStream          byte_array_input_stream;

        if (type_string == null) {
            return null;
        }
        if (type_string.compareTo(
                    ObjectServerHeader.JAVA_INPUTSTREAM_TYPE_KEY) == 0) {
            return (InputStream)named_file_input_stream;
        }
        if (type_string.compareTo(
                    ObjectServerHeader.JAVA_FILEINPUTSTREAM_TYPE_KEY) == 0) {
            return (FileInputStream)named_file_input_stream;
        }
        if (type_string.compareTo(
            ObjectServerHeader.JAVA_BUFFEREDINPUTSTREAM_TYPE_KEY) == 0) {

            internal_named_buffered_input_stream
              = new NamedBufferedInputStream(named_file_input_stream);
            return  internal_named_buffered_input_stream;
        }
        if (type_string.compareTo(
                ObjectServerHeader.JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY) == 0) {

            byte_array_input_stream 
                = new ByteArrayInputStream(
                    ObjectServerHeader.EMPTY_MESSAGE.getBytes() );

            internal_named_sequence_input_stream
                = new NamedSequenceInputStream(
                    named_file_input_stream,
                    byte_array_input_stream );
            return internal_named_sequence_input_stream;
        }
        if (type_string.compareTo(
            ObjectServerHeader.JAVA_DATAINPUTSTREAM_TYPE_KEY) == 0) {

            internal_named_data_input_stream
                = new NamedDataInputStream(named_file_input_stream);
            return internal_named_data_input_stream;
        }
        if (type_string.compareTo(
            ObjectServerHeader.JAVA_FILTERINPUTSTREAM_TYPE_KEY) == 0) {

            internal_named_filter_input_stream
                = new NamedFilterInputStream(named_file_input_stream);
            return internal_named_filter_input_stream;
        }
        if (type_string.compareTo(ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY) == 0) {
            internal_named_object_input_stream
                = new NamedObjectInputStream(named_file_input_stream);
            return internal_named_object_input_stream;
        }
        if (type_string.compareTo(ObjectServerHeader.JAVA_LINENUMBERINPUTSTREAM_TYPE_KEY) == 0) {
            internal_named_line_number_input_stream
                = new NamedLineNumberInputStream(named_file_input_stream);
            return internal_named_line_number_input_stream;
        }
        if (type_string.compareTo(ObjectServerHeader.JAVA_PUSHBACKINPUTSTREAM_TYPE_KEY) == 0) {
            internal_named_pushback_input_stream
                = new NamedPushbackInputStream(named_file_input_stream);
            return internal_named_pushback_input_stream;
        }
        return null;
    }

    public String __getServiceCommandReturn(String command) throws Exception {
        String class_name;

        try {
            class_name = __getServiceCommandElement(command, 3);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        class_name = __replace(class_name, ObjectServerHeader.VARIABLE_SEPARATOR,
            ObjectServerHeader.OBJECT_DOT_SEPARATOR);
        return class_name;
    }

    public String __getServiceCommandClass(String command) throws Exception {
        String class_name;

        try {
            class_name = __getServiceCommandElement(command, 0);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return class_name;
    }

    public boolean __checkCommand(String command) throws Exception {
        String parameter_name;
        String class_name;
        StringTokenizer string_tokenizer;
        ObjectServerException  object_server_exception;

        try {
            class_name = __getServiceCommandReturn(command);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (class_name != null && (__isOutputStream(class_name) == true || 
            __isInputStream(class_name) == true)) {
            if (__getMessageResource() != null) {
                object_server_exception = 
                    new ObjectServerException( 
                        __getMessageResource().getFormattedMessage(
                            ObjectServerError.STREAM_RETURN_ERROR_CLIENT,
                            __getMessageResource().getMessage(
                                ObjectServerError.STREAM_RETURN_ERROR_CLIENT)));
            } else {
                object_server_exception 
                = new oserv.ObjectServerException(
                    ObjectServerError.STREAM_RETURN_ERROR_CLIENT);
            }
            throw object_server_exception;
        }
        try {
            parameter_name = __getServiceCommandParameter(command);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_CHECKCOMMAND, 0, exception_object);
            throw exception_object;
        }
       
        if (parameter_name == null) {
            return true;
        }
        string_tokenizer = new StringTokenizer(parameter_name, 
                ObjectServerHeader.PARAMETER_SEPARATOR);

        while (string_tokenizer.hasMoreTokens()) {
            class_name = string_tokenizer.nextToken();

            if (class_name.compareTo(ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY) == 0) {
                if (__getMessageResource() != null) {
                    object_server_exception = 
                        new ObjectServerException( 
                            __getMessageResource().getFormattedMessage(
                                ObjectServerError.OBJECT_OUTPUT_STREAM_PARAMETER_ERROR_CLIENT,
                                __getMessageResource().getMessage(
                                    ObjectServerError.OBJECT_OUTPUT_STREAM_PARAMETER_ERROR_CLIENT)));
                } else {
                    object_server_exception =
                        new oserv.ObjectServerException(
                            ObjectServerError.OBJECT_OUTPUT_STREAM_PARAMETER_ERROR_CLIENT);
                }
                throw object_server_exception;
            }
            if (class_name.compareTo(ObjectServerHeader.JAVA_PIPEDINPUTSTREAM_TYPE_KEY) == 0) {
                if (__getMessageResource() != null) {
                    object_server_exception = 
                        new ObjectServerException( 
                            __getMessageResource().getFormattedMessage(
                                ObjectServerError.PIPED_INPUT_STREAM_PARAMETER_ERROR_CLIENT,
                                __getMessageResource().getMessage(
                                    ObjectServerError.PIPED_INPUT_STREAM_PARAMETER_ERROR_CLIENT)));
                } else {
                    object_server_exception =
                        new oserv.ObjectServerException(
                            ObjectServerError.PIPED_INPUT_STREAM_PARAMETER_ERROR_CLIENT);
                }
                throw object_server_exception;
            }
            if (class_name.compareTo(ObjectServerHeader.JAVA_OBJECTOUTPUTSTREAM_TYPE_KEY) == 0) {
                if (__getMessageResource() != null) {
                    object_server_exception = 
                        new ObjectServerException( 
                        __getMessageResource().getFormattedMessage(
                            ObjectServerError.OBJECT_INPUT_STREAM_PARAMETER_ERROR_CLIENT,
                            __getMessageResource().getMessage(
                                ObjectServerError.OBJECT_INPUT_STREAM_PARAMETER_ERROR_CLIENT)));
                } else {
                    object_server_exception =
                        new oserv.ObjectServerException(
                            ObjectServerError.OBJECT_INPUT_STREAM_PARAMETER_ERROR_CLIENT);
                }
                throw object_server_exception;
            }
        }

        return true;
    }

    public String __getServiceCommandParameter(String command) throws Exception {
        String parameter_name;

        try {
            parameter_name = __getServiceCommandElement(command, 4);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_GETSERVICECOMMANDPARAMETER, 0, exception_object);
            throw exception_object;
        }
        parameter_name = __replace(parameter_name, ObjectServerHeader.VARIABLE_SEPARATOR,
            ObjectServerHeader.OBJECT_DOT_SEPARATOR);
        return parameter_name;
    }

    public String __getServiceCommandElement(String command, int index) 
        throws Exception {
        int          l;
        int          count;
        String[] element;
        String command_string;
        StringTokenizer     string_tokenizer_object;

        command_string = __replace(command, ObjectServerHeader.OBJECT_SEPARATOR 
            + ObjectServerHeader.OBJECT_SEPARATOR,
            ObjectServerHeader.OBJECT_SEPARATOR 
                + ObjectServerHeader.PADDING_STRING 
                + ObjectServerHeader.OBJECT_SEPARATOR);

        if (String.valueOf(command_string.charAt(command_string.length() - 1)).compareTo(
            ObjectServerHeader.OBJECT_SEPARATOR) == 0) {
            command_string = command_string.concat(ObjectServerHeader.PADDING_STRING);
        }
        string_tokenizer_object = new StringTokenizer(command, ObjectServerHeader.OBJECT_SEPARATOR);
        count = string_tokenizer_object.countTokens();

        element = new String[count];
        l = 0;
        while (string_tokenizer_object.hasMoreTokens()) {
            element[l] = string_tokenizer_object.nextToken();
            if (element[l].compareTo(ObjectServerHeader.BLANK_STRING) == 0) {
                element[l] = null;
            }
            l++;
        }
        if (l <= index) {
            ObjectServerException   object_server_exception;
            if (__getMessageResource() != null) {
                object_server_exception = 
                    new ObjectServerException( 
                        __getMessageResource().getFormattedMessage(
                            ObjectServerError.PARAMETER_ERROR_CLIENT,
                            __getMessageResource().getMessage(
                                ObjectServerError.PARAMETER_ERROR_CLIENT)));
            } else {
                object_server_exception 
                = new oserv.ObjectServerException(
                    ObjectServerError.PARAMETER_ERROR_CLIENT );
            }
            throw object_server_exception;
        }
        return element[index];
    }

    public static String __replace(String s0, String s1, String s2) {
        int    idx;
        String s3;

        if (s0 == null || s1 == null || s2 == null) {
            return s0;
        }
        s3 = s0;
        for (int i = 0; i < s0.length(); i++) {
            idx = s3.indexOf(s1, i);
            if (idx >= 0) {
                s3 = s3.substring(0, idx) + 
                    s2 +
                    s3.substring(idx+s1.length());
            }
        }
        return s3;
    }

    /**
     * This method initializes property values based on default, System and 
     * initial file (oserv.properties).
     *
     * @param     nothing
     * @return    void
     */
    protected void __getProperties() {
        FileInputStream fin;
        String ival;
        String path;
        String fpath;
        File   fes = null;
        StringTokenizer stk;
        boolean found = false;
        String pName, vName;
        Enumeration en;
        Properties pr;
        String packname;
        String workpath;
        String logfile;
        ObjectServerProperties ospr;
        String debug_level_string;

        debug_level_string = System.getProperties().getProperty(
            ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY );
        if (debug_level_string != null) {
            try {
                __setDebug(Integer.valueOf(debug_level_string).intValue());
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_GETPROPERTIES, 0, exception_object);
            }

            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_DEBUG_VALUE, 
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_DEBUG_VALUE), 
                        String.valueOf(__getDebug()))); 
            }
        }
        packname = getClass().getPackage().getName();
        packname = ObjectServerHeader.PACKAGE_NAME;

        try {
            fpath = packname + File.separator + ObjectServerHeader.OSERV_PROPERTIES;
            fes = new File(fpath);
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTIES_FILE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTIES_FILE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        fpath,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(fes.exists())));
            }
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_GETPROPERTIES, 0, exception_object);
            fes = null;
        }
        if (fes == null || fes.exists() == false) {
            for (int i = 0; i < __pathlist.length; i++ ) {
                ival = System.getProperty(__pathlist[i]);
                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                    __getLogStream().printLogln( 
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_LIBRARY_PATH_KEY,
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_LIBRARY_PATH_KEY),
                            __pathlist[i]));
                }

                stk = new StringTokenizer(ival, ObjectServerHeader.LIBRARY_SEPARATOR);
                while(stk.hasMoreTokens()) {
                    path = stk.nextToken();
                    if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                        __getLogStream().printLogln(
                            __getMessageResource().getFormattedMessage(
                                ObjectServerHeader.DEBUG_LIBRARY_ENTRY_KEY,
                                __getMessageResource().getMessage(
                                    ObjectServerHeader.DEBUG_LIBRARY_ENTRY_KEY),
                                __DEBUG_LIBRARIES, 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                path));
                    }
                    fes = new File(path);
                    if (fes.isDirectory() == false) {
                        String jar_suffix_string;
                        int    jar_suffix_index;
                        jar_suffix_index = path.lastIndexOf(ObjectServerHeader.SUFFIX_SEPARATOR);

                        if (jar_suffix_index >= 0) {
                            jar_suffix_string = path.substring(jar_suffix_index+1);
                            if (jar_suffix_string.compareTo(ObjectServerHeader.JAR_SUFFIX) == 0) {
                                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                                    __getLogStream().printLogln( 
                                        __getMessageResource().getFormattedMessage(
                                        ObjectServerDebug.DEBUG_JAR_NAME,
                                            __getMessageResource().getMessage(
                                            ObjectServerDebug.DEBUG_JAR_NAME),
                                            path ));
                                 }
                                 try {
                                     if (__sproperties == null) {
                                         __sproperties = new ObjectServerProperties();
                                         if (__getDebug() > 0) {
                                             __sproperties.setDebug(__getDebug(), __getLogStream());
                                         }
                                         __sproperties.setMessageResource(__getMessageResource());
                                     }
                                     __loadPropertyFromJar(path);
                                 } catch (Exception exception_object) {
                                     __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_GETPROPERTIES, 0, exception_object);
                                 }
                            }
                        }
                        continue;
                    }
                    fpath = path + File.separator + 
                        packname + File.separator + ObjectServerHeader.OSERV_PROPERTIES;
                    fes = new File(fpath);
                    if (fes.exists() == true) {
                        found = true;
                        break;
                    }
                }
                if (found == true) {
                    break;
                }
            }
        }
        if (fes != null && fes.exists() == true) {
            ospr = new ObjectServerProperties();

            fin = null;
            try {
                fin = new FileInputStream(fes);
            } catch (Exception exception_object) {
                __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_GETPROPERTIES, 0, exception_object);
                return;
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTIES_FILE,
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTIES_FILE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        fes.getPath()));
            }
            if (fin != null) {
                try {
                    if (__getDebug() > 0) {
                        ospr.setDebug(__getDebug(), __getLogStream());
                    }
                    ospr.setProperties((InputStream)fin);
                    __sproperties = ospr;
                    if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                        __getLogStream().printLogln( 
                            __getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_END_KEY,
                                __getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_END_KEY),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                __DEBUG_LOAD_PROPERTIES ));
                    }
                } catch (Exception exception_object) {
                    __printExceptionMessage(__TRACE_CLASSNAME,
                            __TRACE_GETPROPERTIES, 0, exception_object);
                    return;
                }
            }
            return;
        }
        try {
            __sproperties = new ObjectServerProperties();
            if (__getDebug() > 0) {
                __sproperties.setDebug(__getDebug(), __getLogStream());
            }
            __sproperties.setMessageResource(__getMessageResource());
            __sproperties.setProperties(null);
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln( __DEBUG_SETUP_PROPERTIES );
            }
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME, __TRACE_GETPROPERTIES, 0, exception_object);
            return;
        }
    }

    /**
     *
     * @param     String     : jar file name
     * @return    void
     * @exception Exception  : 
     */
    public void __readPropertyFromJar(String jarfile) throws Exception {
        int idx;
        int pdx;
        String path;
        String ival;
        String target;
        String vstring;
        String properties_name;
        DataInputStream dis;
        ObjectServerProperties pr;
        ByteArrayInputStream bis;
        StringTokenizer stk;

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && __getLogStream() != null) {
            __getLogStream().printLogln( 
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    __DEBUG_READ_PROPERTIES_FROM_JAR,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        if (jarfile == null) {
            return ;
        }
        properties_name = ObjectServerHeader.PACKAGE_NAME+File.separator+ObjectServerHeader.OSERV_PROPERTIES;
        for (int i = 0; i < __pathlist.length; i++ ) {

            ival = System.getProperty(__pathlist[i]);
            if (ival == null) {
                continue;
            }
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                __getLogStream().printLogln( 
                    __getMessageResource().getFormattedMessage(
                        __DEBUG_READ_PROPERTIES_FROM_JAR,
                        __getMessageResource().getMessage(
                            __DEBUG_READ_PROPERTIES_FROM_JAR),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        __pathlist[i] ));
            }
            stk = new StringTokenizer(ival, ObjectServerHeader.LIBRARY_SEPARATOR);
            while(stk.hasMoreTokens()) {
                path = stk.nextToken();
                idx = path.lastIndexOf(ObjectServerHeader.JAR_SUFFIX) ;
                if (idx < 0) {
                    continue;
                }
                pdx = path.lastIndexOf(File.separator);
                target = path.substring(pdx + 1);

                __loadPropertyFromJar(path);
            }
        }
        __initialize();
        return;
    }

    private void __loadPropertyFromJar(String jarfile) throws Exception {
        int idx;
        int pdx;
        String path;
        String ival;
        String target;
        String vstring;
        String properties_name;
        DataInputStream dis;
        ObjectServerProperties pr;
        ByteArrayInputStream bis;
        StringTokenizer stk;
        FileInputStream fis;
        JarInputStream  jis;
        ByteArrayOutputStream bos;
        byte[] buffer;
        int rcount;
        boolean tableflag;
        ZipEntry zipentry;

        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    __DEBUG_LOAD_PROPERTIES_FROM_JAR,
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        if (jarfile == null) {
            return ;
        }
        if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
            __getLogStream().printLogln(
                __getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_JAR_NAME, 
                    __getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_JAR_NAME), 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    jarfile ));
        }
        properties_name = ObjectServerHeader.PACKAGE_NAME+ObjectServerHeader.directory_separator_key+ObjectServerHeader.OSERV_PROPERTIES;
        buffer = new byte[ObjectServerHeader.BUFFER_SIZE];
        bos = new ByteArrayOutputStream();
        try {
            fis = new FileInputStream(jarfile);
            jis = new JarInputStream(fis);
        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME,
                __TRACE_LOADPROPERTYFROMJAR, 0, exception_object);
            return;
        }
        zipentry = jis.getNextEntry();
        while (zipentry != null) {
            if (zipentry.getName().compareTo( properties_name ) == 0 ||
                zipentry.getName().compareTo(ObjectServerHeader.OSERV_PROPERTIES) == 0) {
                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null && __getMessageResource() != null) {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_ENTRY_NAME, 
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_ENTRY_NAME), 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            zipentry.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            properties_name));
                }
                while (true) {
                    try {
                        rcount = jis.read(buffer, 0, ObjectServerHeader.BUFFER_SIZE);
                    } catch (Exception exception_object) {
                        __printExceptionMessage(__TRACE_CLASSNAME,
                            __TRACE_LOADPROPERTYFROMJAR, 0, exception_object);
                        break;
                    }
                    if (rcount == -1) {
                        break;
                    }
                    bos.write(buffer, 0, rcount);
                }
                break;
            }
            if (bos.size() > 0) {
                break;
            }
            zipentry = jis.getNextEntry();
        }
        if ( bos.size() == 0 ) {
            return;
        }
        buffer = bos.toByteArray();
        try {
            bis = new ByteArrayInputStream(buffer);
            dis = new DataInputStream(bis);
            if (__sproperties == null) {
                __sproperties = new ObjectServerProperties();
                if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && __getLogStream() != null) {
                    __getLogStream().printLogln(
                        __getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_END_KEY, 
                            __getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_END_KEY), 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            __DEBUG_CREATE_PROPERTIES,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            __sproperties.toString()));
                }
            }
            __sproperties.loadSpecificProperties((InputStream)dis, true);
            if (__getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && __getLogStream() != null) {
                __getLogStream().printLogln(
                    __getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_END_KEY, 
                        __getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_END_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        __DEBUG_LOAD_PROPERTIES));
            }

        } catch (Exception exception_object) {
            __printExceptionMessage(__TRACE_CLASSNAME,
                __TRACE_LOADPROPERTYFROMJAR, 0, exception_object);
            throw exception_object;
        }
    }

    public void transmitArchiveFile(String jar_path,
        ObjectOutputStream object_output_stream) throws Exception {
        File   fd;
        Exception   exception_object;

        if (jar_path == null) {
            if (__getMessageResource() != null) {
                exception_object = new Exception( 
                    __getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_CLIENT)));
            } else {
                exception_object = new Exception( 
                    ObjectServerError.PARAMETER_ERROR_CLIENT );
            }
            throw exception_object;
        }
        if (object_output_stream == null) {
            if (__getMessageResource() != null) {
                exception_object = new Exception( 
                    __getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_CLIENT)));
            } else {
                exception_object = new Exception( 
                    ObjectServerError.PARAMETER_ERROR_CLIENT );
            }
            throw exception_object;
        }

        fd = new File(jar_path);
        if (fd.exists() == false) {
            if (__getMessageResource() != null) {
                exception_object = new Exception( 
                    __getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_CLIENT,
                        __getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_CLIENT)));
            } else {
                exception_object = new Exception( 
                    ObjectServerError.PARAMETER_ERROR_CLIENT );
            }
            throw exception_object;
        }
    }
}
