/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectCommand.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/03/13
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ObjectCommand extends ObjectServerHeader {

    public final static String TRACE_CLASSNAME
        = "oserv.ObjectCommand";
    public final static String TRACE_CONNECTSERVER 
        = "connectServer";

    public ObjectCommand() {
        super();

        String value_string;

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
/**
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
*/
        }
    }
   
    public Socket connectServer(String host_string, int iport) 
                                            throws Exception {
        Socket                    connection_socket;
        InetSocketAddress         inet_socket_address;
        RetryObjectInputStream    retry_object_input_stream;
        OutputStream              output_stream;
        InputStream               input_stream;
        ObjectOutputStream        object_output_stream;
        ObjectServerException     object_server_exception;

        if (host_string == null || iport <= 0) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_COMMAND,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_COMMAND)));
            throw object_server_exception;
        }

        inet_socket_address = new InetSocketAddress(host_string, iport);
        connection_socket = new Socket();

        try {
            connection_socket.connect(inet_socket_address);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_CONNECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerError.JAVA_SOCKET_CONNECT_IO_EXCEPTION)));
            throw object_server_exception;
        }
        return connection_socket;
    }
}
