/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectGenerator.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/07/23
 *             2.1 2010/10/06 add specification for an interface 
 *                            that is in current directory.
 *                            be corrected for delete message of stacktrace
 *                            when exception happens.
 *                            be deleted directory that is made by this.
 *             2.1 2010/10/08
 *             2.1 2010/11/04 add exit procedure for property error
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

import oserv.*;

/**
 * This class provides a feature to generate user stub class
 *
 * @see oserv.ObjectServerHeader
 */
public class ObjectGenerator extends ObjectServerHeader {
    public final static String PWD_ENV_KEY = "PWD";
    public final static String UNIX_PATH_SEPARATOR = "/";
    public final static String DOS_PATH_SEPARATOR = "\\";
    public final static String EMPTY_PATH = "";

    public final static String OBJECT_GENERATOR_ARGS
        = "";

    public final static String CURRENT_DIRECTORY = ".";
    public final static String SEMICOLON = ";";

    public final static String LOCK_PREFIX = ".";

    public final static String PARAMETER_ERROR 
        = "Parameter error : ";

    public final static String JAVA_LANG_THROWABLE = "java.lang.Throwable";
    public final static String JAVA_LANG_EXCEPTION = "java.lang.Exception";
    public final static String JAVA_LANG_STRING    = "java.lang.String";

    public final static String SYSTEM_PREFIX = "__SYSTEM_";

    public final static String JAVA_EXCEPTION_NAME
        = "java.lang.Exception";
    public final static String OBJECT_SERVER_EXCEPTION_NAME
        = "oserv.ObjectServerException";
    public final static String OBJECT_SERVER_SERVICE_UNAVAILABEL_EXCEPTION
        = "oserv.ObjectServerServiceUnavailableException";

    public final static String PACKAGE_NAME_WAS_UNMATCHED
        = "Package name was unmatched.";
    public final static String PACKAGE_WAS_NOT_SPECIFIED
        = "Package wasn't specified.";
    public final static String REAL_CLASS_WAS_SPECIFIED
        = "Real class was specified as interface.";
    public final static String DIRECTORY_WAS_NOT_FOUND
        = "Directory wasn't found.";
    public final static String WARNNING_MESSAGE 
        = "Warnning : this creates directory ";

    public final static String SOURCE_DEBUG_FUNCTION_SUFFIX 
        = "DEBUG";
    public final static String SOURCE_BODY_FUNCTION_SUFFIX
        = "";
    public final static String SOURCE_P_HEAD = "p";
    public final static String SOURCE_D_HEAD = "d";

    public final static String SOURCE_HEADER1
        = "/**";
    public final static String SOURCE_HEADER2
        = " * This file is generated by ObjectGenerator";
    public final static String SOURCE_HEADER3
        = " * @Version    ";
    public final static String SOURCE_HEADER4
        = " */";
 
    public final static String SOURCE_COMMENT 
        = "/* This file is generated by ObjectGenerator */";
    public final static String SOURCE_PACKAGE = "package ";
    public final static String SOURCE_IMPORT_JAVA_LANG_WILDCARD
        = "import java.lang.*;";
    public final static String SOURCE_IMPORT_JAVA_NET_WILDCARD
        = "import java.net.*;";
    public final static String SOURCE_IMPORT_JAVA_LANG_REFLECT_WILDCARD
        = "import java.lang.reflect.*;";
    public final static String SOURCE_IMPORT_JAVA_IO_WILDCARD
        = "import java.io.*;";
    public final static String SOURCE_IMPORT_OSERV_OBJECTCLIENT
        = "import oserv.ObjectClient;";
    public final static String SOURCE_BLANK = " ";
    public final static String SOURCE_DOLLAR = "$";
    public final static String SOURCE_EMPTY = "";
    public final static String SOURCE_UNDERSCORE = "_";
    public final static String SOURCE_SLASH = "/";
    public final static String SOURCE_CARRIAGE_RETURN = "\n";
    public final static String SOURCE_DOUBLE_QUOTE = "\"";
    public final static String SOURCE_DOT   = ".";
    public final static String SOURCE_COMMA = ",";
    public final static String SOURCE_START_BRACE = "(";
    public final static String SOURCE_END_BRACE = ")";
    public final static String SOURCE_EQUAL = "=";
    public final static String SOURCE_NEW = "new";
    public final static String SOURCE_SEMICOLON   = ";";
    public final static String SOURCE_START_MIDDLE_BRACE = "{";
    public final static String SOURCE_END_MIDDLE_BRACE = "}";
    public final static String SOURCE_BLANK_P = " p";
    public final static String SOURCE_TAB = "    ";

    public final static String SOURCE_PUBLIC_CLASS = "public class ";
    public final static String SOURCE_EXTENDS_OBJECTCLIENT
        = " extends oserv.ObjectClient implements ";
    public final static String SOURCE_SERIALIZABLE = " { ";

    public final static String SOURCE_PUBLIC_STATIC_STRING_CMD
        = "    public static String cmd;";
    public final static String SOURCE_PUBLIC_STATIC_TYPE
        = "    public static ";
    public final static String SOURCE_STRING_TYPE = "java.lang.String";
    public final static String SOURCE_UNDERSCORE_P = "_p";
    public final static String SOURCE_UNDERSCORE_D = "_d";
    public final static String SOURCE_EQUAL_P = "=p";

    public final static String SOURCE_BLANK_PRIVATE = "    private ";
    public final static String SOURCE_BLANK_PUBLIC = "    public ";
    public final static String SOURCE_BLANK_THROWS = "    throws ";
    public final static String SOURCE_BLANK_START_MIDDLE_BRACE = "    {";


    public final static String SOURCE_BLANK_OBJECT_RT
        = "        Object rt = new Object();";
    public final static String SOURCE_BLANK_CMD_NEW_STRING
        = "        cmd = new String(";
    public final static String SOURCE_BLANK_RETURN = "        return;";
    public final static String SOURCE_BLANK_TRY = "        try {";

    public final static String SOURCE_FIELD_FROM_GETCLASS
        = "        Field[] fld = getClass().getFields();";

    public final static String SOURCE_RETURN_CONNECT_VOID
        = "            __connect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_CONNECT
        = "            rt = __connect(cmd, fld, (Object)this);";


    public final static String SOURCE_RETURN_INT_CONNECT1
        = "            int rt_int;";
    public final static String SOURCE_RETURN_INT_CONNECT2
        = "            rt_int = __intconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_INT_CONNECT3
        = "            return rt_int;";

    public final static String SOURCE_RETURN_LONG_CONNECT1
        = "            long rt_long;";
    public final static String SOURCE_RETURN_LONG_CONNECT2
        = "            rt_long = __longconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_LONG_CONNECT3
        = "            return rt_long;";

    public final static String SOURCE_RETURN_FLOAT_CONNECT1
        = "            float rt_float;";
    public final static String SOURCE_RETURN_FLOAT_CONNECT2
        = "            rt_float = __floatconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_FLOAT_CONNECT3
        = "            return rt_float;";

    public final static String SOURCE_RETURN_DOUBLE_CONNECT1
        = "            double rt_double;";
    public final static String SOURCE_RETURN_DOUBLE_CONNECT2
        = "            rt_double = __doubleconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_DOUBLE_CONNECT3
        = "            return rt_double;";

    public final static String SOURCE_RETURN_SHORT_CONNECT1
        = "            short rt_short;";
    public final static String SOURCE_RETURN_SHORT_CONNECT2
        = "            rt_short = __shortconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_SHORT_CONNECT3
        = "            return rt_short;";

    public final static String SOURCE_RETURN_BYTE_CONNECT1
        = "            byte rt_byte;";
    public final static String SOURCE_RETURN_BYTE_CONNECT2
        = "            rt_byte = __byteconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_BYTE_CONNECT3
        = "            return rt_byte;";

    public final static String SOURCE_RETURN_CHAR_CONNECT1
        = "            char rt_char;";
    public final static String SOURCE_RETURN_CHAR_CONNECT2
        = "            rt_char = __charconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_CHAR_CONNECT3
        = "            return rt_char;";

    public final static String SOURCE_RETURN_BOOLEAN_CONNECT1
        = "            boolean rt_boolean;";
    public final static String SOURCE_RETURN_BOOLEAN_CONNECT2
        = "            rt_boolean = __booleanconnect(cmd, fld, (Object)this);";
    public final static String SOURCE_RETURN_BOOLEAN_CONNECT3
        = "            return rt_boolean;";

    public final static String SOURCE_CATCH_INVOCATION_TARGET_EXCEPTION
        = "        } catch (java.lang.reflect.InvocationTargetException exception_object) {";

    public final static String SOURCE_CATCH_EXCEPTION
        = "        } catch (Exception exception_object) {";

    public final static String THROW_OBJECT_SERVER_EXCEPTION
        = "oserv.ObjectServerException ose = new oserv.ObjectServerException(exception_object);"
        + "throw ose;";

    public final static String IF_INSTANCE_SERIVCE_UNAVAILABLE_EXCEPTION1
        = "            if (exception_object instanceof oserv.ObjectServerServiceUnavailableException) {  throw (oserv.ObjectServerServiceUnavailableException)exception_object; }" ;
    public final static String IF_INSTANCE_SERIVCE_UNAVAILABLE_EXCEPTION2
        = "            if (exception_object instanceof oserv.ObjectServerServiceUnavailableException) {  throw (Exception)exception_object; }" ;

    public final static String IF_INSTANCE_INVOKE
        = "if (exception_object instanceof java.lang.reflect.InvocationTargetException) { throw exception_object; }";

    public final static String IF_INSTANCE_INVOKE_HEAD
        = "if (exception_object instanceof java.lang.reflect.InvocationTargetException) {";

    public final static String IF_INSTANCE_RETURN_HEAD
        = "if (rt instanceof java.lang.reflect.InvocationTargetException) {";

    public final static String IF_INSTANCE_INVOKE_EXCEPTION_OBJECT_THROWABLE 
        = "(java.lang.Throwable)exception_object";
    public final static String IF_INSTANCE_INVOKE_EXCEPTION_OBJECT 
        = "(java.lang.Exception)exception_object";
    public final static String IF_INSTANCE_INVOKE_EXCEPTION_MESSAGE
        = "exception_object.getMessage()";
    public final static String IF_INSTANCE_INVOKE_RETURN_OBJECT = "rt";

    public final static String SOURCE_THROWABLE_CAUSE 
        = "            Throwable cause;";
    public final static String SOURCE_CAUSE_EP 
        = "            cause = exception_object;";
    public final static String SOURCE_WHILE_CAUSE 
        = "            while(cause != null) {";

    public final static String SOURCE_IF_START_KEY 
        = "                if (";
    public final static String SOURCE_COMPARE_CAUSE
        = ".compareTo(cause.getClass().getName()) == 0";
    public final static String SOURCE_IF_END_KEY = ") {";

    public final static String SOURCE_IF_START_CAUSE_KEY 
        = "                if (cause.getClass().getName().compareTo(";
    public final static String SOURCE_IF_END_CAUSE_KEY 
        = ") == 0) {";
    public final static String SOURCE_EXCEPTION_DEFINITION
        = "    public final static String ";

    public final static String SOURCE_CAUSE_GET_CAUSE 
        = "            cause = cause.getCause();";
    public final static String SOURCE_BRACKET = "}";

    public final static String IF_INSTANCE_INVOKE_TAIL
        = "}";

    public final static String SOURCE_IF_INSTANCE_HEAD
        = "if (exception_object instanceof ";
    public final static String SOURCE_IF_INSTANCE_TAIL
        = ") ";

    public final static String SOURCE_THROW_EXCEPTION_END = "throw exception_object;";
    public final static String THROW_EXCEPTION = "{throw exception_object;}";
    public final static String THROW_EXCEPTION_HEAD = "{throw (";
    public final static String THROW_EXCEPTION_TAIL = ")exception_object;}";

    public final static String PRINT_THROW_EXCEPTION_HEAD = "throw (";
    public final static String PRINT_THROW_EXCEPTION_TAIL = ")cause;";

    public final static String SOURCE_PRINT_STACK_TRACE
        = "            exception_object.printStackTrace();";
    public final static String SOURCE_BLANK_THROW_BLANK
        = "            throw ";
    public final static String SOURCE_BLANK_RETURN_BLANK
        = "            return ";
    public final static String SOURCE_BLANK_RETURN_NEW 
        = "            return new ";
    public final static String SOURCE_BLANK_RETURN_NULL_HEAD 
        = "            return (";
    public final static String SOURCE_BLANK_RETURN_NULL_TAIL = ")null;";

    public final static String SOURCE_BLANK_OBJECT     = "java.lang.Object";
    public final static String SOURCE_BLANK_RETURN_INT = "            return (int)-1; ";
    public final static String SOURCE_BLANK_RETURN_LONG = "            return (long)-1; ";
    public final static String SOURCE_BLANK_RETURN_FLOAT = "            return (float)-1.0; ";
    public final static String SOURCE_BLANK_RETURN_DOUBLE = "            return (double)-1.0; ";
    public final static String SOURCE_BLANK_RETURN_BOOLEAN = "            return false; ";
    public final static String SOURCE_BLANK_RETURN_BYTE = "            return (byte)-1; ";
    public final static String SOURCE_BLANK_RETURN_CHAR = "            return (char)-1; ";
    public final static String SOURCE_BLANK_RETURN_SHORT = "            return (short)-1; ";
    public final static String SOURCE_BLANK_RETURN_NEW_INTEGER = "            return new java.lang.Integer(-1);";
    public final static String SOURCE_BLANK_RETURN_NEW_LONG = "            return new java.lang.Long(-1);";
    public final static String SOURCE_BLANK_RETURN_NEW_FLOAT = "            return new java.lang.Float((float)-1.0);";
    public final static String SOURCE_BLANK_RETURN_NEW_DOUBLE = "            return new java.lang.Double(-1.0);";
    public final static String SOURCE_BLANK_RETURN_NEW_BOOLEAN = "            return new java.lang.Boolean(false);";
    public final static String SOURCE_BLANK_RETURN_NEW_BYTE = "            return new java.lang.Byte(-1);";
    public final static String SOURCE_BLANK_RETURN_NEW_CHAR = "            return new java.lang.Character(-1);";
    public final static String SOURCE_BLANK_RETURN_NEW_SHORT = "            return new java.lang.Short(-1);";
    public final static String SOURCE_FUNCTION_CALL = "();";
    public final static String SOURCE_INSTANCEOF
        = "        if (rt instanceof ";
    public final static String SOURCE_THROW_START_BRACE
        = "            throw (";
    public final static String SOURCE_RT = ")rt;";
    public final static String SOURCE_RT_NULL_INSTANCE_RETURN_NULL
        = "if (rt == null) { return null; }";
    /** int **/
    public final static String SOURCE_INT_INSTANCE 
        = "if (rt instanceof Integer) { return ((Integer)rt).intValue(); }";
    public final static String SOURCE_RT_NULL_INT_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_INT + " }";
    public final static String SOURCE_INT_RT = ")Integer.valueOf(rt.toString()).intValue();";
    /**  long **/
    public final static String SOURCE_LONG_INSTANCE 
        = "if (rt instanceof Long) { return ((Long)rt).longValue(); }";
    public final static String SOURCE_RT_NULL_LONG_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_LONG + " }";
    public final static String SOURCE_LONG_RT = ")Long.valueOf(rt.toString()).longValue();";
    /**  float **/
    public final static String SOURCE_FLOAT_INSTANCE 
        = "if (rt instanceof Float) { return ((Float)rt).floatValue(); }";
    public final static String SOURCE_RT_NULL_FLOAT_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_FLOAT + " }";
    public final static String SOURCE_FLOAT_RT = ")Float.valueOf(rt.toString()).floatValue();";
    /**  double **/
    public final static String SOURCE_DOUBLE_INSTANCE 
        = "if (rt instanceof Double) { return ((Double)rt).doubleValue(); }";
    public final static String SOURCE_RT_NULL_DOUBLE_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_DOUBLE + " }";
    public final static String SOURCE_DOUBLE_RT = ")Double.valueOf(rt.toString()).doubleValue();";

    /**  boolean **/
    public final static String SOURCE_BOOLEAN_INSTANCE 
        = "if (rt instanceof Boolean) { return ((Boolean)rt).booleanValue(); }";
    public final static String SOURCE_RT_NULL_BOOLEAN_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_BOOLEAN + " }";
    public final static String SOURCE_BOOLEAN_RT = ")Boolean.valueOf(rt.toString()).booleanValue();";

    /**  byte **/
    public final static String SOURCE_BYTE_INSTANCE 
        = "if (rt instanceof Byte) { return ((Byte)rt).byteValue(); }";
    public final static String SOURCE_RT_NULL_BYTE_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_BYTE + " }";
    public final static String SOURCE_BYTE_RT = ")Byte.valueOf(rt.toString()).byteValue();";

    /**  char **/
    public final static String SOURCE_CHARACTER_INSTANCE 
        = "if (rt instanceof Character) { return ((Character)rt).charValue(); }";
    public final static String SOURCE_RT_NULL_CHAR_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_CHAR + " }";
    public final static String SOURCE_CHAR_RT = ")Integer.valueOf(rt.toString()).intValue();";


    /**  short **/
    public final static String SOURCE_SHORT_INSTANCE 
        = "if (rt instanceof Short) { return ((Short)rt).shortValue(); }";
    public final static String SOURCE_RT_NULL_SHORT_INSTANCE_RETURN_NULL
        = "if (rt == null) { " + SOURCE_BLANK_RETURN_SHORT + " }";
    public final static String SOURCE_SHORT_RT = ")Short.valueOf(rt.toString()).shortValue();";

    public final static String SOURCE_BLANK_RETURN_BRACE
        = "        return (";

    public final static String DEBUG_PACKAGE_NAME = "Package name ";
    public final static String DEBUG_BLANK = " ";
    public final static String DEBUG_DEBUG = "DEBUG ";
    public final static String DEBUG_VAR_HEAD = "Var head ";
    public final static String DEBUG_CURRENT_DIRECTORY_4
        = "Current Directory 4 ";
    public final static String DEBUG_CURRENT_DIRECTORY_3
        = "Current Directory 3 ";
    public final static String DEBUG_CURRENT_DIRECTORY_2
        = "Current Directory 2 ";
    public final static String DEBUG_CURRENT_DIRECTORY_1
        = "Current Directory 1 ";
    public final static String DEBUG_COPY_INTERFACE_CLASS
        = "copyInterfaceClass() ";
    public final static String DEBUG_SET_INTERFACE_CLASS
        = "setInterfaceClass() ";
    public final static String DEBUG_CLASS_FILE_NAME
        = "Class file name ";
    public final static String DEBUG_SOCKET_EXCEPTION
        = "Socket Exception ";
    public final static String DEBUG_GETINTERFACE
        = "getInterfaceClass(java.lang.String) ";
    public final static String DEBUG_GENSTUB_CLASS = "genStubClass() ";
    public final static String DEBUG_URL_PATH = "URL path ";
    public final static String DEBUG_URL_ERROR = "URL Error ";
    public final static String DEBUG_GET_CLASS = "Get Class ";

    public final static String TRACE_MAIN = "main";
    public final static String TRACE_COPYINTERFACECLASS = "copyInterfaceClass";
    public final static String TRACE_CLASSNAME    = "oserv.ObjectGenerator";
    public final static String TRACE_GENSTUBCLASS = "genStubClass";
    public final static String TRACE_SETINTERFACECLASS = "setInterfaceClass";
    public final static String TRACE_REMOVEINTERFACECLASS = "removeInterfaceClass";
    public final static String TRACE_CHECKINTERFACECLASS = "checkInterfaceClass";

    public final static String USAGE_ARGUMENT = "<Interface Class> <Object Class>";
    public final static String INTERFACE_CLASS_NOT_FOUND
        = "Interface class not found.";
    public final static String BLANK_MESSAGE = " ";

    private final static String COMMAND_NAME = "ObjectGenerator";

    public final static int BUFFER_SIZE = 4096;

    private String pwd = null;

    private ArrayList exception_array;

    private static LogPrintStream logStream = null;

    private static boolean add_debug_function = false;

    /**
     * This constructor initializes properties
     */
    public ObjectGenerator() {
        super();
        int port_number;
        getProperties();

        String vstring;
        String value_string;
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }

        vstring = getPropertyValue(OSERV_DEBUG_LEVEL_KEY);
        if (vstring != null) {
            try {
                setDebug(Integer.valueOf(vstring).intValue());
            } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage(
                    ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_GENERATOR)));
                System.exit(ERROR_EXIT_CODE);
            }
        }
        vstring = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        if (vstring == null || vstring.length() == 0) {
            System.err.println( getMessageResource().getFormattedMessage(
                ObjectServerError.HOST_CONFIGURATION_ERROR_GENERATOR,
                getMessageResource().getMessage(
                    ObjectServerError.HOST_CONFIGURATION_ERROR_GENERATOR)));
            System.exit(ERROR_EXIT_CODE);
        }
        vstring = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (vstring != null) {
            try {
                port_number = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage(
	            ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR)));
                System.exit(ERROR_EXIT_CODE);
            }
	} else {
            System.err.println(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR)));
            System.exit(ERROR_EXIT_CODE);
        }
        vstring = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (vstring != null) {
            try {
                port_number = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage(
	            ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR)));
                System.exit(ERROR_EXIT_CODE);
            }
	} else {
            System.err.println( getMessageResource().getFormattedMessage(
	        ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR,
                getMessageResource().getMessage(
                    ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR)));
            System.exit(ERROR_EXIT_CODE);
        }
        vstring = getPropertyValue(OSERV_DEBUG_FUNCTION_KEY);
        if (vstring != null) {
            vstring = vstring.toLowerCase();
            if (OSERV_DEBUG_FUNCTION_ON_KEY.compareTo(vstring) == 0) {
                add_debug_function = true;
            } else if (OSERV_DEBUG_FUNCTION_OFF_KEY.compareTo(vstring) == 0) {
                add_debug_function = false;
            } else {
                System.err.println( getMessageResource().getFormattedMessage(
	            ObjectServerError.DEBUG_FUNCTION_ERROR_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.DEBUG_FUNCTION_ERROR_GENERATOR)));
                System.exit(ERROR_EXIT_CODE);
            }
        }
        /**
        if (logStream == null) {
            logStream = new LogPrintStream(System.out);
        }
        **/
        if (getLogStream() == null) {
            setLogStream(new LogPrintStream(System.out));
        }
        String separator_key = String.valueOf(java.io.File.separatorChar);
        if (separator_key.compareTo(DOS_DIRECTORY_SEPARATOR) == 0) {
            try {
                pwd = ObjectServerProperties.getDosCurrentDirectory();
            } catch (Exception exception_object) {
                pwd = ObjectServerProperties.DOS_WORK_DIRECTORY;
            }
        } else {
            pwd = System.getenv(PWD_ENV_KEY);
        }
    }

    public void printException(PrintStream pstream, String type_name) {
        pstream.println(PRINT_THROW_EXCEPTION_HEAD 
            + type_name + PRINT_THROW_EXCEPTION_TAIL);
    }

    public void printException2(PrintStream pstream, String type_name,
        String    message_string1,
        String    exception_object, 
        String    throwable_object,
        String    message_string2) {
        if (message_string1 != null) {
            if (throwable_object != null) {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                    + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                    + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                    + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                    + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                    + message_string1 + SOURCE_COMMA 
                    + throwable_object + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            } else if (exception_object != null) {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                    + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                    + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                    + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                    + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                    + message_string1 + SOURCE_COMMA 
                    + exception_object + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            }
        } else if (message_string2 != null) {
            if (throwable_object != null) {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                + throwable_object + SOURCE_COMMA 
                + message_string2 + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            } else if (exception_object != null) {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                + exception_object + SOURCE_COMMA 
                + message_string2 + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            }
        } else {
            if (throwable_object != null) {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                + throwable_object + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            } else if (exception_object != null) {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                + exception_object + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            } else {
                pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_TAB 
                + SOURCE_TAB + SOURCE_TAB + type_name + SOURCE_BLANK 
                + replace(type_name.toLowerCase(), SOURCE_DOT, SOURCE_UNDERSCORE)
                + SOURCE_BLANK + SOURCE_EQUAL + SOURCE_BLANK + SOURCE_NEW 
                + SOURCE_BLANK + type_name + SOURCE_START_BRACE 
                + SOURCE_END_BRACE + SOURCE_SEMICOLON);
            }
        }
    }
    /**
     * This method prints a usage.
     */
    public static void printUsage() {
        System.err.println(COMMAND_NAME + MESSAGE_SEPARATOR + USAGE_ARGUMENT);
    }

    public boolean checkException(String exception_string) {
        String entry;

        if (exception_string == null) {
            return false;
        }
        if (exception_array == null) {
            return false;
        }
        for (int i = 0; i < exception_array.size(); i++) {
            entry = (String)exception_array.get(i);
            if (entry != null && entry.compareTo(exception_string) == 0) {
                return true;
            }
        }
        return false;
    }

    public void addException(String exception_string) {
        if (exception_string == null) {
            return ;
        }
        if (exception_array == null) {
            exception_array = new ArrayList();
        }
        exception_array.add((Object)exception_string);
    }
    /**
     * @param    String     - file name
     *           String     - interface name
     *           String     - class name
     * @thows    Exception
     */
    public void genStubClass(String filename, String iname, Class cls) throws Exception {
        Method[] mth;
        Field[] fld;
        File fdes;
        File ddes;
        String basename;
        String class_package_name;
        String pw_dir;
        PrintStream pstream;
        String dirname;
        String dirname2;
        String pname;
        String fname;
        String dname;
        String packname;
        String[] dpathname;
        String[] ppathname;
        Class  rtype;
        Class[] ptypes;
        Class[] etypes;
        String  parakey;
        String  excptkey;
        int idx;
        int idx2;
        int j;
        int mcount;
        boolean mflag;
        boolean exflag;
        boolean ssflag;
        String plist;
        String parahead;
        String excepthead;
        String varhead;
        String varhead_debug = new String();
        String function_head;
        String function_body;
        String function_debug;
        String return_variable_type;
        String return_variable_convert;
        String sentence;
        String sentence_debug = new String();

        if (filename == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.FILENAME_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.FILENAME_PARAMETER_NULL_GENERATOR)));
            throw ose;
        } 
        if (iname == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.INTERFACE_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.INTERFACE_PARAMETER_NULL_GENERATOR)));
            throw ose;
        } 
        if (cls == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.CLASS_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.CLASS_PARAMETER_NULL_GENERATOR)));
            throw ose;
        } 
        return_variable_type = null;
        return_variable_convert = null;

        mth = cls.getMethods();
        fld = getClass().getFields();

        pw_dir = pwd;
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln( getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_PWD_ENV_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_PWD_ENV_KEY), 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                pw_dir));
        }

        dname = pw_dir + File.separator ;

        if (cls.getPackage() == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PACKAGE_WAS_NOT_SPECIFIED_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.PACKAGE_WAS_NOT_SPECIFIED_GENERATOR)));
            throw ose;
        } 
        idx = iname.lastIndexOf(File.separator);
        if (idx < 0) {
            idx = pw_dir.lastIndexOf(File.separator);
            if (idx > 0) {
                pname = pw_dir.substring(idx + 1);
                dirname = pname;
                pw_dir = pw_dir.substring(0, idx);
            } else {
                ObjectServerException exception_object = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.DIRECTORY_WAS_NOT_FOUND_GENERATOR,
                        getMessageResource().getMessage(
                            ObjectServerError.DIRECTORY_WAS_NOT_FOUND_GENERATOR)));
                throw exception_object;
            }
        } else {
            dirname = iname.substring(0, idx);
            pname = replace(dirname, File.separator, OBJECT_DOT_SEPARATOR);

            dpathname = dirname.split(File.separator);
            ppathname = pw_dir.split(File.separator);
            if (dpathname.length <= ppathname.length) {
                for (j = 1; j <= dpathname.length; j++) {
                    if (dpathname[dpathname.length - j] == null ||
                        dpathname[dpathname.length - j].length() == 0) {
                        continue;
                    }
                    if (dpathname[dpathname.length - j].compareTo(ppathname[
					ppathname.length - j]) != 0) {
                        break;
                    }
                }
                if (j > dpathname.length) {
                    pw_dir = new String(SOURCE_EMPTY);
                    for (int k = 0; k < ppathname.length - dpathname.length; k++) {
                        pw_dir = pw_dir + File.separator + ppathname[k];
                    }
                }   
            }
        }
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln( 
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_PACKAGE_NAME,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_PACKAGE_NAME),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    iname,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    pname,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    dirname, 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    cls.getPackage().getName()));
        }
        basename = filename;
        if (pname.compareTo(cls.getPackage().getName()) == 0) {
            idx = filename.lastIndexOf(OBJECT_DOT_SEPARATOR);
            if (idx >= 0) {
                packname = filename.substring(0, idx);
                if (cls.getPackage().getName().compareTo(packname) == 0) {
                    dname = pw_dir + File.separator + dirname;
                    fname = dname + File.separator 
                        + filename.substring(idx + 1) 
                        + SUFFIX_SEPARATOR + JAVA_SUFFIX;
                    basename = filename.substring(idx+1);
                } else {
                    dname = pw_dir + File.separator + dirname;
                    fname = dname + File.separator 
                        + replace(filename, OBJECT_DOT_SEPARATOR, File.separator)
                        + SUFFIX_SEPARATOR + JAVA_SUFFIX;
                }
            } else {
                dname = pw_dir + File.separator + dirname;
                fname = dname + File.separator 
                    + filename + SUFFIX_SEPARATOR + JAVA_SUFFIX;
            }
        } else {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PACKAGE_NAME_WAS_UNMATCHED_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.PACKAGE_NAME_WAS_UNMATCHED_GENERATOR)));
            throw ose;
        }
        ddes = new File(dname);
        mcount = 0;
        if (ddes.exists() == false) {
            System.err.println(
                getMessageResource().getFormattedMessage(
                    WARNNING_MESSAGE,
                    getMessageResource().getMessage(
                        WARNNING_MESSAGE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    dname));
            mcount = 1;
            ddes.mkdir();
        }
        fdes = new File(fname);
        try {
            pstream = new PrintStream(fdes);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GENSTUBCLASS, 0, exception_object);
            if (mcount == 1) {
                ddes.delete();
            }
            throw exception_object;
        }

        pstream.println(SOURCE_HEADER1);
        pstream.println(SOURCE_HEADER2);
        Date date_object;
        date_object = new Date();
        pstream.println(SOURCE_HEADER3 + date_object.toString());
        pstream.println(SOURCE_HEADER4);

        if (cls.getPackage() != null) {
            pstream.println( SOURCE_EMPTY );
            pstream.println( SOURCE_PACKAGE + cls.getPackage().getName() 
            + SOURCE_SEMICOLON );
            pstream.println( SOURCE_EMPTY );
        }

        pstream.println( SOURCE_IMPORT_JAVA_LANG_WILDCARD );
        pstream.println( SOURCE_IMPORT_JAVA_NET_WILDCARD );
        pstream.println( SOURCE_IMPORT_JAVA_LANG_REFLECT_WILDCARD );
        pstream.println( SOURCE_IMPORT_JAVA_IO_WILDCARD );
        pstream.println( SOURCE_IMPORT_OSERV_OBJECTCLIENT );
        pstream.println( SOURCE_EMPTY);

        pstream.println( SOURCE_PUBLIC_CLASS + basename + 
                         SOURCE_EXTENDS_OBJECTCLIENT
                         + cls.getName() + 
                         SOURCE_SERIALIZABLE );

        ssflag = false;
        exflag = false;
        mflag = false;
        for (int i = 0; i < mth.length; i++) {
            parahead = new String(SOURCE_EMPTY);
            excepthead = new String(SOURCE_EMPTY);
            rtype = mth[i].getReturnType();
            try {
                return_variable_type = VariableType.valueOf(rtype.getName());
                return_variable_convert = VariableType.convert(rtype.getName());
            } catch (Exception exception_object) {

                System.err.println( getMessageResource().getFormattedMessage(
		    ObjectServerError.INTERNAL_ERROR_GENERATOR,
                    getMessageResource().getMessage(
		       ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                System.exit(ERROR_EXIT_CODE); 
            }
            ptypes = mth[i].getParameterTypes();
            for (j = 0; j < ptypes.length; j++) {
                if (j != 0) {
                    parahead = parahead.concat(SOURCE_UNDERSCORE);
                }
                try {
                    parahead = parahead.concat(
   	                VariableType.convert(ptypes[j].getName()));
                } catch (Exception exception_object) {
                    System.err.println( getMessageResource().getFormattedMessage( 
			 ObjectServerError.INTERNAL_ERROR_GENERATOR,
                         getMessageResource().getMessage(
			     ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            etypes = mth[i].getExceptionTypes();
            for (j = 0; j < etypes.length; j++) {
                if (j != 0) {
                    excepthead = excepthead.concat(SOURCE_UNDERSCORE);
                }
                excepthead = excepthead.concat(replace(
                    etypes[j].getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE));
            }
            varhead = replace(cls.getPackage().getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE +
                    basename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead;

	    
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_VARIABLE_VALUE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_VARIABLE_VALUE),
                        varhead));
            }
            for (j = 0; j < ptypes.length; j++) {
                try {
                    pstream.println(SOURCE_PUBLIC_STATIC_TYPE+
		        VariableType.valueOf(ptypes[j].getName()) +
                        SOURCE_BLANK + varhead + 
                        SOURCE_UNDERSCORE_P +
                        String.valueOf(j) + 
                        SOURCE_SEMICOLON);
	        } catch (Exception exception_object) {
                    System.err.println( getMessageResource().getFormattedMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR,
                        getMessageResource().getMessage(
                            ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                        System.exit(ERROR_EXIT_CODE);
                }
            }
            pstream.println( SOURCE_EMPTY );
            etypes = mth[i].getExceptionTypes();
            for (j = 0; j < etypes.length; j++) {
                if (checkException(etypes[j].getName()) == true) {
                    continue;
                }
                addException(etypes[j].getName());

                pstream.print(SOURCE_PUBLIC_STATIC_TYPE +
                    SOURCE_BLANK + JAVA_LANG_STRING + SOURCE_BLANK +
                    SYSTEM_PREFIX + replace(etypes[j].getName().toUpperCase(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_EQUAL);

                pstream.print(SOURCE_DOUBLE_QUOTE);
                pstream.print(etypes[j].getName());
                pstream.print(SOURCE_DOUBLE_QUOTE);

                pstream.println(SOURCE_SEMICOLON);
            }
	}
        pstream.println( SOURCE_EMPTY );
        pstream.println( SOURCE_PUBLIC_STATIC_STRING_CMD );
        pstream.println( SOURCE_EMPTY );

        for (int i = 0; i < mth.length; i++) {
            parahead = new String(SOURCE_EMPTY);
            excepthead = new String(SOURCE_EMPTY);
            rtype = mth[i].getReturnType();
            try {
                return_variable_type = VariableType.valueOf(rtype.getName());
                return_variable_convert = VariableType.convert(rtype.getName());
            } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage( 
                    ObjectServerError.INTERNAL_ERROR_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                System.exit(ERROR_EXIT_CODE); 
            }
            ptypes = mth[i].getParameterTypes();
            parakey = new String(SOURCE_EMPTY);
            if (ptypes.length == 0) {
                parakey = parakey.concat(SOURCE_BLANK);
            }
            for (j = 0; j < ptypes.length; j++) {
                if (j != 0) {
                    parahead = parahead.concat(SOURCE_UNDERSCORE);
                    parakey = parakey.concat(PARAMETER_SEPARATOR);
                }
                try {
                    parahead = parahead.concat(
   	                VariableType.convert(ptypes[j].getName()));
                    parakey = parakey.concat(
			VariableType.convert(ptypes[j].getName()));
                } catch (Exception exception_object) {
                    System.err.println( getMessageResource().getFormattedMessage(  
                        ObjectServerError.INTERNAL_ERROR_GENERATOR,
                        getMessageResource().getMessage(
                            ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            etypes = mth[i].getExceptionTypes();
            excptkey = new String(SOURCE_EMPTY);
            for (j = 0; j < etypes.length; j++) {
                if (j != 0) {
                    excepthead = excepthead.concat(SOURCE_UNDERSCORE);
                    excptkey = excptkey.concat(PARAMETER_SEPARATOR);
                }
                excepthead = excepthead.concat(replace(etypes[j].getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE));
                excptkey = excptkey.concat(etypes[j].getName());
            }
            if (cls.getPackage() != null) {
                varhead = replace(cls.getPackage().getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE +
                    basename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead;
                if (add_debug_function == true) {
                    varhead_debug = replace(cls.getPackage().getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE +
                    basename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
                    SOURCE_DEBUG_FUNCTION_SUFFIX +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead;
                }
            } else {
                varhead = filename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead;
                if (add_debug_function == true) {
                    varhead_debug = filename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
                    SOURCE_DEBUG_FUNCTION_SUFFIX +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead;
                }
            }
	    
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(  
                        ObjectServerDebug.DEBUG_VARIABLE_VALUE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_VARIABLE_VALUE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        varhead));
            }

            if (cls.getPackage() != null) {
                sentence  = new String (SOURCE_DOUBLE_QUOTE 
                    + cls.getPackage().getName() + SUFFIX_SEPARATOR
                    + basename + SOURCE_SLASH 
                    + cls.getName() + SOURCE_SLASH 
                    + mth[i].getName() + SOURCE_SLASH 
                    + return_variable_convert + SOURCE_SLASH 
                    + parakey + SOURCE_SLASH + excptkey 
                    + SOURCE_DOUBLE_QUOTE);
                if (add_debug_function == true) {
                    sentence_debug  = new String (SOURCE_DOUBLE_QUOTE 
                    + cls.getPackage().getName() + SUFFIX_SEPARATOR
                    + basename + SOURCE_SLASH 
                    + cls.getName() + SOURCE_SLASH 
                    + mth[i].getName() 
                    + SOURCE_DEBUG_FUNCTION_SUFFIX 
                    + SOURCE_SLASH 
                    + return_variable_convert + SOURCE_SLASH 
                    + parakey + SOURCE_SLASH + excptkey 
                    + SOURCE_DOUBLE_QUOTE);
                }
            } else {
                sentence  = new String(SOURCE_DOUBLE_QUOTE 
                    + filename + 
                    SOURCE_SLASH + cls.getName() + 
                    SOURCE_SLASH + mth[i].getName() + 
                    SOURCE_SLASH + return_variable_convert +
                    SOURCE_SLASH + parakey + 
                    SOURCE_SLASH + excptkey + SOURCE_DOUBLE_QUOTE);
                if (add_debug_function == true) {
                    sentence_debug  = new String(SOURCE_DOUBLE_QUOTE 
                    + filename + 
                    SOURCE_SLASH + cls.getName() + 
                    SOURCE_SLASH + mth[i].getName() + 
                    SOURCE_DEBUG_FUNCTION_SUFFIX +
                    SOURCE_SLASH + return_variable_convert +
                    SOURCE_SLASH + parakey + 
                    SOURCE_SLASH + excptkey + SOURCE_DOUBLE_QUOTE);
                }
            }
            try {
                pstream.println(SOURCE_PUBLIC_STATIC_TYPE+
                        JAVA_LANG_STRING +
                        SOURCE_BLANK + varhead + 
                        SOURCE_UNDERSCORE_D +
                        SOURCE_EQUAL +
                        sentence +
                        SOURCE_SEMICOLON);
	    } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage(
                    ObjectServerError.INTERNAL_ERROR_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                System.exit(ERROR_EXIT_CODE);
            }
            if (add_debug_function == true) {
                try {
                    pstream.println(SOURCE_PUBLIC_STATIC_TYPE+
                        JAVA_LANG_STRING +
                        SOURCE_BLANK + varhead_debug + 
                        SOURCE_UNDERSCORE_D +
                        SOURCE_EQUAL +
                        sentence_debug +
                        SOURCE_SEMICOLON);
	        } catch (Exception exception_object) {
                    System.err.println( 
                        getMessageResource().getFormattedMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR,
                        getMessageResource().getMessage(
                            ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            pstream.println( SOURCE_EMPTY );
            
	}

        int exception_index;
        exception_index = -1;
        for (int i = 0; i < mth.length; i++) {
            rtype = mth[i].getReturnType();
            try {
                return_variable_type = VariableType.valueOf(rtype.getName());
                return_variable_convert = VariableType.convert(rtype.getName());
            } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage(  
                    ObjectServerError.INTERNAL_ERROR_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                System.exit(ERROR_EXIT_CODE); 
            }
            function_head = SOURCE_BLANK_PUBLIC + 
                return_variable_type +
                SOURCE_BLANK + mth[i].getName() + SOURCE_START_BRACE;
            pstream.print( SOURCE_BLANK_PUBLIC + 
                return_variable_type +
                SOURCE_BLANK + mth[i].getName() + 
                SOURCE_BODY_FUNCTION_SUFFIX +
                SOURCE_START_BRACE);
            function_debug = SOURCE_BLANK_PUBLIC + 
                return_variable_type +
                SOURCE_BLANK + mth[i].getName() + 
                SOURCE_DEBUG_FUNCTION_SUFFIX +
                SOURCE_START_BRACE;

            ptypes = mth[i].getParameterTypes();
            for (j = 0; j < ptypes.length; j++) {
                if (j != 0) {
                    function_head  = function_head.concat(SOURCE_COMMA);
                    pstream.print( SOURCE_COMMA );
                    function_debug = function_debug.concat(SOURCE_COMMA);
                }
                try {
                    function_head  = function_head.concat(
			VariableType.valueOf(ptypes[j].getName()) + 
                        SOURCE_BLANK_P + String.valueOf(j));
                    pstream.print( 
			VariableType.valueOf(ptypes[j].getName()) + 
                        SOURCE_BLANK_P + String.valueOf(j));

                    function_debug = function_debug.concat(
		        VariableType.valueOf(ptypes[j].getName()) + 
                        SOURCE_BLANK_P + String.valueOf(j));
                } catch (Exception exception_object) {
                    System.err.println(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerError.INTERNAL_ERROR_GENERATOR,
                        getMessageResource().getMessage(
                            ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                    System.exit(ERROR_EXIT_CODE); 
                }
            }
            function_head  = function_head.concat(
                SOURCE_END_BRACE + SOURCE_BLANK );
            function_head  = function_head.concat(
                SOURCE_CARRIAGE_RETURN);
            pstream.println( 
                SOURCE_END_BRACE + SOURCE_BLANK );
            function_debug = function_debug.concat(
                SOURCE_END_BRACE + SOURCE_BLANK );
            function_debug = function_debug.concat(
                SOURCE_CARRIAGE_RETURN);

            etypes = mth[i].getExceptionTypes();
            for (j = 0; j < etypes.length; j++) {
                if (j == 0) {
                    function_head  = function_head.concat(
                        SOURCE_BLANK_THROWS);
                    pstream.print( 
                        SOURCE_BLANK_THROWS + SOURCE_BLANK);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_THROWS);
                } else {
                    function_head  = function_head.concat(
                        SOURCE_COMMA + SOURCE_BLANK);
                    pstream.print( 
                        SOURCE_COMMA + SOURCE_BLANK);
                    function_debug = function_debug.concat(
                        SOURCE_COMMA + SOURCE_BLANK);
                }
                function_head  = function_head.concat(
                        etypes[j].getName());
                pstream.print( 
                        etypes[j].getName());
                function_debug = function_debug.concat(
                        etypes[j].getName());
                if (etypes[j].getName().compareTo(OBJECT_SERVER_SERVICE_UNAVAILABEL_EXCEPTION) == 0) {
                    ssflag = true;
                }
                if (etypes[j].getName().compareTo(JAVA_EXCEPTION_NAME) == 0) {
                    exflag = true;
                }
                if (etypes[j].getName().compareTo(OBJECT_SERVER_EXCEPTION_NAME) 
                        == 0) {
                    mflag = true;
                }
            }

            function_head  = function_head.concat(
                SOURCE_BLANK_START_MIDDLE_BRACE);
            function_head  = function_head.concat(
                SOURCE_CARRIAGE_RETURN);

            pstream.println( SOURCE_BLANK_START_MIDDLE_BRACE);

            function_debug = function_debug.concat(
                SOURCE_BLANK_START_MIDDLE_BRACE);
            function_debug  = function_debug.concat(
                SOURCE_CARRIAGE_RETURN);

            function_head  = function_head.concat(
                SOURCE_BLANK_CMD_NEW_STRING);
            function_debug = function_debug.concat(
                SOURCE_BLANK_CMD_NEW_STRING);
           /**
            * for compatibility
            */
            pstream.print( SOURCE_BLANK_CMD_NEW_STRING );

            ptypes = mth[i].getParameterTypes();

            plist = new String(SOURCE_EMPTY);
            parakey = new String(SOURCE_EMPTY);
            parahead = new String(SOURCE_EMPTY);
            for(j = 0; j < ptypes.length;j++) {
                if (j != 0) {
                    parakey = parakey.concat(PARAMETER_SEPARATOR);
                    parahead = parahead.concat(SOURCE_UNDERSCORE);
                }
                try {
                    parakey = parakey.concat(
				 VariableType.convert(ptypes[j].getName()));
                    parahead = parahead.concat(
   	                VariableType.convert(ptypes[j].getName()));
                } catch (Exception exception_object) {
                    System.err.println(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerError.INTERNAL_ERROR_GENERATOR,
                            getMessageResource().getMessage(
                                ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                    System.exit(ERROR_EXIT_CODE);
                }
                if (j > 0) {
                    plist = plist.concat(SOURCE_COMMA);
                }
		plist = plist.concat(SOURCE_P_HEAD + String.valueOf(j));
            }

            etypes = mth[i].getExceptionTypes();
            excptkey = new String(SOURCE_EMPTY);
            excepthead = new String(SOURCE_EMPTY);
            for(j = 0; j < etypes.length;j++) {
                if (j != 0) {
                    excptkey = excptkey.concat(PARAMETER_SEPARATOR);
                    excepthead = excepthead.concat(SOURCE_UNDERSCORE);
                }
                excepthead = excepthead.concat(
                    replace(etypes[j].getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE));
                excptkey = excptkey.concat(etypes[j].getName());
            }
            if (cls.getPackage() != null) {
                varhead = replace(cls.getPackage().getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE +
                    basename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead +
                    SOURCE_UNDERSCORE_D;
                function_head  = function_head.concat(varhead);
                pstream.print( varhead );
                function_debug = function_debug.concat( 
                    replace(cls.getPackage().getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE +
                    basename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
                    SOURCE_DEBUG_FUNCTION_SUFFIX +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead +
                    SOURCE_UNDERSCORE_D);
            } else {
                varhead = filename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead +
                    SOURCE_UNDERSCORE_D;
                function_head  = function_head.concat(varhead);
                pstream.print( varhead );
                function_debug = function_debug.concat(filename + 
                    SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
                    SOURCE_DEBUG_FUNCTION_SUFFIX +
		    SOURCE_UNDERSCORE + 
                    return_variable_convert + 
                    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead +
                    SOURCE_UNDERSCORE_D);
            }

            function_head  = function_head.concat(
                SOURCE_END_BRACE + SOURCE_SEMICOLON );
            function_head  = function_head.concat(
                SOURCE_CARRIAGE_RETURN);
            function_debug = function_debug.concat(
                SOURCE_END_BRACE + SOURCE_SEMICOLON );
            function_debug  = function_debug.concat(
                SOURCE_CARRIAGE_RETURN);
            pstream.println( SOURCE_END_BRACE + SOURCE_SEMICOLON );

/*******************
            if (rtype.getName().compareTo(VOID_TYPE_KEY) == 0) {
                function_head  = function_head.concat(SOURCE_TAB +
			         mth[i].getName() +
                                 SOURCE_BODY_FUNCTION_SUFFIX +
                                 SOURCE_START_BRACE +
                                 plist +
                                 SOURCE_END_BRACE +
                                 SOURCE_SEMICOLON);
                function_debug = function_debug.concat(SOURCE_TAB +
			         mth[i].getName() +
                                 SOURCE_BODY_FUNCTION_SUFFIX +
                                 SOURCE_START_BRACE +
                                 plist +
                                 SOURCE_END_BRACE +
                                 SOURCE_SEMICOLON);

                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN +
                    SOURCE_BLANK_RETURN);
                function_debug = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN +
                    SOURCE_BLANK_RETURN);
            } else {
                function_head  = function_head.concat(SOURCE_BLANK_RETURN_BLANK);
                function_debug  = function_debug.concat(SOURCE_BLANK_RETURN_BLANK);
                function_head  = function_head.concat(
			         mth[i].getName() +
                                 SOURCE_BODY_FUNCTION_SUFFIX +
                                 SOURCE_START_BRACE +
                                 plist +
                                 SOURCE_END_BRACE +
                                 SOURCE_SEMICOLON);
                function_debug = function_debug.concat(
			         mth[i].getName() +
                                 SOURCE_BODY_FUNCTION_SUFFIX +
                                 SOURCE_START_BRACE +
                                 plist +
                                 SOURCE_END_BRACE +
                                 SOURCE_SEMICOLON);
	    }
          
            function_head  = function_head.concat(
                SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
            function_debug = function_debug.concat(
                SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
*******************/

            pstream.println( SOURCE_BLANK_OBJECT_RT);
            function_head  = function_head.concat(SOURCE_BLANK_OBJECT_RT);
            function_head  = function_head.concat(
                SOURCE_CARRIAGE_RETURN);
            function_debug = function_debug.concat(SOURCE_BLANK_OBJECT_RT);
            function_debug  = function_debug.concat(
                SOURCE_CARRIAGE_RETURN);

            parahead = new String(SOURCE_EMPTY);
            excepthead = new String(SOURCE_EMPTY);
            for (j = 0; j < ptypes.length; j++) {
                if (j != 0) {
                    parahead = parahead.concat(SOURCE_UNDERSCORE);
                }
                try {
                    parahead = parahead.concat(
	  	        VariableType.convert(ptypes[j].getName()));
                } catch (Exception exception_object) {
                    System.err.println( getMessageResource().getFormattedMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR,
                        getMessageResource().getMessage(
                            ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            for (j = 0; j < etypes.length; j++) {
                if (j != 0) {
                    excepthead = excepthead.concat(SOURCE_UNDERSCORE);
                }
                excepthead = excepthead.concat(
                    replace(etypes[j].getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE));
            }
            varhead = replace(cls.getPackage().getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE +
                    basename + SOURCE_UNDERSCORE + replace(cls.getName(),
                    SOURCE_DOT, SOURCE_UNDERSCORE) +
                    SOURCE_UNDERSCORE + mth[i].getName() +
		    SOURCE_UNDERSCORE + return_variable_convert +
		    SOURCE_UNDERSCORE + parahead + 
                    SOURCE_UNDERSCORE + excepthead;
            for (j = 0; j < ptypes.length; j++) {
                function_head  = function_head.concat(SOURCE_TAB
                    + SOURCE_TAB + varhead 
                    + SOURCE_UNDERSCORE_P + String.valueOf(j) 
                    + SOURCE_EQUAL_P + String.valueOf(j) + SOURCE_SEMICOLON);
                function_debug = function_debug.concat(SOURCE_TAB
                    + SOURCE_TAB + varhead 
                    + SOURCE_UNDERSCORE_P + String.valueOf(j) 
                    + SOURCE_EQUAL_P + String.valueOf(j) + SOURCE_SEMICOLON);
                pstream.println(SOURCE_TAB + SOURCE_TAB + varhead 
                    + SOURCE_UNDERSCORE_P + String.valueOf(j) 
                              + SOURCE_EQUAL_P + String.valueOf(j) 
                              + SOURCE_SEMICOLON);
            }
            pstream.println(SOURCE_FIELD_FROM_GETCLASS);
            function_head  = function_head.concat(SOURCE_FIELD_FROM_GETCLASS);
            function_debug = function_debug.concat(SOURCE_FIELD_FROM_GETCLASS);

            function_head  = function_head.concat(
                SOURCE_CARRIAGE_RETURN);
            function_debug  = function_debug.concat(
                SOURCE_CARRIAGE_RETURN);

            pstream.println(SOURCE_BLANK_TRY);
            function_debug = function_debug.concat(SOURCE_BLANK_TRY);
            function_head  = function_head.concat(SOURCE_BLANK_TRY);
            function_head  = function_head.concat(SOURCE_CARRIAGE_RETURN);
            function_debug  = function_debug.concat(SOURCE_CARRIAGE_RETURN);

            if (rtype.getName().compareTo(int.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_INT_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_INT_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_INT_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_INT_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_INT_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_INT_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_INT_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_INT_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_INT_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(long.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_LONG_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_LONG_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_LONG_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_LONG_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_LONG_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_LONG_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_LONG_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_LONG_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_LONG_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(float.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_FLOAT_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_FLOAT_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_FLOAT_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_FLOAT_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_FLOAT_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_FLOAT_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_FLOAT_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_FLOAT_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_FLOAT_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(double.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_DOUBLE_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_DOUBLE_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_DOUBLE_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_DOUBLE_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_DOUBLE_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_DOUBLE_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_DOUBLE_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_DOUBLE_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_DOUBLE_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(byte.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_BYTE_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_BYTE_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_BYTE_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_BYTE_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_BYTE_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_BYTE_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_BYTE_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_BYTE_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_BYTE_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(boolean.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_BOOLEAN_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_BOOLEAN_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_BOOLEAN_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_BOOLEAN_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_BOOLEAN_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_BOOLEAN_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_BOOLEAN_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_BOOLEAN_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_BOOLEAN_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(char.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_CHAR_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_CHAR_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_CHAR_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_CHAR_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_CHAR_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_CHAR_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_CHAR_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_CHAR_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_CHAR_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(short.class.getName()) == 0) {
                pstream.println(SOURCE_RETURN_SHORT_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_RETURN_SHORT_CONNECT1);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_SHORT_CONNECT1);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_SHORT_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_RETURN_SHORT_CONNECT2);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_SHORT_CONNECT2);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_RETURN_SHORT_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_RETURN_SHORT_CONNECT3);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_SHORT_CONNECT3);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(VOID_TYPE_KEY) == 0) {
                pstream.println(SOURCE_RETURN_CONNECT_VOID);
                function_head  = function_head.concat(
                    SOURCE_RETURN_CONNECT_VOID);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_CONNECT_VOID);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            } else {
                pstream.println(SOURCE_RETURN_CONNECT);
                function_head  = function_head.concat(
                    SOURCE_RETURN_CONNECT);
                function_debug = function_debug.concat(
                    SOURCE_RETURN_CONNECT);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
            }

            if (etypes.length > 0) {
                pstream.println(SOURCE_CATCH_INVOCATION_TARGET_EXCEPTION);
                function_head  = function_head.concat(
                    SOURCE_CATCH_INVOCATION_TARGET_EXCEPTION);
                function_debug = function_debug.concat(
                    SOURCE_CATCH_INVOCATION_TARGET_EXCEPTION);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);

                pstream.println(SOURCE_THROWABLE_CAUSE);
                function_head  = function_head.concat(
                    SOURCE_THROWABLE_CAUSE);
                function_debug = function_debug.concat(
                    SOURCE_THROWABLE_CAUSE);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);

                pstream.println(SOURCE_CAUSE_EP);
                function_head  = function_head.concat(
                    SOURCE_CAUSE_EP);
                function_debug = function_debug.concat(
                    SOURCE_CAUSE_EP);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);

                pstream.println(SOURCE_WHILE_CAUSE);
                function_head  = function_head.concat(
                    SOURCE_WHILE_CAUSE);
                function_debug = function_debug.concat(
                    SOURCE_WHILE_CAUSE);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                for(j = 0; j < etypes.length; j++) {
                    Constructor[] cst;
                    Class[] cptypes;
                    Class class_super;
                    cst = etypes[j].getConstructors();
                    int k;
                    int l = 0;
                    boolean flag;
                    flag = false;
                    cptypes = null;
                    for (k = 0; k < cst.length; k++) {
                        cptypes = cst[k].getParameterTypes();
                        if (cptypes.length == 0) { flag = true; }
                        if (cptypes.length != 1) { continue; }
                        class_super = null;
                        for (l = 0; l < cptypes.length; l++) {
                            if (cptypes[l].getName().compareTo(JAVA_LANG_THROWABLE) == 0
                                || cptypes[l].getName().compareTo(JAVA_LANG_EXCEPTION) == 0) {
                                break;
                            }
                        }
                        if (l < cptypes.length) {
                            break;
                        }
                    }
                    pstream.print(SOURCE_IF_START_CAUSE_KEY);
                    function_head  = function_head.concat(
                        SOURCE_IF_START_CAUSE_KEY);
                    function_debug = function_debug.concat(
                        SOURCE_IF_START_CAUSE_KEY);

                    pstream.print(SYSTEM_PREFIX +
                        replace(etypes[j].getName().toUpperCase(),
                        SOURCE_DOT, SOURCE_UNDERSCORE) );
                    function_head  = function_head.concat(
                        SYSTEM_PREFIX +
                        replace(etypes[j].getName().toUpperCase(),
                        SOURCE_DOT, SOURCE_UNDERSCORE) );
                    function_debug = function_debug.concat( 
                        SYSTEM_PREFIX +
                        replace(etypes[j].getName().toUpperCase(),
                        SOURCE_DOT, SOURCE_UNDERSCORE) );

                    pstream.println(SOURCE_IF_END_CAUSE_KEY);
                    function_head  = function_head.concat(
                        SOURCE_IF_END_CAUSE_KEY);
                    function_debug = function_debug.concat( 
                        SOURCE_IF_END_CAUSE_KEY);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);

                    printException(pstream, etypes[j].getName());
                    function_head  = function_head.concat(
                        PRINT_THROW_EXCEPTION_HEAD 
                        + etypes[j].getName() + PRINT_THROW_EXCEPTION_TAIL);
                    function_debug = function_debug.concat( 
                        PRINT_THROW_EXCEPTION_HEAD 
                        + etypes[j].getName() + PRINT_THROW_EXCEPTION_TAIL);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);

                    pstream.println(SOURCE_TAB + SOURCE_TAB +
                        SOURCE_TAB + SOURCE_TAB + SOURCE_BRACKET);
                    function_head  = function_head.concat(
                        SOURCE_TAB + SOURCE_TAB +
                        SOURCE_TAB + SOURCE_TAB + SOURCE_BRACKET);
                    function_debug = function_debug.concat( 
                        SOURCE_TAB + SOURCE_TAB +
                        SOURCE_TAB + SOURCE_TAB + SOURCE_BRACKET);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                }
                pstream.println(SOURCE_TAB + SOURCE_CAUSE_GET_CAUSE);
                function_head  = function_head.concat(
                    SOURCE_TAB + SOURCE_CAUSE_GET_CAUSE);
                function_debug = function_debug.concat( 
                    SOURCE_TAB + SOURCE_CAUSE_GET_CAUSE);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);
                pstream.println(SOURCE_TAB + SOURCE_TAB +
                    SOURCE_TAB + SOURCE_BRACKET);
                function_head  = function_head.concat(
                    SOURCE_TAB + SOURCE_TAB +
                    SOURCE_TAB + SOURCE_BRACKET);
                function_debug = function_debug.concat( 
                    SOURCE_TAB + SOURCE_TAB +
                    SOURCE_TAB + SOURCE_BRACKET);
                function_head  = function_head.concat(
                    SOURCE_CARRIAGE_RETURN);
                function_debug  = function_debug.concat(
                    SOURCE_CARRIAGE_RETURN);

                if (rtype.getName().compareTo(VOID_TYPE_KEY) == 0) {
                    pstream.println(SOURCE_TAB + SOURCE_BLANK_RETURN);
                    function_head  = function_head.concat(
                        SOURCE_TAB + SOURCE_BLANK_RETURN);
                    function_debug = function_debug.concat( 
                        SOURCE_TAB + SOURCE_BLANK_RETURN);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(int.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_INT);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_INT);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_INT);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(long.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_LONG);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_LONG);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_LONG);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(float.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_FLOAT);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_FLOAT);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_FLOAT);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(double.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_DOUBLE);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_DOUBLE);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_DOUBLE);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(boolean.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_BOOLEAN);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_BOOLEAN);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_BOOLEAN);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(byte.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_BYTE);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_BYTE);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_BYTE);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(char.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_CHAR);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_CHAR);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_CHAR);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(short.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_SHORT);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_SHORT);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_SHORT);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                } else {
                    pstream.println(SOURCE_BLANK_RETURN_NULL_HEAD +
                        return_variable_type +
                        SOURCE_BLANK_RETURN_NULL_TAIL);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_NULL_HEAD +
                        return_variable_type +
                        SOURCE_BLANK_RETURN_NULL_TAIL);
                    function_debug = function_debug.concat( 
                        SOURCE_BLANK_RETURN_NULL_HEAD +
                        return_variable_type +
                        SOURCE_BLANK_RETURN_NULL_TAIL);
                    function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);
                }
            }
            pstream.println(SOURCE_CATCH_EXCEPTION);
            function_head  = function_head.concat(SOURCE_CATCH_EXCEPTION);
            function_debug = function_debug.concat(SOURCE_CATCH_EXCEPTION);
            function_head  = function_head.concat(
                        SOURCE_CARRIAGE_RETURN);
            function_debug  = function_debug.concat(
                        SOURCE_CARRIAGE_RETURN);

            if (etypes.length > 0) {
                for(j = 0; j < etypes.length; j++) {
                    if (etypes[j].getName().compareTo(JAVA_LANG_EXCEPTION) == 0) {
                        exception_index = j;
                        continue;
                    }
                    pstream.print(SOURCE_TAB +
                            SOURCE_TAB +
                            SOURCE_TAB +
                            SOURCE_IF_INSTANCE_HEAD +
                            etypes[j].getName() +
                            SOURCE_IF_INSTANCE_TAIL);
                    function_head  = function_head.concat(
                            SOURCE_TAB +
                            SOURCE_TAB +
                            SOURCE_TAB +
                            SOURCE_IF_INSTANCE_HEAD +
                            etypes[j].getName() +
                            SOURCE_IF_INSTANCE_TAIL);
                    function_debug = function_debug.concat(
                            SOURCE_TAB +
                            SOURCE_TAB +
                            SOURCE_TAB +
                            SOURCE_IF_INSTANCE_HEAD +
                            etypes[j].getName() +
                            SOURCE_IF_INSTANCE_TAIL);
                    pstream.println(THROW_EXCEPTION_HEAD +
                            etypes[j].getName() +
                            THROW_EXCEPTION_TAIL);
                    function_head  = function_head.concat(
                            THROW_EXCEPTION_HEAD +
                            etypes[j].getName() +
                            THROW_EXCEPTION_TAIL);
                    function_debug = function_debug.concat(
                            THROW_EXCEPTION_HEAD +
                            etypes[j].getName() +
                            THROW_EXCEPTION_TAIL);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                }
                if (exception_index != -1) {
                    pstream.println(
                        SOURCE_TAB + SOURCE_TAB + SOURCE_TAB +
                        SOURCE_THROW_EXCEPTION_END);
                    function_head  = function_head.concat(
                        SOURCE_TAB + SOURCE_TAB + SOURCE_TAB +
                        SOURCE_THROW_EXCEPTION_END);
                    function_debug = function_debug.concat(
                        SOURCE_TAB + SOURCE_TAB + SOURCE_TAB +
                        SOURCE_THROW_EXCEPTION_END);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                }
            }
            if (exception_index == -1) {
                if (rtype.getName().compareTo(VOID_TYPE_KEY) == 0) {
                    pstream.println(SOURCE_TAB + SOURCE_BLANK_RETURN);
                    function_head  = function_head.concat(
                        SOURCE_TAB + SOURCE_BLANK_RETURN);
                    function_debug = function_debug.concat(
                        SOURCE_TAB + SOURCE_BLANK_RETURN);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(int.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_INT);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_INT);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_INT);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(long.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_LONG);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_LONG);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_LONG);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(float.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_FLOAT);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_FLOAT);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_FLOAT);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(double.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_DOUBLE);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_DOUBLE);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_DOUBLE);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(boolean.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_BOOLEAN);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_BOOLEAN);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_BOOLEAN);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(byte.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_BYTE);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_BYTE);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_BYTE);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(char.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_CHAR);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_CHAR);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_CHAR);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else if (rtype.getName().compareTo(short.class.getName()) == 0) {
                    pstream.println(SOURCE_BLANK_RETURN_SHORT);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_SHORT);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_SHORT);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                } else {
                    pstream.println(SOURCE_BLANK_RETURN_NULL_HEAD +
                    return_variable_type +
                    SOURCE_BLANK_RETURN_NULL_TAIL);
                    function_head  = function_head.concat(
                        SOURCE_BLANK_RETURN_NULL_HEAD +
                        return_variable_type +
                        SOURCE_BLANK_RETURN_NULL_TAIL);
                    function_debug = function_debug.concat(
                        SOURCE_BLANK_RETURN_NULL_HEAD +
                        return_variable_type +
                        SOURCE_BLANK_RETURN_NULL_TAIL);
                    function_head  = function_head.concat(
                            SOURCE_CARRIAGE_RETURN);
                    function_debug  = function_debug.concat(
                            SOURCE_CARRIAGE_RETURN);
                }
            }
            exception_index = -1;
            pstream.println(SOURCE_TAB + SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
            function_head  = function_head.concat(
                SOURCE_TAB + SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
            function_debug = function_debug.concat(
                SOURCE_TAB + SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
            function_head  = function_head.concat(
                SOURCE_CARRIAGE_RETURN);
            function_debug  = function_debug.concat(
                SOURCE_CARRIAGE_RETURN);
            
            if (rtype.getName().compareTo(VOID_TYPE_KEY) == 0) {
                pstream.println(SOURCE_BLANK_RETURN);
                function_head = function_head.concat(SOURCE_BLANK_RETURN);
                function_debug = function_debug.concat(SOURCE_BLANK_RETURN);
                function_head = function_head.concat(SOURCE_CARRIAGE_RETURN);
                function_debug = function_debug.concat(SOURCE_CARRIAGE_RETURN);
            } else if (rtype.getName().compareTo(int.class.getName()) == 0) {
            } else if (rtype.getName().compareTo(long.class.getName()) == 0) {
            } else if (rtype.getName().compareTo(float.class.getName()) == 0) {
            } else if (rtype.getName().compareTo(double.class.getName()) == 0) {
            } else if (rtype.getName().compareTo(boolean.class.getName())==0) {
            } else if (rtype.getName().compareTo(byte.class.getName()) == 0) {
            } else if (rtype.getName().compareTo(char.class.getName()) == 0) {
            } else if (rtype.getName().compareTo(short.class.getName()) == 0) {
            } else {
                pstream.println(SOURCE_BLANK_RETURN_BRACE + 
                    return_variable_type + SOURCE_RT);
                function_head  = function_head.concat(
                    SOURCE_BLANK_RETURN_BRACE + 
                    return_variable_type + SOURCE_RT);
                function_debug = function_debug.concat(
                    SOURCE_BLANK_RETURN_BRACE + 
                    return_variable_type + SOURCE_RT);
                function_head = function_head.concat(SOURCE_CARRIAGE_RETURN);
                function_debug = function_debug.concat(SOURCE_CARRIAGE_RETURN);
            }
            pstream.println(SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
            function_head  = function_head.concat(
                SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);
            function_debug = function_debug.concat(
                SOURCE_TAB + SOURCE_END_MIDDLE_BRACE);

            if (add_debug_function == true) {
                pstream.println();
                pstream.println(function_debug);
                pstream.println();
            }
            ssflag = false;
            exflag = false;
            mflag = false;
        }
        pstream.println(SOURCE_END_MIDDLE_BRACE);
    }

    public boolean removeInterfaceClass(String cfname, String pack, boolean flag) {
        int  idx;
        File fdes;
        String pname;
        String clsfname;
        String clsbase;
        String packpath;
        String npack;
        boolean return_flag;

        return_flag = false;
        if (cfname == null) {
            return return_flag;
        }
        npack = pack;
        clsbase = cfname;
        idx = cfname.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1);
            npack = cfname.substring(0, idx);
        }
        idx = cfname.lastIndexOf(File.separator);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1);
            npack = cfname.substring(0, idx);
        }
        clsfname = new String(clsbase + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
        if (npack == null) {
            packpath = clsfname;
            pname = pwd;
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(                 
                        ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        pname));
            }
            idx = pname.lastIndexOf(File.separator);
            if (idx >= 0) {
                if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                    pname = pname.substring(0, idx);
                    idx = pname.lastIndexOf(File.separator);
                    pname = pname.substring(idx + 1);
                } else {
                    pname = pname.substring(idx + 1);
                }
            }

            fdes = new File(packpath);
            try {
                if (fdes.exists() == true) {
                    if (cfname.compareTo(clsbase) == 0) {
                        String  package_name;
                        Package package_object;
                        try {
                            package_object = Package.valueOf(clsfname,
                                getMessageResource(),
                                getDebug(),
                                getLogStream() );
                       
                            package_name = package_object.getPackage();
                            package_name = replace(package_name,
                                UNIX_PATH_SEPARATOR,
                                File.separator);
                        } catch (Exception exception_object) {
                            return return_flag;
                        }
                        if (package_name.compareTo(pname) == 0) {
                            return return_flag;
                        }
                        fdes = new File(package_name 
                            + File.separator + packpath);
                        if (fdes.exists() == true) {
                            if (removeLockFile(package_name + File.separator + packpath) == true) {
                                if (flag == true) {
                                    fdes.delete();
                                }
                                return_flag = true;
                            }
                        }
                        fdes = new File(package_name);
                        try {
                            if (fdes.exists() == true) {
                                if (removeLockFile(package_name) == true) {
                                    fdes.delete();
                                    return return_flag;
                                }
                            }
		        } catch (Exception exception_object) {
                            return return_flag;
                        }
		    }
		}
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_REMOVEINTERFACECLASS, 0, exception_object);
            }
        }
        return return_flag;
    }

    public boolean checkLockFile(String origine_file) {
        int idx;
        String file_head;
        String file_tail;
        File fdes;

        idx = origine_file.lastIndexOf(File.separator);
        if (idx > 0) {
            file_head = origine_file.substring(0, idx + 1);
            file_tail = origine_file.substring(idx + 1);
            fdes = new File(file_head + LOCK_PREFIX + file_tail) ;
        } else {
            fdes = new File(LOCK_PREFIX + origine_file) ;
        }
        if (fdes.exists() == true) {
            return true;
        }
        return false;
    }

    private boolean makeLockFile(String origine_file) {
        int idx;
        String file_head;
        String file_tail;
        File fdes;

        idx = origine_file.lastIndexOf(File.separator);
        if (idx > 0) {
            file_head = origine_file.substring(0, idx + 1);
            file_tail = origine_file.substring(idx + 1);
            fdes = new File(file_head + LOCK_PREFIX + file_tail) ;
        } else {
            fdes = new File(LOCK_PREFIX + origine_file) ;
        }
        if (fdes.exists() == true) {
            return false;
        }
        try {
            fdes.createNewFile();
        } catch (Exception exception_object) {
            return false;
        }
        return true;
    }

    private boolean removeLockFile(String origine_file) {
        int idx;
        String file_head;
        String file_tail;
        File fdes;

        idx = origine_file.lastIndexOf(File.separator);
        if (idx > 0) {
            file_head = origine_file.substring(0, idx + 1);
            file_tail = origine_file.substring(idx + 1);
            fdes = new File(file_head + LOCK_PREFIX + file_tail) ;
        } else {
            fdes = new File(LOCK_PREFIX + origine_file) ;
        }
        if (fdes.exists() == false) {
            return false;
        }
        fdes.delete();
        return true;
    }

    public void deleteInterfaceClass(String cfname, int dirent) throws Exception 
    {
        File fdes;
        String clsfname;
        String  headname;
        int     iii;
        int     idx;

        if (cfname == null) {
            ObjectServerException ose = 
                new ObjectServerException( getMessageResource().getFormattedMessage(                 
	            ObjectServerError.CLASS_FILENAME_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.CLASS_FILENAME_PARAMETER_NULL_GENERATOR)));
            throw ose;
        }
        clsfname = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
        fdes = new File(clsfname);
        if (fdes.exists() == true) {
            fdes.delete();
        }
        idx = replace(clsfname, OBJECT_DOT_SEPARATOR,
		   File.separator).lastIndexOf(File.separator);
        headname = clsfname.substring(0, idx);
        for (iii = 0; iii < dirent; iii++) {
            fdes = new File(headname);
            if (fdes.exists() == true) {
                fdes.delete();
            }
            idx = headname.lastIndexOf(File.separator);
            if (idx < 0) {
                break;
            }
            headname = headname.substring(0, idx);
        }
    }

    public String setInterfaceClass(String cfname, String pack) throws Exception 
    {
        int  idx;
        int  rcnt;
        File fdes;
        String pname;
        String clsfname;
        String clsbase;
        String packpath;
        String npack;

        if (cfname == null) {
            ObjectServerException ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(                 
	            ObjectServerError.CLASS_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.CLASS_PARAMETER_NULL_GENERATOR)));
             throw ose;
        }
        npack = pack;
        clsbase = cfname;
        idx = cfname.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1);
            npack = cfname.substring(0, idx);
        }
        idx = cfname.lastIndexOf(File.separator);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1);
            npack = cfname.substring(0, idx);
        }
        clsfname = new String(clsbase + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
        if (npack != null) {
            return null;
        }
        packpath = clsfname;

        pname = pwd;
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(                 
                    ObjectServerDebug.DEBUG_CURRENT_DIRECTORY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CURRENT_DIRECTORY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    pname));
        }
        idx = pname.lastIndexOf(File.separator);
        if (idx >= 0) {
            if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                pname = pname.substring(0, idx);
                idx = pname.lastIndexOf(File.separator);
                pname = pname.substring(idx + 1);
            } else {
                pname = pname.substring(idx + 1);
            }
        }
        fdes = new File(packpath);
        try {
            if (fdes.exists() == true) {
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_CALL_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_CALL_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_SET_INTERFACE_CLASS,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_CODE_002)),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_CLASS_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_CLASS_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            clsbase,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            cfname));
                }
                if (cfname.compareTo(clsbase) == 0) {
                    String  package_name;
                    Package package_object;
                    try {
                        package_object = Package.valueOf(clsfname,
                                getMessageResource(),
                                getDebug(),
                                getLogStream() );
                        package_name = package_object.getPackage();
                        package_name = replace(package_name,
                            UNIX_PATH_SEPARATOR,
                                File.separator);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                    if (package_name.compareTo(pname) == 0) {
                        return pname;
                    }
                    fdes = new File(package_name);
                    if (fdes.exists() == true) {
                        if (checkLockFile(package_name) == true) {
                            return package_name;
                        }
                    }
                }
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_SETINTERFACECLASS, 0, exception_object);
        }
        return null;
    }

    /**
     * @param      String   - class name
     */
    public boolean checkInterfaceClass(String cfname, String pack) throws Exception 
    {
        int  idx;
        int  rcnt;
        File fdes;
        String pname;
        String clsfname;
        String clsbase;
        String packpath;
        String npack;

        if (cfname == null) {
            ObjectServerException ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(                 
                    ObjectServerError.CLASS_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.CLASS_PARAMETER_NULL_GENERATOR)));
            throw ose;
        }
        npack = pack;
        clsbase = cfname;
        idx = cfname.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1);
            npack = cfname.substring(0, idx);
        }
        idx = cfname.lastIndexOf(File.separator);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1);
            npack = cfname.substring(0, idx);
        }
        clsfname = new String(clsbase + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
        if (npack == null) {
            pname = pwd;
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(                 
                        ObjectServerDebug.DEBUG_CURRENT_DIRECTORY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CURRENT_DIRECTORY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        pname));
            }

            idx = pname.lastIndexOf(File.separator);
            if (idx >= 0) {
                if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                    pname = pname.substring(0, idx);
                    idx = pname.lastIndexOf(File.separator);
                    pname = pname.substring(idx + 1);
                } else {
                    pname = pname.substring(idx + 1);
                }
            }
            packpath = clsfname;
            fdes = new File(packpath);
            try {
                if (fdes.exists() == true) {
                    if (cfname.compareTo(clsbase) == 0) {
                        String  package_name;
                        Package package_object;
                        try {
                            package_object = Package.valueOf(clsfname,
                                getMessageResource(),
                                getDebug(),
                                getLogStream() );
                            package_name = package_object.getPackage();
                            package_name = replace(package_name,
                                UNIX_PATH_SEPARATOR,
                                File.separator);
                        } catch (Exception exception_object) {
                            return false;
                        }
                        if (package_name.compareTo(pname) == 0) {
                            return true;
                        }
                        fdes = new File(package_name);
                        try {
                            if (fdes.exists() == false) {
                                fdes.mkdir();
                                if (makeLockFile(package_name) == false) {
                                    return false;
                                }
                            }
                        } catch (Exception exception_object) {
                            return false;
                        }
                        fdes = new File(package_name 
                            + File.separator + packpath);
                        if (fdes.exists() == false) {
                            if (makeLockFile(package_name + File.separator + packpath) == false) {
                                return false;
                            }
                            FileOutputStream fos;
                            FileInputStream  fis;
                            try {
                                fos = new FileOutputStream(fdes);
                                fis = new FileInputStream(packpath);
                                byte[] buffer;
                                buffer = new byte[BUFFER_SIZE];
                                while (true) {
                                    rcnt = fis.read(buffer, 0, BUFFER_SIZE);
                                    if (rcnt == -1) {
                                        break;
                                    }
                                    fos.write(buffer, 0, rcnt);
                                }
                                fis.close();
                                fos.close();

                            } catch (Exception exception_object) {
				removeLockFile(package_name + File.separator + packpath);
                                return false;
                            }
                        }
                        return true;
                    }
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_CHECKINTERFACECLASS, 0, exception_object);
            }
            try {
                pname = pwd;
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_CURRENT_DIRECTORY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_CURRENT_DIRECTORY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                idx = pname.lastIndexOf(File.separator);
                if (idx >= 0) {
                    if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                        pname = pname.substring(0, idx);
                        idx = pname.lastIndexOf(File.separator);
                        pname = pname.substring(idx + 1);
                    } else {
                        pname = pname.substring(idx + 1);
                    }
                }
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                packpath = pname + File.separator + clsfname;
            } catch (Exception exception_object) {
                packpath = clsfname;
            }
        } else {
            try {
                pname = pwd;
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                idx = pname.lastIndexOf(File.separator);
                if (idx >= 0) {
                    if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                        pname = pname.substring(0, idx);
                        idx = pname.lastIndexOf(File.separator);
                        pname = pname.substring(idx + 1);
                    } else {
                        pname = pname.substring(idx + 1);
                    }
                }
                if (pname.compareTo(npack) == 0) {
                    packpath = clsfname;
                } else {
                    packpath = replace(npack, OBJECT_DOT_SEPARATOR, File.separator)
                    + File.separator + clsfname;
                }
            } catch (Exception exception_object) {
                packpath = replace(npack, OBJECT_DOT_SEPARATOR, File.separator)
                   + File.separator + clsfname;
            }
        }

        fdes = new File(packpath);
        if (fdes.exists() == true) {
            return true;
        }
        return false;
    }

    public int copyInterfaceClass(String cfname, String tname, String pack) throws Exception 
    {
        int idx;
        int rtcnt;
        int sumcnt;
        int dircnt;
        byte[] buffer;
        String hostname;
        String clsfname;
        String clsbase;
        String jarname;
        String packname;
        String tpath;
        InputStream con;
        InputStream is;
        DataInputStream dis;
        Socket     ssock;
        File   fdes;
        int    iport;
        String sport;
        String pname;
        Exception sep;
        FileOutputStream fos;
        FileInputStream  fis;
        OutputStream ot;
        ObjectOutputStream oos;
        RetryObjectInputStream ois;
        InetSocketAddress isa;

        if (cfname == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage( 
	            ObjectServerError.CLASS_FILENAME_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.CLASS_FILENAME_PARAMETER_NULL_GENERATOR)));
            throw ose;
        }
        if (tname == null) {
            ObjectServerException ose = 
                new ObjectServerException(
                    getMessageResource().getFormattedMessage( 
	                ObjectServerError.TARGETNAME_PARAMETER_NULL_GENERATOR,
                        getMessageResource().getMessage(
	                    ObjectServerError.TARGETNAME_PARAMETER_NULL_GENERATOR)));
            throw ose;
        }
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln(DEBUG_COPY_INTERFACE_CLASS + cfname);
        }
        pname = null;
        dircnt = 0;
        idx = cfname.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0) {
            clsbase = cfname.substring(idx + 1) + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX;
        } else {
            idx = cfname.lastIndexOf(File.separator);
            if (idx >= 0) {
                clsbase = cfname.substring(idx + 1) + OBJECT_DOT_SEPARATOR
                    + CLASS_SUFFIX;
            } else {
                clsbase = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
            }
        }
        tpath = tname;
        idx = tname.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx < 0) {
            if (pack == null) {
                tpath = tname;
            } else {
                tpath = pack + OBJECT_DOT_SEPARATOR + tname;
            }
        }
 
        buffer = new byte[BUFFER_SIZE];

        if (pack == null) {
            clsfname = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
            try {
                pname = pwd;
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                idx = pname.lastIndexOf(File.separator);
                if (idx >= 0) {
                    if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                        pname = pname.substring(0, idx);
                        idx = pname.lastIndexOf(File.separator);
                        pname = pname.substring(idx + 1);
                    } else {
                        pname = pname.substring(idx + 1);
                    }
                }
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                clsfname = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
                tpath = pname + File.separator + tname;
            } catch (Exception exception_object) {
                clsfname = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
            }
        } else {
            try {
                pname = pwd;
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                idx = pname.lastIndexOf(File.separator);
                if (idx >= 0) {
                    if (pname.substring(idx + 1).compareTo(CURRENT_DIRECTORY) == 0) {
                        pname = pname.substring(0, idx);
                        idx = pname.lastIndexOf(File.separator);
                        pname = pname.substring(idx + 1);
                    } else {
                        pname = pname.substring(idx + 1);
                    }
                }
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(                 
                            ObjectServerDebug.DEBUG_DIRECTORY_PATH,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            pname));
                }
                if (pname.compareTo(pack) != 0) {
                    pname = replace(pack, OBJECT_DOT_SEPARATOR, File.separator);
                    clsfname = pname
                        + File.separator + clsbase;
                    {
                        String  headname = new String(SOURCE_EMPTY);
                        String  entname;
                        String  tailname = pname;
                        int     iii;
                        while (true) {
                            if (tailname.length() > 0) {
                                iii = tailname.indexOf(File.separator);
                                if (iii < 0) {
                                    fdes = new File(tailname);
                                    if (fdes.exists() == false) {
                                        fdes.mkdir();
                                        dircnt++;
                                    }
                                    break;
                                }
                                entname = tailname.substring(0, iii);
                                headname = headname.concat(entname);
                                fdes = new File(headname);
                                if (fdes.exists() == false) {
                                    fdes.mkdir();
                                    dircnt++;
                                }
                                tailname = tailname.substring(iii+1);
                                headname = headname.concat(File.separator);
                            }
                        }
                    }
                } else {
                    clsfname = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
                }
            } catch (Exception exception_object) {
                clsfname = new String(pack + File.separator + cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);
            }
            fdes = new File(clsbase);
            if (fdes.exists() == true) {
                fos = new FileOutputStream(clsfname);
                fis = new FileInputStream(clsbase);
                while (true) {
                    rtcnt = fis.read(buffer, 0, BUFFER_SIZE);
                    if (rtcnt == -1) {
                        break;
                    }
                    try {
                        fos.write(buffer, 0, rtcnt);
                    } catch (Exception exception_object) {
                        break;
                    }
                }
                return dircnt;
            }
        }
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln(DEBUG_CLASS_FILE_NAME + clsfname);
        }
        hostname = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        sport = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if ((hostname == null || sport == null) && dircnt > 0) {
            String  headname = pname;
            int     iii;
            while (true) {
                fdes = new File(headname);
                if (fdes.exists() == true) {
                    fdes.delete();
                    dircnt--;
                    if (dircnt <= 0) { break; }
                }
                iii = headname.lastIndexOf(File.separator);
                if (iii < 0) {
                    break;
                }
                headname = headname.substring(0, iii);
            }
        }
        if (hostname == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(                 
                    ObjectServerError.HOST_CONFIGURATION_ERROR_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.HOST_CONFIGURATION_ERROR_GENERATOR)));
            throw ose;
        }
        if (sport == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(                 
	            ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.PORT_CONFIGURATION_ERROR_GENERATOR)));
            throw ose;
        }

        fdes = null;
        sep = null;
        sumcnt = 0;
        rtcnt = 0;
        fdes = new File(clsfname);
        try {
            if (fdes.createNewFile() != false) {
                ssock = new Socket();
                iport = Integer.valueOf(sport).intValue();
                isa = new InetSocketAddress(hostname, iport);
                ssock.connect(isa);
                ot = ssock.getOutputStream();
                oos = new ObjectOutputStream(ot);

                oos.writeObject(
                    ObjectServerHeader.OSERV_COMMAND_TYPE_OBJECT_KEY);
                jarname = tpath + OBJECT_DOT_SEPARATOR + JAR_SUFFIX;

                RegisterCommand  register_command;
                register_command = RegisterCommand.classCommand(
                    clsbase, jarname,
                    getMessageResource(),
                    getDebug(),
                    getLogStream() );
                oos.writeObject(register_command);
                fos = new FileOutputStream(fdes);
                is = ssock.getInputStream();
                try {
                    ois = RetryObjectInputStream.valueOf(is,
                        getMessageResource(),
                        getDebug(),
                        getLogStream()
                    );
                } catch (Exception second_exception_object) {
                    ObjectServerException ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(                 
	                    ObjectServerError.IO_CONSTRUCTION_ERROR_GENERATOR,
                            getMessageResource().getMessage(
	                        ObjectServerError.IO_CONSTRUCTION_ERROR_GENERATOR)));
                    throw ose;
                }

                Integer size_integer;
                try {
                    size_integer = (Integer)ois.readObject();
                } catch (Exception second_exception_object) {
                    throw second_exception_object;
                }
                for (sumcnt = 0; sumcnt < size_integer.intValue(); ) {
                    try {
                        rtcnt = ois.read(buffer, 0, BUFFER_SIZE);
                    } catch (Exception second_exception_object) {
                        throw second_exception_object;
                    }
                    if (rtcnt == -1) {
                        break;
                    }
                    sumcnt = sumcnt + rtcnt;
                    try {
                        fos.write(buffer, 0, rtcnt);
                    } catch (Exception second_exception_object) {
                        printExceptionMessage(TRACE_CLASSNAME, TRACE_COPYINTERFACECLASS, 0, second_exception_object);
                        break;
                    }
                }
                EOSObject eos_object;
                try {
                    eos_object = (EOSObject)ois.readObject();
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_COPYINTERFACECLASS, 0, second_exception_object);
                    throw second_exception_object;
                }
           }
           if (sumcnt != 0) {
               return dircnt;
           }
        } catch (Exception exception_object) {
            sep = exception_object;
        }
        if (fdes != null && fdes.exists() == true) {
            fdes.delete();
        }
        if (dircnt > 0) {
            String  headname = pname;
            int     iii;
            while (true) {
                fdes = new File(headname);
                if (fdes.exists() == true) {
                    fdes.delete();
                    dircnt--;
                    if (dircnt <= 0) { break; }
                }
                iii = headname.lastIndexOf(File.separator);
                if (iii < 0) {
                    break;
                }
                headname = headname.substring(0, iii);
            }
        }
        if (sumcnt != 0) {
            throw sep;
        }
        return -1;
    }

    /**
     *
     * @param      String   -  class name
     * @return     Class
     * @throws     Exception
     */
    public Class getInterfaceClass(String cfname) throws Exception {
        URL[] urls;
        URL   url;
        String fname;
        String path;
        String pack;
        String clsname;
        String dirname;
        String packname;
        URLClassLoader uloader;
        Class cls;
        String urlpath;
        int idx;
        int idx2;

        if (cfname == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(                 
	            ObjectServerError.CLASS_FILENAME_PARAMETER_NULL_GENERATOR,
                    getMessageResource().getMessage(
	                ObjectServerError.CLASS_FILENAME_PARAMETER_NULL_GENERATOR)));
            throw ose;
        }
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            System.err.println(DEBUG_GETINTERFACE + cfname);
        }
        dirname = null;
        path = pwd;
        idx = cfname.lastIndexOf(File.separator);
        if (idx < 0) {
            idx = pwd.lastIndexOf(File.separator);
            if (idx > 0) {
                dirname = pwd.substring(idx + 1); 
                path = pwd.substring(0, idx);
                clsname = dirname + OBJECT_DOT_SEPARATOR + cfname;
            } else {
                clsname = cfname;
            }
        } else {
            pack = cfname.substring(0, idx);
            packname = replace(pack, OBJECT_DOT_SEPARATOR, File.separator);
            idx = pwd.lastIndexOf(packname);
            if (idx >= 0) {
                path = pwd.substring(0, idx);
            }
            clsname = replace(cfname,File.separator, OBJECT_DOT_SEPARATOR);
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                getLogStream().printLogln(DEBUG_DEBUG + path + 
                    DEBUG_BLANK + clsname + DEBUG_BLANK + 
                    packname + DEBUG_BLANK + pack);
            }
        }

        urls = new URL[1];
        fname = new String(cfname + OBJECT_DOT_SEPARATOR + CLASS_SUFFIX);

        if (File.separator.compareTo(DOS_DIRECTORY_SEPARATOR) == 0) {
            while (true) {
                if (path.charAt(path.length()-1) != File.separatorChar) {
                    break;
                }
                path = path.substring(0, path.length()-1);
            }
        } else {
            if (path.charAt(path.length()-1) != File.separatorChar) {
                path = path.concat(File.separator);
            }
        }
        urlpath = new String(FILE_URL_PROTOCOL + path);

        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln(DEBUG_URL_PATH + urlpath);
        }
        
        try {
            url = new URL(urlpath);
        } catch (Exception exception_object) {
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                getLogStream().printLogln(DEBUG_URL_ERROR + urlpath);
	    }
            throw exception_object;
        }
        urls[0] = url;
        uloader = new URLClassLoader(urls);
        try {
            cls = Class.forName(clsname, true, (ClassLoader)uloader);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            System.err.println(DEBUG_GET_CLASS + cls.getName());
        }
        if (cls.isInterface() == false) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(                 
                    ObjectServerError.REAL_CLASS_WAS_SPECIFIED_GENERATOR,
                    getMessageResource().getMessage(
                        ObjectServerError.REAL_CLASS_WAS_SPECIFIED_GENERATOR)));
            throw ose;
        }
        return cls;
    }

    /**
     * @param      String[]
     * @return     void
     */
    public static void main(String[] arg) {
        ObjectGenerator ogen;
        String iname;
        String tname;
        String packname;
        String packpath;
        String ipackname;
        String ipackpath;
        String ifsbase;
        Class  cls;
        int    mflag;
        int    idx;
        ParseOptions parse_options;

        ogen = new ObjectGenerator();
        try {
            parse_options = ParseOptions.valueOf(
                arg, ObjectGenerator.OBJECT_GENERATOR_ARGS,
                null, null,
                ogen.getMessageResource(),
                ogen.getDebug(),
                ogen.getLogStream() );
        } catch (Exception exception_object) {
            System.err.println(
                ogen.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GENERATOR,
                    ogen.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GENERATOR )));
            ObjectGenerator.printUsage();
            return;
        }
        mflag = -1;
        if (parse_options.getRealArgumentCount() == 2) {
            /* Interface Class */
            try {
                iname = parse_options.getRealArgument(0);
            } catch (Exception exception_object) {
                System.err.println( ogen.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GENERATOR,
                    ogen.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GENERATOR )));
                ObjectGenerator.printUsage();
                return;
            }
            /* Target Class */
            try {
                tname = parse_options.getRealArgument(1);
            } catch (Exception exception_object) {
                System.err.println( ogen.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GENERATOR,
                    ogen.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GENERATOR )));
                ObjectGenerator.printUsage();
                return;
            }

            idx = iname.lastIndexOf(OBJECT_DOT_SEPARATOR);
            ifsbase = iname;
            if (idx >= 0) {
                ipackname = iname.substring(0, idx);
                ipackpath = replace(ipackname, OBJECT_DOT_SEPARATOR,
                    File.separator);
                ifsbase = iname.substring(idx + 1);
            } else {
                idx = iname.lastIndexOf(File.separator);
                if (idx >= 0) {
                    ipackpath = iname.substring(0, idx);
                    ipackname = replace(ipackpath,
                        File.separator, OBJECT_DOT_SEPARATOR);
                    ifsbase = iname.substring(idx + 1);
                } else {
                    ifsbase = iname;
                    ipackpath = null;
                    ipackname = null;
                }
            }
            if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                ogen.getLogStream().printLogln(
                    ogen.getMessageResource().getFormattedMessage(
                        DEBUG_PACKAGE_NAME,
                        ogen.getMessageResource().getMessage(
                            DEBUG_PACKAGE_NAME),
                        BLANK_STRING,
                        tname));
            }
            idx = tname.lastIndexOf(OBJECT_DOT_SEPARATOR);
            if (idx >= 0) {
                packname = tname.substring(0, idx);
                packpath = replace(packname,OBJECT_DOT_SEPARATOR, File.separator);
                if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                    ogen.getLogStream().printLogln(
                        ogen.getMessageResource().getFormattedMessage(
                        DEBUG_PACKAGE_NAME,
                        ogen.getMessageResource().getMessage( DEBUG_PACKAGE_NAME),
                        BLANK_STRING, packpath));
                }
            } else {
                packname = null;
                packpath = null;
            }

            try {
                if (ipackpath == null) {
                    if (ogen.checkInterfaceClass(iname, packname) == false) {
                        mflag = ogen.copyInterfaceClass(iname, tname, packname);
                    }
                } else {
                    if (ogen.checkInterfaceClass(iname, ipackname) == false) {
                        mflag = ogen.copyInterfaceClass(iname, tname, ipackname);
                    }
                }
            } catch (Exception exception_object) {
                if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                    ogen.getLogStream().printLogln(
                        ogen.getMessageResource().getFormattedMessage(
                            DEBUG_COPY_INTERFACE_CLASS ,
                            ogen.getMessageResource().getMessage(
                                DEBUG_COPY_INTERFACE_CLASS), 
                            BLANK_STRING,
                            exception_object.toString()));
                }
                if (ipackpath == null) {
                    ogen.removeInterfaceClass(iname, packname, true);
                }
                System.err.println(
                    ogen.getMessageResource().getFormattedMessage(
                        ObjectServerError.INTERFACE_CLASS_NOT_FOUND_GENERATOR,
                        ogen.getMessageResource().getMessage(
                            ObjectServerError.INTERFACE_CLASS_NOT_FOUND_GENERATOR )));
                return;
            }
            try {
                if (packpath == null) {
                    if (ipackpath == null ) {
                        packpath = ogen.setInterfaceClass(iname, packname);
                        if (packpath == null) {
                            if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE) {
                                System.err.println(
                                    ogen.getMessageResource().getFormattedMessage(
                                        DEBUG_SET_INTERFACE_CLASS,
                                        ogen.getMessageResource().getMessage(
                                            DEBUG_SET_INTERFACE_CLASS),
                                        BLANK_STRING,
                                        iname));
                            }
                            cls = (Class)ogen.getInterfaceClass(iname);
                        } else {
                            if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                                ogen.getLogStream().printLogln(
                                    ogen.getMessageResource().getFormattedMessage(
                                        DEBUG_SET_INTERFACE_CLASS,
                                        ogen.getMessageResource().getMessage(
                                            DEBUG_SET_INTERFACE_CLASS),
                                        BLANK_STRING,
                                        packpath + File.separator + iname));
                            }
                            cls = (Class)ogen.getInterfaceClass(packpath + File.separator + iname);
                        }
                    } else {
                        cls = (Class)ogen.getInterfaceClass(iname);
                    }
                } else {
                    if (ipackpath == null ) {
                        cls = (Class)ogen.getInterfaceClass(packpath + File.separator + iname);
                    } else {
                        cls = (Class)ogen.getInterfaceClass(iname);
                    }
                }
            } catch (Exception exception_object) {
                if (ipackpath == null) {
                    ogen.removeInterfaceClass(iname, packname, true);
                }
                if (mflag >= 0) {
                    try {
                        ogen.deleteInterfaceClass(iname, mflag);
                    } catch (Exception second_exception_object) {
                        ogen.printExceptionMessage(TRACE_CLASSNAME, TRACE_MAIN, 0, second_exception_object);
                    }
                }
                if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                    ogen.getLogStream().printLogln(
                        ogen.getMessageResource().getFormattedMessage(
                            DEBUG_GETINTERFACE,
                            ogen.getMessageResource().getMessage(
                                DEBUG_GETINTERFACE),
                            BLANK_STRING,
                            exception_object.toString()));
                }
                if (exception_object instanceof ClassNotFoundException) {
                    System.err.println(
                        ogen.getMessageResource().getFormattedMessage(
                        ObjectServerError.INTERFACE_CLASS_NOT_FOUND_GENERATOR,
                            ogen.getMessageResource().getMessage(
                    ObjectServerError.INTERFACE_CLASS_NOT_FOUND_GENERATOR)));
                    return;
                }
                if (exception_object instanceof ObjectServerException) {
                    System.err.println(
                        ogen.getMessageResource().getFormattedMessage(
                            ObjectServerError.PARAMETER_ERROR_GENERATOR,
                            ogen.getMessageResource().getMessage(
                                ObjectServerError.PARAMETER_ERROR_GENERATOR )));
                    return;
		}
                System.err.println(
                    ogen.getMessageResource().getFormattedMessage(
                        ObjectServerError.INTERNAL_ERROR_GENERATOR,
                        ogen.getMessageResource().getMessage(
                            ObjectServerError.INTERNAL_ERROR_GENERATOR )));
                return;
            }
            try {
                if (packpath == null) {
                    ogen.genStubClass(tname, iname, cls);
                } else {
                    if (ipackpath == null) {
                        ogen.genStubClass(tname, packpath+File.separator+iname, cls);
                    } else {
                        ogen.genStubClass(tname, iname, cls);
                    }
                }
            } catch (Exception exception_object) {
                if (mflag >= 0) {
                    try {
                        ogen.deleteInterfaceClass(iname, mflag);
                    } catch (Exception second_exception_object) {
                        ogen.printExceptionMessage(TRACE_CLASSNAME, TRACE_MAIN, 0, second_exception_object);
                    }
                }
                if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                    ogen.getLogStream().printLogln(
                        ogen.getMessageResource().getFormattedMessage(
                            ObjectServerError.ERROR_GENSTUB_CLASS_GENERATOR,
                            ogen.getMessageResource().getMessage(
                                ObjectServerError.ERROR_GENSTUB_CLASS_GENERATOR),
                            exception_object.toString()));
                    System.err.println(
                        ogen.getMessageResource().getFormattedMessage(
                            DEBUG_GENSTUB_CLASS,
                            ogen.getMessageResource().getMessage(
                                DEBUG_GENSTUB_CLASS),
                            exception_object.toString()));
                }
            }
            if (ipackpath == null) {
                ogen.removeInterfaceClass(iname, packname, false);
            }
        } else {
            if (ogen.getDebug() > OSERV_DEBUG_TRACE_VALUE && ogen.getLogStream() != null) {
                ogen.getLogStream().printLogln(
                    ogen.getMessageResource().getFormattedMessage(
                        PARAMETER_ERROR,
                        ogen.getMessageResource().getMessage(
                            PARAMETER_ERROR),
                        arg.toString()));
            }
            System.err.println(
                ogen.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_GENERATOR,
                    ogen.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_GENERATOR )));
            ObjectGenerator.printUsage();
            return;
        }
    }
}
