/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     ObjectInputManagerWrapper.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/05/28
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ObjectInputManagerWrapper extends ObjectServerHeader {
    private Object object_instance = null;
    private Class  object_input_manager_class = null;

    private Method to_object_method = null;
    private String TO_OBJECT_METHOD_NAME = "toObject";
    private Class[]  to_object_args_class = {
        byte[].class
    };
    private CallMethod  call_method = null;

    private static String DEBUG_OBJECT_INPUT_MANAGER_WRAPPER 
        = "ObjectInputManagerWrapper";
    private static String DEBUG_TO_OBJECT
        = "toObject";

    public ObjectInputManagerWrapper(Class param_object_input_manager_class) throws Exception {

        String    message;

        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_OBJECT_INPUT_MANAGER_WRAPPER );

        printDebugTraceMessage(message);

        if (param_object_input_manager_class == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        object_input_manager_class = param_object_input_manager_class;

        try {
            object_instance = object_input_manager_class.newInstance();
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_VALUE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_VALUE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                object_instance.toString() );
        printDebugTraceMessage(message);

        try {
            to_object_method = object_input_manager_class.getMethod(
                TO_OBJECT_METHOD_NAME,
                to_object_args_class);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_VALUE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_VALUE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                to_object_method.toString() );
        printDebugTraceMessage(message);
        try {
            call_method = CallMethodFactory.getCallMethod(
                to_object_method, null);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_VALUE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_VALUE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                call_method.toString() );
        printDebugTraceMessage(message);
    }

    public Object toObject(byte[] byte_array) 
        throws Exception {
        String              message;
        Object              object_target;

        if (byte_array == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_TO_OBJECT );

        printDebugTraceMessage(message);

        if (object_instance == null) { 
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        try {
            call_method.set(object_instance, (Object)byte_array, 0);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        try {
            call_method.invoke(object_instance);
        } catch (Exception exception_object) {

            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            object_target = (Object)call_method.get(object_instance);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        return object_target;
    }
}
