/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectKill.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/02/14
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ObjectKill extends ObjectCommand {
    private final static String COMMAND_NAME = "ObjectKill";
    private final static String USAGE_ARGUMENT = "[-s] [Thread id] | -l";
    public static String OBJECT_KILL_ARGS = "l|s";
    public final static String LOOP_OPTION   = "-l";
    public final static String SYSTEM_OPTION = "-s";

    public final static String TRACE_CLASSNAME  = "oserv.ObjectKill";
    public final static String TRACE_LOOPTHREAD = "loopThread"; 
    public final static String TRACE_GETLIST    = "getList";
    public final static String TRACE_KILLTHREAD = "killThread";

    public ObjectKill() {
        super();

        String  value_string;

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }
    }

    public void loopThread() throws Exception {
        int                     iport;
        Socket                  socket_server;
        String                  vstring;
        String                  host_string;
        Object                  object_instance;
        InputStream             input_stream;
        OutputStream            output_stream;
        ObjectOutputStream      object_output_stream;
        ObjectInputStream       object_input_stream;
        RetryObjectInputStream  retry_object_input_stream;
        ObjectServerException   ose;

        host_string = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        if (host_string == null) {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.HOSTNAME_NOT_FOUND_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.HOSTNAME_NOT_FOUND_ERROR_KILL)));
            throw ose;
        }
        vstring = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (vstring != null) {
            try {
                iport = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
                ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION)));
                throw ose;
            }
        } else {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PORT_NOT_FOUND_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.PORT_NOT_FOUND_ERROR_KILL)));
            throw ose;
        }
        try {
            socket_server = connectServer(host_string, iport);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            throw exception_object;
        }
        try {
            output_stream = socket_server.getOutputStream();
        } catch (IOException exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }

        try {
            object_output_stream = new ObjectOutputStream(output_stream);
        } catch (IOException exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            object_output_stream.writeObject(OSERV_COMMAND_TYPE_SYSTEM_KEY);
            object_output_stream.writeObject(OSERV_SYSTEM_LOOP);
            object_output_stream.writeObject(OSERV_SYSTEM_LOOP);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }
        try {
            input_stream = socket_server.getInputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            retry_object_input_stream = RetryObjectInputStream.valueOf(input_stream,
                getMessageResource(),
                getDebug(),
                getLogStream()
            );
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            object_instance = retry_object_input_stream.readObject();
            if ((object_instance instanceof EOSObject) == true) {
                return;
            }
            if ((object_instance instanceof Exception) == true) {
                throw (Exception)object_instance;
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOOPTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            throw exception_object;
        }
    }

    public String[] getList(boolean system_flag) throws Exception {
        int                     iport;
        Socket                  socket_server;
        String                  vstring;
        String                  host_string;
        String[]                list;
        Object                  object_instance;
        InputStream             input_stream;
        OutputStream            output_stream;
        ObjectOutputStream      object_output_stream;
        RetryObjectInputStream  retry_object_input_stream;
        ObjectServerException   ose;

        host_string = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        if (host_string == null) {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_KILL)));
            throw ose;
        }
        vstring = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (vstring != null) {
            try {
                iport = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
                ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION)));
                throw ose;
            }
        } else {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_KILL)));
            throw ose;
        }
        try {
            socket_server = connectServer(host_string, iport);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            throw exception_object;
        }
        try {
            output_stream = socket_server.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            object_output_stream = new ObjectOutputStream(output_stream);
        } catch (IOException exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            object_output_stream.writeObject(OSERV_COMMAND_TYPE_SYSTEM_KEY);
            if (system_flag == true) {
                object_output_stream.writeObject(OSERV_SYSTEM_SYSTEM_LIST);
            } else {
                object_output_stream.writeObject(OSERV_SYSTEM_LIST);
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }
        try {
            input_stream = socket_server.getInputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            retry_object_input_stream 
                    = RetryObjectInputStream.valueOf(input_stream,
                getMessageResource(),
                getDebug(),
                getLogStream()
            );
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        Integer count_object;
        try {
            count_object = (Integer)retry_object_input_stream.readObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETLIST, 0, exception_object);
            throw exception_object; 
        }
        String[]  service_table;
      
        service_table = new String[count_object.intValue()];
        for (int i = 0; i < count_object.intValue(); i++) {
            try {
                service_table[i] = (String)retry_object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                throw exception_object;
            }
        }
        return service_table;
    }

    public void killThread(String id) throws Exception {
        int                     iport;
        Socket                  socket_server;
        String                  vstring;
        String                  host_string;
        InputStream             input_stream;
        OutputStream            output_stream;
        ObjectOutputStream      object_output_stream;
        RetryObjectInputStream  retry_object_input_stream;
        EOSObject               eos_object;
        Object                  object_instance;
        ObjectServerException   ose;

        if (id == null) {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_KILL)));
            throw ose;
        }

        host_string = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        if (host_string == null) {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_KILL)));
            throw ose;
        }
        vstring = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (vstring != null) {
            try {
                iport = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
                ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION)));
                throw ose;
            }
        } else {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_KILL)));
            throw ose;
        }
        try {
            socket_server = connectServer(host_string, iport);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            throw exception_object;
        }
        try {
            output_stream = socket_server.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            object_output_stream = new ObjectOutputStream(output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            object_output_stream.writeObject(OSERV_COMMAND_TYPE_SYSTEM_KEY);
            object_output_stream.writeObject(OSERV_SYSTEM_KILL);
            object_output_stream.writeObject(id);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }
        try {
            input_stream = socket_server.getInputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            throw ose;
        }
        try {
            retry_object_input_stream 
                    = RetryObjectInputStream.valueOf(input_stream,
                getMessageResource(),
                getDebug(),
                getLogStream()
            );
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.IO_CONSTRUCTION_ERROR_KILL,
                    getMessageResource().getMessage(
                        ObjectServerError.IO_CONSTRUCTION_ERROR_KILL)));
            throw ose;
        }
        try {
            object_instance = retry_object_input_stream.retryReadObject();
            if ((object_instance instanceof EOSObject) == true) {
                return;
            }
            if ((object_instance instanceof Exception) == true) {
                throw (Exception)object_instance;
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_KILLTHREAD, 0, exception_object);
            if (socket_server.isConnected() == true) {
                socket_server.shutdownOutput();
                socket_server.shutdownInput();
            }
            throw exception_object;
        }
    }

    /**
     * This method prints a usage.
     */
    public static void printUsage() {
        System.err.println(COMMAND_NAME + MESSAGE_SEPARATOR + USAGE_ARGUMENT);
    }

    public static void main(String[] args) {
        ParseOptions    parse_options;
        String          thread_id;
        ObjectKill      object_kill;
        boolean         system_flag;

        object_kill = new ObjectKill();
        try {
            parse_options = ParseOptions.valueOf(
                args, ObjectKill.OBJECT_KILL_ARGS,
                null, null,
                object_kill.getMessageResource(),
                object_kill.getDebug(),
                object_kill.getLogStream() );
        } catch (Exception exception_object) {
            System.err.println(
                object_kill.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_KILL,
                    object_kill.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_KILL)));
            return;
        }
        if (parse_options.checkOption(SYSTEM_OPTION) == true) {
            system_flag = true;
        } else {
            system_flag = false;
        }
        if (parse_options.checkOption(LOOP_OPTION) == true) {
            try {
                object_kill.loopThread();
            } catch (Exception exception_object) {
                System.err.println(
                    object_kill.getMessageResource().getFormattedMessage(
                        ObjectServerError.IO_ERROR_KILL,
                        object_kill.getMessageResource().getMessage(
                            ObjectServerError.IO_ERROR_KILL)));
                return;
            }
            return;
        }


        if (parse_options.getRealArgumentCount() == 0) {
            String[] thread_list;

            try {
                thread_list = object_kill.getList(system_flag);
            } catch (Exception exception_object) {
                if (exception_object.getMessage() != null &&
                    exception_object.getMessage().compareTo(
                        ObjectServerException.REFUSE_CONNECTION) == 0) {
                    System.err.println(
                        object_kill.getMessageResource().getFormattedMessage(
                            ObjectServerException.REFUSE_CONNECTION,
                            object_kill.getMessageResource().getMessage(
                                ObjectServerException.REFUSE_CONNECTION)));
                    return;
                } 
                System.err.println(
                    object_kill.getMessageResource().getFormattedMessage(
                        ObjectServerError.IO_ERROR_KILL,
                        object_kill.getMessageResource().getMessage(
                            ObjectServerError.IO_ERROR_KILL)));
                return;
            }

            if(thread_list.length == 0) {
                System.err.println( 
                    object_kill.getMessageResource().getFormattedMessage(
                        ObjectServerError.SERVICE_NOT_FOUND_ERROR_KILL,
                        object_kill.getMessageResource().getMessage(
                            ObjectServerError.SERVICE_NOT_FOUND_ERROR_KILL)));
                return;
            }
            System.out.println( object_kill.getMessageResource().getFormattedMessage(
                ObjectServerHeader.KILL_LIST,
                object_kill.getMessageResource().getMessage(
                    ObjectServerHeader.KILL_LIST)));
            System.out.println( object_kill.getMessageResource().getFormattedMessage(
                ObjectServerHeader.KILL_THREAD_NAME,
                object_kill.getMessageResource().getMessage(
                    ObjectServerHeader.KILL_THREAD_NAME)));
            for (int i = 0; i < thread_list.length; i++) {
                System.out.println(thread_list[i]);
            }
            return;
        }

        for (int i = 0; i < parse_options.getRealArgumentCount(); i++) {
            try {
                thread_id = parse_options.getRealArgument(i); 
            } catch (Exception exception_object) {
                continue;
            }
            try {
                object_kill.killThread(thread_id);
            } catch (Exception exception_object) {
                System.err.println(
                    object_kill.getMessageResource().getFormattedMessage(
                        ObjectServerError.IO_ERROR_KILL,
                        object_kill.getMessageResource().getMessage(
                            ObjectServerError.IO_ERROR_KILL)));
                return;
            }
        }
    }
}

