/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     ObjectOutputManagerWrapper.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/05/28
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ObjectOutputManagerWrapper extends ObjectServerHeader {
    private Object object_instance = null;
    private Class  object_output_manager_class = null;

    private Method to_byte_array_method = null;
    private String TO_BYTE_ARRAY_METHOD_NAME = "toByteArray";
    private Class[]  to_byte_array_args_class = {
        Object.class
    };
    private CallMethod  call_method = null;

    private static String DEBUG_OBJECT_OUTPUT_MANAGER_WRAPPER 
        = "ObjectOutputManagerWrapper";
    private static String DEBUG_TO_BYTE_ARRAY
        = "toByteArray";

    public ObjectOutputManagerWrapper(Class param_object_output_manager_class) throws Exception {
        String   message;

        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_OBJECT_OUTPUT_MANAGER_WRAPPER );

        printDebugTraceMessage(message);

        if (param_object_output_manager_class == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        object_output_manager_class = param_object_output_manager_class;

        try {
            object_instance = object_output_manager_class.newInstance();
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            to_byte_array_method = object_output_manager_class.getMethod(
                TO_BYTE_ARRAY_METHOD_NAME,
                to_byte_array_args_class);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method = CallMethodFactory.getCallMethod(
                to_byte_array_method, null);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_VALUE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_VALUE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                call_method.toString() );
        printDebugTraceMessage(message);
    }

    public byte[] toByteArray(Object object_target) 
        throws Exception {
        byte[]               byte_array;
        String               message;

        if (object_target == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_TO_BYTE_ARRAY );

        printDebugTraceMessage(message);

        if (object_instance == null) { 
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method.set(object_instance, (Object)object_target, 0);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            byte_array = (byte[])call_method.get(object_instance);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        return byte_array;
    }
}



