/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectServerException.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/07/20
 *             2.1 2010/10/08
 *
 * This class is defined as internal exception.
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

/**
 * This class is defined as internal exception.
 * 
 * @see Exception
 */
public class ObjectServerException extends Exception {

    public final static String NORMAL_END_MESSAGE = "Normal end";
    public final static String OBJECT_SERVER_INTERNAL_INTERRUPT_MESSAGE 
        = "ObjectServer Internal Interrupt";
    /**
     * Exception
     */
    public final static String REFUSE_CONNECTION 
        = "Refuse request of connection.";
    public final static String SERVICE_NOT_AVAILABLE
        = "Service not available : Please check if service is registered.";
    public final static String TIMEOUT_ERROR_EXCEPTION
        = "Timeout error exception";
    public final static String INTERNAL_ERROR_EXCEPTION
        = "Internal error exception";
    public final static String PARAMETER_ERROR_EXCEPTION
        = "Parameter error exception";
    public final static String OPEN_ERROR_EXCEPTION
        = "Open error exception";

    public final static String ASSERTION_EXCEPTION
        = "Assertion exception";

    public final static String OBJECT_OUTPUT_STREAM_NOT_SUPPORT_EXCEPTION
        = "ObjectOutputStream is not supported.";

    public final static String CLASS_FORMAT_ERROR_EXCEPTION
        = "Internal error on format of class in VariableType.";

    public final static String CLASS_NOT_FOUND_EXCEPTION
        = "Class not found";
    /** 
     * for ObjectStub.java
     */
    public final static String SCHEME_NAME_ERROR_EXCEPTION
        = "Specified scheme name isn't found in ObjectStub().";
    public final static String SERVICE_GETSERVICE_EXCEPTION
        = "Specified service is null in ObjectStub().";

    /**
     * These error message is used in ObjectServerProperties.java
     */
    public final static String INPUT_STREAM_NULL
        = "Input stream is null in ObjectServerProperties().";


    public final static String NOT_SUPPORT_OBJECT_EXCEPTION = "Not supprt for the object";
    public final static String INVOCATION_TARGET_EXCEPTION = "Invocation Target";
    public final static String ILLEGAL_ARGUMENT_EXCEPTION = "Illegal Argument";
    public final static String ILLEGAL_RETURN_VALUE_EXCEPTION = "Bad Return Value";

    public final static String NO_INITIALIZED_EXCEPTION = "No initialized";

    public final static String INSTANCE_CREATION_EXCEPTION = "Not create new instance";

    /**
     * These error message is used in ObjectServerHeader.java
     */
    public final static String COMMAND_NAME_CHECK_PROCESS_STATE
        = "Command name in checkProcessState().";
    public final static String OBJECT_INPUT_STREAM_NULL
        = "ObjectInputStream is null in retryReadObject().";
    public final static String BUFFER_NULL_RETRY_READ_STREAM
        = "Buffer is null in retryReadStream().";


    /**
     * ArgumentInputStream
     */
    public final static String NO_OPERATION_EXCEPTION
        = "not operate yet.";
    public final static String TYPE_NOT_FOUND_EXCEPTION 
        = "type was not found.";

    public final static String NULL_RETURN
        = "Return value was null.";

    public final static String DEBUG_RETURN
        = "DEBUG RETURN:";
/**
    public final static String DEBUG_EXCEPTION_KEY
        = "Exception happens.";
*/
    public final static String USER_DEBUG_SEPARATOR = ":";


    /**
     * for ConnectedSocket
     */
    public final static String HOSTNAME_PARAMETER_ERROR_EXCEPTION
        = "Hostname parameter error exception";

    /**
     * This constructor is called with basic exception information.
     *
     * @param    Exception - basic exception
     */
    public ObjectServerException(Exception exception_object) {
        super(exception_object);
    }

    /**
     * This constructor is called with error message.
     *
     * @param    String - Message String
     */
    public ObjectServerException(String message) {
        super(message);
    }
}
