/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectServerHeader.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/07/22
 *             2.1 2010/09/21 define Read Timeout
 *             2.1 2010/10/08
 *             2.1 2011/09/22 for Forward
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

/**
 * This class is header file holding common values in Object Service Server 
 * (OServ) System.
 *
 * @see        java.lang.Thread
 */
public class ObjectServerHeader extends Thread {
    public static final String OSERV_SCHEME = "oserv";

    public final static String CURRENT_DIRECTORY = ".";
    public final static String PARENT_DIRECTORY = "..";
    public final static char DOS_DIRECTORY_SEPARATOR_CHAR = '\\';
    public final static char DOS_DRIVE_SEPARATOR_CHAR = ':';
    public static String PROCESS_WORKING_DIRECTORY_KEY = "PWD";
    public final static String DOS_DIRECTORY_SEPARATOR = "\\";
    public final static String DOS_WORK_DIRECTORY = "work";
    public final static int ERROR_EXIT_CODE = -1;
    public final static String ERROR_FILE_COULDNOT_OPEN
        = "file couldnot be open.";
    public final static String ERROR_INTERNAL_ERROR
        = "Internal error";

    public final static String START_BRACKET = "[";
    public final static String END_BRACKET = "]";
/*
    private static MessageResource message_resource = new MessageResource();
*/
    private MessageResource message_resource = new MessageResource();

    public final static String EMPTY_MESSAGE = "";
    public static int RETRY_NUMBER = 3;
    private int retry_count = RETRY_NUMBER;

    public final static String SERVICE_THREAD_TYPE
        = "Service Thread Type";
    public final static String MONITOR_THREAD_TYPE
        = "Monitor Thread Type";
    public final static String SYSTEM_THREAD_TYPE
        = "System Thread Type";
    public final static String TRANSIENT_THREAD_TYPE
        = "Transient Thread Type";
    public final static String FORWARD_THREAD_TYPE
        = "Forward Thread Type";
    public final static String REGISTER_THREAD_TYPE
        = "Register Thread Type";
    public final static String UNKNOWN_THREAD_TYPE
        = "Unknown Thread Type";

    public final static String FILE_URL_PROTOCOL = "file://";
    public final static String JAR_URL_PROTOCOL = "jar:";
    public final static String JAR_URL_HEAD = "jar:file:";
    public final static String JAR_URL_END  = "!/";

    public final static String OSERV_PROPERTIES = "oserv.properties";
    public final static String PACKAGE_NAME = "oserv";
    public final static String JAVA_CLASS_PATH = "java.class.path";
    public final static String SUN_BOOT_CLASS_PATH = "sun.boot.class.path";

    public final static String READ_TIMEOUT_EXCEPTION
            = "Read Timeout";

    public final static String __INPUTSTREAM = "__java.io.InputStream";
    public final static String TEMPORARY_DIRECTORY = "tmp";
    public final static int MAX_STREAM = 100;

    public final static String[] pathlist = {
	JAVA_CLASS_PATH };

    public final static String OSERV_TABLE_SIZE_KEY   = "oserv.table.size";
    public final static String OSERV_SIZE_VALUE       = "50";
    public final static String OSERV_WORK_PATH_KEY    = "oserv.work.path";
    public final static String OSERV_PERSISTENT_DIRECTORY_KEY
	= "oserv.persistent.path";
    public final static String OSERV_PERSISTENT_DIRECTORY_VALUE = "persistent";
    public final static String OSERV_TRANSIENT_DIRECTORY_KEY
	= "oserv.transient.path";
    public final static String OSERV_TRANSIENT_DIRECTORY_VALUE = "transient";
    public final static String OSERV_TRANSIENT_HOME_KEY = "oserv.transient.home";
    public final static String OSERV_USER_WORK_KEY
	= "oserv.user.work";
    public final static String OSERV_USER_WORK_VALUE
	= "userwork";
    public final static String USER_DIR_KEY = "user.dir";
    public final static String USER_HOME_KEY = "user.home";

    /**
     * for forward
     */
    public final static String OSERV_FORWARD_FILE_KEY 
        = "oserv.forward.file";

    public final static String OSERV_OBJECT_TABLE_FILE = "oserv.obj";

    public final static String OSERV_FORWARD_TABLE_FILE = "forward.srv";

    public final static String OSERV_REQUEST_PORT_KEY = "oserv.request.port";
    public final static String REQUEST_PORT_NUMBER_VALUE = "2000";

    public final static String OSERV_COMMAND_TYPE_REQUEST_KEY
        = "oserv.command.request";
    public final static String OSERV_COMMAND_TYPE_FORWARD_KEY
        = "oserv.command.forward";
    public final static String OSERV_COMMAND_TYPE_OBJECT_KEY
        = "oserv.command.object";
    public final static String OSERV_COMMAND_TYPE_SYSTEM_KEY
        = "oserv.command.system";
    public final static String OSERV_COMMAND_TYPE_TRANSIENT_KEY
        = "oserv.command.transient";

    /**
     * feature for refusing request of connection
     *
     * host    oserv://host name/
     * network oserv://network address/?command=network&netmask=0xffff0000
     *
     **/
    public final static String OSERV_REQUEST_REFUSE_KEY = "oserv.request.refuse";
    public final static String OSERV_OBJECT_REFUSE_KEY = "oserv.object.refuse";
    public final static String OSERV_FORWARD_REFUSE_KEY = "oserv.forward.refuse";
    public final static String OSERV_SYSTEM_REFUSE_KEY = "oserv.system.refuse";

    public final static String OSERV_REPLY_PORT_KEY = "oserv.reply.port";
    public final static String REPLY_PORT_NUMBER_VALUE = "2002";

    public final static String OSERV_CONNECTION_TIMEOUT_KEY = "oserv.connection.timeout";
    public final static String CONNECTION_TIMEOUT_VALUE = "3000";

    public final static String OSERV_EXECUTION_WAITTIME_KEY = "oserv.execution.waittime";
    public final static String EXECUTION_WAITTIME_VALUE = "10000";
    public final static String OSERV_SERVICE_HOSTNAME_KEY = "oserv.service.hostname";
    public final static String OSERV_RETRY_COUNT_KEY = "oserv.retry.count";
    public final static String OSERV_RETRY_COUNT_VALUE = "4";
    public final static String SERVICE_HOSTNAME_VALUE = "localhost";

    public final static String OSERV_DEBUG_LEVEL_KEY = "oserv.debug.level";
    public final static String OSERV_DEBUG_LEVEL_VALUE = "0";

    public final static String OSERV_DEBUG_FUNCTION_KEY = "oserv.debug.function";
    public final static String OSERV_DEBUG_FUNCTION_ON_KEY  = "on";
    public final static String OSERV_DEBUG_FUNCTION_OFF_KEY = "off";

    public final static String OSERV_MESSAGE_LANGUAGE_KEY = "oserv.message.language";
    public final static String OSERV_MESSAGE_LANGUAGE_VALUE = "jpn";

    public final static String OSERV_LOG_FILE_KEY   = "oserv.log.file";
    public final static String OSERV_LOG_FILE_CONSOLE_VALUE = "console";
    public final static String OSERV_LOG_FILE_TTY_VALUE = "tty";
    public final static String CONSOLE_DEVICE_PATH = "/dev/console";

    public final static String OSERV_BUFFER_MAXIMUM_KEY = "oserv.buffer.maximum";
    public final static String BUFFER_MAXIMUM_VALUE = "1000";

    public static final int BUFFER_SIZE = 4096;
    public final int SEND_BUFFER_SIZE = 10240;

    public final static String OSERV_MAX_THREAD_KEY = "oserv.max.thread";
    public final static int OSERV_MIN_THREAD_VALUE = 10;
    public final static int OSERV_MAX_THREAD_LIMIT_VALUE = 1000;
    public final static int OSERV_MAX_THREAD_DEFAULT_VALUE = 20;
    public final static String OSERV_MAX_THREAD_DEFAULT = "20";

    public final static String THREAD_ID_HEAD = "Thread-";
    public final static String THREAD_ID_END  = ",5,main";

    /**
     * for forward
     */
    public final static String OSERV_FORWARD_CLIENTS_KEY = "oserv.forward.clients";
    public final static String OSERV_FORWARD_SERVERS_KEY = "oserv.forward.servers";
    public final static String OSERV_FORWARD_TUNNEL_KEY = "oserv.forward.tunnel";
    public final static String FORWARD_TUNNEL_VALUE = "0";
    /**
     * for system service
     */
    public final static String OSERV_SYSTEM_FORBIDDEN_KEY = "oserv.system.forbidden";
    public final static String OSERV_SYSTEM_LIST    = "SystemList";
    public final static String OSERV_SYSTEM_KILL    = "SystemKill";
    public final static String OSERV_SYSTEM_LOOP    = "SystemLoop";
    public final static String OSERV_SYSTEM_SERVICE = "SystemService";
    public final static String OSERV_SYSTEM_SET_SERVICE_TYPE = "SystemSetServiceType";
    public final static String OSERV_SYSTEM_SYSTEM_LIST    = "SystemSystemList";
    public final static String OSERV_SYSTEM_SYSTEM_KILL    = "SystemSystemKill";

    public final static String OSERV_REGISTER_DELETE   = "RegisterDelete";
    public final static String OSERV_REGISTER_REGISTER = "RegisterRegister";
    public final static String OSERV_REGISTER_LIST     = "RegisterList";

    public final static int OSERV_DEBUG_RUN_VALUE = 2;
    public final static int OSERV_DEBUG_TRACE_VALUE = 4;
    public final static int OSERV_DEBUG_USER_VALUE = 6;
    public final static int OSERV_DEBUG_FULL_VALUE = 8;

    public final static int COMMAND_SEPARATOR_COUNT = 5;
    public final static int COMMAND_COUNT = 6;
    public final static int COMMAND_CLASS_NAME_INDEX = 0;
    public final static int COMMAND_INTERFACE_NAME_INDEX = 1;
    public final static int COMMAND_FUNCTION_NAME_INDEX = 2;
    public final static int COMMAND_RETURN_INDEX = 3;
    public final static int COMMAND_PARAMETER_INDEX = 4;
    public final static int COMMAND_EXCEPTION_INDEX = 5;

    public final static char COMMENT_LINE = '#';
    public final static String OBJECT_SEPARATOR = "/";
    public final static String OBJECT_DOT_SEPARATOR = ".";
    public final static String ADDRESS_SEPARATOR = ".";
    public final static String QUERY_SEPARATOR = "&";
    public final static String OPTION_SEPARATOR = "?";
    public final static String QUERY_PARAMETER_SEPARATOR = "=";
    public final static String MESSAGE_LIST_SEPARATOR = "=";
    public final static String MESSAGE_SEPARATOR = " ";
    public final static String SUFFIX_SEPARATOR = ".";
    public final static String DIRECTORY_SEPARATOR = "/";
    public final static String LIBRARY_SEPARATOR = ":";
    public final static String LINE_NUMBER_SEPARATOR = ":";
    public final static String PARAMETER_SEPARATOR = ",";
    public final static String VARIABLE_SEPARATOR = "_";
    public final static String START_PARENTHESIS = "(";
    public final static String END_PARENTHESIS = ")";

    public final static String END_IOSTREAM = "End I/O Stream";

    /**
     * for forward
     */
    public final static String ITEM_SEPARATOR = ",";

    public final static String PADDING_STRING = " ";
    public final static String BLANK_STRING = " ";
    public final static String OPTION_KEY = "-";

    public final static String FORWARD_COMMAND_KEY = "command";
    public final static String FORWARD_EXECUTE_KEY = "execute";
    public final static String FORWARD_SERVICE_KEY = "service";
    public final static String FORWARD_FILE_KEY = "file";
    public final static String FORWARD_DELETE_COMMAND = "delete";
    public final static String FORWARD_EXECUTE_COMMAND = "execute";
    public final static String FORWARD_LIST_COMMAND = "list";
    public final static String FORWARD_GET_COMMAND="get"; 
    public final static String FORWARD_SERVICE_COMMAND = "service";

    public final static String KILL_LIST = "Service thread list in ObjectServer.";
    public final static String KILL_THREAD_NAME = "Thread name";

    public final static String FORWARD_HOP_COUNT_KEY = "hop";
    public final static String FORWARD_TERMINAL_KEY = "terminal";

    public static String directory_separator_key = DIRECTORY_SEPARATOR;

    public final static String TEMP_SUFFIX = "temp";
    public final static String JAR_SUFFIX = "jar";
    public final static String CLASS_SUFFIX = "class";
    public final static String JAVA_SUFFIX = "java";
    public final static String VARIABLE_SUFFIX = "var";

    public final static String PARAMETER_NAME_KEY = "_p";

    public final static String PROCESS_LIST = "/bin/ps -p";
    public final static String PROCESS_COMMAND_LIST = "/bin/ps -e";
    public final static String BASH_COMMAND = "/bin/bash";
    public final static String PARENT_ID = "echo $PPID";
    public final static String PROCESS_ID = "./.pid";

    public final static String VOID_TYPE_KEY = "void";
/**
    public final static String STRING_TYPE_KEY = "java.lang.String";
    public final static String INTEGER_TYPE_KEY = "java.lang.Integer";
    public final static String LONG_TYPE_KEY = "java.lang.Long";
    public final static String FLOAT_TYPE_KEY = "java.lang.Float";
    public final static String DOUBLE_TYPE_KEY = "java.lang.Double";
    public final static String BOOLEAN_TYPE_KEY = "java.lang.Boolean";
    public final static String BYTE_TYPE_KEY = "java.lang.Byte";
    public final static String CHAR_TYPE_KEY = "java.lang.Character";
    public final static String SHORT_TYPE_KEY = "java.lang.Short";
    public final static String OBJECT_TYPE_KEY = "java.lang.Object";
**/

    public final static String OSERV_SOD_OBJECT = "oserv.SODObject";
    public final static String OSERV_EOD_OBJECT = "oserv.EODObject";
/*
    public final static String OSERV_NETWORK_OBJECT = "oserv.NetworkObject";
*/
/*
    public final static String OSERV_RETRY_OBJECT_INPUT_STREAM 
        = "oserv.RetryObjectInputStream";
*/
    public final static String OSERV_OBJECT_INPUT_MANAGER
        = "oserv.ObjectInputManager";
    public final static String OSERV_OBJECT_OUTPUT_MANAGER
        = "oserv.ObjectOutputManager";
    public final static String OSERV_BASE_VARIABLE = "oserv.BaseVariable";
    public final static String OSERV_WRITE_VARIABLE = "oserv.WriteVariable";
    public final static String OSERV_READ_VARIABLE = "oserv.ReadVariable";
    public final static String OSERV_CALL_METHOD = "oserv.CallMethod";
    public final static String OSERV_CALL_METHOD_INVOKE = "oserv.CallMethodInvoke";
    public final static String OSERV_CALL_METHOD_TRANSFER = "oserv.CallMethodTransfer";


    public final static String JAVA_INVOCATION_TARGET_EXCEPTION_KEY
        = "java.lang.reflect.InvocationTargetException";
    public final static String JAVA_IO_SERIALIZABLE_TYPE_KEY = "java.io.Serializable";
    public final static String JAVA_OUTPUTSTREAM_TYPE_KEY = "java.io.OutputStream";
    public final static String JAVA_OBJECTOUTPUTSTREAM_TYPE_KEY = "java.io.ObjectOutputStream";
    public final static String JAVA_FILEOUTPUTSTREAM_TYPE_KEY = "java.io.FileOutputStream";
    public final static String JAVA_BUFFEREDOUTPUTSTREAM_TYPE_KEY = "java.io.BufferedOutputStream";
    public final static String JAVA_BYTEARRAYOUTPUTSTREAM_TYPE_KEY = "java.io.ByteArrayOutputStream";
    public final static String JAVA_FILTEROUTPUTSTREAM_TYPE_KEY = "java.io.FilterOutputStream";
    public final static String JAVA_PIPEDOUTPUTSTREAM_TYPE_KEY = "java.io.PipedOutputStream";
    public final static String JAVA_DATAOUTPUTSTREAM_TYPE_KEY = "java.io.DataOutputStream";
    public final static String JAVA_PRINTSTREAM_TYPE_KEY = "java.io.PrintStream";


    public final static String JAVA_INPUTSTREAM_TYPE_KEY = "java.io.InputStream";
    public final static String JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY = "java.io.ByteArrayInputStream";
    public final static String JAVA_FILEINPUTSTREAM_TYPE_KEY = "java.io.FileInputStream";
    public final static String JAVA_FILTERINPUTSTREAM_TYPE_KEY = "java.io.FilterInputStream";
    public final static String JAVA_PIPEDINPUTSTREAM_TYPE_KEY = "java.io.PipedInputStream";
    public final static String JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY = "java.io.SequenceInputStream";
    public final static String JAVA_OBJECTINPUTSTREAM_TYPE_KEY = "java.io.ObjectInputStream";
    public final static String JAVA_DATAINPUTSTREAM_TYPE_KEY = "java.io.DataInputStream";
    public final static String JAVA_LINENUMBERINPUTSTREAM_TYPE_KEY = "java.io.LineNumberInputStream";
    public final static String JAVA_BUFFEREDINPUTSTREAM_TYPE_KEY = "java.io.BufferedInputStream";
    public final static String JAVA_PUSHBACKINPUTSTREAM_TYPE_KEY = "java.io.PushbackInputStream";
    public final static String JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY = "java.io.StringBufferInputStream";

    public final static String SEMICOLON = ";";
    public final static char ARRAY_INT = 'I';
    public final static char ARRAY_BYTE = 'B';
    public final static char ARRAY_LONG = 'J';
    public final static char ARRAY_SHORT = 'S';
    public final static char ARRAY_CHAR = 'C';
    public final static char ARRAY_DOUBLE = 'D';
    public final static char ARRAY_FLOAT = 'F';
    public final static char ARRAY_BOOLEAN = 'Z';
    public final static char ARRAY_REFERENCE = 'L';

    public final static String DEBUG_NOT_FOUND_PROPERTY_FILE 
        = "Not found properties file";
    public final static String DEBUG_FILE_ENTRY_IN_JAR
        = "file entry in jar";
    public final static String DEBUG_LOAD_PROPERTIES_FROM_JAR 
        = "loadPropertiesFromJar().";
    public final static String DEBUG_READ_PROPERTIES_FROM_JAR 
        = "readPropertiesFromJar().";
/**
    public final static String DEBUG_ENTER = "Enter";
    public final static String DEBUG_JAR_ARCHIVE     = "jar archive";
    public final static String DEBUG_LOGFILE_NULL    = "logfile was null";
    public final static String DEBUG_LOG             = "log";
**/
/**
    public final static String DEBUG_LOG_SWITCH_TO_TTY
	= "log switch to tty";
**/
    public final static String DEBUG_READ_PROPERTY_FILE 
        = "Call getProperty()";
/*
    public final static String DEBUG_LEVEL           = "debug level";
    public final static String DEBUG_EXCEPTION_KEY   = "DEBUG Exception";
*/
    public final static String DEBUG_LIBRARY_PATH_KEY = "library path key";
    public final static String DEBUG_LIBRARY_ENTRY_KEY = "library entry";
    public final static String DEBUG_LIBRARIES = "libraries";
    public final static String DEBUG_CREATE_PROPERTIES = "create properties";
    public final static String DEBUG_LOAD_PROPERTIES = "load properties";
    public final static String DEBUG_SETUP_PROPERTIES = "setup properties";

    public final static String DEBUG_PROPERTIES_FILE = "debug properties";

    public final static String DEBUG_CURRENT_PROPERTIES
        = "current properties";

    public final static String TRACE_CLASSNAME           = "oserv.ObjectServerHeader";
    public final static String TRACE_GETPROPERTIES       = "getProperties";
    public final static String TRACE_RETRYREADOBJECT     = "retryReadObject";
    public final static String TRACE_LOADPROPERTYVALUE   = "loadPropertyValue";
    public final static String TRACE_LOADPROPERTYFROMJAR = "loadPropertyFromJar";
    public final static String TRACE_SETUPLOGFILE = "setupLogFile";
    public final static String TRACE_CHECKPROCESSSTATE = "checkProcessState";

    public final static String READ_PARAMETER = "Read Parameter";
    public final static String WRITE_PARAMETER = "Write Parameter";
    public final static String EXTRACT_PARAMETER = "Extract Parameter";

    public final static int sleep_interval = 1000;
    public final static int LOCK_SLEEP = 1000;
    public static int timeout    = 3000;
    public static int execution_waittime = 10000;

    public ObjectServerHeader() {
        super();
    }

    public ObjectServerHeader(Thread runnable) {
        super(runnable);
    }

    public ObjectServerHeader(Thread runnable, String name) {
        super(runnable, name);
    }

    private static ObjectServerProperties sproperties = null;
    private static String debugProperty = null;
    private static String tablesizeProperty = null;
    private static String timeoutProperty = null;
    private static String sleepProperty = null;
    private static String objectPortProperty = null;
    private static String requestPortProperty = null;
    private static String forwardPortProperty = null;
    private static String forwardClientsProperty = null;
    private static String retryProperty = null;
    private static String logfileProperty = null;
    private static String replyPortProperty = null;
    private static String bufferMaxProperty = null;
    private static String hostnameProperty = null;
    private static String forbiddenProperty = null;
    private static String languageProperty = null;

    private int debug = 0;
    private LogPrintStream logStream = null;

    /**
     * This method sets Stream for switching.
     *
     * @param      LogPrintStream 
     */
    protected void setLogStream(LogPrintStream ps) {
        logStream = ps;
    }

    protected LogPrintStream getLogStream() {
        return logStream;
    }

    /**
     * This method sets debug level.
     *
     * @param      int - debug level  
     * @return     void
     */
    protected void setDebug(int level) {
        debug = level;
    }

    protected int getDebug() {
        return debug;
    }

    protected void setRetryCount(int count) {
        retry_count = count;
    }

    protected int getRetryCount() {
        return retry_count;
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public void setMessageLanguage(String language) {
        if (language == null) {
            return;
        }
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        message_resource.setLanguage(language);
    }

    public void printDebugTraceMessage( String  message) {

        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {

            getLogStream().printLogln( message );
        }
    }


    public void printExceptionMessage(String classname, String function, int nest, Exception exception_object) {
        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            writeExceptionTrace(getLogStream(), getMessageResource(), 
                exception_object, classname, function, nest);
        }
    }

    public void printExceptionMessage(LogPrintStream param_log_print_stream,
        String classname, String function, int nest, Exception exception_object) {
        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            writeExceptionTrace(param_log_print_stream, getMessageResource(), 
                exception_object, classname, function, nest);
        }
    }


    public void writeExceptionTrace(
        LogPrintStream param_log_print_stream, 
        MessageResource param_message_resource,
        Exception param_exception,
        String    classname,
        String    function,
        int       nest
    ) {

        if (param_log_print_stream == null || param_message_resource == null 
            || param_exception == null || function == null || classname == null) {
            return ;
         }

         param_log_print_stream.printLogln(
             param_message_resource.getFormattedMessage(
                 ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                 param_message_resource.getMessage(
                    ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                 ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                 TraceMessage.getCallerMethod(param_exception, classname, function, nest),
                 ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                 param_exception.toString() ));
    }

    protected ObjectServerException getObjectServerException(
        String  form_key,    String msg_key
    ) {
        ObjectServerException  object_server;

        object_server = getObjectServerException(form_key, msg_key, null);
        return object_server;
    }

    protected ObjectServerException getObjectServerException(
        String  form_key,    String msg_key,   String message
    ) {
        ObjectServerException    object_server_exception;

        if (message == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    form_key,
                    getMessageResource().getMessage( msg_key )));
            return object_server_exception;
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                form_key,
                getMessageResource().getMessage( msg_key ),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                message));

        return object_server_exception;
    }

    /**
     * This method extracts value string to correspond to specified property
     *
     * @param     String - key string for specifying specific property
     * @return    String - value string to correspond to specified property
     */
    protected String getPropertyValue(String key) {
        String value;
        if (sproperties == null) {
            return null;
        }
        if (key == null) {
            return null;
        }
        if (key.compareTo(OSERV_DEBUG_LEVEL_KEY) == 0) {
            if (debugProperty != null) {
                return debugProperty;
            }
        } else if (key.compareTo(OSERV_TABLE_SIZE_KEY) == 0) {
            if (tablesizeProperty != null) {
                return tablesizeProperty;
            }
        } else if (key.compareTo(OSERV_SYSTEM_FORBIDDEN_KEY) == 0) {
            if (forbiddenProperty != null) {
                return forbiddenProperty;
            }
        } else if (key.compareTo(OSERV_SERVICE_HOSTNAME_KEY) == 0) {
            if (hostnameProperty != null) {
                return hostnameProperty;
            }
        } else if (key.compareTo(OSERV_CONNECTION_TIMEOUT_KEY) == 0) {
            if (timeoutProperty != null) {
                return timeoutProperty;
            }
        } else if (key.compareTo(OSERV_REQUEST_PORT_KEY) == 0) {
            if (requestPortProperty != null) {
                return requestPortProperty;
            }
        } else if (key.compareTo(OSERV_FORWARD_CLIENTS_KEY) == 0) {
            if (forwardClientsProperty != null) {
                return forwardClientsProperty;
            }
        } else if (key.compareTo(OSERV_RETRY_COUNT_KEY) == 0) {
            if (retryProperty != null) {
                return retryProperty;
            }
        } else if (key.compareTo(OSERV_BUFFER_MAXIMUM_KEY) == 0) {
            if (bufferMaxProperty != null) {
                return bufferMaxProperty;
            }
        } else if (key.compareTo(OSERV_LOG_FILE_KEY) == 0) {
            return logfileProperty;
        }
        value = sproperties.getProperty(key);
        return value;
    }

    protected void loadPropertyValue(String key) {
        FileInputStream fin;
        String pName;
        String vName;
        String ppName;
        String vvName;
        Enumeration en;
        Properties  pr;
        String      packname;
        String      path;
        String     fpath;
        StringTokenizer stk;
        String ival;
        File   fes;
        Boolean found;

        if (key == null) {
            return;
        }
        if (key.compareTo(OSERV_BUFFER_MAXIMUM_KEY) == 0 ||
            key.compareTo(OSERV_RETRY_COUNT_KEY) == 0 ||
            key.compareTo(OSERV_WORK_PATH_KEY) == 0 ||
            key.compareTo(OSERV_PERSISTENT_DIRECTORY_KEY) == 0 ||
            key.compareTo(OSERV_REQUEST_PORT_KEY) == 0 || 
            key.compareTo(OSERV_REPLY_PORT_KEY) == 0 ||
            key.compareTo(OSERV_CONNECTION_TIMEOUT_KEY) == 0 ||
            key.compareTo(OSERV_FORWARD_CLIENTS_KEY) == 0 ||
            key.compareTo(OSERV_SERVICE_HOSTNAME_KEY) == 0 ||
            key.compareTo(OSERV_PERSISTENT_DIRECTORY_KEY) == 0 ) {
            return ;
        }

        packname = PACKAGE_NAME;
        packname = replace(packname, OBJECT_DOT_SEPARATOR, OBJECT_SEPARATOR);
        found = false;
        fes = null;
        for (int i = 0; i < pathlist.length; i++ ) {
            ival = System.getProperty(pathlist[i]);
            stk = new StringTokenizer(ival, LIBRARY_SEPARATOR);
            while(stk.hasMoreTokens()) {
                path = stk.nextToken();
                fes = new File(path);
                if (fes.isDirectory() == false) {
                    continue;
                }
                fpath = path + File.separator + 
                        packname + File.separator + OSERV_PROPERTIES;
                fes = new File(fpath);
                if (fes.exists() == true) {
                    found = true;
                    break;
                }
            }
            if (found == true) {
                break;
            }
        }
        if (fes == null) {
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln( 
                    getMessageResource().getFormattedMessage(
                        DEBUG_NOT_FOUND_PROPERTY_FILE,
                        getMessageResource().getMessage(
                            DEBUG_NOT_FOUND_PROPERTY_FILE)));
            }
            return;
        }
        fin = null;
        try {
            fin = new FileInputStream(fes);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_LOADPROPERTYVALUE, 0, exception_object);
            return;
        }
        if (fin != null) {
            pr = new Properties();
            PropertyLoader property_loader;
            try {
                property_loader = new PropertyLoader(fin);
            } catch (Exception exception_object) {
                return;
            }
            while (true) {
                try {
                    pName = property_loader.getKey();
                    if (pName == null) { break; }
                    vName = property_loader.getValue();
                } catch (Exception exception_object) {
                    break;
                }
                if (pName == null || vName == null) { continue; }
                pr.setProperty(pName, vName);
            }

            found = false;
            pName = null;
            vName = null;
            en = pr.propertyNames();
            while (en.hasMoreElements()) {
                pName = (String)en.nextElement();
                if (pName.compareTo(key) != 0) {
                    continue;
                }
                vName = pr.getProperty(pName);
                break;
            }
            if (vName == null) {
                return;
            }
            if (key.compareTo(OSERV_LOG_FILE_KEY) == 0) {
                logfileProperty = vName;
            } else if (key.compareTo(OSERV_TABLE_SIZE_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    vName = OSERV_SIZE_VALUE;
                }
                tablesizeProperty = vName;
            } else if (key.compareTo(OSERV_DEBUG_LEVEL_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    vName = OSERV_DEBUG_LEVEL_VALUE;
                }
                debugProperty = vName;

            } else if (key.compareTo(OSERV_CONNECTION_TIMEOUT_KEY) == 0) {
		try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    vName = CONNECTION_TIMEOUT_VALUE;
                }
                timeoutProperty = vName;
            }
      
            if (vName != null) {
                pr = (Properties)sproperties;
                sproperties = new ObjectServerProperties();
                sproperties.setMessageResource(getMessageResource());
                en = pr.propertyNames();
                while (en.hasMoreElements()) {
                    ppName = (String)en.nextElement();
                    if (ppName.compareTo(pName) == 0) {
                        sproperties.setProperty(pName, vName);
                        continue;
                    }
                    vvName = pr.getProperty(ppName);
                    sproperties.setProperty(ppName, vvName);
                }
            }
        }
        return;
    }

    protected boolean checkProcessState(String command_name) 
        throws Exception {
        File fdes = new File(PROCESS_ID);
        String pid;
        String rpid;
        String head;
        String line;
        String command;
        Process proc;
        DataInputStream dis;
        Runtime rt = Runtime.getRuntime();

        if (command_name == null) {
            ObjectServerException ose =
                new ObjectServerException(
                    message_resource.getFormattedMessage(
                        ObjectServerException.COMMAND_NAME_CHECK_PROCESS_STATE,
                        message_resource.getMessage(
                            ObjectServerException.COMMAND_NAME_CHECK_PROCESS_STATE)));
            throw ose;
        }

        if (fdes.exists() != false) {
            try {
                FileInputStream fis = new FileInputStream(fdes);
                dis = new DataInputStream(fis);
            } catch (Exception exception_object) {
                ObjectServerException ose =
                    new ObjectServerException(
                        message_resource.getFormattedMessage(
                            ObjectServerException.OPEN_ERROR_EXCEPTION,
                            message_resource.getMessage(
                                ObjectServerException.OPEN_ERROR_EXCEPTION),
                    BLANK_STRING,
                    PROCESS_ID ));
                throw ose;
            }
            pid = null;
            while(true) {
                try {
                    pid = dis.readLine();
                    if (pid != null) { break; }
                } catch (Exception exception_object) {
                    break;
                }
            }
            if (pid != null && pid.length() != 0) {
                try {
                    Integer.valueOf(pid).intValue();

                    command = PROCESS_LIST;
                    command = command.concat(BLANK_STRING + pid);
                    proc = rt.exec(command);

                    dis = new DataInputStream(proc.getInputStream());
                    head = dis.readLine();
                    if (head != null) {
                        rpid = dis.readLine();
                        if (rpid != null) {
                            if (rpid.indexOf(pid) == 0) {
                                return false;
                            }
                        }
                    }
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_CHECKPROCESSSTATE, 0, exception_object);
                    ObjectServerException ose =
			new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
                    throw ose;
                }
            }
        }
        try {
            command = BASH_COMMAND;
            proc = rt.exec(command);
            PrintStream pts;
            pts = new PrintStream(proc.getOutputStream());
            pts.println(PARENT_ID);
            pts.close();
            try {
                proc.waitFor();
            } catch (Exception second_exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, 
                    TRACE_CHECKPROCESSSTATE, 0, second_exception_object);
            }
            dis = new DataInputStream(proc.getInputStream());
            line = dis.readLine();
        } catch (Exception exception_object) {
            line = null;
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CHECKPROCESSSTATE, 0,
                exception_object);
        }

        try {
            FileOutputStream fos = new FileOutputStream(fdes);
            PrintStream      pst = new PrintStream(fos);
            pst.println(line);
            pst.close();
        } catch (Exception exception_object) {
            fdes.delete();
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CHECKPROCESSSTATE, 0,
                exception_object);
            ObjectServerException ose =
                new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerException.INTERNAL_ERROR_EXCEPTION)));
            throw ose;
        }
        return true;
    }


    /**
     * This method initializes property values based on default, System and 
     * initial file (oserv.properties).
     *
     * @param     nothing
     * @return    void
     */
    protected void getProperties() {
        FileInputStream fin;
        String ival;
        String path;
        String fpath;
        File   fes = null;
        StringTokenizer stk;
        boolean found = false;
        String pName, vName;
        Enumeration en;
        Properties pr;
        String packname;
        String workpath;
        String logfile;
        ObjectServerProperties ospr;
        String debug_level_string;

        debug_level_string = System.getProperties().getProperty(
            OSERV_DEBUG_LEVEL_KEY );
        if (debug_level_string != null) {
            try {
                setDebug(Integer.valueOf(debug_level_string).intValue());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_GETPROPERTIES, 0, exception_object);
            }

            logfile = System.getProperties().getProperty(
                    OSERV_LOG_FILE_KEY );
            setupLogFile(logfile);
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_DEBUG_VALUE, 
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_DEBUG_VALUE), 
                        String.valueOf(debug))); 
            }
        }
        packname = getClass().getPackage().getName();
        packname = PACKAGE_NAME;

        try {
            fpath = packname + File.separator + OSERV_PROPERTIES;
            fes = new File(fpath);
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTIES_FILE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTIES_FILE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        fpath,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(fes.exists())));
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETPROPERTIES, 0, exception_object);
            fes = null;
        }
        if (fes == null || fes.exists() == false) {
            for (int i = 0; i < pathlist.length; i++ ) {
                ival = System.getProperty(pathlist[i]);
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    getLogStream().printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_LIBRARY_PATH_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_LIBRARY_PATH_KEY),
                            pathlist[i]));
                }

                stk = new StringTokenizer(ival, LIBRARY_SEPARATOR);
                while(stk.hasMoreTokens()) {
                    path = stk.nextToken();
                    if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                        getLogStream().printLogln(
                            getMessageResource().getFormattedMessage(
                                ObjectServerHeader.DEBUG_LIBRARY_ENTRY_KEY,
                                getMessageResource().getMessage(
                                    ObjectServerHeader.DEBUG_LIBRARY_ENTRY_KEY),
                                DEBUG_LIBRARIES, 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                path));
                    }
                    fes = new File(path);

                    if (fes.isDirectory() == false) {
                        String jar_suffix_string;
                        int    jar_suffix_index;
                        jar_suffix_index = path.lastIndexOf(SUFFIX_SEPARATOR);

                        if (jar_suffix_index >= 0) {
                            jar_suffix_string = path.substring(jar_suffix_index+1);
                            if (jar_suffix_string.compareTo(JAR_SUFFIX) == 0) {
                                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                                    getLogStream().printLogln( 
                                        getMessageResource().getFormattedMessage(
                                        ObjectServerDebug.DEBUG_JAR_NAME,
                                            getMessageResource().getMessage(
                                            ObjectServerDebug.DEBUG_JAR_NAME),
                                            path ));
                                 }
                                 try {
                                     if (sproperties == null) {
                                         sproperties = new ObjectServerProperties();
                                         sproperties.setMessageResource(getMessageResource());
                                         if (getDebug() > 0) {
                                             sproperties.setDebug(getDebug(), getLogStream());
                                         }
                                     }
                                     loadPropertyFromJar(path);
                                 } catch (Exception exception_object) {
                                     printExceptionMessage(TRACE_CLASSNAME, TRACE_GETPROPERTIES, 0, exception_object);
                                 }
                            }
                        }
                        continue;
                    }
                    fpath = path + File.separator + 
                        packname + File.separator + OSERV_PROPERTIES;
                    fes = new File(fpath);

                    if (fes.exists() == true) {
                        found = true;
                        break;
                    }
                }
                if (found == true) {
                    break;
                }
            }
        }
        if (fes != null && fes.exists() == true) {
            ospr = new ObjectServerProperties();
            ospr.setMessageResource(getMessageResource());

            fin = null;
            try {
                fin = new FileInputStream(fes);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_GETPROPERTIES, 0, exception_object);
                return;
            }
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTIES_FILE,
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_PROPERTIES_FILE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        fes.getPath()));
            }
            if (fin != null) {
                try {
                    if (getDebug() > 0) {
                        ospr.setDebug(getDebug(), logStream);
                    }
                    ospr.setProperties((InputStream)fin);
                    sproperties = ospr;
                    if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                        getLogStream().printLogln( 
                            message_resource.getFormattedMessage(
                                ObjectServerDebug.DEBUG_END_KEY,
                                message_resource.getMessage(
                                    ObjectServerDebug.DEBUG_END_KEY),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_PROPERTIES ));
                    }
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME,
                            TRACE_GETPROPERTIES, 0, exception_object);
                    return;
                }
            }
            return;
        }
        try {
            sproperties = new ObjectServerProperties();
            sproperties.setMessageResource(getMessageResource());
            if (getDebug() > 0) {
                sproperties.setDebug(getDebug(), logStream);
            }
            sproperties.setProperties(null);
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln( DEBUG_SETUP_PROPERTIES );
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETPROPERTIES, 0, exception_object);
            return;
        }
    }

    /**
     *
     * @param   ObjectInputStream
     * @return  Object
     * @throws  Exception
     */
    protected Object retryReadObject(ObjectInputStream ois) throws Exception {
        Object op;
        int    retry;

        if (ois == null) {
            ObjectServerException ose 
                = new ObjectServerException(
                    message_resource.getFormattedMessage(
                        ObjectServerException.OBJECT_INPUT_STREAM_NULL,
                        message_resource.getMessage(
                            ObjectServerException.OBJECT_INPUT_STREAM_NULL)));
            throw ose;
        }
        op = null;
        for (retry = 0; retry < RETRY_NUMBER; retry++) {
            try {
                op = ois.readObject();
                if (((op instanceof oserv.ObjectServerException) == true)||
                       ((op instanceof Exception) == true)) {
                        throw (Exception)op;
                }
                return op;
            } catch (Exception exception_object) {
                op = null;
                if (exception_object instanceof SocketTimeoutException) {
                    continue;
                }
                printExceptionMessage(TRACE_CLASSNAME, TRACE_RETRYREADOBJECT, 0, exception_object);
                throw exception_object;
            }
        }
        ObjectServerException ose = new ObjectServerException(
            message_resource.getFormattedMessage(
                READ_TIMEOUT_EXCEPTION,
                message_resource.getMessage(READ_TIMEOUT_EXCEPTION)));
        throw ose;
    }

    protected LogPrintStream setupLogFile(String logfile) {
        File fdes;
        String vstring;
        Boolean debugLog = false;
        String logPath;

        try {
            if (logfile == null || logfile.length() == 0) {
                if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                    if (logStream == null) {
                        logStream = new LogPrintStream(System.out) ;
                    }
                    logStream.printLogln(
                        message_resource.getFormattedMessage(
                            ObjectServerDebug.DEBUG_LOG, 
                            message_resource.getMessage(
                                ObjectServerDebug.DEBUG_LOG), 
                            message_resource.getMessage(
                                ObjectServerDebug.DEBUG_CODE_003),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_NULL_KEY));
                }
                return logStream;
            } else {
                if (logfile.toLowerCase().compareTo(
                                    OSERV_LOG_FILE_TTY_VALUE) == 0) {
                    if (logStream == null) {
                        logStream = new LogPrintStream(System.out);
                        if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                            logStream.printLogln( 
                                message_resource.getFormattedMessage(
                                    ObjectServerDebug.DEBUG_LOG, 
                                    message_resource.getMessage(
                                        ObjectServerDebug.DEBUG_LOG), logfile));
                            logStream.printLogln( 
                                message_resource.getFormattedMessage(
                                    ObjectServerDebug.DEBUG_LOG, 
                                    message_resource.getMessage(
                                        ObjectServerDebug.DEBUG_LOG ),
                                    ObjectServerDebug.DEBUG_LOG_SWITCH_TO_TTY));
                        }
                    } else if (logStream.equals(System.out) == false) {
                        if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                            logStream.printLogln( 
                                message_resource.getFormattedMessage(
                                    ObjectServerDebug.DEBUG_LOG, 
                                    message_resource.getMessage(
                                        ObjectServerDebug.DEBUG_LOG ), logfile));
                            logStream.printLogln(
                                message_resource.getFormattedMessage(
                                    ObjectServerDebug.DEBUG_LOG, 
                                    message_resource.getMessage(
                                        ObjectServerDebug.DEBUG_LOG ), 
                                    ObjectServerDebug.DEBUG_LOG_SWITCH_TO_TTY));
                        }
                        logStream = new LogPrintStream(System.out);
                    }
		} else if (logfile.toLowerCase().compareTo(
                        OSERV_LOG_FILE_CONSOLE_VALUE) == 0) {
                    String separator_key = String.valueOf(java.io.File.separatorChar);
                    if (separator_key.compareTo(DOS_DIRECTORY_SEPARATOR) == 0) {
                        logStream = new LogPrintStream(System.out);
                    } else {
                        fdes = new File(CONSOLE_DEVICE_PATH);
                        if (fdes.exists() == true) {
                            logStream = new LogPrintStream(CONSOLE_DEVICE_PATH);
                        } else {
                            logStream = new LogPrintStream(System.out);
                        }
		    }
                } else {    
                    logPath = logfile;
                    if (logfile.length() > 0 && logfile.charAt(0) != File.separatorChar) {
                        logPath = getPropertyValue(OSERV_WORK_PATH_KEY);
                        if (logPath != null && logPath.length() > 0) {
                            logPath = logPath.concat(File.separator + logfile);
                        }
                    }
                   
                    fdes = new File(logPath);
                    if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                        debugLog = false;
                        if (logStream == null) {
                            Date dt = new Date();
                            System.out.println(dt.toLocaleString() +
                                message_resource.getMessage(
                                    ObjectServerDebug.DEBUG_LOG) +
                                fdes.toString());
                            debugLog = true;
                        } else if (logStream.equals(System.out) == true) {
                            Date dt = new Date();
                            System.out.println(dt.toLocaleString() +
                                message_resource.getMessage(
                                    ObjectServerDebug.DEBUG_LOG) +
                                fdes.toString());
                            debugLog = true;
                        }
                    }
                    if (fdes.exists() == true) {
                        FileOutputStream fos = new FileOutputStream(logPath, true);
                        logStream = new LogPrintStream(fos);
                        if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                            if (debugLog == true) {
                                logStream.printLogln(
                                    message_resource.getMessage(
                                        ObjectServerDebug.DEBUG_LOG), 
                                    logStream.toString());
                                debugLog = false;
                            }
                        }
                    } else {
                        logStream = new LogPrintStream(System.out);
                    }
                }
            }
        } catch (Exception exception_object) {
            if (logStream == null) {
                logStream = new LogPrintStream(System.out);
            }
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && logStream != null) {
                writeExceptionTrace(logStream, 
                    getMessageResource(), exception_object, 
                    TRACE_CLASSNAME, TRACE_SETUPLOGFILE, 0);
            }
            if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                logStream.printLogln(
                    message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_LOG, 
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_LOG), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_READ_PROPERTY_FILE));
            }
        }
        return logStream;
    }

    /**
     *
     * @param     String     : jar file name
     * @return    void
     * @exception Exception  : 
     */
    public void readPropertyFromJar(String jarfile) throws Exception {
        int idx;
        int pdx;
        String path;
        String ival;
        String target;
        String vstring;
        String properties_name;
        DataInputStream dis;
        ObjectServerProperties pr;
        ByteArrayInputStream bis;
        StringTokenizer stk;

        if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
            logStream.printLogln( 
                message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    message_resource.getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_READ_PROPERTIES_FROM_JAR,
                    message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        if (jarfile == null) {
            return ;
        }
        properties_name = PACKAGE_NAME+File.separator+OSERV_PROPERTIES;
        for (int i = 0; i < pathlist.length; i++ ) {

            ival = System.getProperty(pathlist[i]);
            if (ival == null) {
                continue;
            }
            if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                logStream.printLogln( 
                    message_resource.getFormattedMessage(
                        DEBUG_READ_PROPERTIES_FROM_JAR,
                        message_resource.getMessage(
                            DEBUG_READ_PROPERTIES_FROM_JAR),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        pathlist[i] ));
            }
            stk = new StringTokenizer(ival, LIBRARY_SEPARATOR);
            while(stk.hasMoreTokens()) {
                path = stk.nextToken();
                idx = path.lastIndexOf(JAR_SUFFIX) ;
                if (idx < 0) {
                    continue;
                }
                pdx = path.lastIndexOf(File.separator);
                target = path.substring(pdx + 1);

                loadPropertyFromJar(path);
            }
        }
        return;
    }

    private void loadPropertyFromJar(String jarfile) throws Exception {
        int idx;
        int pdx;
        String path;
        String ival;
        String target;
        String vstring;
        String properties_name;
        DataInputStream dis;
        ObjectServerProperties pr;
        ByteArrayInputStream bis;
        StringTokenizer stk;
        FileInputStream fis;
        JarInputStream  jis;
        ByteArrayOutputStream bos;
        byte[] buffer;
        int rcount;
        boolean tableflag;
        ZipEntry zipentry;

        if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
            logStream.printLogln(
                message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    message_resource.getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_LOAD_PROPERTIES_FROM_JAR,
                    message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        if (jarfile == null) {
            return ;
        }
        if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
            logStream.printLogln(
                message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_JAR_NAME, 
                    message_resource.getMessage(
                        ObjectServerDebug.DEBUG_JAR_NAME), 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    jarfile ));
        }
        properties_name = PACKAGE_NAME+directory_separator_key+OSERV_PROPERTIES;
        buffer = new byte[BUFFER_SIZE];
        bos = new ByteArrayOutputStream();
        try {
            fis = new FileInputStream(jarfile);
            jis = new JarInputStream(fis);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME,
                TRACE_LOADPROPERTYFROMJAR, 0, exception_object);
            return;
        }
        zipentry = jis.getNextEntry();
        while (zipentry != null) {
            if (zipentry.getName().compareTo( properties_name ) == 0 ||
                zipentry.getName().compareTo(OSERV_PROPERTIES) == 0) {
                if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                    logStream.printLogln(
                        message_resource.getFormattedMessage(
                            ObjectServerDebug.DEBUG_ENTRY_NAME, 
                            message_resource.getMessage(
                                ObjectServerDebug.DEBUG_ENTRY_NAME), 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            zipentry.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            properties_name));
                }
                while (true) {
                    try {
                        rcount = jis.read(buffer, 0, BUFFER_SIZE);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_CLASSNAME,
                                TRACE_LOADPROPERTYFROMJAR, 0, exception_object);
                        break;
                    }
                    if (rcount == -1) {
                        break;
                    }
                    bos.write(buffer, 0, rcount);
                }
                break;
            }
            if (bos.size() > 0) {
                break;
            }
            zipentry = jis.getNextEntry();
        }
        if ( bos.size() == 0 ) {
            return;
        }
        buffer = bos.toByteArray();
        try {
            bis = new ByteArrayInputStream(buffer);
            dis = new DataInputStream(bis);
            if (sproperties == null) {
                sproperties.setMessageResource(getMessageResource());
                sproperties = new ObjectServerProperties();
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_END_KEY, 
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_END_KEY), 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_CREATE_PROPERTIES,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            sproperties.toString()));
                }
            }
            sproperties.loadSpecificProperties((InputStream)dis, true);
            if (getDebug() > OSERV_DEBUG_FULL_VALUE) {
                logStream.printLogln(
                    message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_END_KEY, 
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_END_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_LOAD_PROPERTIES));
            }

        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME,
                TRACE_LOADPROPERTYFROMJAR, 0, exception_object);
            throw exception_object;
        }
    }

    public String getServiceCommandElement(String command, int index) 
        throws Exception {
        int          l;
        int          count;
        String[] element;
        String command_string;
        StringTokenizer     string_tokenizer_object;

        command_string = replace(command, OBJECT_SEPARATOR + OBJECT_SEPARATOR,
            OBJECT_SEPARATOR + PADDING_STRING + OBJECT_SEPARATOR);

        if (String.valueOf(command_string.charAt(command_string.length() - 1)).compareTo(
            OBJECT_SEPARATOR) == 0) {
            command_string = command_string.concat(PADDING_STRING);
        }
        string_tokenizer_object = new StringTokenizer(command, OBJECT_SEPARATOR);
        count = string_tokenizer_object.countTokens();

        element = new String[count];
        l = 0;
        while (string_tokenizer_object.hasMoreTokens()) {
            element[l] = string_tokenizer_object.nextToken();
            if (element[l].compareTo(BLANK_STRING) == 0) {
                element[l] = null;
            }
            l++;
        }
        if (l <= index) {
            Exception   exception_object;
            exception_object = new Exception(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION);
            throw exception_object;
        }
        return element[index];
    }

    public int getServiceCommandCount(String command) throws Exception {
        int          l;
        int          count;
        String[] element;
        String command_string;
        StringTokenizer     string_tokenizer_object;

        command_string = replace(command, OBJECT_SEPARATOR + OBJECT_SEPARATOR,
            OBJECT_SEPARATOR + PADDING_STRING + OBJECT_SEPARATOR);

        if (String.valueOf(command_string.charAt(command_string.length() - 1)).compareTo(
            OBJECT_SEPARATOR) == 0) {
            command_string = command_string.concat(PADDING_STRING);
        }
        string_tokenizer_object = new StringTokenizer(command, OBJECT_SEPARATOR);
        count = string_tokenizer_object.countTokens();

        element = new String[count];
        l = 0;
        while (string_tokenizer_object.hasMoreTokens()) {
            element[l] = string_tokenizer_object.nextToken();
            if (element[l].compareTo(BLANK_STRING) == 0) {
                element[l] = null;
            }
            l++;
        }
        return l;
    }

    public String getServiceCommandClass(String command) throws Exception {
        String class_name;

        try {
            class_name = getServiceCommandElement(command, 0);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return class_name;
    }

    public String getServiceCommandInterface(String command) throws Exception {
        String interface_name;

        try {
            interface_name = getServiceCommandElement(command, 1);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return interface_name;
    }

    public String getServiceCommandMethod(String command) throws Exception {
        String method_name;

        try {
            method_name = getServiceCommandElement(command, 2);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return method_name;
    }

    public String getServiceCommandParameter(String command) throws Exception {
        String parameter_name;

        try {
            parameter_name = getServiceCommandElement(command, 3);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return parameter_name;
    }

    public String getServiceCommandReturn(String command) throws Exception {
        String return_name;

        try {
            return_name = getServiceCommandElement(command, 4);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return return_name;
    }

    public String getServiceCommandException(String command) throws Exception {
        String exception_name;

        try {
            exception_name = getServiceCommandElement(command, 5);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return exception_name;
    }

    public static String replace(String s0, String s1, String s2) {
        int    idx;
        String s3;

        if (s0 == null || s1 == null || s2 == null) {
            return s0;
        }
        s3 = s0;
        for (int i = 0; i < s0.length(); i++) {
            idx = s3.indexOf(s1, i);
            if (idx >= 0) {
                s3 = s3.substring(0, idx) + 
                    s2 +
                    s3.substring(idx+s1.length());
            }
        }
        return s3;
    }

    public static String getQueryParameter(String query_string, String key) throws Exception {
        int     idx;
        String  word;
        String  value;
        String  word_key;
        StringTokenizer  string_tokenizer;

        if (query_string == null || key == null) {
            return null;
        }
        value = null;
        word = query_string;
        idx = word.toUpperCase().indexOf(key.toUpperCase()+QUERY_PARAMETER_SEPARATOR);
        if (idx >= 0) {
            int query_index;
            int param_index;

            word_key = word.substring(idx+key.length() 
				      + QUERY_PARAMETER_SEPARATOR.length());
            query_index = word_key.indexOf(QUERY_SEPARATOR);
            if (query_index >= 0) {
                word_key = word_key.substring(0, query_index);
            }
            value = word_key;
        }
        return value;
    }

    /**
     * getCallerMethod()
     */
    protected String getCallerMethod(Exception exception_object) {
        if (exception_object == null
            || exception_object.getStackTrace() == null) {
            return new String(EMPTY_MESSAGE);
        }
        for (int i = 0; i < exception_object.getStackTrace().length; i++) {
            if (exception_object.getStackTrace()[i].getClassName().indexOf(getClass().getName()) == 0) {
            return exception_object.getStackTrace()[i].getClassName()
             + OBJECT_DOT_SEPARATOR
             + exception_object.getStackTrace()[i].getMethodName()
             + START_PARENTHESIS
             + exception_object.getStackTrace()[i].getFileName()
             + LINE_NUMBER_SEPARATOR
             + String.valueOf(exception_object.getStackTrace()[i].getLineNumber())
             + END_PARENTHESIS
             ;
            }
        }
        return new String(EMPTY_MESSAGE);
    }

    public static boolean isOutputStream (String type) {
        if (type != null && (type.compareTo(
                 ObjectServerHeader.JAVA_PRINTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_PIPEDOUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_BUFFEREDOUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_BYTEARRAYOUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_FILTEROUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_FILEOUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_DATAOUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_OBJECTOUTPUTSTREAM_TYPE_KEY) == 0 
              || type.compareTo(
                 ObjectServerHeader.JAVA_OUTPUTSTREAM_TYPE_KEY) == 0)) {
            return true;
        }
        return false;
    }

    public static boolean isSequenceInputStream (String type) {
        if (type != null && (type.compareTo(
             ObjectServerHeader.JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY) == 0 
        )) {
            return true;
        }
        return false;
    }

    public static boolean isInputStream (String type) {
        if (type != null && (type.compareTo(
             ObjectServerHeader.JAVA_INPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_PIPEDINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_BUFFEREDINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_DATAINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_FILTERINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_LINENUMBERINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_PUSHBACKINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0 
             || type.compareTo(
                 ObjectServerHeader.JAVA_FILEINPUTSTREAM_TYPE_KEY) == 0) 
        ) {
            return true;
        }
        return false;
    }

    public static boolean isObjectVariable(String type) {
        if (type != null && (type.compareTo(Integer.class.getName()) == 0
                 || type.compareTo(Long.class.getName()) == 0
                 || type.compareTo(Float.class.getName()) == 0
                 || type.compareTo(Double.class.getName()) == 0
                 || type.compareTo(Byte.class.getName()) == 0
                 || type.compareTo(Short.class.getName()) == 0
                 || type.compareTo(Boolean.class.getName()) == 0
                 || type.compareTo(Character.class.getName()) == 0
                 || type.compareTo(String.class.getName()) == 0)
        ) {
            return true;
        }
        return false;
    }
}    
