/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectServerProperties.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/11/04
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ObjectServerProperties extends Properties {
    public static String PROCESS_WORKING_DIRECTORY_KEY = "PWD";
    public final static String UNIX_DIRECTORY_SEPARATOR = ObjectServerHeader.DIRECTORY_SEPARATOR;
    public final static String DOS_DIRECTORY_SEPARATOR = "\\";
    public final static String DOS_WORK_DIRECTORY = "c:\\Windows\\Temp";
    public final static String DOS_CURRENT_DIRECTORY_COMMAND
        = "C:\\WINDOWS\\system32\\cmd.exe /C CHDIR";

    public final static String EMPTY_PATH = "";

    public static String directory_separator_key = ObjectServerHeader.DIRECTORY_SEPARATOR;

    public final static String DEBUG_PAD_PROPERTIES
        = "padProperties()";
    public final static String DEBUG_LOAD_SYSTEM_PROPERTIES
        = "loadSystemProperties()";
    public final static String DEBUG_LOAD_SPECIFIC_PROPERTIES
        = "loadSpecificProperties()";
    public final static String DEBUG_INITIAL_PROPERTIES
        = "initialProperties()";
    public final static String DEBUG_SET_PROPERTIES_WITH_INPUTSTREAM
        = "setProperties(InputStream)";
    public final static String DEBUG_SET_PROPERTIES
        = "setProperties()";
    private final static String DEBUG_UNKNOWN_PROPERTY
        = "unknown property";
    private final static String DEBUG_CALL_INITIAL_PROPERTIES
        = "initial properties";
    /*
    private final static String DEBUG_PAD_PROPERTIES 
        = "setup defaults to unset properties";
    */
/**
    private final static String DEBUG_EXCEPTION_KEY   
        = "DEBUG Exception";
*/

    private static int debug_level;
    private static LogPrintStream log_print_stream = null;

    private static boolean     tableflag;
    private static boolean     debugflag;
    private static boolean     objectflag;
    private static boolean     requestflag;
    private static boolean     forwardflag;
    private static boolean     hostflag;
    private static boolean     forbiddenflag;
    private static boolean     languageflag;
    private static boolean     timeoutflag;
    private static boolean     maximumflag;
    private static boolean     logflag;
    private static boolean     retryflag;

    private static boolean     clientsflag;
    private static boolean     serversflag;
    private static boolean     maxthreadflag;

    private static boolean     object_refuse_flag;
    private static boolean     request_refuse_flag;
    private static boolean     forward_refuse_flag;

    private static boolean     tunnelflag;

    private static boolean     workflag;
    private static boolean     transflag;
    private static boolean     userworkflag;

    private static MessageResource message_resource = new MessageResource();

    public static int getDebug() {
        return debug_level;
    }

    public static void setDebug(int level) {
        debug_level = level;
    }

    public void setDebug(int level, LogPrintStream stream) {
        debug_level = level;
        if (debug_level > 0) {
            log_print_stream = stream;
        }
    }

    public static void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public static MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void initialProperties() {
        retryflag   = false;
        tableflag   = false;
        timeoutflag = false;
        debugflag   = false;
        hostflag    = false;
        forbiddenflag= false;
        languageflag = false;
        objectflag  = false;
        maximumflag = false;
        logflag     = false;
        requestflag = false;
        userworkflag = false;
        transflag = false;

        maxthreadflag = false;

        workflag    = false;

        forwardflag = false;
        clientsflag = false;
        serversflag = false;
        tunnelflag = false;

        object_refuse_flag = false;
        request_refuse_flag = false;
        forward_refuse_flag = false;

        return;
    }

    public String getPropertyValue(String key) {
        String rvalue;

        rvalue = getProperty(key);
        return rvalue;
    }

    public static String getDosCurrentDirectory() throws Exception {
        Runtime runtime;
        Process proc;
        OutputStream ons;
        PrintStream ps;
        InputStream ins;
        DataInputStream dis;
        String word;
        ObjectServerException  object_server_exception;

        runtime = Runtime.getRuntime();

        try {
            proc = runtime.exec(DOS_CURRENT_DIRECTORY_COMMAND);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException (
                getMessageResource().getFormattedMessage(
                ObjectServerError.JAVA_RUNTIME_EXEC_IO_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerError.JAVA_RUNTIME_EXEC_IO_EXCEPTION)));
            throw object_server_exception;
        }

        ins = proc.getInputStream();
        dis = new DataInputStream(ins);

        try {
            word = dis.readLine();
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_LINE_IO_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_LINE_IO_EXCEPTION)));
            throw object_server_exception;
        }
        return word;
    }

    public void setProperties(InputStream is) 
        throws Exception {

        if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (log_print_stream != null) {
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_SET_PROPERTIES_WITH_INPUTSTREAM,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CALL_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_INITIAL_PROPERTIES,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_002)));
            }
        }
        initialProperties();
        if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (log_print_stream != null) {
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CALL_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_LOAD_SYSTEM_PROPERTIES,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_002)));
            }
        }
        loadSystemProperties();
        if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (log_print_stream != null) {
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CALL_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_LOAD_SPECIFIC_PROPERTIES,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_002)));
            }
        }
        if (is != null) {
            try {
                loadSpecificProperties(is, false);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (log_print_stream != null) {
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CALL_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_PAD_PROPERTIES,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_002)));
            }
        }
        padProperties();
    }

    public void padProperties() {
        String workpath;

        String separator_key = String.valueOf(java.io.File.separatorChar);
        if (workflag == false) {
            File fdes = new File(EMPTY_PATH);
            workpath = fdes.getAbsolutePath();
            setProperty(ObjectServerHeader.OSERV_WORK_PATH_KEY, workpath);
        }
        setProperty(ObjectServerHeader.OSERV_PERSISTENT_DIRECTORY_KEY,
		    ObjectServerHeader.OSERV_PERSISTENT_DIRECTORY_VALUE);
        if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (log_print_stream != null) {
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTY_NAME,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerHeader.OSERV_PERSISTENT_DIRECTORY_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		        ObjectServerHeader.OSERV_PERSISTENT_DIRECTORY_VALUE));
            }
        }

        setProperty(ObjectServerHeader.OSERV_EXECUTION_WAITTIME_KEY,
		    ObjectServerHeader.EXECUTION_WAITTIME_VALUE);
        if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (log_print_stream != null) {
                log_print_stream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTY_NAME,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerHeader.OSERV_EXECUTION_WAITTIME_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		        ObjectServerHeader.EXECUTION_WAITTIME_VALUE));
            }
        }
        if (retryflag == false) {
            setProperty(ObjectServerHeader.OSERV_RETRY_COUNT_KEY,
		    ObjectServerHeader.OSERV_RETRY_COUNT_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_RETRY_COUNT_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		            ObjectServerHeader.OSERV_RETRY_COUNT_VALUE));
                }
            }
        }
        if (tableflag == false) {
            setProperty(ObjectServerHeader.OSERV_TABLE_SIZE_KEY, 
                    ObjectServerHeader.OSERV_SIZE_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_TABLE_SIZE_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_SIZE_VALUE));
                }
            }
        }
        if (hostflag == false) {
            setProperty(ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY, 
                    ObjectServerHeader.SERVICE_HOSTNAME_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.SERVICE_HOSTNAME_VALUE));
                }
            }
        }
        if (debugflag == false) {
            setProperty(ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY, 
                    ObjectServerHeader.OSERV_DEBUG_LEVEL_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_DEBUG_LEVEL_VALUE));
                }
            }
        }
        if (requestflag == false) {
            setProperty(ObjectServerHeader.OSERV_REQUEST_PORT_KEY, 
                    ObjectServerHeader.REQUEST_PORT_NUMBER_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_REQUEST_PORT_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.REQUEST_PORT_NUMBER_VALUE));
                }
            }
        }
        if (tunnelflag == false) {
            setProperty(ObjectServerHeader.OSERV_FORWARD_TUNNEL_KEY,
                    ObjectServerHeader.FORWARD_TUNNEL_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_FORWARD_TUNNEL_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.FORWARD_TUNNEL_VALUE));
                }
            }
        }
        if (maxthreadflag == false) {
            setProperty(ObjectServerHeader.OSERV_MAX_THREAD_KEY,
		    ObjectServerHeader.OSERV_MAX_THREAD_DEFAULT);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_MAX_THREAD_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		            ObjectServerHeader.OSERV_MAX_THREAD_DEFAULT));
                }
            }
        }
        if (timeoutflag == false) {
            setProperty(ObjectServerHeader.OSERV_CONNECTION_TIMEOUT_KEY,
		    ObjectServerHeader.CONNECTION_TIMEOUT_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_CONNECTION_TIMEOUT_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		            ObjectServerHeader.CONNECTION_TIMEOUT_VALUE));
                }
            }
        }
        if (maximumflag == false) {
            setProperty(ObjectServerHeader.OSERV_BUFFER_MAXIMUM_KEY,
		    ObjectServerHeader.BUFFER_MAXIMUM_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_BUFFER_MAXIMUM_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		            ObjectServerHeader.BUFFER_MAXIMUM_VALUE));
                }
            }
        }
        if (userworkflag == false) {
            setProperty(ObjectServerHeader.OSERV_USER_WORK_KEY,
		    ObjectServerHeader.OSERV_USER_WORK_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_USER_WORK_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		            ObjectServerHeader.OSERV_USER_WORK_VALUE));
                }
            }
        }
        if (logflag == false) {
            setProperty(ObjectServerHeader.OSERV_LOG_FILE_KEY, 
                ObjectServerHeader.OSERV_LOG_FILE_TTY_VALUE);
            if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (log_print_stream != null) {
                    log_print_stream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_NAME,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_LOG_FILE_KEY), 
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_PROPERTY_VALUE,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_VALUE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerHeader.OSERV_LOG_FILE_TTY_VALUE));
                }
            }
        }
    }
    
    public  void loadSpecificProperties(InputStream is)
        throws Exception {
        loadSpecificProperties(is, false);
    }

    public  void loadSpecificProperties(InputStream is, boolean flag)
        throws Exception {

        int        index;
        Properties bkpro;
        Properties pro;
        Enumeration en;
        String      pName;
        String      vName;

        if (is == null) {
            ObjectServerException ose 
                = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerException.INPUT_STREAM_NULL,
                        getMessageResource().getMessage(
                            ObjectServerException.INPUT_STREAM_NULL)));
            throw (Exception) ose;
        }

        PropertyLoader property_loader;
        try {
            property_loader = new PropertyLoader(is);
        } catch (Exception exception_object) {
            ObjectServerException ose 
                = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.PROPERTY_LOADER_IO_EXCEPTION,
                        getMessageResource().getMessage(
                        ObjectServerError.PROPERTY_LOADER_IO_EXCEPTION)));
            throw (Exception) ose;
        }
        while (true) {
            try {
                pName = property_loader.getKey();
                if (pName == null) { break; }
                vName = property_loader.getValue();
            } catch (Exception exception_object) {
                break;
            }

            if (transflag == false && pName.compareTo(
                ObjectServerHeader.OSERV_TRANSIENT_HOME_KEY) == 0) {
                File fdes;
                fdes = new File(vName);
                if (fdes.exists() == false) {
                    continue;
                }
                transflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }

            if (workflag == false && pName.compareTo(
                ObjectServerHeader.OSERV_WORK_PATH_KEY) == 0) {
                File fdes;
                fdes = new File(vName);
                if (fdes.exists() == false) {
                    continue;
                }
                workflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }

            if ((flag == true || maxthreadflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_MAX_THREAD_KEY) == 0) {
                int maxvalue;
                try {
                    maxvalue = Integer.valueOf(vName).intValue();
                    if ((maxvalue > ObjectServerHeader.OSERV_MAX_THREAD_LIMIT_VALUE) || (maxvalue < ObjectServerHeader.OSERV_MIN_THREAD_VALUE)) {
                        setProperty(pName, ObjectServerHeader.OSERV_MAX_THREAD_DEFAULT);
                        continue;
                    }
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                maxthreadflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || retryflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_RETRY_COUNT_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                retryflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || languageflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_MESSAGE_LANGUAGE_KEY) == 0) {
                languageflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || forbiddenflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_SYSTEM_FORBIDDEN_KEY) == 0) {
                forbiddenflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || tableflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_TABLE_SIZE_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                tableflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || debugflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                debugflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || timeoutflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_CONNECTION_TIMEOUT_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                timeoutflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || maximumflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_BUFFER_MAXIMUM_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                maximumflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || requestflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_REQUEST_PORT_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                requestflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || object_refuse_flag == false) 
                && pName.compareTo(
                    ObjectServerHeader.OSERV_OBJECT_REFUSE_KEY) == 0) {
                object_refuse_flag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || request_refuse_flag == false) 
                && pName.compareTo(
                    ObjectServerHeader.OSERV_REQUEST_REFUSE_KEY) == 0) {
                request_refuse_flag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || forward_refuse_flag == false) 
                && pName.compareTo(
                    ObjectServerHeader.OSERV_FORWARD_REFUSE_KEY) == 0) {
                forward_refuse_flag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || clientsflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_FORWARD_CLIENTS_KEY) == 0) {
                clientsflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || serversflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_FORWARD_SERVERS_KEY) == 0) {
                serversflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || tunnelflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_FORWARD_TUNNEL_KEY) == 0) {
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                tunnelflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || logflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_LOG_FILE_KEY) == 0) {
                logflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }

            if ((flag == true || hostflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY) == 0) {
                hostflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
            if ((flag == true || userworkflag == false) && pName.compareTo(
                ObjectServerHeader.OSERV_USER_WORK_KEY) == 0) {
                userworkflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SPECIFIC_PROPERTIES));
                    }
                }
                continue;
            }
        }
    }


    public void loadSystemProperties() {
        String pName;
        String vName;
        Enumeration en;

        Properties spro = System.getProperties();

        en = spro.propertyNames();
        while (en.hasMoreElements()) {
            pName = (String)en.nextElement();
            vName = spro.getProperty(pName);

            if (pName.compareTo(
                ObjectServerHeader.OSERV_WORK_PATH_KEY) == 0) {
                File fdes;
                fdes = new File(vName);
                if (fdes.exists() == false) {
                    continue;
                }
                workflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(
                ObjectServerHeader.OSERV_TRANSIENT_HOME_KEY) == 0) {
                File fdes;
                fdes = new File(vName);
                if (fdes.exists() == false) {
                    continue;
                }
                transflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }

            if (pName.compareTo(ObjectServerHeader.OSERV_RETRY_COUNT_KEY)==0){
                retryflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }

            if (pName.compareTo(ObjectServerHeader.OSERV_TABLE_SIZE_KEY)==0){
                tableflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY)
                == 0) {
                hostflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY)==0){
                debugflag = true;
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SYSTEM_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }

            if (pName.compareTo(ObjectServerHeader.OSERV_REQUEST_PORT_KEY)==0){
                requestflag = true;
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SYSTEM_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString())
                            );
                        }
                    }
                    continue;
                }
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(
                ObjectServerHeader.OSERV_CONNECTION_TIMEOUT_KEY) == 0){
                setProperty(pName, vName);
                timeoutflag = true;
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_MAX_THREAD_KEY) 
                == 0){
                maxthreadflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_BUFFER_MAXIMUM_KEY) 
                == 0){
                maximumflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }

            if (pName.compareTo(ObjectServerHeader.OSERV_LOG_FILE_KEY) == 0){
                logflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_USER_WORK_KEY) == 0){
                userworkflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_MESSAGE_LANGUAGE_KEY) 
                == 0) {
                languageflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_SYSTEM_FORBIDDEN_KEY) 
                == 0) {
                forbiddenflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_OBJECT_REFUSE_KEY) 
                == 0) {
                object_refuse_flag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_REQUEST_REFUSE_KEY) 
                == 0) {
                request_refuse_flag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_FORWARD_REFUSE_KEY) 
                == 0) {
                forward_refuse_flag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(
                ObjectServerHeader.OSERV_FORWARD_CLIENTS_KEY) == 0) {
                clientsflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(
                ObjectServerHeader.OSERV_FORWARD_SERVERS_KEY) == 0) {
                serversflag = true;
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
            if (pName.compareTo(ObjectServerHeader.OSERV_FORWARD_TUNNEL_KEY) 
                == 0) {
                tunnelflag = true;
                try {
                    Integer.valueOf(vName).intValue();
                } catch (Exception exception_object) {
                    if (debug_level > 
                                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                        if (log_print_stream != null) {
                            log_print_stream.printLogln( 
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    DEBUG_LOAD_SYSTEM_PROPERTIES,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    exception_object.toString()));
                        }
                    }
                    continue;
                }
                setProperty(pName, vName);
                if (debug_level > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (log_print_stream != null) {
                        log_print_stream.printLogln( 
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_PROPERTY_NAME, 
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_PROPERTY_NAME), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                pName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                vName,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_LOAD_SYSTEM_PROPERTIES));
                    }
                }
                continue;
            }
        }
    }
}
