/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectServerSecurityManager.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/08/01
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.security.*;
import java.util.*;
import java.awt.*;
import java.sql.*;
import java.lang.reflect.*;
import java.lang.management.*;
import java.util.logging.*;

public class ObjectServerSecurityManager extends SecurityManager {
    public static String OSERV_PROHIBIT = "oserv prohibits.";
    public static String OSERV_SECURITY_VIOLATION = "oserv security violation.";
    public static String OSERV_ISSUE_ILLEGAL  = "oserv issue illegal exception.";
    public static String FILE_PATH_EXCEPTION = "File path exception";

    public static String PARENT_ACCESS_EXCEPTION = "Parent access exception";

    public static String PARENT_DIRECTORY = "../";
    public static String CURRENT_DIRECTORY = "./";

    public static char CURRENT_CHARACTER = '.';
    public static String DOT_SEPARATOR = ".";

    public static String HOST_SEPARATOR = ":";

    private static boolean super_mode = true;

    private static String SET_SECURITY_MANAGER = "setSecurityManager";
    private static String CREATE_SECURITY_MANAGER = "createSecurityManager";
    private static String CREATE_CLASS_LOADER = "createClassLoader";
    private static String READ_FILE_DESCRIPTOR = "readFileDescriptor";
    private static String WRITE_FILE_DESCRIPTOR = "writeFileDescriptor";
    private static String SUPPRESS_ACCESS_CHECKS = "suppressAccessChecks";

    private static String SET_LOG = "setLog";
    private static String CALL_ABORT = "callAbort";
    private static String SET_SYNC_FACTORY = "setSyncFactory";
    private static String SET_NETWORK_TIMEOUT = "setNetworkTimeout";

    private static String QUEUE_PRINT_JOB = "queuePrintJob";
    private static String SET_FACTORY = "setFactory";
    private static String ACCESS_DECLARE_MEMBERS = "accessDeclaredMembers";

    private static String CONTROL = "control";

    private static String MANAGEMENT_CONTROL = "control";
    private static String MANAGEMENT_MONITOR = "monitor";

    private static String ACCESS_CLASS_IN_PACKAGE = "accessClassInPackage.";
    private static String DEFINE_CLASS_IN_PACKAGE = "defineCLassInPackage.";

    private static String SHOW_WINDOW_WITHOUT_WARNING_BARNNER
        = "showWindowWithoutWarningBanner";
    private static String ACCESS_CLIPBOARD = "accessClipboard";
    private static String ACCESS_EVENT_QUEUE = "accessEventQueue";
    private static String ACCESS_SYSTEM_TRAY = "accessSystemTray";
    private static String CREATE_ROBOT = "createRobot";
    private static String FULL_SCREEN_EXCLUSIVE = "fullScreenExclusive";
    private static String LISTEN_TO_ALL_AWT_EVENTS 
        = "listenToAllAWTEvents";
    private static String READ_DISPLAY_PIXELS
        = "readDisplayPixels";
    private static String REPLACE_KEYBOARD_FOCUS_MANAGER
        = "replaceKeyboardFocusManager";
    private static String SET_APPLET_STUB
        = "setAppletStub";
    private static String SET_WINDOW_ALWAYS_ON_TOP
        = "setWindowAlwaysOnTop";
    private static String TOOLKIT_MODALITY
        = "toolkitModlity";
    private static String WATCH_MOUSE_POINTER
        = "watchMousePointer";
    private static String SET_IO = "setIO";
    private static String LOAD_LIBRARY = "loadLibrary";
    private static String EXIT_VM = "exitVM";
    private static String STOP_THREAD = "stopThread";
    private static String MODIFY_THREAD = "modifyThread";
    private static String MODIFY_THREAD_GROUP = "modifyThreadGroup";

    private static String CREATE_ACCESS_CONTROL_CONTEXT
        = "createAccessControlContext";
    private static String GET_DOMAIN_COMBINER = "getDomainCombiner";
    private static String GET_POLICY = "getPolicy";
    private static String SET_POLICY = "setPolicy";
    private static String CREATE_POLICY = "createPolicy.";
    private static String GET_PROPERTY = "getProperty";
    private static String SET_PROPERTY = "setProperty";
    private static String INSERT_PROVIDER = "insertProvider.";
    private static String REMOVE_PROVIDER = "removeProvider.";
    private static String SET_SYSTEM_SCOPE = "setSystemScope";
    private static String SET_IDENTITY_PUBLIC_KEY 
        = "setIdentityPublicKey";
    private static String SET_IDENTITY_INFO = "setIdentityInfo";
    private static String ADD_INDENTITY_CERTIFICATE = "addIdentityCertificate";
    private static String REMOVE_IDENTITY_CERTIFICATE 
        = "removeIdentityCertificate";
    private static String PRINT_IDENTITY = "printIdentity";

    private static String CLEAR_PROVIDER_PROPERTIES = "clearProviderProperties.";
    private static String PUT_PROVIDER_PROPERTY = "putProviderProperty";
    private static String REMOVE_PROVIDER_PROPERTY 
        = "removeProviderProperty.";
    private static String GET_SIGNER_PRIVATE_KEY
        = "getSignerPrivateKey";
    private static String SET_SIGNER_KEY_PAIR = "setSignerKeyPair";

    private static String ENABLE_SUBCLASS_IMPLEMENTATION 
        = "enableSubclassImplementation";
    private static String ENABLE_SUBSTITUTION
        = "enableSubstitution";

    private static String USER_DIR = "user.dir";
    private static String WILDCARD = "*";

    private static String EXECUTE_ACTION = "execute";
    private static String READ_ACTION = "read";
    private static String WRITE_ACTION = "write";
    private static String DELETE_ACTION = "delete";
    private static String READ_WRITE_ACTIONS = "read,write";

    private static String ACCEPT_ACTION  = "accept";
    private static String CONNECT_ACTION = "connect";
    private static String LISTEN_ACTION  = "listen";
    private static String RESOLVE_ACTION = "resolve";
    private static String ACCEPT_CONNECT_ACTION = "accept,connect";

    public static String LOCAL_HOSTNAME = "localhost";

    public static String PATH_TITLE = "Path ";

    public static String SECURITY_DIRECTORY = null;

    public void toSuperMode() {
        super_mode = true;
    }

    public void toUserMode() {
        super_mode = false;
    }

    public ObjectServerSecurityManager(String directory) {
        super();
        SECURITY_DIRECTORY = directory;
    }

    public Object getSecurityContext() throws SecurityException {
        Object context;
        context = super.getSecurityContext();
        return context;
    }

    private void checkSerializablePermission(SerializablePermission permission) 
        throws SecurityException {
        String event;
        event = permission.getName();
        super.inCheck = true;
        return;
    }

    private void checkAWTPermission(AWTPermission permission) 
        throws SecurityException {
        String event;
        event = permission.getName();
        if (event.compareTo(SHOW_WINDOW_WITHOUT_WARNING_BARNNER) == 0 
        ) {
            super.inCheck = true;
            return;
        }
        if (event.compareTo(ACCESS_CLIPBOARD) == 0
            || event.compareTo(ACCESS_EVENT_QUEUE) == 0
            || event.compareTo(ACCESS_SYSTEM_TRAY) == 0
            || event.compareTo(CREATE_ROBOT) == 0
            || event.compareTo(FULL_SCREEN_EXCLUSIVE) == 0
            || event.compareTo(LISTEN_TO_ALL_AWT_EVENTS) == 0
            || event.compareTo(READ_DISPLAY_PIXELS) == 0
            || event.compareTo(REPLACE_KEYBOARD_FOCUS_MANAGER) == 0
            || event.compareTo(SET_APPLET_STUB) == 0
            || event.compareTo(SET_WINDOW_ALWAYS_ON_TOP) == 0
            || event.compareTo(TOOLKIT_MODALITY) == 0
            || event.compareTo(WATCH_MOUSE_POINTER) == 0
        ) {
            super.inCheck = true;
            return;
        }
        super.inCheck = true;
        return;
    }

    private void checkSQLPermission(SQLPermission permission) 
        throws SecurityException {
        String event;
        event = permission.getName();
        super.inCheck = true;
        return;
    }

    private void checkManagementPermission(ManagementPermission permission) 
        throws SecurityException {
        String event;
        event = permission.getName();
        super.inCheck = true;
        return;
    }

    private void checkLoggingPermission(LoggingPermission permission) 
        throws SecurityException {
        String event;
        event = permission.getName();
        super.inCheck = true;
        return;
    }

    private void checkReflectPermission(ReflectPermission permission) 
        throws SecurityException {
        super.inCheck = true;
        return;
    }

    private void checkSecurityPermission(SecurityPermission permission)
        throws SecurityException {
        String event;

        event = permission.getName();
        if ( 
            event.compareTo(CREATE_ACCESS_CONTROL_CONTEXT) == 0
            || event.compareTo(GET_DOMAIN_COMBINER) == 0
            || event.compareTo(GET_POLICY) == 0
            || event.compareTo(SET_POLICY) == 0
            || event.compareTo(CREATE_POLICY) == 0
            || event.compareTo(GET_PROPERTY) == 0
            || event.compareTo(SET_PROPERTY) == 0
            || event.compareTo(INSERT_PROVIDER) == 0
            || event.compareTo(REMOVE_PROVIDER) == 0
            || event.compareTo(SET_SYSTEM_SCOPE) == 0
            || event.compareTo(SET_IDENTITY_PUBLIC_KEY) == 0
            || event.compareTo(SET_IDENTITY_INFO) == 0
            || event.compareTo(ADD_INDENTITY_CERTIFICATE) == 0
            || event.compareTo(REMOVE_IDENTITY_CERTIFICATE) == 0
            || event.compareTo(PRINT_IDENTITY) == 0
            || event.compareTo(CLEAR_PROVIDER_PROPERTIES) == 0
            || event.compareTo(PUT_PROVIDER_PROPERTY) == 0
            || event.compareTo(REMOVE_PROVIDER_PROPERTY) == 0
            || event.compareTo(GET_SIGNER_PRIVATE_KEY) == 0
            || event.compareTo(SET_SIGNER_KEY_PAIR) == 0
        ) {
            super.inCheck = true;
            return;
        }
        super.inCheck = true;
        return;
    }

    private void checkPropertyPermission(PropertyPermission permission) 
        throws SecurityException {
        int  idx;
        String  actions;
        SecurityException security_exception;

        actions = permission.getActions();
        idx = actions.indexOf(WRITE_ACTION);
        if (idx >= 0) {
            super.inCheck = false;
            security_exception = new SecurityException(OSERV_PROHIBIT);
            throw security_exception;
        }
        super.inCheck = true;
        return;
    }

    private void checkRuntimePermission(Permission permission) 
        throws SecurityException {
        int    idx;
        String path;
        String event;
        String eventhead;
        SecurityException security_exception;

        event = permission.getName();
        if (
            (super_mode == true && event.compareTo(SET_SECURITY_MANAGER) == 0)
            || event.compareTo(CREATE_CLASS_LOADER) == 0
            || event.compareTo(READ_FILE_DESCRIPTOR) == 0
            || event.compareTo(WRITE_FILE_DESCRIPTOR) == 0
            || event.compareTo(SET_IO) == 0
            || event.compareTo(MODIFY_THREAD) == 0
            || event.compareTo(MODIFY_THREAD_GROUP) == 0
            || event.compareTo(CREATE_SECURITY_MANAGER) == 0
            || event.compareTo(QUEUE_PRINT_JOB) == 0
            || event.compareTo(SET_FACTORY) == 0
            ) {
            super.inCheck = true;
            return;
        }
        if ( event.compareTo(STOP_THREAD) == 0
         || (super_mode == false && event.compareTo(SET_SECURITY_MANAGER) == 0)
            ) {
            super.inCheck = false;
            security_exception = new SecurityException(OSERV_PROHIBIT);
            throw security_exception;
        }
        idx = event.indexOf(DOT_SEPARATOR);
        if (idx >= 0) {
            eventhead = event.substring(0, idx);
            if (eventhead.compareTo(LOAD_LIBRARY) == 0
                || eventhead.compareTo(EXIT_VM) == 0) {
                super.inCheck = false;
                security_exception = new SecurityException(OSERV_PROHIBIT);
                throw security_exception;
            }
        }
    }

    private void checkSocketPermission(SocketPermission socket_permission,
        java.lang.Object context) throws SecurityException {
        SecurityException security_exception;

        int    idx;
        int    port;
        String event;
        String pstring;
        event = socket_permission.getName();

        idx = event.lastIndexOf(HOST_SEPARATOR);
        if (idx <= 0) {
            super.inCheck = true;
            return;
        }
        pstring = event.substring(idx+1);
        try {
            port = Integer.valueOf(pstring).intValue();
        } catch (Exception exception_object) {
            super.inCheck = true;
            return;
        }
        if (port != -1) {
            super.inCheck = true;
            return;
        }
  
        AccessControlContext acc = (AccessControlContext)context;
        acc.checkPermission(socket_permission);

        super.inCheck = true;
        return;
    }

    private void checkSocketPermission(SocketPermission socket_permission) 
        throws SecurityException {
        SecurityException security_exception;
        super.inCheck = true;
        return;
    }

    private void checkFilePermission(FilePermission file_permission, 
        java.lang.Object object)
        throws SecurityException {
        SecurityException security_exception;
        String actions;
        int idx;

        actions = file_permission.getActions();
        idx = actions.indexOf(WRITE_ACTION);
        if (idx < 0) {
            idx = actions.indexOf(DELETE_ACTION);
            if (idx < 0) {
                super.inCheck = true;
                return;
            }
        }
        if ((file_permission instanceof FilePermission) == true) {
            checkFilePermission(file_permission);
        }
        if ((object instanceof AccessControlContext) == true) {
            AccessControlContext acc = (AccessControlContext)object;
            acc.checkPermission(file_permission);
            return;
        }
        security_exception = new SecurityException(OSERV_SECURITY_VIOLATION);
        throw security_exception;
    }

    private boolean isTemporaryDirectory(
        String security_base, String security_target) {
        int    index;
        String security_temporary;

        if (security_base == null || security_target == null) {
            return false;
        }
        security_temporary = makeCanonical(security_base + File.separator +
            ObjectServerHeader.PARENT_DIRECTORY + File.separator + 
            ObjectServerHeader.TEMPORARY_DIRECTORY + File.separator);
        if (security_target.length() <= security_temporary.length()) {
            return false;
        }
        index = security_target.indexOf(security_temporary);
        if (index == 0) {
            return true;
        }
        return false;
    }

    private void checkFilePermission(FilePermission file_permission) 
        throws SecurityException {
        int     idx;
        String  path;
        String  event;
        String  actions;
        String  eventhead;
        boolean check_flag;
        SecurityException security_exception;

        actions = file_permission.getActions();
        idx = actions.indexOf(WRITE_ACTION);
        if (idx < 0) {
            idx = actions.indexOf(DELETE_ACTION);
            if (idx < 0) {
                super.inCheck = true;
                return;
            }
        }
        if (SECURITY_DIRECTORY != null && SECURITY_DIRECTORY.length() > 0 && SECURITY_DIRECTORY.charAt(0) == File.separatorChar) {
            path = file_permission.getName();
            if (path == null) {
                super.inCheck = false;
                security_exception = new SecurityException(FILE_PATH_EXCEPTION);
                throw security_exception;
            }
            String canonical_string;
            canonical_string = makeCanonical(path);
            if (path.charAt(0) != File.separatorChar) {
                super.inCheck = false;
                security_exception = new SecurityException(FILE_PATH_EXCEPTION);
                throw security_exception;
            }
            if (path.length() < SECURITY_DIRECTORY.length()) {
                super.inCheck = false;
                security_exception = new SecurityException(FILE_PATH_EXCEPTION);
                throw security_exception;
            }
            idx = canonical_string.indexOf(SECURITY_DIRECTORY);
            if (idx != 0) {
                check_flag = isTemporaryDirectory(SECURITY_DIRECTORY, 
                    canonical_string);
                if (check_flag == false) {
                    super.inCheck = false;
                    security_exception = new SecurityException(FILE_PATH_EXCEPTION);
                    throw security_exception;
                }
            }
        }
        super.inCheck = true;
        return;
    }

    public void checkPermission(java.security.Permission permission) 
        throws SecurityException, NullPointerException {
        int    idx;
        String path;
        String event;
        String eventhead;
        String actions;
        SecurityException security_exception;

        if (permission == null) {
            NullPointerException null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        event = permission.getName();
        if (event.compareTo(SET_SECURITY_MANAGER) == 0
            || event.compareTo(CREATE_CLASS_LOADER) == 0
            || event.compareTo(READ_FILE_DESCRIPTOR) == 0
            || event.compareTo(WRITE_FILE_DESCRIPTOR) == 0
            ) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.compareTo(CREATE_SECURITY_MANAGER) == 0
            || event.compareTo(SET_IO) == 0
            || event.compareTo(MODIFY_THREAD) == 0
            || event.compareTo(STOP_THREAD) == 0
            || event.compareTo(MODIFY_THREAD_GROUP) == 0
            || event.compareTo(QUEUE_PRINT_JOB) == 0
            || event.compareTo(SET_FACTORY) == 0
            ) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.indexOf(ACCESS_CLASS_IN_PACKAGE) == 0) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.indexOf(DEFINE_CLASS_IN_PACKAGE) == 0) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.compareTo(CONTROL) == 0) {
            if ((permission instanceof LoggingPermission) == true) {
                checkLoggingPermission((LoggingPermission)permission);
                return;
            }
        }
        if (event.compareTo(MANAGEMENT_CONTROL) == 0
            || event.compareTo(MANAGEMENT_MONITOR) == 0
        ) {
            if ((permission instanceof ManagementPermission) == true) {
                checkManagementPermission((ManagementPermission)permission);
                return;
            }
        }
        if (event.compareTo(ENABLE_SUBCLASS_IMPLEMENTATION) == 0
            || event.compareTo(ENABLE_SUBSTITUTION) == 0 ) {
            checkSerializablePermission((SerializablePermission)permission) ;
            return;
        }
        if (event.compareTo(SUPPRESS_ACCESS_CHECKS) == 0) {
            checkReflectPermission((ReflectPermission)permission);
            return;
        }
        if (event.compareTo(SHOW_WINDOW_WITHOUT_WARNING_BARNNER) == 0
            || event.compareTo( ACCESS_CLIPBOARD ) == 0
            || event.compareTo(ACCESS_EVENT_QUEUE ) == 0
        ) {
            checkAWTPermission((AWTPermission)permission);
            return;
        }
        if (event.compareTo(SET_LOG) == 0
            || event.compareTo(CALL_ABORT) == 0
            || event.compareTo(SET_SYNC_FACTORY) == 0
            || event.compareTo(SET_NETWORK_TIMEOUT) == 0
        ) {
            checkSQLPermission((SQLPermission)permission);
            return;
        }

        idx = event.indexOf(DOT_SEPARATOR);
        if (idx >= 0) {
            eventhead = event.substring(0, idx);
            if (eventhead.compareTo(LOAD_LIBRARY) == 0
                || eventhead.compareTo(EXIT_VM) == 0
            ) {
                checkRuntimePermission(permission) ;
                return;
            }
        }
 
        if ((permission instanceof SecurityPermission) == true) {
            checkSecurityPermission((SecurityPermission)permission);
            return;
        }
        if ((permission instanceof PropertyPermission) == true) {
            checkPropertyPermission((PropertyPermission)permission);
            return;
        }
        if ((permission instanceof FilePermission) == true) {
            checkFilePermission((FilePermission)permission);
            return;
        }
        if ((permission instanceof SocketPermission) == true) {
            checkSocketPermission((SocketPermission)permission) ;
            return;
        }
        super.inCheck = true;
        return;
    }

    public void checkPermission(java.security.Permission permission, 
        java.lang.Object object)
        throws SecurityException, NullPointerException {
        int    idx;
        SecurityException security_exception;
        String path;
        String event;
        String actions;
        String eventhead;

        if (permission == null) {
            NullPointerException null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        event = permission.getName();
        if (event.compareTo(SET_SECURITY_MANAGER) == 0
            || event.compareTo(CREATE_CLASS_LOADER) == 0
            || event.compareTo(READ_FILE_DESCRIPTOR) == 0
            || event.compareTo(WRITE_FILE_DESCRIPTOR) == 0) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.compareTo(CREATE_SECURITY_MANAGER) == 0
            || event.compareTo(SET_IO) == 0
            || event.compareTo(MODIFY_THREAD) == 0
            || event.compareTo(STOP_THREAD) == 0
            || event.compareTo(MODIFY_THREAD_GROUP) == 0
            || event.compareTo(QUEUE_PRINT_JOB) == 0
            || event.compareTo(SET_FACTORY) == 0) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.compareTo(SET_LOG) == 0
            || event.compareTo(CALL_ABORT) == 0
            || event.compareTo(SET_SYNC_FACTORY) == 0
            || event.compareTo(SET_NETWORK_TIMEOUT) == 0) {
            checkSQLPermission((SQLPermission)permission);
            return;
        }
        if (event.compareTo(ENABLE_SUBCLASS_IMPLEMENTATION) == 0
            || event.compareTo(ENABLE_SUBSTITUTION) == 0 ) {
            checkSerializablePermission((SerializablePermission)permission) ;
            return;
        }
        if (event.compareTo(SUPPRESS_ACCESS_CHECKS) == 0) {
            checkReflectPermission((ReflectPermission)permission);
            return;
        }
        if (event.compareTo(SHOW_WINDOW_WITHOUT_WARNING_BARNNER) == 0
            || event.compareTo( ACCESS_CLIPBOARD ) == 0
            || event.compareTo(ACCESS_EVENT_QUEUE ) == 0
        ) {
            checkAWTPermission((AWTPermission)permission);
            return;
        }
        if (event.indexOf(ACCESS_CLASS_IN_PACKAGE) == 0) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.indexOf(DEFINE_CLASS_IN_PACKAGE) == 0) {
            checkRuntimePermission(permission) ;
            return;
        }
        if (event.compareTo(CONTROL) == 0) {
            if ((permission instanceof LoggingPermission) == true) {
                checkLoggingPermission((LoggingPermission)permission);
                return;
            }
        }
        idx = event.indexOf(DOT_SEPARATOR);
        if (idx >= 0) {
            eventhead = event.substring(0, idx);
            if (eventhead.compareTo(LOAD_LIBRARY) == 0
                || eventhead.compareTo(EXIT_VM) == 0) {
                checkRuntimePermission(permission) ;
                return;
            }
        }
        if ((permission instanceof SecurityPermission) == true) {
            checkSecurityPermission((SecurityPermission)permission);
            return;
        }
        if ((permission instanceof PropertyPermission) == true) {
            checkPropertyPermission((PropertyPermission)permission);
            return;
        }
        if ((permission instanceof FilePermission) == true) {
            checkFilePermission((FilePermission)permission);
            return;
        }
        if ((permission instanceof SocketPermission) == true) {
            checkSocketPermission((SocketPermission)permission) ;
            return;
        }
        if ((object instanceof AccessControlContext) == false) {
            super.inCheck = true;
            return;
        }
        AccessControlContext acc = (AccessControlContext)object;
        acc.checkPermission(permission);
        super.inCheck = true;
        return;
    }

    public void checkCreateClassLoader()
        throws SecurityException {
        RuntimePermission runtime_permission;

        runtime_permission = new RuntimePermission(CREATE_CLASS_LOADER);
        checkPermission(runtime_permission);
    }

    public void checkAccess(java.lang.Thread thread) 
        throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        RuntimePermission runtime_permission;
        if (thread == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        runtime_permission 
            = new RuntimePermission(MODIFY_THREAD);
        checkPermission(runtime_permission);
    }

    public void checkAccess(java.lang.ThreadGroup thread_group) 
        throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        RuntimePermission runtime_permission;
        if (thread_group == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        runtime_permission 
            = new RuntimePermission(MODIFY_THREAD_GROUP);
        checkPermission(runtime_permission);
    }

    public void checkExit(int status) 
        throws SecurityException {
        RuntimePermission runtime_permission;

        runtime_permission = new RuntimePermission(
            EXIT_VM + DOT_SEPARATOR + String.valueOf(status));
        checkPermission(runtime_permission);
    }
    
    public void checkExec(java.lang.String command)
        throws SecurityException {
        FilePermission file_permission;
        NullPointerException null_pointer_exception;

        if (command == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        file_permission = new FilePermission(command, EXECUTE_ACTION);
        checkPermission(file_permission);
    }

    public void checkLink(java.lang.String lib)
        throws SecurityException, NullPointerException {
        RuntimePermission runtime_permission;
        NullPointerException null_pointer_exception;

        if (lib == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        runtime_permission = new RuntimePermission(
            LOAD_LIBRARY + DOT_SEPARATOR + lib);
        checkPermission(runtime_permission);
    }

    public void checkRead(java.io.FileDescriptor file_descriptor)
        throws SecurityException, NullPointerException {
        RuntimePermission runtime_permission;
        NullPointerException null_pointer_exception;

        if (file_descriptor == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        runtime_permission = new RuntimePermission(READ_FILE_DESCRIPTOR);
        checkPermission(runtime_permission);
    }

    public void checkRead(java.lang.String file)
        throws SecurityException, NullPointerException {
        FilePermission file_permission;
        NullPointerException null_pointer_exception;

        if (file == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        file_permission = new FilePermission(file, READ_ACTION);
        checkPermission(file_permission);
    }

    public void checkRead(java.lang.String file, java.lang.Object context)
        throws SecurityException, NullPointerException {
        FilePermission file_permission;
        NullPointerException null_pointer_exception;

        if (file == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        file_permission = new FilePermission(file, READ_ACTION);
        checkPermission(file_permission, context);
    }

    public void checkWrite(java.io.FileDescriptor file_descriptor)
        throws SecurityException, NullPointerException {
        RuntimePermission runtime_permission;
        NullPointerException null_pointer_exception;

        if (file_descriptor == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        runtime_permission = new RuntimePermission(WRITE_FILE_DESCRIPTOR);
        checkPermission(runtime_permission);
    }

    public void checkWrite(java.lang.String file)
        throws SecurityException, NullPointerException {
        FilePermission file_permission;
        NullPointerException null_pointer_exception;

        if (file == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        file_permission = new FilePermission(file, WRITE_ACTION);
        checkPermission(file_permission);
    }

    public void checkDelete(java.lang.String file)
        throws SecurityException, NullPointerException {
        FilePermission file_permission;
        NullPointerException null_pointer_exception;

        if (file == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        file_permission = new FilePermission(file, DELETE_ACTION);
        checkPermission(file_permission);
    }

    public void checkConnect(java.lang.String hostname, int port, 
        Object context) throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        SocketPermission socket_permission;

        if (hostname == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if ((context instanceof AccessControlContext) == false) {
            SecurityException security_exception;
        }
        socket_permission = new SocketPermission(
            hostname + HOST_SEPARATOR + String.valueOf(port),
            CONNECT_ACTION);
        checkPermission(socket_permission);
    }

    public void checkConnect(java.lang.String hostname, int port)
        throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        SocketPermission socket_permission;

        if (hostname == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        socket_permission = new SocketPermission(
            hostname + HOST_SEPARATOR + String.valueOf(port),
            CONNECT_ACTION);
        checkPermission(socket_permission);
    }

    public void checkListen(int port)
        throws SecurityException {
        SocketPermission socket_permission;
        socket_permission = new SocketPermission(
            LOCAL_HOSTNAME + HOST_SEPARATOR + String.valueOf(port),
            LISTEN_ACTION);
        checkPermission(socket_permission);
    }

    public void checkAccept(java.lang.String hostname, int port)
        throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        SocketPermission socket_permission;
        if (hostname == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        socket_permission = new SocketPermission(
            hostname + HOST_SEPARATOR + String.valueOf(port),
            ACCEPT_ACTION);
        checkPermission(socket_permission);
    }

    public void checkMulticast(java.net.InetAddress maddr)
        throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        SocketPermission socket_permission;

        if (maddr == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        socket_permission = new SocketPermission(
            maddr.getHostAddress(),
            ACCEPT_CONNECT_ACTION);
        checkPermission(socket_permission);
    }

    public void checkMulticast(java.net.InetAddress maddr, byte ttl)
        throws SecurityException, NullPointerException  {
        NullPointerException null_pointer_exception;
        SocketPermission socket_permission;
        if (maddr == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        socket_permission = new SocketPermission(
            maddr.getHostAddress(),
            ACCEPT_CONNECT_ACTION);
        checkPermission(socket_permission);
    }

    public void checkPropertiesAccess()
        throws SecurityException {
        SecurityException security_exception;
        PropertyPermission property_permission;
        property_permission = new PropertyPermission(
            WILDCARD, READ_WRITE_ACTIONS
        );
        checkPermission(property_permission);
    }

    public void checkPropertyAccess(java.lang.String key)
        throws SecurityException, NullPointerException, IllegalArgumentException {
        SecurityException security_exception;
        PropertyPermission property_permission;
        property_permission = new PropertyPermission(
            key, READ_ACTION
        );
        checkPermission(property_permission);
    }

    public boolean checkTopLevelWindow(java.lang.Object window) 
        throws NullPointerException {
        AWTPermission awtpermission;

        if (window == null) {
            NullPointerException null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        awtpermission = new AWTPermission(
            SHOW_WINDOW_WITHOUT_WARNING_BARNNER);
        try {
            checkPermission(awtpermission);
        } catch (SecurityException security_exception) {
            return false;
        }
        return true;
    }

    public void checkPrintJobAccess()
        throws SecurityException {
        RuntimePermission runtime_permission;
        runtime_permission = new RuntimePermission(QUEUE_PRINT_JOB);
        checkPermission(runtime_permission);
    }

    public void checkSystemClipboardAccess()
        throws SecurityException {
        AWTPermission awtpermission;

        awtpermission = new AWTPermission( ACCESS_CLIPBOARD );

        checkPermission(awtpermission);
        return ;
    }

    public void checkAwtEventQueueAccess()
        throws SecurityException {
        AWTPermission awtpermission;

        awtpermission = new AWTPermission( ACCESS_EVENT_QUEUE );

        checkPermission(awtpermission);
        return ;
    }

    public void checkPackageAccess(java.lang.String pkg)
        throws SecurityException, NullPointerException {
        RuntimePermission runtime_permission;
        NullPointerException null_pointer_exception;

        if (pkg == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        runtime_permission = new RuntimePermission(
            ACCESS_CLASS_IN_PACKAGE + pkg);

        checkPermission(runtime_permission);
    }

    public void checkPackageDefinition(java.lang.String pkg)
        throws SecurityException, NullPointerException {
        RuntimePermission runtime_permission;
        NullPointerException null_pointer_exception;

        if (pkg == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        runtime_permission = new RuntimePermission(
            DEFINE_CLASS_IN_PACKAGE + pkg);
        checkPermission(runtime_permission);
    }

    public void checkSetFactory()
        throws SecurityException {
        RuntimePermission runtime_permission;
        runtime_permission = new RuntimePermission(SET_FACTORY);
        checkPermission(runtime_permission);
    }

    public void checkMemberAccess(java.lang.Class class_object, int which)
        throws SecurityException, NullPointerException {
        NullPointerException null_pointer_exception;
        if (class_object == null) {
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        RuntimePermission runtime_permission;
        runtime_permission = new RuntimePermission(ACCESS_DECLARE_MEMBERS);
        checkPermission(runtime_permission);
    }

    public void checkSecurityAccess(java.lang.String target)
        throws SecurityException, NullPointerException, 
               IllegalArgumentException {
        if (target == null) {
            NullPointerException null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (target.length() == 0) {
            IllegalArgumentException illegal_argument_exception;
            illegal_argument_exception = new IllegalArgumentException();
        }
        SecurityPermission security_permission;
        security_permission = new SecurityPermission(target);
        checkPermission(security_permission);
    }

    public java.lang.ThreadGroup getThreadGroup()
        throws SecurityException {
        ThreadGroup thread_group;
        thread_group = Thread.currentThread().getThreadGroup();
        return thread_group;
    }

    public static String makeCanonical (java.lang.String file) throws SecurityException {
        int idx;
        int start_idx;
        if (file == null || file.length() == 0) {
            return file;
        }
        int length = file.length();
        int[] start;
        int[] end;
        String  source;
        int parent;
        int  ii;

        parent = 0;
        start = new int[length];
        end   = new int[length];
        idx = 0;
        ii = 0;
        if (file.charAt(0) == File.separatorChar) {
            start[idx] = 1;
            start_idx = 1;
        } else {
            start[idx] = 0;
            start_idx = 0;
        }
        for (int i = start_idx; i < length; i++) {
            if (file.charAt(i) == CURRENT_CHARACTER) {
                if (i == length - 1) {
                    if (start[idx] == i) {
                        break;
                    }
                    end[idx] = i;
                    idx = idx + 1;
                    break;
                }
                if (i == length - 2) {
                    if (file.charAt(i + 1) == File.separatorChar) {
                        if (start[idx] == i) {
                            break;
                        }
                        end[idx] = i;
                        idx = idx + 1;
                        break;
                    }
                    if (file.charAt(i + 1) == CURRENT_CHARACTER) {
                        if (idx == 0) {
                            SecurityException exception_object;
                            exception_object = new SecurityException(PARENT_ACCESS_EXCEPTION);
                            throw exception_object;
                        }
                        idx = idx - 1;
                        break;
                    }
                    end[idx] = i + 2;
                    idx = idx + 1;
                    break;
                }               
                if (file.charAt(i + 1) == File.separatorChar) {
                    for (ii = i + 2; ii < length; ii++) {
                        if (file.charAt(ii) != File.separatorChar) { break; }
                    }
                    if (length == ii) {
                        break;
                    }       
                    start[idx] = ii;
                    i = ii - 1;
                    continue;
                }
                if (file.charAt(i + 1) == CURRENT_CHARACTER) {
                    if (file.charAt(i + 2) == File.separatorChar) {
                        for (ii = i + 3; ii < length; ii++) {
                            if (file.charAt(ii) != File.separatorChar) { break; }
                        }
                        if (idx == 0) {
                            SecurityException exception_object;
                            exception_object = new SecurityException(PARENT_ACCESS_EXCEPTION);
                            throw exception_object;
                        }
                        idx = idx - 1;
                        if (ii == length) { break; }
                        start[idx] = ii;
                        i = ii - 1;
                        continue;
                    }
                    for (ii = i + 3; ii < length; ii++) {
                        if (file.charAt(ii) == File.separatorChar) { 
                            break; 
                        }
                    }
                    end[idx] = ii -1;
                    idx = idx + 1;
                    start[idx] = ii;
                    i = ii - 1;
                    continue;
                }
                for (ii = i + 2; ii < length; ii++) {
                    if (file.charAt(ii) == File.separatorChar) { 
                       break; 
                    }
                }
                end[idx] = ii;
                idx = idx + 1;
                i = ii - 1;
                continue;
            }
            if (file.charAt(i) == File.separatorChar) {
                for (ii = i + 1; ii < length; ii++) {
                    if (file.charAt(ii) != File.separatorChar) { break; }
                }
                end[idx] = ii;

                idx = idx + 1;
                start[idx] = ii;
                i = ii -1;
                continue;
            }
            for (ii = i + 1; ii < length; ii++) {
                if (file.charAt(ii) == File.separatorChar) { 
                   break; 
                }
            }
            if (ii == length) {
                end[idx] = ii;
                idx = idx + 1;
                break;
            }
            i = ii - 1;
        }
        if (file.charAt(0) == File.separatorChar) {
            source = File.separator;
        } else {
            source = new String();
         }
        if (idx == 0) {
            if (file.charAt(0) == File.separatorChar) {
                return File.separator;
            }
            return CURRENT_DIRECTORY;
        }
        for (int i = 0; i < idx; i++) {
            source = source + file.substring(start[i], end[i]);
        }
        return source;
    }
}
