/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectStub.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/07/23
 *             2.1 2010/09/17 changed location format from URL to URI.
 *             2.1 2010/10/08
 *             2.1 2012/01/26 changed for Japanese
 *
 * This Class is provided for user
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

import oserv.*;

/**
 * This class provides a method to load user's service class and 
 * to create instance.
 *
 * @see java.lang.Object
 */
public class ObjectStub extends ObjectServerHeader {
    public static final String OSERV_SCHEME = "oserv";
/*
    private static MessageResource message_resource = new MessageResource();
*/

    public static String replace(String s0, String s1, String s2) {
        int    idx;
        String s3;

        s3 = s0;
        for (int i = 0; i < s0.length(); i++) {
            idx = s3.indexOf(s1, i);
            if (idx >= 0) {
                s3 = s3.substring(0, idx) + 
                    s2 +
                    s3.substring(idx+s1.length());
            }
        }
        return s3;
    }
    /**
     * This method is a feature to convert from user's class name to instance
     *
     * @param  String - user's service class name
     * @return Object - created instance
     * @throw  ClassNotFoundException
     *         InstantiationException
     *         IllegalAccessException
     */
    public Object getService(String service) throws 
        java.lang.ClassNotFoundException, 
        java.lang.InstantiationException, 
        java.lang.IllegalAccessException,
        ObjectServerException {

        Object object_instance;
        Class  class_object;
        ObjectServerException    object_server_exception;

        if (service == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerException.SERVICE_GETSERVICE_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerException.SERVICE_GETSERVICE_EXCEPTION)));

            throw object_server_exception;
        }
        try {
            class_object = Class.forName(service);
        } catch (ClassNotFoundException class_not_found_exception) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            object_instance = class_object.newInstance();
        } catch (InstantiationException instantiation_exception) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION)));
            throw object_server_exception;
        } catch (IllegalAccessException illegal_access_exception) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.JAVA_CLASS_NEW_INSTANCE_ILLEGAL_ACCESS_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_ILLEGAL_ACCESS_EXCEPTION)));
            throw object_server_exception;
        }

        return (Object)object_instance;
    }

    /**
     * This method is a feature to convert from user's class name to instance
     *
     * @param  URI    - oserv://hostname:port/package/Serviec
     * @return Object - created instance
     * @throw  ClassNotFoundException
     *         InstantiationException
     *         IllegalAccessException
     */
    public Object getService(URI service) throws 
        java.lang.ClassNotFoundException,
        java.lang.InstantiationException, 
        java.lang.IllegalAccessException,
        oserv.ObjectServerException {
        Object object_instance;
        Class  class_object;

        int port;
        String scheme;
        String hostname;
        String real_service;
        ObjectServerException object_server_exception;

        Properties oserv_properties;

        if (service == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerException.SERVICE_GETSERVICE_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerException.SERVICE_GETSERVICE_EXCEPTION)));
            throw object_server_exception;
        }
        oserv_properties = new Properties();

        scheme = service.getScheme();
        if (scheme.compareTo(OSERV_SCHEME) != 0) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerException.SCHEME_NAME_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerException.SCHEME_NAME_ERROR_EXCEPTION)));
            throw object_server_exception;
        }
        port = service.getPort();
        hostname = service.getHost();
        real_service = service.getPath();
        if (real_service.indexOf(ObjectServerHeader.DIRECTORY_SEPARATOR) == 0) {
            real_service = real_service.substring(1);
        }
        real_service = replace(real_service,
            ObjectServerHeader.DIRECTORY_SEPARATOR,
            ObjectServerHeader.OBJECT_DOT_SEPARATOR);

        oserv_properties.setProperty(
            ObjectServerHeader.OSERV_SERVICE_HOSTNAME_KEY, hostname);
        if (port > 0) {
            oserv_properties.setProperty(
            ObjectServerHeader.OSERV_REQUEST_PORT_KEY, String.valueOf(port));
        }
        System.setProperties(oserv_properties);
        
        try {
            class_object = Class.forName(real_service);
        } catch (ClassNotFoundException exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            object_instance = class_object.newInstance();
        } catch (InstantiationException exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION)));
            throw object_server_exception;
        } catch (IllegalAccessException exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.JAVA_CLASS_NEW_INSTANCE_ILLEGAL_ACCESS_EXCEPTION,
                    getMessageResource().getMessage(
	                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_ILLEGAL_ACCESS_EXCEPTION)));
            throw object_server_exception;
        }
        return (Object)object_instance;
    }
}

