/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectToolTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/03/07
 */

package oserv;

import java.lang.*;
import java.io.*;

public class ObjectToolTest extends ObjectTool {
    private String COMMAND_STRING = "Command";
    private String LIST_COMMAND = "List Command";
    private String NOT_LIST_COMMAND = "Not List Command";
    private String DELETE_COMMAND = "Delete Command";
    private String NOT_DELETE_COMMAND = "Not Delete Command";
    private String CLASS_INPUT_STREAM = "Class input stream";
    private String REGISTER_COMMAND = "Register Command";
    private String NOT_REGISTER_COMMAND = "Not Register Command";

    private String EOF_EXCEPTION = "java.io.EOFException";
    private String REGISTER_COMMAND_MESSAGE = "RegisterCommand Object";
    private String MESSAGE_STRING = "Message";

    public void test1() {
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;

        Object                  command_object;
        String                  command_string;

        try {
            byte_array_output_stream = new ByteArrayOutputStream();
            object_output_stream     = new ObjectOutputStream(
                                           byte_array_output_stream);
            writeCommand(object_output_stream);
        } catch (Exception   exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            byte_array_input_stream = new ByteArrayInputStream(
                byte_array_output_stream.toByteArray() );
            object_input_stream     = new ObjectInputStream(
                                          byte_array_input_stream);
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }

        while (true) {
            try {
                command_object = (Object)object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                exception_object.printStackTrace();
                return;
            }
            if ((command_object instanceof String) == true) {
                command_string = (String)command_object;
                System.out.println(
                    COMMAND_STRING + BLANK_STRING + command_string);
            }
        }
    
    }

    public void test2() {
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;

        Object                  command_object;
        RegisterCommand         register_command;

        try {
            byte_array_output_stream = new ByteArrayOutputStream();
            object_output_stream     = new ObjectOutputStream(
                                           byte_array_output_stream);
            requireList(object_output_stream);
        } catch (Exception   exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            byte_array_input_stream = new ByteArrayInputStream(
                byte_array_output_stream.toByteArray() );
            object_input_stream     = new ObjectInputStream(
                                          byte_array_input_stream);
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }
        while (true) {
            try {
                command_object = (Object)object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                exception_object.printStackTrace();
                return;
            }
            if ((command_object instanceof RegisterCommand) == true) {
                register_command = (RegisterCommand)command_object;
                if (register_command.isList() == true) {
                    System.err.println( LIST_COMMAND );
                } else {
                    System.err.println( NOT_LIST_COMMAND );
                }
            }
        }
    }

    private String TEST3_JAR_CLASS = "sample.SampleServer";

    public void test3() {
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;

        String                  jarclass;
        String                  class_string;
        Object                  command_object;
        RegisterCommand         register_command;
        RegisterCommand         delete_command;

        try {
            byte_array_output_stream = new ByteArrayOutputStream();
            object_output_stream     = new ObjectOutputStream(
                                           byte_array_output_stream);
            jarclass = TEST3_JAR_CLASS;
            requireRemovement(object_output_stream, jarclass);
        } catch (Exception   exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            byte_array_input_stream = new ByteArrayInputStream(
                byte_array_output_stream.toByteArray() );
            object_input_stream     = new ObjectInputStream(
                                          byte_array_input_stream);
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }

        while (true) {
            try {
                command_object = (Object)object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                exception_object.printStackTrace();
                return;
            }
            if ((command_object instanceof String) == true) {
                System.out.println(MESSAGE_STRING +
                    ObjectServerHeader.BLANK_STRING + (String)command_object);
                continue;
            }
            if ((command_object instanceof RegisterCommand) == true) {
                register_command = (RegisterCommand)command_object;
                try {
                    class_string = register_command.getClassName();
                } catch (Exception exception_object) {
                    exception_object.printStackTrace();
                    return;
                }
                if (register_command.isDelete() == true) {
                    System.err.println( DELETE_COMMAND + BLANK_STRING
                    + class_string);
                } else {
                    System.err.println( NOT_DELETE_COMMAND + BLANK_STRING
                    + class_string );
                }
            }
        }
    }

    public void test4() {
        String             class_path;
        ClassInputStream   class_input_stream;

        class_path = TEST3_JAR_CLASS;
        try {
            class_input_stream = ClassInputStream.valueOf(class_path,
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        System.err.println(CLASS_INPUT_STREAM + BLANK_STRING
            + class_input_stream.toString() 
            + BLANK_STRING
            + class_input_stream.getClassPath()
            + BLANK_STRING
            + class_input_stream.getType()
            + BLANK_STRING
            + String.valueOf(class_input_stream.length())
        );
    }

    public void test5() {
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;

        Object                  command_object;
        String                  class_string;
        String                  class_name_string;
        ClassInputStream        class_input_stream;
        RegisterCommand         register_command;

        try {
            byte_array_output_stream = new ByteArrayOutputStream();
            object_output_stream     = new ObjectOutputStream(
                                           byte_array_output_stream);
        } catch (Exception   exception_object) {
            exception_object.printStackTrace();
            return;
        }
        class_string = TEST3_JAR_CLASS;
        try {
            class_input_stream = ClassInputStream.valueOf(class_string,
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        System.err.println(CLASS_INPUT_STREAM + BLANK_STRING
            + class_input_stream.toString() 
            + BLANK_STRING
            + class_input_stream.getClassPath()
            + BLANK_STRING
            + class_input_stream.getType()
            + BLANK_STRING
            + String.valueOf(class_input_stream.length()));

        try {
            requireRegistration(object_output_stream, class_input_stream);
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            byte_array_input_stream = new ByteArrayInputStream(
                byte_array_output_stream.toByteArray() );
            object_input_stream     = new ObjectInputStream(
                                          byte_array_input_stream);
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }

        while (true) {
            try {
                command_object = (Object)object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                exception_object.printStackTrace();
                return;
            }
            if ((command_object instanceof RegisterCommand) == true) {
                register_command = (RegisterCommand)command_object;
                try {
                    class_name_string = register_command.getClassName();
                } catch (Exception exception_object) {
                    exception_object.printStackTrace();
                    return;
                }
                if (register_command.isRegister() == true) {
                    System.err.println( REGISTER_COMMAND +
                        BLANK_STRING + class_name_string);
                } else {
                    System.err.println( NOT_REGISTER_COMMAND );
                }
            }
        }
    }

    private String GENERATE_LIST_COMMAND = "Generate List command";
    private String GENERATE_REMOVE_COMMAND = "Generate Remove command";
    private String GENERATE_REGISTER_COMMAND = "Generate Register command";

    public void test6() {
        ClassInputStream        class_input_stream;
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;

        RegisterCommand         register_command;
        RegisterCommand         list_command;
        RegisterCommand         delete_command;

        String                  class_name;

        class_name = TEST3_JAR_CLASS;

        System.err.println(GENERATE_LIST_COMMAND);
        try {
            list_command = RegisterCommand.listCommand(
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        System.err.println(LIST_COMMAND+BLANK_STRING+
            list_command.toString());
        System.err.println(GENERATE_REMOVE_COMMAND);
        try {
            delete_command = RegisterCommand.deleteCommand(class_name,
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        System.err.println(DELETE_COMMAND+BLANK_STRING+
            delete_command.toString());
        System.err.println(GENERATE_REGISTER_COMMAND);
        try {
            class_input_stream = ClassInputStream.valueOf(
                class_name,
                getMessageResource(),
                getDebug(),
                getLogStream() );
            register_command = RegisterCommand.registerCommand(
                class_input_stream.getClassPath(),
                class_input_stream.getType(),
                (int)class_input_stream.length(),
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        System.err.println(REGISTER_COMMAND+BLANK_STRING+
            register_command.toString());
    }

    public void test11() {
        String   command;
        String   class_name;

        command = ObjectTool.LIST_COMMAND;
        class_name = null;
        try {
            exec(command, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }

    public void test12() {
        String   command;
        String   class_name;

        command = ObjectTool.REMOVE_COMMAND;
        class_name = TEST3_JAR_CLASS;
        try {
            exec(command, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }

    public void test13() {
        String   command;
        String   class_name;
        String   user_home;
        File     fd;

        user_home = "../../OSERV2.1/Test71/";
        fd = new File(user_home);
        user_home = fd.getAbsolutePath();
        System.setProperty("user.home", user_home);
        command = ObjectTool.REGISTER_COMMAND;
        class_name = TEST3_JAR_CLASS;
        try {
            exec(command, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }

    public void test21() {
        String   command;
        String   class_name;
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;

        String                  jarclass;
        String                  class_string;
        Object                  command_object;
        RegisterCommand         register_command;
        RegisterCommand         delete_command;

        try {
            byte_array_output_stream = new ByteArrayOutputStream();
            object_output_stream     = new ObjectOutputStream(
                                           byte_array_output_stream);
            jarclass = TEST3_JAR_CLASS;
            writeCommand(object_output_stream);
            requireRemovement(object_output_stream, jarclass);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            byte_array_input_stream = new ByteArrayInputStream(
                byte_array_output_stream.toByteArray() );
            object_input_stream     = new ObjectInputStream(
                                          byte_array_input_stream);
        } catch (Exception  exception_object) {
            exception_object.printStackTrace();
            return;
        }
        while (true) {
            try {
                command_object = (Object)object_input_stream.readObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                exception_object.printStackTrace();
                return;
            }
            if ((command_object instanceof String) == true) {
System.out.println(MESSAGE_STRING +
    ObjectServerHeader.BLANK_STRING + (String)command_object);
                continue;
            }
            if ((command_object instanceof RegisterCommand) == true) {
System.out.println(REGISTER_COMMAND_MESSAGE +
    ObjectServerHeader.BLANK_STRING + command_object.toString() +
    ObjectServerHeader.BLANK_STRING + String.valueOf(
        ((RegisterCommand)command_object).isDelete()) );
                break;
            }
        }
    }

    public void test31() {
        File     fd;
        String   command;
        String   class_name;
        String   user_home;
        ByteArrayOutputStream   byte_array_output_stream;
        ObjectOutputStream      object_output_stream;

        ByteArrayInputStream    byte_array_input_stream;
        ObjectInputStream       object_input_stream;
        ClassInputStream        class_input_stream;

        command = null;
        class_name = TEST3_JAR_CLASS;
        try {
            exec(command, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
        command = ObjectTool.REGISTER_COMMAND;
        class_name = null;
        try {
            exec(command, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
        command = ObjectTool.REMOVE_COMMAND;
        class_name = null;
        try {
            exec(command, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
        try {
            object_output_stream     = null;
            writeCommand(object_output_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
        try {
            object_output_stream     = null;
            class_name = TEST3_JAR_CLASS;
            requireRemovement(object_output_stream, class_name);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
        user_home = "../../OSERV2.1/Test71/";
        fd = new File(user_home);
        user_home = fd.getAbsolutePath();
        System.setProperty("user.home", user_home);

        class_name = TEST3_JAR_CLASS;
        try {
            class_input_stream = ClassInputStream.valueOf(class_name,
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            object_output_stream     = null;
            requireRegistration(object_output_stream, class_input_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
        }
    }


    private static String TEST1 = "Test1";
    private static String TEST2 = "Test2";
    private static String TEST3 = "Test3";
    private static String TEST4 = "Test4";
    private static String TEST5 = "Test5";
    private static String TEST6 = "Test6";
    private static String TEST11 = "Test11";
    private static String TEST12 = "Test12";
    private static String TEST13 = "Test13";
    private static String TEST21 = "Test21";
    private static String TEST31 = "Test31";

    private static String PARAMETER_ERROR = "Parameter error";

    public static void main(String[] args) {
        ObjectToolTest   object_tool_test;

        object_tool_test = new ObjectToolTest();

        if (args.length == 1) {
            if (args[0] != null && args[0].compareTo(TEST1) == 0) {
                object_tool_test.test1();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST2) == 0) {
                object_tool_test.test2();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST3) == 0) {
                object_tool_test.test3();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST4) == 0) {
                object_tool_test.test4();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST5) == 0) {
                object_tool_test.test5();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST6) == 0) {
                object_tool_test.test6();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST11) == 0) {
                object_tool_test.test11();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST12) == 0) {
                object_tool_test.test12();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST13) == 0) {
                object_tool_test.test13();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST21) == 0) {
                object_tool_test.test21();
                return;
            }
            if (args[0] != null && args[0].compareTo(TEST31) == 0) {
                object_tool_test.test31();
                return;
            }
        }
        System.err.println(PARAMETER_ERROR);
    }
}
