/**
 * Copyright (C) All rights reserved.
 *
 * @Module     Package.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/11/25
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

class cp_info {
    byte  tag;
    byte getTag() {
        return tag;
    }
    void setTag(byte tagparam) {
        tag = tagparam;
    }
}

class CONSTANT_Class_info extends cp_info {
    short  name_index;
}

class CONSTANT_Fieldref_info extends cp_info {
    short  class_index;
    short  name_and_type_index;
}

class CONSTANT_Methodref_info extends cp_info {
    short  class_index;
    short  name_and_type_index;
}

class CONSTANT_InterfaceMethodref_info extends cp_info {
    short   class_index;
    short   name_and_type_index;
}

class CONSTANT_String_info extends cp_info {
    short    string_index;
}

class CONSTANT_Integer_info extends cp_info {
    int       bytes;
}

class CONSTANT_Float_info extends cp_info {
    float       bytes;
}

class CONSTANT_Long_info extends cp_info {
    int       high_bytes;
    int       low_bytes;
}

class CONSTANT_Double_info extends cp_info {
    int       high_bytes;
    int       low_bytes;
}

class CONSTANT_NameAndType_info extends cp_info {
    short  name_index;
    short  descriptor_index;
}

class CONSTANT_Uft8_info extends cp_info {
    short     length;
    String    bytes;
}


public class Package extends SerializableObject {
    public final static int HEADER_MAGIC_SIZE        = 4;
    public final static int HEADER_VERSION_SIZE      = 4;
    public final static int HEADER_ACCESS_FLAGS_SIZE = 2;
    public final static int HEADER_CLASS_INFO_SIZE   = 4;
    public final static String PACKAGE_SEPARATOR = "/";
    public final static String DEBUG_PACKAGE_NAME = "Package ";
    public final static String DEBUG_NULL = "null";
    public final static String DEBUG_SENTENCE_END = ".";

    private static String GET_POOL    = "getPool";
    private static String GET_PACKAGE = "getPackage";

    public final static String ERROR_CONSTANT_CLASS     = "CONSTANT_Class";
    public final static String ERROR_CONSTANT_FIELDREF  = "CONSTANT_Fieldref";
    public final static String ERROR_CONSTANT_METHODREF = "CONSTANT_Methodref";
    public final static String ERROR_CONSTANT_UFT8 = "CONSTANT_Uft8";
    public final static String ERROR_CONSTANT_INTERFACEMETHODREF
        = "CONSTANT_InterfaceMethodref";
    public final static String ERROR_CONSTANT_LONG  = "CONSTANT_Long";
    public final static String ERROR_CONSTANT_DOUBLE = "CONSTANT_Double";
    public final static String ERROR_CONSTANT_FLOAT = "CONSTANT_Float";
    public final static String ERROR_CONSTANT_STRING = "CONSTANT_String";
    public final static String ERROR_CONSTANT_INTEGER = "CONSTANT_Integer";
    public final static String ERROR_CONSTANT_NAMEANDTYPE_NAME_INDEX
        = "CONSTANT_NameAndType name_index";
    public final static String ERROR_CONSTANT_NAMEANDTYPE_DESCRIPTOR_INDEX
        = "CONSTANT_NameAndType descriptor_index";

    public final static String MESSAGE_EXCEPTION = "Exception ";

    public final static String EXCEPTION_PARAMETER_ERROR = "Parameter error";

    public final static String POOL_SIZE = "pool_size";

    static cp_info[] pool = null;
    static String target_file_name = null;

    private Package() {
    }

    public Package(String class_file) throws Exception {
        setFile(class_file);
    }

    private void setFile(String class_file) throws Exception {
        File file_descriptor;

        if (class_file == null) {
            ObjectServerException    object_server_exception;

            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_PACKAGE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_PACKAGE)));

            throw object_server_exception;
        }
        file_descriptor = new File(class_file);
        if (file_descriptor.exists() == true) {
            target_file_name = class_file;
        }
        return ;
    }

    public static Package valueOf(String class_file,
        MessageResource    parameter_message_resource,
        int                parameter_debug,
        LogPrintStream     parameter_log_print_stream)  throws Exception
    {
        Package   package_object;

        package_object = new Package();
        package_object.setMessageResource(parameter_message_resource);
        package_object.setDebugLogStream(
            parameter_debug,   parameter_log_print_stream);

        try {
            package_object.setFile(class_file);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return package_object;
    }

    public void getPool(DataInputStream data_input_stream) throws Exception {
        int    cindex;
        short  npool;
        byte[] bs = new byte[4];
        ObjectServerException   object_server_exception;

        /** constant pool **/
        try {
            npool = data_input_stream.readShort();
        } catch (Exception exception_object) {
            printExceptionMessage(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                exception_object, GET_POOL, 0);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                exception_object);
            throw object_server_exception;
        }
        pool = new cp_info[npool -1];

        cindex = 0;
        for (int i = 0; i < npool - 1; i++) {
            try {
                bs[0] = data_input_stream.readByte();
            } catch (Exception exception_object) {
                printExceptionMessage(
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                    exception_object, GET_POOL, 0);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                    exception_object);
                throw object_server_exception;
            }
            if (bs[0] == 7) {
                CONSTANT_Class_info cinfo = new CONSTANT_Class_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.name_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 9) {
                CONSTANT_Fieldref_info cinfo = new CONSTANT_Fieldref_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.class_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                try {
                    cinfo.name_and_type_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 10) {
                CONSTANT_Methodref_info cinfo = new CONSTANT_Methodref_info();
                cinfo.setTag(bs[0]);
                try {
                    cinfo.class_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                try {
                    cinfo.name_and_type_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 11) {
                CONSTANT_InterfaceMethodref_info cinfo =
                    new CONSTANT_InterfaceMethodref_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.class_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                try {
                    cinfo.name_and_type_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 8) {
                CONSTANT_String_info cinfo = new CONSTANT_String_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.string_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 3) {
                CONSTANT_Integer_info cinfo = new CONSTANT_Integer_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.bytes = data_input_stream.readInt();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 4) {
                CONSTANT_Float_info cinfo = new CONSTANT_Float_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.bytes = data_input_stream.readFloat();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_FLOAT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_FLOAT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_FLOAT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_FLOAT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 5) {
                CONSTANT_Long_info cinfo = new CONSTANT_Long_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.high_bytes = data_input_stream.readInt();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                try {
                    cinfo.low_bytes = data_input_stream.readInt();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 6) {
                CONSTANT_Double_info cinfo = new CONSTANT_Double_info();
                cinfo.setTag(bs[0]);
                try {
                    cinfo.high_bytes = data_input_stream.readInt();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                try {
                    cinfo.low_bytes = data_input_stream.readInt();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_INT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 12) {
                CONSTANT_NameAndType_info cinfo = new CONSTANT_NameAndType_info();
                cinfo.setTag(bs[0]);

                try {
                    cinfo.name_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                try {
                    cinfo.descriptor_index = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }

                pool[i] = cinfo;
                continue;
            }
            if (bs[0] == 1) {
                CONSTANT_Uft8_info cinfo = new CONSTANT_Uft8_info();
                cinfo.setTag(bs[0]);
                try {
                    cinfo.length = data_input_stream.readShort();
                } catch (Exception exception_object) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object, GET_POOL, 0);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                        exception_object);
                    throw object_server_exception;
                }
                ByteArrayOutputStream boa;
                PrintStream pst;
                boa = new ByteArrayOutputStream();
                pst = new PrintStream(boa);

                for (int j = 0; j < cinfo.length;j++) {
                    try {
                        bs[0] = data_input_stream.readByte();
                    } catch (Exception exception_object) {
                        printExceptionMessage(
                            ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                            ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                            exception_object, GET_POOL, 0);
                        object_server_exception = getObjectServerException(
                            ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                            ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_BYTE_IO_EXCEPTION,
                            exception_object);
                        throw object_server_exception;
                    }
                    pst.printf("%c", bs[0]);

                }
                cinfo.bytes = boa.toString();
                pool[i] = cinfo;
                continue;
            }
        }
        return ;
    }

    public String getPackage() throws Exception {
        int    count;
        short  sword1;
        String class_name;
        String package_name;
        FileInputStream file_input_stream;
        DataInputStream data_input_stream;
        ObjectServerException  object_server_exception;

        if (target_file_name == null) {
            return null;
        }

        try {
            file_input_stream = new FileInputStream(target_file_name);
        } catch (Exception exception_object) {
            printExceptionMessage(
                ObjectServerError.JAVA_FILE_INPUT_STREAM_FILE_NOT_FOUND_EXCEPTION,
                ObjectServerError.JAVA_FILE_INPUT_STREAM_FILE_NOT_FOUND_EXCEPTION,
                exception_object, GET_PACKAGE, 0);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FILE_INPUT_STREAM_FILE_NOT_FOUND_EXCEPTION,
                ObjectServerError.JAVA_FILE_INPUT_STREAM_FILE_NOT_FOUND_EXCEPTION,
                exception_object);
            throw object_server_exception;
        }
        data_input_stream = new DataInputStream(file_input_stream);

        try {
            count = HEADER_MAGIC_SIZE + HEADER_VERSION_SIZE;
            data_input_stream.skipBytes(count);
        } catch (Exception exception_object) {
            printExceptionMessage(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_SKIP_BYTES_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_INPUT_STREAM_SKIP_BYTES_IO_EXCEPTION,
                exception_object, GET_PACKAGE, 0);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_SKIP_BYTES_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_INPUT_STREAM_SKIP_BYTES_IO_EXCEPTION,
                exception_object);
            throw object_server_exception;
        }
        try {
            getPool(data_input_stream);
            count = HEADER_ACCESS_FLAGS_SIZE;
            data_input_stream.skipBytes(count);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            sword1 = data_input_stream.readShort();
        } catch (Exception exception_object) {
            printExceptionMessage(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                exception_object, GET_PACKAGE, 0);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_SHORT_IO_EXCEPTION,
                exception_object);
            throw object_server_exception;
        }
        class_name = ((CONSTANT_Uft8_info)pool[((CONSTANT_Class_info)pool[sword1-1]).name_index-1]).bytes ;
        package_name = null;
        int idx = class_name.lastIndexOf(PACKAGE_SEPARATOR);
        if (idx > 0) {
            package_name = class_name.substring(0, idx);
            class_name = class_name.substring(idx+1);
        }
        return package_name;
 
    }

    public static void main(String[] ag) {
        Package package_object;
        try {
            package_object = new Package(ag[0]);
        } catch (Exception exception_object) {
            System.out.println(MESSAGE_EXCEPTION + exception_object.getMessage());
            return;
        }
        String package_name;

        try {
            package_name = package_object.getPackage();
        } catch (Exception exception_object) {
            System.out.println(MESSAGE_EXCEPTION + exception_object.getMessage());
            return;
        }
        if (package_name == null) {
            System.out.println(DEBUG_PACKAGE_NAME + DEBUG_NULL
                + DEBUG_SENTENCE_END);
        } else {
            System.out.println(DEBUG_PACKAGE_NAME + package_name
                + DEBUG_SENTENCE_END);
        }
    }
}
