/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ParseOptions.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/10/03
 *             2.1 2013/03/05 add exclusive feature.
 *                            this allows to use [] and | for exclusion.
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;

public class ParseOptions extends SerializableObject {

    private String[]  flag_options = new String[0];
    private String[]  value_options = new String[0];
    private String[]  long_flag_options = new String[0];
    private String[]  long_value_options = new String[0];

    private String[]  flag_options_args = new String[0];
    private String[]  value_options_args = new String[0];
    private String[]  value_options_values = new String[0];
    private String[]  long_flag_options_args = new String[0];
    private String[]  long_value_options_args = new String[0];
    private String[]  long_value_options_values = new String[0];

    private String[]  real_args = new String[0];
    private int real_args_index = 0;

    private static char ARGUMENT_MARK = ':';
    private static char OPTION_KEY = '-';
    private static String SPACE_KEY = " ";
    private static String NULL_KEY_STRING = "null";
    private static String OPTION_KEY_STRING = String.valueOf(OPTION_KEY);

    private static String TRACE_PARSEOPTIONSBODY = "ParseOptionsBody";
    private static String TRACE_PARSEOPTIONSPROCEDURE = "ParseOptionsProcedure";
    private static String TRACE_CHECKOPTIONS = "checkOptions";

    public static String ILLEGAL_ACCESS_EXCEPTION = "Illegal access exception";
    public static String ILLEGAL_FORMAT_EXCEPTION = "Illegal format exception";
    public static String ILLEGAL_OPTION_EXCEPTION = "Illegal option exception";
    public static String INDEX_OUTOFRANGE_EXCEPTION = "Index is out of range";

    private int DEBUG_OFF = 0;
    private int DEBUG_TRACE = 2;
    private int debug = DEBUG_OFF;

    private static String DEBUG_ARGUMENT = "Argument";
    
    private ParseOptions() {
    }

    public ParseOptions(String[] args, String options) throws Exception {
        String[]  long_arg = new String[0];
        String[]  long_opt = new String[0];

        printDebugTraceMessage( getMessageResource().getFormattedMessage(
            ObjectServerDebug.DEBUG_CALL_KEY,
            getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TRACE_PARSEOPTIONSBODY,
            getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
        try {
            ParseOptionsBody(args, options, long_arg, long_opt);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public ParseOptions(
        String[] args, String options, 
        String[] long_arg, String[] long_opt) throws Exception {

        printDebugTraceMessage( getMessageResource().getFormattedMessage(
            ObjectServerDebug.DEBUG_CALL_KEY,
            getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TRACE_PARSEOPTIONSBODY,
             getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
        try {
            ParseOptionsBody(args, options, long_arg, long_opt);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public static ParseOptions valueOf(String[] args, String options,
        String[] param_long_arg, String[] param_long_opt, 
        MessageResource    parameter_message_resource,
        int                parameter_debug,
        LogPrintStream     parameter_log_print_stream ) throws Exception {
        ParseOptions       parse_options_object;
        String[]  long_arg = new String[0];
        String[]  long_opt = new String[0];

        parse_options_object = new ParseOptions();
        parse_options_object.setMessageResource(parameter_message_resource);
        parse_options_object.setDebugLogStream(
            parameter_debug, parameter_log_print_stream );

        if (param_long_arg != null) {
            long_arg = param_long_arg;
        }
        if (param_long_opt != null) {
            long_opt = param_long_opt;
        }
        parse_options_object.printDebugTraceMessage( 
            parameter_message_resource.getFormattedMessage(
            ObjectServerDebug.DEBUG_CALL_KEY,
            parameter_message_resource.getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TRACE_PARSEOPTIONSBODY,
            parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
        try {
           parse_options_object.ParseOptionsBody(args, options, long_arg, long_opt);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return parse_options_object;
    }

    private ArrayList checkExclusiveOptions(String param_options) 
        throws Exception {
        int        size;
        int        start_position;
        String[]   option_array;
        String[]   child_array;
        ArrayList  child_list;
        Integer    integer_length;
        int        length;
        String     message;
        ObjectServerException  object_server_exception;

        option_array = new String[0];
        child_array = null;
        start_position = 0;
        for (int i = 0; i < param_options.length(); i++) {
            if (param_options.charAt(i) == '[') {
                if (param_options.length() == i+1) {
                    object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.PARAMETER_ERROR_PARSEOPTIONS,
                        getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_PARSEOPTIONS)));

                    message = getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage( 
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                        ObjectServerHeader.BLANK_STRING,
                        object_server_exception.getMessage() );                        
                    printDebugTraceMessage( message );

                    throw object_server_exception;
                }
                child_list = checkExclusiveOptions( 
                    param_options.substring(i+1) );
                if (child_list == null) {
                    AssertionError   assertion_error;
                    assertion_error = new AssertionError(child_list==null);
                    throw assertion_error;
                }
                child_array = (String[])child_list.get(0);
                integer_length = (Integer)child_list.get(1);
                length = integer_length.intValue();
                i = i + length;

                if (start_position == option_array.length) {
                    String[]  bk_option_array;
                    bk_option_array = new String[option_array.length+
                        child_array.length];
                    for (int j = 0; j < option_array.length; j++) {
                        bk_option_array[j] = option_array[j]; 
                    }
                    for (int j = 0; j < child_array.length; j++) {
                        bk_option_array[option_array.length+j] 
                            = child_array[j]; 
                    }
                    option_array = bk_option_array;
                    continue;
                } else {
                    size = option_array.length-start_position ;
                    String[]  bk_option_array;
                    bk_option_array = new String[
                            start_position + size * child_array.length
                    ];
                    for (int j = 0; j < start_position; j++) {
                        bk_option_array[j] = option_array[j]; 
                    }
                    for (int j = 0; j < child_array.length; j++) {
                        for (int k = 0; k < size; k++) {
                            bk_option_array[ option_array.length
                                + j * size + k] 
                                = option_array[start_position+k]
                                + child_array[j];
                        }
                    }
                    option_array = bk_option_array;
                }
                continue;
            } else if (param_options.charAt(i) == ']') {
                length = i + 1;
                integer_length = Integer.valueOf(length);
                child_list = new ArrayList(2);
                child_list.add( (Object)option_array );
                child_list.add( integer_length );
                return child_list;
            } else if (param_options.charAt(i) == '|') {
                start_position = option_array.length;
            } else {
                if (start_position == option_array.length) {
                    String[]  bk_option_array;
                    bk_option_array = new String[option_array.length+1];
                    for (int j = 0; j < option_array.length; j++) {
                        bk_option_array[j] = option_array[j]; 
                    }
                    option_array = bk_option_array;
                    option_array[option_array.length-1]
                            = String.valueOf(param_options.charAt(i));
                    continue;
                }
                for (int j = start_position; j < option_array.length; j++) {
                    if (option_array[j] != null) {
                        option_array[j] = option_array[j].concat(
                            String.valueOf(param_options.charAt(i)) );
                        continue;
                    }
                    AssertionError   assertion_error;
                    assertion_error = new AssertionError(option_array[j]==null);
                    throw assertion_error;
                }
            }
        }
        integer_length = Integer.valueOf(param_options.length());
        child_list = new ArrayList(2);
        child_list.add( (Object)option_array );
        child_list.add( integer_length );
        return child_list;
    }

    private String stripExclusiveOptions(String param_options) {
        String strip_options;

        strip_options = new String();
        for (int i = 0; i < param_options.length(); i++) {
            if (param_options.charAt(i) == '|'
                || param_options.charAt(i) == '['
                || param_options.charAt(i) == ']') {
                continue;
            }
            strip_options.concat(String.valueOf( strip_options.charAt(i) ));
        }
        return strip_options;
    }


    /**
     * This method is a body of constructor
     */
    private void ParseOptionsBody(
        String[] args, String param_options, 
        String[] param_long_arg, String[] param_long_opt) throws Exception {

        Exception          error;
        Integer            integer_length;
        ArrayList          array_list;
        String[]           option_lists;
        ObjectServerException  object_server_exception;
        String[]           long_arg;
        String[]           long_opt;
        String             options_string;

        options_string = param_options;
        if (options_string == null) {
            options_string = new String();
        }
        long_arg = param_long_arg;
        if (long_arg == null) {
            long_arg = new String[0];
        }
        long_opt = param_long_opt;
        if (long_opt == null) {
            long_opt = new String[0];
        }

        printDebugTraceMessage( getMessageResource().getFormattedMessage(
            ObjectServerDebug.DEBUG_EXECUTE_KEY,
            getMessageResource().getMessage( 
                ObjectServerDebug.DEBUG_EXECUTE_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TRACE_PARSEOPTIONSBODY,
            getMessageResource().getMessage(
                ObjectServerDebug.DEBUG_CODE_001)));

        if (args == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_PARSEOPTIONS,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_PARSEOPTIONS)));
            throw object_server_exception;
        }

        try {
            array_list = checkExclusiveOptions(options_string);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        option_lists = (String[])array_list.get(0);
        integer_length = (Integer)array_list.get(1);

        error = null;

        for (int i = 0; i < option_lists.length; i++) {
            try {
                ParseOptionsProcedure(
                    args, option_lists[i], long_arg, long_opt);
            } catch (Exception exception_object) {
                if (error == null) {
                    error = exception_object;
                }
                continue;
            }
            error = null;
            break;            
        }
        if (option_lists == null || option_lists.length == 0) {
            try {
                ParseOptionsProcedure( args, null, long_arg, long_opt);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        if (error != null) {
            throw error;
        }
    }

    private void ParseOptionsProcedure(
        String[] args, String options, 
        String[] long_arg, String[] long_opt) throws Exception {

        int    i;
        int    j;
        int    k;
        String[]  bk;
        String    key_string;
        String                  message;
        boolean                 option_flag = true;
        ObjectServerException   object_server_exception;
        String[]  argument_array;

        String   option_key;

        printDebugTraceMessage( getMessageResource().getFormattedMessage(
            ObjectServerDebug.DEBUG_EXECUTE_KEY,
            getMessageResource().getMessage( 
                ObjectServerDebug.DEBUG_EXECUTE_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TRACE_PARSEOPTIONSPROCEDURE,
            getMessageResource().getMessage(
                ObjectServerDebug.DEBUG_CODE_001)));

        argument_array = args;
        long_flag_options = long_arg;
        if (long_arg == null) {
            long_flag_options = new String[0];
        }
        long_value_options = long_opt;
        if (long_value_options == null) {
            long_value_options = new String[0];
        }

        if (options != null) {
            option_key = null;
            for (i = 0; i < options.length(); i++) {
                if (options.charAt(i) == ARGUMENT_MARK) {
                    if (option_key == null) {
                        object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS,
                            getMessageResource().getMessage(
                                ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS)));
                        message = getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage( 
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerHeader.BLANK_STRING,
                            object_server_exception.getMessage() );                        
                        printDebugTraceMessage( message );

                        throw object_server_exception;
                    }
                    bk = new String[value_options.length+1];
                    for (j = 0; j < value_options.length; j++) {
                        bk[j] = value_options[j];
                    }
                    value_options = bk;
                    value_options[value_options.length-1] = option_key;
                    option_key = null;
                    continue;
                } else {
                    if (option_key != null) {
                        bk = new String[flag_options.length+1];
                        for (j = 0; j < flag_options.length; j++) {
                            bk[j] = flag_options[j];
                        }
                        flag_options = bk;
                        flag_options[flag_options.length-1] = option_key;
                    }
                    option_key = OPTION_KEY_STRING + 
                            String.valueOf(options.charAt(i));
                }
            }
            if (option_key != null) {
                bk = new String[flag_options.length+1];
                for (j = 0; j < flag_options.length; j++) {
                    bk[j] = flag_options[j];
                }
                flag_options = bk;
                flag_options[flag_options.length-1] = option_key;
            }
        }

        for (i = 0; i < argument_array.length; i++) {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_ARGUMENT_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_ARGUMENT_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(i) + 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY + 
                    argument_array[i]));
            if (argument_array[i].length() > 1 &&
                argument_array[i].charAt(0) == OPTION_KEY) {
                key_string = argument_array[i];
                for (j = 0; j < long_flag_options.length; j++) {
                    if (long_flag_options[j].compareTo(key_string) == 0) {
                        bk = new String[long_flag_options_args.length+1];
                        for (k = 0;k < long_flag_options_args.length;k++) {
                            bk[k] = long_flag_options_args[k];
                        }
                        long_flag_options_args = bk;
                        long_flag_options_args[long_flag_options_args.length-1]
                            = key_string;
                        break;
                    }
                }
                if (j < long_flag_options.length) {
                    continue;
                }
                for (j = 0; j < long_value_options.length; j++) {
                    if (long_value_options[j].compareTo(key_string) == 0) {
                        bk = new String[long_value_options_args.length+1];
                        for (k = 0;k < long_value_options_args.length;k++) {
                            bk[k] = long_value_options_args[k];
                        }
                        long_value_options_args = bk;
                        bk = new String[long_value_options_values.length+1];
                        for (k = 0;
                            k < long_value_options_values.length;k++) {
                            bk[k] = long_value_options_values[k];
                        }
                        long_value_options_values = bk;
                        long_value_options_args[
                            long_value_options_args.length-1] = key_string;
                        i = i + 1;
                        if (i >= argument_array.length) {
                            object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS,
                                getMessageResource().getMessage(
                                    ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS)));
                            message = getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                                getMessageResource().getMessage( 
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                                ObjectServerHeader.BLANK_STRING,
                                object_server_exception.getMessage() );                        
                            printDebugTraceMessage( message );

                            throw object_server_exception;
                        }
                        long_value_options_values[
                            long_value_options_values.length-1] 
                            = argument_array[i];
                        break;
                    }
                }
                if (j < long_flag_options.length) {
                    continue;
                }
            }
            if (argument_array[i].length() > 0 &&
                argument_array[i].charAt(0) == OPTION_KEY) {
                if (option_flag == false) {
                    object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS,
                            getMessageResource().getMessage(
                            ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS)));
                    message = getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage( 
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                        ObjectServerHeader.BLANK_STRING,
                        object_server_exception.getMessage() );                        
                    printDebugTraceMessage( message );
                    throw object_server_exception;
                }
                key_string = argument_array[i];

                for (j = 0; j < long_flag_options.length; j++) {
                    if (long_flag_options[j].compareTo(key_string) == 0) {
                        bk = new String[long_flag_options_args.length+1];
                        for (k = 0;k < long_flag_options_args.length;k++) {
                            bk[k] = long_flag_options_args[k];
                        }
                        long_flag_options_args = bk;
                        long_flag_options_args[long_flag_options_args.length-1]
                            = key_string;
                        break;
                    }
                }
                if (j < long_flag_options.length) {
                    continue;
                }
                for (j = 0; j < long_value_options.length; j++) {
                    if (long_value_options[j].compareTo(key_string) == 0) {
                        bk = new String[long_value_options_args.length+1];
                        for (k = 0;k < long_value_options_args.length;k++) {
                            bk[k] = long_value_options_args[k];
                        }
                        long_value_options_args = bk;

                        bk = new String[long_value_options_values.length+1];
                        for (k = 0;k < long_value_options_values.length;k++) {
                            bk[k] = long_value_options_values[k];
                        }
                        long_value_options_values = bk;

                        long_value_options_args[long_value_options_args.length-1]
                            = key_string;

                        i = i + 1;
                        long_value_options_values[
                            long_value_options_values.length-1] 
                            = argument_array[i];
                        break;
                    }
                }
                if (j < long_value_options.length) {
                    continue;
                }
                if (key_string.length() != 2) {
                    bk = new String[argument_array.length+key_string.length()-1];
                    for (j = 0; j <= i; j++) {
                        bk[j] = argument_array[j];
                    } 
                    for ( ; j < i + 1 + key_string.length() - 1; j++) {
                        bk[j] = OPTION_KEY_STRING +
                            String.valueOf(key_string.charAt(j-i));
                    }
                    for (;j<key_string.length()-1+argument_array.length;j++) {
                        bk[j] = argument_array[j-(key_string.length()-1)];
                    }
                    argument_array = bk;
                    continue;
                }

                for (j = 0; j < flag_options.length; j++) {
                    if (flag_options[j].compareTo(key_string) == 0) {
                        bk = new String[flag_options_args.length+1];
                        for (k = 0;k < flag_options_args.length;k++) {
                            bk[k] = flag_options_args[k];
                        }
                        flag_options_args = bk;
                        flag_options_args[flag_options_args.length-1]
                            = key_string;
                        break;
                    }
                }
                if (j < flag_options.length) {
                    continue;
                }

                for (j = 0; j < value_options.length; j++) {
                    if (value_options[j].compareTo(key_string) == 0) {
                        bk = new String[value_options_args.length+1];
                        for (k = 0;k < value_options_args.length;k++) {
                            bk[k] = value_options_args[k];
                        }
                        value_options_args = bk;
                        bk = new String[value_options_values.length+1];
                        for (k = 0;k < value_options_values.length;k++) {
                            bk[k] = value_options_values[k];
                        }
                        value_options_values = bk;
                        value_options_args[value_options_args.length-1]
                            = key_string;
                        i = i + 1;
                        key_string = argument_array[i];
                        value_options_values[value_options_values.length-1]
                            = key_string;
                        break;
                    }
                }
                if (j < value_options.length) {
                    continue;
                }
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS,
                        getMessageResource().getMessage(
                            ObjectServerError.ILLEGAL_FORMAT_PARSEOPTIONS)));
                message = getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                    ObjectServerHeader.BLANK_STRING,
                    object_server_exception.getMessage() );                        
                printDebugTraceMessage( message );
                throw object_server_exception;
            }

            if (option_flag == true) {
                option_flag = false;
            }
            bk = new String[real_args.length+1];
            for (j = 0; j < real_args.length; j++) {
                bk[j] = real_args[j];
            }
            real_args = bk;
            real_args[real_args.length-1] = argument_array[i];
        }
    }

    public boolean checkOptions(String options) {
        int    i;
        int    j;
        String key;
        String key_string;

        printDebugTraceMessage( getMessageResource().getFormattedMessage(
            ObjectServerDebug.DEBUG_EXECUTE_KEY,
            getMessageResource().getMessage( 
                ObjectServerDebug.DEBUG_EXECUTE_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TRACE_CHECKOPTIONS,
            getMessageResource().getMessage(
                ObjectServerDebug.DEBUG_CODE_001)));
        if (options == null) {
            return false;
        }
        key = options;
        for (i = 0; i < key.length(); i++) {
            if (key.charAt(i) == ARGUMENT_MARK) {
                return false;
            }
            key_string = String.valueOf(OPTION_KEY) + 
                         String.valueOf(key.charAt(i));
            for (j = 0; j < flag_options.length; j++) {
                if (flag_options[j].compareTo(key_string) == 0) {
                    break;
                }
            }
            if (j < flag_options.length) {
                continue;
            }
            for (j = 0; j < value_options.length; j++) {
                if (value_options[j].compareTo(key_string) == 0) {
                    break;
                }
            }
            if (j >= value_options.length) {
                return false;
            }
            i = i + 1;
            if (i >=  key.length()) {
                return false;
            }
            if (key.charAt(i) != ARGUMENT_MARK) {
                return false;
            }
        }
        return false;
    }

    public boolean checkOption(String key) {
        int i;
        int j;
        String[][] target = new String[4][];
        target[0] = flag_options_args;
        target[1] = value_options_args;
        target[2] = long_flag_options_args;
        target[3] = long_value_options_args;
        
        if (key == null) {
            return false;
        }
        for (i = 0; i < target.length; i++) {
            for (j = 0; j < target[i].length; j++) {
                if (target[i][j].compareTo(key) == 0) {
                    break;
                }
            }
            if (j < target[i].length) {
                return true;
            }
        }
        return false;
    }

    public String getOption(String key) {
        int      i;
        String   return_value;

        if (key == null) {
            return null;
        }
        return_value = null;

        if (key.length() == 2) {
            for (i = 0; i < value_options_args.length; i++) {
                if (value_options_args[i].compareTo(key) == 0) {
                    return value_options_values[i];
                }
            }
        } else {
            for (i = 0; i < long_value_options_args.length; i++) {
                if (long_value_options_args[i].compareTo(key) == 0) {
                    return long_value_options_values[i];
                }
            }
        }
        return return_value;
    }

    /**
     * This method returns the number of the Argument
     */
    public int getRealArgumentCount() {
        return real_args.length;
    }

    public String getRealArgument(int index) throws Exception {
        Exception  exception_object;
        String     message;

        if (real_args.length <= index || index < 0) {
            exception_object = new Exception(
                getMessageResource().getFormattedMessage(
                ObjectServerError.INDEX_OUTOFRANGE_PARSEOPTIONS,
                getMessageResource().getMessage(
                ObjectServerError.INDEX_OUTOFRANGE_PARSEOPTIONS)));
            message = getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                ObjectServerHeader.BLANK_STRING,
                exception_object.getMessage() );                        
            printDebugTraceMessage(message);

            throw exception_object;
        }
        return real_args[index];
    }

    public void resetRealArgument() {
        real_args_index = 0;
    }

    public String nextRealArgument() {
        String  real_args_string;

        if (real_args_index >= real_args.length) {
            return null;
        }
        real_args_string = real_args[real_args_index];
        real_args_index = real_args_index + 1;
        return real_args_string;
    }

    private static String ARG1 = "a";
    private static String ARG2 = "b";
    private static String ARG3 = "c";
    private static String ARGS = ARG1 + String.valueOf(ARGUMENT_MARK) 
        + ARG2 + ARG3;
    private static String SECOND_OPTION_VALID   = "Second option ";
    private static String SECOND_OPTION_INVALID = "Second invalid option ";
    private static String ARGUMENT_TITLE        = "Argument";
    private static String WITH_KEY  = "with";
    private static String CHECK_KEY = "-check";
    private static String HELP_KEY  = "-help";
    private static String VALUE_KEY = "Value";
    private static String SPACE = " ";
    private static String SPECIFY_HELP_OPTION = "-help was specified.";
    private static String SPECIFY_CHECK_OPTION = "-check was specified with";

    public static void main(String[] args) {
        ParseOptions   parse_options;
        boolean        arg2_flag;
        String         arg1_value;
        String         option_head;
        String         option_string;
        String[]       long_flag = { HELP_KEY };
        String[]       long_key = { CHECK_KEY };

        try {
            parse_options = new ParseOptions(args, ARGS, long_flag, long_key);
        } catch (Exception exception_object) {
            System.err.println(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.ILLEGAL_OPTION_PARSEOPTIONS,
                    getMessageResource().getMessage(
                        ObjectServerError.ILLEGAL_OPTION_PARSEOPTIONS)));
            return;
        }

        arg1_value = parse_options.getOption(OPTION_KEY_STRING + ARG1);
        arg2_flag = parse_options.checkOption(OPTION_KEY_STRING + ARG2);        

        if (arg1_value != null) {
            option_string = SPACE_KEY + WITH_KEY + SPACE_KEY + arg1_value;
        } else {
            option_string = NULL_KEY_STRING;
        }
        if (arg2_flag == true) {
            option_head = SECOND_OPTION_VALID;
        } else {
            option_head = SECOND_OPTION_INVALID;
        }
        System.out.println(option_head + option_string );
        if (parse_options.checkOption(HELP_KEY) == true) {
            System.out.println(SPECIFY_HELP_OPTION);
        }
        if (parse_options.checkOption(CHECK_KEY) == true) {
            System.out.println(SPECIFY_CHECK_OPTION + SPACE
                + parse_options.getOption(CHECK_KEY));
        }

        for (int i = 0; i < parse_options.getRealArgumentCount(); i++) {
            try {
                System.out.println(ARGUMENT_TITLE
                    + SPACE_KEY
                    + parse_options.getRealArgument(i) 
                );
            } catch (Exception exception_object) {
                System.out.println(exception_object.toString());
            }
        }
    }
}
