/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     ReadVariable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/05/28
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

public class ReadVariable extends BaseVariable {
    public static String PARAMETER_ERROR_EXCEPTION
        = "Parameter error exception.";

    public static int BUFFER_SIZE = 8192;
    public static String CLASS_SUFFIX = ".class";
    public static String DOT_STRING = ".";
    public final static String TEST_CLASS_NAME = "Test";

    public final static String START_BRACKET = "[";
    public final static String END_BRACKET = "]";
    public final static String JAVA_IO_SERIALIZABLE_TYPE_KEY = "java.io.Serializable";
    public final static String SEMICOLON = ";";
    public final static char ARRAY_INT = 'I';
    public final static char ARRAY_BYTE = 'B';
    public final static char ARRAY_LONG = 'J';
    public final static char ARRAY_SHORT = 'S';
    public final static char ARRAY_CHAR = 'C';
    public final static char ARRAY_DOUBLE = 'D';
    public final static char ARRAY_FLOAT = 'F';
    public final static char ARRAY_BOOLEAN = 'Z';
    public final static char ARRAY_REFERENCE = 'L';
    /**
     * @param    Filename
     * @return   void
     */
    public void readVariable(String filename, Class cls, Object obj) 
        throws Exception {
        FileInputStream   file_input_stream;
        ObjectInputStream object_input_stream;
        Field[]  fld;
        int mds;
        File    fdes;
        int      iop;
        long     lop;
        short    sop;
        float    fop;
        double   dop;
        Object    op;
        boolean  bop;
        byte    byop;
        char     cop;
        String fldnm;

        if (filename == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (cls == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        fdes = new File(filename);
        if (fdes.exists() == false) {
            return;
        }

        try {
            file_input_stream = new FileInputStream(filename);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            object_input_stream = new ObjectInputStream((InputStream)file_input_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        fld = cls.getFields();

        for (int j = 0; j < fld.length; j++) {
            mds = fld[j].getModifiers();
            if (Modifier.isFinal(mds) == true ) { continue; }
            /*
            if (Modifier.isPublic(mds) == false ) { continue; }
            */
            /**
            if (Modifier.isStatic(mds) == false ) { continue; }
            **/

            if (fld[j].getType().getName().compareTo(int.class.getName()) == 0) {
                try {
                    iop = (int)object_input_stream.readInt();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setInt(obj, iop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(long.class.getName()) == 0) {
                try {
                    lop = (long)object_input_stream.readLong();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setLong(obj, lop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(float.class.getName()) == 0) {
                try {
                    fop = (float)object_input_stream.readFloat();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setFloat(obj, fop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(double.class.getName()) == 0) {
                try {
                    dop = (double)object_input_stream.readDouble();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setDouble(obj, dop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(boolean.class.getName()) == 0) {
                try {
                    bop = (boolean)object_input_stream.readBoolean();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setBoolean(obj, bop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(byte.class.getName()) == 0) {
                try {
                    byop = (byte)object_input_stream.readByte();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setByte(obj, byop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(char.class.getName()) == 0) {
                try {
                    cop = (char)object_input_stream.readChar();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setChar(obj, cop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[j].getType().getName().compareTo(short.class.getName()) == 0) {
                try {
                    sop = (short)object_input_stream.readShort();
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                try {
                    fld[j].setShort(obj, sop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (isSerializable(fld[j]) == false) {
                continue;
            }
            try {
                op = object_input_stream.readObject();
            } catch (Exception exception_object) {
                throw exception_object;
            }
            try {
                fld[j].set(obj, op);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        try {
            object_input_stream.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }
}


