/**
 * Copyright (C) All rights reserved.
 *
 * @Module     RegisterCommand.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/02/28
 *
 */

package oserv;

import java.lang.*;
import java.io.*;

public class RegisterCommand extends SerializableObject {
    private String command = new String();
    private int    file_size = 0;
    private String class_name = new String();
    private String file_type = new String();
    private String execution_type = new String();
    private String target_name = new String();
    public final static String PERSISTENT_TYPE = "Persistent";
    public final static String TRANSIENT_TYPE = "Transient";

    private static final String  REGISTER_COMMAND = "REGISTER";
    private static final String  CLASS_COMMAND    = "CLASS";
    private static final String  DELETE_COMMAND   = "DELETE";
    private static final String  LIST_COMMAND     = "LIST";

    private static String register_command_string = new String("oserv.NetworkObject");
    private static String register_command_type = new String("class");
    private static String register_command_jar_type = new String("jar");

    private static final String ILLEGAL_COMMAND_TYPE = "Illegal command type";

    private    static String      JPN_LANGUAGE = "jpn";

    private final static String DEBUG_REGISTER_COMMAND = "registerCommand";
    private final static String DEBUG_DELETE_COMMAND = "deleteCommand";
    private final static String DEBUG_LIST_COMMAND = "listCommand";
    private final static String DEBUG_CLASS_COMMAND = "classCommand";

    private final static String DEBUG_FILE_ERROR = "Archive file";

    private RegisterCommand (
        String command_string,
        String class_name_string,
        String file_type_string,
        String target_name_string,
        int    size )  throws Exception {
    
        command    = command_string;
        class_name = class_name_string;
        file_type  = file_type_string;
        file_size  = size;
        target_name = target_name_string;
    }

    public static RegisterCommand registerCommand(
        String path, String type, int size,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) throws Exception {
        RegisterCommand   register_command_object;

        if (parameter_debug > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE 
                && parameter_log_stream != null 
                && parameter_message_resource != null ) {
            parameter_log_stream.printLogln(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    parameter_message_resource.getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_REGISTER_COMMAND,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        if (type == null || 
            (type.compareTo(ObjectServerHeader.JAR_SUFFIX) != 0 &&
            type.compareTo(ObjectServerHeader.CLASS_SUFFIX) != 0)) {
            Exception   exception_object;
            exception_object = new Exception(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION);
            throw exception_object;
        }
   
        try {
            register_command_object = new RegisterCommand(
                REGISTER_COMMAND,
                path,
                type,
                new String(),
                size);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        register_command_object.setMessageResource(parameter_message_resource);
        register_command_object.setDebug(parameter_debug);
        register_command_object.setLogStream(parameter_log_stream); 
        return  register_command_object;
    }

    public static RegisterCommand deleteCommand(String  path,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) throws Exception {
        RegisterCommand     delete_command;

        if (parameter_debug > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE 
                && parameter_log_stream != null 
                && parameter_message_resource != null ) {
            parameter_log_stream.printLogln(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    parameter_message_resource.getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_DELETE_COMMAND,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        try {
            delete_command = new RegisterCommand(
                DELETE_COMMAND,
                path,
                new String(),
                new String(),
                0);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        delete_command.setMessageResource(parameter_message_resource);
        delete_command.setDebug(parameter_debug);
        delete_command.setLogStream(parameter_log_stream); 
        return delete_command; 
    }

    public static RegisterCommand listCommand(
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) throws Exception {
        RegisterCommand       list_command;

        if (parameter_debug > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE 
                && parameter_log_stream != null 
                && parameter_message_resource != null ) {
            parameter_log_stream.printLogln(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    parameter_message_resource.getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_LIST_COMMAND,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        try {
            list_command = new RegisterCommand(LIST_COMMAND, 
                new String(),
                new String(),
                new String(),
                0);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        list_command.setMessageResource(parameter_message_resource);
        list_command.setDebug(parameter_debug);
        list_command.setLogStream(parameter_log_stream); 
        return list_command;
    }

    public static RegisterCommand classCommand(
        String  class_name, String  target_name_string,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) throws Exception {
        RegisterCommand       class_command;

        if (parameter_debug > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE 
                && parameter_log_stream != null 
                && parameter_message_resource != null ) {
            parameter_log_stream.printLogln(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    parameter_message_resource.getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_CLASS_COMMAND,
                    parameter_message_resource.getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }
        try {
            class_command = new RegisterCommand(CLASS_COMMAND, 
                class_name,
                new String(),
                target_name_string,
                0);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        class_command.setMessageResource(parameter_message_resource);
        class_command.setDebug(parameter_debug);
        class_command.setLogStream(parameter_log_stream); 
        return class_command;
    }

    public boolean isRegister() {
        if (REGISTER_COMMAND.compareTo(command) == 0) {
            if (isClass() == true) {
                return false;
            }
            return true;
        }
        return false;
    }

    public boolean isDelete() {
        if (DELETE_COMMAND.compareTo(command) == 0) {
            return true;
        }
        return false;
    }

    public boolean isList() {
        if (LIST_COMMAND.compareTo(command) == 0) {
            return true;
        }
        return false;
    }

    public boolean isClass() {
        if (CLASS_COMMAND.compareTo(command) == 0) {
            return true;
        }
        return false;
    }

    public String getClassName() throws Exception {
        if (isRegister() == false && isDelete() == false && isClass() == false) {
            String      message;
            Exception   exception_object;

            message = getMessageResource().getFormattedMessage(
                ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND,
                getMessageResource().getMessage( 
                    ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND));

            exception_object = new Exception( message );
            throw exception_object;
        }
        return class_name;
    }

    public String getType() throws Exception {
        if (isRegister() == false) {
            String      message;
            Exception   exception_object;

            message = getMessageResource().getFormattedMessage(
                ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND,
                getMessageResource().getMessage( 
                    ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND));
            exception_object = new Exception( message );
            throw exception_object;
        }
        return file_type;
    }

    public String getTargetName() throws Exception {
        if (isClass() == false) {
            String      message;
            Exception   exception_object;

            message = getMessageResource().getFormattedMessage(
                ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND,
                getMessageResource().getMessage( 
                    ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND));
            exception_object = new Exception( message );
            throw exception_object;
        }
        return target_name;
    }

    public int getFileSize() throws Exception {
        if (isRegister() == false) {
            String      message;
            Exception   exception_object;

            message = getMessageResource().getFormattedMessage(
                ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND,
                getMessageResource().getMessage( 
                    ObjectServerError.ILLEGAL_COMMAND_TYPE_REGISTERCOMMAND));
            exception_object = new Exception( message );
            throw exception_object;
        }
        return file_size;
    }

    public void setExecuteType (String type_string) {
        if (type_string == null) { return; }

        if (type_string.compareTo(PERSISTENT_TYPE) == 0) {
            execution_type = PERSISTENT_TYPE;
            return;
        }
        if (type_string.compareTo(TRANSIENT_TYPE) == 0) {
            execution_type = TRANSIENT_TYPE;
            return;
        }
    }

    public String getExecuteType () {
        return execution_type;
    }

    public static void main(String[] args) {
        RegisterCommand   register_command_object;
        File              file_object;
        int               file_size_int;
        int               index;
        int               debug;
        String            message;
        String            register_archive;
        LogPrintStream    log_print_stream;
        MessageResource   message_resource;

        debug = 9;
        log_print_stream = new LogPrintStream(System.out);
        message_resource = new MessageResource();
        message_resource.setLanguage(JPN_LANGUAGE);

        if (args.length == 1) {
            if (args[0].compareTo(REGISTER_COMMAND) == 0) {
                try {
                    register_archive = register_command_string + 
                        ObjectServerHeader.SUFFIX_SEPARATOR+
                        register_command_type;
                    file_object = new File(register_archive);
                    if (file_object.exists() == false) {
                        index = register_command_string.lastIndexOf(
                            ObjectServerHeader.SUFFIX_SEPARATOR);
                        register_archive = register_command_string;
                        if (index >= 0) {
                            register_archive = register_command_string.substring(index+1);
                        }
                        register_archive = register_archive + 
                            ObjectServerHeader.SUFFIX_SEPARATOR+
                            register_command_type;
                        file_object = new File(register_archive);
                    }
                    if (file_object.exists() == false) {
                        String user_home;
                        user_home = System.getProperty(
                            ObjectServerHeader.USER_HOME_KEY) ;
                        register_archive = 
                            user_home + File.separator + register_archive;
                        file_object = new File(register_archive);
                    }
                    if (file_object.exists() == false) {
                        message = message_resource.getFormattedMessage(
    ObjectServerDebug.DEBUG_FILE_NOT_EXIST,
                                message_resource.getMessage(
    ObjectServerDebug.DEBUG_FILE_NOT_EXIST
                                )
                            );
                        System.err.println(message);
                        return;
                    }
                    file_size_int = (int)file_object.length();

                    register_command_object = RegisterCommand.registerCommand(
                        register_command_string,
                        register_command_type,
                        file_size_int,
                        message_resource,
                        debug,
                        log_print_stream );
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                }
                return;
            }
            if (args[0].compareTo(DELETE_COMMAND) == 0) {
                try {
                    register_command_object = RegisterCommand.deleteCommand(
                    register_command_string,
                    message_resource,
                    debug,
                    log_print_stream );
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                }
                return;
            }
            if (args[0].compareTo(LIST_COMMAND) == 0) {
                try {
                    register_command_object = RegisterCommand.listCommand(
                        message_resource,
                        debug,
                        log_print_stream );
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                }
                return;
            }
            if (args[0].compareTo(CLASS_COMMAND) == 0) {
                try {
                    register_command_object = RegisterCommand.classCommand(
                        register_command_string,
                        register_command_string+
                        ObjectServerHeader.OBJECT_DOT_SEPARATOR+
                        register_command_jar_type,
                        message_resource,
                        debug,
                        log_print_stream );
                } catch (Exception exception_object) {
                    System.err.println(exception_object.getMessage());
                }
                return;
            }
        }
    }
}
