/**
 * Copyright (C) All rights are reserved. 
 *
 * @Module     RegistrationInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/11/11
 *
 * This class is derived from FileInputStream for extension of the character.
 */

package oserv;

import java.lang.*;
import java.net.*;
import java.io.*;
import java.lang.reflect.*;

/**
 * 
 * @see FileInputStream
 */
public class RegistrationInputStream extends FileInputStream {
    String file_name;
    File   file_data;
    String file_class = null;
    FileDescriptor file_descriptor;

    public final static long ERROR_LENGTH = -1;

    public RegistrationInputStream(String file) throws FileNotFoundException {
        super(file);

        file_class = null;
    }

    public RegistrationInputStream(String file, String class_name) 
        throws FileNotFoundException {
        super(file);
        
        file_name = file;
        file_class = class_name;
        if (file == null) {
            file_class = null;
            return;
        }
     
        file_data = new File(file);
    }

    public RegistrationInputStream(File file) throws FileNotFoundException {
        super(file);
        file_class = null;
    }

    public RegistrationInputStream(File file, String class_name) 
        throws FileNotFoundException {
        super(file);
        if (file == null) {
            file_name = null;
            file_class = null;
            return;
        }
        file_class = class_name;
        file_data  = file;
        file_name  = file.getPath();
    }

    public RegistrationInputStream(FileDescriptor fd) 
        throws FileNotFoundException {
        super(fd);
        file_class = null;
    }

    public long length() {
        long    length;

        if (file_class == null) {
            return ERROR_LENGTH;
        }
        length = file_data.length();
        return length;
    }

    public String getClassPath() {
        return file_class;
    }
}
