/**
 * Copyright (C) All rights reserved.
 *
 * @Module     RetryObjectInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/07/23
 *             2.1 2011/09/23 for forward
 *             2.1 2011/12/30 for native type
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

import oserv.*;

public class RetryObjectInputStream extends ObjectInputStream {

    private int retry_count = ObjectServerHeader.RETRY_NUMBER;

/**
    public static String PARAMETER_ERROR_EXCEPTION
        = "Parameter error exception.";
**/
    public static String READ_OBJECT = "ReadObject()";
    public static String RETRY_READ_OBJECT = "retryReadObject()";
    public static String RETRY_READ = "retryRead()";

    protected int debug = 0;
    protected LogPrintStream logStream = null;
    private MessageResource message_resource = new MessageResource();

    protected RetryObjectInputStream(InputStream input_stream) throws IOException {
        super(input_stream);
    }

    public void setDebugLogStream(int debug_level, LogPrintStream ps) {
         debug = debug_level;
         logStream = ps;
    }

    public void setRetryCount(int count) {
        retry_count = count;
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public Object retryReadObject( ) 
            throws ObjectServerException, Exception, IOException, ClassNotFoundException {
        Object obj;

        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        RETRY_READ_OBJECT,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }
     
        try {
            obj = readObject(null, 0, 0);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        return obj;
    }

    private ObjectServerException getAssertionException() {
        ObjectServerException  object_server_exception;
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.ASSERTION_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.ASSERTION_EXCEPTION))
        );
        return object_server_exception;
    }

    private ObjectServerException getTimeoutErrorException() {
        ObjectServerException  object_server_exception;
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.TIMEOUT_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.TIMEOUT_ERROR_EXCEPTION))
        );
        return object_server_exception;
    }

    private ObjectServerException getInternalErrorException() {
        ObjectServerException  object_server_exception;
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION))
        );
        return object_server_exception;
    }

    private Object readObject(byte[] buffer, int offset, int length) 
            throws ObjectServerException, Exception, IOException, ClassNotFoundException, EOFException {
        Object obj;
        int retry;
        int  rcnt;
        int rdcnt;
        Integer read_length;
        String key_message = null;
        NetworkObject network_object;
        ObjectServerException object_server_exception;

        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        READ_OBJECT,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }

        if (buffer != null && (offset < 0 || length < 0 ||
            buffer.length < offset + length)) {
            object_server_exception = getAssertionException();
            throw object_server_exception;
        }
        rdcnt = 0;
        for (retry = 0; retry < retry_count; retry++) {
            if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) { logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_RETRY_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_RETRY_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(retry)));
                }
            }
            try {
                if (buffer == null) {
                    obj = readObject();

                    if (obj != null) {
                        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                            if (logStream != null) {
                                logStream.printLogln(
                                    getMessageResource().getFormattedMessage(
                                        ObjectServerDebug.DEBUG_RETRY_KEY,
                                        getMessageResource().getMessage(
                                            ObjectServerDebug.DEBUG_RETRY_KEY),
                                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                        obj.toString()));
                                if ((obj instanceof NetworkObject) == true) {
                                    if (((NetworkObject)obj).getName().compareTo( ObjectServerHeader.VOID_TYPE_KEY) == 0) {
                                        key_message = ObjectServerHeader.VOID_TYPE_KEY;
                                    } else if (((NetworkObject)obj).getName().compareTo( int.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getInt() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( long.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getLong() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( float.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getFloat() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( double.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getDouble() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( byte.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getByte() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( boolean.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getBoolean() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( char.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getChar() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else if (((NetworkObject)obj).getName().compareTo( short.class.getName()) == 0) {
                                        try {
                                            key_message = String.valueOf( ((NetworkObject)obj).getShort() );
                                        } catch (Exception value_exception_object) {
                                            object_server_exception = getAssertionException();
                                            throw object_server_exception;
                                        }
                                    } else {
                                        key_message = obj.toString();
                                    }
                                } else {
                                    key_message = obj.toString();
                                }
                                logStream.printLogln(
                                    getMessageResource().getFormattedMessage(
                                        ObjectServerHeader.READ_PARAMETER,
                                        getMessageResource().getMessage(
                                            ObjectServerHeader.READ_PARAMETER),
                                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                        Object.class.getName(),
                                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    key_message));
                            }
                        }
                        if ((obj instanceof NetworkObject) == true) {
                            String type_string;
                            type_string = ((NetworkObject)obj).getName();

                            if (type_string.compareTo(int.class.getName()) != 0
                                && type_string.compareTo(long.class.getName()) != 0
                                && type_string.compareTo(float.class.getName()) != 0
                                && type_string.compareTo(double.class.getName()) != 0
                                && type_string.compareTo(short.class.getName()) != 0
                                && type_string.compareTo(boolean.class.getName()) != 0
                                && type_string.compareTo(byte.class.getName()) != 0
                                && type_string.compareTo(char.class.getName()) != 0 ) {

                                if ((( ((NetworkObject)obj).get() instanceof oserv.ObjectServerException) == true)||
                                   (( ((NetworkObject)obj).get() instanceof Exception) == true)) {

                                    Exception  exception_instance;
                                    exception_instance = (Exception)((NetworkObject)obj).get();


                                    if (( ((NetworkObject)obj).get() instanceof oserv.ObjectServerException) == true) {
                                        String message;
                                        message = ((Exception)
                                        ((NetworkObject)obj).get()).getMessage();
                                        if (message != null &&
                                            (ObjectServerException.NORMAL_END_MESSAGE.compareTo(message) == 0)) {
                                            return ((NetworkObject)obj).get();
                                        }
                                        if (message != null &&
                                            (ObjectServerException.NULL_RETURN.compareTo(message) == 0)) {
                                            return ((NetworkObject)obj).get();
                                        }
                                    }
                                    throw exception_instance;
                                }
                            }
                        }
                        if ((obj instanceof Exception) == true) {
                            String message;
                            message = ((Exception)obj).getMessage();
                            if (message != null &&
                                (ObjectServerException.NORMAL_END_MESSAGE.compareTo(message) == 0)) {
                                return obj;
                            }
                            if (message != null &&
                                (ObjectServerException.NULL_RETURN.compareTo(message) == 0)) {
                                return obj;
                            }
                            throw ((Exception)obj);
                        }
                    } else {
                        object_server_exception = getAssertionException();
                        throw object_server_exception;
                    }
                    return obj;
                } else {
                    readFully(buffer, 0, length);
                    read_length = new Integer(length);
                    return (Object)read_length;
                }
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    throw exception_object;
                }
                if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    if (logStream != null) {
                        logStream.printLogln(
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                exception_object.toString()));
                    }
                }
                obj = null;
                if (exception_object instanceof SocketTimeoutException) {
                    continue;
                }
                throw exception_object;
/*
                object_server_exception = getInternalErrorException();
                throw object_server_exception;
*/
            }
        }
        if (retry == retry_count) {
            object_server_exception = getTimeoutErrorException();
            throw object_server_exception;
        }
        if (rdcnt == 0) {
            rdcnt = -1;
        }

        read_length = new Integer(rdcnt);
        return (Object)read_length;
    }

    public int  retryRead(byte[] buffer, int offset, int length) 
            throws Exception, IOException {
        int     rcnt;
        Integer return_count;

        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        RETRY_READ,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }
        return_count = (Integer)readObject(buffer, offset, length);
        rcnt = return_count.intValue();
        return rcnt;
    }

    public static RetryObjectInputStream valueOf(
        InputStream       input_stream,
        MessageResource   parameter_message_resource,
        int               parameter_debug,
        LogPrintStream    parameter_log_stream) throws Exception {

        Exception                 save_exception;
        RetryObjectInputStream    retry_object_input_stream;
        ObjectServerException     object_server_exception;

        save_exception = null;
        retry_object_input_stream = null;
        try {
            retry_object_input_stream = new RetryObjectInputStream(input_stream);
        } catch (Exception exception_object) {
            if (parameter_debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (parameter_log_stream != null) {
                    parameter_log_stream.printLogln(
                        parameter_message_resource.getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            parameter_message_resource.getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString() ));
                }
            }
            object_server_exception = new ObjectServerException(
                parameter_message_resource.getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION,
                    parameter_message_resource.getMessage(
                       ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        retry_object_input_stream.setMessageResource(parameter_message_resource);
        retry_object_input_stream.setDebugLogStream(
            parameter_debug,  parameter_log_stream);

        return retry_object_input_stream;
    }

/**
    public static RetryObjectInputStream valueOf(InputStream  input_stream) throws Exception {
        RetryObjectInputStream    retry_object_input_stream;
        ObjectServerException     object_server_exception;

        try {
            retry_object_input_stream = new RetryObjectInputStream(input_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return retry_object_input_stream;
    }
**/
}
