/**
 * Copyright (C) All rights reserved.
 *
 * @Module     SerializableObject.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/01/24
 *
 * This object is used to wrap normal object without serializable interface.
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.lang.reflect.*;


public class SerializableObject extends Object implements Serializable {
    private static int                    debug;
    private static LogPrintStream         logStream = null;

    public static Object rt;

    private static String CLONE_OBJECT = "cloneObject";

    private static MessageResource message_resource = new MessageResource();

    public static void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public static MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setDebugLogStream(int debug_level, LogPrintStream log_print_stream) {
        if (debug_level != 0) {
            debug = debug_level;
        }
        if (log_print_stream != null) {
            logStream = log_print_stream;
        }
        if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && logStream != null) {
            logStream.printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_DEBUG_VALUE,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_DEBUG_VALUE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(debug)));
        }
    }

    public void printDebugTraceMessage( String  message) {
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln( message );
        }
    }
    public void printDebugFullMessage( String  message) {
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {

            getLogStream().printLogln( message );
        }
    }

    protected static void setDebug(int level) {
        debug = level;
    }

    protected static int getDebug() {
        return debug;
    }

    protected static void setLogStream(LogPrintStream ps) {
        logStream = ps;
    }

    protected static LogPrintStream getLogStream() {
        return logStream;
    }

    public SerializableObject() {
        super();
    }

    
    protected ObjectServerException getObjectServerException(
        String form_key, String msg_key, Exception exception_object) {

        ObjectServerException  object_server_exception;

        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                form_key,
                getMessageResource().getMessage( msg_key )));

        return object_server_exception;
    }

    protected void printExceptionMessage(
        String format_key, String message_key,
        Exception exception_object, String function, int nest) {
        if (debug > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE) {
            PrintMessage.printMessage(
                getLogStream(),
                getMessageResource(),
                format_key,
                message_key,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                TraceMessage.getCurrentTraceString(
                    SerializableObject.class.getName(), function, nest),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                exception_object.toString()
            );
        }
    }

    public Object cloneObject(Object obj) throws Exception {
        Class cls;
        Object inst;
        Field[] fld;
        int mds;
        ObjectServerException   ose;

        cls = obj.getClass();
        fld = cls.getFields();

        try {
            inst = cls.newInstance();
        } catch (InstantiationException exception_object) {
            printExceptionMessage(
                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                exception_object, CLONE_OBJECT, 0);

            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                    getMessageResource().getMessage( ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION)));
            throw ose;
        } catch (Exception exception_object) {
            printExceptionMessage(
                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_INSTANTIATION_EXCEPTION,
                exception_object, CLONE_OBJECT, 0);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_CLASS_NEW_INSTANCE_ILLEGAL_ACCESS_EXCEPTION,
                    getMessageResource().getMessage( ObjectServerError.JAVA_CLASS_NEW_INSTANCE_ILLEGAL_ACCESS_EXCEPTION)));
            throw ose;
        }
        for (int i = 0; i < fld.length; i++) {
            mds = fld[i].getModifiers();
            if (Modifier.isFinal(mds) == true ) { continue; }

            if (fld[i].getType().getName().compareTo(int.class.getName()) == 0) {
                int  iop;
                try {
                    iop = fld[i].getInt(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_INT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setInt(inst, iop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(long.class.getName()) == 0) {
                long  lop;
                try {
                    lop = fld[i].getLong(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_LONG_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setLong(inst, lop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(float.class.getName()) == 0) {
                float  fop;
                try {
                    fop = fld[i].getFloat(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_FLOAT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setFloat(inst, fop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_FLOAT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(double.class.getName()) == 0) {
                double  dop;
                try {
                    dop = fld[i].getDouble(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setDouble(inst, dop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_INT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(boolean.class.getName()) == 0) {
                boolean  bop;
                try {
                    bop = fld[i].getBoolean(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_BOOLEAN_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_BOOLEAN_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_DOUBLE_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setBoolean(inst, bop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_BOOLEAN_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(byte.class.getName()) == 0) {
                byte  byop;
                try {
                    byop = fld[i].getByte(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_BYTE_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setByte(inst, byop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_BYTE_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_BYTE_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_LONG_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_BYTE_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_BYTE_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(char.class.getName()) == 0) {
                char  cop;
                try {
                    cop = fld[i].getChar(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_CHAR_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setChar(inst, cop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_CHAR_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else
            if (fld[i].getType().getName().compareTo(short.class.getName()) == 0) {
                short  sop;
                try {
                    sop = fld[i].getShort(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_SHORT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].setShort(inst, sop);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_SHORT_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            } else {
                Object op;
                try {
                    op = fld[i].get(obj);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_GET_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
                try {
                    fld[i].set(obj, op);
                } catch (IllegalArgumentException illegal_argument_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ARGUMENT_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ARGUMENT_EXCEPTION,
                        illegal_argument_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ARGUMENT_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ARGUMENT_EXCEPTION)));
                    throw ose;
                } catch (IllegalAccessException illegal_access_exception) {
                    printExceptionMessage(
                        ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ACCESS_EXCEPTION,
                        ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ACCESS_EXCEPTION,
                        illegal_access_exception, CLONE_OBJECT, 0);
                    ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ACCESS_EXCEPTION,
                            getMessageResource().getMessage( ObjectServerError.JAVA_FIELD_SET_ILLEGAL_ACCESS_EXCEPTION)));
                    throw ose;
                }
            }
        }
        return inst;
    }
}
