/**
 * Copyright (C) All rights reserved.
 *
 * @Module     StringsTable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/03/09
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class StringsTable extends SerializableObject {
    private String[] entries = new String[0];
    private String[] types = new String[0];

/**
    private static int debug = 0;
    private static LogPrintStream logStream = null;
    private static MessageResource message_resource = new MessageResource();
**/

    private final static String DEBUG_ADD
        = "Add";
    private final static String DEBUG_DELETE
        = "Delete";
    private final static String DEBUG_GET
        = "Get";

/**
    public void setDebug(int parameter_debug) {
        debug = parameter_debug;
    }

    public int getDebug() {
        int local_debug;

        local_debug = debug;
        return local_debug;
    }
**/

/*
    private static String default_type = ThreadTable.PARALLEL_SERVICE;
*/
    private static String default_type = ThreadTable.SERIALIZABLE_SERVICE;
    public void setDefaultType(String type) {
        if (type != null && 
           (type.compareTo(ThreadTable.PARALLEL_SERVICE) == 0 ||
            type.compareTo(ThreadTable.SERIALIZABLE_SERVICE) == 0 ||
            type.compareTo(ThreadTable.SESSION_SERVICE) == 0)) {
             default_type = type;
        }
    }

    public String getDefaultType() {
        return default_type;
    }

/**
    public void setLogStream(LogPrintStream ps) {
        logStream = ps;
    }

    public LogPrintStream getLogStream() {
        return logStream;
    }
**/

/**
    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }
**/

    public StringsTable add(String entry, String type)  throws Exception {
        ObjectServerException  object_server_exception;
        int         index;
        int         count;
        String[]    bk_entries;
        String[]    bk_types;

        /**
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_ADD));
        }
        **/
        if (entry == null || type == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_STRINGTABLE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_STRINGTABLE)));
            throw object_server_exception;
        }
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_TRACE_KEY),
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_TRACE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    entry.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    type.toString())); 
        }
        try {
            index = getIndex(entry);
            return this;
        } catch (Exception exception_object) {
        }
        
        count = entries.length;
        bk_entries = new String[count+1];
        for (int i = 0; i < count; i++) {
            bk_entries[i] = entries[i];
        }
        entries = bk_entries;
        entries[count] = entry;
        bk_types = new String[count+1];
        for (int i = 0; i < count; i++) {
            bk_types[i] = types[i];
        }
        types = bk_types;
        types[count] = type;
        return this;
    }
    
    public StringsTable delete(String entry) throws Exception {
        ObjectServerException  object_server_exception;
        int         loop;
        int         count;
        int         index;
        String[]    bk_entries;

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_DELETE));
        }
        if (entry == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_STRINGTABLE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_STRINGTABLE)));
            throw object_server_exception;
        }
        try {
            index = getIndex(entry);
        } catch (Exception exception_object) {
            if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                        getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY)));
            }
            throw exception_object;
        }
        count = entries.length;
        bk_entries = new String[count-1];
        loop = 0;
        for (int i = 0; i < bk_entries.length; i++) {
            if (i == index) {
                loop = loop + 1;
            }
            bk_entries[i] = entries[loop];
            loop++;
        }
        entries = bk_entries;
        return this;
    }

    private int getIndex(String entry) throws Exception {
        ObjectServerException  object_server_exception;
        int  count;
        
        if (entry == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_STRINGTABLE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_STRINGTABLE)));
            throw object_server_exception;
        }
         
        count = entries.length;
        for (int i = 0; i < count; i++) {
            if (entry.compareTo(entries[i]) == 0) {
                return i;
            }
        }
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerError.ENTRY_NOT_FOUND_STRINGTABLE,
                getMessageResource().getMessage(
                    ObjectServerError.ENTRY_NOT_FOUND_STRINGTABLE)));
        throw object_server_exception;
    }

    public String get(int index) throws Exception {
        ObjectServerException  object_server_exception;
        String   return_string;

        /***
        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_GET));
        }
        ***/
        if (index < 0 || index >= entries.length) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_STRINGTABLE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_STRINGTABLE)));
            throw object_server_exception;
        }
        return_string = entries[index];
        return return_string;
    }

    public boolean equal(String entry) {
        int   index;

        if (entry == null) {
            return false;
        }
        try {
            index = getIndex(entry);
        } catch (Exception exception_object) {
            return false;
        }
        return true;
    }

    public int getCount() {
        return entries.length;
    }

    public StringsTable clone() {
        StringsTable clone_table;

        clone_table = new StringsTable();
        for (int i = 0; i < entries.length; i++) {
            try {
                clone_table.add(entries[i], types[i]);
            } catch (Exception exception_object) {
            }
        }
        return clone_table;
    }

    public String getType(int index) throws Exception {
        String  return_type;
        ObjectServerException  object_server_exception;

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_GET)); 
        } 
        if (index < 0 || index >= types.length) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_STRINGTABLE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_STRINGTABLE)));
            throw object_server_exception;
        }
        return_type = types[index]; 
        return return_type; 
    } 

    public void setType(String entry, String para_type) throws Exception { 
        int  index; 
        ObjectServerException object_server_exception;

        if (entry == null || para_type == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_STRINGTABLE,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_STRINGTABLE)));
            throw object_server_exception;
        }
        try { 
            index = getIndex(entry); 
        } catch (Exception exception_object) { 
            throw exception_object; 
        }
        types[index] = para_type; 

    }
}
