/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ThreadTable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/03/01
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ThreadTable extends SerializableObject {
    public final static String SERVICE_TABLE = "ServiceThreadTable";
    public final static String SYSTEM_TABLE  = "SystemThreadTable";
    public final static String FORWARD_TABLE = "ForwardThreadTable";
    public final static String REGISTER_TABLE  = "RegisterThreadTable";
    public final static String TRANSIENT_TABLE = "TransientThreadTable";

    private String system_type = null;

    private Thread[] threads = new Thread[0];
    private Socket[] sockets = new Socket[0];
    private InputStream[] inputs = new InputStream[0];
    private Thread[] monitors = new Thread[0];
    private String[] objects = new String[0];

    private String[] types = new String[0];
    public static String SERIALIZABLE_SERVICE = "SerializableService";
    public static String PARALLEL_SERVICE = "ParallelService";
    public static String SESSION_SERVICE = "SessionService";

    private static String command_default_type = SERIALIZABLE_SERVICE;

    private String PARAMETER_ERROR = "Parameter error";
    private String NO_INITIALIZATION_EXCEPTION = "No initialization exception";

    private String DEBUG_GETINDEX = "getIndex";
    private String DEBUG_GETTHREADBYMONITOR = "getThreadbyMonitor";

    private String TRACE_CLASSNAME = "oserv.ThreadTable";
    private String TRACE_GETINDEX = "getIndex";

    public ThreadTable() {
    }

    public ThreadTable(String type) {
        setType(type);
    }

    public void setType(String type) {
        system_type = type;
    }

    public String getType() {
        return system_type;
    }

    public void setDefaultCommandType(String command_type) {
        command_default_type = command_type;
    }

    public String getDefaultCommandType() {
        return command_default_type;
    }

    public int getIndex(Thread target) throws Exception {
        Thread  entry;
        String  entry_string;
        String  target_string;
        Exception exception_object;

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE
            && getLogStream() != null) {
            if (target == null) {
                getLogStream().printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_GETINDEX, 
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            } else {
                getLogStream().printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        target.toString(),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }
        if (target == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        if (threads == null || inputs == null || monitors == null) {
            String line_information = TraceMessage.getCurrentTraceString(
                                          TRACE_CLASSNAME, TRACE_GETINDEX, 0) ;

            exception_object = new Exception (
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_ASSERTION_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_ASSERTION_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    line_information));
            throw exception_object;
        }
        target_string = target.getName();
        for (int i = 0; i < threads.length; i++) {
            entry = (Thread)threads[i];
            entry_string = entry.getName();
            if (entry_string.compareTo(target_string) == 0) {
                return i;
            }
        }
        exception_object = new Exception( 
            getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
        throw exception_object;
    }

    public Thread getThreadbyMonitor(Thread target) throws Exception {
        int index;
        Thread   thread_entry;
        Exception exception_object;

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE
            && getLogStream() != null) {
            if (target == null) {
                getLogStream().printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_GETTHREADBYMONITOR,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            } else {
                getLogStream().printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        target.toString(),
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }
        if (target == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        try {
            index = getIndexbyMonitor(target);
            thread_entry = getThread(index);
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return thread_entry;
    }

    public int getIndexbyMonitor(Thread target) throws Exception {
        Thread  entry;
        String  entry_string;
        String  target_string;
        Exception exception_object;

        if (target == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        target_string = target.getName();
        for (int i = 0; i < monitors.length; i++) {
            entry = (Thread)monitors[i];
            entry_string = entry.getName();
            if (entry_string.compareTo(target_string) == 0) {
                return i;
            }
        }
        exception_object = new Exception( getMessageResource().getFormattedMessage(
            ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
            getMessageResource().getMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
        throw exception_object;
    }

    public Thread getThread(int index) throws Exception {
        Thread  target;
        Exception exception_object;

        if (index >= threads.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        target = (Thread)threads[index];
        return target;
    }

    public Socket getSocket(Thread target) throws Exception {
        int      index;
        Socket   target_socket;
        Exception exception_object;

        if (target == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        try {
            index = getIndex(target);
            target_socket = (Socket)getSocket(index);
            return target_socket;
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
    }

    public Socket getSocket(int index) throws Exception {
        Socket  target;
        Exception exception_object;

        if (index >= sockets.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        target = (Socket)sockets[index];
        return target;
    }

    public InputStream getInputStream(Thread target) throws Exception {
        int           index;
        InputStream   target_input;
        Exception exception_object;

        if (target == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        try {
            index = getIndex(target);
            target_input = (InputStream)getInputStream(index);
            return target_input;
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
    }

    public InputStream getInputStream(int index) throws Exception {
        InputStream target;
        Exception exception_object;

        if (index >= sockets.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        target = (InputStream)inputs[index];
        return target;
    }

    public Thread getMonitor(Thread target) throws Exception {
        int      index;
        Thread   target_object;
        Exception exception_object;

        if (target == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        try {
            index = getIndex(target);
            target_object = (Thread)getMonitor(index);
            return target_object;
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
    }

    public Thread getMonitor(int index) throws Exception {
        Thread target;
        Exception exception_object;

        if (index >= sockets.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        target = (Thread)monitors[index];
        return target;
    }

    public boolean add(Thread target, Socket socket_object, InputStream input_object, Thread monitor_object) throws Exception {
        Exception exception_object;

        if (target == null || socket_object == null || input_object == null ) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }

        if (getDebug() > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE
            && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_TRACE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_TRACE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(threads.length)));
        }
        Thread[]  bk_threads;
        bk_threads = new Thread[threads.length+1];
        for (int i = 0; i < threads.length; i++) {
            bk_threads[i] = threads[i];
        }
        threads = bk_threads;
        threads[threads.length-1] = target;

        Socket[]  bk_sockets;
        bk_sockets = new Socket[sockets.length+1];
        for (int i = 0; i < sockets.length; i++) {
            bk_sockets[i] = sockets[i];
        }
        sockets = bk_sockets;
        sockets[sockets.length-1] = socket_object;

        InputStream[]  bk_inputs;
        bk_inputs = new InputStream[inputs.length+1];
        for (int i = 0; i < inputs.length; i++) {
            bk_inputs[i] = inputs[i];
        }
        inputs = bk_inputs;
        inputs[inputs.length-1] = input_object;

        Thread[] bk_monitors;
        bk_monitors = new Thread[monitors.length+1];
        for (int i = 0; i < monitors.length; i++) {
            bk_monitors[i] = monitors[i];
        }
        monitors = bk_monitors;
        monitors[monitors.length-1] = monitor_object;

        String[] bk_objects;
        bk_objects = new String[objects.length+1];
        for (int i = 0; i < objects.length; i++) {
            bk_objects[i] = objects[i];
        }
        objects = bk_objects;
        objects[objects.length-1] = null;

        String[] bk_types;
        bk_types = new String[types.length+1];
        for (int i = 0; i < types.length; i++) {
            bk_types[i] = types[i];
        }
        types = bk_types;
        types[types.length-1] = getDefaultCommandType();
        return true;
    }

    public void setObject(int index, String command) throws Exception {
        Exception exception_object;

        if (index >= objects.length || index < 0 || command == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        objects[index] = command;
        return;
    }

    public String getObject(int index) throws Exception {
        Exception exception_object;

        if (index >= objects.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        return objects[index];
    }

    public void setType(int index, String type) throws Exception {
        Exception exception_object;
        if (type == null || index >= types.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        types[index] = type;
    }

    public String getType(int index) throws Exception {
        Exception exception_object;

        if (types == null || index >= types.length || index < 0) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_THREAD_TABLE)));
            throw exception_object;
        }
        return types[index];
    }

    public boolean isSerializable(int index) throws Exception {
        String local_type = types[index];

        if (local_type == null) {
            return false;
        }
        if (local_type.compareTo(SERIALIZABLE_SERVICE) == 0) {
            return true;
        }
        return false;
    }

    public boolean remove(Thread target) {
        int      index;
        boolean  return_flag;
        Exception exception_object;

        if (target == null) {
            return false; 
        }
        try {
            index = getIndex(target);
            return_flag = remove(index);
        } catch (Exception real_exception_object) {
            return false;
        }
        return return_flag;
    }

    public boolean remove(int index) {
        int  loop_index;

        if (threads != null && index >= threads.length || index < 0) {
            return false;
        }
        Thread[]  bk_threads;
        bk_threads = new Thread[threads.length - 1];
        loop_index = 0;
        for (int i = 0; i < threads.length; i++) {
            if (i == index) {
                continue;
            }
            bk_threads[loop_index] = threads[i];
            loop_index++;
        }
        threads = bk_threads;
        
        Socket[]  bk_sockets;
        bk_sockets = new Socket[sockets.length - 1];
        loop_index = 0;
        for (int i = 0; i < sockets.length; i++) {
            if (i == index) {
                continue;
            }
            bk_sockets[loop_index] = sockets[i];
            loop_index++;
        }
        sockets = bk_sockets;

        InputStream[]  bk_inputs;
        bk_inputs = new InputStream[inputs.length - 1];
        loop_index = 0;
        for (int i = 0; i < inputs.length; i++) {
            if (i == index) {
                continue;
            }
            bk_inputs[loop_index] = inputs[i];
            loop_index++;
        }
        inputs = bk_inputs;

        Thread[] bk_monitors;
        bk_monitors = new Thread[monitors.length - 1];
        loop_index = 0;
        for (int i = 0; i < monitors.length; i++) {
            if (i == index) {
                continue;
            }
            bk_monitors[i] = monitors[i];
            loop_index++;
        }
        monitors = bk_monitors;

        String[] bk_objects;
        bk_objects = new String[objects.length - 1];
        loop_index = 0;
        for (int i = 0; i < objects.length; i++) {
            if (i == index) {
                continue;
            }
            bk_objects[i] = objects[i];
            loop_index++;
        }
        objects = bk_objects;

        String[] bk_types;
        bk_types = new String[types.length - 1];
        loop_index = 0;
        for (int i = 0; i < types.length; i++) {
            if (i == index) {
                continue;
            }
            bk_types[i] = types[i];
            loop_index++;
        }
        types = bk_types;
        return true;
    }

    public int getCount() throws Exception {
        Exception exception_object;
        if (threads == null) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.NO_INITIALIZATION_EXCEPTION_THREAD_TABLE,
                getMessageResource().getMessage(
                    ObjectServerError.NO_INITIALIZATION_EXCEPTION_THREAD_TABLE)));
            throw exception_object;
        }
        return threads.length;
    }

    public ThreadTable getEntry(Thread target) throws Exception {
        int           index;
        ThreadTable  thread_table_object;
        thread_table_object = new ThreadTable();

        try {
            index = getIndex(target);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        thread_table_object.add(
            threads[index], sockets[index], inputs[index], monitors[index]);
        return thread_table_object;
    }

    public ThreadTable clone() {
        ThreadTable  thread_table_object;
        thread_table_object = new ThreadTable();

        int         count;
        count = threads.length;

        for (int i = 0; i < count; i++) {
            try {
                thread_table_object.add(threads[i],
                    sockets[i], inputs[i], monitors[i]);
            } catch (Exception exception_object) {
                continue;
            }
        }        
        return thread_table_object;
    }

    public static void main(String[] args) {
        int          index;
        Thread       thread_object;
        Thread       monitor_object;
        InputStream  input_object;
        Socket       socket_object;
        ThreadTable  thread_table;
        Thread       thread_entry;
        Thread       monitor_entry;
        InputStream  input_entry;
        Socket       socket_entry;

        thread_table = new ThreadTable();

        thread_object = new Thread();
        input_object  = System.in;
        socket_object = new Socket();
        monitor_object = new Thread();

        System.out.println(thread_object.toString());
        System.out.println(input_object.toString());
        System.out.println(socket_object.toString());
        System.out.println(monitor_object.toString());

        try {
            thread_table.add(thread_object, socket_object, input_object,
                monitor_object);

            index = thread_table.getIndex(thread_object);
            thread_entry = thread_table.getThread(index);
            input_entry = thread_table.getInputStream(index);
            socket_entry = thread_table.getSocket(index);
            monitor_entry = thread_table.getMonitor(index);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        System.out.println(thread_entry.toString());
        System.out.println(input_entry.toString());
        System.out.println(socket_entry.toString());
        System.out.println(monitor_entry.toString());
    }
}
