/**
 * Copyright (C) All rights reserved.
 *
 * @Module:    TraceMessage.java
 * @Author:    Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version:   2.1 2012/03/16
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.util.*;

public class TraceMessage extends Object {
    private static String EMPTY_MESSAGE = "";
    private static String OBJECT_DOT_SEPARATOR = ".";
    private static String LINE_NUMBER_SEPARATOR = ":";
    private static String START_PARENTHESIS = "(";
    private static String END_PARENTHESIS = ")";

    public static String getTraceString(StackTraceElement stack_trace_element) {
        String line_message;

        if (stack_trace_element == null) {
            line_message = new String(EMPTY_MESSAGE);
            return line_message;
        }

        line_message = stack_trace_element.getClassName()
             + OBJECT_DOT_SEPARATOR
             + stack_trace_element.getMethodName()
             + START_PARENTHESIS
             + stack_trace_element.getFileName()
             + LINE_NUMBER_SEPARATOR
             + String.valueOf(stack_trace_element.getLineNumber())
             + END_PARENTHESIS ;
        return line_message; 
    }

    public static String getCurrentTraceString(
        String class_name, String method_name, int nest) {
        String trace_string;
        StackTraceElement   stack_trace_element;
        StackTraceElement[] stack_trace_element_array;
        stack_trace_element_array = Thread.currentThread().getStackTrace();

        stack_trace_element = getStackTraceElement(stack_trace_element_array,
                                  class_name, method_name, nest);
        trace_string = getTraceString(stack_trace_element);

        return trace_string;
    }

    public static String getCallerTraceString(
        String class_name, String method_name, int nest) {
        String trace_string;
        StackTraceElement   stack_trace_element;
        StackTraceElement[] stack_trace_element_array;
        stack_trace_element_array = Thread.currentThread().getStackTrace();

        stack_trace_element = getPreviousTraceElement(stack_trace_element_array,
                                  class_name, method_name, nest);
        trace_string = getTraceString(stack_trace_element);

        return trace_string;
    }

    public static StackTraceElement getPreviousTraceElement(
        StackTraceElement[] stack_trace_element,
        String class_name,
        String method_name, int nest) {
        int     local_nest = 0;

        for (int i = 0; i < stack_trace_element.length; i++) {
            if (stack_trace_element[i].getClassName().compareTo(class_name) == 0
            && stack_trace_element[i].getMethodName().compareTo(method_name) == 0) {
                if (local_nest == nest) {
                    if (i > 0) {
                        return stack_trace_element[i-1];
                    }
                }
                local_nest++;
            }
        }
        return null;
    }

    public static StackTraceElement getNextTraceElement(
        StackTraceElement[] stack_trace_element,
        String class_name,
        String method_name, int nest) {
        int     local_nest = 0;

        for (int i = 0; i < stack_trace_element.length; i++) {
            if (stack_trace_element[i].getClassName().compareTo(class_name) == 0
            && stack_trace_element[i].getMethodName().compareTo(method_name) == 0) {
                if (local_nest == nest) {
                    if (i < stack_trace_element.length -1) {
                        return stack_trace_element[i+1];
                    }
                }
                local_nest++;
            }
        }
        return null;
    }

    public static StackTraceElement getStackTraceElement(StackTraceElement[] stack_trace_element, String class_name, String method_name, int nest) {
        String  trace_string;
        int     local_nest = 0;

        for (int i = 0; i < stack_trace_element.length; i++) {

            if (stack_trace_element[i].getClassName().compareTo(class_name) == 0
            && stack_trace_element[i].getMethodName().compareTo(method_name) == 0) {
                if (local_nest == nest) {
                    return stack_trace_element[i];
                }
                local_nest++;
            }
        }
        return null;
    }

    /**
     * getCallerMethod()
     */
    public static String getCallerMethod(Exception exception_object, String class_name, String method_name, int nest) {
        String  trace_string;
        StackTraceElement stack_trace_element;
        StackTraceElement[] stack_trace_element_array;

        if (exception_object == null
            || exception_object.getStackTrace() == null
            || method_name == null) {
            return new String(EMPTY_MESSAGE);
        }
        stack_trace_element_array = exception_object.getStackTrace();
        stack_trace_element = getStackTraceElement(stack_trace_element_array,
                                  class_name, method_name, nest);
        if (stack_trace_element == null) {
            return new String(EMPTY_MESSAGE);
        }
        trace_string = getTraceString(stack_trace_element);

        return trace_string;
    }
}
