/**
 * Copyright (C) All rights reserved.
 *
 * @Module     UTFInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/03/28
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;

public class UTFInputStream extends DataInputStream {

    private static int     MAX_LINE = 16;

    private static int     ONE_CODE_START = 0;
    private static int     ONE_CODE_END = 0x7F;
    private static int     TWO_CODE_START = 0xC2;
    private static int     TWO_CODE_END = 0xDF;
    private static int     THREE_CODE_START = 0xE0;
    private static int     THREE_CODE_END = 0xEF;
    private static int     FOUR_CODE_START = 0xF0;
    private static int     FOUR_CODE_END = 0xF7;
    private static int     FIVE_CODE_START = 0xF8;
    private static int     FIVE_CODE_END = 0xFB;
    private static int     END_CODE = 0x100;
    private static int     ONEUTF8=1;
    private static int     TWOUTF8=2;
    private static int     THREEUTF8=3;
    private static int     FOURUTF8=4;
    private static int     FIVEUTF8=5;
    private static int     SIXUTF8=6;

    private static int     UTF8SIZE = 3;
    private static byte[]  line_buffer = null;
    private static int     seek_pointer = -1;
    private static int     rt = 0;

    private static byte[]  one_byte = new byte[1];

    private String UTF8STRING = "UTF-8";

    private String LINE_STRING = "Line ";

    private int getByteSize(byte code) {
        return 0;
    }

    private static MessageResource message_resource = new MessageResource();

    public UTFInputStream(InputStream input_stream) throws Exception {
        super(input_stream);
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    private byte getByte() throws Exception {
        int  rt = 0;
        byte[]  one_byte = new byte[1];
        ObjectServerException    object_server_exception;

        try {
            rt = read(one_byte, 0, 1);
        } catch (Exception exception_object) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_IO_EXCEPTION,
                    getMessageResource().getMessage(
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_IO_EXCEPTION)));
            throw exception_object;
        }
        if (rt == -1) { 
            EOFException exception_object;
            exception_object = new EOFException();
            throw exception_object;
        }
        return one_byte[0];
    }

    public String getLine() throws Exception {
        int       index;
        byte      first;
        byte[]    word;
        String    line = null;

        index = 0;
        while (true) {
            try {
                first = getByte();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    return line;
                }
                throw exception_object;
            }
            if (first >= ONE_CODE_START) {
                if (first == 0xa) {
                    return line;
                }
                if (line == null) { line = new String();}
                word = new byte[ONEUTF8];
                word[0] = first;
                line = line.concat(new String(word, 0, 1));
                index = 0;
                continue;
            }
            if (first - (TWO_CODE_START - END_CODE) >= 0 && first - (TWO_CODE_END - END_CODE) <= 0) {
                word = new byte[TWOUTF8];
            } else 
            if (first - (THREE_CODE_START - END_CODE) >= 0 && first - (THREE_CODE_END - END_CODE) <= 0) {
                word = new byte[THREEUTF8];
            } else if (first - (FOUR_CODE_START - END_CODE) >= 0 && first - (FOUR_CODE_END - END_CODE) <= 0) {
                word = new byte[FOURUTF8];
            } else if (first - (FIVE_CODE_START - END_CODE) >= 0 && first - (FIVE_CODE_END - END_CODE) <= 0) {
                word = new byte[FIVEUTF8];
            } else {
                word = new byte[SIXUTF8];
            }
                word[0] = first;
                try {
                    for (int j = 1; j < word.length; j++) {
                        word[j] = getByte();
                    }
                } catch (Exception exception_object) {
                    if ((exception_object instanceof EOFException) == true) {
                        return line;
                    }
                    throw exception_object;
                }
                if (line == null) { line = new String();}
                line = line.concat(new String(word, UTF8STRING));
        }
    }

    public static void main(String[] args) {
        String           line;
        UTFInputStream   utf_input_stream;
        FileInputStream  file_input_stream;

        if (args.length == 1) {

            try {
                file_input_stream = new FileInputStream(args[0]);
                utf_input_stream = new UTFInputStream(file_input_stream);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
            while (true) {
                try {
                    line = utf_input_stream.getLine();
                } catch (Exception exception_object) {
                    break;
                }
                if (line == null) { break; }
                System.out.println(utf_input_stream.LINE_STRING + line);
            }
        }
    }
}
