/**
 * Copyright (C) All rights reserved.
 *
 * @Module     UserClassLoader.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/06/11
 *
 **/

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

public class UserClassLoader extends java.lang.ClassLoader  {
    public static int BUFFER_SIZE = 8192;
    public static String CLASS_SUFFIX = ".class";
    public static String DOT_STRING = ".";

    public final static String OSERV_CALL_METHOD
        = "oserv.CallMethod";
    public final static String OSERV_CALL_METHOD_INVOKE
        = "oserv.CallMethodInvoke";
    public final static String OSERV_CALL_METHOD_MANAGER
        = "oserv.CallMethodManager";
    public final static String OSERV_OBJECT_INPUT_MANAGER
        = "oserv.ObjectInputManager";
    public final static String OSERV_OBJECT_OUTPUT_MANAGER
        = "oserv.ObjectOutputManager";
    public final static String OSERV_WRITE_VARIABLE = "oserv.WriteVariable";
    public final static String OSERV_READ_VARIABLE = "oserv.ReadVariable";

    private MessageResource message_resource = new MessageResource();

    private ArrayList list = new ArrayList();

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public static String replace(String s0, String s1, String s2) {
        int    idx;
        String s3;

        if (s0 == null || s1 == null || s2 == null) {
            return s0;
        }
        s3 = s0;
        for (int i = 0; i < s0.length(); i++) {
            idx = s3.indexOf(s1, i);
            if (idx >= 0) {
                s3 = s3.substring(0, idx) + 
                    s2 +
                    s3.substring(idx+s1.length());
            }
        }
        return s3;
    }

    public boolean addEntry(String filepath) throws Exception {
        String  entry;

        if (filepath == null) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
                    ObjectServerError.FILENAME_PARAMETER_NULL_LOADER,
                    getMessageResource().getMessage(
                        ObjectServerError.FILENAME_PARAMETER_NULL_LOADER)));
            throw object_server_exception;
        }
        for (int i = 0; i < list.size(); i++) {
            entry = (String)list.get(i);        
            if (entry.compareTo(filepath) == 0) {
                return false;
            }
        }
        list.add(filepath);
        return true;
    }

    public Class readUserClass(String filepath, String main_class) throws Exception {
        int                     rt;
        int                     idx;
        byte[]                  buffer;
        byte[]                  class_body;
        FileInputStream         file_input_stream;
        JarInputStream          jar_input_stream;
        ByteArrayOutputStream   byte_array_output_stream;
        ZipEntry                zip_entry;
        String                  class_name;
        Class                   cls;
        Class                   user_class;
        ClassLoader             class_loader;
        Object                  object_target;
        ArrayList               class_object = new ArrayList();
        ArrayList               class_object_body = new ArrayList();

        buffer = new byte[BUFFER_SIZE];

        if (filepath == null) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
                    ObjectServerError.FILENAME_PARAMETER_NULL_LOADER,
                    getMessageResource().getMessage(
                        ObjectServerError.FILENAME_PARAMETER_NULL_LOADER)));
            throw object_server_exception;
        }
        try {
            addEntry(filepath);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        user_class = null;
        try {
            file_input_stream = new FileInputStream(filepath);
        } catch (Exception exception_object) {
            ObjectServerException    object_server_exception;
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            jar_input_stream = new JarInputStream( file_input_stream );
        } catch (Exception exception_object) {
            ObjectServerException    object_server_exception;
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                ObjectServerError.JAVA_JAR_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                ObjectServerError.JAVA_JAR_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        for (    zip_entry = jar_input_stream.getNextEntry();
                 zip_entry != null;
                 zip_entry = jar_input_stream.getNextEntry() ) {

            class_name = zip_entry.getName();
            idx = class_name.lastIndexOf(CLASS_SUFFIX);
            if (idx <= 0) {
                continue;
            }
            class_name = class_name.substring(0, idx);
            class_name = replace(class_name, File.separator, DOT_STRING);

            byte_array_output_stream = new ByteArrayOutputStream();
            while (true) {
                try {
                    rt = jar_input_stream.read(buffer, 0, buffer.length);
                } catch (Exception exception_object) {
                    ObjectServerException  object_server_exception;
                    object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION,
                            getMessageResource().getMessage(
                                ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION)));
                    throw object_server_exception;
                }
                if (rt == -1) {
                    break;
                }
                byte_array_output_stream.write(buffer, 0, rt);
            }
            class_body = byte_array_output_stream.toByteArray();
            try {
                cls = defineClass(class_name, class_body, 0, class_body.length);
            } catch (NoClassDefFoundError exception_object) {
                class_object.add((Object)class_name);
                class_object_body.add((Object)class_body);
                continue;
            } catch (Exception exception_object) {
                ObjectServerException  object_server_exception;
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION)));

                throw object_server_exception;
            }
            if (user_class == null && main_class != null
                && main_class.compareTo(class_name) == 0) {
                user_class = cls;
            }
        }
        try {
            jar_input_stream.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        for (int i = 0; i < class_object.size(); i++) {
            for (int j = 0; j < class_object.size(); j++) {
                class_name = (String)class_object.get(j);
                class_body = (byte[])class_object_body.get(j);

                if (class_name == null || class_name.length() == 0) {
                    continue;
                }

                try {
                    cls = defineClass(class_name, class_body, 0, class_body.length);
                    class_object.set(j, ObjectServerHeader.EMPTY_MESSAGE);
                } catch (NoClassDefFoundError exception_object) {
                    continue;
                } catch (Exception exception_object) {
                    ObjectServerException  object_server_exception;
                    object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION,
                            getMessageResource().getMessage(
                                ObjectServerError.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION)));

                    throw object_server_exception;
                }
                if (user_class == null && main_class != null
                    && main_class.compareTo(class_name) == 0) {
                    user_class = cls;
                }
            }
        }
        return user_class;
    }

    public Class getSpecifiedClass(String class_name, byte[] oserv_body) throws Exception {
        Class    cls;

        try {
            cls = defineClass(class_name, oserv_body, 0, oserv_body.length);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerError.JAVA_CLASS_LOADER_DEFINE_CLASS_EXCEPTION)));

            throw object_server_exception;
        }
        return cls;
    }
}
