/**
 * Copyright (C) All rights reserved.
 *
 * @Module     VariableType.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/11/30
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

import oserv.*;

public class VariableType {

    private static MessageResource message_resource = new MessageResource();
    
    public final static String ARRAY_KEY = "[";
    public final static String ARRAY_END_KEY = "]";
    public final static char   ARRAY_CHAR_KEY = '[';
    public final static String ARRAY_WORD = "Array";
    public final static String SEMICOLON = ";";
    public final static String ARRAY_SLASH = "/";
    public final static String ARRAY_DOT   = ".";
    public final static String ARRAY_UNDERSCORE = "_";

    public final static char ARRAY_INT = 'I';
    public final static char ARRAY_BYTE = 'B';
    public final static char ARRAY_LONG = 'J';
    public final static char ARRAY_SHORT = 'S';
    public final static char ARRAY_CHAR = 'C';
    public final static char ARRAY_DOUBLE = 'D';
    public final static char ARRAY_FLOAT = 'F';
    public final static char ARRAY_BOOLEAN = 'Z';
    public final static char ARRAY_REFERENCE = 'L';
    public final static String ARRAY_INT_STRING = "int";
    public final static String ARRAY_BYTE_STRING = "byte";
    public final static String ARRAY_LONG_STRING = "long";
    public final static String ARRAY_SHORT_STRING = "short";
    public final static String ARRAY_CHAR_STRING = "char";
    public final static String ARRAY_DOUBLE_STRING = "double";
    public final static String ARRAY_FLOAT_STRING = "float";
    public final static String ARRAY_BOOLEAN_STRING = "boolean";

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public static MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public static String replace(String s0, String s1, String s2) {
        int    idx;
        String s3;

        s3 = s0;
        for (int i = 0; i < s0.length(); i++) {
            idx = s3.indexOf(s1, i);
            if (idx >= 0) {
                s3 = s3.substring(0, idx) + 
                    s2 +
                    s3.substring(idx+s1.length());
            }
        }
        return s3;
    }

    public static String valueOf(String name) throws ObjectServerException {
        int   k;
        String variable_name = null;

        if (name.indexOf(ARRAY_KEY) >= 0) {
            String array_dimension;
            array_dimension = new String();
            for (k = 0; k < name.length(); k++) {
                if (name.charAt(k) != ARRAY_CHAR_KEY) {
                    break;
                }
                array_dimension = array_dimension.concat(
                    ARRAY_KEY + ARRAY_END_KEY);
            }
            if(k == name.length()) {
                ObjectServerException ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION)));
                throw ose;
            }
            if (name.charAt(k) == ARRAY_INT) {
                variable_name = ARRAY_INT_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_BYTE) {
                variable_name = ARRAY_BYTE_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_LONG) {
                variable_name = ARRAY_LONG_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_SHORT) {
                variable_name = ARRAY_SHORT_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_CHAR) {
                variable_name = ARRAY_CHAR_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_FLOAT) {
                variable_name = ARRAY_FLOAT_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_DOUBLE) {
                variable_name = ARRAY_DOUBLE_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_BOOLEAN) {
                variable_name = ARRAY_BOOLEAN_STRING + array_dimension;
            } else if (name.charAt(k) == ARRAY_REFERENCE) {

                int idx = name.lastIndexOf(SEMICOLON);
                if (idx > 0) {
                    String wd;
                    wd = name.substring(k+1, idx);

                    variable_name = replace(wd, ARRAY_SLASH, ARRAY_DOT)
                    + array_dimension;
                    return variable_name;
                } else {
                    ObjectServerException ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION,
                            getMessageResource().getMessage(
                                ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION)));

                    throw ose; 
                }
            } else {
                if (getMessageResource() != null) {
                    ObjectServerException ose = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION,
                            getMessageResource().getMessage(
                                ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION)));
                    throw ose; 
                }
            }
            return variable_name;
        }
        return name;
    }

    public static String convert(String name) throws ObjectServerException {
        int  k;
        if (name.indexOf(ARRAY_KEY) < 0) {
            String wd;
            wd = name;
            wd = replace(wd, ARRAY_SLASH, ARRAY_UNDERSCORE);
            wd = replace(wd, ARRAY_DOT, ARRAY_UNDERSCORE);
            return wd;
        }
        for (k = 0; k < name.length(); k++) {
            if (name.charAt(k) != ARRAY_CHAR_KEY) {
                break;
            }
        }
        if (name.charAt(k) == ARRAY_INT) {
            return (String.valueOf(k) +  ARRAY_INT_STRING);
        } else if (name.charAt(k) == ARRAY_BYTE) {
            return (String.valueOf(k) +  ARRAY_BYTE_STRING);
        } else if (name.charAt(k) == ARRAY_LONG) {
            return (String.valueOf(k) +  ARRAY_LONG_STRING);
        } else if (name.charAt(k) == ARRAY_SHORT) {
            return (String.valueOf(k) + ARRAY_SHORT_STRING);
        } else if (name.charAt(k) == ARRAY_CHAR) {
            return (String.valueOf(k) + ARRAY_CHAR_STRING);
        } else if (name.charAt(k) == ARRAY_FLOAT) {
            return (String.valueOf(k) + ARRAY_FLOAT_STRING);
        } else if (name.charAt(k) == ARRAY_DOUBLE) {
            return (String.valueOf(k) + ARRAY_DOUBLE_STRING);
        } else if (name.charAt(k) == ARRAY_BOOLEAN) {
            return (String.valueOf(k) +  ARRAY_BOOLEAN_STRING);
        } else if (name.charAt(k) == ARRAY_REFERENCE) {
            int idx = name.lastIndexOf(SEMICOLON);
            if (idx > 0) {
                String wd;
                wd = name.substring(k+1, idx);
                wd = replace(wd, ARRAY_SLASH, ARRAY_UNDERSCORE);
                wd = replace(wd, ARRAY_DOT, ARRAY_UNDERSCORE);

                return (String.valueOf(k) + wd);

            } else {
                ObjectServerException ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION)));
                throw ose;
            }
        } else {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.CLASS_FORMAT_ERROR_EXCEPTION)));
            throw ose;
        }
    }
}
