/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     WriteVariable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/05/28
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

public class WriteVariable extends BaseVariable {
    public static String PARAMETER_ERROR_EXCEPTION
        = "Parameter error exception.";

    public static int BUFFER_SIZE = 8192;
    public static String CLASS_SUFFIX = ".class";
    public static String DOT_STRING = ".";
    public final static String TEST_CLASS_NAME = "Test";

    public final static String START_BRACKET = "[";
    public final static String END_BRACKET = "]";
    public final static String JAVA_IO_SERIALIZABLE_TYPE_KEY = "java.io.Serializable";
    public final static String SEMICOLON = ";";
    public final static char ARRAY_INT = 'I';
    public final static char ARRAY_BYTE = 'B';
    public final static char ARRAY_LONG = 'J';
    public final static char ARRAY_SHORT = 'S';
    public final static char ARRAY_CHAR = 'C';
    public final static char ARRAY_DOUBLE = 'D';
    public final static char ARRAY_FLOAT = 'F';
    public final static char ARRAY_BOOLEAN = 'Z';
    public final static char ARRAY_REFERENCE = 'L';

    /**
     * This method provides a feature for writing static variable
     *
     * @param       String  - file name
     *              Class   - main class
     *              Object  - real instance of class
     * @return      void
     * @throws      Exception
     */
    public void writeVariable(String filename, Class cls, Object obj) 
        throws Exception {
        ObjectOutputStream object_output_stream;
        FileOutputStream file_output_stream;
        Field[]  fld;
        int mds;
        int         int_value;
        int      iop;
        int   cntfld;
        long     lop;
        short    sop;
        float    fop;
        double   dop;
        Object    op;
        boolean  bop;
        byte    byop;
        char     cop;
        String fldnm;
        File    fdes;

        if (filename == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (cls == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }
        if (obj == null) {
            NullPointerException  null_pointer_exception;
            null_pointer_exception = new NullPointerException();
            throw null_pointer_exception;
        }

        fdes = new File(filename);
        if (fdes.exists() == true) {
            fdes.delete();
        }
        try {
            file_output_stream = new FileOutputStream(fdes);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            object_output_stream = new ObjectOutputStream((OutputStream)file_output_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        fld = cls.getFields();

        for (int i = 0; i < fld.length; i++) {
            mds = fld[i].getModifiers();
            if (Modifier.isFinal(mds) == true ) { continue; }
            /*
            if (Modifier.isPublic(mds) == false ) { continue; }
            */
            /**
            if (Modifier.isStatic(mds) == false ) { continue; }
             **/

            if (fld[i].getType().getName().compareTo(int.class.getName()) == 0) {
                int_value = fld[i].getInt(obj);
                try {
                    object_output_stream.writeInt(int_value);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[i].getType().getName().compareTo(long.class.getName()) == 0) {
                lop = fld[i].getLong(obj);
                try {
                    object_output_stream.writeLong(lop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[i].getType().getName().compareTo(float.class.getName()) == 0) {
                fop = fld[i].getFloat(obj);
                try {
                    object_output_stream.writeFloat(fop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[i].getType().getName().compareTo(double.class.getName()) == 0) {
                dop = fld[i].getDouble(obj);

                try {
                    object_output_stream.writeDouble(dop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[i].getType().getName().compareTo(boolean.class.getName()) == 0) {
                bop = fld[i].getBoolean(obj);

                try {
                    object_output_stream.writeBoolean(bop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (fld[i].getType().getName().compareTo(byte.class.getName()) == 0) {
                byop = fld[i].getByte(obj);

                try {
                    object_output_stream.writeByte(byop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }

            if (fld[i].getType().getName().compareTo(char.class.getName()) == 0) {
                cop = fld[i].getChar(obj);

                try {
                    object_output_stream.writeChar(cop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }

            if (fld[i].getType().getName().compareTo(short.class.getName()) == 0) {
                sop = fld[i].getShort(obj);

                try {
                    object_output_stream.writeShort(sop);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                continue;
            }
            if (isSerializable(fld[i]) == true) {
                op = fld[i].get(obj);
                try {
                    object_output_stream.writeObject(op);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
            }
        }
        try {
            object_output_stream.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }
}
