/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     WriteVariableWrapper.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2013/05/28
 *
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class WriteVariableWrapper extends ObjectServerHeader {
    private Object object_instance = null;
    private Class  write_variable_class = null;

    private Method write_variable_method = null;
    private String   WRITE_VARIABLE_METHOD_NAME = "writeVariable";
    private Object[] write_variable_args_object = new Object[3];
    private Class[]  write_variable_args_class = {
        String.class,
        Class.class,
        Object.class
    };
    private CallMethod  call_method = null;

    public WriteVariableWrapper(Class param_write_variable_class) throws Exception {
        if (param_write_variable_class == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        write_variable_class = param_write_variable_class;

        try {
            object_instance = write_variable_class.newInstance();
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            write_variable_method = write_variable_class.getMethod(
                WRITE_VARIABLE_METHOD_NAME,
                write_variable_args_class);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }

        try {
            call_method = CallMethodFactory.getCallMethod(
                write_variable_method, null);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
    }

    public void writeVariable(String filename, Class cls, Object obj) 
        throws Exception {

        if (object_instance == null) { 
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        if (filename == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        if (cls == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        if (obj == null) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method.set(object_instance, (Object)filename, 0);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method.set(object_instance, (Object)cls, 1);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method.set(object_instance, (Object)obj, 2);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
        try {
            call_method.invoke(object_instance);
        } catch (Exception exception_object) {
            ObjectServerException  object_server_error;
            String error_message;

            error_message = getMessageResource().getFormattedMessage(
                ObjectServerException.INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    ObjectServerException.INTERNAL_ERROR_EXCEPTION));
            object_server_error = new ObjectServerException(error_message);
            throw object_server_error;
        }
    }
}


