/**
 * Copyright (C) All rights reserved.
 *
 * @Module     CallMethodDebug2Test.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/06/19
 */

package oserv.test;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;

import oserv.*;

public class CallMethodDebug2Test extends ObjectCommand {

    private static String PARAMETER_ERROR = "Parameter error";

    private final static String OSERV_CALL_METHOD_DEBUG2
        = "oserv.test.CallMethodDebug2";

    private static String CHECK_INVOKE ="check_invoke";
    private static Class[]  check_invoke_args_class = { 
        Object.class,
        Method.class,
        Object[].class };
    private static Object[] check_invoke_args_object = new Object[3];
    private static Method  check_invoke_method = null;


    private static String CALL_METHOD_SAMPLE_CLASS = "oserv.test.CallMethodSample";
    private static Class  call_method_sample_class = null;
    private static String INT_METHOD = "intMethod";
    private static Class[] int_method_args_class = { int.class };
    private static Object[] int_method_args_object = new Object[1];
    private static Method  call_method_sample_int_method;

    private static String INTEGER_MESSAGE = "Integer ";

    private static String RETURN_IS_NULL = "Return value is null";

    public void execTestCase(UserClassLoader user_class_loader) {
        byte[]   oserv_body;
        String   class_name;

        Class   call_method_manager_class;
        Object  object_instance;
        Object  sample_object;

        class_name = OSERV_CALL_METHOD_DEBUG2;
        oserv_body = CallMethodDebug2Data.getClassData();

        try {
            call_method_manager_class = 
                user_class_loader.getSpecifiedClass(
                    class_name, oserv_body);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        System.out.println(call_method_manager_class.getName());
        try {
            object_instance = call_method_manager_class.newInstance();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        class_name = CALL_METHOD_SAMPLE_CLASS;
        oserv_body = CallMethodSampleData.getClassData();
        try {
            call_method_sample_class = 
                user_class_loader.getSpecifiedClass(
                    class_name, oserv_body);
            sample_object = call_method_sample_class.newInstance();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            call_method_sample_int_method = 
                call_method_sample_class.getDeclaredMethod(
                INT_METHOD,
                int_method_args_class );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            check_invoke_method =
                call_method_manager_class.getDeclaredMethod(
                CHECK_INVOKE,
                check_invoke_args_class );
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        check_invoke_args_object[0] = sample_object;

        check_invoke_args_object[1] =
            (Object)call_method_sample_int_method;

        int_method_args_object[0] = 1;
        check_invoke_args_object[2] = int_method_args_object;
        Object   rt;
        try {
            rt = check_invoke_method.invoke(
                object_instance,
                check_invoke_args_object);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        if (rt != null) {
        if ((rt instanceof Integer) == true) {
            System.out.println(String.valueOf ( ((Integer)rt).intValue() ) );
        } else {
            System.out.println(Object.class.getName() + rt.toString());
        }
        } else {
            System.out.println(RETURN_IS_NULL);
        }
    }
 
    public static void main(String[] args) {
        UserClassLoader  user_class_loader;

        user_class_loader = new UserClassLoader();
        CallMethodDebug2Test  call_method_debug2_test;
        call_method_debug2_test = new CallMethodDebug2Test();

        if (args == null || args.length < 0) {
            System.out.println(PARAMETER_ERROR);
            return;
        }
        call_method_debug2_test.execTestCase(user_class_loader);
    }
}
