/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     CallMethodWrapperTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/07/18
 *
 */

package oserv.test;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

import oserv.*;

public class CallMethodWrapperTest extends DebugCommon {
    private Class sample_class = CallMethodSample.class;

    private String TARGET_METHOD_NAME = "intMethod";
    private Class[] target_args = {
        int.class };
    private Method target_method = null;

    private String VALUE_HEADER = "Int value is ";

    public CallMethodWrapperTest() {
    }

    public void setMethod() throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        try {
            target_method = sample_class.getMethod(
                TARGET_METHOD_NAME, target_args);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        call_method = new CallMethod();
        target_class = call_method.getClass();

        call_method.setMethod(target_method);

        call_method_wrapper = new CallMethodWrapper(target_class);
        call_method_wrapper.setMethod(target_method);
    }

    public void invoke() throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;
        Object              object_instance;
        int                 int_value;

        try {
            target_method = sample_class.getMethod(
                TARGET_METHOD_NAME, target_args);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            object_instance = sample_class.newInstance();
        } catch (Exception exception_object) {
            throw exception_object;
        }

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method.setMethod(target_method);

        int_value = 17;
        call_method.setInt(object_instance, int_value, 0);
        try {
            call_method.invoke( object_instance ); 
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            int_value = call_method.getInt(object_instance);
            System.out.println(VALUE_HEADER + String.valueOf(int_value));
        } catch (Exception exception_object) {
            throw exception_object;
        } 

        call_method_wrapper = new CallMethodWrapper(target_class);
        call_method_wrapper.setMethod(target_method);
        int_value = 17;
        call_method_wrapper.setInt(object_instance, int_value, 0);
        try {
            call_method_wrapper.invoke( object_instance ); 
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            int_value = call_method_wrapper.getInt(object_instance);
            System.out.println(VALUE_HEADER + String.valueOf(int_value));
        } catch (Exception exception_object) {
            throw exception_object;
        } 
    }

    public void getParameterCount() throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();

        call_method_wrapper = new CallMethodWrapper(target_class);
    }

    public void getParameterType(int index) {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

    }

    public void getParameter(int index) {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;
    }

    public void setInt(Object obj, int arg, int index) throws Exception{
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        try {
            call_method.setInt(obj, arg, index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            call_method_wrapper.setInt(obj, arg, index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setLong(Object obj, long arg, int index) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        try {
            call_method.setLong(obj, arg, index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            call_method_wrapper.setLong(obj, arg, index);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void setFloat(Object obj, float arg, int index) {
    }

    public void setDouble(Object obj, double arg, int index) {
    }

    public void setBoolean(Object obj, boolean arg, int index) {
    }

    public void setByte(Object obj, byte arg, int index) {
    }

    public void setShort(Object obj, short arg, int index) {
    }

    public void setChar(Object obj, char arg, int index) {
    }

    public void set(Object obj, int arg, int index) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        call_method.set(obj, arg, index);
        call_method_wrapper.set(obj, arg, index);
    }

    public void getInt(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        int  int_rt_1;
        int  int_rt_2;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        int_rt_1 = call_method.getInt(obj);

        int_rt_2 = call_method_wrapper.getInt(obj);
    }

    public void getLong(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        long  long_rt_1;
        long  long_rt_2;

        long_rt_1 = call_method.getLong(obj);

        long_rt_2 = call_method_wrapper.getLong(obj);
    }

    public void getFloat(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        float  float_rt_1;
        float  float_rt_2;

        float_rt_1 = call_method.getFloat(obj);
        float_rt_2 = call_method_wrapper.getFloat(obj);
    }

    public void getDouble(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        double  double_rt_1;
        double  double_rt_2;

        double_rt_1 = call_method.getDouble(obj);
        double_rt_2 = call_method_wrapper.getDouble(obj);
    }

    public void getShort(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        short  short_rt_1;
        short  short_rt_2;

        short_rt_1 = call_method.getShort(obj);
        short_rt_2 = call_method_wrapper.getShort(obj);
    }

    public void getByte(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        byte   byte_rt_1;
        byte   byte_rt_2;

        byte_rt_1 = call_method.getByte(obj);
        byte_rt_2 = call_method_wrapper.getByte(obj);
    }

    public void getBoolean(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        boolean   boolean_rt_1;
        boolean   boolean_rt_2;

        boolean_rt_1 = call_method.getBoolean(obj);
        boolean_rt_2 = call_method_wrapper.getBoolean(obj);
    }

    public void getChar(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        char   char_rt_1;
        char   char_rt_2;

        char_rt_1 = call_method.getChar(obj);
        char_rt_2 = call_method_wrapper.getChar(obj);
    }

    public void get(Object obj) throws Exception {
        CallMethod          call_method;
        CallMethodWrapper   call_method_wrapper;
        Class               target_class;

        call_method = new CallMethod();
        target_class = call_method.getClass();
        call_method_wrapper = new CallMethodWrapper(target_class);

        Object   object_rt_1;
        Object   object_rt_2;

        object_rt_1 = call_method.get(obj);
        object_rt_2 = call_method_wrapper.get(obj);
    }

    public static String PARAMETER_ERROR = "Parameter error.";

    public static void main(String[] args) {
        CallMethodWrapperTest    call_method_wrapper_test;
        Method[]    method_list;
        Object[]    object_args = new Object[0];

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }

        call_method_wrapper_test = new CallMethodWrapperTest();

        method_list = CallMethodWrapperTest.class.getDeclaredMethods();
        for (int i = 0; i < method_list.length; i++) {
            if (method_list[i].getName().compareTo(args[0]) == 0) {
                try {
                    method_list[i].invoke(call_method_wrapper_test,
                        object_args);
                } catch (Exception exception_object) {
                    exception_object.printStackTrace();
                    break;
                }
                break;   
            }
        }
    }
}

