/**
 * Copyright (C) All rights reserved.
 *
 * @Module     UserClassLoaderTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/07/09
 *
 **/

package oserv.test;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

import oserv.*;

public class UserClassLoaderTest extends UserClassLoader  {
    private static String OSERV_TEST_USER_CLASS_LOADER_SAMPLE
        = "oserv.test.UserClassLoaderSample";
    private static String CLASS_NAME = "Class";
    private static String CHECK_CLASS_NAME = "Check Class";
    private static String SPACE = " ";

    public Class readUserClass(String filepath, String main_class) throws Exception {
        int                     idx;
        Class                   cls;
        Class                   cls2;
        String                  class_name;
        FileInputStream         file_input_stream;
        JarInputStream          jar_input_stream;
        ZipEntry                zip_entry;

        try {
            cls = super.readUserClass(filepath, main_class);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        try {
            file_input_stream = new FileInputStream(filepath);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        try {
            jar_input_stream = new JarInputStream(file_input_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        for (    zip_entry = jar_input_stream.getNextEntry();
                 zip_entry != null;
                 zip_entry = jar_input_stream.getNextEntry() ) {

            class_name = zip_entry.getName();
            idx = class_name.lastIndexOf(CLASS_SUFFIX);
            if (idx <= 0) {
                continue;
            }
            class_name = class_name.substring(0, idx);
            class_name = replace(class_name, File.separator, DOT_STRING);
System.out.println(CHECK_CLASS_NAME + SPACE + class_name );
            try {
                cls2 = Class.forName(class_name,
                    true,
                   (ClassLoader)cls.getClassLoader());
            } catch (Exception exception_object) {
                throw exception_object;
            }
System.out.println(CLASS_NAME + SPACE + cls2.getName() + SPACE +
cls2.getClassLoader().getClass().getName() );
        }

        return cls;
    }

    public Class getSpecifiedClass(String class_name, byte[] oserv_body) throws Exception {
        Class cls;

        try {
            cls = super.getSpecifiedClass(class_name, oserv_body);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return cls;
    }

    public static void main(String[] args) {
        Class                cls;
        Class                cls2;
        UserClassLoaderTest  user_class_loader_test;

        user_class_loader_test = new UserClassLoaderTest();

        if (args.length == 0) {
            try {
                cls = user_class_loader_test.getSpecifiedClass(
                    OSERV_TEST_USER_CLASS_LOADER_SAMPLE,
                    oserv.test.UserClassLoaderSampleData.getClassData());
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
            System.out.println(CLASS_NAME + SPACE + cls.getName()
                + SPACE + cls.getClassLoader().getClass().getName());

            try {
                cls2 = Class.forName(OSERV_TEST_USER_CLASS_LOADER_SAMPLE,
                    true,
                   (ClassLoader)cls.getClassLoader());
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
            System.out.println(CLASS_NAME + SPACE + cls2.getName()
                + SPACE + cls2.getClassLoader().getClass().getName());
        }
        if (args.length == 2) {
            try {
                cls = user_class_loader_test.readUserClass(
                    args[0],  args[1]);
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
            System.out.println(CLASS_NAME + SPACE + cls.getName()
                + SPACE + cls.getClassLoader().getClass().getName());
        }
    }
}

