/**
 * Copyright (C) All rights reserved.
 *
 * @Module     VariableTest.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2013/07/08
 *
 */

package oserv.test;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;

import oserv.*;

public class VariableTest {

    private Object object_instance;
    private Class  object_class;

    private static String SPACE = " ";

    public void setupTarget(String class_name) throws Exception {

        try {
            object_class = Class.forName(class_name);
        } catch (Exception exception_object) {
            throw exception_object;
        }

        try {
            object_instance = object_class.newInstance(); 
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void readVariable(String variable_file) throws Exception {
        ReadVariable     read_variable;

        read_variable = new ReadVariable();

        try {
            read_variable.readVariable(variable_file,
                object_class, object_instance);              
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void writeVariable(String variable_file) throws Exception {
        WriteVariable    write_variable;

        write_variable = new WriteVariable();

        try {
            write_variable.writeVariable(variable_file,
                object_class, object_instance);              
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    public void showVariable() throws Exception {
        Field[]    variable_fields;

        try {
            variable_fields = object_class.getFields();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        for (int i = 0; i < variable_fields.length; i++) {
            if (variable_fields[i].getType().getName().compareTo(
                int.class.getName()) == 0) {
                int  int_value;

                try {
                    int_value = variable_fields[i].getInt(object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(int_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                long.class.getName()) == 0) {
                long  long_value;
                try {
                    long_value = variable_fields[i].getLong(object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(long_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                float.class.getName()) == 0) {
                float  float_value;
                try {
                    float_value = variable_fields[i].getFloat(object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(float_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                double.class.getName()) == 0) {
                double  double_value;
                try {
                    double_value = variable_fields[i].getDouble(
                        object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(double_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                short.class.getName()) == 0) {
                short  short_value;
                try {
                    short_value = variable_fields[i].getShort(
                        object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(short_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                boolean.class.getName()) == 0) {
                boolean  boolean_value;
                try {
                    boolean_value = variable_fields[i].getBoolean(
                        object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(boolean_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                byte.class.getName()) == 0) {
                byte  byte_value;
                try {
                    byte_value = variable_fields[i].getByte(
                        object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(byte_value) );
                continue;
            }
            if (variable_fields[i].getType().getName().compareTo(
                char.class.getName()) == 0) {
                char  char_value;
                try {
                    char_value = variable_fields[i].getChar(
                        object_instance);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                System.out.println(
                    variable_fields[i].getName() + SPACE +
                    variable_fields[i].getType().getName() + SPACE +
                    String.valueOf(char_value) );
                continue;
            }
            Object   local_object;
            try {
                local_object = variable_fields[i].get(object_instance);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            String message = "object is null";
            if (local_object != null) {
                message = local_object.toString();
            }
            System.out.println(
                variable_fields[i].getName() + SPACE +
                variable_fields[i].getType().getName() + SPACE +
                message );
        }
    }

    public void executeMethod(String method_name) throws Exception {
        Method  method_list;
        Class[]   args_list;
        Object[]  args_object;

        args_list   = new Class[0];
        args_object = new Object[0];
        try {
            method_list = object_class.getMethod(method_name, args_list);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        try {
            method_list.invoke(object_instance, args_object);
        } catch (Exception exception_object) {
            throw exception_object;
        }
    }

    private static String PARAMETER_ERROR = "Parameter error";


    public static void main(String[] args) {
        VariableTest   variable_test;

        if (args.length != 3 || args[0] == null || 
            args[1] == null || args[2] == null) {
            System.err.println(PARAMETER_ERROR);
            return;
        }

        variable_test = new VariableTest();

        try {
            variable_test.setupTarget(args[0]);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            variable_test.readVariable(args[1]);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            variable_test.showVariable();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            variable_test.executeMethod(args[2]);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            variable_test.writeVariable(args[1]);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            variable_test.showVariable();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        
    }
}
