/**
 * Copyright (C) All rights reserved.
 *
 * @Module     CallMethod.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/08/09
 *             2.1 2011/12/22 import from Exec sample.
 */

package server;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

import exec.*;

public class CallMethod extends AccessibleObject implements java.lang.reflect.GenericDeclaration, java.lang.reflect.Member {
    private Method     method_object;

    private String TRACE_CLASSNAME  = "oserv.CallMethod";
    private String TRACE_CALLMETHOD = "<init>";
    private String TRACE_SETPARAMETER = "setParameter";
    private String TRACE_INVOKE = "invoke";

    private Class      return_type;
    private Class[]    parameter_classes;
    private boolean[]  parameter_validation;
    private Object[]   parameter_box;
    private boolean    call_flag;

    private int      int_parameter_value;
    private long     long_parameter_value;
    private char     char_parameter_value;
    private short    short_parameter_value;
    private byte     byte_parameter_value;
    private boolean  boolean_parameter_value;
    private float    float_parameter_value;
    private double   double_parameter_value;
    private Object   object_parameter_value;

    private int      int_return_value;
    private char     char_return_value;
    private short    short_return_value;
    private long     long_return_value;
    private float    float_return_value;
    private double   double_return_value;
    private boolean  boolean_return_value;
    private byte     byte_return_value;
    private Object   object_return_value;

    public final static String VOID_TYPE = "void";
    public final static String INT_TYPE = "int";
    public final static String LONG_TYPE = "long";
    public final static String FLOAT_TYPE = "float";
    public final static String DOUBLE_TYPE = "double";
    public final static String CHAR_TYPE = "char";
    public final static String SHORT_TYPE = "short";
    public final static String BYTE_TYPE = "byte";
    public final static String BYTE_ARRAY_TYPE = "byte[]";
    public final static String BOOLEAN_TYPE = "boolean";

    public static String SPACE = " ";

    public static String USAGE = "Usage: CallMethod ClassName MethodName";
    public static String ILLEGAL_METHOD_NAME = "Illegal method name";
    public static String INSTANCE_CREATION_ERROR = "Instance creation error.";
    public static String CONSTRUCTION_ERROR = "Construction error.";

    public final static String ILLEGAL_ARGUMENT_EXCEPTION = "Illegal Argument";
    public final static String ILLEGAL_RETURN_VALUE_EXCEPTION = "Bad Return Value";
    public final static String INTERNAL_ERROR_EXCEPTION
        = "Internal error exception";

    public static String INT_ERROR = "Int method error";
    public static String LONG_ERROR = "Long method error";
    public static String FLOAT_ERROR = "Float method error";
    public static String DOUBLE_ERROR = "Double method error";
    public static String SHORT_ERROR = "Short method error";
    public static String BYTE_ERROR = "Byte method error";
    public static String CHAR_ERROR = "Char method error";
    public static String BOOLEAN_ERROR = "Boolean method error";
    public static String CALL_METHOD = "CallMethod";
    public static String CALL_METHOD_SAMPLE = "oserv.CallMethodSample";

    public static String LIMIT_KEY = "Limit";
    public static String VALUE_KEY = "value";

    public final static String DEBUG_COUNT_KEY      = "Count";
    public final static String DEBUG_BLANK_STRING_KEY     = " ";

    public static String INT_TEST = "intMethod";
    public static String LONG_TEST = "longMethod";
    public static String FLOAT_TEST = "floatMethod";
    public static String DOUBLE_TEST = "doubleMethod";
    public static String SHORT_TEST = "shortMethod";
    public static String CHAR_TEST = "charMethod";
    public static String BYTE_TEST = "byteMethod";
    public static String BOOLEAN_TEST = "booleanMethod";
    public static String OBJECT_TEST = "objectMethod";
    public static String BYTE_ARRAY_TEST = "byteArrayMethod";

    public int debug = 0;
    public LogPrintStream logStream = null;
    private static MessageResource message_resource = new MessageResource();

    public CallMethod(Method method) throws Exception {
        super();
        if (method == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception (
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        method_object = method;
        parameter_classes = method_object.getParameterTypes();
        parameter_box = new Object[parameter_classes.length];
        parameter_validation = new boolean[parameter_classes.length];
        for (int i = 0; i < parameter_classes.length; i++) {
            parameter_validation[i] = false;
        }
        call_flag = false;
        return_type = method.getReturnType();
    }

    public int getParameterCount() throws Exception {
        return parameter_box.length;
    }

    public String getParameterType(int index) throws Exception {
        return parameter_classes[index].getName();
    }

    public Object getParameter(int index) throws Exception {
        return parameter_box[index];
    }

    public void setDebug(int debug_int) {
        debug = debug_int;
    }
    public int getDebug() {
        return debug;
    }

    public void setLogStream(LogPrintStream param_log) {
        logStream = param_log;
    }
    public LogPrintStream getLogStream() {
        return logStream;
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }
    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setParameter(Object obj, int index) throws Exception {
        call_flag = false;

        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        if (parameter_classes.length <= index) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        if (parameter_classes[index].getName().compareTo(BYTE_TYPE) == 0) {

            parameter_box[index] = (Object)byte_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(CHAR_TYPE) == 0) {

            parameter_box[index] = (Object)char_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(INT_TYPE) == 0) {

            parameter_box[index] = (Object)int_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(LONG_TYPE) == 0) {

            parameter_box[index] = (Object)long_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(SHORT_TYPE) == 0) {
            parameter_box[index] = (Object)short_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(BOOLEAN_TYPE) == 0) {
            parameter_box[index] = (Object)boolean_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(FLOAT_TYPE) == 0) {
            parameter_box[index] = (Object)float_parameter_value;
            parameter_validation[index] = true;
            return;
        }
        if (parameter_classes[index].getName().compareTo(DOUBLE_TYPE) == 0) {
            parameter_box[index] = (Object)double_parameter_value;
            parameter_validation[index] = true;
            return;
        }

        parameter_box[index] = object_parameter_value;
        parameter_validation[index] = true;
    }

    public void setcharParameter(Object obj, char arg, int index) throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        char_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setintParameter(Object obj, int arg, int index) throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        int_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setlongParameter(Object obj, long arg, int index) throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        long_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setshortParameter(Object obj, short arg, int index) throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        short_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setfloatParameter(Object obj, float arg, int index) throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        float_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setdoubleParameter(Object obj, double arg, int index) throws 
        Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        double_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setbyteParameter(Object obj, byte arg, int index) 
                        throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        byte_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setbooleanParameter(Object obj, boolean arg, int index) 
                        throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;

            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        boolean_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void setObjectParameter(Object obj, Object arg, int index) throws Exception {
        if (method_object == null) {
            Exception illegal_argument_exception;

            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        object_parameter_value = arg;
        setParameter(obj, index) ;
    }

    public void invoke(Object object) throws Exception {
        Object rt;
        Exception invocation_real_exception;
        Exception real_exception;
        Exception invocation_target_exception;
        Exception illegal_argument_exception;
        Exception illegal_return_value_exception;

        if (method_object == null) {
            illegal_argument_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    ILLEGAL_ARGUMENT_EXCEPTION,
                    getMessageResource().getMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION)));
            throw illegal_argument_exception;
        }
        for (int i = 0; i < parameter_validation.length; i++) {
            if (parameter_validation[i] == false) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            }
        }
        if (getDebug() > 0) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        DEBUG_COUNT_KEY,
                        getMessageResource().getMessage(
                            DEBUG_COUNT_KEY),
                        DEBUG_BLANK_STRING_KEY,
                        String.valueOf(parameter_box.length)));
            }
        }
        if (return_type.getName().compareTo(VOID_TYPE) == 0) {
            try {
                method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
        } else
        if (return_type.getName().compareTo(INT_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
           /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            int_return_value = (int)Integer.valueOf(rt.toString()).intValue();
        } else 
        if (return_type.getName().compareTo(LONG_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            long_return_value = (long)Long.valueOf(rt.toString()).longValue();
        } else 
        if (return_type.getName().compareTo(FLOAT_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            float_return_value = (float)Float.valueOf(rt.toString()).floatValue();
        } else 
        if (return_type.getName().compareTo(DOUBLE_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            double_return_value = (double)Double.valueOf(rt.toString()).doubleValue();
        } else 
        if (return_type.getName().compareTo(BYTE_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            byte_return_value = (byte)Byte.valueOf(rt.toString()).byteValue();
        } else 
        if (return_type.getName().compareTo(BOOLEAN_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            boolean_return_value = (boolean)Boolean.valueOf(rt.toString()).booleanValue();
        } else 
        if (return_type.getName().compareTo(CHAR_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            if ((rt instanceof Character) == true) {
                char_return_value = ((Character)rt).charValue();
            } else {
                char_return_value = (char)Integer.valueOf(rt.toString()).intValue();
            }
        } else 
        if (return_type.getName().compareTo(SHORT_TYPE) == 0) {
            try {
                rt = method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            /*
            if (rt == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
            */
            short_return_value = (short)Short.valueOf(rt.toString()).shortValue();
        } else {
            try {
                object_return_value = (Object)method_object.invoke(object, parameter_box);
            } catch (IllegalArgumentException exception_object) {
                illegal_argument_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_ARGUMENT_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_ARGUMENT_EXCEPTION)));
                throw illegal_argument_exception;
            } catch (InvocationTargetException exception_object) {
                throw exception_object;
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (object_return_value == null) {
                illegal_return_value_exception = new Exception(
                    getMessageResource().getFormattedMessage(
                        ILLEGAL_RETURN_VALUE_EXCEPTION,
                        getMessageResource().getMessage(
                            ILLEGAL_RETURN_VALUE_EXCEPTION)));
                throw illegal_return_value_exception;
            }
        }
        call_flag = true;
        for (int i = 0; i < parameter_validation.length; i++) {
            parameter_validation[i] = false;
        }
    }
    
    public int getintValue(Object obj) throws Exception {
        Exception  object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(INT_TYPE) == 0) {
            return int_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public long getlongValue(Object obj) throws Exception {
        Exception  object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(LONG_TYPE) == 0) {
            return long_return_value;
        }

        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public float getfloatValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(FLOAT_TYPE) == 0) {
            return float_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public double getdoubleValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(DOUBLE_TYPE) == 0) {
            return double_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public short getshortValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(SHORT_TYPE) == 0) {
            return short_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public char getcharValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(CHAR_TYPE) == 0) {
            return char_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public boolean getbooleanValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(BOOLEAN_TYPE) == 0) {
            return boolean_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public byte getbyteValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(BYTE_TYPE) == 0) {
            return byte_return_value;
        }
        object_server_exception = new Exception(
            getMessageResource().getFormattedMessage(
                INTERNAL_ERROR_EXCEPTION,
                getMessageResource().getMessage(
                    INTERNAL_ERROR_EXCEPTION)));
        throw object_server_exception;
    }

    public Object getObjectValue(Object obj) throws Exception {
        Exception    object_server_exception;
        if (call_flag == false) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        } 
        if (return_type.getName().compareTo(VOID_TYPE) == 0
         || return_type.getName().compareTo(BYTE_TYPE) == 0
         || return_type.getName().compareTo(BOOLEAN_TYPE) == 0
         || return_type.getName().compareTo(INT_TYPE) == 0
         || return_type.getName().compareTo(LONG_TYPE) == 0
         || return_type.getName().compareTo(DOUBLE_TYPE) == 0
         || return_type.getName().compareTo(FLOAT_TYPE) == 0
         || return_type.getName().compareTo(SHORT_TYPE) == 0
         || return_type.getName().compareTo(CHAR_TYPE) == 0
        ) {
            object_server_exception = new Exception(
                getMessageResource().getFormattedMessage(
                    INTERNAL_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        INTERNAL_ERROR_EXCEPTION)));
            throw object_server_exception;
        }
        return object_return_value;
    }

    /**
     * native Methods of reflect.Method class.
     */
    public java.lang.Class getDeclaringClass() {
        return method_object.getDeclaringClass();
    }

    public String getName() {
        return method_object.getName();
    }

    public int getModifiers() {
        return method_object.getModifiers();
    }

    public java.lang.reflect.TypeVariable[] getTypeParameters() {
        return method_object.getTypeParameters();
    }

    public java.lang.Class getReturnType() {
        return method_object.getReturnType();
    }

    public java.lang.reflect.Type getGenericReturnType() {
        return method_object.getGenericReturnType();
    }

    public java.lang.Class[] getParameterTypes() {
        return method_object.getParameterTypes();
    }

    public java.lang.reflect.Type[] getGenericParameterTypes() {
        return method_object.getGenericParameterTypes();
    }

    public java.lang.Class[] getExceptionTypes() {
        return method_object.getExceptionTypes();
    }

    public java.lang.reflect.Type[] getGenericExceptionTypes() {
        return method_object.getGenericExceptionTypes();
    }

    public boolean equals(java.lang.Object param) {
        return method_object.equals(param);
    }

    public int hashCode() {
        return method_object.hashCode();
    }

    public String toString() {
        return method_object.toString();
    }

    public java.lang.String toGenericString() {
        return method_object.toGenericString();
    }

    public boolean isBridge() {
        return method_object.isBridge();
    }

    public boolean isVarArgs() {
        return method_object.isVarArgs();
    }

    public boolean isSynthetic() {
        return method_object.isSynthetic();
    }

    public java.lang.annotation.Annotation getAnnotation(java.lang.Class cls) {
        return method_object.getAnnotation(cls);
    }

    public java.lang.annotation.Annotation[] getDeclaredAnnotations() {
        return method_object.getDeclaredAnnotations();
    }

    public java.lang.Object getDefaultValue() {
        return method_object.getDefaultValue();
    }

    public java.lang.annotation.Annotation[][] getParameterAnnotations() {
        return method_object.getParameterAnnotations();
    }


    public static void main(String[] args) {
        int        index;
        String     target;
        Class      test_class;
        Object     object;
        Method[]   test_methods;
        CallMethod call_method_object;

        try {
            test_class = Class.forName(CALL_METHOD_SAMPLE);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        target = null;
        if (args[0].compareTo(INT_TYPE) == 0) {
            target = INT_TEST;
        } else if (args[0].compareTo(LONG_TYPE) == 0) {
            target = LONG_TEST;
        } else if (args[0].compareTo(FLOAT_TYPE) == 0) {
            target = FLOAT_TEST;
        } else if (args[0].compareTo(DOUBLE_TYPE) == 0) {
            target = DOUBLE_TEST;
        } else if (args[0].compareTo(SHORT_TYPE) == 0) {
            target = SHORT_TEST;
        } else if (args[0].compareTo(BYTE_TYPE) == 0) {
            target = BYTE_TEST;
        } else if (args[0].compareTo(BYTE_ARRAY_TYPE) == 0) {
            target = BYTE_ARRAY_TEST;
        } else if (args[0].compareTo(BOOLEAN_TYPE) == 0) {
            target = BOOLEAN_TEST;
        } else if (args[0].compareTo(CHAR_TYPE) == 0) {
            target = CHAR_TEST;
        } else {
            target = OBJECT_TEST;
        }

        test_methods = test_class.getMethods();
        for (index = 0; index < test_methods.length; index++) {
            if (test_methods[index].getName().compareTo(target) == 0) {
                break;
            }
        }
        if (index >= test_methods.length) {
            System.err.println(ILLEGAL_METHOD_NAME);
            return;
        } 
        try {
            object = test_class.newInstance();
        } catch (Exception exception_object) {
            System.err.println(INSTANCE_CREATION_ERROR);
            return;
        }

        try {
            call_method_object = new CallMethod(test_methods[index]);
        } catch (Exception exception_object) {
            System.err.println(INSTANCE_CREATION_ERROR);
            return;
        }

        if (target.compareTo(INT_TEST) == 0) {
            try {
                int value;
                int return_value;
                value = Integer.valueOf(args[1]).intValue();
                index = 0;
                call_method_object.setintParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getintValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value)
                );
                return;
            } catch (Exception exception_object) {
                System.err.println(INT_ERROR);
                return;
            }
        }
        if (target.compareTo(LONG_TEST) == 0) {
            try {
                long value;
                long return_value;
                value = Long.valueOf(args[1]).longValue();
                index = 0;
                call_method_object.setlongParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getlongValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value)
                );
                return;
            } catch (Exception exception_object) {
                System.err.println(LONG_ERROR);
                return;
            }
        }
        if (target.compareTo(FLOAT_TEST) == 0) {
            try {
                float value;
                float return_value;
                value = Float.valueOf(args[1]).floatValue();
                index = 0;
                call_method_object.setfloatParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getfloatValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value));
                return;
            } catch (Exception exception_object) {
                System.err.println(FLOAT_ERROR);
                return;
            }
        }
        if (target.compareTo(DOUBLE_TEST) == 0) {
            try {
                double value;
                double return_value;
                value = Double.valueOf(args[1]).doubleValue();
                index = 0;
                call_method_object.setdoubleParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getdoubleValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value));
                return;
            } catch (Exception exception_object) {
                System.err.println(DOUBLE_ERROR);
                return;
            }
        }
        if (target.compareTo(SHORT_TEST) == 0) {
            try {
                short value;
                short return_value;
                value = Short.valueOf(args[1]).shortValue();
                index = 0;
                call_method_object.setshortParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getshortValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value));
                return;
            } catch (Exception exception_object) {
                System.err.println(SHORT_ERROR);
                return;
            }
        }
        if (target.compareTo(CHAR_TEST) == 0 && args[1].length() > 0) {
            try {
                char value;
                char return_value;
                value = args[1].charAt(0);
                index = 0;
                call_method_object.setcharParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getcharValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value));
                return;
            } catch (Exception exception_object) {
                System.err.println(CHAR_ERROR);
                return;
            }
        }
        if (target.compareTo(BYTE_TEST) == 0) {
            try {
                byte value;
                byte return_value;
                value = (byte)Byte.valueOf(args[1]).byteValue();
                index = 0;
                call_method_object.setbyteParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getbyteValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value));
                return;
            } catch (Exception exception_object) {
                System.err.println(BYTE_ERROR);
                return;
            }
        }
        if (target.compareTo(BOOLEAN_TEST) == 0) {
            try {
                boolean value;
                boolean return_value;
                value = (boolean)Boolean.valueOf(args[1]).booleanValue();
                index = 0;
                call_method_object.setbooleanParameter(object, value, index);
                call_method_object.invoke(object);
                return_value = call_method_object.getbooleanValue(object);
                System.out.println(
                    String.valueOf(value)
                    + SPACE
                    + String.valueOf(return_value));
                return;
            } catch (Exception exception_object) {
                System.err.println(BOOLEAN_ERROR);
                return;
            }
        }
        if (target.compareTo(BYTE_ARRAY_TEST) == 0) {
            try {
                int    limit;
                byte[] value;
                byte[] return_value;
                limit = (byte)Integer.valueOf(args[1]).intValue();
                value = new byte[limit];
                for (int i = 0; i < limit; i++) {
                    value[i] = (byte)i;
                }
                index = 0;
                call_method_object.setObjectParameter(object, (Object)value, index);
                call_method_object.invoke(object);
                return_value = (byte[])call_method_object.getObjectValue(object);
                System.out.println(LIMIT_KEY
                    + String.valueOf(return_value.length)
                );
                for (int i = 0; i < limit; i++) {
                    System.out.println(VALUE_KEY
                        + String.valueOf(return_value[i]));
                }
                return;
            } catch (Exception exception_object) {
                System.err.println(BYTE_ERROR);
                return;
            }
        }
    }
}
