/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ExecuteServer.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    1.0 2012/05/15
 */

package server;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;

import exec.*;

public class ExecuteServer extends java.lang.ClassLoader implements ExecuteService {
    private static String EMPTY = "";
    private static String SPACE = " ";
    private static String PARENT = "..";
    private static String TMPDIR = "tmp";
    private static String PATH_KEY = "Path";
    private static String NOT_FOUND_EXCEPTION = "Not found exception.";
    private static String PARAMETER_ERROR_EXCEPTION = "Parameter error exception";
    private static String FILE_NAME_ERROR  = "File not found";
    private static String CLASS_NAME_ERROR = "Class not found";
    private static String METHOD_NAME_ERROR = "Method not found";
    private static String OUTPUT_NAME_ERROR = "Output not found";
    private static String JAR_NAME_ERROR = "Jar not found";
    public final static String SUFFIX_SEPARATOR = ".";
    private String CLASS_SUFFIX = ".class";
    private String DOT_STRING = ".";
    public final static String JAR_SUFFIX = "jar";

    private static String LOADCLASSFROMJAR = "loadClassFromJar";
    private static String EXECUTE = "execute";
    private static String SETUPDEBUG = "setupDebug";

    public final static String JAR_URL_HEAD = "jar:file:";
    public final static String JAR_URL_END  = "!/";

    private static String EXECUTE_METHOD_NAME = "exec";
    private static String MAIN_METHOD_NAME = "main";

    private static String PARAMETER_ERROR = "Parameter error";

    private static String DEBUG_VALUE_STRING = "debug_value";
    private static String MESSAGE_RESOURCE_STRING = "message_resource";
    private static String LOG_PRINT_STREAM = "log_print_stream";

    public static int debug_value = 0;
    public static MessageResource message_resource = null;
    private static LogPrintStream  log_print_stream = null;

    public void setDebug(int debug) {
        debug_value = debug;
    }

    public int getDebug() {
        return debug_value;
    }

    public void setMessageResource( MessageResource param_message_resource ) {
        message_resource = param_message_resource;
    }

    public MessageResource getMessageResource() {
        return message_resource;
    }

    public void setLogPrintStream( LogPrintStream param_log_print_stream ) {
        log_print_stream = param_log_print_stream;
    }

    public LogPrintStream  getLogPrintStream() {
        return log_print_stream;
    }

    public boolean setupDebug(int level) throws Exception {
        boolean  flag;
        String   message;

        flag = true;
        if (debug_value == 0) {
            flag = false;
        }

        setDebug(level);
        if (getLogPrintStream() == null) {
            setLogPrintStream( new LogPrintStream( System.err ) ) ;
        }
        if (message_resource == null) {
            message_resource = new MessageResource();
            try {
                message_resource.setLanguage(Locale.JAPAN.getISO3Language());
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        message = message_resource.getFormattedMessage(
            MessageText.DEBUG_SETUP_MESSAGE,
            message_resource.getMessage( 
                    MessageText.DEBUG_SETUP_MESSAGE ),
            SPACE,
            String.valueOf(level) );
        getLogPrintStream().printLogln( message );

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = message_resource.getFormattedMessage(
                MessageText.END_METHOD,
                message_resource.getMessage( 
                    MessageText.END_METHOD ),
                SPACE,
                SETUPDEBUG);
            getLogPrintStream().printLogln( message );
        }
        return flag;
    }

    public void OServ() {
System.out.println("EXECUTE OSERV **********************************");
    }

    public String replace(String path_name, String path_key, String class_key) {
        int    idx;
        String work;
        String class_string;

        if (path_name == null || 
            path_key  == null ||
            class_key == null ) {
            return null;
        }
 
        class_string = new String();
        work = path_name;
        while (true) {
            idx = work.indexOf(path_key);
            if (idx < 0) {
                class_string = class_string.concat(work);
                break;
            }
            class_string = class_string.concat(
                work.substring(0, idx) + class_key);
            work = work.substring(idx+1);
        }
        return class_string;
    }

    public Class loadClassFromJar(FileInputStream  file_input_stream, String target_name) throws Exception {
        int                     rt;
        int                     idx;
        byte[]                  buffer;
        byte[]                  class_body;
        Class                   cls;
        Class                   class_object;
        String                  message;
        JarInputStream          jar_input_stream;
        ByteArrayOutputStream   byte_array_output_stream;
        ZipEntry                zip_entry;
        String                  class_name;

        if (file_input_stream == null) {
            Exception  exception_object;

            message = message_resource.getFormattedMessage(
                MessageText.ARGUMENT_PARAMETER_EXCEPTION,
                message_resource.getMessage( 
                    MessageText.ARGUMENT_PARAMETER_EXCEPTION ) );
/**
            exception_object = new Exception(PARAMETER_ERROR_EXCEPTION);
**/
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = message_resource.getFormattedMessage(
                MessageText.ENTER_METHOD,
                message_resource.getMessage( 
                    MessageText.ENTER_METHOD ),
                SPACE,
                LOADCLASSFROMJAR);
            getLogPrintStream().printLogln( message );
        }
        try {
            jar_input_stream = new JarInputStream( file_input_stream );
        } catch (Exception exception_object) {
            if (getLogPrintStream() != null) {
                message = message_resource.getFormattedMessage(
                    MessageText.DEBUG_EXCEPTION_KEY,
                    message_resource.getMessage( 
                        MessageText.DEBUG_EXCEPTION_KEY ),
                    SPACE,
                    exception_object.toString() );
                getLogPrintStream().printLogln( message );
            }
            throw exception_object;
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = message_resource.getFormattedMessage(
                MessageText.DEBUG_VALUE_KEY,
                message_resource.getMessage( 
                    MessageText.DEBUG_VALUE_KEY ),
                SPACE,
                jar_input_stream.toString() );
            getLogPrintStream().printLogln( message );
        }
        class_object = null;
        buffer = new byte[4096];
        for (    zip_entry = jar_input_stream.getNextEntry();
                 zip_entry != null;
                 zip_entry = jar_input_stream.getNextEntry() ) {

            class_name = zip_entry.getName();
            idx = class_name.lastIndexOf(CLASS_SUFFIX);
            if (idx <= 0) {
                continue;
            }
            class_name = class_name.substring(0, idx);
            if (getDebug() > 0 && getLogPrintStream() != null) {
                message = message_resource.getFormattedMessage(
                    MessageText.DEBUG_VALUE_KEY,
                    message_resource.getMessage( 
                        MessageText.DEBUG_VALUE_KEY ),
                    SPACE,
                    class_name );
                getLogPrintStream().printLogln( message );
            }
            
            byte_array_output_stream = new ByteArrayOutputStream();
            while (true) {
                try {
                    rt = jar_input_stream.read(buffer, 0, buffer.length);
                } catch (Exception exception_object) {
                    throw exception_object;
                }
                if (rt == -1) {
                    break;
                }
/*
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message = message_resource.getFormattedMessage(
                        MessageText.DEBUG_READ_OPERATION_KEY,
                        message_resource.getMessage( 
                            MessageText.DEBUG_READ_OPERATION_KEY ),
                        SPACE,
                        String.valueOf(rt)  );
                    getLogPrintStream().printLogln( message );
                }
*/
                byte_array_output_stream.write(buffer, 0, rt);
/*
                if (getDebug() > 0 && getLogPrintStream() != null) {
                    message = message_resource.getFormattedMessage(
                        MessageText.DEBUG_WRITE_OPERATION_KEY,
                        message_resource.getMessage( 
                            MessageText.DEBUG_WRITE_OPERATION_KEY ),
                        SPACE,
                        String.valueOf(rt)  );
                    getLogPrintStream().printLogln( message );
                }
*/
            }
            class_body = byte_array_output_stream.toByteArray();
            class_name = replace(class_name, File.separator, DOT_STRING);
            try {
                cls = Class.forName(class_name);
                if (class_name.compareTo(target_name) == 0) {
                    class_object = cls;
                }
                continue;
            } catch (Exception exception_object) {
            }
            try {
                cls = defineClass(class_name, class_body, 0, class_body.length);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (class_name.compareTo(target_name) == 0) {
                class_object = cls;
            }
        }
        if (getDebug() > 0 && getLogPrintStream() != null) {
            message = message_resource.getFormattedMessage(
                MessageText.END_METHOD,
                message_resource.getMessage( 
                    MessageText.END_METHOD ),
                SPACE,
                LOADCLASSFROMJAR);
            getLogPrintStream().printLogln( message );
        }
        return class_object;
    }

    public void execute(String method_name, String class_name, FileInputStream file_input, byte[] param_byte, PrintStream print_stream,
        MessageResource  message_resource_object
    ) throws Exception {
        int                idx;
        Method[]           mth;
        URL                url;
        URL[]              urls;
        File               fd;
        Class              cls;
        Object             obj;
        Object             robj;
        Object[]           obj_args;
        FileOutputStream   file_output_stream;
        String             line;
        String             filepath;
        String             message;
        String[]           args;
        String[]           bk;
        Field                 field;
        MessageResource    message_resource;
        URLClassLoader     uloader;
        PrintStream           save_stdout;
        ByteArrayInputStream  byte_array_input_stream;
        DataInputStream       data_input_stream;
        FileInputStream       file_input_stream;
        DecodeInputStream     decode_input_stream;
        CallMethod            call_method_object;

System.err.println("Execute");
        if (message_resource_object != null) {
            setMessageResource( message_resource_object ) ;
        }

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_resource = getMessageResource();
            if (message_resource == null) {
                message_resource = new MessageResource();
            }
            message = message_resource.getFormattedMessage(
                MessageText.ENTER_METHOD,
                message_resource.getMessage( 
                    MessageText.ENTER_METHOD ),
                SPACE,
                EXECUTE);
            getLogPrintStream().printLogln( message );
        }
        if (class_name == null) {
            Exception  exception_object;
            message_resource = getMessageResource();
            if (message_resource == null) {
                message_resource = new MessageResource();
            }
            message = message_resource.getFormattedMessage(
                CLASS_NAME_ERROR,
                message_resource.getMessage( CLASS_NAME_ERROR ));
            exception_object = new Exception( message );
            throw exception_object;
        }
        if (method_name == null) {
            Exception  exception_object;
            message_resource = getMessageResource();
            if (message_resource == null) {
                message_resource = new MessageResource();
            }
            message = message_resource.getFormattedMessage(
                METHOD_NAME_ERROR,
                message_resource.getMessage( METHOD_NAME_ERROR ));
            exception_object = new Exception( message );
            throw exception_object;
        }
System.err.println("Method");
        if (getDebug() == 0) {
Exception exception_object;
exception_object = new Exception();
throw exception_object;
        }
        try {
            cls = loadClassFromJar(file_input, class_name);
        } catch (Exception exception_object) {
            /**
            exception_object.printStackTrace();
            **/
            throw exception_object;
        }
        try {
            obj = cls.newInstance();
        } catch (Exception exception_object) {
            /**
            exception_object.printStackTrace();
            **/
            throw exception_object;
        }
     
/*
        if ((obj instanceof DebugObject) == true) {
            ((DebugObject)obj).setDebug(getDebug());
            ((DebugObject)obj).setMessageResource(getMessageResource());
            ((DebugObject)obj).setLogPrintStream(getLogPrintStream());
        }
*/
        args = new String[0];
        byte_array_input_stream = new ByteArrayInputStream(param_byte);
        decode_input_stream = new DecodeInputStream( byte_array_input_stream );

        while (true) {
            try {
                line = decode_input_stream.readCodeLine();
            } catch (Exception exception_object) {
                break;
            }
            if (line == null) {
                break;
            }
            bk = new String[args.length+1];
            for (int i = 0; i < args.length; i++) {
                bk[i] = args[i];
            }
            args = bk;
            args[args.length-1] = line;
        }

        obj_args = new Object[1];
        obj_args[0] = (Object)args;

        mth = cls.getMethods();
        for (int i = 0; i < mth.length; i++) {
            if (mth[i].getName().compareTo(method_name) == 0) {
                if (getDebug() > 0) {
                    try {
                        field = cls.getField(DEBUG_VALUE_STRING);
                        field.setInt(obj, getDebug());
                        field = cls.getField(MESSAGE_RESOURCE_STRING);
                        field.set(obj, getMessageResource());
                        field = cls.getField(LOG_PRINT_STREAM);
                        field.set(obj, getLogPrintStream());
                    } catch (Exception exception_object) {
                    }
                }
System.out.println("Call method " + mth[i].getName());
                save_stdout = System.out;
                System.setOut(print_stream);
                try {
                    call_method_object = new CallMethod(mth[i]);
                    call_method_object.setObjectParameter(obj, (Object)args, 0);
                    call_method_object.invoke(obj);
                } catch (Exception ep) {
                    throw ep;
                }

                System.setOut(save_stdout);
                break;
            }
        }

        if (getDebug() > 0 && getLogPrintStream() != null) {
            message_resource = getMessageResource();
            if (message_resource == null) {
                message_resource = new MessageResource();
            }
            message = message_resource.getFormattedMessage(
                MessageText.END_METHOD,
                message_resource.getMessage( 
                    MessageText.END_METHOD ),
                SPACE,
                LOADCLASSFROMJAR);
            getLogPrintStream().printLogln( message );
        }
    }

    public void removeFile(String file_name) throws Exception {
        File   fd;

        fd = new File(EMPTY);
        fd = new File(fd.getAbsolutePath() + File.separator +
            PARENT + File.separator + TMPDIR + File.separator + file_name);

        if (fd.exists() == true) {
            try {
                fd.delete();
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }
        return;
    }

    private static int BUFFER_SIZE = 4096;

    public void writeFile(InputStream input_stream, String file_name) throws Exception {
        int                rt;
        int                offset;
        byte[]             buffer;
        byte[]             buffer2;
        File               fd;
        String             path;

        FileOutputStream   file_output_stream;

        fd = new File(EMPTY);
        path = fd.getAbsolutePath();
        fd = new File(path + File.separator +
            PARENT + File.separator + TMPDIR + File.separator + file_name);

        file_output_stream = new FileOutputStream(fd); 

        offset = 0;
        buffer = new byte[BUFFER_SIZE];
        while (true) {
            try {
                rt = input_stream.read(buffer, 0, buffer.length);
            } catch (Exception exception_object) {
                throw exception_object;
            }
            if (rt == -1) {
                break;
            }
            file_output_stream.write(buffer, 0, rt);
        }
        
        try {
            file_output_stream.close();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return;
    }

    public static void main(String[] args) {
        String                message;
        String                class_name;
        String                file_input;
        String                output_file;
        String                parent_file_input;
        ExecuteServer         execute_server;
        byte[]                byte_parameter;
        String                method_name;
        ByteArrayOutputStream byte_array_output_stream;
        PrintStream           print_stream;
        FileInputStream       file_input_stream;
        FileOutputStream      file_output_stream;
        int                   debug_value;
        String                debug_string;
        MessageResource       message_resource;
        EncodePrintStream     encode_print_stream;

        message_resource = new MessageResource();
        debug_string = System.getProperty(MessageText.EXEC_REGION_LANG);
        try {
            if (debug_string == null) {
                message_resource.setLanguage(Locale.JAPAN.getISO3Language());
            } else {
                message_resource.setLanguage(debug_string);
            }
        } catch (Exception exception_object) {
            System.err.println( exception_object.getMessage() );
            return ;
        }
        execute_server = new ExecuteServer();

/**
        execute_server.setMessageResource( message_resource );
**/

        debug_string = System.getProperty(MessageText.EXEC_REGION_DEBUG);
        if (debug_string == null || debug_string.length() == 0) {
            debug_string = System.getProperty(MessageText.DEBUG_LEVEL_KEY);
        }
        if (debug_string != null && debug_string.length() > 0) {
            try {
                debug_value = Integer.valueOf(debug_string).intValue();
            } catch (Exception exception_object) {
                System.err.println( exception_object.getMessage() );
                return ;
            }
            try {
                execute_server.setupDebug(debug_value);
            } catch (Exception exception_object) {
                System.err.println( exception_object.getMessage() );
                return ;
            }
/*
            execute_server.setDebug(debug_value);
            execute_server.setLogPrintStream(new LogPrintStream(System.err));
*/
        }

        if (args.length <= 2) {
            message = message_resource.getFormattedMessage(
                MessageText.ARGUMENT_PARAMETER_EXCEPTION,
                message_resource.getMessage(
                    MessageText.ARGUMENT_PARAMETER_EXCEPTION));
            System.err.println(message);
            return;
        }
        byte_array_output_stream = new ByteArrayOutputStream();
        encode_print_stream = new EncodePrintStream(byte_array_output_stream);

        method_name = args[0];
        class_name = args[1];
        parent_file_input = args[2];
        file_input = args[2];
        if (parent_file_input.lastIndexOf(File.separator) >= 0) {
            file_input = parent_file_input.substring(
                parent_file_input.lastIndexOf(File.separator)+1);
        }
        output_file = args[3];

        try {
            file_output_stream = new FileOutputStream(output_file);
            file_input_stream = new FileInputStream( parent_file_input ); 
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            return;
        }

        for (int i = 4; i < args.length; i++) {
            encode_print_stream.println(args[i]);
        }
        byte_parameter = byte_array_output_stream.toByteArray();
        print_stream = new PrintStream(file_output_stream);

        try {
            execute_server.execute(method_name, class_name, 
                file_input_stream, 
                byte_parameter, print_stream, message_resource);
        } catch (Exception exception_object) {
            System.err.println(exception_object.getMessage());
            return;
        }
    }
}

