/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectServer.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/07/23
 *             2.1 2010/10/08
 *             2.1 2010/11/04 add exit procedure for property error
 *             2.1 2011/01/07 add directory security for 
 *                            individual service using user.dir property
 *             2.1 2011/01/17 add feature to prevent 
 *                            from replacement of SecurityManager
 *             2.1 2011/01/26 add int signal procedure
 *             2.1 2011/09/22 add forward feature
 *             2.1 2011/11/15 network security feature
 *             2.1 2011/12/20 Japanese feature
 *             2.1 2012/01/05 Single port version
 *             2.1 2012/01/19 Remove procedure to transfer numbr of parameter.
 *             2.1 2012/02/13 Add monitor feature.
 *             2.1 2012/02/21 Add feature to kill a service
 *             2.1 2012/03/09 Adopt synchronized keyword for exclusive access
 *             2.1 2012/03/12 Add a feature for serializable execution
 *             2.1 2012/04/02 Add Input stream feature
 *             2.1 2012/04/30 Modify exclusion procedure for thread tables.
 *             2.1 2012/05/01 Add transient feature
 *             2.1 2012/05/17 Add output stream feature.
 *
 * This class is Server class for managing object codes.
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.jar.*;
import java.util.zip.*;
import java.security.*;

import sun.misc.*;

import oserv.*;

class OServInterruptHandler implements Thread.UncaughtExceptionHandler {
    private int debug = 0;
    private LogPrintStream logStream = null;
    private MessageResource message_resource = new MessageResource();
    private String thread_type = null;
    private NamedInputStream[]  stream_list = new NamedInputStream[0];

    private String DEBUG_SET_DEBUG_LOG_STREAM_WITH_MESSAGE
        = "setDebugLogStreamWithMessage()";

    public void setDebugLogStreamWithMessage(int parameter_debug, LogPrintStream ps, MessageResource parameter_message) {
        if (ps == null || parameter_message == null) {
            return;
        }
        debug = parameter_debug;
        logStream = ps;
        message_resource = parameter_message;
        if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
            logStream.printLogln( message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_END_KEY,
                message_resource.getMessage( ObjectServerDebug.DEBUG_END_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_SET_DEBUG_LOG_STREAM_WITH_MESSAGE,
                message_resource.getMessage(ObjectServerDebug.DEBUG_CODE_002)),
                toString());
        }
    }

    public void setThreadType(String type) {
        thread_type = type;
    }

    public void addStream(NamedInputStream stream_param) {
        NamedInputStream[]   local_stream;

        if (stream_param != null) {
            local_stream = new NamedInputStream[stream_list.length+1];
            for (int i = 0; i < stream_list.length; i++) {
                local_stream[i] = stream_list[i];
            }
            stream_list = local_stream;
            stream_list[stream_list.length-1] = stream_param;
        }
    }

    public void uncaughtException(Thread interrupted_thread, 
        Throwable event) {
        String  interrupted_thread_string;
        String  event_string;
        String  key_string;

        event_string = null;
        interrupted_thread_string = null;
        if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
            interrupted_thread_string = ObjectServerDebug.DEBUG_NULL_KEY;
            if (interrupted_thread != null) {
                interrupted_thread_string = interrupted_thread.toString();
            }
            event_string = ObjectServerDebug.DEBUG_NULL_KEY;
            if (event != null) {
                event_string = event.toString();
            }
        }
        if (debug > ObjectServerHeader.OSERV_DEBUG_TRACE_VALUE && logStream != null) {
            logStream.printLogln( message_resource.getFormattedMessage(
                ObjectServerDebug.DEBUG_THREAD_DEATH,
                message_resource.getMessage(
                    ObjectServerDebug.DEBUG_THREAD_DEATH),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                event_string));
        }

        if (stream_list.length > 0) {
            if (thread_type.compareTo(ObjectServerHeader.SERVICE_THREAD_TYPE) == 0) {
                for (int i = 0; i < stream_list.length; i++) {
                    try {
                        stream_list[i].close();
                    } catch (Exception exception_object) {
                    }
                }
            }
        }
        if (thread_type != null) {
            key_string = ObjectServerDebug.DEBUG_NULL_KEY;
            if (thread_type.compareTo(ObjectServerHeader.MONITOR_THREAD_TYPE) == 0) {
                key_string = ObjectServerDebug.DEBUG_MONITOR_THREAD_END;
            } else if (thread_type.compareTo(ObjectServerHeader.SERVICE_THREAD_TYPE) == 0) {
                key_string = ObjectServerDebug.DEBUG_SERVICE_THREAD_END;
            } else if (thread_type.compareTo(ObjectServerHeader.SYSTEM_THREAD_TYPE) == 0) {
                key_string = ObjectServerDebug.DEBUG_SYSTEM_THREAD_END;
            } else if (thread_type.compareTo(ObjectServerHeader.TRANSIENT_THREAD_TYPE) == 0) {
                key_string = ObjectServerDebug.DEBUG_TRANSIENT_THREAD_END;
            }
            if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
                logStream.printLogln( message_resource.getFormattedMessage(
                    key_string,
                    message_resource.getMessage(key_string),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    interrupted_thread_string));
            }

/*
            if (thread_type.compareTo(ObjectServerHeader.MONITOR_THREAD_TYPE) == 0) {
                if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
                    logStream.printLogln( message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_MONITOR_THREAD_END,
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_MONITOR_THREAD_END),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        interrupted_thread_string));
                }
            } else if (thread_type.compareTo(ObjectServerHeader.SERVICE_THREAD_TYPE) == 0) {
                if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
                    logStream.printLogln( message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_SERVICE_THREAD_END,
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_SERVICE_THREAD_END),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        interrupted_thread_string));
                }
   
            } else if (thread_type.compareTo(ObjectServerHeader.SYSTEM_THREAD_TYPE) == 0) {
                if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
                    logStream.printLogln( message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_SYSTEM_THREAD_END,
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_SYSTEM_THREAD_END),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        interrupted_thread_string));
                }
            } else if (thread_type.compareTo(ObjectServerHeader.TRANSIENT_THREAD_TYPE) == 0) {
                if (debug > ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
                    logStream.printLogln( message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_TRANSIENT_THREAD_END,
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_TRANSIENT_THREAD_END),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        interrupted_thread_string));
                }
            }
**/
        }
        return;
    }
}

class ObjectSignalHandler implements SignalHandler {

    public void handle(Signal sig) {
    }
}

/**
 * This class provides a server feature
 *
 * @see oserv.ObjectServerHeader
 */
public class ObjectServer extends RegisterServer implements Serializable {
    public static String HUP_SIGNAL = "HUP";
    public static String INT_SIGNAL = "INT";
    public static String BUS_SIGNAL = "BUS";

    private static int oserv_size;
    private static int oserv_inc  = 0;
    private static int oserv_number = 0;
    private static String[]  oserv;

    private static ObjectTable object_table = new ObjectTable();

    public static final String VERSION_INFORMATION = "2.1";
    public static final String OBJECT_SERVER_ARGS    = "vh";
    public static final String VOPTION = "-v";
    public static final String HOPTION = "-h";

    public static final String COMMAND_KEY = "COMMAND";

    public static ObjectSignalHandler signal_handler;

    private static String persistent_path = null;

    private static String transient_path = null;

    private static String temporary_path = null;
    /** for security manager **/
    private static String user_path = null;
    private static String jar_path = null;
    private static String objfile = null;
    private static String forward_file = null;

    private static int tunnel_value = 0;

    public static int BUFFER_MAXIMUM = 10000;
    public static int BUFFER_SIZE = 4096;

    private static Thread main_thread = null;

    public static int max_thread = 0;
    public static int monitor_interval = 500;

    private String RESERVE_THREAD     = "ReserveThread";
    private String RELEASE_THREAD     = "ReleaseThread";
    private String READ_THREAD_TABLE  = "ReadThreadTable";
    private String TERMINATE_THREAD   = "TerminateTable";
    private String EXCLUSIVE_SET_OF_COMMAND_TO_THREAD = "ExclusiveSetOfCommandToThread";

    private static ThreadTable  register_table = new ThreadTable(REGISTER_THREAD_TYPE);
    private static ThreadTable  service_table = new ThreadTable(SERVICE_THREAD_TYPE);
    private static ThreadTable  forward_table = new ThreadTable(FORWARD_THREAD_TYPE);
    private static ThreadTable  system_table = new ThreadTable(SYSTEM_THREAD_TYPE);
    private static ThreadTable  transient_table = new ThreadTable(TRANSIENT_THREAD_TYPE);


    private static boolean system_forbidden = true;
    /**
     * Forward service table
     */
    private URI[] forward_service_table = new URI[0];
    private int   forward_service_count = 0;
    private static int      FORWARD_UNIT_SIZE = 50;

    /**
     * Forward client table
     */
    private URI[] forward_client_table = new URI[0];
    private int   forward_client_count = -1;
    private URI[] forward_server_table = new URI[0];
    private int   forward_server_count = -1;

    /**
     * Refuse table
     */
    private URITable refuse_request_table = new URITable();
    private URITable refuse_object_table = new URITable();
    private URITable refuse_forward_table = new URITable();
    private URITable refuse_system_table = new URITable();
    private URITable refuse_transient_table = new URITable();

    public final static String OBJECT_CLASS = "Object";
    public final static String STRING_CLASS = "String";
    public final static String LOCK_FILE = "./.lock";
    public File flock = new File(LOCK_FILE);

    public final static String THREAD_LOCK_FILE = "./.thread";
    public File thread_lock = new File(THREAD_LOCK_FILE);

    public final static String PROCESS_ID = "./.pid";
    public final static String EMPTY_PATH = "";
    public final static String EMPTY_MESSAGE = "";

    public final static String DEBUG_RETURN_FAIL 
        = "Couldn't write return value to socket stream.";
    public final static String DEBUG_ALREADY_CLOSED_CONNECTION
        = "Connection was closed.";
    public final static String DEBUG_FUNCTION_SUFFIX = "DEBUG";

    public final String ERROR_MESSAGE1 = "Templary Output File Error";
    public final String ERROR_MESSAGE2 = "";

    public final String ERROR_SERVER_RUNNING = "OServ is running.";
    public final String ERROR_FAIL_LOCK = "Fail to lock.";
    public final String ERROR_FAIL_CHECK_PROCESS_STATE 
        = "Fail to check ProcessState";
    public final String ERROR_FAIL_READ_OBJECTTABLE
        = "Fail to read ObjectTable";
    public final String ERROR_FAIL_OPERATE_OBJECTFILE
        = "Fail to operate ObjectFile";
    public final String ERROR_FAIL_READ_FORWARDTABLE
        = "Fail to read ForwardTable";
    public final String ERROR_FAIL_CLOSE
        = "Fail to close";

    public final String ERROR_REGISTRATION_STARTUP 
        = "Fail to startup object service";
    public final String ERROR_CONNECTION_STARTUP 
        = "Fail to startup request service ";
    public final String ERROR_FORWARD_STARTUP 
        = "Fail to startup forward service";

    public final static String MKDIR_ERROR = "mkdirs method failed.";

    public final static String NORMAL_END_MESSAGE = "Normal end";

    public final static String PROCESS_LIST = "/bin/ps -p";
    public final static String PROCESS_COMMAND_LIST = "/bin/ps -e";

    public final String PERSISTENT_DIRECTORY_ERROR =
        "Persistent directory configuration error";
    public final String USER_DIRECTORY_ERROR =
        "User directory configuration error";
    public final String DONOT_OPERATE_ENTRY = "NOP";

    public static int maximum = BUFFER_MAXIMUM;

    /**
     * LAST MESSAGE INDEX = 755  / 100
     */
    private static int      private_message_format_max =  1000;
    private static int      private_message_max =  1000;
    private static String[] private_message
        = new String[private_message_max];
    private static String[] private_message_format 
        = new String[private_message_format_max];

    public final static String USAGE_LINE    = "[-v | -h]";
    public final static String COMMAND_NAME  = "ObjectServer";
    public final static String COMMAND_SPACE = "            ";

    public final static String OSERV_METHOD_NAME = "OServ";

    public final static String BASIC_EXCEPTION = "Exception";

    public final static String DEBUG_CODE          = "Code ";
    public final static String DEBUG_PARAM         = "Param";
    public final static String DEBUG_PARAMETER     = "Parameter";
    public final static String DEBUG_VARIABLE_FILE_NAME 
        = "Variable file name";
    public final static String DEBUG_EMPTY         = "";

    public final static String DEBUG_GETFORWARDSERVERHOSTCOUNT
        = "getForwardServerHostCount()";
    public final static String DEBUG_WRITEFORWARDURITABLE
        = "writeForwardURITable()";
    public final static String DEBUG_SENDFORWARDTABLE
        = "sendForwardTable()";
    public final static String DEBUG_SETFORWARDSERVICEENTRY
        = "setForwardServiceEntry()";
    public final static String DEBUG_SETDEBUGLOGSTREAM
        = "setDebugLogStream()";
    public final static String DEBUG_GETSERVICESFROMFORWARDS
        = "getServicesFromForwards()";
    public final static String DEBUG_CASTSERVICETOFORWARDS
        = "castServiceToForwards()";
    public final static String DEBUG_CHECKCOMMANDFORWARDSERVICE
        = "checkCommandForwardService()";
    public final static String DEBUG_RESERVESYSTEMTHREAD 
        = "reserveSystemThread()";
    public final static String DEBUG_RESERVEREGISTERTHREAD 
        = "reserveRegisterThread()";
    public final static String DEBUG_RESERVESERVICETHREAD 
        = "reserveServiceThread()";
    public final static String DEBUG_RESERVEFORWARDTHREAD 
        = "reserveForwardThread()";
    public final static String DEBUG_STARTFORWARDTHREAD 
        = "start ForwardThread()";
    public final static String DEBUG_STARTSERVICETHREAD 
        = "start ServiceThread()";
    public final static String DEBUG_STARTREGISTERTHREAD 
        = "start RegisterThread()";
    public final static String DEBUG_STARTSYSTEMTHREAD 
        = "start SystemThread()";
    public final static String DEBUG_SENDCOMMANDTOFORWARD
        = "sendCommandToForward()";
    public final static String DEBUG_RESETFORWARDSERVICE
        = "resetForwardService()";
    public final static String DEBUG_TRANSFERSERVICE
        = "transferService()";
    public final static String DEBUG_GETTHREADTABLE
        = "getThreadTable()";

    public final static String DEBUG_OPERATETHREADTABLE
        = "operateThreadTable()";

    public final static String DEBUG_OPERATESERVICETHREADTABLE
        = "operateServiceThreadTable()";
    public final static String DEBUG_OPERATEREGISTERTHREADTABLE
        = "operateRegisterThreadTable()";
    public final static String DEBUG_OPERATEFORWARDTHREADTABLE
        = "operateForwardThreadTable()";
    public final static String DEBUG_OPERATESYSTEMTHREADTABLE
        = "operateSystemThreadTable()";

    public final static String DEBUG_RESERVETHREAD
        = "reserveThread()";
    public final static String DEBUG_CHECKTHREADCOMMAND
        = "checkThreadCommand()";
    public final static String DEBUG_SETTHREADCOMMAND
        = "setThreadCommand()";
    public final static String DEBUG_TERMINATETHREAD
        = "terminateThread()";
    public final static String DEBUG_STOPTHREAD
        = "stopThread()";

    public final static String DEBUG_ILLEGAL_OPERATION
        = "illegal operation";
    public final static String DEBUG_NULL_OPERATION_TYPE
        = "null OperationType";
    public final static String DEBUG_NULL_ENTRY
        = "null Entry";
    public final static String DEBUG_NULL_PARAMETER
        = "null Parameter";
    public final static String DEBUG_NULL_SOCKET
        = "null Socket";
    public final static String DEBUG_NULL_INPUTSTREAM
        = "null InputStream";
    public final static String DEBUG_NULL_THREADID
        = "null ThreadID";
    public final static String DEBUG_NULL_QUERY
        = "null Query";

    public final static String DEBUG_COMMAND_NOT_FOUND
        = "Command not found";

    public final static String DEBUG_OPERATEOBJECTTABLE
        = "operateObjectTable()";
    public final static String DEBUG_ADD_OBJECTTABLE
        = "object_table.add";
    public final static String DEBUG_SEARCH_OBJECTTABLE
        = "object_table.search";
    public final static String DEBUG_EQUAL_OBJECTTABLE
        = "object_table.equal";
    public final static String DEBUG_CLONE_OBJECTTABLE
        = "object_table.clone";
    public final static String DEBUG_SETTYPE_OBJECTTABLE
        = "object_table.setType";
    public final static String DEBUG_DELETE_OBJECTTABLE
        = "object_table.delete";

    public final static String DEBUG_SETRETRYCOUNT
        = "setRetryCount()";
    public final static String DEBUG_READFORWARDREQUESTSFROMSOCKET 
        = "readForwardRequestsFromSocket()";
    public final static String DEBUG_WRITE_EOS_OBJECT = "writeEOSObject()";
    public final static String DEBUG_MAKEFORWARDCLIENTTABLE
	= "makeForwardClientTable()";
    public final static String DEBUG_MAKEFORWARDSERVERTABLE
        = "makeForwardServerTable()";
    public final static String DEBUG_READOBJECTTABLE
        = "readObjectTable()";
    public final static String DEBUG_DOCOMMANDFORWARDSERVICE
        = "doCommandForwardService()";
    public final static String DEBUG_CHECKACCESS 
        = "checkAccess()";
    public final static String DEBUG_DOSYSTEMCOMMAND
        = "doSystemCommand()";
    public final static String DEBUG_DO_REGISTER_CONNECTION
        = "doRegisterConnection()";
    public final static String DEBUG_DO_REGISTER_LIST
        = "doRegisterList()";
    public final static String DEBUG_DO_REGISTER_DELETE
        = "doRegisterDelete()";
    public final static String DEBUG_DO_REGISTER_COMMAND
        = "doRegisterCommand()";
    public final static String DEBUG_DO_REGISTER_CLASS
        = "doRegisterClass()";

    public final static String TRACE_CLASSNAME
        = "oserv.ObjectServer";

    public final static String TRACE_WRITESTREAM
        = "writeStream";
    public final static String TRACE_OBJECTSERVER
        = "<init>";
    public final static String TRACE_GETSYSTEMFORBIDDEN
        = "getSystemForbidden";
    public final static String TRACE_SETSYSTEMFORBIDDEN
        = "setSystemForbidden";
    public final static String TRACE_GETEXCEPTIONMESSAGE
        = "getExceptionMessage";
    public final static String TRACE_ASSERTIONMESSAGE
        = "printAssertionMessage";
    public final static String TRACE_PRINTEXCEPTIONMESSAGE
        = "printExceptionMessage";
    public final static String TRACE_PRINTTRACEMESSAGE
        = "printTraceMessage";
    public final static String TRACE_PRINTVALUEMESSAGE
        = "printValueMessage";
    public final static String TRACE_PRINTEXECUTEMESSAGE
        = "printExecuteMessage";
    public final static String TRACE_PRINTLOGCALLMESSAGE
        = "printLogCallMessage";
    public final static String TRACE_PRINTLOGMESSAGE
        = "printLogMessage";
    public final static String TRACE_GETUSEROUTPUTMESSAGE
        = "getUserOutputMessage";
    public final static String TRACE_WRITEEOSOBJECT
        = "writeEOSObject";
    public final static String TRACE_GETFORMATTEDOUTPUTMESSAGE
        = "getFormattedOutputMessage";
    public final static String TRACE_GETBODYOUTPUTMESSAGE
        = "getBodyOutputMessage";
    public final static String TRACE_GETOUTPUTMESSAGE
        = "getOutputMessage";
    public final static String TRACE_UPDATEDEBUGLEVEL
        = "updateDebugLevel";
    public final static String TRACE_PRINTUSAGE
        = "printUsage";
    public final static String TRACE_PRINTVERSION
        = "printVersion";
    public final static String TRACE_CONVERTENTRYTOFILENAME
        = "convertEntryToFilename";
    public final static String TRACE_CONVERTENTRYTOFILEPATH
        = "convertEntryToFilepath";
    public final static String TRACE_CONVERTFILENAMETOFILEPATH
        = "convertFilenameToFilepath";
    public final static String TRACE_SHUTDOWNSOCKET
        = "shutdownSocket";
    public final static String TRACE_READVARIABLE
        = "readVariable";
    public final static String TRACE_GETJARINPUTSTREAM
        = "getJarInputStream";
    public final static String TRACE_SEEKJARINPUTSTREAM
        = "seekJarInputStream";
    public final static String TRACE_GETCLASSSIZEINJAR
        = "getClassSizeInJar";
    public final static String TRACE_WRITECLASSFILE
        = "writeClassFile";
    public final static String TRACE_EXECUTEOSERV
        = "executeOServ";
    public final static String TRACE_READOBJECTTABLE
        = "readObjectTable";
    public final static String TRACE_WRITEOBJECTTABLE
        = "writeObjectTable";
    public final static String TRACE_WRITEOBJECTTABLEENTRY
        = "writeObjectTableEntry";
    public final static String TRACE_GETPARAMETERTYPE
        = "getParameterTypes";
    public final static String TRACE_TRANSFERSERVICE
        = "transferService";
    public final static String TRACE_CHECKFORWARDSERVICE
        = "checkForwardService";
    public final static String TRACE_GETFORWARDSERVICE
        = "getForwardService";
    public final static String TRACE_ISRESETFORWARDSERVICE
        = "isResetForwardService";
    public final static String TRACE_RESETFORWARDSERVICE
        = "resetForwardService";
    public final static String TRACE_MAKEFORWARDSERVERTABLE
        = "makeForwardServerTable";
    public final static String TRACE_MAKEFORWARDCLIENTTABLE
        = "makeForwardClientTable";
    public final static String TRACE_READFORWARDSERVERENTRY
        = "readForwardServerEntry";
    public final static String TRACE_READFORWARDCLIENTENTRY
        = "readForwardClientEntry";
    public final static String TRACE_READFORWARDSERVERCOUNT
        = "readForwardServerCount";
    public final static String TRACE_READFORWARDCLIENTCOUNT
        = "readForwardClientCount";
    public final static String TRACE_READFORWARDREQUESTSFROMSOCKET
        = "readForwardRequestsFromSocket";
    public final static String TRACE_READFORWARDTABLE
        = "readForwardTable";
    public final static String TRACE_SETFORWARDSERVICEENTRY
        = "setForwardServiceEntry";
    public final static String TRACE_ISCOMMANDFORWARDSERVICE
        = "isCommandForwardService";
    public final static String TRACE_MAKEQUERYPARAMETER
        = "makeQueryParameter";
    public final static String TRACE_CHECKCOMMANDFORWARDSERVICE
        = "checkCommandForwardService";
    public final static String TRACE_DOCOMMANDFORWARDSERVICE
        = "doCommandForwardService";
    public final static String TRACE_WRITEFORWARDSTRINGTABLE
        = "writeForwardStringTable";
    public final static String TRACE_SENDFORWARDTABLE
        = "sendForwardTable";
    public final static String TRACE_WRITEFORWARDTABLE
        = "writeForwardTable";
    public final static String TRACE_WRITEFORWARDURITABLE
        = "writeForwardURITable";
    public final static String TRACE_WRITERETURN
        = "writeReturn";
    public final static String TRACE_WRITEOUTPUTSTREAMS
        = "writeOutputStreams";
    public final static String TRACE_DELETESTREAM
        = "deleteStream";

    public final static String TRACE_LOCK = "lock";
    public final static String TRACE_UNLOCK = "unlock";
    public final static String TRACE_WRITEJARFILE = "writeJarFile";
    public final static String TRACE_COPYJARFILE = "copyJarFile";
    public final static String TRACE_READJARFILE = "readJarFile";
    public final static String TRACE_DELETEDIRECTORYFILE = "deleteDirectoryFile";
    public final static String TRACE_SWITCHLOGFILE = "switchLogFile";
    public final static String TRACE_RESERVEDISPATCHTHREAD = "reserveDispatchThread";
    public final static String TRACE_OPERATEOBJECTTABLE = "operateObjectTable";
    public final static String TRACE_OPERATEREGISTERTHREADTABLE = "operateRegisterThreadTable";
    public final static String TRACE_OPERATESERVICETHREADTABLE = "operateServiceThreadTable";
    public final static String TRACE_OPERATEFORWARDTHREADTABLE = "operateForwardThreadTable";
    public final static String TRACE_OPERATESYSTEMTHREADTABLE = "operateSystemThreadTable";
    public final static String TRACE_OPERATETHREADTABLE = "operateThreadTable";
    public final static String TRACE_ISMONITOR = "isMonitor";
    public final static String TRACE_ISSYSTEM = "isSystem";
    public final static String TRACE_ISMONITORON = "isMonitorOn";
    public final static String TRACE_ISHANDLERON = "isHandlerOn";
    public final static String TRACE_RESERVETHREAD = "releaseThread";
    public final static String TRACE_GETTHREADTYPE = "getThreadType";
    public final static String TRACE_GETTHREAD = "getThread";
    public final static String TRACE_GETTHREADTABLE = "getThreadTable";
    public final static String TRACE_TERMINATETHREAD = "terminateThread";
    public final static String TRACE_ISSERIALIZABLETHREAD = "isSerializableThread";
    public final static String TRACE_SETTHREADCOMMAND = "setThreadCommand";
    public final static String TRACE_CHECKTHREADCOMMAND = "checkThreadCommand";
    public final static String TRACE_DISPATCH = "dispatch";
    public final static String TRACE_DOREGISTERCONNECTION = "doRegisterConnection";
    public final static String TRACE_DOREGISTERLIST = "doRegisterList";
    public final static String TRACE_DOREGISTERDELETE = "doRegisterDelete";
    public final static String TRACE_DOREGISTERCOMMAND = "doRegisterCommand";
    public final static String TRACE_DOREGISTERCLASS = "doRegisterClass";
    public final static String TRACE_GETNETWORKOBJECT = "getNetworkObject";
    public final static String TRACE_SETPARAMETER = "setParameter";
    public final static String TRACE_GETSERVICENAME = "getServiceName";
    public final static String TRACE_INVOKE = "invoke";
    public final static String TRACE_CHECKACCESS = "checkAccess";
    public final static String TRACE_GETOBJECTOUTPUTSTREAM = "getObjectOutputStream";
    public final static String TRACE_WRITEFORBIDDENEXCEPTION
        = "writeForbiddenException";
    public final static String TRACE_WRITEREFUSEEXCEPTION = "writeRefuseException";
    public final static String TRACE_RUN = "run";
    public final static String TRACE_STOPTHREAD = "stopThread";
    public final static String TRACE_GETTHREADID = "getThreadId";
    public final static String TRACE_SETSERVICETYPE = "setServiceType";
    public final static String TRACE_DOSYSTEMCOMMAND = "doSystemCommand";
    public final static String TRACE_DOSYSTEMCONNECTION = "doSystemConnection";
    public final static String TRACE_DOMONITORCONNECTION = "doMonitorConnection";
    public final static String TRACE_DOSERVICECONNECTION = "doServiceConnection";
    public final static String TRACE_SENDCOMMANDTOFORWARD = "sendCommandToForward";
    public final static String TRACE_GETFORWARDSERVERHOSTCOUNT = "getForwardServerHostCount";
    public final static String TRACE_GETTUNNELENTRY = "getTunnelEntry";
    public final static String TRACE_GETURIENTRY = "getURIEntry";
    public final static String TRACE_CASTSERVICESTOFORWARDS = "castServicesToForwards";
    public final static String TRACE_CASTSERVICETOFORWARDS = "castServiceToForwards";
    public final static String TRACE_CASTENTRYTOFORWARDS = "castEntryToForwards";
    public final static String TRACE_GETSERVICESFROMFORWARDS = "getServicesFromForwards";
    public final static String TRACE_SETUPFORWARD = "setupForward";
    public final static String TRACE_DOFORWARDCONNECTION = "doForwardConnection";
    public final static String TRACE_MAIN = "main";

    private static boolean initialized = false;
    private static boolean user_debug = false;

    private static Locale[] locale_array = {
        Locale.ENGLISH,
        Locale.JAPANESE,
        Locale.JAPAN,
        Locale.US,
        Locale.UK,
    };

    /**
     * Message class instance
     */
    public final static int NEGATIVE_ONE = -1;
    public final static String DEBUG_NULL = "Null";
    public final static String DEBUG_SET_PROPERTY = "setProperty";
    public final static String DEBUG_READ_PROPERTY_FILE = "Call getProperty()";
    public static LogPrintStream logStream = null;

    /**
     *   a failure on Forward is ignored
     *   a failure on property leads to the halt
     */
    
    /**
     * This constructor doesn't take any parameter
     * and it initializes property values.
     *
     *    OSERV_CONNECTION_TIMEOUT_KEY
     *    OSERV_RETRY_COUNT_KEY
     *    OSERV_BUFFER_MAXIMUM_KEY
     *    OSERV_WORK_PATH_KEY
     *    PROCESS_WORKING_DIRECTORY_KEY
     *    OSERV_PERSISTENT_DIRECTORY_KEY
     *    OSERV_TABLE_SIZE_KEY
     */
    public ObjectServer( ) {
        super();
        String workpath;
        String persist;
        String transient_string;
        String userwork;
        int    size;
        String tablesize;
        File   fdes;
        File   ddes;
        int    retry_count = 0;
        int    forbidden = 1;
        int    locale_index = 0;
        String value_string;
        String logfile;
        boolean tunnel_flag = false;

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            for (locale_index = 0; locale_index < locale_array.length; locale_index++) {
                if (locale_array[locale_index].getISO3Language().compareTo(
                    value_string) == 0) {
                    setMessageLanguage(value_string);
                    break;
                }
            }
            if (locale_index == locale_array.length) {
                System.err.println( getFormattedOutputMessage(
                    31, ObjectServerError.ILLEGAL_LANGUAGE_SERVER,
                    getOutputMessage(
                       31, ObjectServerError.ILLEGAL_LANGUAGE_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }

        String separator_key = String.valueOf(java.io.File.separatorChar);
        if (separator_key.compareTo(DOS_DIRECTORY_SEPARATOR) != 0) {
            Boolean check = false;
            try {
                check = lock();
                if (check == false) {
                    System.err.println( getFormattedOutputMessage(
                        1, ObjectServerError.ERROR_SERVER_RUNNING_SERVER,
                        getOutputMessage(
                           1, ObjectServerError.ERROR_SERVER_RUNNING_SERVER)));
                    System.exit(ERROR_EXIT_CODE);
                }
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    2, ObjectServerError.ERROR_FAIL_LOCK_SERVER,
                    getOutputMessage(
                        2, ObjectServerError.ERROR_FAIL_LOCK_SERVER )));
                System.exit(ERROR_EXIT_CODE);
            }
            try {
                check = checkProcessState(COMMAND_NAME) ;
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage( 3, 
                    ObjectServerError.ERROR_FAIL_CHECK_PROCESS_STATE_SERVER,
                    getOutputMessage( 3, 
                        ObjectServerError.ERROR_FAIL_CHECK_PROCESS_STATE_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
            if (check == false) {
                unlock();
                System.err.println( 
                    getFormattedOutputMessage(
                        4, ObjectServerError.ERROR_SERVER_RUNNING_SERVER,
                        getOutputMessage(
                            4, ObjectServerError.ERROR_SERVER_RUNNING_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
            unlock();
            signal_handler = new ObjectSignalHandler();
            Signal.handle(new Signal(HUP_SIGNAL), signal_handler);
            Signal.handle(new Signal(INT_SIGNAL), signal_handler);
            Signal.handle(new Signal(BUS_SIGNAL), signal_handler);
        }

        oserv_size = Integer.valueOf(OSERV_SIZE_VALUE).intValue();
        oserv_inc = oserv_size;

        try {
            updateDebugLevel();
        } catch (Exception exception_object) {
            System.err.println( getFormattedOutputMessage(
                24, ObjectServerError.DEBUG_LEVEL_CONFIGURATION_ERROR_SERVER,
                getOutputMessage(
                    24, ObjectServerError.DEBUG_LEVEL_CONFIGURATION_ERROR_SERVER)) );
            System.exit(ERROR_EXIT_CODE);
        }

        value_string = getPropertyValue(OSERV_CONNECTION_TIMEOUT_KEY);
        if (value_string != null && value_string.length() != 0) {
            try {
                timeout = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    5, ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_SERVER,
                    getOutputMessage(
                        5, ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_SERVER)) );
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getFormattedOutputMessage(
                6, ObjectServerError.CONNECTION_TIMEOUT_VALUE_NOT_FOUND_SERVER,
                getOutputMessage(
                    6, ObjectServerError.CONNECTION_TIMEOUT_VALUE_NOT_FOUND_SERVER)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_RETRY_COUNT_KEY);
        if (value_string != null && value_string.length() != 0) {
            try {
                retry_count = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    7, ObjectServerError.RETRY_COUNT_VALUE_ERROR_SERVER,
                    getOutputMessage(
                        7, ObjectServerError.RETRY_COUNT_VALUE_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getFormattedOutputMessage(
                8, ObjectServerError.RETRY_COUNT_VALUE_NOT_FOUND_SERVER,
                getOutputMessage(
                    8, ObjectServerError.RETRY_COUNT_VALUE_NOT_FOUND_SERVER)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_BUFFER_MAXIMUM_KEY);
        if (value_string != null && value_string.length() != 0) {
            try {
                maximum = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    9, ObjectServerError.BUFFER_MAXIMUM_VALUE_ERROR_SERVER,
                    getOutputMessage(
                        9, ObjectServerError.BUFFER_MAXIMUM_VALUE_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getFormattedOutputMessage(
                10, ObjectServerError.BUFFER_MAXIMUM_VALUE_NOT_FOUND_SERVER,
                getOutputMessage(
                    10, ObjectServerError.BUFFER_MAXIMUM_VALUE_NOT_FOUND_SERVER)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_FORWARD_TUNNEL_KEY);
        if (value_string != null && value_string.length() != 0) {
            try {
                tunnel_value = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                tunnel_flag = true;
            }
        }
        if (retry_count > getRetryCount()) {
            setRetryCount(retry_count);
        }

        workpath = getPropertyValue(OSERV_WORK_PATH_KEY);
        if (workpath == null) {
            workpath = System.getenv(PROCESS_WORKING_DIRECTORY_KEY);
        }
        persist = getPropertyValue(OSERV_PERSISTENT_DIRECTORY_KEY);
        if (persist == null) {
            persist = OSERV_PERSISTENT_DIRECTORY_VALUE;
        }
        transient_string = getPropertyValue(OSERV_TRANSIENT_DIRECTORY_KEY);
        if (transient_string == null) {
            transient_string = OSERV_TRANSIENT_DIRECTORY_VALUE;
        }
        userwork = getPropertyValue(OSERV_USER_WORK_KEY);
        if (userwork == null || userwork.length() == 0) {
            userwork = OSERV_USER_WORK_VALUE;
        }
        if (userwork.charAt(0) != File.separatorChar) {
            user_path = new String(workpath + File.separator + userwork);
        } else {
            user_path = userwork;
        }

        boolean transient_flag;
        File user_file = new File(user_path);
 
        File transient_file;
        transient_path = new String(user_path + File.separator + transient_string);
        persistent_path = new String(workpath + File.separator + persist);
            
        transient_file = new File(transient_path);

        temporary_path = new String(user_path + File.separator + 
                           TEMPORARY_DIRECTORY);

        objfile = new String(persistent_path + File.separator + 
                           OSERV_OBJECT_TABLE_FILE);
        forward_file = new String(persistent_path + File.separator +
                           OSERV_FORWARD_TABLE_FILE);

        tablesize = getPropertyValue(OSERV_TABLE_SIZE_KEY);
        if (tablesize != null) {
            try {
                size = Integer.valueOf(tablesize).intValue() ;
                if (size > oserv_size) {
                    oserv_size = size;
                    oserv_inc = size;
                }
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    11, ObjectServerError.TABLE_SIZE_VALUE_ERROR_SERVER,
                    getOutputMessage(
                        11, ObjectServerError.TABLE_SIZE_VALUE_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getFormattedOutputMessage(
                12, ObjectServerError.TABLE_SIZE_VALUE_ERROR_SERVER,
                getOutputMessage(
                    12, ObjectServerError.TABLE_SIZE_VALUE_ERROR_SERVER)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_MAX_THREAD_KEY);
        if (value_string != null && value_string.length() != 0) {
            try {
                max_thread = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    13, ObjectServerError.MAX_THREAD_ERROR_SERVER,
                    getOutputMessage(
                        13, ObjectServerError.MAX_THREAD_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getFormattedOutputMessage(
                14, ObjectServerError.MAX_THREAD_NOT_FOUND_SERVER,
                getOutputMessage(
                    14, ObjectServerError.MAX_THREAD_NOT_FOUND_SERVER)));
            System.exit(ERROR_EXIT_CODE);
        }

        String   vstring;
        vstring = getPropertyValue(OSERV_DEBUG_FUNCTION_KEY);
        if (vstring != null) {
            vstring = vstring.toLowerCase();
            if (OSERV_DEBUG_FUNCTION_ON_KEY.compareTo(vstring) == 0) {
                user_debug = true;
            } else if (OSERV_DEBUG_FUNCTION_OFF_KEY.compareTo(vstring) == 0) {
                user_debug = false;
            } else {
                System.err.println( getFormattedOutputMessage(
                    42, ObjectServerError.DEBUG_FUNCTION_ERROR_SERVER,
                    getOutputMessage(
                        42, ObjectServerError.DEBUG_FUNCTION_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        }

	/**
         * Switch log 
         */
        switchLogFile();
        printDebugMessage(OSERV_DEBUG_TRACE_VALUE, getFormattedOutputMessage(
            15, ObjectServerDebug.DEBUG_MAX_THREAD,
            getOutputMessage( 15, ObjectServerDebug.DEBUG_MAX_THREAD),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            String.valueOf(max_thread),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            getOutputMessage(16,ObjectServerDebug.DEBUG_INITIALIZATION)));

        if (tunnel_flag == true) {
            printDebugMessage(OSERV_DEBUG_FULL_VALUE, 
                getFormattedOutputMessage(32,
                    ObjectServerDebug.DEBUG_BAD_TUNNEL_VALUE,
                    getOutputMessage(32, 
                        ObjectServerDebug.DEBUG_BAD_TUNNEL_VALUE)));
        }

        try {
            ddes = new File(persistent_path);
            printDebugMessage(OSERV_DEBUG_FULL_VALUE, getFormattedOutputMessage(
               17, ObjectServerDebug.DEBUG_PERSISTENT_PATH,
               getOutputMessage( 17, ObjectServerDebug.DEBUG_PERSISTENT_PATH),
               ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
               persistent_path,
               ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
               getOutputMessage( 18, ObjectServerDebug.DEBUG_FILE_DESCRIPTOR),
               ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
               objfile));
            if (ddes.exists() == false) {
                printDebugMessage(OSERV_DEBUG_FULL_VALUE, getFormattedOutputMessage(
                    19, ObjectServerDebug.DEBUG_PERSISTENT_DIRECTORY_NOT_FOUND,
                    getOutputMessage(
                    19, ObjectServerDebug.DEBUG_PERSISTENT_DIRECTORY_NOT_FOUND)));
                if (ddes.mkdirs() == false) {
                    System.err.println( getFormattedOutputMessage(
                        20, ObjectServerError.PERSISTENT_DIRECTORY_ERROR_SERVER,
                        getOutputMessage(
                            20, ObjectServerError.PERSISTENT_DIRECTORY_ERROR_SERVER) ));
                    System.exit(ERROR_EXIT_CODE);
                }
            } else {
                if (ddes.isDirectory() == false) {
                    System.err.println( getFormattedOutputMessage(
                        21, ObjectServerError.PERSISTENT_DIRECTORY_ERROR_SERVER,
                        getOutputMessage(
                            21, ObjectServerError.PERSISTENT_DIRECTORY_ERROR_SERVER) ));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            object_table.setMessageResource(getMessageResource());
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                object_table.setDebug(getDebug());
                object_table.setLogStream(getLogStream());
            }
	    fdes = new File(objfile);
            printDebugMessage(OSERV_DEBUG_FULL_VALUE, getFormattedOutputMessage(22,
                ObjectServerDebug.DEBUG_CHECK_OBJECT_TABLE_FILE,
                getOutputMessage(22, ObjectServerDebug.DEBUG_CHECK_OBJECT_TABLE_FILE),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, objfile));
            if (fdes.exists() == true) {
                printKeyMessage(23, 23, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_READ_OBJECT_TABLE );
                try {
                    readObjectTable(objfile);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
                    System.err.println(
                        getFormattedOutputMessage(25,
                            ObjectServerError.ERROR_FAIL_READ_OBJECTTABLE_SERVER,
                            getOutputMessage(25,
                                ObjectServerError.ERROR_FAIL_READ_OBJECTTABLE_SERVER) ));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            user_file = new File(user_path);
            if (user_file.exists() == false) {
                printKeyMessage(26, 26, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_MAKE_USER_FILE_DIRECTORY);
                if (user_file.mkdirs() == false) {
                    System.err.println( 
                        getFormattedOutputMessage(27,
                            ObjectServerError.USER_DIRECTORY_ERROR_SERVER,
                            getOutputMessage(27,
                                ObjectServerError.USER_DIRECTORY_ERROR_SERVER)));
                    System.exit(ERROR_EXIT_CODE);
                }
            } else {
                if (ddes.isDirectory() == false) {
                    System.err.println( 
                        getFormattedOutputMessage(28,
                            ObjectServerError.USER_DIRECTORY_ERROR_SERVER,
                            getOutputMessage(28,
                                ObjectServerError.USER_DIRECTORY_ERROR_SERVER) ));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
            File file_object;
            file_object = new File(temporary_path);
            if (file_object.exists() != true) {
                try {
                    file_object.mkdir();
                } catch (Exception exception_object) {
                    System.err.println( getFormattedOutputMessage(
                        132, ObjectServerError.TEMPORARY_DIRECTORY_NOT_FOUND_ERROR_SERVER,
                        getOutputMessage(
                            132, ObjectServerError.TEMPORARY_DIRECTORY_NOT_FOUND_ERROR_SERVER)));
                    System.exit(ERROR_EXIT_CODE);
                }
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
            System.err.println( getFormattedOutputMessage(30,
                ObjectServerError.ERROR_FAIL_OPERATE_OBJECTFILE_SERVER,
                getOutputMessage(30,
                    ObjectServerError.ERROR_FAIL_OPERATE_OBJECTFILE_SERVER )));
            System.exit(ERROR_EXIT_CODE);
        }
        if (transient_file.exists() == false) {
            try {
                transient_flag = transient_file.mkdir();
                if (transient_flag == false) {
                    System.err.println( getFormattedOutputMessage(
                        129, ObjectServerError.TRANSIENT_DIRECTORY_NOT_CREATED_ERROR_SERVER,
                        getOutputMessage(
                            129, ObjectServerError.TRANSIENT_DIRECTORY_NOT_CREATED_ERROR_SERVER)));
                    System.exit(ERROR_EXIT_CODE);
                }
            } catch (Exception exception_object) {
                System.err.println( getFormattedOutputMessage(
                    134, ObjectServerError.TRANSIENT_DIRECTORY_NOT_CREATED_ERROR_SERVER,
                    getOutputMessage(
                        134, ObjectServerError.TRANSIENT_DIRECTORY_NOT_CREATED_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        }

        FileInputStream   file_input_stream;
        ObjectInputStream object_input_stream;

	fdes = new File(forward_file);
        if (fdes.exists() == true) {
            object_input_stream = null;
            try {
                file_input_stream = new FileInputStream(fdes);
                object_input_stream = new ObjectInputStream(file_input_stream);
                printKeyMessage(33, 33, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_READ_FORWARD_TABLE );
                readForwardTable(object_input_stream);
                object_input_stream.close();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
                fdes.delete();
            }
        }

        value_string = getPropertyValue(OSERV_FORWARD_CLIENTS_KEY);
        if (value_string != null) {
            printKeyMessage(37, 37, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_FORWARD_MAKE_CLIENTS);
            makeForwardClientTable(value_string);
        }
        value_string = getPropertyValue(OSERV_FORWARD_SERVERS_KEY);
        if (value_string != null) {
            printKeyMessage(39, 39, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_FORWARD_MAKE_SERVERS);

            makeForwardServerTable(value_string);
        }
        value_string = getPropertyValue(OSERV_REQUEST_REFUSE_KEY);
        if (value_string != null) {
            printVariableMessage(41, 41, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_REFUSE_REQUEST_TABLE, value_string);
            try {
                refuse_request_table = URITable.makeURITable(value_string);
                refuse_request_table.setMessageResource(getMessageResource());
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    ArrayList debug_messages;
                    debug_messages = refuse_forward_table.getDebugBox();
                    for (int j = 0; j < debug_messages.size(); j++) {
                        getLogStream().printLogln(
                            (String)debug_messages.get(j) );
                    }
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
            }
        }
        value_string = getPropertyValue(OSERV_OBJECT_REFUSE_KEY);
        if (value_string != null) {
            printVariableMessage(43, 43, OSERV_DEBUG_FULL_VALUE, 
                ObjectServerDebug.DEBUG_REFUSE_OBJECT_TABLE,
                value_string);
            try {
                refuse_object_table = URITable.makeURITable(value_string);
                refuse_object_table.setMessageResource(getMessageResource());
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    ArrayList debug_messages;
                    debug_messages = refuse_forward_table.getDebugBox();
                    for (int j = 0; j < debug_messages.size(); j++) {
                        getLogStream().printLogln(
                            (String)debug_messages.get(j) );
                    }
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
            }
        }
        value_string = getPropertyValue(OSERV_FORWARD_REFUSE_KEY);
        if (value_string != null) {
            printVariableMessage(45, 45, OSERV_DEBUG_FULL_VALUE, 
                ObjectServerDebug.DEBUG_REFUSE_FORWARD_TABLE,
                value_string);
            try {
                refuse_forward_table = URITable.makeURITable(value_string);
                refuse_forward_table.setMessageResource(getMessageResource());
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    ArrayList debug_messages;
                    debug_messages = refuse_forward_table.getDebugBox();
                    for (int j = 0; j < debug_messages.size(); j++) {
                        getLogStream().printLogln(
                            (String)debug_messages.get(j) );
                    }
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
            }
        }
        value_string = getPropertyValue(OSERV_SYSTEM_REFUSE_KEY);
        if (value_string != null) {
            printVariableMessage(47, 47, OSERV_DEBUG_FULL_VALUE, 
                ObjectServerDebug.DEBUG_REFUSE_SYSTEM_TABLE,
                value_string);
            try {
                refuse_system_table = URITable.makeURITable(value_string);
                refuse_system_table.setMessageResource(getMessageResource());
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    ArrayList debug_messages;
                    debug_messages = refuse_system_table.getDebugBox();
                    for (int j = 0; j < debug_messages.size(); j++) {
                        getLogStream().printLogln(
                            (String)debug_messages.get(j) );
                    }
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
            }
        }

        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            register_table.setDebug(getDebug());
            register_table.setLogStream(getLogStream());
            service_table.setDebug(getDebug());
            service_table.setLogStream(getLogStream());

            forward_table.setDebug(getDebug());
            forward_table.setLogStream(getLogStream());
            system_table.setDebug(getDebug());
            system_table.setLogStream(getLogStream());

            transient_table.setDebug(getDebug());
            transient_table.setLogStream(getLogStream());
        }
        register_table.setMessageResource(getMessageResource());
        service_table.setMessageResource(getMessageResource());
        forward_table.setMessageResource(getMessageResource());
        system_table.setMessageResource(getMessageResource());
        transient_table.setMessageResource(getMessageResource());

        value_string = getPropertyValue(OSERV_SYSTEM_FORBIDDEN_KEY);
        if (value_string != null) {
            printTraceMessage(TRACE_OBJECTSERVER, 0, value_string);
            try {
                forbidden = Integer.valueOf(value_string).intValue();
                if (forbidden == 0) {
                    setSystemForbidden(false);
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OBJECTSERVER, 0, exception_object);
            }
        } else {
            printTraceMessage(TRACE_OBJECTSERVER, 0);
        }
    }

    private void setInitialized() {
        initialized = true;
    }

    private boolean getInitialized() {
        return initialized;
    }

    private boolean getSystemForbidden() {
        return system_forbidden;
    }

    private void setSystemForbidden(boolean status) {

        system_forbidden = status;
    }

    private String getExceptionMessage(
        Exception exception_object,
        String    function,
        int       nest) {
        String return_message;

        return_message = getFormattedOutputMessage(427,
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                            getOutputMessage(427,
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY), 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            TraceMessage.getCallerMethod(exception_object,
			        TRACE_CLASSNAME, function, nest),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString());
        return return_message;
    }

    private void printAssertionMessage(
        String function, int nest, String message) {
        String message_string;

        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            message_string = getFormattedOutputMessage(531,
                ObjectServerDebug.DEBUG_ASSERTION_KEY, 
                getOutputMessage(531,
                    ObjectServerDebug.DEBUG_ASSERTION_KEY), 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, message,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                TraceMessage.getCurrentTraceString(TRACE_CLASSNAME, function, nest) );
              getLogStream().printLogln( message_string );
        }
    }

    private void printExceptionMessage(String function, int nest, Exception exception_object, String... list) {
        if (exception_object == null || function == null) {
            return;
        }
        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
            getLogStream().printLogln( 
	        getExceptionMessage(exception_object, function, nest), list);
        }
    }

    private void printTraceMessage(String function, int nest, String... value) {
        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            String  message_string;

            message_string = getFormattedOutputMessage(432, 
            ObjectServerDebug.DEBUG_TRACE_KEY,
            getOutputMessage(432, ObjectServerDebug.DEBUG_TRACE_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TraceMessage.getCurrentTraceString(TRACE_CLASSNAME, function, nest),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY);

            if (value == null) {
                getLogStream().printLogln( message_string );
            } else {
                getLogStream().printLogln( message_string, value);
            }
        }
    }

    private void printCallerTraceMessage(String function, int nest, String... value) {
        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            String  message_string;

            message_string = getFormattedOutputMessage(444, 
            ObjectServerDebug.DEBUG_TRACE_KEY,
            getOutputMessage(444, ObjectServerDebug.DEBUG_TRACE_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            TraceMessage.getCallerTraceString(TRACE_CLASSNAME, function, nest),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY);

            if (value == null) {
                getLogStream().printLogln( message_string );
            } else {
                getLogStream().printLogln( message_string, value);
            }
        }
    }

    private void printDebugMessage(int level, String... list) {
        if (getDebug() > level && getLogStream() != null) {
              getLogStream().printLogln(list);
        }
    }

    private void printVariableMessage(int index1, int index2, int level,
        String key, String value, String... list) {
        if (value == null) { return; }
        if (getDebug() > level && getLogStream() != null) {
              getLogStream().printLogln( getFormattedOutputMessage(index1, 
                  key, getOutputMessage(index2, key),
                  ObjectServerDebug.DEBUG_BLANK_STRING_KEY, value ), list);
        }
    }

    private void printKeyMessage(int index1, int index2, int level, String key) {
        if (key == null) { return; }
        if (getDebug() > level && getLogStream() != null) {
              getLogStream().printLogln( getFormattedOutputMessage(index1, 
                  key, getOutputMessage(index2, key) ));
        }
    }

    private void printValueMessage(String value) {
        printVariableMessage(128, 128, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_VALUE_KEY, value);
    }

    private void printEndMessage(int level, String function_name, String... list) {
        String  message_string;

        if (function_name == null) { return; }

        if (getDebug() > level && getLogStream() != null) {
            message_string = getFormattedOutputMessage(183, 
                ObjectServerDebug.DEBUG_END_KEY,
                getOutputMessage(183, ObjectServerDebug.DEBUG_END_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, function_name,
                getOutputMessage(185, ObjectServerDebug.DEBUG_CODE_002));

            if (list == null) {
                getLogStream().printLogln( message_string );
            } else {
                message_string = message_string.concat(
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY);
                getLogStream().printLogln( message_string, list );
            }
        }
    }

    private void printExecuteMessage(String function_name, String... list) {
        String  message_string;

        if (function_name == null) { return; }

        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            message_string = getFormattedOutputMessage(229, 
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getOutputMessage(229, ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, function_name,
                getOutputMessage(36, ObjectServerDebug.DEBUG_CODE_001));

            if (list == null) {
                getLogStream().printLogln( message_string );
            } else {
                message_string = message_string.concat(
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY);
                getLogStream().printLogln( message_string, list );
            }
        }
    }

    private void printLogCallMessage(String key, String... list) {
        String  message_string;
        if (key == null) { return; }

        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            message_string = getFormattedOutputMessage(222, 
            ObjectServerDebug.DEBUG_CALL_KEY,
            getOutputMessage(222, ObjectServerDebug.DEBUG_CALL_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, key,
                getOutputMessage(29,ObjectServerDebug.DEBUG_CODE_002));

            if (list == null) {
                getLogStream().printLogln( message_string );
            } else {
                getLogStream().printLogln( message_string, list );
            }
        }
    }

    private void printLogMessage(int mode, int index, int message_index, String header, String code, String key, String... list) {
        if (mode < 0 || index < 0 || header == null) {
            return;
        }
        if (getDebug() > mode && getLogStream() != null) {
            if (list == null) {
                if (code == null) {
                    getLogStream().printLogln(
                        getFormattedOutputMessage(index, header,
                            getOutputMessage(message_index, header)));
                } else {
                    getLogStream().printLogln(
                        getFormattedOutputMessage(index, header,
                            getOutputMessage(message_index, header),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                key, code));
                }
            } else {
                if (code == null) {
                    getLogStream().printLogln(
                        getFormattedOutputMessage(index, header,
                            getOutputMessage(message_index, header)), list);
                } else {
                    getLogStream().printLogln(
                        getFormattedOutputMessage(index, header,
                            getOutputMessage(message_index, header),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            key, code), list);
                }
            }
        }
    }

    private String getUserOutputMessage(int index_number, 
        int message_index,
        String key, String... message) {

        String    output_message;
        String[]  param_message;
        String    return_message;

        if (key != null) {
            return key;
        }
        output_message = getOutputMessage(message_index, key);
        if (message == null || message.length == 0) {
            return output_message;
        }
        param_message = new String[message.length+1];
        for (int i = 0; i < param_message.length; i++) {
            param_message[i] = message[i];
        }
        param_message[param_message.length-1] = output_message;
        return_message = getBodyOutputMessage(index_number, key,
            param_message);
        return return_message;
    }

    private void writeEOSObject(ObjectOutputStream object_output_stream,
        String message) {

        EOSObject eos_object = new EOSObject(message);
        try {
            object_output_stream.writeObject(eos_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEEOSOBJECT, 0, exception_object);
            return;
        }
    }
 
    private String getFormattedOutputMessage(int index_number, 
        String key, String... message) {
        String  return_message;

        return_message = getBodyOutputMessage(index_number, key, message);
        return return_message;
    }

    private String getBodyOutputMessage(int index_number, 
        String key, String[] message) {
        StringTokenizer  stk;
        String  order_index;
        String  return_message;
        int     message_index;

        if (key == null || message == null || message.length == 0) {
            return key;
        }
        if (index_number >= private_message_format_max) {
            order_index = new String();
            for (int i = 0; i < message.length; i++) {
                if (i > 0) {
                    order_index = order_index.concat(
                        ObjectServerHeader.PARAMETER_SEPARATOR);
                }
                order_index = order_index.concat(String.valueOf(i));
            }
        }
        if (private_message_format[index_number] == null) {
            try {
                private_message_format[index_number] 
                    = getMessageResource().getMessageFormat(key);
            } catch (Exception exception_object) {
                order_index = new String();
                for (int i = 0; i < message.length; i++) {
                    if (i > 0) {
                        order_index = order_index.concat(
                            ObjectServerHeader.PARAMETER_SEPARATOR);
                    }
                    order_index = order_index.concat(String.valueOf(i));
                }
                private_message_format[index_number] = order_index;
            }
        }
        order_index = private_message_format[index_number];
        return_message = new String();
        stk = new StringTokenizer( order_index, PARAMETER_SEPARATOR);

        while (stk.hasMoreTokens()) {
            try {
                message_index = Integer.valueOf(stk.nextToken());
            } catch (Exception exception_object) {
                continue;
            }
            if (message_index < message.length &&
                message[message_index] != null &&
                message[message_index].length() > 0) {
                if (message[message_index].charAt(0) == '\\') {
                    if ((message[message_index].length() > 1) 
                    && (message[message_index].charAt(1) != '\\')) {
                        continue;
                    }
                    return_message 
                    = return_message.concat(
                        message[message_index].substring(1));
                }
                return_message 
                    = return_message.concat(message[message_index]);
            }
        }
        return return_message;
    }

    private String getOutputMessage(int index_number, String message) {
        if (index_number >= private_message_max) {
            return message;
        }
        if (private_message[index_number] == null) {
            private_message[index_number] 
                = getMessageResource().getMessage(message);
        }
        return private_message[index_number];
    }

    private void updateDebugLevel() throws Exception {
        String vstring;
        int    debug;
        ObjectServerException   object_server_exception;

        vstring = getPropertyValue(OSERV_DEBUG_LEVEL_KEY);

        if (vstring != null && vstring.length() != 0) {
            try {
                debug = Integer.valueOf(vstring).intValue();
            } catch (Exception  exception_object) {
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                    ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION);
                throw object_server_exception;
            }
            setDebug(debug);
        } else {
            printKeyMessage(50, 50, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_SERVER);
        }
    }

    /**
     * This method serves printing usage.
     */
    private static void printUsage() {
        System.out.println(COMMAND_NAME + MESSAGE_SEPARATOR + USAGE_LINE);
    }

    /**
     * This method prints version information.
     */
    private static void printVersion() {
        System.out.println(VERSION_INFORMATION);
        return;
    }

    private String convertFilenameToEntry(String filename) {
        String entry;
        int    index;

        entry = filename;
        if (filename == null) {
            return entry;
        }
        index = filename.lastIndexOf(SUFFIX_SEPARATOR+JAR_SUFFIX);
        if (index >= 0) {
            entry = filename.substring(0, index);
        }
        return entry;
    }

    private String convertEntryToFilename(String entry) {
        String filename;

        if (entry == null) {
            return entry;
        }
        filename  = entry.concat(SUFFIX_SEPARATOR+JAR_SUFFIX);
        return filename;
    }

    private String convertEntryToFilepath(String entry) {
        String filepath;
        String filename;

        if (entry == null) {
            return entry;
        }
        filename = convertEntryToFilename(entry);
        filepath = convertFilenameToFilepath(persistent_path, filename);
        return filepath;
    }

    private String convertFilenameToFilepath(String base_path, String filename) {
        String filepath;

        if (filename == null || base_path == null) {
            return filename;
        }
        filepath = new String(base_path + File.separator + filename);
        return filepath;
    }

    private void shutdownSocket(Socket socket_object) {
        if (socket_object == null) {
            return;
        }
        if (socket_object.isConnected() == true) {
            try {
                socket_object.shutdownInput();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_SHUTDOWNSOCKET, 0, exception_object);
            }
        }
        if (socket_object.isConnected() == true) {
            try {
                socket_object.shutdownOutput();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_SHUTDOWNSOCKET, 0, exception_object);
            }
        }
    }

    private JarInputStream getJarInputStream(String filepath) 
        throws Exception {

        ZipEntry zipentry;
        FileInputStream file_input_stream;
        JarInputStream jar_input_stream;
        ObjectServerException    object_server_exception;

        try {
            file_input_stream = new FileInputStream(filepath);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETJARINPUTSTREAM, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            jar_input_stream = new JarInputStream(file_input_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETJARINPUTSTREAM, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_JAR_INPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_JAR_INPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        return jar_input_stream;
    }

    private void seekJarInputStream(
        JarInputStream jar_input_stream, String class_name, String filename) 
        throws Exception {
        int      idx;
        String   classpath;
        ZipEntry zipentry;
        ObjectServerException    object_server_exception;

        idx = class_name.lastIndexOf(directory_separator_key);
        classpath = class_name;
        if (idx < 0) {
            idx = filename.lastIndexOf(OBJECT_DOT_SEPARATOR + JAR_SUFFIX);
            if (idx >= 0) {
                idx = filename.substring(0, idx).lastIndexOf(OBJECT_DOT_SEPARATOR);
            } else {
                idx = filename.lastIndexOf(OBJECT_DOT_SEPARATOR);
            }
            if (idx >= 0) {
                classpath = filename.substring(0, idx) + directory_separator_key + class_name;
            }
        }

        printVariableMessage(87, 87, OSERV_DEBUG_TRACE_VALUE, ObjectServerDebug.DEBUG_JAR_NAME, classpath);
        try {
            zipentry = jar_input_stream.getNextEntry();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SEEKJARINPUTSTREAM, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_JAR_INPUT_STREAM_GET_NEXT_ENTRY_IO_EXCEPTION,
                ObjectServerError.JAVA_JAR_INPUT_STREAM_GET_NEXT_ENTRY_IO_EXCEPTION);
            throw object_server_exception;
        }
        while (zipentry != null) {
            printDebugMessage(OSERV_DEBUG_FULL_VALUE, getFormattedOutputMessage(88, 
                ObjectServerDebug.DEBUG_ENTRY_NAME,
                getOutputMessage(88, ObjectServerDebug.DEBUG_ENTRY_NAME),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                zipentry.getName(),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                classpath));
            if (classpath.compareTo(zipentry.getName()) == 0) {
                printVariableMessage(89, 89, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_MATCH_ENTRY, classpath);
                break;
            }
            zipentry = jar_input_stream.getNextEntry();
        }
    }

    private int getClassSizeInJar( String filepath, String class_name, String filename) throws Exception {

        int   size;
        byte[] buffer;
        int    idx;
        int    rcount;
        JarInputStream jar_input_stream;
        buffer = new byte[maximum];
        ObjectServerException    object_server_exception;

        try {
            jar_input_stream = getJarInputStream(filepath);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETCLASSSIZEINJAR, 0, exception_object);
            throw exception_object;
        }

        try {
            seekJarInputStream(jar_input_stream, class_name, filename);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETCLASSSIZEINJAR, 0, exception_object);
            throw exception_object;
        }
        size = 0;
        while (true) {
            try { 
                rcount = jar_input_stream.read(buffer, 0, (int)buffer.length);
            } catch (Exception exception_object) {
                if (exception_object != null && (exception_object instanceof EOFException) == true) {
                    break;
                }
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION,
                    ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION);
                throw object_server_exception;
            }
            if (rcount == -1) {
                break;
            }
            size = size + rcount;
        }
        try {
            jar_input_stream.close();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETCLASSSIZEINJAR, 0, exception_object);
        }
        return size;
    }

    /**
     * writeClassFile
     */
    private void writeClassFile(ObjectOutputStream parameter_output_stream, String class_name, String filename) throws Exception {
        File    file_descriptor;
        String  content;
        byte[] buffer;
        int    idx;
        int    rcount;
        Integer size_integer;
        String filepath;
        String classpath;
        ZipEntry zipentry;
        FileInputStream file_input_stream;
        JarInputStream jar_input_stream;
        NetworkObject    network_object;
        ObjectServerException   object_server_exception;

        if (filename == null) {
            object_server_exception
                = new ObjectServerException( getUserOutputMessage(730, 730,
	            ObjectServerError.FILENAME_PARAMETER_NULL_SERVER));
            throw object_server_exception;
        }
        if (class_name == null) {
            object_server_exception
                = new ObjectServerException( getUserOutputMessage(731, 731,
	            ObjectServerError.CLASSNAME_PARAMETER_NULL_SERVER));
            throw object_server_exception;
        }
        if (parameter_output_stream == null) {
            object_server_exception  
                = new ObjectServerException( getUserOutputMessage(732, 732,
	            ObjectServerError.OUTPUT_STREAM_PARAMETER_NULL_SERVER));
            throw object_server_exception;
        }
        buffer = new byte[maximum];
        printExecuteMessage(
           getOutputMessage(80, ObjectServerDebug.DEBUG_WRITECLASSFILE_SERVER) );
        filepath = convertFilenameToFilepath(persistent_path, filename);
        printVariableMessage(81, 81, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_FILE_NAME, filepath);
        file_descriptor = new File(filepath);
        if (file_descriptor.exists() == false) {
            printDebugMessage( OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(83,
                ObjectServerDebug.DEBUG_NOT_FOUND_KEY,
                getOutputMessage(83, ObjectServerDebug.DEBUG_WRITECLASSFILE_SERVER),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                getOutputMessage(133, ObjectServerDebug.DEBUG_NOT_FOUND_KEY)));
            try {
                parameter_output_stream.write(buffer, 0, 0);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
            }
            return;
        }

        try {
            size_integer = Integer.valueOf(
                getClassSizeInJar(filepath, class_name, filename));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION);
            throw object_server_exception;
        }

        try {
            parameter_output_stream.writeObject(size_integer);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }

        try {
            jar_input_stream = getJarInputStream(filepath);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
            throw exception_object;
        }

        try {
            seekJarInputStream(jar_input_stream, class_name, filename);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
            throw exception_object;
        }
        printKeyMessage(90, 90, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_OBJECT_READ_FILE_CONTENT );

        while (true) {
            try { 
                rcount = jar_input_stream.read(buffer, 0, (int)buffer.length);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
                if (exception_object != null && (exception_object instanceof EOFException) == true) {
                    break;
                }
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION,
                    ObjectServerError.JAVA_JAR_INPUT_STREAM_READ_IO_EXCEPTION);
                throw object_server_exception;
            }
            if (rcount == -1) {
                break;
            }
            try {
                parameter_output_stream.write(buffer, 0, rcount);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITECLASSFILE, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_IO_EXCEPTION);
                throw object_server_exception;
            }
        }
        return;
    }

    private void executeOServ(String type_string, String service_type, String entry_param, String service) throws ObjectServerException {
        URL      url;
        Method   method_object;
        int      idx;
        Class    class_object;
        ClassLoader    save_class_loader;
        Object   object_instance;
        String   variable_file;
        String   jar_file;
        String   filepath;
        String   archive_path;
        String   entry;
        Class[]  parameter_class = new Class[0];
        ObjectServerException   object_server_exception;
        Class     write_variable_class;
        WriteVariableWrapper write_variable_wrapper;

        if (entry_param == null) {
            printDebugMessage( OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(117,
                    ObjectServerDebug.DEBUG_PARAMETER_ERROR, 
                    getOutputMessage(117,
                        ObjectServerDebug.DEBUG_PARAMETER_ERROR) ));

            object_server_exception = getObjectServerException(
                ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                ObjectServerException.PARAMETER_ERROR_EXCEPTION);
            throw object_server_exception;
        }
        printExecuteMessage( getOutputMessage(130, TRACE_EXECUTEOSERV),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, entry_param);

        save_class_loader = null;
        entry = entry_param;
        idx = entry.lastIndexOf(SUFFIX_SEPARATOR + JAR_SUFFIX);
        if (idx > 0) {
            entry = entry.substring(0, idx);
        }
        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            variable_file = convertFilenameToFilepath(persistent_path, entry +
                OBJECT_DOT_SEPARATOR + VARIABLE_SUFFIX);
            jar_file = convertFilenameToFilepath(persistent_path, entry +
                OBJECT_DOT_SEPARATOR + JAR_SUFFIX);
        } else {
            variable_file = convertFilenameToFilepath(transient_path, entry +
                OBJECT_DOT_SEPARATOR + VARIABLE_SUFFIX);
            jar_file = convertFilenameToFilepath(transient_path, entry +
                OBJECT_DOT_SEPARATOR + JAR_SUFFIX);
        }
        archive_path = jar_file;

        filepath = new String(JAR_URL_HEAD + jar_file + JAR_URL_END);
        printDebugMessage( OSERV_DEBUG_FULL_VALUE,
            getFormattedOutputMessage(118, ObjectServerDebug.DEBUG_SERVICE_NAME,
                getOutputMessage(118, ObjectServerDebug.DEBUG_SERVICE_NAME), 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, service, ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                getOutputMessage(113, ObjectServerDebug.DEBUG_FILE_NAME), 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, variable_file, ObjectServerDebug.DEBUG_BLANK_STRING_KEY,entry));
        try {
            url = new URL(filepath);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);

            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_URL_EXCEPTION,
                ObjectServerError.JAVA_URL_EXCEPTION);
            throw object_server_exception ;
        }

        URL[] urls = new URL[1];
        urls[0] = url;
        URLClassLoader uloader = new URLClassLoader(urls);      
        printVariableMessage(120, 120, OSERV_DEBUG_TRACE_VALUE, ObjectServerDebug.DEBUG_URL_KEY, url.toString());

        save_class_loader = getContextClassLoader();
        setContextClassLoader(uloader);

        UserClassLoader  user_class_loader;
        user_class_loader = new UserClassLoader();
        user_class_loader.setMessageResource(getMessageResource());

        try {
            class_object = user_class_loader.readUserClass(archive_path,
                service);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_INVOKE, 0, exception_object);
            return;
        }
        if (class_object == null) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION,
                ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION);
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, object_server_exception);
            return;
        }

        Class base_variable_class;
        try {
            base_variable_class = 
                user_class_loader.getSpecifiedClass(OSERV_BASE_VARIABLE,
                BaseVariableData.getClassData());
            write_variable_class = 
                user_class_loader.getSpecifiedClass(OSERV_WRITE_VARIABLE,
                WriteVariableData.getClassData());
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
            if (save_class_loader != null) {
                setContextClassLoader(save_class_loader);
                save_class_loader = null;
            }
            object_server_exception = new ObjectServerException( exception_object );
            throw object_server_exception;
        }
        try {
            write_variable_wrapper 
                = new WriteVariableWrapper(
                    write_variable_class);

            write_variable_wrapper.setMessageResource(
                    getMessageResource() );
            if (getDebug() > 0) {
                write_variable_wrapper.setDebug(getDebug() );
                write_variable_wrapper.setLogStream(getLogStream());
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
            if (save_class_loader != null) {
                setContextClassLoader(save_class_loader);
                save_class_loader = null;
            }
            object_server_exception = new ObjectServerException( exception_object );
            throw object_server_exception ;
        }

        try {
            method_object = class_object.getMethod(OSERV_METHOD_NAME, parameter_class);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
            return;
        }
        if (method_object == null) {
            object_server_exception = new ObjectServerException(
                ObjectServerError.JAVA_CLASS_GET_METHODS_EXCEPTION);
            printExceptionMessage(TRACE_EXECUTEOSERV,0,object_server_exception);
            return;
        }
        try {
            object_instance = class_object.newInstance();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_EXCEPTION,
                ObjectServerError.JAVA_CLASS_NEW_INSTANCE_EXCEPTION);
            throw object_server_exception ;
        }
        
        ObjectServerSecurityManager oserv_security_manager;
        SecurityManager original;
        original = System.getSecurityManager();

        String request_user_path;

        if (type_string.compareTo(RegisterCommand.PERSISTENT_TYPE) == 0) {
            if (service_type.compareTo(ThreadTable.SERIALIZABLE_SERVICE) == 0) {
                request_user_path = user_path + File.separator + service;
            } else {
                request_user_path = user_path + File.separator + service
                      + getThreadId(Thread.currentThread().getName());
            }
        } else {
            request_user_path = user_path + File.separator 
                    + convertFilenameToFilepath(transient_path, service);
        }

        File request_user_path_file;
        request_user_path_file = new File(request_user_path);
        if (request_user_path_file.exists() == false) {
            request_user_path_file.mkdir();
        }
        System.clearProperty(USER_DIR_KEY);
        System.setProperty(USER_DIR_KEY, request_user_path);

        printDebugMessage( OSERV_DEBUG_FULL_VALUE,
            getFormattedOutputMessage(119,
                ObjectServerDebug.DEBUG_DIRECTORY_PATH, 
                getOutputMessage(119,
                       ObjectServerDebug.DEBUG_DIRECTORY_PATH),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            request_user_path));

        oserv_security_manager = new ObjectServerSecurityManager(request_user_path); 
        System.setSecurityManager(oserv_security_manager);
        oserv_security_manager.toUserMode();

        PrintStream sysout;
        PrintStream save_out;
        PrintStream save_err;
        NullOutputStream null_output_stream;
        null_output_stream = new NullOutputStream();
        sysout = new PrintStream(null_output_stream);
        save_out = System.out;
        save_err = System.err;
        System.out.flush();
        System.err.flush();

        printKeyMessage(125,125,OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_SWITCH_OUTPUT);
        System.setOut(sysout);
        System.setErr(sysout);
        try {
            method_object.invoke(object_instance, (Object[])null);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_METHOD_INVOKE_EXCEPTION,
                ObjectServerError.JAVA_METHOD_INVOKE_EXCEPTION);
            throw object_server_exception ;
        }
        System.out.flush();
        System.err.flush();

        System.setOut(save_out);
        System.setErr(save_err);

        oserv_security_manager.toSuperMode();
        System.setSecurityManager(original);
        System.clearProperty(USER_DIR_KEY);

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            try {
                write_variable_wrapper.writeVariable(
                    variable_file, class_object, object_instance);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
                object_server_exception = new ObjectServerException(exception_object);
                throw object_server_exception;
            }
        }
        if (save_class_loader != null) {
            setContextClassLoader(save_class_loader);
            save_class_loader = null;
        }
    }


    /**
     * This method provides a feature for reading object table.
     *
     * @param     String  name of object table file
     * @return    void
     * @throws    Exception
     */
    private void readObjectTable(String objtbl) throws Exception {
        FileInputStream file_input_stream;
        ObjectInputStream object_input_stream;
        Object    s1;
        String    filepath;
        int       count;
        Object    object_value;
        String    argument;
        ObjectTable  clone_table;
        ObjectServerException    object_server_exception;

        if (objtbl == null) {
            object_server_exception 
                = new ObjectServerException( getUserOutputMessage(206, 206,
	        ObjectServerError.OBJECT_TABLE_PARAMETER_NULL_SERVER));
            throw object_server_exception;
        }
        printExecuteMessage( getOutputMessage(135, DEBUG_READOBJECTTABLE),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            getFormattedOutputMessage(139, ObjectServerDebug.DEBUG_TABLE_NAME,
                getOutputMessage(139, ObjectServerDebug.DEBUG_TABLE_NAME),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, objtbl));
        try {
            file_input_stream = new FileInputStream(objtbl);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        printLogCallMessage(
                    ObjectServerDebug.DEBUG_JAVA_OBJECTINPUTSTREAM );
        printVariableMessage(182,182,  OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME,
                    file_input_stream.toString());
        try {
            object_input_stream = new ObjectInputStream((InputStream)file_input_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_INPUT_STREAM_IO_EXCEPTION);
            throw exception_object;
        }    
        printDebugMessage( OSERV_DEBUG_FULL_VALUE,
            getFormattedOutputMessage(76, ObjectServerDebug.DEBUG_CREATE_KEY,
                getOutputMessage(76, ObjectServerDebug.DEBUG_CREATE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                ObjectServerDebug.DEBUG_JAVA_OBJECTINPUTSTREAM,
                getOutputMessage(79, ObjectServerDebug.DEBUG_CODE_001)),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                object_input_stream.toString());
        Integer integer_oserv_size;
        Integer integer_oserv_number;
        try {
            object_value = object_input_stream.readObject();
            if (object_value != null && (object_value instanceof Integer) != false) {
                printVariableMessage(158, 158, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME,object_value.toString());
            }
            integer_oserv_number = (Integer)object_value;
            oserv_number = integer_oserv_number.intValue();
        } catch (Exception exception_object) {
            try {
                object_input_stream.close();
            } catch (Exception eep) {
                printExceptionMessage(TRACE_READOBJECTTABLE, 0, eep);
            }    
            printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
            throw exception_object;
        }    

        printVariableMessage(141,141,OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_READ_TABLE_NUMBER, integer_oserv_number.toString());
        count = 0;
        for (int i = 0; i < oserv_number; i++) {
            String entry_name;
            File   fdes;

            argument = null;
            try {
                object_value = (String)object_input_stream.readObject();
                if (object_value != null && (object_value instanceof String) != false) {
                    printVariableMessage(154,154,OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        object_value.toString());
                }
                entry_name = (String)object_value;
                object_value = (String)object_input_stream.readObject();
                if (object_value != null && (object_value instanceof String) != false) {
                    printVariableMessage(153,153,OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        object_value.toString());
                }
                argument = (String)object_value;
            } catch (Exception exception_object) {
                try {
                    object_input_stream.close();
                } catch (Exception eep) {
                    printExceptionMessage(TRACE_READOBJECTTABLE, 0, eep);
                }    
                printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
                throw exception_object;
            }
            printVariableMessage(144, 144, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_ENTRY_NAME, entry_name);

            filepath = convertFilenameToFilepath(
                persistent_path,
                entry_name + SUFFIX_SEPARATOR+JAR_SUFFIX);

            printVariableMessage(145, 145, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_FILE_NAME, filepath);

            fdes = new File(filepath);
            if (fdes.exists() == false) {
                printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                    getFormattedOutputMessage(149,
                        ObjectServerDebug.DEBUG_FILE_NOT_EXIST,
                        getOutputMessage(149,
                            ObjectServerDebug.DEBUG_FILE_NOT_EXIST)));
                continue;
            }
            printLogCallMessage( ObjectTable.ADD_OBJECT_TABLE,
                entry_name, ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    argument);
            try {
                clone_table = operateObjectTable(ObjectTable.ADD_OBJECT_TABLE, entry_name, argument);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
                continue;
            }
            try {
                executeOServ(RegisterCommand.PERSISTENT_TYPE, 
                    clone_table.getType(clone_table.getCount()-1),
                    entry_name, entry_name);

            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
                if (exception_object.getMessage() == null ||
                    exception_object.getMessage().compareTo(
                    ObjectServerError.JAVA_CLASS_GET_METHODS_EXCEPTION) != 0) {
                    try {
                        operateObjectTable(ObjectTable.DELETE_OBJECT_TABLE, entry_name, null);
                    } catch (Exception second_exception_object) {
                        printExceptionMessage(TRACE_READOBJECTTABLE, 0, second_exception_object);
                    }
                    continue; 
                }
            }
        }
        try {
            object_input_stream.close();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_READOBJECTTABLE, 0, exception_object);
            throw exception_object;
        }    
        return;
    }

    /**
     * This method provides a feature for writing Object Table
     * Object table contain service entry name
     *
     * @param     String name of object table file
     * @return    void
     * @throws    Exception
     */
    private void writeObjectTable(String objtbl) throws Exception {
        File             fdes;
        FileOutputStream file_output_stream;
        ObjectServerException   object_server_exception;

        if (objtbl == null) {
             object_server_exception
                = new ObjectServerException( getUserOutputMessage(438, 438,
	        ObjectServerError.OBJECT_TABLE_PARAMETER_NULL_SERVER));

            printVariableMessage(161, 161, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_OBJECT_SERVER_EXCEPTION_KEY, object_server_exception.getMessage());
            throw object_server_exception;
        }
        fdes = new File(objtbl);
        if (fdes.exists() == true) {
            printVariableMessage(151, 151, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_DELETE_KEY, objtbl);

            fdes.delete();
        }
        try {
            file_output_stream = new FileOutputStream(fdes);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOBJECTTABLE, 1, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            writeObjectTable((OutputStream)file_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOBJECTTABLE, 1, exception_object);
            throw exception_object;
        }
    }

    private void writeObjectTableEntry(ObjectOutputStream object_output_stream) throws Exception {
        int  count;
        String entry;
        String type;
        ObjectServerException object_server_exception;

        if (object_output_stream == null) {
            object_server_exception
                = new ObjectServerException( getUserOutputMessage(579, 579,
	        ObjectServerError.OBJECT_OUTPUT_STREAM_PARAMETER_NULL_SERVER));
            throw (Exception) object_server_exception;
        }
        ObjectTable clone_table;
        try {
            clone_table = operateObjectTable(ObjectTable.READ_OBJECT_TABLE, null, null);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOBJECTTABLEENTRY, 0, exception_object);
            throw exception_object;
        }
        
        count = clone_table.getCount();

        Integer integer_oserv_number = new Integer(count);
        try {
            object_output_stream.writeObject(integer_oserv_number);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOBJECTTABLEENTRY, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }    
        for (int i = 0; i < count;i++) {
            try {
                entry = clone_table.get(i);
                object_output_stream.writeObject(entry);

                type = clone_table.getType(i);
                object_output_stream.writeObject(type);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITEOBJECTTABLEENTRY, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }    
        }
    }
    /**
     * This method is essential body of writing object table.
     *
     * @param      OutputStream
     * @return     void    
     * @throws     Exception
     */
    private void writeObjectTable(OutputStream parameter_output_stream) throws Exception {
        ObjectOutputStream object_output_stream;
        int  count;
        ObjectServerException object_server_exception;

        if (parameter_output_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(40, 40,
	        ObjectServerError.OBJECT_TABLE_PARAMETER_NULL_SERVER));

            printVariableMessage(168, 168, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_OBJECT_SERVER_EXCEPTION_KEY, 
                ose.getMessage());

            throw ose;
        }
        try {
            object_output_stream = new ObjectOutputStream(parameter_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOBJECTTABLE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }    
        try {
            writeObjectTableEntry(object_output_stream);
            object_output_stream.close();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOBJECTTABLE, 0, exception_object);
            throw exception_object;
        }    
    }

    private String getReturnTypes(String command) {
        int      block_index;
        String[] blocks;
        StringTokenizer string_tokenizer;

        string_tokenizer = new StringTokenizer(command, OBJECT_SEPARATOR);
        blocks = new String[string_tokenizer.countTokens()];
        block_index = 0;
        while (string_tokenizer.hasMoreTokens()) {
            blocks[block_index] = (String)string_tokenizer.nextToken();
            
            if (blocks[block_index] != null) {
                if (blocks[block_index].compareTo(BLANK_STRING) == 0) {
                    blocks[block_index] = null;
                }
            }
            block_index++;
        }
        if (blocks.length < 4) {
            return null;
        }
        return blocks[3]; 
    }

    private String getParameterTypes(String command) {
        int      block_index;
        String[] blocks;
        StringTokenizer string_tokenizer;

        string_tokenizer = new StringTokenizer(command, OBJECT_SEPARATOR);
        blocks = new String[string_tokenizer.countTokens()];
        block_index = 0;
        while (string_tokenizer.hasMoreTokens()) {
            blocks[block_index] = (String)string_tokenizer.nextToken();
            
            if (blocks[block_index] != null) {
                if (blocks[block_index].compareTo(BLANK_STRING) == 0) {
                    blocks[block_index] = null;
                }
            }
            block_index++;
        }
        if (blocks.length < 5) {
            return null;
        }
        return blocks[4]; 
    }

    private Object transferService(URI service_entry, String command, 
	   RetryObjectInputStream  parameter_retry_object_input_stream,
           ObjectOutputStream object_output_stream) throws Exception {
        URI    request;
        Object rt;
        Object obj;
        Object object_inst;
        String host_address;
        String port_number;
        int    iport;
        OutputStream socket_output_stream;
        InputStream  socket_input_stream;
        ObjectOutputStream socket_object_output_stream;
        RetryObjectInputStream retry_object_input_stream;
        Socket csocket;
        InetSocketAddress inet_socket_address;
        String service_entry_string;
        ArgumentInputStream ain;
        VariableByte variable_byte;
        ObjectServerException   object_server_exception;

        String[] ptypes;
        String ptype_string;
        StringTokenizer string_tokenizer;
        int    ptype_index;

        if (service_entry == null) {
            ObjectServerServiceUnavailableException ose 
                = new ObjectServerServiceUnavailableException(
                getUserOutputMessage(514, 514,
                ObjectServerException.SERVICE_NOT_AVAILABLE));
            throw (Exception)ose;
        }
        host_address = service_entry.getHost();
        iport  = service_entry.getPort();
        ConnectedSocket connected_socket;
        connected_socket = new ConnectedSocket();
        if (getDebug() > 0) {
            connected_socket.setDebugLogStream(getDebug(), getLogStream());
        }
        connected_socket.setMessageResource(getMessageResource());
        try {
            connected_socket.connect(host_address, iport);  
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_SOCKET_CONNECT_IO_EXCEPTION,
                ObjectServerError.JAVA_SOCKET_CONNECT_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            socket_object_output_stream = connected_socket.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        printVariableMessage(233, 233, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_REQUEST_CONNECTION_START_KEY,
	        connected_socket.toString());

        printVariableMessage(181, 181, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_REQUEST_KEY,
            OSERV_COMMAND_TYPE_REQUEST_KEY);

        printVariableMessage(510, 510, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_COMMAND_KEY, service_entry.toString());

        if (parameter_retry_object_input_stream == null) {
            try {
                socket_object_output_stream.writeObject(OSERV_COMMAND_TYPE_FORWARD_KEY);
                socket_object_output_stream.writeObject(service_entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
                try {
                    connected_socket.close();
                } catch (IOException io_exception) {
                }
                printVariableMessage(179, 179, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_COMMAND_KEY, service_entry.toString());
                printVariableMessage(190, 190, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
            printVariableMessage(511, 511, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_COMMAND_KEY, service_entry.toString());
            try {
                retry_object_input_stream = connected_socket.getInputStream();
            } catch (Exception exception_object) {
                printKeyMessage(513, 513, OSERV_DEBUG_TRACE_VALUE, ObjectServerDebug.DEBUG_JAVA_SOCKET_GETINPUTSTREAM);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION);
                throw object_server_exception;
            }
            ptype_string = getParameterTypes(command);
            if (ptype_string != null) {
                while (true) {
                    try {
                        object_inst = retry_object_input_stream.readObject();
                        if (object_inst != null && (object_inst instanceof VariableByte) != true) {
                            printKeyMessage(71, 71, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_DATA_CORRUPTION);
                            break;
                        }
                        variable_byte = (VariableByte)object_inst;
                    } catch (Exception exception_object) {
                        printKeyMessage(512, 512, OSERV_DEBUG_TRACE_VALUE, ObjectServerDebug.DEBUG_DATA_CORRUPTION);
                        break;
                    }
                    try {
                        socket_object_output_stream.writeObject(variable_byte);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
                        break;
                    }
                }
            }
            return null;
        } 
        try {
            socket_object_output_stream.writeObject( OSERV_COMMAND_TYPE_REQUEST_KEY );
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);

            try {
                connected_socket.close();
            } catch (IOException io_exception) {
                printExceptionMessage(TRACE_TRANSFERSERVICE, 0, io_exception);
            }
            printVariableMessage(55, 55, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }

        service_entry_string = service_entry.toString();

        printVariableMessage(328, 328, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_COMMAND_KEY, command);

        ptype_string = getParameterTypes(command);
        ptype_index = 0;
        if (ptype_string != null) {
            string_tokenizer = new StringTokenizer(ptype_string, PARAMETER_SEPARATOR);
            ptypes = new String[string_tokenizer.countTokens()];
            while (string_tokenizer.hasMoreTokens()) {
                ptypes[ptype_index] = (String)string_tokenizer.nextToken();
                ptype_index++;
            }
        } else {
            ptypes = new String[0];
        }
        printVariableMessage(180, 180, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_COUNT_KEY, String.valueOf(ptypes.length));

        try {
            socket_object_output_stream.writeObject(command);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
            ObjectServerException ose 
                = new ObjectServerException(exception_object);

            try {
                connected_socket.close();
            } catch (IOException io_exception) {
                printExceptionMessage(TRACE_TRANSFERSERVICE, 0, io_exception);
            }
            printVariableMessage(635, 635, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());
            throw ose;
        }

        if (ptype_string != null) {
            NetworkObject network_object;
            Integer       count_object;

            printVariableMessage(530, 530, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_COUNT_KEY, String.valueOf(ptypes.length));

            for (int i = 0; i < ptypes.length; i++) {
                try {
                    object_inst = parameter_retry_object_input_stream.retryReadObject();
                    printVariableMessage(634, 634, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_OBJECT_NAME, object_inst.toString() );
                    if (object_inst != null) {
                        if ((object_inst instanceof SODObject) == true) {
                            try {
                                transferStream(parameter_retry_object_input_stream,
                                    socket_object_output_stream);
                            } catch (Exception exception_object) {
                               throw exception_object;
                            }
                        } else {
                            try {
                                socket_object_output_stream.writeObject(object_inst);
                                socket_object_output_stream.flush();
                           } catch (Exception exception_object) {
                               printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
                               throw exception_object;
                           }
                        }
                    }
                    
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
                    ObjectServerException ose 
                        = new ObjectServerException(exception_object);

                    try {
                        connected_socket.close();
                    } catch (IOException io_exception) {
                        printExceptionMessage(TRACE_TRANSFERSERVICE, 0, io_exception);
                    }

                    printVariableMessage(236, 236, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());
                    throw ose;
                }
            }
            printTraceMessage(TRACE_TRANSFERSERVICE, 0, 
                getFormattedOutputMessage(237, ObjectServerDebug.DEBUG_END_KEY,
                    getOutputMessage(237, ObjectServerDebug.DEBUG_END_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    getOutputMessage(241, ObjectServerDebug.DEBUG_PARAMETER_TRANSFER)));
        }

        printLogCallMessage(ObjectServerDebug.DEBUG_JAVA_SOCKET_GETINPUTSTREAM);

        try {
            retry_object_input_stream = connected_socket.getInputStream();
        } catch (Exception exception_object) {
            try {
                connected_socket.shutdownSocket();
            } catch (IOException io_exception) {
                printExceptionMessage(TRACE_TRANSFERSERVICE, 0, io_exception);
            }

            printVariableMessage(436, 436, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());
            printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);

            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            rt = retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printVariableMessage(437, 437, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());
            printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
            throw exception_object;
        }
        if (rt != null) {
            printTraceMessage(TRACE_TRANSFERSERVICE, 0, rt.toString());
            if ((rt instanceof SODObject) != true) {
                try {
                    object_output_stream.writeObject(rt);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
                }
            }
        } else {
            printTraceMessage(TRACE_TRANSFERSERVICE, 0);
        }
        if (rt != null && (rt instanceof SODObject) == true) {
            try {
                object_output_stream.writeObject(rt);

                while (true) {
                    try {
                        obj = retry_object_input_stream.retryReadObject();
                    } catch (Exception exception_object) {
                        break;
                    }
                    if (obj != null && (obj instanceof EODObject) == true) {
                        printTraceMessage(TRACE_TRANSFERSERVICE, 0, obj.toString());
                        rt = obj;
                        break;
                    }
                    printTraceMessage(TRACE_TRANSFERSERVICE, 0, obj.toString());
                    object_output_stream.writeObject(obj);
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_TRANSFERSERVICE, 0, exception_object);
            }
        }
        if (rt != null) {
            printVariableMessage(344, 344, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_RETURN_KEY, rt.toString());
        }
        printVariableMessage(243, 243, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_CONNECTION_END_KEY, connected_socket.toString());

        return rt;
    }

    private boolean checkForwardService(URI service_entry) 
        throws Exception {
        URI    target_entry;
        String service_entry_string;
        String target_entry_string;

        if (service_entry == null) {
            return false;
        }
        for (int i = 0; i < forward_service_count; i++) {
            target_entry = forward_service_table[i];
            target_entry_string = target_entry.toString();
            service_entry_string = service_entry.toString();
            if (target_entry_string.compareTo(service_entry_string) == 0) {
                return true;
            }
        }
        return false;
    }

    private String getForwardService(URI service_entry) throws Exception {
        String service;

        if (service_entry == null) {
            return null;
        }
        service = service_entry.getPath();
        if (service.indexOf(DIRECTORY_SEPARATOR) == 0) {
            service = service.substring(DIRECTORY_SEPARATOR.length());
        }
        return service;
    }

    private boolean isResetForwardService(URI service_entry) throws Exception {
        String path;
        String query_string;

        printExecuteMessage(
            getOutputMessage(193, ObjectServerDebug.DEBUG_FORWARD_IS_RESET_REQUEST) );
        if (service_entry == null) {
            printTraceMessage(TRACE_ISRESETFORWARDSERVICE, 0, ObjectServerDebug.DEBUG_FORWARD_IS_RESET_REQUEST,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, String.valueOf(Boolean.FALSE));
            return false;
        }
        query_string = service_entry.getQuery();
        printTraceMessage(TRACE_ISRESETFORWARDSERVICE, 0, ObjectServerDebug.DEBUG_FORWARD_IS_NOT_RESET,
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, query_string);

        if (query_string != null && query_string.length() > 0) {
            printTraceMessage(TRACE_ISRESETFORWARDSERVICE, 0, ObjectServerDebug.DEBUG_FORWARD_IS_NOT_RESET,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, query_string);
            return false;
        }

        path = service_entry.getPath();
        if (path == null || path.compareTo(DIRECTORY_SEPARATOR) == 0) {
            printTraceMessage(TRACE_ISRESETFORWARDSERVICE, 0, ObjectServerDebug.DEBUG_FORWARD_IS_RESET_REQUEST_TRUE);
            return true;
        }
        printTraceMessage(TRACE_ISRESETFORWARDSERVICE, 0, ObjectServerDebug.DEBUG_FORWARD_IS_NOT_RESET);
        return false;
    }
    

    private void resetForwardService(URI service_entry) 
        throws Exception {
        int count;
        URI    target_entry;
        String service_entry_string;
        String target_entry_string;
        String service_host_entry;
        String target_host_entry;
        String service_port_entry;
        String target_port_entry;

        printExecuteMessage( DEBUG_RESETFORWARDSERVICE );
        if (service_entry == null) {
            printVariableMessage(197, 197, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_KEY, DEBUG_NULL);
            return;
        }
        printVariableMessage(198, 198, OSERV_DEBUG_FULL_VALUE, ObjectServerDebug.DEBUG_REQUEST_KEY, service_entry.toString());
        count = 0;
        service_host_entry = service_entry.getHost();
        service_port_entry = String.valueOf(service_entry.getPort());
        for (int i = 0; i < forward_service_count; i++) {
            target_entry = forward_service_table[i] ;
            target_host_entry = target_entry.getHost();
            target_port_entry = String.valueOf(target_entry.getPort());
            if (target_host_entry.indexOf(service_host_entry) == 0
                && target_port_entry.indexOf(service_port_entry) == 0 ) {
                count = count + 1;
            }
        }
        URI[] bk;
        int new_forward_service_count;
        bk = new URI[forward_service_count - count];
        new_forward_service_count = 0;
        for (int i = 0; i < forward_service_count; i++) {
            target_entry = forward_service_table[i] ;
            target_host_entry = target_entry.getHost();
            target_port_entry = String.valueOf(target_entry.getPort());
            if (target_host_entry.indexOf(service_host_entry) == 0
                && target_port_entry.indexOf(service_port_entry) == 0 ) {
                continue;
            }
            bk[new_forward_service_count] = forward_service_table[i];
            new_forward_service_count = new_forward_service_count + 1;
        }
        forward_service_table = bk;
        forward_service_count = new_forward_service_count;
    }

    /**
     * makeForwardServerTable(String entries) 
     */
    private void makeForwardServerTable(String entries) {
        int    count;
        String entry;
        StringTokenizer   string_tokenizer;

        printTraceMessage(TRACE_MAKEFORWARDSERVERTABLE, 0, DEBUG_MAKEFORWARDSERVERTABLE);
        if (entries == null) {
            printTraceMessage(TRACE_MAKEFORWARDSERVERTABLE, 0, DEBUG_MAKEFORWARDSERVERTABLE,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, DEBUG_NULL);
            return;
        }
        string_tokenizer = new StringTokenizer(entries, ITEM_SEPARATOR);
        count = string_tokenizer.countTokens();
        forward_server_table = new URI[count];
        forward_server_count = 0;
        for (int i = 0; i < count; ) {
            entry = string_tokenizer.nextToken();
            try {
                forward_server_table[i] = new URI(entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_MAKEFORWARDSERVERTABLE, 0, exception_object);
                count--;
                continue;
            }
            forward_server_count++;
            i++;
        }
    }

    /**
     * makeForwardClientTable(String entries) 
     */
    private void makeForwardClientTable(String entries) {
        int    count;
        String entry;
        StringTokenizer   string_tokenizer;

        printExecuteMessage(
                getOutputMessage(600, DEBUG_MAKEFORWARDCLIENTTABLE) );
        if (entries == null) {
            printTraceMessage(TRACE_MAKEFORWARDCLIENTTABLE, 0, DEBUG_MAKEFORWARDCLIENTTABLE, 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, DEBUG_NULL);
            return;
        }
        string_tokenizer = new StringTokenizer(entries, ITEM_SEPARATOR);
        count = string_tokenizer.countTokens();
        forward_client_table = new URI[count];
        forward_client_count = 0;
        for (int i = 0; i < count; ) {
            entry = string_tokenizer.nextToken();
            if (entry == null || entry.length() == 0) {
                count--;
                continue;
            }
            try {
                forward_client_table[i] = new URI(entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_MAKEFORWARDCLIENTTABLE, 0, exception_object);
                count--;
                continue;
            }
            forward_client_count++;
            i++;
        }
    }

    /**
     * readForwardServerEntry()
     */
    private URI readForwardServerEntry(int index) {
        URI  uri_entry;

        uri_entry = null;
        if (forward_server_count <= index || index < 0) {
            return uri_entry;
        }
        uri_entry = forward_server_table[index];
        return uri_entry;
    }

    /**
     * readForwardClientEntry()
     */
    private URI readForwardClientEntry(int index) {
        URI  uri_entry;

        uri_entry = null;
        if (forward_client_count <= index || index < 0) {
            return uri_entry;
        }
        uri_entry = forward_client_table[index];
        return uri_entry;
    }

    /**
     * readForwardServerCount()
     */
    private int readForwardServerCount() {
        return forward_server_count;
    }

    /**
     * readForwardClientCount()
     */
    private int readForwardClientCount() {
        return forward_client_count;
    }

    private Object readForwardRequestsFromSocket(
	RetryObjectInputStream retry_object_input_stream,
        ObjectOutputStream object_output_stream,
        String  request_host_address) throws Exception {

        URI    service_entry;
        Object rt;
        File   fd;
        Object     object_value;
        FileOutputStream  file_output_stream;
        ObjectOutputStream file_object_output_stream;

        printExecuteMessage(
                getOutputMessage(599, DEBUG_READFORWARDREQUESTSFROMSOCKET) );
   
        while (true) {
            try {
                object_value = retry_object_input_stream.retryReadObject();
                if (object_value != null && (object_value instanceof URI) == false) {
                    printVariableMessage(121, 121, OSERV_DEBUG_FULL_VALUE, 
                        ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
                    continue;
                }
                service_entry = (URI)object_value;
            } catch (Exception exception_object) {
                if (exception_object != null && (exception_object instanceof EOFException) == true) {
                    printEndMessage(OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_SOCKET);
                    break;
                }
                printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                break;
            }
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                getLogStream().printLogln( getFormattedOutputMessage(207,
                    ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_SOCKET,
                    getOutputMessage(207, ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_SOCKET),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ObjectServerDebug.DEBUG_COMMAND_KEY,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    service_entry.toString()));
            }
            if (isCommandForwardService(service_entry) == true) {
                printTraceMessage( TRACE_READFORWARDREQUESTSFROMSOCKET, 0,
                        getOutputMessage(150, DEBUG_DOCOMMANDFORWARDSERVICE));
                try {
                    rt = doCommandForwardService(service_entry, 
                        retry_object_input_stream, object_output_stream);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                    rt = exception_object;
                }
                return rt;
            }

            while (true) {
                if (isResetForwardService(service_entry) == true) {
                    printLogCallMessage(DEBUG_RESETFORWARDSERVICE);
                    resetForwardService(service_entry);
                } else { 
                    printLogCallMessage(DEBUG_SETFORWARDSERVICEENTRY);
                    setForwardServiceEntry(service_entry);
                }
                try {
                    object_value = retry_object_input_stream.retryReadObject();
                    if (object_value != null && (object_value instanceof URI) == false) {
                        printVariableMessage(122, 122, OSERV_DEBUG_FULL_VALUE,
                            ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
                        continue;
                    }
                    service_entry = (URI)object_value;
                } catch (Exception exception_object) {
                    if (exception_object != null && (exception_object instanceof EOFException) == true) {
                        printEndMessage(OSERV_DEBUG_FULL_VALUE,
                            ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_SOCKET);
                        break;
                    }
                    printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                    break;
                }
                if (service_entry == null) { break; }
                printDebugMessage( OSERV_DEBUG_TRACE_VALUE,
                    getFormattedOutputMessage(184,
                    ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_SOCKET,
                    getOutputMessage(184, ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_SOCKET),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ObjectServerDebug.DEBUG_COMMAND_KEY,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    service_entry.toString()));
            }
            writeEOSObject(object_output_stream, EOSObject.END_OF_SOCKET_MESSAGE);
            if (forward_service_count > 0) {
                printVariableMessage(217, 217, OSERV_DEBUG_FULL_VALUE, 
                    ObjectServerDebug.DEBUG_FILE_NAME, forward_file);
                try {
                    fd = new File(forward_file);
                    if (fd.exists() == true) {
                        printVariableMessage(106, 106, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_DELETE_KEY, forward_file);
                        fd.delete();
                    }
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                    break;
                }
                try {
                    file_output_stream = new FileOutputStream(fd);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                    break;
                }
                try {
                    file_object_output_stream = new ObjectOutputStream(file_output_stream);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                    break;
                }
                printTraceMessage( TRACE_READFORWARDREQUESTSFROMSOCKET, 0,
                    getOutputMessage(194,ObjectServerDebug.DEBUG_FORWARD_WRITE_FORWARD_FILE),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, forward_file);
                try {
                    writeForwardTable(file_object_output_stream);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_READFORWARDREQUESTSFROMSOCKET, 0, exception_object);
                }
            }
            break;
        }
        return null;
    }

    /**
     * readForwardTable
     */
    private Object readForwardTable(ObjectInputStream object_input_stream) 
        throws Exception {
        URI    service_entry;
        Object rt;
        Object object_value;

        if (object_input_stream == null) {
            printKeyMessage(221, 221, OSERV_DEBUG_FULL_VALUE,
                ObjectServerError.READ_FORWARD_TABLE_PARAMETER_ERROR_SERVER);

            return null;
        }
        while (true) {
            try {
                object_value = object_input_stream.readObject();
                if (object_value != null && (object_value instanceof URI) == false) {
                    printVariableMessage(124, 124, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
                    continue;
                }
                service_entry = (URI)object_value;
            } catch (Exception exception_object) {
                if (exception_object != null && (exception_object instanceof EOFException) == true) {
                    break;
                }
                printExceptionMessage(TRACE_READFORWARDTABLE, 0, exception_object);
                throw exception_object;
            }
            printTraceMessage(TRACE_READFORWARDTABLE, 0,  
                getOutputMessage(388, ObjectServerDebug.DEBUG_FORWARD_READ_ENTRY_FROM_FILE),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, service_entry.toString());
            setForwardServiceEntry(service_entry);
        }
        return null;
    }

    private void setForwardServiceEntry(URI service_entry) throws Exception {
        int         idx;
        String      query_string;
        String      hop_count_string;
        String      termination;
        String      terminal_host;
        String      terminal_host_address;
        String      terminal_port;
        URI         uri_termination;
        String      host_address;
        String      port_address;
        InetAddress inet_address;
        ObjectServerException   object_server_exception;

        printExecuteMessage(getOutputMessage(78,DEBUG_SETFORWARDSERVICEENTRY));
        if (service_entry == null) {
            printVariableMessage(225, 225, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY, DEBUG_NULL);

            object_server_exception = getObjectServerException(
                ObjectServerError.PARAMETER_ERROR_SERVER,
                ObjectServerError.PARAMETER_ERROR_SERVER);
            throw object_server_exception;
        }

        for (idx = 0; idx < forward_service_count; idx++) {
            if (service_entry.compareTo(forward_service_table[idx]) == 0) {
                printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                    getFormattedOutputMessage(226,
                        ObjectServerDebug.DEBUG_MATCH_ENTRY,
                        getOutputMessage(226,
                            ObjectServerDebug.DEBUG_MATCH_ENTRY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(forward_service_count),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(idx),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(forward_service_table.length),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        service_entry.toString()));
                break;
            }
        } 
        if (idx < forward_service_count) {
            printKeyMessage(227, 227, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_RETURN_KEY);
            return;
        }
        query_string = service_entry.getQuery();
        if (query_string != null) {
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(228,
                    ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY,
                    getOutputMessage(228,
                        ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ObjectServerDebug.DEBUG_QUERY_STRING,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    query_string));
            termination = null;
            try {
                termination = 
                    getQueryParameter(query_string, FORWARD_TERMINAL_KEY);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_SETFORWARDSERVICEENTRY, 0, exception_object);
                throw exception_object;
            }
            if (termination != null) {
                printDebugMessage(OSERV_DEBUG_TRACE_VALUE, getFormattedOutputMessage(230,
                    ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY,
                    getOutputMessage(230, ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, FORWARD_TERMINAL_KEY,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, termination));
                try {
                    uri_termination = new URI(termination);
                    terminal_host = uri_termination.getHost();
                    inet_address = InetAddress.getByName(terminal_host);
                    terminal_host_address = inet_address.getHostAddress();
                    int port = uri_termination.getPort();
                    if (port == -1) {
                        printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                            getFormattedOutputMessage(231,
                                ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY,
                                getOutputMessage(231,
                                        ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                ObjectServerDebug.DEBUG_FORWARD_ILLEGAL_URI,
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                termination));
                        return;
                    }
                    terminal_port = String.valueOf(port);
                    printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                        getFormattedOutputMessage(232,
                            ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY,
                            getOutputMessage(232,
                                ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            uri_termination.toString(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            terminal_host,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            terminal_host_address,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            terminal_port ));
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_SETFORWARDSERVICEENTRY, 0, exception_object);
                    throw exception_object;
                }
                try {
                    port_address = getPropertyValue(OSERV_REQUEST_PORT_KEY);
                    inet_address = InetAddress.getLocalHost();
                    host_address = inet_address.getHostAddress();
                   
                    printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                        getFormattedOutputMessage(234,
                            ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY,
                            getOutputMessage(234,
                                ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, host_address,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, port_address));
                    if (port_address != null &&
                        host_address.compareTo(terminal_host_address) == 0 &&
                        port_address.compareTo(terminal_port) == 0) {

                        printVariableMessage(235, 235, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY,
                            ObjectServerDebug.DEBUG_FORWARD_SERVICE_IS_OWN_SERVICE);
                        return;
                    }
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_SETFORWARDSERVICEENTRY, 0, exception_object);
                    throw exception_object;
                }
            
                hop_count_string = null;
                try {
                    hop_count_string 
                    = getQueryParameter(query_string, FORWARD_HOP_COUNT_KEY);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_SETFORWARDSERVICEENTRY, 0, exception_object);
                    throw exception_object;
                }
                if (hop_count_string == null) {
                    printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                        getFormattedOutputMessage(238,
                            ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY,
                            getOutputMessage(238,
                                ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_FORWARD_TUNNEL_COUNT,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(tunnel_value)));
                } else {
                    printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                            getFormattedOutputMessage(239,
                                ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY,
                            getOutputMessage(239,
                                ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_FORWARD_HOP_COUNT,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            hop_count_string,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_FORWARD_TUNNEL_COUNT,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(tunnel_value)));
                }
                if (hop_count_string != null &&
                    tunnel_value < Integer.valueOf(
                            hop_count_string).intValue()) {
                    printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                        getFormattedOutputMessage(240,
                            ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY,
                            getOutputMessage(240,
                                ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_FORWARD_HOP_COUNT,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            hop_count_string));
                    return;
                }
                int path_count;
                path_count = getForwardServerHostCount(uri_termination);
                if (path_count > tunnel_value) {
                    printTraceMessage( TRACE_SETFORWARDSERVICEENTRY, 0,
                        ObjectServerDebug.DEBUG_FORWARD_PATH_COUNT,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                        String.valueOf(path_count) );
                    return;
                }
            } 
        }
        printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
            getFormattedOutputMessage(242,
                ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY,
                getOutputMessage(242,
                    ObjectServerDebug.DEBUG_FORWARD_SET_SERVICE_ENTRY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                ObjectServerDebug.DEBUG_COUNT_KEY,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(forward_service_count) ));
        if (forward_service_count >= forward_service_table.length) {
            URI[] bk;
            bk = new URI[forward_service_count+FORWARD_UNIT_SIZE];
            for (int i = 0; i < forward_service_table.length; i++) {
                forward_service_table[i] = bk[i];
            }
            forward_service_table = bk;
        }
        forward_service_table[forward_service_count] = service_entry;
        forward_service_count = forward_service_count + 1;
    }

    /**
     * isCommandForwardService
     */
    private boolean isCommandForwardService(URI service_entry)
    {
        String query_string;

        query_string = service_entry.getQuery();
        if (query_string == null) {
            return false;
        }
        return true;
    }

    private String makeQueryParameter(String key, String value) {
        String query_string;

        query_string = new String(key + QUERY_PARAMETER_SEPARATOR + value);
        return query_string;
    }

    private boolean checkCommandForwardService(URI service_entry) {
        String query_string;
        String command;

        printExecuteMessage(
                getOutputMessage(616, DEBUG_CHECKCOMMANDFORWARDSERVICE) );
        if (service_entry == null) {
            printVariableMessage( 244, 244, OSERV_DEBUG_TRACE_VALUE, 
                ObjectServerDebug.DEBUG_FORWARD_COMMAND_REQUEST, DEBUG_NULL);
        }
        query_string = service_entry.getQuery();
        if (query_string == null) {
            return false;
        }

        try {
            command = getQueryParameter(query_string, FORWARD_COMMAND_KEY);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CHECKCOMMANDFORWARDSERVICE, 0, exception_object);
            return false;
        }
        if (command == null) {
            return false;
        }
        printVariableMessage( 245, 245, OSERV_DEBUG_TRACE_VALUE, 
                ObjectServerDebug.DEBUG_COMMAND_KEY, command);
        if (command.toLowerCase().compareTo(FORWARD_EXECUTE_COMMAND) == 0) {
            return true;
        }
        return false;
    }

    /**
     * doCommandForwardService
     */
    private Object doCommandForwardService(URI service_entry, 
	RetryObjectInputStream    retry_object_input_stream,
        ObjectOutputStream   object_output_stream) throws Exception
    {
        int    idx;
        URI    forward_entry;
        URI    target_service_entry;
        Object rt;
        byte[] file_byte;
        byte[] buffer;
        String query_string;
        String target_service_entry_string;
        String file;
        String command;
        String execute;
        String filename;
        String entry;
        URI    target_entry;
        String service;
        Exception error;
        Long   file_length;
        File   file_descriptor;
        FileInputStream file_input_stream;
        ObjectServerException object_server_exception;

        printExecuteMessage( getOutputMessage(618, DEBUG_DOCOMMANDFORWARDSERVICE) );
        if (service_entry == null) {
            printVariableMessage(247, 247, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_SERVICE_NAME, DEBUG_NULL);

            ObjectServerServiceUnavailableException ose 
                = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(733,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(733,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
            throw ose;
        } else {
            printVariableMessage(248, 248, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_SERVICE_NAME, service_entry.toString());
                
        }
        query_string = service_entry.getQuery();
        if (query_string == null) {
            ObjectServerServiceUnavailableException ose 
                = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(734,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(734,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
            throw ose;
        }

        try {
            command = getQueryParameter(query_string, FORWARD_COMMAND_KEY);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
            throw exception_object;
        }
        if (command == null) {
            ObjectServerServiceUnavailableException ose 
                = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(735,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(735,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
            return ose;
        }
        printVariableMessage(250, 250, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_COMMAND_KEY, command);

        if (command.toLowerCase().compareTo(FORWARD_DELETE_COMMAND) == 0) {
            target_service_entry_string = service_entry.toString();
            idx = target_service_entry_string.indexOf(OPTION_SEPARATOR);
            if (idx >= 0) {
                target_service_entry_string
                    = target_service_entry_string.substring(0, idx);
                try {
                    target_service_entry = new URI(target_service_entry_string);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_URI_EXCEPTION,
                        ObjectServerError.JAVA_URI_EXCEPTION);
                    throw object_server_exception;
                }
                for (int i = 0; i < forward_service_count; i++) {
                    forward_entry = forward_service_table[i];
                    if (forward_entry.compareTo(target_service_entry) == 0) {
                        for (int j = i; j < forward_service_count-1; j++) {
                            forward_service_table[j]
                                = forward_service_table[j+1];
                        }
                        forward_service_count =
                            forward_service_count - 1;
                        return null;
                    }
                }
            }
            ObjectServerServiceUnavailableException ose 
                = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(736,
                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(736,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
            throw ose;
        }
        if (command.toLowerCase().compareTo(FORWARD_LIST_COMMAND) == 0) {
            int count;
            int entry_number;
            int oserv_index;
            int bad_oserv_number;
            URI[] send_forward_service;
            bad_oserv_number = 0;
            ObjectTable  clone_table;
            try {
                clone_table = operateObjectTable(ObjectTable.READ_OBJECT_TABLE, null, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                throw exception_object;
            }
            count = clone_table.getCount();
            oserv_index = 0;
            if (tunnel_value <= 0) {
                send_forward_service = new URI[count];
                for (int i = 0; i < count; i++) {
                    try {
                        entry = clone_table.get(i);
                    } catch (Exception exception_object) {
                        URI[] bk = new URI[send_forward_service.length-1];
                        for (int j = 0; j < oserv_index;j++) {
                            bk[j] = send_forward_service[j];
                        }
                        send_forward_service = bk;
                        continue;
                    }
                    send_forward_service[oserv_index] 
                        = getURIEntry(entry, (String)null);
                    oserv_index = oserv_index + 1;
                }
            } else {
                entry_number = count+forward_service_count;
                send_forward_service = new URI[entry_number];
                for (int i = 0; i < count; i++) {
                    try {
                        entry = clone_table.get(i);
                    } catch (Exception exception_object) {
                        URI[] bk = new URI[send_forward_service.length-1];
                        for (int j = 0; j < oserv_index;j++) {
                            bk[j] = send_forward_service[j];
                        }
                        send_forward_service = bk;
                        continue;
                    }
                    send_forward_service[oserv_index] 
                        = getURIEntry(entry, (String)null);
                    oserv_index = oserv_index + 1;
                }
                for (int i = 0; i < forward_service_count; i++) {
                    try {
                        send_forward_service[oserv_index] 
                            = getTunnelEntry(forward_service_table[i]);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                        continue;
                    }
                    oserv_index = oserv_index + 1;
                }
            }
            Integer  forward_entry_count;
            forward_entry_count = Integer.valueOf(send_forward_service.length) ;

            printVariableMessage(258, 258, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_SERVICE_ENTRY_NUMBER,
                forward_entry_count.toString());

            try {
                object_output_stream.writeObject( forward_entry_count );
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
            for (int i = 0; i < oserv_index; i++) {
                try {
                    object_output_stream.writeObject(
                        send_forward_service[i] );
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                    throw object_server_exception;
                }
            }
            return null;
        }
        if (command.toLowerCase().compareTo(FORWARD_GET_COMMAND) == 0) {
            long   size;
            int    offset;
            int    rcnt;
            try {
                target_service_entry_string 
                = getQueryParameter(query_string, FORWARD_SERVICE_KEY);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                throw exception_object;
            }
            if (target_service_entry_string == null) {
                ObjectServerServiceUnavailableException ose 
                    = new ObjectServerServiceUnavailableException(
                    getFormattedOutputMessage(737,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                        getOutputMessage(737,
                            ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
                throw ose;
            }
            boolean check = false;
            try {
                operateObjectTable(ObjectTable.SEARCH_OBJECT_TABLE, 
                    target_service_entry_string, null) ;
                check = true;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                throw exception_object;
            }
            if (check == true) {
                filename = convertEntryToFilepath(
                                    target_service_entry_string) ;
                printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                    getFormattedOutputMessage(262,
                        DEBUG_DOCOMMANDFORWARDSERVICE,
                        getOutputMessage(262, DEBUG_DOCOMMANDFORWARDSERVICE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        FORWARD_GET_COMMAND,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_FILE_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY, filename));

                file_descriptor = new File(filename);
                if (file_descriptor.exists() == false 
                        || file_descriptor.length() == 0) {
                    ObjectServerServiceUnavailableException ose 
                        = new ObjectServerServiceUnavailableException(
                        getFormattedOutputMessage(738,
                            ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                            getOutputMessage(738,
                                ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
                    throw ose;
                }
                size = file_descriptor.length();
                file_byte = new byte[(int)size];
                try {
                    file_input_stream = new FileInputStream(file_descriptor);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                    object_server_exception = getObjectServerException(
                         ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION,
                         ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION);
                    throw object_server_exception;
                }
                offset = 0;
                while (true) {
                    try {
                        rcnt = file_input_stream.read(file_byte, offset, (int)size);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                        if ((exception_object instanceof java.io.EOFException) == true) {
                            break;
                        }
                        return exception_object;
                    }
                    if (rcnt == -1) {
                        break;
                    }
                    offset = offset + rcnt;
                    size = size - (long)rcnt;
                    if (size > 0) {
                        continue;
                    }
                    return file_byte;
                }
                ObjectServerServiceUnavailableException ose 
                    = new ObjectServerServiceUnavailableException(
                    getFormattedOutputMessage(373,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                        getOutputMessage(373,
                            ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
                throw ose;
            }
            ObjectServerServiceUnavailableException ose 
                = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(205,
                    ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(205,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
            throw ose;
        }
        printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
            getFormattedOutputMessage(265,
                ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                getOutputMessage(265, ObjectServerDebug.DEBUG_FORWARD_SERVICES),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, command,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, FORWARD_SERVICE_COMMAND));
        if (command.toLowerCase().compareTo(FORWARD_SERVICE_COMMAND) == 0) {
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                    getFormattedOutputMessage(266,
                        ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                        getOutputMessage(266,
                            ObjectServerDebug.DEBUG_FORWARD_SERVICES),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_FORWARD_SERVICE_IN_KEY,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        service_entry.toString(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        service_entry.getPath()));
            if (getSystemForbidden() == true) {
                ObjectServerServiceUnavailableException ose 
                    = new ObjectServerServiceUnavailableException(
                    getFormattedOutputMessage(155,
                        ObjectServerError.FORBIDDEN_FORWARD_SERVICE_SERVER,
                        getOutputMessage(155,
                            ObjectServerError.FORBIDDEN_FORWARD_SERVICE_SERVER)));
                throw ose;
            }

            try {
                target_service_entry_string = getForwardService(service_entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                ObjectServerServiceUnavailableException ose 
                    = new ObjectServerServiceUnavailableException(
                    getFormattedOutputMessage(204,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                        getOutputMessage(204,
                            ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
                throw ose;
            }

            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(267, ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                    getOutputMessage(267, ObjectServerDebug.DEBUG_FORWARD_SERVICES),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    target_service_entry_string));

            try {
                file = getQueryParameter(query_string, FORWARD_FILE_KEY);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                file = null;
            }

            if (target_service_entry_string == null) {
                ObjectServerServiceUnavailableException ose 
                    = new ObjectServerServiceUnavailableException(
                    getFormattedOutputMessage(442,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                        getOutputMessage(442,
                            ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
                throw ose;
            }
            boolean check = false;
            try {
                operateObjectTable(ObjectTable.SEARCH_OBJECT_TABLE, 
                        target_service_entry_string, null);
                    check = true;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                throw exception_object;
            }
            if (check == true) {
                filename = convertEntryToFilepath(
                                    target_service_entry_string) ;
                printVariableMessage(269, 269, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_JAR_NAME, filename);

                JarArchiveObject jar_archive_object;
                try {
                    jar_archive_object 
                    = JarArchiveObjectFactory.createJarArchiveObject(filename,
                        getMessageResource(),
                        getDebug(), getLogStream());
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                    throw exception_object;
                }
                if (file == null) {
                    printVariableMessage(271, 271, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                        ObjectServerDebug.DEBUG_FORWARD_READ_JAR_ARCHIVE_FILE_KEY);
                    try {
                        jar_archive_object.readJarArchiveFile(
                            object_output_stream);
		    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                        throw exception_object;
                    }
                } else {
                    printVariableMessage(273, 273, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                        ObjectServerDebug.DEBUG_FORWARD_GET_FILE_CONTENTS_KEY);
                    try {
                        jar_archive_object.getFileContents(file, 
                        object_output_stream);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                        throw exception_object;
                    }
		}
                return null;
            }
            error = null;
            URI pre_forward_command;
            for (int i = 0; i < forward_service_count; i++) {
                target_entry = forward_service_table[i];
                try {
                    service = getForwardService(target_entry);
                    printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                        getFormattedOutputMessage(275,
                            ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                            getOutputMessage(275,
                                ObjectServerDebug.DEBUG_FORWARD_SERVICES),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, service, 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            target_service_entry_string));
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                    throw exception_object;
                }
                if (service.compareTo(target_service_entry_string) == 0) {
                    try {
                        pre_forward_command = new URI (OSERV_SCHEME +
                            LIBRARY_SEPARATOR + OBJECT_SEPARATOR +
                            OBJECT_SEPARATOR + target_entry.getHost() + 
                            LIBRARY_SEPARATOR +
                            String.valueOf(target_entry.getPort()) +
                            service_entry.getPath() + OPTION_SEPARATOR +
                            service_entry.getQuery());
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                        throw exception_object;
                    }
                    printLogCallMessage( getOutputMessage(620, DEBUG_TRANSFERSERVICE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getFormattedOutputMessage(621, ObjectServerDebug.DEBUG_COMMAND_KEY,
                            getOutputMessage(621, ObjectServerDebug.DEBUG_COMMAND_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, command));

                    try {
                        rt = transferService(pre_forward_command, command, 
                            null, object_output_stream);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_DOCOMMANDFORWARDSERVICE, 0, exception_object);
                        if (error == null) {
                            error = exception_object;
                        }
                        continue;
                    }
                    return rt;            
                }
            }
            if (error != null) {
                throw error;
            }

            ObjectServerServiceUnavailableException ose 
                    = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(203,
                    ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(203,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
            throw ose;
        }
        printEndMessage(OSERV_DEBUG_TRACE_VALUE,DEBUG_DOCOMMANDFORWARDSERVICE);

        ObjectServerServiceUnavailableException ose 
            = new ObjectServerServiceUnavailableException(
                getFormattedOutputMessage(202,
                    ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER,
                    getOutputMessage(202,
                        ObjectServerError.ILLEGAL_FORWARD_SERVICE_SERVER)));
        throw ose;
    }

    private void writeForwardStringTable(
        ObjectOutputStream  object_output_stream) throws Exception {
        int     idx;
        URI     service_entry;
        String  service_entry_string;
        Integer integer_forward_size;
        ObjectServerException    object_server_exception;

        printVariableMessage(281, 281, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_WRITE_FORWARD_STRING_TABLE,
            String.valueOf(forward_service_table.length));

        integer_forward_size = new Integer(forward_service_table.length);

        printVariableMessage(283, 283, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_WRITE_FORWARD_STRING_TABLE,
            integer_forward_size.toString());

        try {
            object_output_stream.writeObject(integer_forward_size);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEFORWARDSTRINGTABLE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }    
        Integer integer_forward_number = new Integer(forward_service_count);
        printVariableMessage(385, 385, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_WRITE_FORWARD_STRING_TABLE,
            integer_forward_number.toString());
        try {
            object_output_stream.writeObject(integer_forward_number);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEFORWARDSTRINGTABLE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }    
        for (int i = 0; i < forward_service_count; i++) {
            service_entry = forward_service_table[i];
            service_entry_string = service_entry.toString();
            idx = service_entry_string.indexOf(OPTION_SEPARATOR);
            if (idx >= 0) {
                service_entry_string = service_entry_string.substring(0, idx);
                printVariableMessage(187, 187, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_WRITE_FORWARD_STRING_TABLE,
                    service_entry_string );
            }

            try {
                object_output_stream.writeObject(service_entry_string);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITEFORWARDSTRINGTABLE, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
        }
    }
    
    private void sendForwardTable(ObjectOutputStream  object_output_stream,
        URI[] my_services) throws Exception {
        printExecuteMessage( getOutputMessage(624, DEBUG_SENDFORWARDTABLE),
            getFormattedOutputMessage(626, ObjectServerDebug.DEBUG_TABLE_SIZE,
            getOutputMessage(626, ObjectServerDebug.DEBUG_TABLE_SIZE),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            String.valueOf(my_services.length)));

        writeForwardURITable(object_output_stream, my_services,
            my_services.length);
    }
    /**
     * writeForwardTable
     */
    private void writeForwardTable(ObjectOutputStream  object_output_stream) 
        throws Exception {
        writeForwardURITable(object_output_stream, forward_service_table,
            forward_service_count);
    }

    private void writeForwardURITable(ObjectOutputStream  object_output_stream,
        URI[] services, int count) 
        throws Exception {
        URI service_entry;

        printExecuteMessage( getOutputMessage(627, DEBUG_WRITEFORWARDURITABLE));
        printVariableMessage(111, 111, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_COUNT_KEY, String.valueOf(count));
        for (int i = 0; i < count; i++) {
            service_entry = services[i];
            printVariableMessage(291, 291, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_SERVICE_NAME, service_entry.toString());
            try {
                object_output_stream.writeObject(service_entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITEFORWARDURITABLE, 0, exception_object);
                continue;
            }
        }
    }

    /**
     * This method make lock file for exclusive procedures.
     */
    private boolean lock() throws Exception {
        try {
            if (flock.createNewFile() == true) {
                return true;
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_LOCK, 0, exception_object);
            throw exception_object;
        }
        return false;
    }

    /**
     * This method removes lock file
     */
    private void unlock() {
        try {
            flock.delete();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_UNLOCK, 0, exception_object);
        }
    }


    /**
     * This method writes Jar file for registered service
     *
     * @param     DataOutputStream 
     *            byte[]
     * @return    void
     * @throws    Exception
     */
    private void writeJarFile (DataOutputStream data_output_stream, byte[] buffer) throws Exception {
        File   fdes;
        File   ddes;
        String dirpath;
        int    idx;
        ObjectServerException   object_server_exception;

        if (data_output_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(745, 745,
	            ObjectServerError.OUTPUT_STREAM_PARAMETER_NULL_SERVER));
            printVariableMessage(645, 645, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, ose.toString());
            throw ose;
        }
        if (buffer == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(746, 746,
	          ObjectServerError.WRITE_JAR_FILE_BUFFER_SERVER));
            printVariableMessage(274, 274, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, ose.toString());
            throw ose;
        }
        try {
            data_output_stream.write(buffer, 0, (int)buffer.length);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEJARFILE, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_DATA_OUTPUT_STREAM_WRITE_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_OUTPUT_STREAM_WRITE_IO_EXCEPTION);
            throw object_server_exception;
        }    
    }

    private void copyJarFile (RetryObjectInputStream retry_object_input_stream, DataOutputStream data_output_stream, int length) throws Exception {
        byte[]   buffer;
        int      bkcount;
        ObjectServerException   object_server_exception;

        if (retry_object_input_stream == null || data_output_stream == null) {
            object_server_exception   
                = new ObjectServerException( getUserOutputMessage(48, 48,
	        ObjectServerError.COPY_JAR_FILE_PARAMETER_NULL_SERVER));
            printVariableMessage(646, 646, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, object_server_exception.toString());
            throw object_server_exception;
        }
        if (length < 0) {
            object_server_exception   
                = new ObjectServerException( getUserOutputMessage(74, 74,
	        ObjectServerError.COPY_JAR_FILE_LENGTH_ZERO_SERVER));
            printVariableMessage(647, 647, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, object_server_exception.toString());
            throw object_server_exception;
        }
        bkcount = (length + maximum - 1)/maximum;

        printVariableMessage(295, 295, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM,
            retry_object_input_stream.toString());
        printVariableMessage(637, 637, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_JAVA_INPUTSTREAM,
            data_output_stream.toString());
        printVariableMessage(638, 638, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_MAXIMUM, 
            String.valueOf(maximum));

        for (int j = 0; j < bkcount; j++ ) {
            if (length > (j + 1) * maximum) {
                buffer = new byte[maximum];
            } else {
                buffer = new byte[length - (j * maximum)];
            }
            printVariableMessage(209, 209, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_BUFFER_SIZE,
                String.valueOf(buffer.length));
            try {
                retry_object_input_stream.retryRead(buffer, 0, (int)buffer.length);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_COPYJARFILE, 0, exception_object);
                throw exception_object;
            }
            for (int k = 0; k < buffer.length; k++) {
                if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                    if ((k % 8) == 0)  {
                        System.out.print(DEBUG_CODE);
                    }
                    System.out.print(String.valueOf(k));
                    if ((k % 8) == 7)  {
                        System.out.println();
                    }
                }
            }
            try {
                data_output_stream.write(buffer, 0, (int)buffer.length);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_COPYJARFILE, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_DATA_OUTPUT_STREAM_WRITE_IO_EXCEPTION,
                    ObjectServerError.JAVA_DATA_OUTPUT_STREAM_WRITE_IO_EXCEPTION);
                throw object_server_exception;
            }    
        }
    }
    /**
     * This method reads Jar file for accepting request
     *
     * @param     InputStream  - Socket connect 
     *            byte[]       - Buffer
     * @return    void
     * @throws    Exception
     */
    private void readJarFile (RetryObjectInputStream retry_object_input_stream, byte[] buffer) throws Exception {
        Object s1;

        if (retry_object_input_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(199, 199,
	            ObjectServerError.INPUT_STREAM_PARAMETER_NULL_SERVER));
            printVariableMessage(299, 299, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, ose.getMessage());
            throw ose;
        }
        if (buffer == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(580, 580,
	                ObjectServerError.BUFFER_PARAMETER_NULL_SERVER));
            printVariableMessage(200, 200, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, ose.getMessage());
            throw ose;
        }
        try {
            retry_object_input_stream.retryRead(buffer, 0, (int)buffer.length);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_READJARFILE, 0, exception_object);
            throw exception_object;
        }    
        return;
    }

    private void deleteDirectoryFile(File directory_file) {
        File[] list_files;
        if (directory_file.isDirectory() != true) {
            return ;
        }
        list_files = directory_file.listFiles();
        for (int i = 0; i < list_files.length; i++) {
            if (list_files[i].exists() == false) {
                continue;
            }
            if (list_files[i].isDirectory() == true) {
                deleteDirectoryFile(list_files[i]);
                continue;
            }
            list_files[i].delete();
        }
        directory_file.delete();
    }

    private void switchLogFile() {
        File fdes;
        String logfile;
        String vstring;
        Boolean debugLog = false;
        String logPath;

        try {
            loadPropertyValue(OSERV_DEBUG_LEVEL_KEY);
            updateDebugLevel();
            loadPropertyValue(OSERV_LOG_FILE_KEY);
            logfile = getPropertyValue(OSERV_LOG_FILE_KEY);
            setLogStream (setupLogFile(logfile));
        } catch (Exception exception_object) {
            if (getLogStream() == null) {
                setLogStream( new LogPrintStream(System.out));
            }
        }
    }

    private boolean reserveDispatchThread(Socket ssock) {
        Object                  object_value;
        String                  request_type;
        String                  description_key;
        InputStream             input_stream;
        RetryObjectInputStream  retry_object_input_stream;

        try {
            description_key = ObjectServerDebug.DEBUG_KEEP_ALIVE_TRUE;
            if (ssock.getKeepAlive() != true) {
                description_key = ObjectServerDebug.DEBUG_KEEP_ALIVE_FALSE;
            }
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(334,
                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                    getOutputMessage(334,
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ssock.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    description_key));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }
        try {
            ssock.setKeepAlive(true);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }
        try {
            description_key = ObjectServerDebug.DEBUG_KEEP_ALIVE_TRUE;
            if (ssock.getKeepAlive() != true) {
                description_key = ObjectServerDebug.DEBUG_KEEP_ALIVE_FALSE;
            }
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(333,
                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                    getOutputMessage(333,
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ssock.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    description_key));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }
        try {
            description_key = ObjectServerDebug.DEBUG_SO_LINGER_FALSE;
            if (ssock.getSoLinger() != -1) {
                description_key = String.valueOf(ssock.getSoLinger());
            }
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(156,
                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                    getOutputMessage(156,
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ssock.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    description_key));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }
       
        try {
            ssock.setSoLinger(Boolean.FALSE, -1);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
            return false;
        }
        try {
            description_key = ObjectServerDebug.DEBUG_SO_LINGER_FALSE;
            if (ssock.getSoLinger() != -1) {
                description_key = String.valueOf(ssock.getSoLinger());
            }
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(368,
                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                    getOutputMessage(368,
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ssock.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    description_key));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }
        try {
            description_key = ObjectServerDebug.DEBUG_TCP_NO_DELAY_TRUE;
            if (ssock.getTcpNoDelay() != true) {
                description_key = ObjectServerDebug.DEBUG_TCP_NO_DELAY_FALSE;
            }
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(287,
                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                    getOutputMessage(287,
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ssock.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    description_key));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }

        try {
            ssock.setTcpNoDelay(true);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
            return false;
        }

        try {
            description_key = ObjectServerDebug.DEBUG_TCP_NO_DELAY_TRUE;
            if (ssock.getTcpNoDelay() != true) {
                description_key = ObjectServerDebug.DEBUG_TCP_NO_DELAY_FALSE;
            }
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(188,
                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                    getOutputMessage(188,
                        ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ssock.toString(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    description_key));
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
        }
        try {
            input_stream = ssock.getInputStream();
            retry_object_input_stream = RetryObjectInputStream.valueOf(
                input_stream, getMessageResource(),
                getDebug(), getLogStream());
            object_value = retry_object_input_stream.retryReadObject();
            if (object_value != null && (object_value instanceof String) != false) {
                printVariableMessage(127, 127, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
            }
            request_type = (String)object_value;
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
            return false;
        }
        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            getLogStream().printLogln( getFormattedOutputMessage(471,
                ObjectServerDebug.DEBUG_REQUEST_KEY,
                getOutputMessage(471, ObjectServerDebug.DEBUG_REQUEST_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, request_type));
        }
        String thread_type;
        ThreadTable thread_table;

        if (request_type.compareTo(OSERV_COMMAND_TYPE_REQUEST_KEY) == 0) {
            try {
                operateThreadTable(RESERVE_THREAD, 
                    SERVICE_THREAD_TYPE,
                    ssock, 
                    retry_object_input_stream, 
                    service_table,
                    null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
                return false;
            }
        } else if (request_type.compareTo(OSERV_COMMAND_TYPE_OBJECT_KEY) == 0) {
            try {
                operateThreadTable(RESERVE_THREAD, 
                    REGISTER_THREAD_TYPE,
                    ssock, 
                    retry_object_input_stream, register_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
                return false;
            }
        } else if (request_type.compareTo(OSERV_COMMAND_TYPE_FORWARD_KEY) == 0) {
            try {
                operateThreadTable(RESERVE_THREAD, 
                    FORWARD_THREAD_TYPE,
                    ssock, 
                    retry_object_input_stream, forward_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
                return false;
            }
        } else if (request_type.compareTo(OSERV_COMMAND_TYPE_SYSTEM_KEY) == 0) {
            try {
                operateThreadTable(RESERVE_THREAD, 
                    SYSTEM_THREAD_TYPE,
                    ssock, retry_object_input_stream, system_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
                return false;
            }
        } else if (request_type.compareTo(OSERV_COMMAND_TYPE_TRANSIENT_KEY) == 0) {
            try {
                operateThreadTable(RESERVE_THREAD, 
                    TRANSIENT_THREAD_TYPE,
                    ssock, 
                    retry_object_input_stream, 
                    transient_table,
                    null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, exception_object);
                return false;
            }
        } else {
            printAssertionMessage(TRACE_RESERVEDISPATCHTHREAD, 0, request_type );
            return false;
        }
        return true;
    }

    private synchronized ObjectTable operateObjectTable(
        String  operation_type,
        String  entry,
        String   argument) throws Exception {

        ObjectTable  clone;
        ObjectServerException    object_server_exception;

        printExecuteMessage( DEBUG_OPERATEOBJECTTABLE );
        if (operation_type == null) {
            printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_NULL_OPERATION_TYPE);
            return null;
        }
        printTraceMessage(TRACE_OPERATEOBJECTTABLE, 0, operation_type);
        if (operation_type.compareTo(ObjectTable.READ_OBJECT_TABLE) == 0) {
            if (object_table == null) {
                printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_CLONE_OBJECTTABLE);
            }
            clone = object_table.clone();
            return clone;
        }
        if (operation_type.compareTo(ObjectTable.DELETE_OBJECT_TABLE) == 0) {
            if (entry == null) {
                printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_NULL_ENTRY);
                return null;
            }
            printTraceMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_DELETE_OBJECTTABLE);
            try {
                object_table.delete(entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATEOBJECTTABLE, 0, exception_object);
                return null;
            }
            clone = object_table.clone();
            return clone;
        }
        if (operation_type.compareTo(ObjectTable.ADD_OBJECT_TABLE) == 0) {
            if (entry == null || argument == null) {
                printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_NULL_ENTRY);
                return null;
            }
            printLogCallMessage(DEBUG_ADD_OBJECTTABLE,
                entry, ObjectServerDebug.DEBUG_BLANK_STRING_KEY, argument);

            try {
                object_table.add(entry, argument);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATEOBJECTTABLE, 0, exception_object);
                return null;
            }
            clone = object_table.clone();
            return clone;
        }
        if (operation_type.compareTo(ObjectTable.SEARCH_OBJECT_TABLE) == 0) {
            if (entry == null) {
                printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_NULL_ENTRY);
                return null;
            }
            printLogCallMessage( DEBUG_EQUAL_OBJECTTABLE );

            if (object_table.equal(entry) == true) {
                return object_table;
            } else {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.OBJECT_NOT_FOUND_EXCEPTION_SERVER,
                        getMessageResource().getMessage(
                            ObjectServerError.OBJECT_NOT_FOUND_EXCEPTION_SERVER)));
                throw object_server_exception;
            }
        }
        if (operation_type.compareTo(ObjectTable.SET_SERVICE_TYPE) == 0) {
            if (argument == null) {
                printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_NULL_PARAMETER);
                return null;
            }
            if (entry == null) {
                printAssertionMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_NULL_ENTRY);
                return null;
            }
            printTraceMessage(TRACE_OPERATEOBJECTTABLE, 0, DEBUG_SETTYPE_OBJECTTABLE);
            try {
                object_table.setType(entry, argument);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATEOBJECTTABLE, 0, exception_object);
                return null;
            }
            return null;
        }
        return null;
    }

    private synchronized ThreadTable operateThreadTable(
        String  operation_type,
        String  thread_type,
        Socket  socket_object,
        RetryObjectInputStream    retry_object_input_stream,
        ThreadTable    thread_table,
        String         thread_argument)
        throws Exception {
        boolean        flag;
        ThreadTable    thread_table_object;
        Thread         thread_object;
        ObjectServerException object_server_exception;

        if (operation_type == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
            throw object_server_exception;
        }
        printExecuteMessage(DEBUG_OPERATETHREADTABLE, operation_type);

        thread_object = null;
        if (operation_type.compareTo(RESERVE_THREAD) == 0) {
            if (socket_object == null || retry_object_input_stream == null) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                        getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
                throw object_server_exception;
            }
            printLogCallMessage(DEBUG_RESERVETHREAD, 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, thread_type);
            flag = reserveThread(thread_type, socket_object,
                retry_object_input_stream, thread_table);

            printTraceMessage(TRACE_OPERATETHREADTABLE, 0, thread_type);
            return thread_table;
        }
        if (operation_type.compareTo(EXCLUSIVE_SET_OF_COMMAND_TO_THREAD) == 0) {
            String  thread_command;
            thread_object = Thread.currentThread();
            thread_command = thread_argument;

            if (thread_command == null) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                        getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
                throw object_server_exception;
            }
            printLogCallMessage( DEBUG_CHECKTHREADCOMMAND );

            try {
                flag = checkThreadCommand(thread_table,
                    thread_command);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATETHREADTABLE, 0, exception_object);
                throw exception_object;
            }
            if (flag == true) {
                if (isSerializableThread(thread_table, thread_object) == true) {

                    ObjectServerBusyException busy_exception;
                    busy_exception = new ObjectServerBusyException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.BUSY_EXCEPTION_OPERATETHREADTABLE_SERVER,
                        getMessageResource().getMessage(
                            ObjectServerError.BUSY_EXCEPTION_OPERATETHREADTABLE_SERVER)));
                    throw busy_exception;
                }
            }
            printLogCallMessage( DEBUG_SETTHREADCOMMAND );

            try {
                setThreadCommand(thread_table, thread_object, thread_command);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATETHREADTABLE, 0, exception_object);
                throw exception_object;
            }
            printTraceMessage(TRACE_OPERATETHREADTABLE, 0, thread_type);
            return thread_table;
        }

        if (operation_type.compareTo(READ_THREAD_TABLE) == 0) {
            printLogCallMessage(DEBUG_GETTHREADTABLE);

            thread_table_object = getThreadTable(thread_type, thread_table);
            printTraceMessage(TRACE_OPERATETHREADTABLE, 0, thread_type);
            return thread_table_object;
        }
        if (operation_type.compareTo(RELEASE_THREAD) == 0) {
            thread_object = Thread.currentThread();
            flag = releaseThread(thread_type, thread_object, thread_table);
            thread_table_object = getThreadTable(thread_type, thread_table);
            printTraceMessage(TRACE_OPERATETHREADTABLE, 0, thread_type);
            return thread_table_object;
        }
        if (operation_type.compareTo(TERMINATE_THREAD) == 0) {
            String thread_id;
            thread_id = thread_argument;

            if (thread_id == null) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                        getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
                throw object_server_exception;
            }
            try {
                thread_object = getThread(thread_type, thread_id, thread_table);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATETHREADTABLE, 0, exception_object);
                throw exception_object; 
            }
        }
        if (operation_type.compareTo(TERMINATE_THREAD) == 0) {
        
            thread_table_object = thread_table.getEntry(thread_object);

            printLogCallMessage( DEBUG_TERMINATETHREAD,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, thread_type,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, thread_object.toString());
            try {
                terminateThread(thread_type, thread_object, thread_table);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_OPERATETHREADTABLE, 0, exception_object);
                throw exception_object;
            }
            flag = releaseThread(thread_type, thread_object, thread_table);
            printTraceMessage(TRACE_OPERATETHREADTABLE, 0, thread_type);
            return thread_table_object;
        }
    
        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
        throw object_server_exception;
    }

    private boolean isMonitor(String thread_type) {
        if (thread_type.compareTo(MONITOR_THREAD_TYPE) == 0) {
            return true;
        }
        return false;
    }

    private boolean isTransient(String thread_type) {
        if (thread_type.compareTo(TRANSIENT_THREAD_TYPE) == 0) {
            return true;
        }
        return false;
    }

    private boolean isSystem(String thread_type) {
        if (thread_type.compareTo(SYSTEM_THREAD_TYPE) == 0) {
            return true;
        }
        return false;
    }

    private boolean isMonitorOn(String thread_type) {
        if (thread_type == null) {
            return false;
        }
        if (thread_type.compareTo(SERVICE_THREAD_TYPE) == 0 ||
            thread_type.compareTo(TRANSIENT_THREAD_TYPE) == 0 ) {
            return true;
        }
        return false;
    }

    private boolean isHandlerOn(String thread_type) {
        if (thread_type == null) {
            return false;
        }
        if (thread_type.compareTo(SERVICE_THREAD_TYPE) == 0 ||
            thread_type.compareTo(SYSTEM_THREAD_TYPE) == 0) {
            return true;
        }
        return false;
    }

    private boolean reserveThread(
        String thread_type,
        Socket socket_object,
        RetryObjectInputStream retry_object_input_stream,
        ThreadTable    thread_table
    ) {
        ObjectFunction  target_thread;
        Thread          monitor_thread;

        monitor_thread = null;
        if (thread_type.compareTo(REGISTER_THREAD_TYPE) == 0) {
            target_thread = new RegisterServer(this, thread_type);
        } else if (thread_type.compareTo(TRANSIENT_THREAD_TYPE) == 0) {

            target_thread = new TransientServer(this, thread_type);

        } else if (thread_type.compareTo(SERVICE_THREAD_TYPE) == 0) {
            target_thread = new RequestServer(this, thread_type);
        } else if (thread_type.compareTo(FORWARD_THREAD_TYPE) == 0) {
            target_thread = new ForwardServer(this, thread_type);
        } else if (thread_type.compareTo(SYSTEM_THREAD_TYPE) == 0) {
            target_thread = new SystemServer(this, thread_type);
        } else {
            target_thread = new ObjectFunction(this, thread_type);
        }
        target_thread.setMyType(thread_type);
        if (isMonitorOn(thread_type) == true) {
            if (isTransient(thread_type) == true) {
                monitor_thread = new TransientMonitor(this, MONITOR_THREAD_TYPE);
            } else {
                monitor_thread = new MonitorServer(this, MONITOR_THREAD_TYPE);
            }
            OServInterruptHandler local_monitor_handler;
            OServInterruptHandler local_thread_handler;
            local_monitor_handler = new OServInterruptHandler();
            local_monitor_handler.setThreadType(MONITOR_THREAD_TYPE);
            local_monitor_handler.setDebugLogStreamWithMessage(getDebug(), getLogStream(), getMessageResource());
            local_thread_handler = new OServInterruptHandler();
            local_thread_handler.setThreadType(thread_type);
            local_thread_handler.setDebugLogStreamWithMessage(getDebug(), getLogStream(), getMessageResource());
            monitor_thread.setUncaughtExceptionHandler(local_monitor_handler);
            target_thread.setUncaughtExceptionHandler(local_thread_handler);
        } else if (isHandlerOn(thread_type) == true) {
            OServInterruptHandler local_thread_handler;
            local_thread_handler = new OServInterruptHandler();
            local_thread_handler.setThreadType(thread_type);
            local_thread_handler.setDebugLogStreamWithMessage(getDebug(), getLogStream(), getMessageResource());
            target_thread.setUncaughtExceptionHandler(local_thread_handler);
        }
        try {
            thread_table.add(target_thread, socket_object, 
                    retry_object_input_stream, monitor_thread);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_RESERVETHREAD, 0, exception_object);
            return false;
        }
        target_thread.start();
        if (isMonitorOn(thread_type) == true) {
            monitor_thread.start();
        }
        return true;
    }
    
    private boolean releaseThread(
        String       thread_type, 
        Thread       target,
        ThreadTable  thread_table) {

        return thread_table.remove(target);
    }

    private String  getThreadType( String target_id ) {
        Thread    target;

        try {
            target = getThread(SERVICE_THREAD_TYPE, target_id, service_table);
            if (target != null) {
                return SERVICE_THREAD_TYPE;
            }
            target = getThread(SYSTEM_THREAD_TYPE, target_id, system_table);
            if (target != null) {
                return SYSTEM_THREAD_TYPE;
            }
            target = getThread(FORWARD_THREAD_TYPE, target_id, forward_table);
            if (target != null) {
                return FORWARD_THREAD_TYPE;
            }
            target=getThread(TRANSIENT_THREAD_TYPE,target_id,transient_table);
            if (target != null) {
                return TRANSIENT_THREAD_TYPE;
            }
            target=getThread(REGISTER_THREAD_TYPE,target_id,register_table);
            if (target != null) {
                return REGISTER_THREAD_TYPE;
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETTHREADTYPE, 0, exception_object);
        }
        return UNKNOWN_THREAD_TYPE;
    }

    private Thread getThread(
        String       thread_type, 
        String       thread_id,
        ThreadTable  parameter_thread_table) throws Exception {
        int          count;
        Thread       target;
        String       target_id;
        ThreadTable  thread_table;
        ThreadTable  clone_table;

        if (parameter_thread_table == null ) {
            if (thread_type.compareTo(SERVICE_THREAD_TYPE) == 0) {
                thread_table = service_table;
            } else if (thread_type.compareTo(SYSTEM_THREAD_TYPE) == 0) {
                thread_table = system_table;
            } else if (thread_type.compareTo(TRANSIENT_THREAD_TYPE) == 0) {
                thread_table = transient_table;
            } else if (thread_type.compareTo(FORWARD_THREAD_TYPE) == 0) {
                thread_table = forward_table;
            } else if (thread_type.compareTo(REGISTER_THREAD_TYPE) == 0) {
                thread_table = register_table;
            } else {
                printAssertionMessage(TRACE_GETTHREAD, 0, getFormattedOutputMessage(136,
                    ObjectServerDebug.DEBUG_NULL_THREADTABLE,
                    getOutputMessage(136, ObjectServerDebug.DEBUG_NULL_THREADTABLE)));
                ObjectServerException object_server_exception;
                object_server_exception = new ObjectServerException(
                    getFormattedOutputMessage(335,
                        ObjectServerException.ASSERTION_EXCEPTION,
                        getOutputMessage(335,
                        ObjectServerException.ASSERTION_EXCEPTION)));
                 throw object_server_exception;
            }
        } else {
            thread_table = parameter_thread_table;
        }

        clone_table = getThreadTable(thread_type, thread_table);
        try {
            count = clone_table.getCount();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETTHREAD, 0, exception_object);
            throw exception_object;
        }
        for (int i = 0; i < count; i++) {
            target = clone_table.getThread(i);
            target_id = getThreadId(target.getName());
            if (target_id != null && target_id.compareTo(thread_id) == 0) {
                return target;
            }
        }
        return null;
    }

    private ThreadTable getThreadTable(
        String       thread_type, 
        ThreadTable  thread_table) {
        ThreadTable  clone_table;

        clone_table = thread_table.clone();
        return clone_table;
    }

    private boolean terminateThread(
        String       thread_type, 
        Thread       target_thread,
        ThreadTable  thread_table) throws Exception {
        Thread    monitor_child;
        Thread    service_child;
        Thread    system_child;

        if (isMonitorOn(thread_type) == true) {
            try {
                monitor_child = thread_table.getMonitor(target_thread);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_TERMINATETHREAD, 0, exception_object);
                throw exception_object;
            }
            stopThread(monitor_child,
                getFormattedOutputMessage(314,
                    ObjectServerHeader.MONITOR_THREAD_TYPE,
                    getOutputMessage(314,
                        ObjectServerHeader.MONITOR_THREAD_TYPE)));
            return true;
        } else if (isMonitor(thread_type) == true) {
            service_child = thread_table.getThreadbyMonitor(target_thread);
            stopThread(service_child, 
                getFormattedOutputMessage(313, ObjectServerHeader.SERVICE_THREAD_TYPE,
                    getOutputMessage(313, ObjectServerHeader.SERVICE_THREAD_TYPE)));
            return true;
        } else if (isSystem(thread_type) == true) {
            stopThread(target_thread,
                getFormattedOutputMessage(312, ObjectServerHeader.SYSTEM_THREAD_TYPE,
                    getOutputMessage(312, ObjectServerHeader.SYSTEM_THREAD_TYPE)));
            return true;
        }
        return false;
    }

    private boolean isSerializableThread(ThreadTable thread_table, Thread target_thread) {
        boolean flag;
        int     index;

        try {
            index = thread_table.getIndex(target_thread);
            flag = thread_table.isSerializable(index);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_ISSERIALIZABLETHREAD, 0, exception_object);
            return false;
        } 
        return flag;
    }

    private void setThreadCommand(ThreadTable thread_table, Thread target_thread, String command) throws Exception {
        int index;
        ObjectServerException object_server_exception;

        if (thread_table == null || target_thread == null || command == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
            throw object_server_exception;
        }
        index = thread_table.getIndex(target_thread);

        thread_table.setObject(index, command);
    }

    private boolean checkThreadCommand(ThreadTable thread_table, String command) throws Exception {
        int       index;
        int       count;
        String    object;
        ObjectServerException object_server_exception;

        if (thread_table == null || command == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_OPERATETHREADTABLE_SERVER)));
            throw object_server_exception;
        }

        printExecuteMessage( DEBUG_CHECKTHREADCOMMAND,
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, command,
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, thread_table.toString());

        try {
            count = thread_table.getCount();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CHECKTHREADCOMMAND, 0, exception_object);
            throw exception_object;
        }

        for (int i = 0; i < count; i++) {
            object = thread_table.getObject(i);
            if (object == null) { 
                continue;
            }
            if (command != null && command.compareTo(object) == 0) {
                return true;
            }
        }
        return false;
    }


    /**
     * dispatch()
     */
    private void dispatch() throws Exception {
        int          port;
        Socket       ssock;
        ServerSocket server_socket;
        ObjectServerException    object_server_exception;

        String sport = getPropertyValue( ObjectServerHeader.OSERV_REQUEST_PORT_KEY);
        if (sport == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(259, 259,
                ObjectServerError.REQUEST_PORT_NOT_FOUND_ERROR_SERVER));

            printVariableMessage(60, 60, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, ose.getMessage());
            throw ose;
        }
        printVariableMessage(186, 186, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_REQUEST_PORT, sport);

        try {
            port = Integer.valueOf(sport).intValue();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DISPATCH, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION);
            throw object_server_exception;
        }
        try {
            server_socket = new ServerSocket(port);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DISPATCH, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_SERVER_SOCKET_IO_EXCEPTION,
                ObjectServerError.JAVA_SERVER_SOCKET_IO_EXCEPTION);
            throw object_server_exception;
        }
        while (true) {
            try {
                ssock = server_socket.accept();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DISPATCH, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_SOCKET_ACCEPT_IO_EXCEPTION,
                    ObjectServerError.JAVA_SOCKET_ACCEPT_IO_EXCEPTION);
                throw object_server_exception;
            }
            if (ssock.isConnected() == true) {
                try {
                    ssock.setSoTimeout(timeout);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DISPATCH, 0, exception_object);
                }
            }
            printVariableMessage(61, 61, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_CONNECTION_START_KEY,
	     ssock.toString());
            if (reserveDispatchThread(ssock) == false) {
                printKeyMessage(386, 386, OSERV_DEBUG_RUN_VALUE,
                            LogMessage.SERVER_LOG_ACCEPT_ILLEGAL);
                printVariableMessage(318, 318, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_CONNECTION_END_KEY, ssock.toString());
                ssock.close();
                continue;
            }
        }
    }


    private void doRegisterConnection(Socket socket_object,
        RetryObjectInputStream  retry_object_input_stream) throws Exception {
        ObjectServerException object_server_exception;
        RegisterCommand    register_command;

        if (socket_object == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(298, 298,
                    ObjectServerError.SOCKET_NULL_DO_REGISTER_CONNECTION_SERVER));
            printLogMessage(OSERV_DEBUG_FULL_VALUE, 99, 99, null, null,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, ose.getMessage());
        
            throw ose;
        }
        if (retry_object_input_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(100, 100,
                    ObjectServerError.INPUTSTREAM_NULL_DO_REGISTER_CONNECTION_SERVER));
            printLogMessage(OSERV_DEBUG_FULL_VALUE, 101, 101, null, null,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, ose.getMessage());
            throw ose;
        }
        printExecuteMessage(DEBUG_DO_REGISTER_CONNECTION,
            socket_object.toString(), 
            retry_object_input_stream.toString());
        try {
            register_command = 
                (RegisterCommand)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCONNECTION, 0, exception_object);
            throw exception_object;
        }

        if (register_command.isList() == true) {
            printLogCallMessage(DEBUG_DO_REGISTER_LIST);

            doRegisterList(socket_object, 
                retry_object_input_stream, register_command);
            return;
        }
        if (register_command.isDelete() == true) {
            printLogCallMessage(DEBUG_DO_REGISTER_DELETE);
            doRegisterDelete(socket_object, 
                retry_object_input_stream, register_command);
            return;
        }
        if (register_command.isRegister() == true) {
            printLogCallMessage(DEBUG_DO_REGISTER_COMMAND);
            doRegisterCommand(socket_object, 
                retry_object_input_stream, register_command);
            return;
        }

        if (register_command.isClass() == true) {
            printLogCallMessage(DEBUG_DO_REGISTER_CLASS);
            doRegisterClass(socket_object,
                retry_object_input_stream, register_command);
            return;
        }

        object_server_exception = new ObjectServerException(
            getUserOutputMessage(297, 297,
            ObjectServerError.COMMAND_NOT_FOUND_REGISTER_CONNECTION_SERVER));
        throw object_server_exception;
    }

    private void doRegisterList(Socket socket_object,
        RetryObjectInputStream  retry_object_input_stream,
        RegisterCommand       register_command) throws Exception {

        ObjectOutputStream     object_output_stream;

        printKeyMessage(326, 326, OSERV_DEBUG_RUN_VALUE,
                    LogMessage.SERVER_LOG_LIST_COMMAND );

        try {
            object_output_stream = getObjectOutputStream(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERLIST, 0, exception_object);
            return;
        }
        printLogCallMessage( ObjectServerDebug.DEBUG_WRITEOBJECTTABLEENTRY);
        try {
            writeObjectTableEntry(object_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERLIST, 0, exception_object);
            return;
        }
        printLogCallMessage( ObjectServerDebug.DEBUG_WRITEFORWARDSTRINGTABLE );
        try {
            writeForwardStringTable(object_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERLIST, 0, exception_object);
            return;
        }
        writeEOSObject(object_output_stream, EOSObject.END_OF_SOCKET_MESSAGE);
    }

    private void doRegisterDelete(Socket socket_object,
        RetryObjectInputStream  retry_object_input_stream,
        RegisterCommand       register_command) throws Exception {

        int    idx;
        File   fdes;
        String  query_string;
        URI    uri_entry;
        String pfile;
        File   pfile_file;
        String suffix;
        String filepath;
        File   filepath_file;
        String classpath;
        String filename;
        String request_user_path;
        File request_user_path_file;
        DataOutputStream       data_output_stream;
        FileOutputStream       file_output_stream;
        ObjectOutputStream     object_output_stream;
        OutputStream  socket_output_stream;
        InputStream   socket_input_stream;

        classpath = register_command.getClassName();

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            filename  = new String(classpath  +SUFFIX_SEPARATOR+JAR_SUFFIX);
        } else {
            suffix = getThreadId(Thread.currentThread().getName());
            filename = new String(classpath +suffix+SUFFIX_SEPARATOR+JAR_SUFFIX);
        }
        printKeyMessage(345, 345, OSERV_DEBUG_RUN_VALUE,
            LogMessage.SERVER_LOG_DELETE_COMMAND);
        printVariableMessage(346, 346, OSERV_DEBUG_RUN_VALUE,
            LogMessage.SERVER_LOG_DELETE_OBJECT, classpath);

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            filepath = convertFilenameToFilepath( persistent_path, filename);
            pfile = replace(filepath, JAR_SUFFIX, VARIABLE_SUFFIX);
            pfile_file    = new File(pfile);
            if (pfile_file.exists() == true) {
                pfile_file.delete();
            }
            printVariableMessage(348, 348, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_OBJECT_DELETE_VARIABLE_FILE, pfile);
        } else {
            filepath = convertFilenameToFilepath( transient_path, filename);
        }
        printVariableMessage(347, 347, OSERV_DEBUG_RUN_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_DELETE_FILE, filepath);

        filepath_file = new File(filepath);
        if (filepath_file.exists() == true) {
            filepath_file.delete(); 
        }

        boolean check = false;
        printExecuteMessage( 
            getOutputMessage(629, ObjectServerDebug.DEBUG_DELETE_ENTRY_KEY) );

        printVariableMessage(350, 350, OSERV_DEBUG_TRACE_VALUE, 
            ObjectServerDebug.DEBUG_SERVICE_NAME, classpath);

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            try {
                operateObjectTable(ObjectTable.DELETE_OBJECT_TABLE, classpath, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERDELETE, 0, exception_object);
                return;
            }
        }
        idx = filename.indexOf(File.separator);
        if (idx >= 0) {
            File delete_dir_file;
            delete_dir_file = new File(persistent_path + File.separator + filename.substring(0, idx));
        }
        printLogCallMessage(getOutputMessage(681, 
            ObjectServerDebug.DEBUG_WRITEOBJECTTABLE_SERVER));
     
        printVariableMessage(581, 581, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_FILE_NAME, objfile);

        try {
            writeObjectTable(objfile);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERDELETE, 0, exception_object);
        }

        if (Thread.currentThread().getClass().getName().compareTo(
                TransientServer.getClassName()) != 0) {
            printLogCallMessage(ObjectServerDebug.DEBUG_MAKEQUERYPARAMETER);

            query_string = makeQueryParameter(FORWARD_COMMAND_KEY, 
                FORWARD_DELETE_COMMAND);
            printLogCallMessage(ObjectServerDebug.DEBUG_GETURIENTRY);
            printVariableMessage(366, 366, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_QUERY_STRING, query_string);

            uri_entry = getURIEntry(classpath, (String)query_string);
            if (uri_entry != null) {
                printLogCallMessage(ObjectServerDebug.DEBUG_CASTSERVICETOFORWARDS);
            }
            try {
                castServiceToForwards(uri_entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERDELETE, 0, exception_object);
            }
            try {
                object_output_stream = getObjectOutputStream(socket_object);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERDELETE, 0, exception_object);
                throw exception_object;
            }
            try {
                writeEOSObject(object_output_stream, EOSObject.END_OF_SOCKET_MESSAGE);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERDELETE, 0, exception_object);
                throw exception_object;
            }
        }
    }

    private void doRegisterCommand(Socket socket_object,
        RetryObjectInputStream  retry_object_input_stream,
        RegisterCommand       register_command) throws Exception {

        int    idx;
        File   fdes;
        File   ddes;
        File   pdes;
        File   temp_des;
        String filename;
        String pfile;
        String temp_file;
        String dirpath;
        String filepath;
        String classpath;
        String suffix;
        String request_user_path;
        boolean check;
        byte[] buffer;
        byte[] class_body;
        Integer ival;
        int    rt;
        int    bkcount;
        URI    uri_entry;
        ObjectTable  clone_table;
        String       execute_type;
        File request_user_path_file;
        DataOutputStream       data_output_stream;
        FileOutputStream       file_output_stream;
        JarOutputStream        jar_output_stream;
        DataOutputStream       data_output_stream_temp;
        FileOutputStream       file_output_stream_temp;

        DataInputStream       data_input_stream_temp;
        FileInputStream       file_input_stream_temp;
        JarInputStream        jar_input_stream_temp;
        ZipEntry                zip_entry;

        ObjectOutputStream     object_output_stream;
        OutputStream  socket_output_stream;
        InputStream   socket_input_stream;
        ObjectServerException  object_server_exception;
        ByteArrayOutputStream    byte_array_output_stream;

        check = false;
        classpath = register_command.getClassName();

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            filename = convertEntryToFilename(classpath);
        } else {
            suffix = getThreadId(Thread.currentThread().getName());
            filename = convertEntryToFilename(classpath+suffix);
        }

        printKeyMessage(369, 369, OSERV_DEBUG_RUN_VALUE,
            LogMessage.SERVER_LOG_REGISTER_COMMAND );
        printVariableMessage(370, 370, OSERV_DEBUG_RUN_VALUE,
            LogMessage.SERVER_LOG_REGISTER_OBJECT, filename);

        printKeyMessage(371, 371, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_JAVA_INPUTSTREAM);

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            filepath = convertFilenameToFilepath(persistent_path, filename);
        } else {
            filepath = convertFilenameToFilepath(transient_path, filename);
        }
        pfile = replace(filepath, JAR_SUFFIX, VARIABLE_SUFFIX);
        temp_file = replace(filepath, JAR_SUFFIX, TEMP_SUFFIX);

        printKeyMessage(375, 375, OSERV_DEBUG_RUN_VALUE, temp_file);

        if (register_command.getExecuteType().compareTo(
                        RegisterCommand.PERSISTENT_TYPE) == 0) {
            printDebugMessage( OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(372,
                    ObjectServerDebug.DEBUG_FILE_NAME,
                    getOutputMessage(372,
                        ObjectServerDebug.DEBUG_FILE_NAME),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                    filepath), 
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                getFormattedOutputMessage(630,
                    ObjectServerDebug.DEBUG_PERSISTENT_PATH,
                    getOutputMessage(630,
                        ObjectServerDebug.DEBUG_PERSISTENT_PATH),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                    persistent_path),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                getFormattedOutputMessage(631,
                    ObjectServerDebug.DEBUG_JAR_NAME,
                    getOutputMessage(631,
                        ObjectServerDebug.DEBUG_JAR_NAME),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                pfile));
        } else {
            printDebugMessage( OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(374,
                    ObjectServerDebug.DEBUG_FILE_NAME,
                    getOutputMessage(374,
                        ObjectServerDebug.DEBUG_FILE_NAME),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                    filepath), 
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
            getFormattedOutputMessage(632,
                ObjectServerDebug.DEBUG_PERSISTENT_PATH,
                getOutputMessage(632,
                    ObjectServerDebug.DEBUG_PERSISTENT_PATH),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, persistent_path),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            getFormattedOutputMessage(636,
                ObjectServerDebug.DEBUG_JAR_NAME,
                getOutputMessage(636,
                    ObjectServerDebug.DEBUG_JAR_NAME),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                pfile));
        }

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {

            printVariableMessage(306, 306, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_SEARCH_OPERATE_OBJECT_TABLE, classpath);
            try {
                operateObjectTable(ObjectTable.SEARCH_OBJECT_TABLE, classpath, null);
                check = true;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            }
            if (check == true) {
                printTraceMessage(TRACE_DOREGISTERCOMMAND, 0);
            }
        }

        fdes = new File(filepath);
        if (fdes.exists() == true) {
            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                getFormattedOutputMessage(107,
                    ObjectServerDebug.DEBUG_DELETE_KEY, 
                    getOutputMessage(107,
                        ObjectServerDebug.DEBUG_DELETE_KEY), 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    TraceMessage.getCurrentTraceString(
                        TRACE_CLASSNAME, TRACE_DOREGISTERCOMMAND, 0),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    filepath));
            fdes.delete();
            pdes = new File(pfile);
            if (pdes.exists() == true) {
                pdes.delete();
            }
        }
        printVariableMessage(104, 104, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_MAKE_KEY,
            ObjectServerDebug.DEBUG_JAVA_FILEOUTPUTSTREAM);

        temp_des = new File(temp_file);

        try {
            file_output_stream = new FileOutputStream(fdes);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);


            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            file_output_stream_temp = new FileOutputStream(temp_des);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        printVariableMessage(482, 482, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_MAKE_KEY,
            ObjectServerDebug.DEBUG_JAVA_DATAOUTPUTSTREAM);

        try {
            data_output_stream = new DataOutputStream((OutputStream)file_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            try {
                object_output_stream = getObjectOutputStream(socket_object);
                printLogCallMessage( ObjectServerDebug.DEBUG_WRITEOBJECT );
                object_output_stream.writeObject(exception_object);
            } catch (Exception eep) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, eep);
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_DATA_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            data_output_stream_temp = new DataOutputStream((OutputStream)file_output_stream_temp);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            try {
                object_output_stream = getObjectOutputStream(socket_object);
                printLogCallMessage( ObjectServerDebug.DEBUG_WRITEOBJECT );
                object_output_stream.writeObject(exception_object);
            } catch (Exception eep) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, eep);
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_DATA_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_DATA_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }

        bkcount = (register_command.getFileSize() + maximum - 1)/maximum;
        for (int j = 0; j < bkcount; j++ ) {
            if (register_command.getFileSize() > (j + 1) * maximum) {
                buffer = new byte[maximum];
            } else {
                buffer = new byte[
                    register_command.getFileSize() - (j * maximum)];
            }
            try {
                printLogCallMessage( ObjectServerDebug.DEBUG_READJARFILE );
                readJarFile(retry_object_input_stream, buffer);
                printLogCallMessage( ObjectServerDebug.DEBUG_WRITEJARFILE );

                writeJarFile(data_output_stream_temp, buffer);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
                try {
                    printLogCallMessage( ObjectServerDebug.DEBUG_WRITEOBJECT );
                    object_output_stream = getObjectOutputStream(socket_object);
                    object_output_stream.writeObject(exception_object);
                } catch (Exception eep) {
                    printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, eep);
                }
                temp_des.delete();
                return;
            }
	}
        try {
            data_output_stream_temp.close();
            file_input_stream_temp = new FileInputStream(temp_des);
            jar_input_stream_temp = new JarInputStream(file_input_stream_temp);
            jar_output_stream = new JarOutputStream(file_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            temp_des.delete();
            throw exception_object;
        }

        buffer = new byte[4096];
        for (    zip_entry = jar_input_stream_temp.getNextEntry();
                 zip_entry != null;
                 zip_entry = jar_input_stream_temp.getNextEntry() ) {

            try {
                jar_output_stream.putNextEntry(zip_entry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
                temp_des.delete();
                throw exception_object;
            }

            byte_array_output_stream = new ByteArrayOutputStream();
            while (true) {
                try {
                    rt = jar_input_stream_temp.read(buffer, 0, buffer.length);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
                    temp_des.delete();
                    throw exception_object;
                }
                if (rt == -1) {
                    break;
                }
                byte_array_output_stream.write(buffer, 0, rt);
            }
            class_body = byte_array_output_stream.toByteArray();
            try {
                jar_output_stream.write(class_body, 0, class_body.length);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
                temp_des.delete();
                throw exception_object;
            }
        }

        try {
            jar_input_stream_temp.close();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
        }
        temp_des.delete();
      
        try {
            jar_output_stream.close();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            throw exception_object;
        }
        
        printLogCallMessage( DEBUG_WRITE_EOS_OBJECT );
          
        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            try {
                object_output_stream = getObjectOutputStream(socket_object);
                writeEOSObject(object_output_stream, EOSObject.END_OF_SOCKET_MESSAGE);

            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            }
            printVariableMessage(393, 393, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_WRITEOBJECTTABLE_AT_END_OF_REGISTER, objfile);
        }

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {
            printVariableMessage(307, 307, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_ADD_OPERATE_OBJECT_TABLE, classpath);
            try {
                clone_table = operateObjectTable(ObjectTable.ADD_OBJECT_TABLE, classpath, object_table.getDefaultType());
                writeObjectTable(objfile);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
                return; 
            }
            execute_type = clone_table.getType(clone_table.getCount()-1);
        } else {
            execute_type = null;
        }

        try {
            executeOServ(register_command.getExecuteType(), 
                execute_type,
                filename, classpath);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
            if (exception_object.getMessage() == null ||
                exception_object.getMessage().compareTo(
                ObjectServerError.JAVA_CLASS_GET_METHODS_EXCEPTION) != 0) {
                try {
                    operateObjectTable(ObjectTable.DELETE_OBJECT_TABLE, classpath, null);
                    writeObjectTable(objfile);
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, second_exception_object);
                }
                return; 
            }
        }

        printLogCallMessage(ObjectServerDebug.DEBUG_GETURIENTRY);

        if (Thread.currentThread().getClass().getName().compareTo(
            TransientServer.getClassName()) != 0) {

            uri_entry = getURIEntry(classpath, (String)null);
            if (uri_entry != null) {
                printLogCallMessage(ObjectServerDebug.DEBUG_CASTSERVICETOFORWARDS);
                try {
                    castServiceToForwards(uri_entry);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOREGISTERCOMMAND, 0, exception_object);
                }
            }
        }
    }

    private void doRegisterClass(Socket socket_object,
        RetryObjectInputStream  retry_object_input_stream,
        RegisterCommand       register_command) throws Exception {

        String s1;
        int c1;
        int c2;
        String filename;
        String jar_file;
        String class_name;
        int idx;
        String request_user_path;
        File request_user_path_file;
        DataOutputStream       data_output_stream;
        FileOutputStream       file_output_stream;
        ObjectOutputStream     object_output_stream;
        OutputStream  socket_output_stream;
        InputStream   socket_input_stream;

        class_name = register_command.getClassName();
        filename = register_command.getTargetName();
        object_output_stream = getObjectOutputStream(socket_object);
        printLogCallMessage( ObjectServerDebug.DEBUG_WRITECLASSFILE_SERVER );

        writeClassFile(object_output_stream, class_name, filename);
        writeEOSObject(object_output_stream, EOSObject.END_OF_SOCKET_MESSAGE);
    }

    private void deleteInputStream(CallMethodWrapper call_method) throws Exception {
        int count;
        String  file_name;
        String  parameter_type;
        File    file_descriptor;
        Object  object_instance;
        NamedInputStream       named_input_stream ;

        try {
            count = call_method.getParameterCount();
        } catch (Exception exception_object) {
            throw exception_object;
        }
        if (count <= 0) {
            return;
        }
        for (int i = 0; i < count; i++) {
            try {
                parameter_type = call_method.getParameterType(i);
            } catch (Exception exception_object) {
                continue;
            }
            if (parameter_type == null) { continue; }
            if (parameter_type.compareTo( int.class.getName()) == 0
            || parameter_type.compareTo( long.class.getName()) == 0
            || parameter_type.compareTo( float.class.getName()) == 0
            || parameter_type.compareTo( double.class.getName()) == 0
            || parameter_type.compareTo( short.class.getName()) == 0
            || parameter_type.compareTo( char.class.getName()) == 0
            || parameter_type.compareTo( byte.class.getName()) == 0
            || parameter_type.compareTo( boolean.class.getName()) == 0
            ) {
                continue;
            }
            try {
                if (call_method.isInputStreamParameter(i) == false) {
                    continue;
                }
            } catch (Exception exception_object) {
                continue;
            }

            object_instance = call_method.getParameter(i);
            if (object_instance != null &&
                (object_instance instanceof NamedInputStream) == true) {

                named_input_stream = (NamedInputStream)object_instance;

                try {
                    named_input_stream.close();
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DELETESTREAM, 0, exception_object);
                    throw exception_object;
                }
                file_name = named_input_stream.getFileName();
                file_descriptor = new File(file_name);
                if (file_descriptor.exists() == true) {
                    printVariableMessage(303, 303, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FILE_NAME, 
                    file_name);
                    file_descriptor.delete();
                }
            }
        }
    }

    private Object getNetworkObject(Object obj, String type,
        CallMethodWrapper call_method) throws Exception {
        int      int_value;
        long     long_value;
        float    float_value;
        double   double_value;
        byte     byte_value;
        boolean  boolean_value;
        char     char_value;
        short    short_value;
        Object   object_value;
        byte[]   convert_value;

        NetworkObject  network_object;

        network_object = new NetworkObject();

        if (type.compareTo( ObjectServerHeader.VOID_TYPE_KEY) == 0) {
            network_object.setVoidType();
            return network_object;
        }
        if (type.compareTo( int.class.getName()) == 0) {
            try {
                int_value = call_method.getInt(obj);
            } catch (Exception exception_object) {
                throw exception_object;
            }

            printValueMessage(String.valueOf(int_value));
            network_object.setInt(int_value);
            return network_object;
        } 
        if (type.compareTo( long.class.getName()) == 0) {
            long_value = call_method.getLong(obj);
            printValueMessage(String.valueOf(long_value));
            network_object.setLong(long_value);
            return network_object;
        } 
        if (type.compareTo( float.class.getName()) == 0) {
            float_value = call_method.getFloat(obj);
            printValueMessage(String.valueOf(float_value));
            network_object.setFloat(float_value);
            return network_object;
        }
        if (type.compareTo( double.class.getName()) == 0) {
            double_value = call_method.getDouble(obj);
            printValueMessage(String.valueOf(double_value));
            network_object.setDouble(double_value);
            return network_object;
        } 
        if (type.compareTo( short.class.getName()) == 0) {
            short_value = call_method.getShort(obj);
            printValueMessage(String.valueOf(short_value));
            network_object.setShort(short_value);
            return network_object;
        } 
        if (type.compareTo( byte.class.getName()) == 0) {
            byte_value = call_method.getByte(obj);
            printValueMessage(String.valueOf(byte_value));
            network_object.setByte(byte_value);
            return network_object;
        } 
        if (type.compareTo( boolean.class.getName()) == 0) {
            boolean_value = call_method.getBoolean(obj);
            printValueMessage(String.valueOf(boolean_value));
            network_object.setBoolean(boolean_value);
            return network_object;
         } 
         if (type.compareTo( char.class.getName()) == 0) {
            char_value = call_method.getChar(obj);
            printValueMessage(String.valueOf(char_value));
            network_object.setChar(char_value);
            return network_object;
        } 
        try {
            object_value = call_method.get(obj);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETNETWORKOBJECT, 0, exception_object);
            throw exception_object;
        }
        network_object.set(object_value);

        return network_object;
    }

    private void setParameter(Object obj, String ptype, int index_number,
        CallMethodWrapper call_method, ArgumentInputStream argument_input_stream) 
        throws Exception {
        int      int_value;
        long     long_value;
        float    float_value;
        double   double_value;
        byte     byte_value;
        boolean  boolean_value;
        char     char_value;
        short    short_value;
        Object   object_value;

        try {
            if (ptype.compareTo(
                            int.class.getName()) == 0) {
                int_value = argument_input_stream.getintArgument();
                printValueMessage(String.valueOf(int_value));
                call_method.setInt(obj, int_value, index_number);
            } else if (ptype.compareTo(
                            long.class.getName()) == 0) {
                long_value = argument_input_stream.getlongArgument();
                printValueMessage(String.valueOf(long_value));
                call_method.setLong(obj, long_value, index_number);
            } else if (ptype.compareTo(
                        float.class.getName()) == 0) {
                float_value = argument_input_stream.getfloatArgument();
                printValueMessage(String.valueOf(float_value));
                call_method.setFloat(obj, float_value, index_number);
            } else if (ptype.compareTo(
                    double.class.getName()) == 0) {
                double_value = argument_input_stream.getdoubleArgument();
                printValueMessage(String.valueOf(double_value));
                call_method.setDouble(obj, double_value, index_number);
            } else if (ptype.compareTo(
                        byte.class.getName()) == 0) {
                byte_value = argument_input_stream.getbyteArgument();
                printValueMessage(String.valueOf(byte_value));
                call_method.setByte(obj, byte_value, index_number);
            } else if (ptype.compareTo(
                        boolean.class.getName()) == 0) {
                boolean_value = argument_input_stream.getbooleanArgument();
                printValueMessage(String.valueOf(boolean_value));
                call_method.setBoolean(obj, boolean_value, index_number);
            } else if (ptype.compareTo(
                        char.class.getName()) == 0) {
                char_value = argument_input_stream.getcharArgument();
                printValueMessage(String.valueOf(char_value));
                call_method.setChar(obj, char_value, index_number);
            } else if (ptype.compareTo(
                            short.class.getName()) == 0) {
                short_value = argument_input_stream.getshortArgument();
                printValueMessage(String.valueOf(short_value));
                call_method.setShort(obj, short_value, index_number);
            } else {
                object_value = argument_input_stream.getObjectArgument();
                printValueMessage(object_value.getClass().getName());
               
                call_method.set(obj, object_value, index_number);

                if (object_value != null && (object_value instanceof NamedInputStream) == true) {
                    NamedInputStream named_input_stream;
                    named_input_stream 
                        = (NamedInputStream)object_value;
                    OServInterruptHandler local_thread_handler;
                    local_thread_handler = (OServInterruptHandler)Thread.currentThread().getUncaughtExceptionHandler();
                    local_thread_handler.addStream(named_input_stream);
                }
            }
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETPARAMETER, 0, exception_object);
            throw exception_object;
        }
        return;
    }

    private String getServiceName(String cmd) throws Exception {
        int               l;
        int               count;
        String            command;
        String[]          sag;
        StringTokenizer   string_tokenizer_object;

        if (cmd == null || cmd.length() == 0) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(305, 305,
                    ObjectServerError.COMMAND_PARAMETER_NULL_SERVER));
            throw ose;
        }
        command = replace(cmd, OBJECT_SEPARATOR + OBJECT_SEPARATOR,
            OBJECT_SEPARATOR + PADDING_STRING + OBJECT_SEPARATOR);
        if (String.valueOf(command.charAt(cmd.length() - 1)).compareTo(
            OBJECT_SEPARATOR) == 0) {
            command = command.concat(PADDING_STRING);
        }
        string_tokenizer_object = new StringTokenizer(command, OBJECT_SEPARATOR);
        count = string_tokenizer_object.countTokens();

        sag = new String[count];
        l = 0;
        while (string_tokenizer_object.hasMoreTokens()) {
            sag[l] = string_tokenizer_object.nextToken();
            l++;
        }
        if (count < 2 || sag[0].length() == 0 || sag[0].compareTo(PADDING_STRING) == 0 ) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(536, 536,
                    ObjectServerError.COMMAND_PARAMETER_NULL_SERVER));
            throw ose;
        }
        command = sag[0];
        return command; 
    }

    /**
     * This method calls specific method of target class registered.
     *
     * @param     String   -  command
     *            ObjectInputStream
     * @return    Object
     *
     * Format of command is as like follows,
     *    Class name/Interface name/Method name/Return Type/
     *    Parameter Type, ../Exception Type, ../
     */
    private Object invoke(String type_string, String cmd, RetryObjectInputStream retry_object_input_stream,
        ObjectOutputStream object_output_stream) throws Exception {
        URL      url;
        Method[] method_object;
        Class sp;
        Object rt;
        Object ret;
        Object obj;
        Class[] ifs;
        Class[] fag;
        String[] sag;
        Object[] rag;
        int      l;
        int      count;
        String[] rptypes;
        Class[] ptypes;
        String   filepath;
        String   archive_path;
        String   pfile = null;
        String  reqclass;
        String  reqpath;
        String  fldtype;
        String  command;
        String   methodname;
        String   logfile;
        String   execution_type;
        File     fdes;
        File     edes;
        Integer  lsize;
        int      idx;
        int      index;
        int      read_parameter_count;
        ClassLoader   save_class_loader;
        URI      target_entry;
        Object   error;
        boolean  check_flag = false;
        ObjectTable clone_object = null;
        ArgumentInputStream argument_input_stream;
        CallMethod call_method = null;
        NetworkObject network_object = new NetworkObject();
        PrintStream save_out;
        PrintStream save_err;
        PrintStream sysout;
        String      service;
        StringTokenizer string_tokenizer_object;
        ObjectServerException    object_server_exception;

        WriteVariableWrapper write_variable_wrapper;
        ReadVariableWrapper read_variable_wrapper;
        CallMethodWrapper call_method_wrapper = null;
        Class     write_variable_class;
        Class     read_variable_class;
        Class     call_method_class;

        printKeyMessage(396, 396, OSERV_DEBUG_RUN_VALUE,
                    LogMessage.SERVER_LOG_INVOKE_REQUEST );

        if (cmd == null || cmd.length() == 0) {
            object_server_exception 
                = new ObjectServerException( getUserOutputMessage(353, 353,
                    ObjectServerError.COMMAND_PARAMETER_NULL_SERVER));
            throw object_server_exception;
        }
        if (retry_object_input_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(196, 196,
	            ObjectServerError.INPUT_STREAM_PARAMETER_NULL_SERVER));
            throw ose;
        }
        printVariableMessage(397, 397, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_INVOKE_SERVICE, cmd);
        printVariableMessage(398, 398, OSERV_DEBUG_RUN_VALUE, 
            ObjectServerDebug.DEBUG_INVOKE_SERVICE, cmd);

        command = replace(cmd, OBJECT_SEPARATOR + OBJECT_SEPARATOR,
            OBJECT_SEPARATOR + PADDING_STRING + OBJECT_SEPARATOR);
        if (String.valueOf(command.charAt(command.length() - 1)).compareTo(
            OBJECT_SEPARATOR) == 0) {
            command = command.concat(PADDING_STRING);
        }
        string_tokenizer_object = new StringTokenizer(command, OBJECT_SEPARATOR);
        count = string_tokenizer_object.countTokens();
        printVariableMessage(399, 399, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_COUNT_OF_PARAMETER, String.valueOf(count));

        sag = new String[count];
        l = 0;
        while (string_tokenizer_object.hasMoreTokens()) {
            sag[l] = string_tokenizer_object.nextToken();
            if (sag[l].compareTo(BLANK_STRING) == 0) {
                sag[l] = null;
            }
            l++;
        }

        printVariableMessage(400, 400, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_SPECIFIED_PARAMETER,
            String.valueOf((int)sag.length));

        for (l = 0; l < sag.length; l++) {
            if (sag[l] != null) {
                printVariableMessage(401, 401, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_PARAM_KEY, 
                    replace(sag[l], VARIABLE_SEPARATOR,
                        OBJECT_DOT_SEPARATOR),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(l));
            } else {
                printVariableMessage(401, 401, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_PARAM_KEY, 
                    ObjectServerDebug.DEBUG_NULL_KEY,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(l));
            }
        }

        methodname = sag[2];
        user_debug = false;
        /*
        */
        idx = methodname.lastIndexOf(DEBUG_FUNCTION_SUFFIX);
        if (idx > 0) {
            user_debug = true;
            methodname = methodname.substring(0, idx);
        }
        /*
        */
        reqpath = sag[0];
        reqclass = reqpath;

        idx = reqpath.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0 && idx < reqpath.length() - 1) {
            reqclass = reqpath.substring(idx + 1);
        }
        error = null;
        boolean check = false;
        String  ckreq;

        save_class_loader = null;
        if (type_string.compareTo(RegisterCommand.PERSISTENT_TYPE) == 0) {
            ckreq = reqpath + SUFFIX_SEPARATOR + JAR_SUFFIX;
            try {
                operateObjectTable(ObjectTable.SEARCH_OBJECT_TABLE, 
                    reqpath, null);
                check = true;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
            }
        } else {
            ckreq = reqpath 
                + getThreadId(Thread.currentThread().getName())
                + SUFFIX_SEPARATOR + JAR_SUFFIX;
            check = true;
        }
        if (check == true) {
            if (type_string.compareTo(RegisterCommand.PERSISTENT_TYPE) == 0) {
                filepath = convertFilenameToFilepath(
                    persistent_path,
                    ckreq);
            } else {
                filepath = convertFilenameToFilepath(
                    transient_path,
                    ckreq);
            }
            archive_path = filepath;
            filepath = new String(JAR_URL_HEAD + filepath + JAR_URL_END);

            printVariableMessage(402, 402, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_FILE_NAME, filepath);

            try {
                url = new URL(filepath);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);

                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_URL_EXCEPTION,
                    ObjectServerError.JAVA_URL_EXCEPTION);
                throw object_server_exception ;
            }
            URL[] urls = new URL[1];
            urls[0] = url;

            URLClassLoader uloader = new URLClassLoader(urls);      
            printVariableMessage(404, 404, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_URL_KEY, url.toString());
            printVariableMessage(405, 405, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_CLASS_LOADER_KEY, uloader.toString());

            if (getContextClassLoader() != null) {
                printVariableMessage(403, 403, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_CLASS_LOADER_KEY, 
                    getContextClassLoader().toString());
            } else {
                printVariableMessage(403, 403, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_CLASS_LOADER_KEY, 
                    ObjectServerDebug.DEBUG_NULL_KEY);
            }

            save_class_loader = getContextClassLoader();
            setContextClassLoader(uloader);

            UserClassLoader  user_class_loader;
            user_class_loader = new UserClassLoader();
            user_class_loader.setMessageResource(getMessageResource());
            try {
                sp = user_class_loader.readUserClass(archive_path, reqpath);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }
            if (sp == null) {
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION,
                    ObjectServerError.JAVA_CLASS_FOR_NAME_CLASS_NOT_FOUND_EXCEPTION);
                printExceptionMessage(TRACE_INVOKE, 0, object_server_exception);
                throw object_server_exception ;
            }


            Class object_output_manager_class;
            Class object_input_manager_class;
            Class call_method_invoke_class;
            Class call_method_transfer_class;
            try {
                object_output_manager_class = 
                    user_class_loader.getSpecifiedClass(
                        OSERV_OBJECT_OUTPUT_MANAGER,
                        ObjectOutputManagerData.getClassData());
                object_input_manager_class = 
                    user_class_loader.getSpecifiedClass(
                        OSERV_OBJECT_INPUT_MANAGER,
                        ObjectInputManagerData.getClassData());

                call_method_invoke_class = 
                    user_class_loader.getSpecifiedClass(
                        OSERV_CALL_METHOD_INVOKE,
                        CallMethodInvokeData.getClassData());
                call_method_transfer_class = 
                    user_class_loader.getSpecifiedClass(
                        OSERV_CALL_METHOD_TRANSFER,
                        CallMethodTransferData.getClassData());
                call_method_class = 
                    user_class_loader.getSpecifiedClass(
                        OSERV_CALL_METHOD,
                        CallMethodData.getClassData());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }
            try {
                call_method_wrapper 
                    = new CallMethodWrapper(call_method_class);
                call_method_wrapper.setMessageResource(
                    getMessageResource());
                if (getDebug() > 0) {
                    call_method_wrapper.setDebug(getDebug() );
                    call_method_wrapper.setLogStream(getLogStream());
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }
            Class base_variable_class;
            try {
                base_variable_class = 
                    user_class_loader.getSpecifiedClass(OSERV_BASE_VARIABLE,
                    BaseVariableData.getClassData());
                write_variable_class =
                    user_class_loader.getSpecifiedClass(
                        OSERV_WRITE_VARIABLE,
                        WriteVariableData.getClassData());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }
            try {
                write_variable_wrapper 
                    = new WriteVariableWrapper(write_variable_class);
                write_variable_wrapper.setMessageResource(
                    getMessageResource() );
                if (getDebug() > 0) {
                    write_variable_wrapper.setDebug(getDebug() );
                    write_variable_wrapper.setLogStream(getLogStream());
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }
            try {
                read_variable_class =
                    user_class_loader.getSpecifiedClass(OSERV_READ_VARIABLE,
                     ReadVariableData.getClassData());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }
            try {
                read_variable_wrapper 
                    = new ReadVariableWrapper(read_variable_class);
                read_variable_wrapper.setMessageResource(
                    getMessageResource() );
                if (getDebug() > 0) {
                    read_variable_wrapper.setDebug(getDebug() );
                    read_variable_wrapper.setLogStream(getLogStream());
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                throw exception_object;
            }

            try {
                obj = sp.newInstance();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_CLASS_NEW_INSTANCE_EXCEPTION,
                    ObjectServerError.JAVA_CLASS_NEW_INSTANCE_EXCEPTION);
                throw object_server_exception ;
            }
            if ((sp.getName()).compareTo(sag[0]) != 0) {
                printVariableMessage(406, 406, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_CLASS_NAME, sp.getName());

                ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(308, 308,
                    ObjectServerError.PARAMETER_ERROR_SERVER));
                throw ose;
            }
            ifs = sp.getInterfaces();
            printVariableMessage(407, 407, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_INTERFACE_NUMBER,
                String.valueOf((int)ifs.length));

            for (int j = 0;j < ifs.length;j++) {
                if ((ifs[j].getName()).compareTo(sag[1]) != 0) {
                    printVariableMessage(408, 408, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_INTERFACE_CLASS, ifs[j].getName());
                    continue;
                }
                printVariableMessage(409, 409, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_INTERFACE_CLASS, ifs[j].getName());

                try {
                    method_object = ifs[j].getMethods();
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_EXECUTEOSERV, 0, exception_object);
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_CLASS_GET_METHODS_EXCEPTION,
                        ObjectServerError.JAVA_CLASS_GET_METHODS_EXCEPTION);
                    throw object_server_exception ;
                }
                for (int k = 0;k < method_object.length;k++) {
                    if ((method_object[k].getName()).compareTo(methodname) != 0) {
                        continue;
                    }
                    printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                        getFormattedOutputMessage(411,
                            ObjectServerDebug.DEBUG_RETURN_TYPE,
                            getOutputMessage(411,
                                ObjectServerDebug.DEBUG_RETURN_TYPE),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            method_object[k].getReturnType().getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            sag[3]));
                    String return_variable;
                    return_variable = VariableType.convert(
                        method_object[k].getReturnType().getName());
                    if (return_variable.compareTo(sag[3]) != 0) {
                        printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                            getFormattedOutputMessage(679,
                                ObjectServerDebug.DEBUG_RETURN_TYPE,
                                getOutputMessage(679,
                                    ObjectServerDebug.DEBUG_RETURN_TYPE),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
		                method_object[k].getReturnType().getName(), 
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                sag[3]));
                        continue;
                    }
                    if (sag.length >= 5 && sag[4] != null && sag[4].compareTo(PADDING_STRING) != 0) {
                        ptypes = method_object[k].getParameterTypes();
                        string_tokenizer_object  = new StringTokenizer(sag[4], PARAMETER_SEPARATOR);
                        if (ptypes != null && ptypes.length != string_tokenizer_object.countTokens()) {
                            printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                                getFormattedOutputMessage(412,
                                    ObjectServerDebug.DEBUG_PARAMETER_COUNT_ERROR,
                                    getOutputMessage(412,
                                        ObjectServerDebug.DEBUG_PARAMETER_COUNT_ERROR),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf((int)ptypes.length),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf(string_tokenizer_object.countTokens())));
                            continue;
                        }
                        l = 0;
                        while(string_tokenizer_object.hasMoreTokens()) {
                            String p1 = string_tokenizer_object.nextToken();
                            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                                getFormattedOutputMessage(680,
                                    ObjectServerDebug.DEBUG_PARAM_KEY,
                                    getOutputMessage(680,
                                        ObjectServerDebug.DEBUG_PARAM_KEY),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    p1,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    ptypes[l].getName()));
                            String variable_name;
                            try {
                                variable_name = VariableType.convert(
                                    ptypes[l].getName());
                            } catch (Exception exception_object) {
                                printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                                break;
                            }

                            if (p1.compareTo(variable_name) != 0) {
                                break;
                            }
                            l = l + 1;
                        }
                        if (l < ptypes.length) {
                            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                                getFormattedOutputMessage(414,
                                    ObjectServerDebug.DEBUG_PARAMETER_COUNT_ERROR,
                                    getOutputMessage(414,
                                        ObjectServerDebug.DEBUG_PARAMETER_COUNT_ERROR),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf(l),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf((int)ptypes.length)));
                            continue;
                        }
                    }
                    ptypes = method_object[k].getExceptionTypes();
                    if (sag.length >= 6 && sag[5] != null && sag[5].compareTo(PADDING_STRING) != 0) {
                    
                        string_tokenizer_object = new StringTokenizer(sag[5], PARAMETER_SEPARATOR);
                        if (ptypes.length != string_tokenizer_object.countTokens()) {
                            printDebugMessage(OSERV_DEBUG_TRACE_VALUE,
                                getFormattedOutputMessage(415,
                                    ObjectServerDebug.DEBUG_EXCEPTION_COUNT_ERROR,
                                    getOutputMessage(415,
                                        ObjectServerDebug.DEBUG_EXCEPTION_COUNT_ERROR),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf((int)ptypes.length),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf(string_tokenizer_object.countTokens())));
                            continue;
                        }
                        l = 0;
                        while(string_tokenizer_object.hasMoreTokens()) {
                            String p1 = string_tokenizer_object.nextToken();
                            if (p1.compareTo(ptypes[l].getName()) != 0) {
                                break;
                            }
                            l = l + 1;
                        }
                        if (l < ptypes.length) {
                            printDebugMessage( OSERV_DEBUG_TRACE_VALUE,
                                getFormattedOutputMessage(417,
                                    ObjectServerDebug.DEBUG_EXCEPTION_COUNT_ERROR,
                                    getOutputMessage(417,
                                        ObjectServerDebug.DEBUG_EXCEPTION_COUNT_ERROR),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf(l),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    String.valueOf((int)ptypes.length)));
                            continue;
                        }
                    }
                    ptypes = method_object[k].getParameterTypes();
                    printVariableMessage(633, 633, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_PARAMETER_NUMBER,
		    	String.valueOf(ptypes.length));

                    String request_user_path;
                    if (type_string.compareTo(RegisterCommand.PERSISTENT_TYPE) == 0) {
                        check_flag = false;

                        try {
                            clone_object = operateObjectTable(
                                ObjectTable.SEARCH_OBJECT_TABLE, 
                                reqpath, null);
                            check_flag = true;
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                        }
                        if (check_flag != false && clone_object.getType(0).compareTo(
                            ThreadTable.SERIALIZABLE_SERVICE) == 0) {

                            request_user_path = user_path + File.separator + reqpath ;

                        } else {
                          request_user_path = user_path + File.separator + reqpath
                          + getThreadId(Thread.currentThread().getName());
                        }
                    } else {
                        request_user_path = user_path + File.separator 
                        + convertFilenameToFilepath(transient_path, reqpath);
                    }

                    File request_user_path_file;
                    request_user_path_file = new File(request_user_path);
                    if (request_user_path_file.exists() == false) {
                        request_user_path_file.mkdir();
                    }

                    printLogCallMessage(ObjectServerDebug.DEBUG_JAVA_SETPROPERTY,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY, request_user_path);

                    try {
                        System.clearProperty(USER_DIR_KEY);
                        System.setProperty(USER_DIR_KEY, request_user_path);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                        ObjectServerException ose = new ObjectServerException(exception_object);
                        error = ose;
                        break;
                    
                    }

                    argument_input_stream = null;
                    if (ptypes.length > 0) {
                        try {
                            argument_input_stream = new ArgumentInputStream(
                                retry_object_input_stream );
                            argument_input_stream.setPersistentPath(
                                request_user_path );
                            argument_input_stream.setMessageResource(getMessageResource());
                            call_method_wrapper.setMethod(
                                method_object[k]);
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                            ObjectServerException ose = new ObjectServerException(exception_object);
                            error = ose;
                            break;
                        }

                        printVariableMessage(420, 420, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_OBJECT_NAME,
                            argument_input_stream.toString());
                        printVariableMessage(421, 421, OSERV_DEBUG_FULL_VALUE,
                            ObjectServerDebug.DEBUG_OBJECT_NAME,
                            String.valueOf(getDebug()) );

                        if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                            argument_input_stream.setLogStream(getLogStream());
                            argument_input_stream.setDebug(getDebug());
                        }
                        printLogCallMessage(DEBUG_SETRETRYCOUNT);
                        argument_input_stream.setRetryCount(getRetryCount());
                        for (int n = 0; n < ptypes.length; n++) {
                            printVariableMessage(91, 91, OSERV_DEBUG_FULL_VALUE,
                                    ObjectServerDebug.DEBUG_PARAM_KEY,
                                    ptypes[n].getName());
                            try {
                                argument_input_stream.nextArgument(ptypes[n].getName());
                                setParameter(obj, ptypes[n].getName(), n,
                                    call_method_wrapper, argument_input_stream);
                            } catch (Exception exception_object) {
                                ObjectServerException ose = new ObjectServerException(exception_object);
                                error = ose;
                                break;
                            }
                        }
                        if (error != null) {
                            break;
                        }
                    } else {
                        rag = new Object[0];
                        printKeyMessage(428, 428, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_EMPTY_OBJECT_ARRAY_SERVER);
                        call_method_wrapper.setMethod(method_object[k]);
                    }
                    clone_object = null;
                    check_flag = false;

                    try {
                        clone_object = operateObjectTable(
                                ObjectTable.SEARCH_OBJECT_TABLE, 
                                reqpath, null);
                        check_flag = true;
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                    }
                    if ((type_string.compareTo(
                        RegisterCommand.PERSISTENT_TYPE) == 0) || 
                        (check_flag != false && 
                         clone_object.getType(0).compareTo(
                        ThreadTable.PARALLEL_SERVICE) != 0) ) {
                        pfile = convertFilenameToFilepath(
                          persistent_path,
                          reqpath + SUFFIX_SEPARATOR 
                        + VARIABLE_SUFFIX);

                        printVariableMessage(430, 430, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_VARIABLE_FILE_NAME, pfile);

                        try {
                            read_variable_wrapper.readVariable(pfile, sp, obj);

                            printEndMessage(OSERV_DEBUG_TRACE_VALUE,
                                getOutputMessage(261,
                                    ObjectServerDebug.DEBUG_READVARIABLE_SERVER));
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                            ObjectServerException ose = new ObjectServerException(exception_object);
                            error = ose;
                        }
                    }
                    if (error != null) {
                        break;
                    }
                    rt = new Object();
                    ObjectServerSecurityManager oserv_security_manager;
                    SecurityManager original;
                    original = System.getSecurityManager();

                    oserv_security_manager = new ObjectServerSecurityManager(request_user_path); 
                    System.setSecurityManager(oserv_security_manager);
                    oserv_security_manager.toUserMode();

                    printVariableMessage(109, 109, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_METHOD_KEY,
                        method_object[k].getName());

                    try {
                        if (user_debug == true) {
                            ByteArrayOutputStream bos;
                            bos = new ByteArrayOutputStream();
                            ObjectOutputStream oos;
                            oos = new ObjectOutputStream(bos);

                            Class ctype = method_object[k].getReturnType();
                            oos.writeObject(ctype.getName());

                            ByteArrayOutputStream debug_bos;
                            debug_bos = new ByteArrayOutputStream();
                            sysout = new PrintStream(debug_bos);
                            save_out = System.out;
                            save_err = System.err;
                            System.out.flush();
                            System.err.flush();

                            printKeyMessage(434, 434, OSERV_DEBUG_TRACE_VALUE,
                                        ObjectServerDebug.DEBUG_SWITCH_OUTPUT);
                                
                            /**
                             * When switching output, it must call follows
                             * System.setOut(sysout); System.setErr(sysout);
                             */

                            call_method_wrapper.invoke(obj);
                            deleteInputStream(call_method_wrapper);
                           
                            rt = getNetworkObject(obj, method_object[k].getReturnType().getName(), call_method_wrapper);

                            System.out.flush();
                            System.err.flush();
                            /**
                             * When retrieving output, it must call follows
                             * System.setOut(save_out); System.setErr(save_err);
                             */

                            printKeyMessage(435, 435, OSERV_DEBUG_TRACE_VALUE,
                                        ObjectServerDebug.DEBUG_SWITCH_BACK);
                            String msg = sysout.toString();
                            String sbuf = bos.toString();
                            int sz = sbuf.length();
                            String ssz = String.valueOf(sz);
                            String body;
                        
                            if (msg == null) {
                                msg = new String(EMPTY_MESSAGE);
                            }
                            body = new String(
                                ObjectServerException.DEBUG_RETURN +
                                ssz  + 
                                ObjectServerException.USER_DEBUG_SEPARATOR +
                                sbuf + 
                                ObjectServerException.USER_DEBUG_SEPARATOR +
                                msg);
                            ObjectServerException ose;
                            ose = new ObjectServerException(body);
                            error = (Object)ose;
                        } else {
                            NullOutputStream null_output_stream;
                            null_output_stream = new NullOutputStream();
                            sysout = new PrintStream(null_output_stream);
                            save_out = System.out;
                            save_err = System.err;
                            System.out.flush();
                            System.err.flush();


                            call_method_wrapper.invoke(obj);
                            deleteInputStream(call_method_wrapper);
                            rt = getNetworkObject(obj, method_object[k].getReturnType().getName(), call_method_wrapper);

                            System.out.flush();
                            System.err.flush();
                        }
                    } catch (Exception exception_object) {
                        deleteInputStream(call_method_wrapper);
                        oserv_security_manager.toSuperMode();
                        System.setSecurityManager(original);
                        System.clearProperty(USER_DIR_KEY);
                        if (save_class_loader != null) {
                            setContextClassLoader(save_class_loader);
                            save_class_loader = null;
                        }

                        printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                        error = exception_object;
                        break;
                    }
                    oserv_security_manager.toSuperMode();
                    System.setSecurityManager(original);
                    System.clearProperty(USER_DIR_KEY);

                    if ((type_string.compareTo(
                        RegisterCommand.PERSISTENT_TYPE) == 0) ||
                        (check_flag != false && clone_object.getType(0).compareTo(
                        ThreadTable.PARALLEL_SERVICE) != 0) ) {
                        printLogCallMessage( ObjectServerDebug.DEBUG_WRITEVARIABLE_SERVER );

                        try {
                            write_variable_wrapper.writeVariable(pfile, sp, obj);
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                            edes = new File(pfile);
                            if (edes.exists() == true) {
                                printVariableMessage(483, 483, OSERV_DEBUG_TRACE_VALUE, 
                                ObjectServerDebug.DEBUG_DELETE_KEY, pfile);
	    		        edes.delete();
                            }
                            /** 2013/05/21
                            System.exit(ERROR_EXIT_CODE);
                            **/
                        }
                        printEndMessage(OSERV_DEBUG_TRACE_VALUE,
                                ObjectServerDebug.DEBUG_WRITEVARIABLE_SERVER);
                    }

                    if (rt != null) {
                        try {
                            writeOutputStreams(call_method_wrapper, object_output_stream);
                        } catch (Exception exception_object) {
                            rt = (Object)new NetworkObject();
                            ((NetworkObject)rt).set(exception_object);
                        }
                        try {
                            writeReturn(rt, object_output_stream);
                        } catch (Exception exception_object) {
                            if (save_class_loader != null) {
                                setContextClassLoader(save_class_loader);
                                save_class_loader = null;
                            }
                            throw exception_object;
                        }
                        rt = null;
                    }
                    if (save_class_loader != null) {
                        setContextClassLoader(save_class_loader);
                        save_class_loader = null;
                    }
                    if (call_method_wrapper != null) {
                        try {
                            clearOutputStreams(call_method_wrapper);
                        } catch (Exception exception_object) {
                            throw exception_object;
                        }
                    }
                    if (request_user_path_file.exists() == true) {
                        if (check_flag != false && clone_object.getType(0).compareTo(ThreadTable.SERIALIZABLE_SERVICE) != 0) {
                            deleteDirectoryFile(request_user_path_file);
                        }
                    }
                    return rt;
                }
                break;
            }
            if (error != null) {
                if (call_method_wrapper != null) {
                    try {
                        clearOutputStreams(call_method_wrapper);
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                }
                if (error != null && (error instanceof Exception) == true) {
                    throw ((Exception)error);
                }
            }
        }
        printVariableMessage(452, 452, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_LOCAL_OBJECT_NOT_FOUND, reqpath);

        /**
         * forward service table
         */
        for (int i = 0; i < forward_service_count; i++) {
            target_entry = forward_service_table[i];
            service = getForwardService(target_entry);
            printDebugMessage(OSERV_DEBUG_FULL_VALUE,
                getFormattedOutputMessage(453,
                    ObjectServerDebug.DEBUG_FORWARD_SERVICES,
                    getOutputMessage(453, ObjectServerDebug.DEBUG_FORWARD_SERVICES),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, service, 
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, reqpath));
            if (service.compareTo(reqpath) == 0) {
                try {
                    rt = transferService(target_entry, cmd, retry_object_input_stream, object_output_stream);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_INVOKE, 0, exception_object);
                    if (error == null) {
                        error = (Object)exception_object;
                    }
                    continue;
                }
                return rt;            
            }
        }
        if (error != null) {
            if (error != null && (error instanceof Exception) == true) {
                throw ((Exception)error);
            }
        }
        ObjectServerServiceUnavailableException ose 
            = new ObjectServerServiceUnavailableException(
                getUserOutputMessage(195, 195,
            ObjectServerException.SERVICE_NOT_AVAILABLE));
        return ose;
    }

    private boolean checkAccess(Socket socket_object, 
        RetryObjectInputStream retry_object_input_stream,
        URITable refuse_table) {
        String  host_address;
        Object  object_value;

        OutputStream       socket_output_stream;
        ObjectOutputStream object_output_stream;

        printExecuteMessage( DEBUG_CHECKACCESS );
        if (socket_object == null) {
            printVariableMessage(172, 172, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_RETURN_VALUE, String.valueOf(Boolean.FALSE));
            return false;
        }
        if (refuse_table == null) {
            printVariableMessage(173, 173, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_RETURN_VALUE, String.valueOf(Boolean.TRUE));
            return true;
        }
        host_address = socket_object.getInetAddress().getHostAddress();
        printVariableMessage(177, 177, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_HOST_NAME, host_address);

        printVariableMessage(176, 176, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_COUNT_KEY,
            String.valueOf(refuse_table.countEntries()) );

        if (refuse_table.countEntries() > 0) {
            if (refuse_table.isValid(host_address) == false) {
                printLogCallMessage( ObjectServerDebug.DEBUG_CHECK_ACCESS_SERVER );
                if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                    ArrayList debug_messages;
                    debug_messages = refuse_table.getDebugBox();
                    for (int j = 0; j < debug_messages.size(); j++) {
                        getLogStream().printLogln(
                            (String)debug_messages.get(j) );
                    }
                }
                printVariableMessage(178, 178, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_RETURN_VALUE,
                    ObjectServerException.REFUSE_CONNECTION);

                ObjectServerException ose
                    = new ObjectServerException(
                        getUserOutputMessage(315, 315,
                    ObjectServerException.REFUSE_CONNECTION)); 

                try {
                    object_output_stream = getObjectOutputStream(socket_object);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CHECKACCESS, 0, exception_object);
                    return false;
                }
                try {
                    object_output_stream.writeObject(ose);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CHECKACCESS, 0, exception_object);
                    return false;
                }
                while (true) {
                    try {
                        object_value = retry_object_input_stream.retryReadObject();
                        printVariableMessage(322, 322, OSERV_DEBUG_FULL_VALUE,
                            ObjectServerDebug.DEBUG_OBJECT_VALUE, object_value.toString());
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_CHECKACCESS, 0, exception_object);
                        break;
                    }
                }
                printVariableMessage(174, 174, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_RETURN_VALUE, String.valueOf(Boolean.FALSE));
                return false; 
            }
            printKeyMessage(57, 57, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_ACCEPT_CONNECTION);
            if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {

                ArrayList debug_messages;
                debug_messages = refuse_table.getDebugBox();
                for (int j = 0; j < debug_messages.size(); j++) {
                    getLogStream().printLogln(
                            (String)debug_messages.get(j) );
                }
            }
        }
        printVariableMessage(175, 175, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_RETURN_VALUE, String.valueOf(Boolean.TRUE));
        return true;
    }

    private ObjectOutputStream getObjectOutputStream(Socket socket_object) 
    throws Exception {
        OutputStream            socket_output_stream;
        ObjectOutputStream      object_output_stream;
        ObjectServerException   object_server_exception;
        try {
            socket_output_stream = socket_object.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETOBJECTOUTPUTSTREAM, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        try {
            object_output_stream = new ObjectOutputStream((OutputStream)socket_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETOBJECTOUTPUTSTREAM, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION);
            throw object_server_exception;
        }
        return object_output_stream;
    }

    private void writeForbiddenException(Socket socket_object) {
        ObjectOutputStream   object_output_stream;
        ObjectServerException   object_server_exception;

        try {
            object_output_stream = getObjectOutputStream(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEFORBIDDENEXCEPTION, 0, exception_object);
            return;
        }

        object_server_exception = new ObjectServerException(
                getUserOutputMessage(311, 311,
                ObjectServerException.REFUSE_CONNECTION)
        );
        try {
            object_output_stream.writeObject(object_server_exception);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEFORBIDDENEXCEPTION, 0, exception_object);
        }
    }

    private void writeRefuseException(Socket socket_object) {
        OutputStream            socket_output_stream;
        ObjectOutputStream      object_output_stream;
        ObjectServerException   object_server_exception;

        try {
            object_output_stream = getObjectOutputStream(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEREFUSEEXCEPTION, 0, exception_object);
            return;
        }

        object_server_exception = new ObjectServerException(
            getUserOutputMessage(316, 316, 
                ObjectServerException.REFUSE_CONNECTION));
        try {
            object_output_stream.writeObject(object_server_exception);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEREFUSEEXCEPTION, 0, exception_object);
        }
    }

    /**
     * This method runs as a thread body
     */
    public void run() {
        Thread                  connect_child;
        Thread                  monitor_child;
        Thread                  target;
        Socket                  target_socket;
        RetryObjectInputStream  target_input;

        if (getInitialized() == false) {
            setInitialized();
            printKeyMessage(455, 455, OSERV_DEBUG_RUN_VALUE,
                LogMessage.SERVER_LOG_STARTUP);
            switchLogFile();
            try {
                setupForward();
            } catch (Exception exception_object) {
                printExceptionMessage(new LogPrintStream(System.err),
                    TRACE_CLASSNAME, TRACE_RUN, 0, exception_object);
                System.exit(ERROR_EXIT_CODE);
            }
            try {
                dispatch();
            } catch (Exception exception_object) {
                printExceptionMessage(new LogPrintStream(System.err),
                    TRACE_CLASSNAME, TRACE_RUN, 0, exception_object);
                System.exit(ERROR_EXIT_CODE);
            }
            System.exit(0);
            return;
       } 
        target = null;
        target_socket = null;
        target_input = null;
        connect_child = null;
        monitor_child = null;

        printDebugMessage(OSERV_DEBUG_FULL_VALUE,
            getFormattedOutputMessage(329,
                ObjectServerDebug.DEBUG_THREAD_NAME,
                getOutputMessage(329,
                    ObjectServerDebug.DEBUG_THREAD_NAME),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                getMyType(),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                Thread.currentThread().getName(),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                Thread.currentThread().getClass().getName()));
        ThreadTable clone_table;
        URITable    refuse_table;
        String      thread_type;
        
        clone_table = null;
        target = null;
        target_socket = null;
        target_input  = null;
        refuse_table = null;
        thread_type = null;

        if (Thread.currentThread().getClass().getName().compareTo(
            RegisterServer.getClassName()) == 0) {

            printTraceMessage(TRACE_RUN, 0, DEBUG_OPERATETHREADTABLE) ;

            try {
                clone_table = operateThreadTable(READ_THREAD_TABLE,
                    null, null, null,   register_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            refuse_table = refuse_object_table;
            thread_type = REGISTER_THREAD_TYPE;
        } else if (Thread.currentThread().getClass().getName().compareTo(
           RequestServer.getClassName()) == 0) {
            printTraceMessage(TRACE_RUN, 0, DEBUG_OPERATETHREADTABLE) ;

            try {
                clone_table = operateThreadTable(READ_THREAD_TABLE,
                    null, null, null,   service_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            refuse_table = refuse_request_table;
            thread_type = SERVICE_THREAD_TYPE;
        } else if (Thread.currentThread().getClass().getName().compareTo(
            ForwardServer.getClassName()) == 0) {
            printTraceMessage(TRACE_RUN, 0, DEBUG_OPERATETHREADTABLE) ;

            try {
                clone_table = operateThreadTable(READ_THREAD_TABLE,
                    null, null, null,   forward_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            refuse_table = refuse_forward_table;
            thread_type = FORWARD_THREAD_TYPE;
        } else if (Thread.currentThread().getClass().getName().compareTo(
                TransientServer.getClassName()) == 0) {
            printTraceMessage(TRACE_RUN, 0, DEBUG_OPERATETHREADTABLE) ;

            try {
                clone_table = operateThreadTable(READ_THREAD_TABLE,
                    null, null, null,   transient_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            refuse_table = refuse_transient_table;
            thread_type = TRANSIENT_THREAD_TYPE;
        } else if (Thread.currentThread().getClass().getName().compareTo(
            SystemServer.getClassName()) == 0) {
            printTraceMessage(TRACE_RUN, 0, DEBUG_OPERATETHREADTABLE) ;

            try {
                clone_table = operateThreadTable(READ_THREAD_TABLE,
                    null, null, null,   system_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            refuse_table = refuse_system_table;
            thread_type = SYSTEM_THREAD_TYPE;
        } else if (Thread.currentThread().getClass().getName().compareTo(
            MonitorServer.getClassName()) == 0) {
        } else if (Thread.currentThread().getClass().getName().compareTo(
            TransientMonitor.getClassName()) == 0) {
        } else {
            printAssertionMessage(TRACE_RUN, 0, getFormattedOutputMessage(433,
                UNKNOWN_THREAD_TYPE, getOutputMessage(433,
                UNKNOWN_THREAD_TYPE), Thread.currentThread().getClass().getName() ));

            ObjectServerException object_server_exception;
            object_server_exception = new ObjectServerException(
                getFormattedOutputMessage(534,
                UNKNOWN_THREAD_TYPE, getOutputMessage(534,
                UNKNOWN_THREAD_TYPE) ));
            return; 
        }
        if (Thread.currentThread().getClass().getName().compareTo(
             MonitorServer.getClassName()) != 0 &&
            Thread.currentThread().getClass().getName().compareTo(
             TransientMonitor.getClassName()) != 0) {

            int local_index;
            Thread current_thread;
            try {
                current_thread = Thread.currentThread();
                local_index =    clone_table.getIndex(current_thread);
                target =         clone_table.getThread(local_index);
                target_socket =  clone_table.getSocket(local_index);
                target_input  = (RetryObjectInputStream)clone_table.getInputStream(local_index);

            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            if (checkAccess(target_socket, target_input, refuse_table) == false
            || (Thread.currentThread().getClass().getName().compareTo(SystemServer.getClassName()) == 0
            && getSystemForbidden() == true)) {
                if (Thread.currentThread().getClass().getName().compareTo(SystemServer.getClassName()) == 0
                    && getSystemForbidden() == true) {
                    writeForbiddenException(target_socket);
                } else {
                    writeRefuseException(target_socket);
                }
                printVariableMessage(211, 211, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_REFUSE_CONNECTION, target_socket.toString());
                if (Thread.currentThread().getClass().getName().compareTo(RegisterServer.getClassName()) == 0) {
                    try {
                        operateThreadTable(RELEASE_THREAD, 
                            REGISTER_THREAD_TYPE,
                            null, null,  register_table, null);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_RUN, 0, exception_object);
                    }
                } else if (Thread.currentThread().getClass().getName().compareTo(RequestServer.getClassName()) == 0) {
                    try {
                        operateThreadTable(RELEASE_THREAD, 
                            SERVICE_THREAD_TYPE,
                            null, null, service_table, null);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_RUN, 0, exception_object);
                    }
                } else  if (Thread.currentThread().getClass().getName().compareTo(ForwardServer.getClassName()) == 0) {
                    try {
                        operateThreadTable(RELEASE_THREAD,
                            FORWARD_THREAD_TYPE,
                            null, null, forward_table, null);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_RUN, 0, exception_object);
                    }
                } else  if (Thread.currentThread().getClass().getName().compareTo(TransientServer.getClassName()) == 0) {
                    try {
                        operateThreadTable(RELEASE_THREAD,
                            TRANSIENT_THREAD_TYPE,
                            null, null, transient_table, null);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_RUN, 0, exception_object);
                    }
                } else if (Thread.currentThread().getClass().getName().compareTo(SystemServer.getClassName()) == 0) {
                    try {
                        operateThreadTable(RELEASE_THREAD, 
                            SYSTEM_THREAD_TYPE,
                            null, null, system_table, null);
                    } catch (Exception exception_object) {
                        printExceptionMessage(TRACE_RUN, 0, exception_object);
                    }
                }
                printVariableMessage(356, 356, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_CONNECTION_END_KEY, target_socket.toString());
                return;
            }
        }

        if (Thread.currentThread().getClass().getName().compareTo(RegisterServer.getClassName()) == 0) {
            try {
                doRegisterConnection(target_socket, target_input);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            try {
                operateThreadTable(RELEASE_THREAD, 
                    REGISTER_THREAD_TYPE, null, null, register_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
        } else if (Thread.currentThread().getClass().getName().compareTo(RequestServer.getClassName()) == 0) {
            try {
                doServiceConnection(target_socket, target_input);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            try {
                operateThreadTable(RELEASE_THREAD, 
                    SERVICE_THREAD_TYPE,
                    null, null, service_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
        } else if (Thread.currentThread().getClass().getName().compareTo(ForwardServer.getClassName()) == 0) {
            try {
                doForwardConnection(target_socket, target_input);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            try {
                operateThreadTable(RELEASE_THREAD, 
                    FORWARD_THREAD_TYPE,
                    null, null, forward_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
        } else if (Thread.currentThread().getClass().getName().compareTo(SystemServer.getClassName()) == 0) {
            try {
                doSystemConnection(target_socket, target_input);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            try {
                operateThreadTable(RELEASE_THREAD, 
                            SYSTEM_THREAD_TYPE,
                            null, null, system_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
        } else if (Thread.currentThread().getClass().getName().compareTo(TransientServer.getClassName()) == 0) {
            try {
                doTransientConnection(target_socket, target_input);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            try {
                operateThreadTable(RELEASE_THREAD, 
                            TRANSIENT_THREAD_TYPE,
                            null, null, transient_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
        } else if (Thread.currentThread().getClass().getName().compareTo(
            MonitorServer.getClassName()) == 0 
            || Thread.currentThread().getClass().getName().compareTo(
            TransientMonitor.getClassName()) == 0 
        ) {
            /** modify **/
            try {
                if (Thread.currentThread().getClass().getName().compareTo(
                    TransientMonitor.getClassName()) == 0) {
                target = transient_table.getThreadbyMonitor(Thread.currentThread());
                } else {
                target = service_table.getThreadbyMonitor(Thread.currentThread());
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            try {
                doMonitorConnection(target_socket, target);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_RUN, 0, exception_object);
            }
            return;
        } else {
            printAssertionMessage(TRACE_RUN, 0, DEBUG_COMMAND_NOT_FOUND );
            return;
        }
        printVariableMessage(496, 496, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_CONNECTION_END_KEY, target_socket.toString());
    }


    private void stopThread(Thread target, String message) {
        ThreadDeath  thread_death;
        thread_death = new ThreadDeath();

        if (target == null) { return; }
        target.stop(thread_death);
        printVariableMessage(213, 213, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_RETURN_KEY, message);
    }

    private String getThreadId(String object_name) {
        int       index_position;
        String    service_string;

        if (object_name == null) { return null; }
        service_string = object_name;
        index_position = service_string.indexOf(START_BRACKET);
        if (index_position >= 0) {
            service_string = service_string.substring(index_position+1);
        }
        index_position = service_string.indexOf(END_BRACKET);
        if (index_position >= 0) {
            service_string = service_string.substring(0,index_position);
        }
        index_position = service_string.indexOf(ITEM_SEPARATOR);
        if (index_position >= 0) {
            service_string = service_string.substring(0,index_position);
        }

        return service_string;    
    }

    private Object setServiceType(Socket socket_object,
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        String  command;
        String  service_type;
        EOSObject  eos_object; 

        try {
            command = (String)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETSERVICETYPE, 0, exception_object);
            throw exception_object;
        }
        try {
            service_type = (String)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETSERVICETYPE, 0, exception_object);
            throw exception_object;
        }
        try {
            operateObjectTable(ObjectTable.SET_SERVICE_TYPE, command, service_type);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETSERVICETYPE, 0, exception_object);
            throw exception_object;
        }
        try {
            writeObjectTable(objfile);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETSERVICETYPE, 0, exception_object);
        }

        eos_object = new EOSObject(EOSObject.END_OF_SOCKET_MESSAGE);
        return eos_object;
    }

    private Object doSystemCommand(Socket socket_object, String command_string,
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        int       target;
        int       index;
        int       service_count;
        int       index_position;
        Object    object_value;
        String    target_id;
        String    service_string;
        Thread[]  connect_threads;
        Thread[]  monitor_threads;
        Thread[]  system_threads;
        Thread    service_thread;
        Thread    monitor_thread;
        String[]  service;
        String    current_thread_name;
        Socket    target_socket;
        ObjectServerException error;
        EOSObject eos_object;
        ObjectServerException object_server_exception;
        ObjectOutputStream object_output_stream;

        if (command_string == null || retry_object_input_stream == null) {
            object_server_exception   
                = new ObjectServerException( getUserOutputMessage(214, 214,
                ObjectServerError.PARAMETER_ERROR_DO_SYSTEM_COMMAND_SERVER));
            throw object_server_exception;
        }
        printExecuteMessage( DEBUG_DOSYSTEMCOMMAND,
            getFormattedOutputMessage(382, ObjectServerDebug.DEBUG_COMMAND_KEY,
            getOutputMessage(382, ObjectServerDebug.DEBUG_COMMAND_KEY),
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, command_string));

        if ((command_string.compareTo(OSERV_SYSTEM_LIST) == 0) ||
            (command_string.compareTo(OSERV_SYSTEM_SYSTEM_LIST) == 0) ) {
            ThreadTable clone_table;
            if (command_string.compareTo(OSERV_SYSTEM_SYSTEM_LIST) == 0) {
                try {
                    clone_table = operateThreadTable(READ_THREAD_TABLE, 
                            SYSTEM_THREAD_TYPE,
                            null, null, system_table, null);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                    throw exception_object;
                }
            } else if (command_string.compareTo(OSERV_SYSTEM_LIST) == 0) {
                try {
                    clone_table = operateThreadTable(READ_THREAD_TABLE, 
                        SERVICE_THREAD_TYPE,
                        null, null, service_table, null);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                    throw exception_object;
                }
            } else {
                ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(110, 110,
                ObjectServerError.PARAMETER_ERROR_DO_SYSTEM_COMMAND_SERVER));
                throw ose;
            }
            printTraceMessage(TRACE_DOSYSTEMCOMMAND, 0, String.valueOf(clone_table.getCount()));

            current_thread_name = null;
            if (command_string.compareTo(OSERV_SYSTEM_SYSTEM_LIST) == 0) {
                service = new String[clone_table.getCount()-1];
                current_thread_name = getThreadId(Thread.currentThread().getName());
            } else {
                service = new String[clone_table.getCount()];
            }
            index = 0;
            for (int i = 0; i < service.length; i++) {
                try {
                    service_string = clone_table.getThread(index).getName();
                    index++;
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                    throw exception_object;
                }
                service_string = getThreadId(service_string);
                if (command_string.compareTo(OSERV_SYSTEM_SYSTEM_LIST) == 0
                   && service_string.compareTo(current_thread_name) == 0) {
                   continue;
                }
                service[i] = service_string;
            }

            try {
                object_output_stream = getObjectOutputStream(socket_object);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                throw exception_object;
            }
            Integer  table_count;
            table_count = Integer.valueOf(service.length);
            try {
                object_output_stream.writeObject(table_count);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
            for (int i = 0; i < service.length; i++) {
                try {
                    object_output_stream.writeObject(service[i]);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                }
            }
            return null;
        } else if (command_string.compareTo(OSERV_SYSTEM_KILL) == 0) {
            try {
                object_value = retry_object_input_stream.retryReadObject();
                target_id = THREAD_ID_HEAD + (String)object_value;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                throw exception_object;
            }
            String      table_type;
            ThreadTable clone_table;

            table_type = getThreadType(target_id);
           
            if (table_type.compareTo(SYSTEM_THREAD_TYPE) == 0) {
                try {
                    clone_table = operateThreadTable(TERMINATE_THREAD, 
                        SYSTEM_THREAD_TYPE,
                        socket_object, retry_object_input_stream, 
                        system_table, target_id);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                    throw exception_object;
                }
            } else if (table_type.compareTo(SERVICE_THREAD_TYPE) == 0) {
                try {
                    clone_table = operateThreadTable(TERMINATE_THREAD,
                    SERVICE_THREAD_TYPE,
                    socket_object, retry_object_input_stream, 
                    service_table, target_id);
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                    throw exception_object;
                }
                if (clone_table.getCount() != 1) {
                    printAssertionMessage(TRACE_DOSYSTEMCOMMAND, 0, getFormattedOutputMessage(330, 
                        ObjectServerError.ID_NOT_FOUND_EXCEPTION_SERVER,
                        getOutputMessage(330, 
                            ObjectServerError.ID_NOT_FOUND_EXCEPTION_SERVER)));

                    object_server_exception = new ObjectServerException(
                        getFormattedOutputMessage(331, 
                            ObjectServerError.ID_NOT_FOUND_EXCEPTION_SERVER,
                            getOutputMessage(331, 
                                ObjectServerError.ID_NOT_FOUND_EXCEPTION_SERVER)));
                    throw object_server_exception;
                }
                service_thread = clone_table.getThread(0);
                target_socket = clone_table.getSocket(0);
                shutdownSocket(target_socket);
                stopThread(service_thread, 
                    getFormattedOutputMessage(332,
                        ObjectServerHeader.SERVICE_THREAD_TYPE,
                        getOutputMessage(332,
                            ObjectServerHeader.SERVICE_THREAD_TYPE)));
            }
            
            eos_object = new EOSObject(EOSObject.END_OF_SOCKET_MESSAGE);
            return eos_object;
        }
        if (command_string.compareTo(OSERV_SYSTEM_SET_SERVICE_TYPE) == 0) {
            try {
                setServiceType(socket_object, retry_object_input_stream);    
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                throw exception_object;
            }
            eos_object = new EOSObject(EOSObject.END_OF_SOCKET_MESSAGE);
            return eos_object;
        }
        if (command_string.compareTo(OSERV_SYSTEM_LOOP) == 0) {
            try {
                operateThreadTable(RESERVE_THREAD, SERVICE_THREAD_TYPE,
                    socket_object,
                    retry_object_input_stream, service_table, null);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCOMMAND, 0, exception_object);
                return exception_object;
            }

            while (true) {
                try {
                    Thread.sleep(monitor_interval);
                } catch (Exception exception_object) {
                }
            }
            /*
            eos_object = new EOSObject(EOSObject.END_OF_SOCKET_MESSAGE);
            return eos_object;
            */
        }
        object_server_exception 
            = new ObjectServerException( getUserOutputMessage(695, 695,
            ObjectServerError.COMMAND_NOT_FOUND_DO_SYSTEM_COMMAND_SERVER));
        throw object_server_exception;
    }

    private void doSystemConnection(Socket issock,
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        Object  command_object;
        String  command_string;
        Object  rt;
        OutputStream            socket_output_stream;
        ObjectOutputStream      object_output_stream;
        ObjectServerException   error;

        if (issock == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(358, 358,
                ObjectServerError.SOCKET_NULL_DO_SYSTEM_CONNECTION_SERVER));
            throw ose;
        }
        printKeyMessage(278, 278, OSERV_DEBUG_RUN_VALUE,
                LogMessage.SERVER_LOG_ACCEPT_SYSTEM );

        if (issock.isConnected() == false) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(73, 73,
                ObjectServerError.SOCKET_NULL_DO_SYSTEM_CONNECTION_SERVER));
            throw ose;
        }
        if (retry_object_input_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(72, 72,
                ObjectServerError.SOCKET_FAILURE_DO_SYSTEM_CONNECTION_SERVER));
            throw ose;
        }
        try {
            command_object = retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSYSTEMCONNECTION, 0, exception_object);
            return;
        }
        if (command_object != null && (command_object instanceof java.lang.Object) == true) {
            command_string = (String)command_object;
            try {
                rt = doSystemCommand(issock, command_string, retry_object_input_stream);
            } catch (Exception exception_object) {
                rt = (Object)new NetworkObject();
                ((NetworkObject)rt).set(exception_object);
            }
        } else {
            error = new ObjectServerException( getUserOutputMessage(192, 192,
              ObjectServerError.COMMAND_NOT_FOUND_DO_SYSTEM_CONNECTION_SERVER));
            rt = (Object)new NetworkObject();
            ((NetworkObject)rt).set(error);
        }

        try {
            object_output_stream = getObjectOutputStream(issock);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSYSTEMCONNECTION, 0, exception_object);
            return;
        }
        if (rt != null) {
            try {
                object_output_stream.writeObject(rt);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCONNECTION, 0, exception_object);
            }
        }
        return;   
    }

    private void doMonitorConnection(Socket issock,
        Thread service_thread) throws Exception {
        boolean  connection_status;

        while ( true ) {
            try {
                Thread.sleep(monitor_interval);
                if (issock != null) {
                    connection_status = issock.isConnected();
                    if (connection_status != true) {
                        break;
                    }
                }
                continue;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSYSTEMCONNECTION, 0, exception_object);
                continue;
            }
        }
            
        if (service_thread != null) {
            stopThread(service_thread, getFormattedOutputMessage(666,
                ObjectServerHeader.SERVICE_THREAD_TYPE,
                getOutputMessage(666, ObjectServerHeader.SERVICE_THREAD_TYPE)));
        }
        printVariableMessage(464, 464, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_CONNECTION_END_KEY, issock.toString());
    }

    private void transferStream(
        RetryObjectInputStream retry_object_input_stream,
        ObjectOutputStream object_output_stream
    ) throws Exception {

        Object       object_inst;
        SODObject    sod_object;
        EODObject    eod_object;
        sod_object = new SODObject(SODObject.START_OF_DATA_MESSAGE);
        eod_object = new EODObject(EODObject.END_OF_DATA_MESSAGE);
        ObjectServerException   object_server_exception;

        printLogCallMessage(ObjectServerDebug.DEBUG_WRITEOBJECT,
            sod_object.toString());
        try {
            object_output_stream.writeObject(sod_object);
        } catch (Exception exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
        while (true) {
            try {
                object_inst = retry_object_input_stream.retryReadObject();
            } catch (Exception exception_object) {
                break;
            }
            if (object_inst == null) { break ; }
            if (object_inst != null && (object_inst instanceof EODObject) == true) {
                break;
            }
            if (object_inst != null && (object_inst instanceof NetworkObject) != true) {
                break;
            }
            printLogCallMessage(ObjectServerDebug.DEBUG_WRITEOBJECT,
                object_inst.toString());
            try {
                object_output_stream.writeObject(object_inst);
            } catch (Exception exception_object) {
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
        }
        printLogCallMessage(ObjectServerDebug.DEBUG_WRITEOBJECT,
            eod_object.toString());
        try {
            object_output_stream.writeObject(eod_object);
        } catch (Exception exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }

    private Object readStream(
        ObjectInputStream object_input_stream,
        OutputStream output_stream
    ) throws Exception {

        EODObject  eod_object;
        eod_object = new EODObject(EODObject.END_OF_DATA_MESSAGE);
        return eod_object;
    }

    private void clearOutputStreams(CallMethodWrapper call_method) throws Exception {
        int       count;
        byte[]    buffer;
        File      fd;
        String    file_name;
        String    parameter_type;
        Object    object_instance;
        NamedOutputStream  named_output_stream;
        NamedInputStream   named_input_stream;

        if (call_method == null) {
            return;
        }
        try {
            count = call_method.getParameterCount();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
            throw exception_object;
        }
        if (count <= 0) {
            return;
        }
        buffer = new byte[maximum];
        for (int i = 0; i < count; i++) {
            try {
                parameter_type = call_method.getParameterType(i);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                throw exception_object;
            }
            if (isOutputStream(parameter_type) == true) {
                object_instance = call_method.getParameter(i);
                if ((object_instance instanceof NamedOutputStream) == true) {
                    named_output_stream = (NamedOutputStream)object_instance;
                    file_name = named_output_stream.getFileName();
                    fd = new File(file_name);
                    if (fd.exists() == true) {
                        printVariableMessage(302, 302, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_FILE_NAME, 
                        file_name);
                        fd.delete();
                    }
                }
            }
        }
    }

    private void writeOutputStreams(CallMethodWrapper call_method, 
        ObjectOutputStream  object_output_stream) throws Exception {
        int rt;
        int count;
        byte[] buffer;
        byte[] bk_buffer;
        File      fd;
        String file_name;
        byte[]  magic_object;
        byte[]  version_object;
        Object    object_instance;
        String    parameter_type;
        SODObject sod_object;
        EODObject eod_object;
        FileInputStream file_input_stream;
        FileOutputStream file_output_stream;
        PipedInputStream piped_input_stream;
        NamedPipedOutputStream  named_piped_output_stream;
        NamedOutputStream  named_output_stream;
        ByteArrayOutputStream internal_byte_array_output_stream;
        NetworkObject network_object;
        ObjectServerException  object_server_exception;

        try {
            count = call_method.getParameterCount();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
            throw exception_object;
        }
        if (count <= 0) {
            return;
        }
        buffer = new byte[maximum];
        for (int i = 0; i < count; i++) {
            try {
                parameter_type = call_method.getParameterType(i);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                throw exception_object;
            }
            if (isOutputStream(parameter_type) == true) {
                object_instance = call_method.getParameter(i);
                if (object_instance != null && 
                   (object_instance instanceof java.io.ByteArrayOutputStream) == true) {
                    sod_object = new SODObject(SODObject.START_OF_DATA_MESSAGE);
                    sod_object.setType(object_instance.getClass().getName());
                    sod_object.setParameterIndex(i);

                    try {
                        object_output_stream.writeObject(sod_object);
                    } catch (Exception exception_object) {
                        object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                        throw object_server_exception;
                    }
                    internal_byte_array_output_stream =
                        (ByteArrayOutputStream)object_instance;
                    buffer = internal_byte_array_output_stream.toByteArray();

                    network_object = new NetworkObject();
                    network_object.set( buffer );
                    try {
                        object_output_stream.writeObject(network_object);
                    } catch (Exception exception_object) {
                        object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                        throw object_server_exception;
                    }

                    eod_object = new EODObject(ObjectServerHeader.END_IOSTREAM);
                    try {
                        object_output_stream.writeObject(eod_object);
                    } catch (Exception exception_object) {
                        object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                        throw object_server_exception;
                    }
                } else if (object_instance != null &&
                    (object_instance instanceof NamedOutputStream) == true) {

                    sod_object = new SODObject(SODObject.START_OF_DATA_MESSAGE);
                    sod_object.setType(object_instance.getClass().getName());
                    sod_object.setParameterIndex(i);

                    try {
                        object_output_stream.writeObject(sod_object);
                    } catch (Exception exception_object) {
                        object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                        throw object_server_exception;
                    }

                    printVariableMessage(300, 300, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_NAME, sod_object.toString());

                    if ((object_instance instanceof NamedPipedOutputStream) == true) {
                        named_piped_output_stream = (NamedPipedOutputStream)object_instance;
                        try {
                            named_piped_output_stream.close();
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                            throw exception_object;
                        }
                        piped_input_stream = named_piped_output_stream.getPipedInputStream();
                        while (true) {
                            try {
                                rt = piped_input_stream.read(
                                    buffer, 0, (int)buffer.length);
                            } catch (Exception exception_object) {
                                if ((exception_object instanceof java.io.EOFException) == true) {
                                    break;
                                }
                                printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                                throw exception_object;
                            }
                            if (rt == -1) { break; }
                            if (buffer.length != rt) {
                                bk_buffer = new byte[rt];
                                for (int j = 0; j < rt; j++) {
                                    bk_buffer[j] = buffer[j];
                                }
                                buffer = bk_buffer;
                            }
                            network_object = new NetworkObject();
                            network_object.set( buffer );

                            try {
                                object_output_stream.writeObject(network_object);
                            } catch (Exception exception_object) {
                                object_server_exception = getObjectServerException(
                                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                                throw object_server_exception;
                            }

                            printVariableMessage(301, 301, OSERV_DEBUG_FULL_VALUE,
                                ObjectServerDebug.DEBUG_OBJECT_NAME, network_object.toString());
                        }
                    } else {
                        named_output_stream = (NamedOutputStream)object_instance;
                        file_name = named_output_stream.getFileName();
                        fd = new File(file_name);
                        try {
                            named_output_stream.closeStream();
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                            throw exception_object;
                        }
                        try {
                            file_input_stream = new FileInputStream(file_name);
                        } catch (Exception exception_object) {
                            printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                            object_server_exception = getObjectServerException(
                            ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION,
                            ObjectServerError.JAVA_FILE_INPUT_STREAM_IO_EXCEPTION);
                            throw object_server_exception;
                        }
                        fd = new File(file_name);
                        if ((object_instance instanceof NamedObjectOutputStream) == true) {
                            try {
                                magic_object = new byte[2];
                                rt = file_input_stream.read(
                                    magic_object, 0, 2);
                                version_object = new byte[2];
                                rt = file_input_stream.read(
                                    version_object, 0, 2);
                            } catch (Exception exception_object) {
                                printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                                throw exception_object;
                            }
                        }

                        while (true) {
                            try {
                                rt = file_input_stream.read(
                                    buffer, 0, (int)buffer.length);
                            } catch (Exception exception_object) {
                                if ((exception_object instanceof java.io.EOFException) == true) {
                                    break;
                                }
                                printExceptionMessage(TRACE_WRITEOUTPUTSTREAMS, 0, exception_object);
                                throw exception_object;
                            }
                            if (rt == -1) { break; }
                            if (buffer.length != rt) {
                                bk_buffer = new byte[rt];
                                for (int j = 0; j < rt; j++) {
                                    bk_buffer[j] = buffer[j];
                                }
                                buffer = bk_buffer;
                            }
                            network_object = new NetworkObject();
                            network_object.set( buffer );
                            try {
                                object_output_stream.writeObject(network_object);
                            } catch (Exception exception_object) {
                                object_server_exception = getObjectServerException(
                                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                                throw object_server_exception;
                            }
                            printVariableMessage(285, 285, OSERV_DEBUG_FULL_VALUE,
                                ObjectServerDebug.DEBUG_OBJECT_NAME, network_object.toString());
                        }
                        fd.delete();
                    }
                    eod_object = new EODObject(ObjectServerHeader.END_IOSTREAM);
                    try {
                        object_output_stream.writeObject(eod_object);
                    } catch (Exception exception_object) {
                        object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                        throw object_server_exception;
                    }
                    printVariableMessage(296, 296, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_NAME, eod_object.toString());
                }
            }
        }
    }

    private void writeReturn(Object  return_object,
        ObjectOutputStream object_output_stream) throws Exception {
        Object return_value;
        ObjectServerException    object_server_exception;

        if (object_output_stream == null) {
            printAssertionMessage(TRACE_WRITERETURN, 0, DEBUG_NULL_OPERATION_TYPE);
            return; 
        }
        return_value = null;
        if (return_object != null && 
            ((return_object instanceof NetworkObject) == true) &&
            ((NetworkObject)return_object).get() != null && 
            (((NetworkObject)return_object).get() instanceof InputStream) == true) {
            try {
                return_value = writeStream( 
                    (InputStream)((NetworkObject)return_object).get(), 
                    object_output_stream);
            } catch (Exception exception_object) {
                return_value = exception_object;
            }
        } else {
            return_value = return_object;
        }

        if (return_value != null) {
            try {
                object_output_stream.writeObject(return_value);
                printVariableMessage(293, 293, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME, return_value.toString());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITERETURN, 0, exception_object);
                if (exception_object != null && (exception_object instanceof NotSerializableException) == true) {
                    object_server_exception = getObjectServerException(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                    return_value = (Object)object_server_exception;
                    try {
                        object_output_stream.writeObject(return_value);
                    } catch (Exception sub_exception_object) {
                        printExceptionMessage(TRACE_WRITERETURN, 0, sub_exception_object);
                    }
                }
                return;
            }
        }
    }

    private Object writeStream(InputStream input_stream, 
        ObjectOutputStream object_output_stream) throws Exception {

        int          rval;
        int          BUFFER_SIZE = 4096;
        byte[]       buffer;
        byte[]       org_buffer;
        NetworkObject network_object;
        ObjectServerException   object_server_exception;
        SODObject    sod_object;
        sod_object = new SODObject(SODObject.START_OF_DATA_MESSAGE);
        sod_object.setType(input_stream.getClass().getName());

        try {
            object_output_stream.writeObject(sod_object);
            printVariableMessage(290, 290, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_OBJECT_NAME, sod_object.toString());
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_WRITESTREAM, 0, exception_object);
            throw exception_object;
        }

        org_buffer = new byte[maximum];

        while (true) {
            try {
                rval = input_stream.read(org_buffer, 0, (int)org_buffer.length);
            } catch (Exception exception_object) {
                break;
            } 
            if (rval == -1) { break; }
            buffer = new byte[rval];
            for (int i = 0; i < rval; i++) {
                buffer[i] = org_buffer[i];
            }
            network_object = new NetworkObject();
            network_object.set( buffer );
            try {
                object_output_stream.writeObject(network_object);
                printVariableMessage(292, 292, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME, network_object.toString());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_WRITESTREAM, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
        }
        EODObject  eod_object;
        eod_object = new EODObject(EODObject.END_OF_DATA_MESSAGE);
        return eod_object;
    }

    private void doTransientConnection(Socket socket_object,
        RetryObjectInputStream retry_object_input_stream) throws Exception {

        Object             rt;
        String             cmd;
        String             service_name;
        Object             object_value;
        String             object_file_name;
        RegisterCommand  register_command_object;
        ObjectOutputStream object_output_stream;

        try {
            register_command_object = (RegisterCommand)
                retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOREGISTERCONNECTION, 0, exception_object);
            throw exception_object;
        }

        doRegisterCommand(socket_object, 
                retry_object_input_stream, register_command_object);
        try {
            object_output_stream = getObjectOutputStream(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            return;
        }

        try {
            object_value = retry_object_input_stream.retryReadObject();
            if (object_value != null && (object_value instanceof java.lang.String) != false) {
                printVariableMessage(108, 108, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
            }
            cmd = (String)object_value;
            service_name = getServiceName(cmd);
            operateThreadTable(EXCLUSIVE_SET_OF_COMMAND_TO_THREAD,
                TRANSIENT_THREAD_TYPE, null, null,   transient_table, service_name);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            try {
                object_output_stream.writeObject(exception_object);
            } catch (Exception real_exception_object) {
                printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, real_exception_object);
            }
            return;
        }

        try {
            rt = invoke(RegisterCommand.TRANSIENT_TYPE,
                cmd, retry_object_input_stream, object_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            try {
                object_output_stream.writeObject(exception_object);
            } catch (Exception real_exception_object) {
                printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, real_exception_object);
            }
        }

        doRegisterDelete(socket_object, 
                retry_object_input_stream, register_command_object);

    }

    private void doServiceConnection(Socket issock,
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        String                  sport;
        int                     iport;
        String                  cmd;
        ObjectOutputStream      object_output_stream;
        OutputStream            socket_output_stream;
        InputStream             socket_input_stream;
        Object[]                obj;
        Object                  rt;
        Object                  error;
        Socket                  ossock;
        int                     retry;
        Object                  object_value;

        if (issock == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(75, 75,
                ObjectServerError.SOCKET_NULL_DO_SERVICE_CONNECTION_SERVER));
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, ose);
            throw ose;
        }

        printKeyMessage(465, 465, OSERV_DEBUG_RUN_VALUE,
                LogMessage.SERVER_LOG_ACCEPT_REQUEST);

        if (issock.isConnected() == false) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(58, 58,
                ObjectServerError.SOCKET_FAILURE_DO_SERVICE_CONNECTION_SERVER));
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, ose);
            throw ose;
        }
        printVariableMessage(189, 189, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_SOCKET_INFORMATION, issock.toString());

        if (retry_object_input_stream == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(152, 152,
                ObjectServerError.SOCKET_FAILURE_DO_SERVICE_CONNECTION_SERVER));
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, ose);
            throw ose;
        }
        printVariableMessage(163, 163, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_SOCKET_NAME, issock.toString());

        printVariableMessage(162, 162, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_JAVA_INPUTSTREAM,
            retry_object_input_stream.toString());
        try {
            object_output_stream = getObjectOutputStream(issock);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            throw exception_object;
        }
        printVariableMessage(165, 165, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM_VALUE,
            object_output_stream.toString());
        cmd = null;
        rt = null;
        error = null;
        try {
            object_value = retry_object_input_stream.retryReadObject();
            if (object_value != null && (object_value instanceof java.lang.String) != false) {
                printVariableMessage(116, 116, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
            }
            cmd = (String)object_value;
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            cmd = null;
            ObjectServerException ose 
                    = new ObjectServerException(exception_object);
            error = (Object)ose;
        }    
        if (cmd == null) {
            printKeyMessage(479, 479, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_NULL_COMMAND_RECEIVE);
            if (rt == null) {
                ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(478, 478,
                        ObjectServerError.COMMAND_PARAMETER_NULL_SERVER)); 
                error = (Object)ose; 
            }
            rt = (Object)new NetworkObject();
            ((NetworkObject)rt).set(error);
            try {
                object_output_stream.writeObject(rt);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            }
            return;
        }
        if (getSystemForbidden() == false) {
            if (cmd.compareTo(OSERV_SYSTEM_LOOP) == 0) {
                while (true) {
                    try {
                        Thread.sleep(monitor_interval);
                    } catch (Exception exception_object) {
                    }
                }
                /**
                eos_object = new EOSObject(EOSObject.END_OF_SOCKET_MESSAGE);
                return eos_object;
                **/
            }
        }

        String  service_name;
        try {
            service_name = getServiceName(cmd);
            operateThreadTable(EXCLUSIVE_SET_OF_COMMAND_TO_THREAD,
                SERVICE_THREAD_TYPE, null, null,   service_table, service_name);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            rt = (Object)new NetworkObject();
            ((NetworkObject)rt).set(exception_object);
            try {
                object_output_stream.writeObject(rt);
            } catch (Exception real_exception_object) {
                printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, real_exception_object);
            }
            return;
        }

        printVariableMessage(481, 481, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_OBJECT_VALUE, cmd.toString());

        try {
            rt = invoke(RegisterCommand.PERSISTENT_TYPE,
                cmd, retry_object_input_stream, object_output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            rt = (Object)new NetworkObject();
            ((NetworkObject)rt).set(exception_object);
        }
        if (getDebug() > OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
            if (rt != null) {
                printVariableMessage(484, 484, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_RESULT_OBJECT, rt.toString());
                if (rt != null && (rt instanceof NetworkObject) == true) {
                    printVariableMessage(410, 410, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_TYPE_KEY, ((NetworkObject)rt).getName());
                    if (((NetworkObject)rt).getName().compareTo(
                        NetworkObject.VOID_TYPE
                    ) == 0)  {
                        printValueMessage( NetworkObject.VOID_TYPE );
                    } else if (((NetworkObject)rt).getName().compareTo(
                        int.class.getName()
                    ) == 0)  {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getInt()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        long.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getLong()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        float.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getFloat()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        double.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getDouble()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        boolean.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getBoolean()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        byte.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getByte()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        char.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getChar()));
                    } else if (((NetworkObject)rt).getName().compareTo(
                        short.class.getName()
                    ) == 0) {
                        printValueMessage( String.valueOf(((NetworkObject)rt).getShort()));
                    } else {
                        if (((NetworkObject)rt).get().toString().compareTo(
                            String.class.getName()) == 0) {
                            printValueMessage( (String)((NetworkObject)rt).get());
                        } else {
                            printValueMessage( ((NetworkObject)rt).get().toString());
                        }
                    }
                }
            } else {
                printVariableMessage(485, 485, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_RESULT_OBJECT, DEBUG_NULL);
            }
        }

        if (issock.isConnected() == false) {
            printKeyMessage(486, 486, OSERV_DEBUG_TRACE_VALUE,
                DEBUG_ALREADY_CLOSED_CONNECTION );
            return;
        }

        if (rt != null) {
            try {
                writeReturn(rt, object_output_stream);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }

        String return_string;
        return_string = DEBUG_NULL;
        if (rt != null) {
            return_string = rt.toString();
        }
        printVariableMessage(289, 289, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_SEND_BACK_OBJECT, return_string);
    }

    private void sendCommandToForward(ObjectOutputStream object_output_stream,
        URI send_entry) throws Exception 
    {
        ObjectServerException  object_server_exception;

        if (object_output_stream == null || send_entry == null) {
            return;
        }
        printExecuteMessage( DEBUG_SENDCOMMANDTOFORWARD );
        printVariableMessage(495, 495, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_REQUEST_KEY, send_entry.toString());

        try {
            object_output_stream.writeObject(send_entry);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SENDCOMMANDTOFORWARD, 0, exception_object);
            object_server_exception = getObjectServerException(
            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }

    private int getForwardServerHostCount(URI target) {
        int        count;
        URI        service_entry;
        String     query_string;
        String     terminal;
        String     hostname_address;
        URI        host_entry;
        String     target_host;
        InetAddress inet_address;
        String     target_host_address;
        String     target_path;
        String     service_path;

        if (target == null) {
            return 0;
        }
        target_host = target.getHost();
        try {
            inet_address = InetAddress.getByName(target_host);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETFORWARDSERVERHOSTCOUNT, 0, exception_object);
            return 0;
        }
        target_host_address = inet_address.getHostAddress();
        target_path = target.getPath();

        printExecuteMessage( DEBUG_GETFORWARDSERVERHOSTCOUNT, 
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
            target.toString());
        count = 0;
        for (int i = 0; i < forward_service_count; i++) {
            service_entry = forward_service_table[i];
            service_path = service_entry.getPath();
            try {
                inet_address = InetAddress.getByName(service_entry.getHost());
            } catch (Exception exception_object) {
                continue;
            }
            hostname_address = inet_address.getHostAddress();
            if (target_path.compareTo(service_path) == 0 && 
                target_host_address.compareTo(hostname_address) == 0) {
                count = count + 1;
                continue;
            }
            query_string = service_entry.getQuery();
            if (query_string == null) {
                continue;
            }
            try {
                terminal = getQueryParameter(query_string, FORWARD_TERMINAL_KEY);
                if (terminal == null) {
                    continue;
                }
                service_entry = new URI(terminal);
                service_path = service_entry.getPath();
                inet_address = InetAddress.getByName(service_entry.getHost());
                hostname_address = inet_address.getHostAddress();
                if (target_path.compareTo(service_path) == 0 &&
                    target_host_address.compareTo(hostname_address) == 0) {
                    count = count + 1;
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETFORWARDSERVERHOSTCOUNT, 0, exception_object);
            }
        }
        return count;
    }

    private URI getTunnelEntry(URI back) throws Exception {
        int     hop_count;
        String  terminal_string;
        URI     terminal_entry;
        String  hop_count_string;
        String  entry_string;
        String  query_string;
        URI     tunnel_entry;
        ObjectServerException   object_server_exception;

        if (back == null) {
        }
        query_string = back.getQuery();
        entry_string = back.getPath();
        if (entry_string.indexOf(OBJECT_SEPARATOR) == 0) {
            entry_string = entry_string.substring(1);
        }
        if (query_string == null || query_string.length() == 0) {
            hop_count = 2;
            hop_count_string = String.valueOf(hop_count);
            terminal_entry = new URI(OSERV_SCHEME, 
                (String)null, back.getHost(), back.getPort(), 
                OBJECT_SEPARATOR + entry_string, 
                (String)null, (String)null);
            terminal_string = terminal_entry.toString();
            query_string = new String(FORWARD_TERMINAL_KEY +
                           QUERY_PARAMETER_SEPARATOR +
                           terminal_string +
                           QUERY_SEPARATOR +
                           FORWARD_HOP_COUNT_KEY +
                           QUERY_PARAMETER_SEPARATOR +
                           hop_count_string );

            tunnel_entry = getURIEntry(entry_string, query_string);
            return tunnel_entry;
        }
        try {
            terminal_string = 
                    getQueryParameter(query_string, FORWARD_TERMINAL_KEY);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            throw exception_object;
        }
        try {
            hop_count_string 
                = getQueryParameter(query_string, FORWARD_HOP_COUNT_KEY);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            throw exception_object;
        }
        try {
            hop_count = Integer.valueOf(hop_count_string).intValue();
            hop_count = hop_count + 1;
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOSERVICECONNECTION, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION);
            throw object_server_exception;
        }
        query_string = new String(FORWARD_TERMINAL_KEY +
                           QUERY_PARAMETER_SEPARATOR +
                           terminal_string +
                           QUERY_SEPARATOR +
                           FORWARD_HOP_COUNT_KEY +
                           QUERY_PARAMETER_SEPARATOR +
                           String.valueOf(hop_count) );

        tunnel_entry = getURIEntry(entry_string, query_string);
        return tunnel_entry;
    }

    private URI getURIEntry(String entry, String query) {
        URI  myEntry;
        int     iport;
        String  host_address;
        String  port_string;
        InetSocketAddress  inet_socket_address;
        InetAddress inet_address;
        ObjectServerException   object_server_exception;

        try {
            inet_address = InetAddress.getLocalHost();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETURIENTRY, 0, exception_object);
            return null;
        }
        host_address = inet_address.getHostAddress();

        port_string = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (port_string == null) {
            printKeyMessage(503, 503, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_REQUEST_PORT_NOT_FOUND);
            return null;
        }
        try {
            iport = Integer.valueOf(port_string).intValue();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETURIENTRY, 0, exception_object);
            return null;
        }

        try {
            myEntry = new URI(OSERV_SCHEME, (String)null, host_address, iport, 
            OBJECT_SEPARATOR + entry, (String)query, (String)null);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETURIENTRY, 0, exception_object);
            return null;
        }

        printVariableMessage(506, 506, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_ENTRY,
                    myEntry.toString());
        return myEntry;
    }

    private void castServicesToForwards() throws Exception {
        URI[]  myServices;
        int bad_oserv_number = 0;
        int my_index;
        int                iport;
        URI                myEntry;
        String             port_string;
        String             host_address;
        InetAddress        inet_address;
        ObjectServerException  object_server_exception;

        port_string = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (port_string == null) {
            printKeyMessage(602, 602, OSERV_DEBUG_TRACE_VALUE,
                            ObjectServerDebug.DEBUG_REQUEST_PORT_NOT_FOUND);
            return;
        }

        inet_address = InetAddress.getLocalHost();
        printVariableMessage(164, 164, OSERV_DEBUG_FULL_VALUE,
           ObjectServerDebug.DEBUG_SOCKET_NAME, inet_address.toString());
        
        host_address = inet_address.getHostAddress();
        printVariableMessage(423, 423, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_HOST_ADDRESS, host_address);

        try {
            iport = Integer.valueOf(port_string).intValue();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION);
            throw object_server_exception;
        }

        /**
         * for reset
         */
        myEntry = new URI(OSERV_SCHEME, (String)null, host_address, iport, 
            OBJECT_SEPARATOR, (String)null, (String)null);

        printVariableMessage(525, 525, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_ENTRY, myEntry.toString());

        myServices = new URI[object_table.getCount()+forward_service_count+1];
        my_index = 0;
        myServices[my_index] = myEntry;
        my_index++;

        for (int i = 0; i < object_table.getCount(); i++) {
            myServices[my_index] = new URI(OSERV_SCHEME, (String)null,
                host_address, iport,
                OBJECT_SEPARATOR + object_table.get(i), 
                (String)null, (String)null );
            printVariableMessage(526, 526, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_SERVICES, 
                    myServices[my_index].toString());

            my_index = my_index + 1;
        }
        for (int i = 0; i < forward_service_count; i++) {
            try {
                myServices[my_index] 
                            = getTunnelEntry(forward_service_table[i]);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                continue;
            }
            my_index = my_index + 1;
        }

        castEntryToForwards(myServices) ;
    }

    /**
     * cast services to forward 
     */
    private void castServiceToForwards(URI myService) throws Exception {
        URI[] myServices;
        myServices = new URI[1];
        myServices[0] = myService;

        castEntryToForwards(myServices);
    }

    private void castEntryToForwards(URI[] myServices) throws Exception {
        int                count;
        URI                entry;
        int                iport;
        Socket             csocket;
        String             port_string;
        String             host_string;
        InetSocketAddress  inet_socket_address;
        String             host_address;
        String             target_port;
        Object             object_value;
        InetAddress        inet_address;
        ObjectOutputStream object_output_stream;
        RetryObjectInputStream  retry_object_input_stream;
        ObjectServerException   object_server_exception;

        printKeyMessage(522, 522, OSERV_DEBUG_RUN_VALUE, 
            LogMessage.SERVER_LOG_CAST_SERVICE);

        if (myServices == null) {
            ObjectServerException ose 
                = new ObjectServerException( getUserOutputMessage(170, 170,
                            ObjectServerError.URI_PARAMETER_NULL_SERVER)); 

            printVariableMessage(70, 70, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_EXCEPTION_KEY, ose.toString());

            throw (Exception)ose;
        }

        count = readForwardClientCount();
        if (count == -1) {
            printKeyMessage(527, 527, OSERV_DEBUG_TRACE_VALUE,
		    ObjectServerDebug.DEBUG_FORWARD_EMPTY_CLIENT_TABLE);
            return;
        }
        for (int i = 0; i < count; i++) {
            entry = readForwardClientEntry(i);
            printVariableMessage(528, 528, OSERV_DEBUG_TRACE_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_ENTRY, entry.toString());

            host_string = entry.getHost();
            iport = entry.getPort();

            ConnectedSocket connected_socket;
            connected_socket = new ConnectedSocket();
            if (getDebug() > 0) {
                printLogCallMessage( DEBUG_SETDEBUGLOGSTREAM );
                connected_socket.setDebugLogStream(getDebug(), getLogStream());
            }
            connected_socket.setMessageResource(getMessageResource());

            printVariableMessage(167, 167, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_HOST_ADDRESS, host_string);
            printVariableMessage(166, 166, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_PORT_NUMBER, String.valueOf(iport));


            try {
                connected_socket.connect(host_string, iport);  
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_SOCKET_CONNECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_SOCKET_CONNECT_IO_EXCEPTION);
                throw object_server_exception;
            }
            try {
                object_output_stream = connected_socket.getOutputStream();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION);
                throw object_server_exception;
            }
            printVariableMessage(137, 137, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_CONNECTION_START_KEY,
		connected_socket.toString());

            try {
                object_output_stream.writeObject(OSERV_COMMAND_TYPE_FORWARD_KEY);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                try {
                    connected_socket.close();  
                } catch (IOException io_exception) {
                    printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, io_exception);
                }
                printVariableMessage(138, 138, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY,
                    connected_socket.toString());
                continue;
            }
           
            try {
                sendForwardTable(object_output_stream, myServices);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                try {
                    connected_socket.close();  
                } catch (IOException io_exception) {
                    printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, io_exception);
                }
                printVariableMessage(294,294, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY, connected_socket.toString());
                continue;
            }
            if (connected_socket.isConnected() == true) {
                try {
                    connected_socket.shutdownOutput();  
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                }
            }
            try {
                retry_object_input_stream = connected_socket.getInputStream();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                continue;
            }
            try {
                object_value = retry_object_input_stream.retryReadObject();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                continue;
            }
            if (object_value != null && (object_value instanceof EOSObject) == true) {
                printVariableMessage(126, 126,    OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
            }
            
            if (connected_socket.isConnected() == true) {
                try {
                    connected_socket.shutdownInput();  
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CASTSERVICESTOFORWARDS, 0, exception_object);
                }
            }
            printVariableMessage(341, 341, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY,
	        connected_socket.toString());
        }
    }

    /**
     * get services to forward 
     */
    private void getServicesFromForwards() throws Exception {
        int  count;
        URI  entry;
        URI  myEntry;
        int     iport;
        File    fd;
        URI[]   myServices;
        Socket  csocket;
        String  query_string;
        String  port_string;
        String  host_string;
        Object  object_value;
        InetSocketAddress  inet_socket_address;
        String  host_address;
        String  target_port;
        InetAddress inet_address;
        FileOutputStream        file_output_stream;
        ObjectOutputStream      socket_object_output_stream;
        ObjectOutputStream     object_output_stream;
        InputStream            input_stream;
        RetryObjectInputStream retry_object_input_stream;
        ObjectServerException  object_server_exception;

        printKeyMessage(542, 542, OSERV_DEBUG_RUN_VALUE,
                    LogMessage.SERVER_LOG_GET_SERVICES_FROM_FORWARD);

        port_string = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (port_string == null) {
         
            printKeyMessage(603, 603, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_REQUEST_PORT_NOT_FOUND );
            return;
        }

        inet_address = InetAddress.getLocalHost();
        host_address = inet_address.getHostAddress();

        try {
            iport = Integer.valueOf(port_string).intValue();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION);
            throw object_server_exception;
        }

        query_string = makeQueryParameter(FORWARD_COMMAND_KEY, 
                        FORWARD_LIST_COMMAND);
        myEntry = new URI(OSERV_SCHEME, (String)null, host_address, iport, 
            OBJECT_SEPARATOR, (String)query_string, (String)null);

        printVariableMessage(544, 544, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_ENTRY, myEntry.toString());

        count = readForwardServerCount();
        if (count == -1) {
            printKeyMessage(545,545, OSERV_DEBUG_TRACE_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_EMPTY_SERVER_TABLE );
            return;
        }
        printVariableMessage(546, 546, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_FORWARD_SERVER_COUNT,String.valueOf(count));
        
        for (int i = 0; i < count; i++) {
            entry = readForwardServerEntry(i);
            printVariableMessage(547, 547, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_CONFIGURED_FORWARDS, 
                entry.toString());
            host_string = entry.getHost();
            iport = entry.getPort();

            ConnectedSocket connected_socket;
            connected_socket = new ConnectedSocket();
            if (getDebug() > 0) {
                connected_socket.setDebugLogStream(getDebug(), getLogStream());
            }
            connected_socket.setMessageResource(getMessageResource());

            try {
                connected_socket.connect(host_address, iport);  
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                continue;
            }
            try {
                socket_object_output_stream = connected_socket.getOutputStream();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                continue;
            }
            printVariableMessage(142, 142, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_CONNECTION_START_KEY,
	        connected_socket.toString());

            try {
                socket_object_output_stream.writeObject(OSERV_COMMAND_TYPE_FORWARD_KEY);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                try {
                    connected_socket.close();  
                } catch (IOException io_exception) {
                    printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, io_exception);
                }
                printVariableMessage(44, 44, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY,
	            connected_socket.toString());
                continue;
            }
            try {
                sendCommandToForward( socket_object_output_stream, myEntry);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                try {
                    connected_socket.close();  
                } catch (IOException io_exception) {
                    printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, io_exception);
                }
                printVariableMessage(246, 246, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY,
	            connected_socket.toString());
                continue;
            }
            try {
                retry_object_input_stream = connected_socket.getInputStream();
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                continue;
            }
            try {
                retry_object_input_stream.setRetryCount(getRetryCount());
                if (getDebug() > 0 && getLogStream() != null) {
                    retry_object_input_stream.setDebugLogStream(getDebug(), getLogStream());
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                try {
                    connected_socket.close();  
                } catch (IOException io_exception) {
                    printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, io_exception);
                }
                printVariableMessage(46, 46, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY, 
                    connected_socket.toString());
                continue;
            }
            Integer uri_table_count;
            try {
                object_value = 
                    retry_object_input_stream.retryReadObject();
                if (object_value != null && (object_value instanceof Integer) != false) {
                    printVariableMessage(114, 114,
                        OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
                }
                uri_table_count = (Integer)object_value;
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                if (exception_object != null && (exception_object instanceof ClassCastException) == true) {
                    Exception real_exception_object;
                    object_value = retry_object_input_stream.retryReadObject();
                    if (object_value != null && (object_value instanceof Integer) != false) {
                        printVariableMessage(159, 159,
                            OSERV_DEBUG_FULL_VALUE,
                            ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
                    }
                }
                try {
                    connected_socket.close();  
                } catch (IOException io_exception) {
                    printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, io_exception);
                }
                printVariableMessage(147, 147, 
                    OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY, connected_socket.toString());
                continue;
            }
            URI uri_entry;
            for (int j = 0; j < uri_table_count.intValue(); j++) {
                printKeyMessage(564, 564, OSERV_DEBUG_FULL_VALUE,
                        ObjectServerDebug.DEBUG_READ_OBJECT );
                try {
                    object_value = retry_object_input_stream.retryReadObject();
                    if (object_value != null && (object_value instanceof URI) == false) {
                        printVariableMessage(123, 123, OSERV_DEBUG_FULL_VALUE,
                            ObjectServerDebug.DEBUG_OBJECT_NAME, object_value.toString());
                        continue;
                    }
                    uri_entry = (URI)object_value;
                } catch (Exception exception_object) {
                    if (exception_object != null && (exception_object instanceof java.io.EOFException) == false) {
                        printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                    }
                    if (exception_object != null && (exception_object instanceof ClassCastException) == true) {
                        Exception real_exception_object;
                        object_value = retry_object_input_stream.retryReadObject();
                        if (object_value != null && (object_value instanceof URI) != false) {
                            printVariableMessage(160, 160, OSERV_DEBUG_FULL_VALUE,
                                ObjectServerDebug.DEBUG_OBJECT_NAME,
                                object_value.toString());
                        }
                    }
                    break;
                }
                printTraceMessage(TRACE_GETSERVICESFROMFORWARDS, 0, 
                    ObjectServerDebug.DEBUG_FORWARD_SET_FORWARD_SERVICE_ENTRY,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY, uri_entry.toString());
                try {
                    setForwardServiceEntry(uri_entry);
                } catch (Exception exception_object) {
                }
            }
            try {
                connected_socket.close();  
            } catch (IOException io_exception) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, io_exception);
            }
            printVariableMessage(169, 169, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_CONNECTION_END_KEY, 
	        connected_socket.toString());
        }
        if (forward_service_count > 0) {
            printVariableMessage(56, 56, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, forward_file);
            fd = new File(forward_file);
            if (fd.exists() == true) {
                printVariableMessage(488, 488, OSERV_DEBUG_TRACE_VALUE,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY, forward_file);

                fd.delete();
            }
            try {
                file_output_stream = new FileOutputStream(fd);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION,
                    ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION);
                return;
            }
            try {
                object_output_stream = new ObjectOutputStream(file_output_stream);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
                return;
            }
            printVariableMessage(272, 272, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_FORWARD_WRITE_FORWARD_FILE,
                forward_file); 
             
            try {
                writeForwardTable(object_output_stream);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_GETSERVICESFROMFORWARDS, 0, exception_object);
            }
        }
    }

    /**
     * setup Forward environment
     */
    private void setupForward() throws Exception {

        printLogCallMessage( getOutputMessage(69, DEBUG_GETSERVICESFROMFORWARDS) );
        try {
            getServicesFromForwards();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETUPFORWARD, 0, exception_object);
        }
        printLogCallMessage( getOutputMessage(86, DEBUG_CASTSERVICETOFORWARDS) );
        try {
            castServicesToForwards();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_SETUPFORWARD, 0, exception_object);
        }
    }

    /**
     * doForwardConnection
     */
    private void doForwardConnection(Socket socket_object,
        RetryObjectInputStream  retry_object_input_stream) throws Exception {
        File                       fd;
        Object                     rt;
        InputStream                socket_input_stream;
        OutputStream               socket_output_stream;
        ObjectOutputStream         object_output_stream;
        String                     request_host_address;
        String                     request_port_number;
        InetAddress                request_address;
        Object                     object_value;
        ObjectServerException      object_server_exception;

        if (socket_object == null) {
            object_server_exception   
                = new ObjectServerException( getUserOutputMessage(276, 276,
	        ObjectServerError.SOCKET_NULL_DO_FORWARD_CONNECTION_SERVER)); 
            throw object_server_exception;
        }
        printKeyMessage(77, 77, OSERV_DEBUG_RUN_VALUE, LogMessage.SERVER_LOG_ACCEPT_FORWARD);
        if (socket_object.isConnected() == false) {
            printKeyMessage(277, 277, OSERV_DEBUG_TRACE_VALUE, ObjectServerDebug.DEBUG_FORWARD_CONNECTION_FAIL );
            return;
        }
        try {
            object_output_stream = getObjectOutputStream(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOFORWARDCONNECTION, 0, exception_object);
            throw exception_object;
        }
        request_host_address = socket_object.getInetAddress().getHostAddress();
        request_port_number = String.valueOf(socket_object.getPort());

        printTraceMessage(TRACE_DOFORWARDCONNECTION, 0, ObjectServerDebug.DEBUG_FORWARD_REQUEST_HOST_ADDRESS, 
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, request_host_address, 
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY, request_port_number);

        printVariableMessage(458, 458, OSERV_DEBUG_FULL_VALUE,
                    ObjectServerDebug.DEBUG_FORWARD_REFUSE_TABLE_COUNT,
                    String.valueOf(refuse_forward_table.countEntries()) );
        printVariableMessage(115, 115, OSERV_DEBUG_FULL_VALUE,
            ObjectServerDebug.DEBUG_SOCKET_NAME, socket_object.toString());
        printVariableMessage(377, 377, OSERV_DEBUG_TRACE_VALUE,
            ObjectServerDebug.DEBUG_JAVA_OUTPUTSTREAM_VALUE,
            object_output_stream.toString());

        try {
            rt = readForwardRequestsFromSocket(retry_object_input_stream, object_output_stream, request_host_address);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_DOFORWARDCONNECTION, 0, exception_object);
            throw exception_object;
        }
        if (rt != null) {
            printVariableMessage(594, 594, OSERV_DEBUG_FULL_VALUE,
                ObjectServerDebug.DEBUG_RETURN_VALUE, rt.toString());
            try {
                object_output_stream.writeObject(rt);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_DOFORWARDCONNECTION, 0, exception_object);
                object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
        }
    }

    /**
     * This class is executed as an independent program
     */
    public static void main(String[] ag) {
        ParseOptions       parse_options;
        ObjectServer       object_server_object;

        object_server_object = new ObjectServer();
        try {
            parse_options = ParseOptions.valueOf(
                ag, ObjectServer.OBJECT_SERVER_ARGS,
                null, null,
                object_server_object.getMessageResource(),
                object_server_object.getDebug(),
                object_server_object.getLogStream() );
        } catch (Exception exception_object) {
            ObjectServer.printUsage();
            return;
        }
        if (parse_options.checkOption(ObjectServer.VOPTION) == true) {
            ObjectServer.printVersion();
            return;
        }
        if (parse_options.checkOption(ObjectServer.HOPTION) == true) {
            ObjectServer.printUsage();
            return;
        }

        main_thread = (Thread)object_server_object;
        /*
        main_thread.start();
        */
        main_thread.run();
    }
}
